\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

\\-----------------------------------------------------------------------------

\\ Kevin Ryde, November 2021
\\
\\ Usage: gp <a349008.gp
\\
\\ Calculate terms of A349008, being area "HA" of the convex hull around
\\ R5 dragon curve expansion level n.
\\
\\ The implementation follows the formulas in the sequence, but powers of 5
\\ and b maintained instead of say 5^(n-2-j) each time, which a speedup but
\\ by less than it might be thought actually.

default(strictargs,1);
default(recover,0);

b = 1+2*I;

\\ return z rotated if necessary to it's in quadrant 1 or 3
RotQ(z) = if(sign(real(z))==sign(imag(z)), z, z*I);

\\ return complex z with the real part sheared by doubling
ShearRe(z) = z + real(z);

MinReIm(z) = min(abs(real(z)),abs(imag(z)));

HAgrow(z) = MinReIm(ShearRe(RotQ(z)));

a(n) =
{
  if(n==0, 0,     \\ initial exceptions, then general case formula
     n==1, 2,
     my(B=1,        \\ b^j
        F=5^(n-2)); \\ 5^(n-2-j)
     17*F - 1 
     + sum(j=1,n-2,
           F/=5; B*=b;
           2*((3*F-1)*HAgrow(      B)
              +  F   *HAgrow((4-I)*B))));
}

{
  print("A349008 hull areas, for n>=0");
  print1("  ");
  for(n=0,9, print1(a(n),", "));
  print("...\n");
}
