\\ Copyright 2013, 2014, 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

read("r5dragon-defines.gp")
check_exact(got,want) =
{
  if(got != want,
     print("wrong\n");
     print("got  " got);
     print("want " want);
     quit(1),
     print("ok"));
}

gf_terms(g,n) = \
  my(x = variable(g), \
     zeros = min(n,valuation(g,x)), \
     v = Vec(g + O(x^n))); \
  if(zeros>=0, concat(vector(zeros,i,0), v), \
               v[-zeros+1 .. #v]);

vector_modulo(v,i) = v[(i% #v)+1];

\\-----------------------------------------------------------------------------

GJseg(k) = 1/12*(11*5^k - 2*(-1)^k - 9);
vector(20,k,dir(GJseg(k)))

\\ GJ(k) = (9/10 + 3/10*I)*b^k + (2/5-1/5*I)*(b^k-1)/(3^k-1);
\\ GJ(k) = 1/10*(  (9+3*I)*b^k + (4-2*I)*b^k/(3^k-1) - (4-2*I)/(3^k-1) );

GJ_real_int(k) = floor(real(GJ(k)));
GJ_real_frac(k) = real(GJ(k)) - GJ_real_int(k);
GJ_imag_int(k) = floor(imag(GJ(k)));
GJ_imag_frac(k) = imag(GJ(k)) - GJ_imag_int(k);

F(k) = (9/10 + 3/10*I)*b^k;
F_real_int(k)  = floor(real(F(k)));
F_real_frac(k) = real(F(k)) - F_real_int(k);
F_imag_int(k)  = floor(imag(F(k)));
F_imag_frac(k) = imag(F(k)) - F_imag_int(k);

vector(10,k, F_real_int(k) - GJ_real_int(k))
vector(10,k, F_imag_int(k) - GJ_imag_int(k))

gF(x) = (9/10 + 3/10*I)*1/(1-b*x);
check_exact(gf_terms(gF(x),100), \
            vector(100,k,my(k=k-1); F(k)))


\\ real [9,3,1,7  ]
\\ imag [  3,1,7,9]
\\ [3+I, 1+7*I, 7+9*I, 9+3*I]
vector(10,k,my(k=k-1); F_real_frac(k))
vector(10,k,my(k=k-1); F_imag_frac(k))
vector(10,k, (b^k - conj(b^k))/5^floor(k/2))

gbk_real(x) = (1-x)/(1-2*x+5*x^2);  \\ A006495
gbk_imag(x) =  2*x /(1-2*x+5*x^2);  \\ A006496
\\ a(n)=2a(n-1)-5a(n-2)
\\ mod 10 is
\\ x
\\ y                                 ==      + y
\\ 2*y-5*x                           == -5*x + 2*y 
\\ 2*(2*y-5*x)-5*y == -10*x - y      ==      - y
\\ 2*-y - 5*(2*y-5*x) == 25*x - 12*y ==  5*x - 2*y

\\ repeat [9,3,1,7]
gper(x) = 1/(1-x^4) * (9 + 3*x + 1*x^2 + 7*x^3);
gf_terms(gper(x),16)

check_exact(gper(x), (7*x^2 - 6*x + 9)/(-x^3 + x^2 - x + 1))
check_exact(gper(x), 5/(1-x) + (4 - 2*x)/(1 + x^2))

per(k) = 5 + 3*(-1)^ceil(k/2) + 1*(-1)^floor(k/2);
check_exact(vector(32,k,my(k=k-1); per(k)), \
            concat(vector(8,i,[9,3,1,7])))

gGJC(x) = gF(x) - 1/10*(gper(x) + I*(gper(x) - 9)/x );

GJC(k) = GJ_real_int(k) + I*GJ_imag_int(k);
check_exact(gf_terms(gGJC(x),103)[4..-1], \
            vector(100,k,GJC(k+2)))    /* k>=3 */

\\-----------------------------------------------------------------------------


\\ vector(15,k, GJ_real_int(4*k+5))
\\ 
\\ for(k=1,40, printf("%.3f\n",GJ_real_frac(4*k+3)*1.0))
\\ printf("min %.7f\n",1.0*vecmin(vector(2000,k, my(g=GJ_real_frac(k+2)); min(g,1-g))));

\\ for(k=1,10, printf("%.5f\n", sqrt(5)^k/(3^k-1)));

\\ r5dragon_midpoints(k) returns a vector of complex numbers which are
\\ the midpoints of each line segment of the level k curve.
\\
r5dragon_points(k) = {
  my(end=1,v=[0]);
  for(i=1,k,
     v = concat(v,
         concat(apply(z->end       + z*I, v),
         concat(apply(z->end*(1+I) - z,   v),
         concat(apply(z->end*I     + z*I, v),
                apply(z->end*2*I   + z,   v)))));
     end *= b);
  concat(v,[b^k]);
}
check_exact(r5dragon_points(0), [0,1]);
check_exact(r5dragon_points(1), [0,1,1+I,I,2*I,2*I+1]);


n_to_xy_z_pos = [0, 1, 1+I, I, 2*I];
n_to_xy_z_dir = [0, 1, 2, 1, 0];
n_to_xy_z(n) =
{
  my(v = digits(n,5), dir=0, ret=0);
  for(i=1,#v,
     my(d=v[i]);
     ret += b^(#v-i)*I^dir * n_to_xy_z_pos[d+1];
     dir += n_to_xy_z_dir[d+1]);
  ret;
}
r5dragon_points_by_n_to_xy_z(k) = vector(5^k+1,n,my(n=n-1); n_to_xy_z(n));
for(k=0,5, check_exact(r5dragon_points_by_n_to_xy_z(k),r5dragon_points(k)));

vector(20,k,dir(GJseg(k)))

square_mid(k) =
{
  my(n = GJseg(k),
     z1 = n_to_xy_z(n),
     z2 = n_to_xy_z(n+1));
  (z1+z2)/2 + (z2-z1)/2 * I;
}
square_delta(k) = square_mid(k) - GJ(k);
\\for(k=1,40, print(abs(square_delta(k)*1.0)))


GJsquare(k) = 1/4*(3^k - 2 + (-1)^k);
print("GJsquare ",vector(4,k,GJsquare(k-1)));
GJseg(k) = 1/12*(11*5^k - 2*(-1)^k - 9);
print("GJseg ",vector(4,k,GJseg(k-1)));

\\ 1,1, -1,-1, ...
gGJdir(x) = 1/(1-x^4)*(1+x-x^2-x^3);
check_exact(gGJdir(x), (1+x)/(1+x^2));
check_exact(gf_terms(gGJdir(x),32), \
            concat(vector(8,i, [1, 1, -1, -1])))

gGJseg_z(x) = x* 1/(1-x^4)*(2*I + (1+I)*x - 2*I*x^2 - (1+I)*x^3) * 1/(1-b*x);
check_exact(gf_terms(gGJseg_z(x), 100), \
            vector(100,k,my(k=k-1); n_to_xy_z(GJseg(k))));

{
  check_exact((9/10 + 3/10*I)   * 1/(1 - b*x)
              + (-2/5 - 3/10*I) * 1/(1 + I*x)
              - 1/2             * 1/(1 - I*x),
              gGJseg_z(x))
}
{
  check_exact((9/10 + 3/10*I)            * 1/(1 - b*x)
              - 1/10*((9+3*I) + (3+I)*x) * 1/(1 + x^2)
              ,
              gGJseg_z(x))
}

gGJseg_z_mid(k) = gGJseg_z(k) + (1/2+1/2+I)*(gGJdir(x)-1);
{
  check_exact(1/10*(   (9+3*I)              * 1/(1 - b*x)
                    + ((1+7*I) + (7+9*I)*x) * 1/(1 + x^2) )
              - (1 + I)
              ,
              gGJseg_z_mid(x));
}

GJseg_z(k) =
{
  sum(  j=0, ceil((k-2)/2),  2*I *b^(k-1-2*j) * (-1)^j)
  + sum(j=0,floor((k-2)/2), (1+I)*b^(k-2-2*j) * (-1)^j);
}
check_exact(vector(100,k,my(k=k-1); GJseg_z(k)), \
            vector(100,k,my(k=k-1); n_to_xy_z(GJseg(k))));

GJseg_z(k) =
{
  sum(j=0, k-1,  b^(k-1-j) * vector_modulo([2*I, (1+I), -2*I, -(1+I)], j));
}
check_exact(vector(10,k,my(k=k-1); GJseg_z(k)), \
            vector(10,k,my(k=k-1); n_to_xy_z(GJseg(k))));

GJseg_z(k) =
{
  (9/10 + 3/10*I)*b^k + vector_modulo([-9/10 - 3/10*I,
                                       -3/10 - 1/10*I,
                                       9/10 + 3/10*I,
                                       3/10 + 1/10*I],k);
}
check_exact(vector(10,k,my(k=k-1); GJseg_z(k)), \
            vector(10,k,my(k=k-1); n_to_xy_z(GJseg(k))));


quit
GJseg_z(k) =
{
  if(k%2==0,
     sum(j=0,k/2-1,  (2*I*b + (1+I)) *b^j * (-1)^(k/2-1 - j)),
     (-1)^((k-1)/2)*(1+I)*b
     * sum(j=0,(k-1)/2,  (2*I*b + (1+I)) *b^j * (-1)^((k-1)/2 - j))
    );
}
check_exact(vector(10,k,my(k=k-1); GJseg_z(k)), \
            vector(10,k,my(k=k-1); n_to_xy_z(GJseg(k))));
