#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min';
use Math::Complex;

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon ':all';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $left_levels = 10;
my $right_levels = 5*4;
print "  % generated by tools/$FindBin::Script\n";

my $b = Math::Complex->make(1,2);
### $x_hi
### $y_hi

$Printwrap = 2;
foreach my $k (1 .. $left_levels) {
  my $z = Math::Complex->make(2/5, -1/5) * ($b**$k - 1) / (3**$k - 1);
  my $str = sprintf " \\coordinate (p%d) at (%.6f,%.6f);", $k, $z->Re, $z->Im;
  printwrap($str);
}
printwrap_newline();
{
  $Printwrap = 2;
  printwrap("  \\draw[my line colour] ");
  $Printwrap = 4;
  my $join = '';
  foreach my $k (1 .. $left_levels) {
    printwrap("$join(p$k)");
    $join = '--';
  }
  printwrap(";\n");
}
{
  $Printwrap = 2;
  printwrap("\\path foreach \\n in {1,...,$left_levels}");
  $Printwrap += 2;
  printwrap("{ (p\\n)node[my dot] };\n");
}

print "  \\begin{scope}[shift=(pos-right)]\n";
$Printwrap = 4;
foreach my $k (1 .. $right_levels) {
  my ($x,$y) = GJ_xy($k);
  $x -= int($x);
  $y -= int($y);
  if ($x < 0) { $x += 1; }
  if ($y <= 0) { $y += 1; }
  $x = $x->numify;
  $y = $y->numify;
  my $str = sprintf " \\coordinate (r%d) at (%.6f,%.6f);", $k, $x, $y;
  printwrap($str);
}
printwrap_newline();

{
  $Printwrap = 4;
  printwrap("  \\draw[my line colour] ");
  $Printwrap += 2;
  foreach my $rem (0 .. 3) {
    my $join = '';
    for (my $k = 1+$rem; $k <= $right_levels; $k+=4) {
      printwrap("$join(r$k)");
      $join = '--';
    }
  }
  printwrap(" ");
  printwrap(";\n");
}
{
  $Printwrap = 4;
  printwrap("\\path foreach \\n in {1,...,$right_levels}");
  $Printwrap += 2;
  printwrap("{ (r\\n)node[my dot] };\n");
}

print "  \\end{scope}\n";
print "  % end generated\n";
exit 0;
