#!/usr/bin/perl -w

# Copyright 2019 Kevin Hyde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use List::Util 'max';

use lib $FindBin::Bin;
use MyR5Dragon 'dir';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

my $level = 4;

print "  % generated by tools/$FindBin::Script\n";

sub num_digits {
  my ($n) = @_;
  my $k = 0;
  while ($n) { $k++; $n=int($n/5); }
  return $k;
}
CHECK {
  num_digits(4) == 1 or die;
  num_digits(5) == 2 or die;
}

sub n_to_x {
  my ($n) = @_;
  ### $n
  if ($n==0) { return 0; }
  my $k = num_digits($n);
  my $high = 5**($k-1);
  my $width = 5**$k - 5**($k-1);
  my $r = $n - $high;
  ### $high
  ### $r
  ### $width
  return ($r ? "$k+$r/$width" : $k);
}
CHECK {
  n_to_x(0) eq 0 or die;
  n_to_x(1) eq 1 or die;
  n_to_x(4) eq '1+3/4' or die;
  n_to_x(5) eq 2 or die;
  n_to_x(24) eq '2+19/20' or die;
}

# plot line
my $y_max = 0;
my $prev_y = dir(0);
$Printwrap = 2;
printwrap("\\draw (0,$prev_y)");
$Printwrap += 2;
my $n_hi = 5**$level-1;
foreach my $n (1 .. $n_hi) {
  my $y = dir($n);
  my $x = n_to_x($n);
  printwrap("--($x,$prev_y)");
  printwrap("--($x,$y)");
  $prev_y = $y;
  $y_max = max($y,$y_max);
}
my $x_max = n_to_x($n_hi+1);
printwrap("--($x_max,$prev_y);\n");

print "  \\coordinate (x-max) at ($x_max,0);\n";
print "  \\coordinate (y-max) at (0,$y_max);\n";

# X ticks
$Printwrap = 2;
printwrap("\\draw (0,0)");
$Printwrap += 2;
foreach my $k (-1 .. $level) {
  my $n = ($k<0 ? 0 : 5**$k);
  my $x = n_to_x($n);
  my $semi = ($k==$level ? ";\n" : "");
  printwrap(" ($x,0) -- +(x-tick)$semi");
}

# X numbers
print "  \\begin{scope}[font=\\scriptsize]\n";
foreach my $k (-1 .. $level-1) {
  my $n = ($k<0 ? 0 : 5**$k);
  my $x = n_to_x($n);
  print "  \\node at ($x,0) [below right,my x number] {\$$n\$};\n";
}

# Y ticks and numbers
$Printwrap = 2;
printwrap("\\draw (0,0)");
$Printwrap += 2;
foreach my $y (1 .. $y_max) {
  my $semi = ($y==$y_max ? ";\n" : "");
  printwrap(" (0,$y) -- +(y-tick) node[at start,left] {\$$y\$}$semi");
}
print "  \\end{scope}\n";

print "  % end generated\n";
exit 0;
