#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon;
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

my $level = 6;
if (@ARGV) {
  $level = shift @ARGV;
}

my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

print "  \\begin{scope}[rotate={-atan2($y_hi,$x_hi)},scale={1/sqrt(5)^$level}]\n";

my $R = 0;
my $boundary_segments = MyLineSegments->new;
my $both_boundary_segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my $right = $path->_UNDOCUMENTED__n_segment_is_right_boundary($n,$level);
  my $left  = $path->_UNDOCUMENTED__n_segment_is_left_boundary($n,$level);
  if (! $right && ! $left) { next; }
  if ($right) { $R++; }

  my ($x,$y) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);

  my $s = ($right && $left
           ? $both_boundary_segments
           : $boundary_segments);
  $s->add($x,$y, $x2,$y2);
}

{
  $Printwrap = 2;
  printwrap("\\path[pic actions] ");
  $Printwrap += 2;
  $boundary_segments->printwrap;
  printwrap(";");
  my $count = $boundary_segments->lines_joined_count_points;
  printwrap(" % $count points\n");
}

{
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $both_boundary_segments->printwrap;
  printwrap(";");
  my $count = $both_boundary_segments->lines_joined_count_points;
  printwrap(" % $count points\n");
}

print "  \\end{scope}\n";
print "  % end generated\n";
exit 0;
