#!/usr/bin/perl -w

# Copyright 2017, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::Geometry::Planar;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()
use Math::Trig 'rad2deg';

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon 'bigint_class';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 12;

# smallest side
# my(k=12); printf("%.20f\n", 3/sqrt(5)^k)

print "  % generated by tools/$FindBin::Script\n";

my $ex = bigint_class()->new(1);
my $ey = bigint_class()->new(0);
my $points = [ [bigint_class()->new(0), bigint_class()->new(0)],
               [$ex,$ey] ];
foreach (1 .. $level) {
  ($points,$ex,$ey) = MyR5Dragon::hull_expand($points,$ex,$ey);
}

require Math::BigFloat;
$ex =   Math::BigFloat->new($ex);
$ey = - Math::BigFloat->new($ey);

my $power = Math::BigFloat->new(5)->bpow($level);
$ex /= $power;
$ey /= $power;

foreach my $point (@$points) {
  my ($x,$y) = @$point;
  ($x,$y) =  ($ex*$x - $ey*$y,
              $ex*$y + $ey*$x);
  # GP-Test  ('x+'y*I)*('ex+'ey*I) == \
  # GP-Test    ('ex*'x - 'ey*'y) \
  # GP-Test  + ('ex*'y + 'ey*'x)*I

  $x = $x->numify;
  $y = $y->numify;
  @$point = ($x,$y);
}

sub decimal {
  my ($x,$places) = @_;
  $places //= 6;
  $x = sprintf '%.*f', $places, $x;
  $x =~ s/\.?0+$//;
  $x =~ s/^0\././;
  return $x;
}

$Printwrap = 2;
printwrap("\\draw");
$Printwrap += 2;
foreach my $p (@$points) {
  ### $p
  my ($x,$y) = @$p;
  $x = decimal($x);
  $y = decimal($y);
  printwrap(" ($x,$y) --");
  #  printwrap(" ($x,$y) node[my dot] --");
}
printwrap(" cycle\n");

foreach my $i (0 .. $#$points) {
  my $p1 = $points->[$i-2];
  my $p2 = $points->[$i-1];
  my $p3 = $points->[$i];
  my ($p1x,$p1y) = @$p1;
  my ($p2x,$p2y) = @$p2;
  my ($p3x,$p3y) = @$p3;
  ### points: "i=$i  $p1x $p1y   $p2x $p2y   $p3x $p3y"

  my $s1x = $p2x - $p1x;
  my $s1y = $p2y - $p1y;
  my $s2x = $p3x - $p2x;
  my $s2y = $p3y - $p2y;

  my $a1 = rad2deg(atan2($s1y,$s1x));
  my $a2 = rad2deg(atan2($s2y,$s2x));
  if ($a2 < $a1) { $a2 += 360; }
  my $a = ($a1 + $a2) / 2;   # mean
  ### angles: "$a1 and $a2 mean $a"
  $a -= 90;                  # outward perpendicular
  $a = decimal($a,3);
  $p2x = decimal($p2x);
  $p2y = decimal($p2y);
  printwrap_newline();
  printwrap(" ($p2x,$p2y)");
  $Printwrap += 2;
  printwrap(" ++($a:\\MyTickFrom)");
  printwrap(" --");
  printwrap(" +($a:\\MyTickLen)");
  $Printwrap -= 2;
}
printwrap(";\n");

my $shortest;
foreach my $i (0 .. $#$points) {
  my $p1 = 
  my $p2 = $points->[$i];
  my $len = Math::Geometry::Planar::SegmentLength
    ([ $points->[$i-1], $points->[$i] ]);
  $shortest = min($shortest//$len, $len);
}
$shortest = sprintf '%.8f', $shortest;

my $num_points = scalar(@$points);
print "  % k=$level, $num_points points, shortest $shortest\n";
print "% GP-Test  HnumSides($level) == $num_points\n";

print "  % end generated\n";
exit 0;
