#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon 'bigint_class';
use MyPrintwrap ':all';


my $max_k = 10;
if (@ARGV) { $max_k = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_k\n";

my $zero = bigint_class()->new(0);
my $one  = bigint_class()->new(1);
my $points = [ [$zero,$zero], [$one,$zero] ];
my $ex = $one;
my $ey = $zero;

print "% GP-DEFINE  \\\\ convex hulls around R5 curve k, first side horizontal\n";
printwrap_indent("% GP-DEFINE  ");
printwrap("hull_generated_samples = {[\n");
foreach my $k (0 .. $max_k) {
  $Printwrap = 2;
  printwrap("/* k=$k */ [ ");
  $Printwrap += 2;
  foreach my $i (0 .. $#$points) {
    my $p = $points->[$i];
    my ($x,$y) = @$p;
    my $comma = ($i==$#$points ? '' : ',');
    printwrap("$x+I*$y$comma");
  }
  my $comma = ($k==$max_k ? '' : ',');
  printwrap(" ]$comma\n");

  ($points, $ex,$ey) = MyR5Dragon::hull_expand($points, $ex,$ey);
}

print "% GP-DEFINE  ]};\n";
print "% end generated\n";

exit 0;
