#!/usr/bin/perl -w

# Copyright 2017, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon;
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 4;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

{
  my $path = Math::PlanePath::R5DragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  my ($mx,$my) = ($x_hi/2, $y_hi/2);

  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi-1) {
    $segments->add($path->n_to_xy($n),
                   $path->n_to_xy($n+1));
  }
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap = 4;
  $segments->printwrap;
  printwrap(";\n");

  my $xm = $x_hi / 2;
  my $ym = $y_hi / 2;
  print "  \\coordinate (middle) at ($xm,$ym);\n";
  print "  \\coordinate (end) at ($x_hi,$y_hi);\n";
}

{
  my $ex = 1;
  my $ey = 0;
  my $points = [ [0,0], [$ex,$ey] ];
  foreach (1 .. $level) {
    ($points,$ex,$ey) = MyR5Dragon::hull_expand($points,$ex,$ey);
  }

  ### $points
  push @$points, shift @$points;

  $Printwrap = 2;
  printwrap("\\draw[dashed]");
  $Printwrap += 2;
  my $join = '';
  my ($prev_x,$prev_y) = @{$points->[-1]};
  my $i = 1;
  foreach my $p (@$points) {
    my ($x,$y) = @$p;

    my $dx = $x - $prev_x;
    my $dy = $y - $prev_y;
    ### dxdy: "$dx, $dy"
    if ($dx==0 || $dy==0) {
      $join = '';
    } else {
      $join = ' --';
    }
    $prev_x = $x;
    $prev_y = $y;

    printwrap("$join ($x,$y) node[my dot,name=p$i]");
    $i++;
    $i %= scalar(@$points);
  }
  printwrap(";\n");

  my $sides = scalar(@$points);
  print "  \\newcommand\\MySides{$sides}\n";
  print "% GP-Test  #Hsides_by_generated($level) == $sides\n";
}

print "  % end generated\n";
exit 0;
