#!/usr/bin/perl -w

# Copyright 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Graph::Maker::QuartetTree;

use lib $FindBin::Bin;
use MyR5Dragon 'xy_to_node_name';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 2;

print "  % generated by tools/$FindBin::Script\n";

my %set = ('1,0' => 1,      # left
           '2,-1' => 1,
           '2,0' => 1,

           '4,0' => 1,      # middle
           '4,1' => 1,

           '3,3' => 1,      # top
           '3,2' => 1,
           '2,2' => 1,

           '6,2' => 1,      # right
           '5,2' => 1,

           '4,-1' => 1,     # bottom;
           '5,-2' => 1,
           '5,-1' => 1);
my $QuartetTotdomnum = scalar keys %set;

$Printwrap = 2;
my $graph = Graph::Maker->new('quartet_tree',
                              level => $level,
                              undirected=>1);

V: foreach my $v (sort $graph->vertices) {
  foreach my $neighbour ($graph->neighbours($v)) {
    next V if $set{$neighbour};
  }
  die "oops, not total dominating set";
}

foreach my $v (sort $graph->vertices) {
  my ($x,$y) = split /,/, $v;
  my $name = xy_to_node_name($x,$y);
  my $dot = 'open';
  if ($set{$v}) {
    delete $set{$v};
    $dot = 'set';
  }
  printwrap(" \\node ($name) at ($x,$y) [my $dot dot];");
}

my @edges;
foreach my $edge ($graph->edges) {
  my ($from,$to) = sort @$edge;
  my $from_name = xy_to_node_name(split /,/, $from);
  my $to_name   = xy_to_node_name(split /,/, $to);
  push @edges, " ($from_name)--($to_name)";
}
$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
foreach my $edge (sort @edges) {
  printwrap($edge);
}
printwrap(";\n");

print "% GP-Test-Last  QuartetTotdomnum($level) == $QuartetTotdomnum\n";
print "  % end generated\n";
exit 0;
