#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use List::Util 'min','max';
use Math::PlanePath::R5DragonCurve;

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

my $segments = MyLineSegments->new;

my $width = 10;
my $height = 7;

# horizontal bricks
foreach my $i (0 .. $height-1) {
  my $x = $i;
  my $y = 2*$i;
  foreach my $j (0 .. $width-2) {

    $segments->add($x,$y, $x+2,$y);
    $segments->add($x+2,$y, $x+2,$y-1);
    $segments->add($x,$y-1, $x+2,$y-1);
    $segments->add($x,$y-1, $x,$y);

    $x += 2;
    $y -= 1;
  }
}

# vertical bricks
foreach my $i (0 .. $width-1) {
  my $x = 2*$i;
  my $y = -$i;
  foreach my $j (0 .. $height-2) {

    $segments->add($x,$y, $x+1,$y);
    $segments->add($x+1,$y, $x+1,$y+2);
    $segments->add($x+1,$y+2, $x,$y+2);
    $segments->add($x,$y+2, $x,$y);

    $x += 1;
    $y += 2;
  }
}

# rotate by +arctan(1/2)
sub transform {
  my ($x,$y) = @_;
  return ($x*.4 + $y*-.2,
          $x*.2 + $y*.4);
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap(transform => \&transform);
printwrap(";\n");

my $tx_min;
my $ty_min;
my $tx_max;
my $ty_max;
foreach my $line ($segments->lines) {
  my ($x1,$y1, $x2,$y2) = @$line;
  ($x1,$y1) = transform($x1,$y1);
  ($x2,$y2) = transform($x2,$y2);

  $tx_min = min($tx_min//$x1, $x1,$x2);
  $ty_min = min($ty_min//$y1, $y1,$y2);
  $tx_max = max($tx_max//$x1, $x1,$x2);
  $ty_max = max($ty_max//$y1, $y1,$y2);
}

print "  \\foreach \\x in {0,...,",$width-1,"} \\foreach \\y in {0,...,",$height-1,"}\n";
print "    { \\fill (\\x,\\y) circle; }\n";

print "  \\coordinate (bottom-right) at ($tx_max,$ty_min);\n";
print "  \\coordinate (top-right)    at ($tx_max,$ty_max);\n";

print "  % end generated\n";
exit 0;
