\\ Copyright 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ THIS IS A GENERATED FILE
\\ Extracted from r5dragon.tex by gp-inline.
\\
\\ These are Pari/GP functions and definitions used in r5dragon.tex for
\\ various self-tests and formula checking there.  Names and styles are
\\ highly specific to the calculations there, and many things will change or
\\ disappear, etc, but some of the functions might be useful for
\\ experimenting or testing.

/* ------------------------------------------------------------------------- */
/* gp-inline test boilerplate begin */
gp_inline__location = "";
gp_inline__good = 0;
gp_inline__bad = 0;
gp_inline__check(location,bool) =
{
  gp_inline__location = location;
  check(bool);
}
/* this is thinking about a check() for use in loops or "GP-Inline ..."
   testing, but it's not settled so don't rely on it */
check(bool) =
{
  /* use "===" so that a vector like [1] or float 1.0 not reckoned success */
  if(bool===1, gp_inline__good++,
     bool===0, gp_inline__bad++;
               print(gp_inline__location": gp-inline fail"),
     gp_inline__bad++;
     print(gp_inline__location": gp-inline expected result 0 or 1, got ",
           bool));
}
/* gp-inline test boilerplate end */
/* ------------------------------------------------------------------------- */

gp_inline__location="../r5dragon.tex:211";
 default(strictargs,1);

gp_inline__location="../r5dragon.tex:219";
 \\ my http://user42.tuxfamily.org/pari-geom/index.html
 iferr(read("tools/geom.gp"), e, \
    read("../tools/geom.gp"));
 iferr(read("tools/geom-inertia.gp"), e, \
    read("../tools/geom-inertia.gp"));

gp_inline__location="../r5dragon.tex:225";
 \\ my http://user42.tuxfamily.org/pari-memoize/index.html
 iferr(read("tools/memoize.gp"),e, \
    read("../tools/memoize.gp"));

gp_inline__location="../r5dragon.tex:229";
 \\ my http://user42.tuxfamily.org/pari-vpar/index.html
 iferr(read("tools/vpar.gp"),      e,read("../tools/vpar.gp"));
 iferr(read("tools/vpar-wip.gp"),  e,read("../tools/vpar-wip.gp"));

gp_inline__location="../r5dragon.tex:233";
 iferr(read("tools/vecper.gp"),e, \
    read("../tools/vecper.gp"));
 iferr(read("tools/r5dragon-oeis-samples.gp"),e, \
    read("../tools/r5dragon-oeis-samples.gp"));

gp_inline__location="../r5dragon.tex:238";
 degrees(x) = x/180*Pi;
 to_degrees(x) = x/Pi*180;
 vector_modulo(v,i) = v[(i% #v)+1];

gp_inline__location="../r5dragon.tex:242";
 differ(x,y,epsilon) =
 {
   \\ if(type(x)!=type(y),
   \\    return(Strprintf("different types %s %s",type(x),type(y))));
   if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
      my(d=differ(real(x),real(y),epsilon));
      if(d != 0, return(Str("real ",d)));
      d=differ(imag(x),imag(y),epsilon);
      if(d != 0, return(Str("imag ",d)));
      return(0));

   if(type(x)=="t_VEC" || type(x)=="t_COL",
      if(length(x)!=length(y),
         return(Str("different lengths ",length(x)," ",length(y))));
      for(i=1,#x,
         my(d=differ(x[i],y[i],epsilon));
         if(d != 0, return(Str("at index "i" "d))));
      return(0));

   if(! differ_equal(x,y,epsilon),
      return(Strprintf("different values\n",x,"\n",y)));
   return(0);
 }
 differ_equal(x,y,epsilon) = abs(x - y) < epsilon;

gp_inline__location="../r5dragon.tex:267";
 nearly_equal_epsilon = 1e-15;
 nearly_equal(x,y, epsilon=nearly_equal_epsilon) = {
   \\ print(type(x)," ",type(y)," ",x," ",y);
   my(d=x-y); \\ ensure compatible types
   if(type(x)=="t_MAT",x=Vec(x);y=Vec(y));
   if(type(x)=="t_VEC" || type(x)=="t_COL",
        sum(i=1,max(#x,#y), !nearly_equal(x[i],y[i],epsilon)) == 0,
      type(x)=="t_QUAD" || type(x)=="t_COMPLEX",
           nearly_equal(real(x-y),0,epsilon)
        && nearly_equal(imag(x-y),0,epsilon),
      abs(d) < epsilon);
 }

gp_inline__location="../r5dragon.tex:280";
 \\ n is an integer.  Return an integer whose decimal digits are
 \\ the base-5 digits of n.
 to_base5(n) = fromdigits(digits(n,5),10);

gp_inline__location="../r5dragon.tex:286";
 \\ n is an integer whose decimal digits are in the range 0-4
 \\ and are taken to be base-5.  The return is their value
 \\ in  base-5.
 from_base5(n) = {
   my(v=digits(n,10));
   for(i=1,#v,if(v[i]>4,error("from_base5() bad digits: ",v)));
   fromdigits(v,5);
 }

gp_inline__location="../r5dragon.tex:295";
 \\ n is an integer.  Return an integer whose decimal digits
 \\ are the ternary digits of n.
 to_ternary(n) = fromdigits(digits(n,3),10);

gp_inline__location="../r5dragon.tex:301";
 \\ n is an integer with decimal digits representing bits.
 \\ Return the value of those bits.
 from_binary(n) = fromdigits(digits(n),2);

gp_inline__location="../r5dragon.tex:307";
 \\ n is an integer.  Return an integer whose decimal digits
 \\ are the bits of n.
 to_binary(n)=fromdigits(binary(n));

gp_inline__location="../r5dragon.tex:313";
 \\ n is an integer with decimal digits representing octal digits.
 \\ Return the value of those bits.
 from_octal(n)=fromdigits(digits(n),8);

gp_inline__location="../r5dragon.tex:320";
 \\ n is an integer.  Return an integer whose decimal digits
 \\ are the octal digits of n.
 to_octal(n)=fromdigits(digits(n,8));

gp_inline__location="../r5dragon.tex:327";
 fromdigits_any(v,base) = my(ret=0); for(i=1,#v, ret=ret*base+v[i]); ret;

gp_inline__location="../r5dragon.tex:329";
 \\ v is a vector of fractional decimal digits high to low.
 \\ Eg. [1,2,3] is 123/1000 = 0.123
 \\ Return them as a rational like 12981/10^5.
 \\ If v is longer than the current floating point precision(),
 \\ less a little, then use only that much of v.
 \\ "point" is how many digits before the decimal point.
 \\ "add" is an amount to add at the low end of the digits.
 fromdigits_to_precision(v,point=0, add=0) = \
   my(p = precision(1.0) - 4); \
   if(#v>p, v = v[1..p]); \
   (fromdigits(v)+add) / 10^(#v-point);

gp_inline__location="../r5dragon.tex:344";
 \\ Like digits() but pad with high 0s to make k many digits.
 \\ Must have n < base^k.
 digits_padded(n,k,base=10) = {
   my(v = digits(n,base));
   if(#v>k, error("digits_padded() n=",n,
                  " is bigger than k=",k," digits"));
   concat(vector(max(0,k-#v)),v);
 }

gp_inline__location="../r5dragon.tex:357";
 \\ n is an integer.  Return its bits as a vector high to low
 \\ padded out with high 0s to make k many bits.
 \\ n must fit within k bits, ie. 0 <= n < 2^k.
 binary_padded(n,k) = {
   my(v = binary(n));
   if(#v>k, error("binary_padded() n=",n," is bigger than k=",k," bits"));
   concat(vector(max(0,k-#v)),v);
 }

gp_inline__location="../r5dragon.tex:369";
 vector_mean(v) = vecsum(v)/#v;

gp_inline__location="../r5dragon.tex:372";
 select_first_n(f,n) = {
   my(l=List([]), i=0);
   while(#l<n, if(f(i),listput(l,i)); i++);
   Vec(l);
 }

gp_inline__location="../r5dragon.tex:378";
 \\ Return a vector of the first n coefficients of
 \\ generating function g, starting from term x^0.
 \\ g is normally a polynomial fraction (type t_RFRAC)
 \\ but a plain polynomial is accepted too and returns
 \\ its lowest n coefficients.
 gf_terms(g,n) = \
   my(x = variable(g), \
      zeros = min(n,valuation(g,x)), \
      v = Vec(g + O(x^n))); \
   if(zeros>=0, concat(vector(zeros,i,0), v), \
                v[-zeros+1 .. #v]);

gp_inline__location="../r5dragon.tex:397";
 gf_term(g,n) = gf_terms(g,n+1)[n+1];

gp_inline__location="../r5dragon.tex:399";
 \\ Generating function of all ones 1,1,1,1,1,1,...
 gOnes(x)=1/(1-x);

gp_inline__location="../r5dragon.tex:403";
 sqrt2 = geom_sqrt_as_quad(2);
 sqrt5 = geom_sqrt_as_quad(5);
 sqrt10 = geom_sqrt_as_quad(10);
 sqrt17 = geom_sqrt_as_quad(17);
 sqrt85 = geom_sqrt_as_quad(85);

gp_inline__location="../r5dragon.tex:414";
 is_integer(x) = (x==floor(x));

gp_inline__location="../r5dragon.tex:416";
 \\ return true if z is an Gaussian integer
 z_is_integer(z) = is_integer(real(z)) && is_integer(imag(z));

gp_inline__location="../r5dragon.tex:419";
 \\ Return vector v with one copy of elem deleted.
 \\ v must contain at least one copy of elem (or an error).
 vector_delete_must(v,elem) = {
   for(i=1,#v,if(v[i]==elem,return(v[^i])));
   error("does not contain ",elem);
 }

gp_inline__location="../r5dragon.tex:426";
 \\ Return vector v with each element of vector elems deleted.
 \\ One copy of each of elems is deleted, and v must contain enough
 \\ such copies (or an error).
 vector_deletes_must(v,elems) = {
   for(i=1,#elems, v=vector_delete_must(v,elems[i]));
   v;
 }

gp_inline__location="../r5dragon.tex:434";
 \\ Return a vector which is n many repeats of v concatenated.
 \\ If n=0 then return an empty vector.
 vector_repeats(v,n) = {
   if(n==0,return([]));
   concat(vector(n,i,v));
 }

gp_inline__location="../r5dragon.tex:441";
 \\ Return a vector which is all the cross sums u[i]+v[j].
 vector_cross_sums(u,v) = {
   if(#u==0 || #v==0,
      [],
      concat(vector(#u,i, vector(#v,j, u[i]+v[j]))));
 }

gp_inline__location="../r5dragon.tex:450";
 vector_cumulative(v) = for(i=2,#v,v[i]+=v[i-1]); v;

gp_inline__location="../r5dragon.tex:456";
 \\ periodic 0,2,4 1,3
 A257164(n) = (2*n)%5;

gp_inline__location="../r5dragon.tex:465";
 LowestNon0(n) = {
   n>=1 || error("LowestNon0() is for n>=1");
   while(n%5==0, n\=5); n%5;
 }
 A277543(n) = LowestNon0(n);

gp_inline__location="../r5dragon.tex:489";
 gLowestNon0(x,ksize) = {
   sum(k=0,ksize-1,
       (x^(5^k) + 2*x^(2*5^k) + 3*x^(3*5^k) + 4*x^(4*5^k))
       / (1 - x^(5^(k+1))));
 }

gp_inline__location="../r5dragon.tex:500";
 Contains3or4(n) = while(n, if(n%5>=3,return(1)); n\=5); 0;
 IsBase5all01(n) = while(n, (n%5)<=1 || return(0); n\=5); 1;

gp_inline__location="../r5dragon.tex:508";
 A277548_vector(limit) = {
   my(l=List([]));
   for(k=0,limit,
     my(p=5^k); if(p>=limit,break);
     for(m=0,limit,
       my(e=(5*m+4)*p); if(e>=limit,break);
       listput(l,e)));
   vecsort(Vec(l));
 }

gp_inline__location="../r5dragon.tex:537";
 LowestNon1(n) = {
   n>=0 || error("LowestNon1() is for n>=0");
   my(r); until(r!=1, [n,r]=divrem(n,5)); r;
 }

gp_inline__location="../r5dragon.tex:565";
 gLowestNon1(x,ksize) = {
   sum(k=0,ksize-1,
       x^((5^k-1)/4) * (  2*x^(2*5^k)
                        + 3*x^(3*5^k)
                        + 4*x^(4*5^k)
                       ) / (1 - x^(5^(k+1))));
 }

gp_inline__location="../r5dragon.tex:592";
 LowestNon2(n) = {
   n>=0 || error("LowestNon2() is for n>=0");
   my(r); until(r!=2, [n,r]=divrem(n,5)); r;
 }

gp_inline__location="../r5dragon.tex:641";
 gLowestNon2_term(x,k) = {
   x^((5^k-1)/2) * (    x^(  5^k)
                    + 3*x^(3*5^k)
                    + 4*x^(4*5^k)
                   ) / (1 - x^(5^(k+1)));
 }

gp_inline__location="../r5dragon.tex:649";
 gLowestNon2(x,ksize) = sum(k=0,ksize-1, gLowestNon2_term(x,k));

gp_inline__location="../r5dragon.tex:676";
 LowestNon3(n) = {
   n>=0 || error("LowestNon3() is for n>=0");
   my(r); until(r!=3, [n,r]=divrem(n,5)); r;
 }

gp_inline__location="../r5dragon.tex:698";
 gLowestNon3(x,ksize) = {
   sum(k=0,ksize-1,
       x^(3*(5^k-1)/4) * (    x^(  5^k)
                          + 2*x^(2*5^k)
                          + 4*x^(4*5^k)
                         ) / (1 - x^(5^(k+1))));
 }

gp_inline__location="../r5dragon.tex:722";
 LowestNon4(n) = {
   n>=0 || error("LowestNon4() is for n>=0");
   my(r); until(r!=4, [n,r]=divrem(n,5)); r;
 }

gp_inline__location="../r5dragon.tex:740";
 gLowestNon4(x,ksize) = {
   sum(k=0,ksize-1,
       x^(5^k-1) * (    x^(  5^k)
                    + 2*x^(2*5^k)
                    + 3*x^(3*5^k)
                   ) / (1 - x^(5^(k+1))));
 }

gp_inline__location="../r5dragon.tex:767";
 b = 1 + 2*I;

gp_inline__location="../r5dragon.tex:769";
 r5dragon_points(k) = {
   if(k==0,return([0,1]));
   my(v=r5dragon_points(k-1),
      p1=v[#v],
      p2=(1+I)*p1,
      p3=   I *p1,
      p4= 2*I *p1);
   v=v[1..#v-1];
   concat([v,
           apply(z->I*z+p1,v),
           apply(z->-z +p2, v),
           apply(z->I*z+p3,v),
           apply(z->z  +p4, v),
           [b^k]]);
 }

gp_inline__location="../r5dragon.tex:2874";
 turn(n) = {
   n>=1 || error("turn() is for n>=1");
   if(LowestNon0(n)<=2, 1, -1);
 }

gp_inline__location="../r5dragon.tex:2885";
 turn_samples_str = \
   "{+}{+}{-}{-}{+}{+}{+}{-}{-}{+}{+}{+}{-}{-}{-}{+}{+}{-}{-}{-}{+}{+}{-}{-}";
 turn_samples = [ 1,1,-1,-1,1,1,1,-1,-1,1,1,1,-1,-1,-1,1,1,-1,-1,-1,1,1,-1,-1 ];

gp_inline__location="../r5dragon.tex:2907";
 LowestNon0_samples = {[1,2,3,4,1,1,2,3,4,2,1,2,3,4,3,1,2]};

gp_inline__location="../r5dragon.tex:2929";
 next_turn(n) = {
   n>=0 || error("next_turn() is for n>=0");
   if(LowestNon4(n)<=1, 1, -1);
 }

gp_inline__location="../r5dragon.tex:2945";
 LowestNon4_samples = {[0,1,2,3,0,0,1,2,3,1,0,1,2,3,2,0,1]};

gp_inline__location="../r5dragon.tex:3027";
 TurnLpred(n) = if(n>=1, LowestNon0(n) <= 2);

gp_inline__location="../r5dragon.tex:3030";
 TurnLpred_samples = {[1,1,0,0,1,1,1,0,0,1,1,1,0,0,0,1,1]};

gp_inline__location="../r5dragon.tex:3050";
 TurnRpred(n) = if(n>=1, LowestNon0(n) >= 3);

gp_inline__location="../r5dragon.tex:3053";
 TurnRpred_samples = {[0,0,1,1,0,0,0,1,1,0,0,0,1,1,1,0,0]};

gp_inline__location="../r5dragon.tex:3066";
 \\ A175337 morphism with offset=0 so "next turn" style numbering
 \\ turns starting n=0 for the first turn
 A175337(n) = {
   n>=0 || error("A175337 is for n>=0");
   TurnRpred(n+1);
 }

gp_inline__location="../r5dragon.tex:3084";
 gA175337(x,limit) = {
   sum(k=0,limit,
     x^(3*5^k - 1) * (1 + x^(5^k)) / (1 - x^(5^(k+1)))
   );
 }

gp_inline__location="../r5dragon.tex:3116";
 gTurnLpred_term(x,k) = (x^(5^k) + x^(2*5^k)) / (1 - x^(5^(k+1)));
 gTurnLpred_to_k(x,k) = sum(j=0,k, gTurnLpred_term(x,j));

gp_inline__location="../r5dragon.tex:3130";
 gTurnRpred_term(x,k) = (x^(3*5^k) + x^(4*5^k)) / (1 - x^(5^(k+1)));
 gTurnRpred_to_k(x,k) = sum(j=0,k, gTurnRpred_term(x,j));

gp_inline__location="../r5dragon.tex:3169";
 gturn_unsimplified_term(x,k) = {
   (x^(5^k) + x^(2*5^k) - x^(3*5^k) - x^(4*5^k) )
   / (1 - x^(5^(k+1)));
 }

gp_inline__location="../r5dragon.tex:3185";
 gturn_term(x,k) = {
   x^(5^k) * ( 1 + x^(5^k) )^2
   / (1 + x^(5^k) + x^(2*5^k) + x^(3*5^k) + x^(4*5^k));
 }
 gturn_to_k(x,k) = sum(j=0,k, gturn_term(x,j));

gp_inline__location="../r5dragon.tex:3220";
 turn01(n) = if(turn(n)<0,0,1);
 gturn01(x,k) = 1/2*(gOnes(x) + gturn_to_k(x,k)) - 1/2;

gp_inline__location="../r5dragon.tex:3234";
 gturn_by_expand(x,k) = gturn_to_k(x^5,k-1) \
   + x/(1-x^5) + x^2/(1-x^5) - x^3/(1-x^5) - x^4/(1-x^5);
 gturn_by_expand2(x,k) = gturn_to_k(x^5,k-1) \
   + (x^3 + 2*x^2 + x)/(x^4 + x^3 + x^2 + x + 1);

gp_inline__location="../r5dragon.tex:3241";
 turnlimit(k,x) = {
   if(k==0,return(0));
   turnlimit(k-1,x^5) + (x^3 + 2*x^2 + x)/(x^4 + x^3 + x^2 + x + 1);
 }

gp_inline__location="../r5dragon.tex:3246";
 turnlimit_term(k,x) = \
   x=x^(5^k); (x^3 + 2*x^2 + x)/(x^4 + x^3 + x^2 + x + 1);

gp_inline__location="../r5dragon.tex:3276";
 A175337_F_vector(k) = {
   my(v=['F]);
   for(i=1,k,
     v=concat(apply(s->
       if(s=='F, ['F, 0, 'F, 0, 'F, 1, 'F, 1, 'F], [s]), v)));
   v;
 }

gp_inline__location="../r5dragon.tex:3312";
 \\ with 10 digits worth of 0s as 3s
 n_to_3to7(n) = fromdigits(apply(d->d+3,digits_padded(n,10,5)),8);

gp_inline__location="../r5dragon.tex:3346";
 turn_of_3to7(u) = if(LowOctal4Middle(u)==0,1,-1);

gp_inline__location="../r5dragon.tex:3352";
 LowOctal4Middle(n) = bitand(n, (MaskToLowOctal4(n) + 1)/4);

gp_inline__location="../r5dragon.tex:3360";
 MaskToLowOctal4(u) = {
   MaskToLowBit1
   (bitand(u, from_binary(100100100100100100100100100100)));
 }

gp_inline__location="../r5dragon.tex:3370";
 MaskToLowBit1(u) = bitxor(u,u-1);

gp_inline__location="../r5dragon.tex:3383";
 increment_3to7(u) = bitor(u+1, LowThrees(u+1));

gp_inline__location="../r5dragon.tex:3393";
 LowThrees(u) = {
   bitand(floor(MaskToLowOctal4(u) / 8),
          from_binary(011011011011011011011011011011011011011));
 }

gp_inline__location="../r5dragon.tex:3402";
 LowThrees_by_digits(u) = {
   my(v=digits(u,8),ret=0);
   forstep(i=#v,1,-1, if(v[i]==0,ret=8*ret+3,break()));
   ret;
 }

gp_inline__location="../r5dragon.tex:3447";
 IsPow2_by_valuation(n) = n>0 && n==2^valuation(n,2);

gp_inline__location="../r5dragon.tex:3449";
 IsPow2_by_bitand(n) = bitand(n,n-1) == 0; /* true for n=0 */

gp_inline__location="../r5dragon.tex:3454";
 MaskAboveLowestOne(n) = 1 + bitxor(n,n-1);
 IsPow2_by_mask(n) = n < MaskAboveLowestOne(n);

gp_inline__location="../r5dragon.tex:3520";
 TurnRpred_expand = {[ [0,0,1,1,0],
                       [0,0,1,1,1] ]};
 \\ vector of n=1 to n=len inclusive
 TurnRpred_vec_by_morphism(len) = {
   my(v=[0]);
   while(#v<len, v=concat(apply(d->TurnRpred_expand[d+1],v)));
   v[1..len];
 }

gp_inline__location="../r5dragon.tex:3537";
 \\ return a vector of the run lengths of turns in level k
 TurnRun_vector(k) = {
   if(k==0,return([]));
   my(prev=turn(1), len=1, l=List([]));
   for(n=2,5^k-1,
       my(t = turn(n));
       if(t==prev, len++,          \\ continue run
          listput(l,len); len=1);  \\ new run
       prev=t);
   listput(l,len);
   Vec(l);
 }
 TurnRun_vector_by_middle(k) = {
   if(k==0,return([]));
   my(v=vector(2*5^(k-1),n, 2));
   forstep(i=2,#v-2,2,
     if(turn(i/2)==1, v[i+1]++,  \\ left, inc prev
                      v[i]++));  \\ right, inc next
   v;
 }

gp_inline__location="../r5dragon.tex:3563";
 TurnRun_by_pairs(j) = {
   if(j==0, return(2));
   j++;  \\ to pairs 2,3 etc around turn of prev level
   2 + ((turn(j\2)==1) == (j%2));
 }

gp_inline__location="../r5dragon.tex:3570";
 vector_run_lengths(v) = {
   if(#v==0,return([]));
   my(l=List([]),run=1);
   for(i=2,#v, if(v[i]==v[i-1],run++, listput(l,run);run=1));
   listput(l,run);
   Vec(l);
 }

gp_inline__location="../r5dragon.tex:3582";
 function_run_lengths(f,n) = {
   my(previous=f(1), run=1, v=vector(n), pos=1, i=2);
   while(pos<=#v,
         my(value=f(i));
         if(value==previous,
            run++,
            v[pos]=run; pos++; run=1);
         previous=value; i++);
   v;
 }

gp_inline__location="../r5dragon.tex:3614";
 TurnRun(m) = {
   if(m==0,   2,
      m%2==0, 5/2 + 1/2* turn(m/2),
              5/2 - 1/2* turn((m+1)/2));
 }

gp_inline__location="../r5dragon.tex:3632";
 TurnRun_by_negpow(m) = {
   if(m==0,   2,
      5/2 + 1/2 * (-1)^m * turn(ceil(m/2)));
 }

gp_inline__location="../r5dragon.tex:3639";
 TurnRun_samples = {[2,2,3,2,3,3,2,3,2,2,3,2,3,2,3]};

gp_inline__location="../r5dragon.tex:3655";
 turn_samples_2 = {[+1,+1,-1,-1,+1,+1,+1]};

gp_inline__location="../r5dragon.tex:3675";
 gTurnRun_by_gturn_to_k(x,k) = \
   -1/2 + 5/2/(1-x) + 1/2*(1-1/x)*gturn_to_k(x^2,k);

gp_inline__location="../r5dragon.tex:3730";
 TurnRunStart(m) = \
   5/2*m + if(m%2==0, 1-TurnLpred(5*m/2), 1/2);

gp_inline__location="../r5dragon.tex:3733";
 TurnRunSpred(n) = if(n==1 || turn(n)!=turn(n-1), 1, 0);
 TurnRunStart_by_pred(m) = \
   my(n=0); while(m>=0, n++; if(TurnRunSpred(n),m--)); n;

gp_inline__location="../r5dragon.tex:3743";
 TurnRunStart_samples = {[1,3,5,8,10,13,16,18,21,23,25,28,30]};

gp_inline__location="../r5dragon.tex:3893";
 TurnLeft_k(m) = {
   m>=1 || error("TurnLeft_k() is for m>=1");
   my(ret='oops);
   for(k=0,m, if(1/2*(5^k+1)<=m,
     ret=k,   \\ still below
     break)); \\ not below, stop
   ret;
 }

gp_inline__location="../r5dragon.tex:3907";
 TurnLeft(m) = {
   m>=0 || error("TurnLeft() is for m>=0");
   if(m==0,return(1));
   my(k=TurnLeft_k(m), l=List([]));
   if(m <= 5^k,
      listput(l, 5^k   + TurnLeft(m - 1/2*(5^k + 1))));
   if(5^k < m && m < 1/2*(3*5^k+1),
      listput(l, 2*5^k + TurnLeft(m - (5^k + 1))));
   if(1/2*(3*5^k+1) <= m && m < 2*5^k,
      listput(l, 3*5^k + TurnLeft(m - 1/2*(3*5^k + 1))));
   if(m >= 2*5^k,
      listput(l, 4*5^k + TurnLeft(m - 2*5^k)));
   #l==1 || error("oops");
   l[1];
 }

gp_inline__location="../r5dragon.tex:3926";
 TurnLeft_samples = {[1,2,5,6,7,10,11,12,16,17,21,22]};

gp_inline__location="../r5dragon.tex:3972";
 TurnRight_k(m) = {
   m>=0 || error("TurnRight_k() is for m>=1");
   my(ret='oops);
   for(k=0,m, if(1/2*(5^k-1)<=m,
     ret=k,   \\ still below
     break)); \\ not below, stop
   ret;
 }

gp_inline__location="../r5dragon.tex:3985";
 TurnRight(m) = {
   m>=0 || error("TurnRight() is for m>=0, got m=",m);
   my(k=TurnRight_k(m), l=List([]));
   if(m < 5^k-1,
      listput(l, 5^k   + TurnRight(m - 1/2*(5^k - 1))));
   if(5^k-1 <= m && m < 1/2*(3*5^k - 3),
      listput(l, 2*5^k + TurnRight(m - (5^k - 1))));
   if(m == 1/2*(3*5^k - 3),
      listput(l, 3*5^k));
   if(1/2*(3*5^k - 1) <= m && m < 2*5^k - 1,
      listput(l, 3*5^k + TurnRight(m - 1/2*(3*5^k - 1))));
   if(m == 2*5^k - 1,
      listput(l, 4*5^k));
   if(m >= 2*5^k,
      listput(l, 4*5^k + TurnRight(m - 2*5^k)));
   #l==1 || error("oops m=",m," k=",k," l=",Vec(l));
   l[1];
 }

gp_inline__location="../r5dragon.tex:4012";
 TurnRight_samples = {[3,4,8,9,13,14,15,18,19,20,23,24]};

gp_inline__location="../r5dragon.tex:4340";
 \\ Return base-5 digit at pos in integer n.
 \\ pos=0 is the least significant digit.
 base5_digit(n,pos) = (n\5^pos) % 5;

gp_inline__location="../r5dragon.tex:4394";
 TurnLeft_by_digits(m) = {
   my(n=2*m);
   forstep(pos=if(n,logint(n,5)),0,-1,   \\ high to low
     my(d=base5_digit(n,pos));
     \\ print("n="to_base5(n)" pos="pos" d="d);
     if(d==1 || d==3, n--,
        d==2, n-=2;
              if(base5_digit(n,pos)!=2,return(n+2))));
   \\ print("final n="to_base5(n)" = "n);
   n+1;
 }

gp_inline__location="../r5dragon.tex:4460";
 TurnRight_by_digits(m) = {
   my(n=2*m+2);
   forstep(pos=if(n,logint(n,5)),0,-1,   \\ high to low
     my(d=base5_digit(n,pos));
     if(d==1 || d==3, n++,
        d==2, n+=2;
              if(base5_digit(n,pos)!=2,return(n-1))));
   n;
 }

gp_inline__location="../r5dragon.tex:4585";
 TurnLeft_compact(n) = { my(v=digits(2*n,5),b,j);
   for(i=1,#v,
     if(b=[0,1,2,1,0][v[i]+1],
       j=#v; while((v[j]-=b)<0, v[j]+=5;b=1;j--);
       if(j==i,return(fromdigits(v,5)+2))));
   fromdigits(v,5)+1; }

gp_inline__location="../r5dragon.tex:4594";
 TurnRight_compact(n) = { my(v=digits(2*n+2,5),c,j);
   for(i=1,#v,
     if(c=[0,1,2,1,0][v[i]+1],
       j=#v; while((v[j]+=c)>4, v[j]-=5;c=1;j--);
       if(j==i,return(fromdigits(v,5)-1))));
   fromdigits(v,5); }

gp_inline__location="../r5dragon.tex:4665";
 count_digits(n,d) = length(digits(n,5));
 digit_to_dir(digit) = if(digit==1||digit==3,1, digit==2,2, 0);
 dir(n) = vecsum(apply(digit_to_dir, digits(n,5)));

gp_inline__location="../r5dragon.tex:4673";
 dir_samples = {[0,1,2,1,0,1,2,3,2,1,2,3,4,3,2,1,2,3,2,1]};

gp_inline__location="../r5dragon.tex:4688";
 \\ Pari code per Charles R Greathouse IV in A085004
 A085004(n) = 2*sum(k=1,n, (k+sqrtint(5*k^2))%4>1) - n;

gp_inline__location="../r5dragon.tex:4695";
 vector_find_subvec(v,sub) = \
   for(i=1,#v-#sub+1, if(v[i..i+#sub-1] == sub, return(i))); 0;

gp_inline__location="../r5dragon.tex:4716";
 gdir_term(x,k) = {
   x^(5^k) * ( 1 + x^(5^k) )^2
   / (1-x) /(1 + x^(5^k) + x^(2*5^k) + x^(3*5^k) + x^(4*5^k));
 }

gp_inline__location="../r5dragon.tex:4721";
 gdir_to_k(x,k) = sum(j=0,k, gdir_term(x,j));

gp_inline__location="../r5dragon.tex:4765";
 CountDigits(n,d) = my(v=digits(n,5)); sum(i=1,#v, v[i]==d);

gp_inline__location="../r5dragon.tex:5097";
 dir_new_highs_n_by_search(k) = {
   my(h=-1,l=List([]));
   for(n=0,5^k-1,
     my(d=dir(n));
     if(d>h,h=d;listput(l,n)));
   Vec(l);
 }

gp_inline__location="../r5dragon.tex:5123";
 dir_new_high_n(m) = {
   if(m==0,0,
      m--;
      if(m%2==0, m\=2; 1/2*(3*5^m-1),
                 m\=2; 1/2*(5*5^m-1)));
 }

gp_inline__location="../r5dragon.tex:5154";
 \\ base5 digits 122..22 and 222..22
 A238366(n) = (if(n%2,1,3)*5^(n\/2) - 1)/2;

gp_inline__location="../r5dragon.tex:5162";
 A094626(n) = (if(n%2,11,2)*10^(n\2) - 2)/9;

gp_inline__location="../r5dragon.tex:5180";
 dir_max(k) = 2*k;
 dir_max_by_search(k) = {
   my(n_max, d=vecmax(vector(5^k,n,n--; dir(n)), &n_max));
   [d, n_max-1];
 }

gp_inline__location="../r5dragon.tex:5189";
 dir_max_at_n(k) = 1/2 * (5^k - 1);

gp_inline__location="../r5dragon.tex:5199";
 dir_max_at_n_samples = {[0,2,12,62,312,1562]};

gp_inline__location="../r5dragon.tex:5226";
 n_dir_eq_0(j) = 4*fromdigits(binary(j),5);

gp_inline__location="../r5dragon.tex:5232";
 n_dir_eq_0_samples = {[0,4,20,24,100,104,120,124]};

gp_inline__location="../r5dragon.tex:5256";
 vector_find_square_subword(v,nextlengthfunc=(len)->len+1) = {
   for(i=1,#v,
     my(len=0,limit=(#v-i+1)\2);
     while((len=nextlengthfunc(len)) <= limit,
       len>=0 || next;
       for(j=i,i+len-1,
         if(v[j]!=v[j+len],next(2)));
       return([i,len])));
   'none;
 }
 vector_is_square_free_word(v) = \
   vector_find_square_subword(v)=='none;

gp_inline__location="../r5dragon.tex:5269";
 \\ A170823
 \\ 1 -> 12321,  2-> 23132,  3->31213,  start 1
 my(table=[[1,2,3,2,1], [2,3,1,3,2], [3,1,2,1,3]]); \
 A170823_vector(k) = {
   my(v=[1]);
   for(i=1,k, v=concat(vecextract(table,v)));
   v;
 }

gp_inline__location="../r5dragon.tex:5280";
 my(table=[0,1,2,1,0]); \
 A170823(n) = my(v=digits(n,5)); sum(i=1,#v,table[v[i]+1])%3+1;

gp_inline__location="../r5dragon.tex:5394";
 turn_part_runlengths_ok(v) = {
   v=vector_run_lengths(v);
   if(v[1]>3 || v[#v]>3, return(0));
   for(i=2,#v-1, v[i]==2 || v[i]==3 || return(0));
   1;
 }

gp_inline__location="../r5dragon.tex:5434";
 TurnsL(n) = 1/2*(      n + dir(n) );
 TurnsR(n) = 1/2*     ( n - dir(n) );

gp_inline__location="../r5dragon.tex:5450";
 TurnsL_samples = {[1,2,2,2,3,4,5,5,5,6,7,8,8,8,8]};

gp_inline__location="../r5dragon.tex:5466";
 TurnsR_samples = {[0,0,1,2,2,2,2,3,4,4,4,4,5,6,7]};

gp_inline__location="../r5dragon.tex:5491";
 gTurnsL_term(x,k) = \
   (x^(5^k) + x^(2*5^k)) /( (1-x)*(1 - x^(5^(k+1))) );
 gTurnsL(x,k) = sum(j=0,k, gTurnsL_term(x,j));

gp_inline__location="../r5dragon.tex:5502";
 gTurnsR_term(x,k) = \
   (x^(3*5^k) + x^(4*5^k)) /( (1-x)*(1 - x^(5^(k+1))));
 gTurnsR(x,k) = sum(j=0,k, gTurnsR_term(x,j));

gp_inline__location="../r5dragon.tex:5557";
 dir_mod4_samples = {[0,1,2,1,0,1,2,3,2,1,2,3,0,3,2,1]};

gp_inline__location="../r5dragon.tex:5574";
 dir_mod4_eq_0_at_n_samples = {[0,4,12,20,24,32,36,38,42]};

gp_inline__location="../r5dragon.tex:5585";
 dir_mod4_eq_1_at_n_samples = {[1,3,5,9,15,19,21,23,25]};

gp_inline__location="../r5dragon.tex:5598";
 dir_mod4_eq_2_at_n_samples = {[2,6,8,10,14,16,18,22,26]};

gp_inline__location="../r5dragon.tex:5611";
 dir_mod4_eq_3_at_n_samples = {[7,11,13,17,27,31,33,35,39]};

gp_inline__location="../r5dragon.tex:5621";
 dir4_state_table = {[0,1,2,1,0; 1,2,3,2,1; 2,3,0,3,2; 3,0,1,0,3]};

gp_inline__location="../r5dragon.tex:5624";
 dir4_transition(state,bit) = dir4_state_table[state+1,bit+1];
 dir4_transitions(state,v) = {
   for(i=1,#v,  /* high to low */
     state=dir4_transition(state,v[i]));
   state;
 }

gp_inline__location="../r5dragon.tex:5761";
 r5dragon_digit_vector = {[0,1,1+I,I,2*I]};

gp_inline__location="../r5dragon.tex:5770";
 r5dragon_digit(a) = r5dragon_digit_vector[a+1];

gp_inline__location="../r5dragon.tex:5849";
 r5dragon_point(n) = {
   my(v=digits(n,5), k=#v,
      total_dir=0, ret=0);
   forstep(j=k-1,0, -1,  \\ high to low
     my(a=v[k-j]);
     ret += r5dragon_digit(a) * b^j * I^total_dir;
     total_dir += dir(a));
   ret;
 }

gp_inline__location="../r5dragon.tex:5862";
 point_samples = {[0,1,1+I,I,2*I,1+2*I,1+3*I,3*I]};

gp_inline__location="../r5dragon.tex:5896";
 arg_b_degrees_sample = {(63434948/1000000)};

gp_inline__location="../r5dragon.tex:5903";
 arg_b_radians_sample = {(1107148/1000000)};

gp_inline__location="../r5dragon.tex:5927";
 imag_b_k_mod5_vector = {[1,2,4,3]};

gp_inline__location="../r5dragon.tex:6124";
 z_mod_2(z) = (real(z) + imag(z)) % 2;

gp_inline__location="../r5dragon.tex:6130";
 unpoint_z_to_low_digit_vector = [0, 1+I, I, 1, 2*I];
 unpoint_z_to_low_digit(z) = {
   for(i=1,#unpoint_z_to_low_digit_vector,
       if(z_is_integer((z-unpoint_z_to_low_digit_vector[i])/b),
          return(i-1)));
 }

gp_inline__location="../r5dragon.tex:6139";
 \\ return [arm,n] where arm=0,1,2,3
 r5dragon_unpoint(z,d) = {
   my(debug=0);
   z_is_integer(z)
     || error("r5dragon_unpoint() should be integer z, got ",z);
   if(z_mod_2(z)==0,
     d==1 || d==-1
       || error("r5dragon_unpoint() even should be d=+/-1, got ",d),
     d==I || d==-I
       || error("r5dragon_unpoint() odd should be d=+/-I, got ",d));
   if(z==0,return([0,0]));
   my(v=List([]));  \\ digits low to high
   if(debug,print("r5dragon_unpoint() z="z"  d="d));
   while(! (z==0 || ((z==I || z==-I) && d==-z)),
     my(a=unpoint_z_to_low_digit(z/d));
     if(debug,print("at z="z" d="d"  digit="a));
     d = d / I^dir(a);
     z = z - d*r5dragon_digit(a);
     z = z/b;
     if(! z_is_integer(z), error("oops, z=",z));
     listput(v,a));
   v=Vecrev(v);
   if(debug,print("end z="z" d="d" digits high to low "v));
   if(z==0 && d==1,   [0, fromdigits(v,5)],
      z==I && d==-I,  [1, 5^#v-fromdigits(v,5)],
      z==0 && d==-1,  [0, fromdigits(v,5)],
      z==-I && d==I,  [3, 5^#v-fromdigits(v,5)],
      error("oops, r5dragon_unpoint() end z="z" d="d));
 }

gp_inline__location="../r5dragon.tex:6203";
 unpoint_x_plus_2y_vector = {[0,3,2,1,4]};

gp_inline__location="../r5dragon.tex:6338";
 other_state_table  = {[ 1,3,2,5,4;   \\ 1 = start
                         3,2,6,6,2;   \\ 2 = lm0
                         3,3,2,7,7;   \\ 3 = lm1
                         8,8,5,4,4;   \\ 4 = rm0
                         5,9,9,5,4;   \\ 5 = rm1
                         6,6,6,6,6;   \\ 6 = unch
                         7,7,7,7,7;   \\ 7 = unch
                         8,8,8,8,8;   \\ 8 = unch
                         9,9,9,9,9    \\ 9 = unch
                       ]};
 other_state_output = {[ 0,2,1,4,3;   \\ start
                         2,1,3,4,0;   \\ lm0
                         4,1,0,2,3;   \\ lm1
                         1,2,4,3,0;   \\ rm0
                         4,0,1,3,2;   \\ rm1
                         0,1,2,3,4;   \\ 6 = unch
                         0,1,2,3,4;   \\ 7 = unch
                         0,1,2,3,4;   \\ 8 = unch
                         0,1,2,3,4    \\ 9 = unch
                       ]};
 other_m0   = 1;
 other_lm0  = 2;
 other_lm1  = 3;
 other_rm0  = 4;
 other_rm1  = 5;
 other_unch_lm0 = 6;
 other_unch_lm1 = 7;
 other_unch_rm0 = 8;
 other_unch_rm1 = 9;
 other_transition(state,digit) = other_state_table[state,digit+1];
 other_transitions(state,v) = /* v is high to low */  \
   forstep(i=#v,1,-1,state=other_transition(state,v[i])); state;
 other_output(state,digit) = other_state_output[state,digit+1];
 other_arm_and_n(n) = {
   if(n==0,return([0,0]));
   my(v=digits(n,5)); v=concat(0,v);
   my(other_v=v, state=other_m0);
   forstep(i=#v,1,-1,  \\ low to high
     \\ print("at i="i" state="state"  digit="v[i]
     \\       " output "other_output(state,v[i]));
     other_v[i]=other_output(state,v[i]);
     state=other_transition(state,v[i]));
   if(state>=other_unch_lm0, [ 0, fromdigits(other_v,5)],
      state==other_lm1,      [-1, 5^#v-fromdigits(other_v,5)],
      state==other_rm1,      [ 1, 5^#v-fromdigits(other_v,5)],
      error("oops, final state ",state));
 }

gp_inline__location="../r5dragon.tex:6519";
 other_n_samples = [ 0,3,14,1,8,15,18,69,4,13,70,73,16,9,2];

gp_inline__location="../r5dragon.tex:6524";
 other_arm_samples = {[0,-1,-1,1,0,-1,-1,-1,0,0,-1,-1,0,0,1]};

gp_inline__location="../r5dragon.tex:6591";
 \\ z is a complex integer.  Return it reduced modulo b to one
 \\ of the r5dragon_digit_vector values.
 z_mod_b(z) = {
   unpoint_z_to_low_digit_vector[unpoint_z_to_low_digit(z)+1];
 }

gp_inline__location="../r5dragon.tex:6670";
 other_c_left_side_vector = {[0,1+I,I,1,2*I]};

gp_inline__location="../r5dragon.tex:6687";
 other_delta_dz_to_c_map = {
   Map(matrix(5,2,c,col,c--;
              if(col==2,c,
                 z_mod_b(I^-dir(c)*r5dragon_digit(c)))));
 }
 other_delta_dz_to_c(delta_dz,a) = {
   my(delta=delta_dz[1],
      dz   =delta_dz[2],
      z_c  =z_mod_b(  I^(-dir(a)-delta)*r5dragon_digit(a)
                    + I^(-delta)*dz ));
   mapget(other_delta_dz_to_c_map,z_c);
 }

gp_inline__location="../r5dragon.tex:6721";
 other_delta_dz_to_new_delta(delta_dz,a) = {
   my(delta=delta_dz[1],
      dz   =delta_dz[2],
      c   =other_delta_dz_to_c(delta_dz,a));
   (delta - dir(c) + dir(a)) % 4;
 }

gp_inline__location="../r5dragon.tex:6813";
 other_delta_dz_transition(delta_dz,a) = {
   my(delta=delta_dz[1],
      dz   =delta_dz[2],
      c         = other_delta_dz_to_c(delta_dz,a),
      new_delta = other_delta_dz_to_new_delta(delta_dz,a),
      new_dz_times_b = dz*I^dir(a)
                     - I^new_delta*r5dragon_digit(c)
                     + r5dragon_digit(a)
     );
   z_mod_b(new_dz_times_b) == 0
     || error("other_delta_dz_transition() not multiple of b: ",
              new_dz_times_b,"  c=",c);
   [new_delta, new_dz_times_b/b];
 }

gp_inline__location="../r5dragon.tex:6903";
 other_delta_dz_start =        [2,                   0];

gp_inline__location="../r5dragon.tex:6907";
 other_delta_dz_lm0 =                     [3,                   1];

gp_inline__location="../r5dragon.tex:6910";
 other_delta_dz_lm1   =                 [1,                   -I];

gp_inline__location="../r5dragon.tex:6918";
 other_delta_dz_rm0   =                 [1,                   1];

gp_inline__location="../r5dragon.tex:6921";
 other_delta_dz_rm1   =                 [3,                   I];

gp_inline__location="../r5dragon.tex:6999";
 other_arm_and_n_by_delta_dz(n) = {
   my(debug=0);
   my(delta_dz=other_delta_dz_start,
      l=List([]));
   while(n || ! (delta_dz == [0,0]
              || delta_dz == other_delta_dz_lm1
              || delta_dz == other_delta_dz_rm1 ),
     my(a=n%5);
     my(c = other_delta_dz_to_c(delta_dz,a));
     if(debug, print("at "delta_dz" n="n" a="n%5" is c="c));
     n\=5;
     listput(l,c);
     delta_dz=other_delta_dz_transition(delta_dz,a));
   if(debug, print("final "delta_dz));
   my(n=fromdigits(Vecrev(l),5),
      arm=0,
      delta=delta_dz[1],
      dz   =delta_dz[2]);
   if(dz==-I, arm=-1,
      dz==I, arm=1);
   if(debug,print("arm ",arm));
   if(arm, n=5^#l-n);
   [arm,n];
 }

gp_inline__location="../r5dragon.tex:7089";
 rotate_mod5_skipping(n,add,skip) = \
   if(n!=skip, n=(n+add)%5; if(n==skip,n=(n+add)%5)); n;

gp_inline__location="../r5dragon.tex:7381";
 otherright_digit_matrix = [4,3,2; 1,1,1; 0,4,3; 2,0,4; 3,2,0];
 otherright_part_matrix  = [1,1,1; 1,2,3; 3,0,0; 0,3,0; 0,0,3];

gp_inline__location="../r5dragon.tex:7540";
 otherleft_digit_matrix = [4,2,1; 0,4,2; 1,0,4; 3,3,3; 2,1,0];
 otherleft_part_matrix  = [1,0,0; 0,1,0; 0,0,1; 1,2,3; 3,3,3];

gp_inline__location="../r5dragon.tex:7578";
 \\ return [ [n,arm], [n,arm], [n,arm] ] for s,t,e right segments
 \\ arm=0 for same curve as n, or arm=-1 for arm to the right
 otherright_ste(n) = {
   my(nv=concat(0,digits(n,5)));
   my(parts=[[],[],[]]);
   for(i=1,#nv,
       parts=vector(3,p,
          my(new_p=otherright_part_matrix[nv[i]+1, p]);
             concat(if(new_p==0,nv[1..i-1],parts[new_p]),
                    [otherright_digit_matrix[nv[i]+1, p]])));
   \\ return(parts);
   apply(v->my(arm=if(v[1],-1,0));
            if(arm, v=apply(d->4-d,v));
            [fromdigits(v,5),arm], parts);
 }

gp_inline__location="../r5dragon.tex:7599";
 otherright_ste_by_unpoint(n) = {
   my(z=r5dragon_point(n), d=dir(n), dz=I^d);
   my(parts=[r5dragon_unpoint(z+dz*-I,    dz*I),
             r5dragon_unpoint(z+dz*(1-I), -dz),
             r5dragon_unpoint(z+dz,       dz*-I)]);
   apply(an->my(arm=if(an[1]==3,-1,an[1]));
             [an[2]-(arm!=0), arm], parts);
 }

gp_inline__location="../r5dragon.tex:7613";
 \\ return [ [n,arm], [n,arm], [n,arm] ] for s,t,e left segments
 \\ arm=0 for same curve as n, or arm=1 for arm to the left
 otherleft_ste(n) = {
   my(nv=concat(0,digits(n,5)));
   my(parts=[[],[],[]]);
   for(i=1,#nv,
       parts=vector(3,p,
          my(new_p=otherleft_part_matrix[nv[i]+1, p]);
             concat(if(new_p==0,nv[1..i-1],parts[new_p]),
                    [otherleft_digit_matrix[nv[i]+1, p]])));
   \\ return(parts);
   apply(v->my(arm=if(v[1]==4,1,0));
            if(arm, v=apply(d->4-d,v));
            [fromdigits(v,5),arm], parts);
 }

gp_inline__location="../r5dragon.tex:7630";
 otherleft_ste_by_unpoint(n) = {
   my(z=r5dragon_point(n), d=dir(n), dz=I^d);
   my(parts=[r5dragon_unpoint(z+dz*I,     dz*-I),
             r5dragon_unpoint(z+dz*(1+I), -dz),
             r5dragon_unpoint(z+dz,       dz*I)]);
   apply(an->my(arm=if(an[1]==3,-1,an[1]));
             [an[2]-(arm!=0), arm], parts);
 }

gp_inline__location="../r5dragon.tex:7655";
 other_differences_list_by_n(k) = {
   my(limit=5^k,l=List([]));
   for(n=1,limit-1,
       my(an=other_arm_and_n(n));
       if(an[1]==0&&an[2]<limit&&n>an[2],
          listput(l,abs(n-an[2]))));
   Vec(l);
 }
 Odistinct_list_by_n(k) = Set(other_differences_list_by_n(k));

gp_inline__location="../r5dragon.tex:7686";
 Opred_k(k,n) = {
   if(n==0,return(0));
   my(p=0);
   while(n,
     if(n%5==4, n -= 4,
        n%5==1, n -= -4);
     if(n<0 || n==5 || n%5 || !(k-2>=p),return(0));
     n/=5; p++);
   1;
 }

gp_inline__location="../r5dragon.tex:7698";
 Odistinct_list_by_pred(k) = {
   my(l=List([]));
   for(n=1,5^k, if(Opred_k(k,n), listput(l,n)));
   Vec(l);
 }

gp_inline__location="../r5dragon.tex:7709";
 Opred(n) = Opred_k(1e100,n);

gp_inline__location="../r5dragon.tex:7713";
 Opred_at_n_samples = {[4,16,20,24,76,80,84,96,100,104,116,120]};

gp_inline__location="../r5dragon.tex:8117";
 \\ Return a vector of digits of n in nega-base-5, which means
 \\ digits -2,-1,0,1,2 (instead of 0 to 4).
 negabase5_digits(n) = {
   my(l=List([]));
   while(n,
     my(d=n%5);
     if(d>=3,d-=5);
     listput(l,d);
     n-=d; n/=5);
   Vecrev(l);
 }
 vector_is_0pm1(v) = {
   for(i=1,#v, if(!(v[i]==0 || v[i]==1 || v[i]==-1), return(0)));
   1;
 }

gp_inline__location="../r5dragon.tex:8147";
 Opred_by_31(n) = {
   if(n==0,return(0));
   my(v=digits(n,5),want=3,notwant=1);
   for(i=1,#v,
       if(v[i]==2 || v[i]==notwant,return(0));
       if(v[i]==want, want=4-want; notwant=4-notwant));
   want==3;
 }

gp_inline__location="../r5dragon.tex:8211";
 Opred_at_n_base5_samples = {[4,31,40,44,301,310,314,341,400,404,431,440]};

gp_inline__location="../r5dragon.tex:8251";
 Opred_by_signs(n) = {
   my(debug=0);
   if(debug,print("Opred_by_signs() n="n));
   my(s);
   while(n>0 && n%5==0, n/=5);
   if(n%5==1, s=1;n--,
      (-n)%5==1, s=-1;n++,
      return(0));
   if(debug,print("initial s="s" n="n));
   n%5==0 || error();
   n/=5;
   while(n>=2,
     if(debug,print(" at s="s" n="n));
     if((n*s)%5==0, n-=0*s,
        (n*s)%5==4, n-=4*s,
        (-n*s)%5==2, s=-s; n-=2*s,
        return(0));
     n%5==0 || error();
     n/=5);
   if(debug,print(" final s="s" n="n));
   n==1 && s==-1;
 }

gp_inline__location="../r5dragon.tex:8322";
 Odistinct_formula(k) = 2/7*(15^(k-1) - 1);
 Odistinct(k) = if(k==0,0, Odistinct_formula(k));

gp_inline__location="../r5dragon.tex:8326";
 Odistinct_by_n(k) = vecsum(Odistinct_list_by_n(k));

gp_inline__location="../r5dragon.tex:8331";
 Odistinct_samples = {[0,0,4,64,964,14464]};

gp_inline__location="../r5dragon.tex:8348";
 rep4s(k,radix=10) = fromdigits(vector(k,i,4),radix);

gp_inline__location="../r5dragon.tex:8392";
 vector_median(v) = v[ceil(#v/2)];
 vector_median_fraction(v) = vector_median(v) / #v;

gp_inline__location="../r5dragon.tex:8405";
 Odistinct_mean_limit = {(4)/(35)};

gp_inline__location="../r5dragon.tex:8411";
 Odistinct_mean_limit_sample = {(0114285/1000000)};

gp_inline__location="../r5dragon.tex:8429";
 \\ Return a vector of digits of n in given base.
 \\ Digits are alternating signs + or - (or 0).
 \\ The least significant non-zero digit is sign s=+/-1.
 digits_alternating_signs(n,base=10,s=1) = {
   my(l=List([]));
   while(n,
     my(d=((n*s)%base)*s);
     listput(l,d);
     n-=d; n/=base;
     if(d,s=-s));
   Vecrev(l);
 }

gp_inline__location="../r5dragon.tex:8456";
 \\ base-5 alternating +4, -4, ...
 is_base5_alternating4s(n) = {
   if(n==0,return(1));
   while(n%5==0,n/=5);
   my(want=if(n%5==4,4,-4));
   while(n,
     while(n%5==0,n/=5);
     n-=want;
     if(n%5!=0,return(0));
     want=-want);
   1;
 }

gp_inline__location="../r5dragon.tex:8492";
 other_difference_m(m) = {
   my(k=1); while(1/2*(3^k-1)<=m, k++); k--;
   my(v=digits_padded(m-1/2*(3^k-1),k+1,3));
   v=apply(d->d-1,v);
   v[1]=1;
   \\ print(m" "k"  "m-1/2*(3^k-1)"  "v);
   4*fromdigits(v,5);
 }

gp_inline__location="../r5dragon.tex:8537";
 Skd(k,d) = \
   1/4 * ( 5^k + (-I)^d*b^k + conj((-I)^d*b^k) + (-1)^d );

gp_inline__location="../r5dragon.tex:8549";
 Skd_by_norm(k,d) = \
   1/4*norm(b^k + I^d) - vector_modulo([0,1/2],d);

gp_inline__location="../r5dragon.tex:8554";
 Skd_by_abs(k,d) = \
   1/4*geom_abs(b^k + I^d)^2 - vector_modulo([0,1/2],d);

gp_inline__location="../r5dragon.tex:8561";
 S0_samples = {[1,2,5,26,153,802,3965,19546]};

gp_inline__location="../r5dragon.tex:8567";
 S1_samples = {[0,2,8,30,144,762,3928,19670]};

gp_inline__location="../r5dragon.tex:8573";
 S2_samples = {[0,1,8,37,160,761,3848,19517]};

gp_inline__location="../r5dragon.tex:8579";
 S3_samples = {[0,0,4,32,168,800,3884,19392]};

gp_inline__location="../r5dragon.tex:8603";
 S0_conj(k) = (  5^k +   b^k +   conj(b)^k + 1)/4;
 S1_conj(k) = (  5^k - I*b^k + I*conj(b)^k - 1)/4;
 S2_conj(k) = (  5^k -   b^k -   conj(b)^k + 1)/4;
 S3_conj(k) = (  5^k + I*b^k - I*conj(b)^k - 1)/4;
 S0_reim(k) = (  5^k + 2*real(b^k) + 1)/4;
 S1_reim(k) = (  5^k + 2*imag(b^k) - 1)/4;
 S2_reim(k) = (  5^k - 2*real(b^k) + 1)/4;
 S3_reim(k) = (  5^k - 2*imag(b^k) - 1)/4;
 Skd_by_conj(k,d) = if(d%4==0,S0_conj(k), d%4==1,S1_conj(k), d%4==2,S2_conj(k), S3_conj(k));
 Skd_by_reim(k,d) = if(d%4==0,S0_reim(k), d%4==1,S1_reim(k), d%4==2,S2_reim(k), S3_reim(k));

gp_inline__location="../r5dragon.tex:8618";
 Skd_by_count(k,d) = sum(i=0,5^k-1, dir(i)%4==d);

gp_inline__location="../r5dragon.tex:8622";
 Skd_by_product(k,d) = {
   1/4*(b^k + I^d)*conj(b^k + I^d)
   - vector_modulo([0,1/2],d);
 }

gp_inline__location="../r5dragon.tex:8629";
 gSk0(x) = (1-6*x+11*x^2-10*x^3) / ((1-x)*(1-5*x)*(1-2*x+5*x^2));
 gSk1(x) = (  2*x- 8*x^2+10*x^3) / ((1-x)*(1-5*x)*(1-2*x+5*x^2));
 gSk2(x) = (    x       - 5*x^3) / ((1-x)*(1-5*x)*(1-2*x+5*x^2));
 gSk3(x) = (       4*x^2       ) / ((1-x)*(1-5*x)*(1-2*x+5*x^2));

gp_inline__location="../r5dragon.tex:8644";
 S_by_2re(k,d) = (5^k + (-I)^d*b^k + I^d*conj(b)^k + (-1)^d) / 4;

gp_inline__location="../r5dragon.tex:8699";
 S_by_mutual(k,d) = 2*Skd(k-1,d) + 2*Skd(k-1,d-1) + Skd(k-1,d-2);

gp_inline__location="../r5dragon.tex:8715";
 Skd_matrix = [2,0,1,2; 2,2,0,1; 1,2,2,0; 0,1,2,2];
 Skd_rotate_0 = [1,0,0,0; 0,0,0,0; 0,0,0,0; 0,0,0,0];
 Skd_rotate_1 = [0,0,0,0; 1,0,0,0; 0,0,0,0; 0,0,0,0];
 Skd_rotate_2 = [0,0,0,0; 0,0,0,0; 1,0,0,0; 0,0,0,0];
 Skd_rotate_3 = [0,0,0,0; 0,0,0,0; 0,0,0,0; 1,0,0,0];
 Skd_t =   Skd_rotate_0 * Skd_matrix^0 \
         + Skd_rotate_1 * Skd_matrix^1 \
         + Skd_rotate_2 * Skd_matrix^2 \
         + Skd_rotate_3 * Skd_matrix^3;
 Skd_v = Skd_t * Skd_matrix * Skd_t^-1;
 Skd_by_recurrence(k,d) = if(k<4,Skd(k,d), 8*Skd(k-1,d) - 22*Skd(k-2,d) + 40*Skd(k-3,d) - 25*Skd(k-4,d));

gp_inline__location="../r5dragon.tex:8743";
 poly(x) = x^4 - 8*x^3 + 22*x^2 - 40*x + 25;

gp_inline__location="../r5dragon.tex:8786";
 W_term(d) = 1/4;
 X_term(d) = 1/4*(-I)^d;
 Y_term(d) = 1/4*I^d;
 Z_term(d) = 1/4*(-1)^d;

gp_inline__location="../r5dragon.tex:8791";
 S_by_WXYZ(k,d) = W_term(d)*5^k + X_term(d)*b^k + Y_term(d)*conj(b)^k + Z_term(d);

gp_inline__location="../r5dragon.tex:8823";
 Skd_middle(k,d) = (-I)^d*b^k + conj((-I)^d*b^k);

gp_inline__location="../r5dragon.tex:8843";
 Re_b_samples = {[1,1,-3,-11,-7,41]};

gp_inline__location="../r5dragon.tex:8853";
 A006495(n) = n>=0 || error(); real(b^n);

gp_inline__location="../r5dragon.tex:8860";
 Im_b_samples = {[0,2,4,-2,-24,-38]};

gp_inline__location="../r5dragon.tex:8871";
 A006496(n) = n>=0 || error(); imag(b^n);

gp_inline__location="../r5dragon.tex:8875";
 v_real(k) = if(k==0,1, k==1,1, 2*v_real(k-1) - 5*v_real(k-2));
 v_imag(k) = if(k==0,0, k==1,2, 2*v_imag(k-1) - 5*v_imag(k-2));
 v_real = memoize(v_real);
 v_imag = memoize(v_imag);

gp_inline__location="../r5dragon.tex:8917";
 SM_by_Skd(k,d) = Skd(k,2*k+d);

gp_inline__location="../r5dragon.tex:8935";
 SM(k,d) = \
   1/4 * ( 5^k + (-I)^d*(-b)^k + conj((-I)^d*(-b)^k) + (-1)^d );

gp_inline__location="../r5dragon.tex:8952";
 SM_by_norm(k,d) = \
   1/4*norm(b^k + I^(d+2*k)) - vector_modulo([0,1/2],d);

gp_inline__location="../r5dragon.tex:8957";
 SM_by_abs(k,d) = \
   1/4*geom_abs(b^k + I^(d+2*k))^2 - vector_modulo([0,1/2],d);

gp_inline__location="../r5dragon.tex:8964";
 SM0_samples = {[1,1,5,37,153,761,3965,19517]};

gp_inline__location="../r5dragon.tex:8970";
 SM1_samples = {[0,0,8,32,144,800,3928,19392]};

gp_inline__location="../r5dragon.tex:8976";
 SM2_samples = {[0,2,8,26,160,802,3848,19546]};

gp_inline__location="../r5dragon.tex:8982";
 SM3_samples = {[0,2,4,30,168,762,3884,19670]};

gp_inline__location="../r5dragon.tex:9004";
 gSM0(x) = (1-3*x -x^2 -5*x^3) / ((1-x)*(1-5*x)*(1+2*x+5*x^2));
 gSM1(x) = (     8*x^2       ) / ((1-x)*(1-5*x)*(1+2*x+5*x^2));
 gSM2(x) = (2*x       -10*x^3) / ((1-x)*(1-5*x)*(1+2*x+5*x^2));
 gSM3(x) = (2*x- 4*x^2+10*x^3) / ((1-x)*(1-5*x)*(1+2*x+5*x^2));

gp_inline__location="../r5dragon.tex:9017";
 SN_by_norm(n,d) = \
   1/4*norm(r5dragon_point(n));

gp_inline__location="../r5dragon.tex:9044";
 SN(n,d) = {
   1/4 * (n
          + 2*real((-I)^d * r5dragon_point(n))
          + if(n%2==1, (-1)^d)
         );
 }
 SN_by_count(n,d) = d%=4; sum(j=0,n-1, dir(j)%4==d);

gp_inline__location="../r5dragon.tex:9056";
 SN0_samples = {[0,1,1,1,1,2,2,2,2,2,2,2,2,3,3]};

gp_inline__location="../r5dragon.tex:9063";
 SN1_samples = {[0,0,1,1,2,2,3,3,3,3,4,4,4,4,4]};

gp_inline__location="../r5dragon.tex:9070";
 SN2_samples = {[0,0,0,1,1,1,1,2,2,3,3,4,4,4,4]};

gp_inline__location="../r5dragon.tex:9077";
 SN3_samples = {[0,0,0,0,0,0,0,0,1,1,1,1,2,2,3]};

gp_inline__location="../r5dragon.tex:9100";
 DirPred(n,d) = dir(n)%4 == d%4;

gp_inline__location="../r5dragon.tex:9104";
 DirPred0_samples = {[1,0,0,0,1,0,0,0,0,0,0,0,1]};

gp_inline__location="../r5dragon.tex:9111";
 DirPred1_samples = {[0,1,0,1,0,1,0,0,0,1,0,0,0]};

gp_inline__location="../r5dragon.tex:9118";
 DirPred2_samples = {[0,0,1,0,0,0,1,0,1,0,1,0,0]};

gp_inline__location="../r5dragon.tex:9125";
 DirPred3_samples = {[0,0,0,0,0,0,0,1,0,0,0,1,0]};

gp_inline__location="../r5dragon.tex:9153";
 z_to_DirPred(z,d) = \
   1/4*( 2 + 2*real((-I)^d*z) - 2*abs(imag((-I)^d*z)) );

gp_inline__location="../r5dragon.tex:9168";
 dpoint(n) = I^dir(n);

gp_inline__location="../r5dragon.tex:9340";
 base5len(n) = if(n==0,0, 1 + logint(n,5));

gp_inline__location="../r5dragon.tex:9351";
 base5len_samples = {[0,1,1,1,1,2,2,2,2,2,2,2,2]};

gp_inline__location="../r5dragon.tex:9369";
 point_upper_bound(n) = (sqrt5^base5len(n)-1)/(sqrt5-1);

gp_inline__location="../r5dragon.tex:9415";
 B(k) = 4*3^k - 2;

gp_inline__location="../r5dragon.tex:9417";
 B_samples = {[2,10,34,106,322,970,2914]};

gp_inline__location="../r5dragon.tex:9436";
 R(k) = 2*3^k - 1;

gp_inline__location="../r5dragon.tex:9438";
 R_samples = {[1,5,17,53,161,485,1457]};

gp_inline__location="../r5dragon.tex:9454";
 U(k) = 2*3^k + 1;

gp_inline__location="../r5dragon.tex:9456";
 U_samples = {[3,7,19,55,163,487,1459,4375]};

gp_inline__location="../r5dragon.tex:9611";
 R_from_recurrence(k) = if(k<2,R(k), 4*R(k-1) - 3*R(k-2));
 U_from_recurrence(k) = if(k<2,U(k), 4*U(k-1) - 3*U(k-2));

gp_inline__location="../r5dragon.tex:9690";
 BQ(k) = 2*3^k;

gp_inline__location="../r5dragon.tex:9693";
 BQ_samples = {[2,6,18,54,162,486,1458,4374]};

gp_inline__location="../r5dragon.tex:9712";
 RQ(k) = 3^k;
 LQ(k) = RQ(k);

gp_inline__location="../r5dragon.tex:9716";
 RQ_samples = {[1,3,9,27,81,243,729,2187]};

gp_inline__location="../r5dragon.tex:9736";
 UQ(k) = RQ(k);

gp_inline__location="../r5dragon.tex:9845";
 BQ1(k) = 3^k + 1;

gp_inline__location="../r5dragon.tex:9856";
 BQ3(k) = 3^k - 1;

gp_inline__location="../r5dragon.tex:9870";
 RQ1(k) = 1/2*(3^k + 1);

gp_inline__location="../r5dragon.tex:9872";
 RQ1_samples = {[1,2,5,14,41,122,365,1094]};

gp_inline__location="../r5dragon.tex:9887";
 UQ1(k) = 1/2*(3^k - 1);

gp_inline__location="../r5dragon.tex:9889";
 UQ1_samples = {[0,1,4,13,40,121,364,1093]};

gp_inline__location="../r5dragon.tex:9905";
 RQ3(k) = UQ1(k);

gp_inline__location="../r5dragon.tex:9908";
 UQ3(k) = RQ1(k);

gp_inline__location="../r5dragon.tex:9954";
 UQ1_by_recurrence(k) = if(k==0,0, k==1,1, \
   4*UQ1_by_recurrence(k-1) - 3*UQ1_by_recurrence(k-2));

gp_inline__location="../r5dragon.tex:10006";
 gen_RQsides_samples = {[1,1,3,1,1,3,1,3,3,1,1,3,1,1,3,1,3,3,1,1,3,
   1,3,3,1,3,3,1,1,3,1,1,3,1,3,3,1,1,3,1,1,3,1,3,3,1,1,3,1,3,3,1,3,
   3,1,1,3,1,1,3,1,3,3,1,1,3,1,3,3,1,3,3,1,1,3,1,3,3,1,3,3,1,1,3,1,
   1,3,1,3,3,1,1,3,1,1,3,1,3,3,1]};
 gen_LQsidesInf_samples = {[3,3,1,3,3,1,3,1,1,3,3,1,3,3,1,3,1,1,3,
   3,1,3,1,1,3,1,1,3,3,1,3,3,1,3,1,1,3,3,1,3,3,1,3,1,1,3,3,1,3,1,1,
   3,1,1,3,3,1,3,3,1,3,1,1,3,3,1,3,1,1,3,1,1,3,3,1,3,1,1,3,1,1,3,3,
   1,3,3,1,3,1,1,3,3,1,3,3,1,3,1,1,3]};

gp_inline__location="../r5dragon.tex:10029";
 RQsides(m) = if(TernaryLowestNon1(m)==0, 1, 3);

gp_inline__location="../r5dragon.tex:10038";
 RQsides_samples = {[1,1,3,1,1,3,1,3,3,1,1,3,1,1,3,1,3,3,1,1]};

gp_inline__location="../r5dragon.tex:10057";
 TernaryLowestNon1(n) = {
   n>=0 || error("TernaryLowestNon1() is for n>=0");
   while(n%3==1, n\=3); n%3;
 }

gp_inline__location="../r5dragon.tex:10063";
 TernaryLowestNon1_samples = {[0,0,2,0,0,2,0,2,2,0,0,2,0,0,2,0,2,2,0,0]};

gp_inline__location="../r5dragon.tex:10086";
 TernaryCountLow1s(n) = \
   my(count=0); while(n%3==1,count++;n\=3); count;

gp_inline__location="../r5dragon.tex:10098";
 \\ per formula by Cyril Damamme in A253786
 A253786(n) = valuation(2*n-1,3);

gp_inline__location="../r5dragon.tex:10146";
 RQsidesTotal(k) = 2*3^k-1;

gp_inline__location="../r5dragon.tex:10150";
 RQsidesTotal_by_count(k) = sum(m=0,3^k-1, RQsides(m));

gp_inline__location="../r5dragon.tex:10171";
 LQsidesInf(m) = if(TernaryLowestNon1(m)==0, 3, 1);

gp_inline__location="../r5dragon.tex:10184";
 LQsidesInf_samples = {[3,3,1,3,3,1,3,1,1,3,3,1,3,3,1,3,1,1,3,3]};

gp_inline__location="../r5dragon.tex:10221";
 Rpred_disallowed_pairs = {Set([
          22,   23,   24,
    30,   32,   33,   34,
    40
 ])};

gp_inline__location="../r5dragon.tex:10231";
 Rpred_pair_is_allowed(n) = setsearch(Rpred_disallowed_pairs,n)==0;
 vector_delete_1s(v) = select(x->x!=1, v);
 Rpred_of_digits(v) = {
   v=vector_delete_1s(v);
   for(i=1,#v-1,
     if(!Rpred_pair_is_allowed(fromdigits(v[i..i+1])),
        return(0)));
   1;
 }
 Rpred(n) = Rpred_of_digits(digits(n,5));

gp_inline__location="../r5dragon.tex:10251";
 Rpred_samples = {[1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,1,0]};

gp_inline__location="../r5dragon.tex:10264";
 Rn_samples = {[0,1,2,3,4,5,6,7,8,9,10,11,16,21,22,23,24,25]};

gp_inline__location="../r5dragon.tex:10271";
 Rn_samples_base5 = {[0,1,2,3,4,10,11,12,13,14,20,21,31,41,42,43,44,100]};

gp_inline__location="../r5dragon.tex:10456";
 Rpred_state_table = {[ 1,1,2,3,4;   \\ 1 = R
                        1,2,5,5,5;   \\ 2 = X
                        5,3,5,5,5;   \\ 3 = Y
                        5,4,2,3,4;   \\ 4 = Z
                        5,5,5,5,5    \\ 5 = non
                      ]};
 Rpred_R = 1;
 Rpred_X = 2;
 Rpred_Y = 3;
 Rpred_Z = 4;
 Rpred_non = 5;
 Rpred_transition(state,digit) = Rpred_state_table[state,digit+1];
 Rpred_transitions(state,v) = \
   for(i=1,#v,state=Rpred_transition(state,v[i])); state;
 Rpred_state(n) = Rpred_transitions(1,digits(n,5));
 Rpred_by_states(n) = Rpred_state(n) != Rpred_non;

gp_inline__location="../r5dragon.tex:10548";
 Rpred_LtoH_state_table = {[ 2,1,3,3,3;   \\ 1 = start
                             2,2,3,4,4;   \\ 2 = r2
                             2,3,4,4,3;   \\ 3 = r3
                             4,4,4,4,4    \\ 4 = non
                           ]};
 Rpred_LtoH_transition(state,digit) = \
   Rpred_LtoH_state_table[state,digit+1];
 Rpred_LtoH_transitions(state,v) = \
   for(i=1,#v,state=Rpred_LtoH_transition(state,v[i])); state;
 Rpred_by_LtoH(n) = \
   Rpred_LtoH_transitions(1,Vecrev(digits(n,5))) != 4;

gp_inline__location="../r5dragon.tex:10562";
 sides_by_unpoint(n,rot,nmax) = {
   my(z=r5dragon_point(n), dz=I^dir(n), sides=0);
   for(d=0,3,
       my(v=r5dragon_unpoint(z,dz));
       sides += (v[1]==0 && v[2]<=nmax);
       z += dz; dz *= rot);
   sides;
 }
 Rsides_by_unpoint(n) = sides_by_unpoint(n,-I, 5^#digits(n,5));
 Lsides_k_by_unpoint(n,k) = sides_by_unpoint(n, I, 5^k-1);

gp_inline__location="../r5dragon.tex:10652";
 \\                      R X Y Z non
 Rpred_state_to_sides = [1,3,3,3,4];
 Rsides(n) = Rpred_state_to_sides[Rpred_state(n)];

gp_inline__location="../r5dragon.tex:10656";
 Rsides_samples = {[1,1,3,3,3,1,1,3,3,3,1,3,4,4,4]};

gp_inline__location="../r5dragon.tex:10679";
 LowestNon1_samples = {[0,0,2,3,4,0,0,2,3,4,0,2,2,3,4]};

gp_inline__location="../r5dragon.tex:10712";
 Lpred_disallowed_pairs = {Set([
                      04,
    10,   11,   12,   14,
    20,   21,   22
 ])};

gp_inline__location="../r5dragon.tex:10721";
 Lpred_pair_is_allowed(n) = setsearch(Lpred_disallowed_pairs,n)==0;
 vector_delete_3s(v) = select(x->x!=3, v);
 Lpred_of_digits(v) = {
   v=vector_delete_3s(v);
   for(i=1,#v-1,
     if(!Lpred_pair_is_allowed(fromdigits(v[i..i+1])),
        return(0)));
   1;
 }
 Lpred_k(k,n) = Lpred_of_digits(digits_padded(n,k,5));

gp_inline__location="../r5dragon.tex:10747";
 Lpred_infinite_k(n) = \
   for(k=0,oo, if(5^k>n, return(k+1)));
 Lpred_infinite(n) = Lpred_k(Lpred_infinite_k(n), n);

gp_inline__location="../r5dragon.tex:10756";
 Lpred_infinite_by_digits(n) = \
   Lpred_of_digits(concat(0,digits(n,5)));

gp_inline__location="../r5dragon.tex:10762";
 Lpred_infinite_samples = {[1,1,1,1,0,0,0,0,1,0,0,0,0,1,1]};

gp_inline__location="../r5dragon.tex:10773";
 Lpred_n_infinite_samples = {[0,1,2,3,8,13,14,15,16,17,18,43]};

gp_inline__location="../r5dragon.tex:10781";
 Lpred_n_infinite_samples_base5 = {[0,1,2,3,13,23,24,30,31,32,33,133]};

gp_inline__location="../r5dragon.tex:10875";
 Lsides_k(n,k) = {
   my(v=digits_padded(n,k,5));
   v=apply(d->4-d, v);
   Rpred_state_to_sides[Rpred_transitions(Rpred_R,v)];
 }

gp_inline__location="../r5dragon.tex:10895";
 LsidesInf(n) = Lsides_k(n,Lpred_infinite_k(n));

gp_inline__location="../r5dragon.tex:10898";
 LsidesInf_samples = {[3,3,3,3,4,4,4,4,3,4,4,4,4,3,1]};

gp_inline__location="../r5dragon.tex:10913";
 Rpred_Lpred_both_sallowed_pairs = {[00,01,02,03,13,31,41,42,43,44]};

gp_inline__location="../r5dragon.tex:10917";
 all_pairs = vector(25,n,to_base5(n-1));

gp_inline__location="../r5dragon.tex:10938";
 Bboth_vector = {[0,1,2,3,8,16]};

gp_inline__location="../r5dragon.tex:10946";
 Bboth_list(k) = {
   if(k==0,[0],
      k==1,[0,1,2,3,4],
      Set(concat(Bboth_vector,apply(n->5^k-1-n,Bboth_vector))));
 }
 BbothPred_k(k,n) = Rpred(n) && Lpred_k(k,n);
 Bboth_list_by_search(k) = \
   select(n->BbothPred_k(k,n),vector(5^k,n,n-1));

gp_inline__location="../r5dragon.tex:10973";
 X(k) = 3^k - k;

gp_inline__location="../r5dragon.tex:10975";
 X_samples = {[1,2,7,24,77,238,723,2180,6553]};

gp_inline__location="../r5dragon.tex:10988";
 Y(k) = 1;

gp_inline__location="../r5dragon.tex:10993";
 Z(k) = 3^k + k;

gp_inline__location="../r5dragon.tex:10995";
 Z_samples = {[1,4,11,30,85,248,735,2194,6569]};

gp_inline__location="../r5dragon.tex:11053";
 Z_from_parts(k) = R(k+1) - 2*R(k) - X(k) - Y(k);

gp_inline__location="../r5dragon.tex:11070";
 U_from_parts(k) = X(k) + Y(k) + Z(k);

gp_inline__location="../r5dragon.tex:11281";
 A(k) = {
   k>=0 || error("A is for k>=0");
   (1/2)*(5^k + 1) - 3^k;
 }

gp_inline__location="../r5dragon.tex:11287";
 A_samples = {[0,0,4,36,232,1320,7084,36876]};

gp_inline__location="../r5dragon.tex:11298";
 AL(k) = AR(k);

gp_inline__location="../r5dragon.tex:11305";
 AR(k) = (1/4)*(5^k - 2*3^k + 1);

gp_inline__location="../r5dragon.tex:11308";
 AR_samples = {[0,0,2,18,116,660,3542,18438]};

gp_inline__location="../r5dragon.tex:11316";
 A007798(n) = 1/4*(5^n - 2*3^n + 1);

gp_inline__location="../r5dragon.tex:11432";
 A_square(k) = 5^k;
 A_square_from_AL_LQ(k) = 4*AL(k) + 4*1/2*(LQ(k)-1) + 1;

gp_inline__location="../r5dragon.tex:11441";
 AL_from_LQ_unsimplified(k) = 1/4*(5^k - 1) - 1/2*(3^k - 1);

gp_inline__location="../r5dragon.tex:11770";
 TurnRuns3_formula(k) = 5^(k-1) - 1;
 TurnRuns3(k) = {
   k>=0 || error("TurnRuns3() is for k>=0");
   if(k==0,0, TurnRuns3_formula(k));
 }

gp_inline__location="../r5dragon.tex:11776";
 TurnRuns3_by_count(k) = {
   my(v=TurnRun_vector(k));
   sum(i=1,#v,v[i]==3);
 }

gp_inline__location="../r5dragon.tex:11782";
 TurnRuns3_samples = {[0,0,4,24,124,624,3124]};

gp_inline__location="../r5dragon.tex:11802";
 Turn2_formula(k) = 5^(k-1) + 1;
 Turn2(k) = if(k==0,0, Turn2_formula(k));

gp_inline__location="../r5dragon.tex:11806";
 \\ Look for runs of "value" within the turn() sequence.
 \\ Return vector [c1,c2,c3] counts of how many runs of length
 \\ 1,2,3 occur between n=1 to n=5^k-1 inclusive.
 turn_count_runs(k,value) = {
   my(prev=0, len=0, runs=[0,0,0]);
   for(n=1,5^k-1,
       my(t = turn(n));
       if(prev==value && t==value, len++,        \\ continue run
          prev==value && t!=value, runs[len]++,  \\ end run
          prev!=value && t==value, len=1);       \\ start run
       prev=t);
   if(prev==value, runs[len]++);
   runs;
 }

gp_inline__location="../r5dragon.tex:11834";
 TurnRuns3_over_A_limit = {2/5};

gp_inline__location="../r5dragon.tex:11963";
 TurnRuns3pairs(k) = if(k<=1,0, 5^(k-2));

gp_inline__location="../r5dragon.tex:11967";
 TurnRuns3pairs_LLLRRR(k) = {
   my(v=vector(5^k-1,n, turn(n)));
   sum(i=1,#v-5,
       v[i]  ==1  && v[i+1]==1  && v[i+2]==1 &&
       v[i+3]==-1 && v[i+4]==-1 && v[i+5]==-1);
 }

gp_inline__location="../r5dragon.tex:11976";
 TurnRuns3pairs_RRRLLL(k) = {
   my(v=vector(5^k-1,n, turn(n)));
   sum(i=1,#v-5,
       v[i]  ==-1 && v[i+1]==-1 && v[i+2]==-1 &&
       v[i+3]==1  && v[i+4]==1  && v[i+5]==1);
 }

gp_inline__location="../r5dragon.tex:11995";
 A1111(k) = if(k==0,0, A(k-1));

gp_inline__location="../r5dragon.tex:12005";
 A31(k) = if(k==0,0, A(k) - TurnRuns3(k) - A1111(k));

gp_inline__location="../r5dragon.tex:12285";
 J(k) = 1/2*(3^k-1);

gp_inline__location="../r5dragon.tex:12290";
 J_samples = {[0,1,4,13,40,121,364,1093]};

gp_inline__location="../r5dragon.tex:12301";
 J_ternary_samples = {[0,1,11,111,1111]};

gp_inline__location="../r5dragon.tex:12389";
 J_from_A(k) = (A(k+1) - 5*A(k))/4;
 J_unsimplified(k) = (1/2*(5^(k+1)+1)-3^(k+1) - 5*(1/2*(5^k+1)-3^k) )/4;

gp_inline__location="../r5dragon.tex:12533";
 JN(k) = 1/4 * (3*5^k + 1);

gp_inline__location="../r5dragon.tex:12537";
 JN_by_search(k) = {
   my(end=5^k);
   for(n=0,end,
       my(an=other_arm_and_n(n));
       if(an[1]==0&&an[2]>end,return(n)));
   end;
 }

gp_inline__location="../r5dragon.tex:12547";
 JN_samples = {[1,4,19,94,469,2344]};

gp_inline__location="../r5dragon.tex:12562";
 JN_by_digits(k) = {
   if(k==0,return(1));
   my(num_digits=k);
   fromdigits(vector(num_digits,i,if(i==num_digits,4,3)),5);
 }

gp_inline__location="../r5dragon.tex:12575";
 JNother(k) = 1/4 * (7*5^k - 3);

gp_inline__location="../r5dragon.tex:12578";
 JNother_by_other(k) = {
   my(an=other_arm_and_n(JN(k)));
   if(an[1]==0,an[2], 1);
 }

gp_inline__location="../r5dragon.tex:12585";
 JNother_samples = {[1,8,43,218,1093,5468]};

gp_inline__location="../r5dragon.tex:12612";
 JND(k) = 1/4*(5^k-1);

gp_inline__location="../r5dragon.tex:12616";
 JND_samples = {[0,1,6,31,156,781]};

gp_inline__location="../r5dragon.tex:12808";
 Odistinct_by_recurrence(k) = \
   if(k<2,0, Odistinct(k-1) + 4*5^(k-2)*(2*J(k-2)+1));

gp_inline__location="../r5dragon.tex:12820";
 Odistinct_by_sum(k) = ;

gp_inline__location="../r5dragon.tex:12951";
 S_from_B(k) = 1/2*B(k) + 1;

gp_inline__location="../r5dragon.tex:12955";
 S(k) = 2*3^k;

gp_inline__location="../r5dragon.tex:12958";
 S_samples = {[2,6,18,54,162,486,1458,4374]};

gp_inline__location="../r5dragon.tex:13011";
 S_from_J_shortfall(k) = if(k<=0,S(k), 5*S(k-1)-4 - 8*J(k-1));

gp_inline__location="../r5dragon.tex:13055";
 D(k) = A(k);

gp_inline__location="../r5dragon.tex:13154";
 P(k) = S(k) + D(k);

gp_inline__location="../r5dragon.tex:13160";
 P_from_D(k) = 5^k + 1 - D(k);

gp_inline__location="../r5dragon.tex:13166";
 P_from_powers(k) = (1/2)*(5^k + 1) + 3^k;

gp_inline__location="../r5dragon.tex:13170";
 P_samples = {[2,6,22,90,394,1806,8542]};

gp_inline__location="../r5dragon.tex:13299";
 Horizontals_from_S(k) = 1/2*S(k);

gp_inline__location="../r5dragon.tex:13301";
 Horizontals(k) = 3^k;

gp_inline__location="../r5dragon.tex:13311";
 Verticals_from_S(k) = 1/2*S(k) - 1;

gp_inline__location="../r5dragon.tex:13313";
 Verticals(k) = 1/2*S(k) - 1;

gp_inline__location="../r5dragon.tex:13328";
 Lines_from_sum(k) = Horizontals(k) + Verticals(k);

gp_inline__location="../r5dragon.tex:13330";
 Lines(k) = 2*3^k - 1;

gp_inline__location="../r5dragon.tex:13382";
 Dpred_k(k,n) = {
   other_transitions(other_m0,digits_padded(n,k,5))
     >= other_unch_lm0;
 }

gp_inline__location="../r5dragon.tex:13393";
 DpredInf(n) = \
   other_transitions(1,concat(0,digits(n,5))) >= other_unch_lm0;

gp_inline__location="../r5dragon.tex:13398";
 DpredInf_samples = {[0,0,0,0,1,0,0,0,1,1,0,0,1,1,0,0,1,1]};

gp_inline__location="../r5dragon.tex:13406";
 DpredInf_at_n_samples = {[4,8,9,12,13,16,17,19,20,21,23,24,28,29]};

gp_inline__location="../r5dragon.tex:13416";
 Spred_k(k,n) = 1 - Dpred_k(k,n);

gp_inline__location="../r5dragon.tex:13420";
 SpredInf(n) = 1 - DpredInf(n);

gp_inline__location="../r5dragon.tex:13422";
 SpredInf_samples = {[1,1,1,1,0,1,1,1,0,0,1,1,0,0,1,1,0,0]};

gp_inline__location="../r5dragon.tex:13430";
 SpredInf_at_n_samples = {[0,1,2,3,5,6,7,10,11,14,15,18,22,25]};

gp_inline__location="../r5dragon.tex:13552";
 DpredFirstInf(n) = {
   my(state=other_transitions(other_m0,concat(0,digits(n,5))));
   state==other_unch_lm0 || state==other_unch_rm0;
 }

gp_inline__location="../r5dragon.tex:13559";
 DpredSecond(n) = {
   my(state=other_transitions(other_m0,digits(n,5)));
   state==other_unch_lm1 || state==other_unch_rm1;
 }

gp_inline__location="../r5dragon.tex:13570";
 DpredFirstInf_samples = {[0,0,0,0,1,0,0,0,0,1,0,0,1,0,0,0,0,1,0]};

gp_inline__location="../r5dragon.tex:13578";
 DpredFirstInf_at_n_samples = {[4,9,12,17,19,20,23,24,29,34,37,42]};

gp_inline__location="../r5dragon.tex:13587";
 DpredSecond_samples = {[0,0,0,0,0,0,0,0,1,0,0,0,0,1,0,0,1,0,0]};

gp_inline__location="../r5dragon.tex:13595";
 DpredSecond_at_n_samples = {[8,13,16,21,28,33,38,39,40,41,43,46]};

gp_inline__location="../r5dragon.tex:13616";
 VisitsInf(n) = DpredInf(n)+1;

gp_inline__location="../r5dragon.tex:13618";
 VisitsInf_samples = {[1,1,1,1,2,1,1,1,2,2,1,1,2,2,1,1,2,2,1]};

gp_inline__location="../r5dragon.tex:13629";
 VisitNum(n) = DpredSecond(n)+1;

gp_inline__location="../r5dragon.tex:13631";
 VisitNum_samples = {[1,1,1,1,1,1,1,1,2,1,1,1,1,2,1,1,2,1,1]};

gp_inline__location="../r5dragon.tex:13795";
 Epred(n) = DpredSecond(n+1);

gp_inline__location="../r5dragon.tex:13800";
 Epred_at_samples = {[7,12,15,20,27,32,37,38,39,40]};

gp_inline__location="../r5dragon.tex:13846";
 EpredR(n) = DpredSecond(n+1) && turn(n+1)==1;

gp_inline__location="../r5dragon.tex:13850";
 EpredR_at_samples = {[15,20,40,45,65,70,75,76,79,80]};

gp_inline__location="../r5dragon.tex:13865";
 EpredL(n) = DpredSecond(n+1) && turn(n+1)==-1;

gp_inline__location="../r5dragon.tex:13869";
 EpredL_at_samples = {[7,12,27,32,37,38,39,42,52,57]};

gp_inline__location="../r5dragon.tex:13887";
 Epred_LtoH_state_table = {[ 3,2,5,4,1;    /* 1 = m0 */
                             3,2,6,6,2;    /* 2 = lm0 */
                             3,3,2,7,7;    /* 3 = lm1 */
                             6,6,5,4,4;    /* 4 = rm0 */
                             5,8,8,5,4;    /* 5 = rm1 */
                             6,6,6,6,6;    /* 6 = non */
                             7,7,7,7,7;    /* 7 = EpredR */
                             8,8,8,8,8 ]}; /* 8 = EpredL */
 Epred_LtoH_start  = 1;
 Epred_LtoH_lm0    = 2;
 Epred_LtoH_lm1    = 3;
 Epred_LtoH_rm0    = 4;
 Epred_LtoH_rm1    = 5;
 Epred_LtoH_non    = 6;
 Epred_LtoH_EpredR = 7;
 Epred_LtoH_EpredL = 8;
 Epred_LtoH_transition(state,d) = Epred_LtoH_state_table[state,d+1];
 Epred_LtoH_transitions(state,v) = /* v is high to low */  \
  forstep(i=#v,1,-1,state=Epred_LtoH_transition(state,v[i])); state;
 Epred_LtoH_state(n) = \
   Epred_LtoH_transitions(Epred_LtoH_start, digits(n,5));

gp_inline__location="../r5dragon.tex:14169";
 EpredR_state_table = {[ 0,0,0,1,2;
                         3,1,0,1,2;
                         3,2,0,1,2;
                         3,3,1,2,3 ]};
 EpredR_transition(state,d) = EpredR_state_table[state+1,d+1];

gp_inline__location="../r5dragon.tex:14175";
 EpredR_state(n) = my(v=digits(n,5),state=0); \
   for(i=1,#v, state=EpredR_transition(state,v[i])); state;
 EpredR_by_high_to_low(n) = EpredR_state(n)==3;

gp_inline__location="../r5dragon.tex:14355";
 DeleteThrees(n) = fromdigits(vector_delete_3s(digits(n,5)),5);

gp_inline__location="../r5dragon.tex:14379";
 EpredR_by_3040(n) = {
   my(v = vector_delete_1s(digits(n,5)));
   forstep(i=#v,1, -1,  /* low to high */
     if(v[i]==3 && i<#v && v[i+1]==0, return(1));  /* 30 */
     if(v[i]==4 && i<#v && v[i+1]==0, return(1));  /* 40 */
     if(v[i]==3,                      return(0));  /* no 3 */
     if(v[i]==2 && !(i<#v && v[i+1]==0), return(0)); /* 2 pair 20 */
     );
   0;
 }

gp_inline__location="../r5dragon.tex:14398";
 EpredR_by_pairs(n) = {
   my(v = vector_delete_1s(digits(n,5)));
   if(#v && (v[#v]==2 || v[#v]==3),return(0));
   forstep(i=#v-1,1, -1,  /* low to high */
     if(v[i]==3 && v[i+1]==0, return(1));  /* 30 */
     if(v[i]==4 && v[i+1]==0, return(1));  /* 40 */
     if(v[i]==2 && v[i+1]==2, return(0));  /* 22 */
     if(v[i]==3 && v[i+1]==2, return(0));  /* 32 */
     if(v[i]==2 && v[i+1]==4, return(0));  /* 24 */
     if(v[i]==3 && v[i+1]==4, return(0))); /* 34 */
   0;
 }

gp_inline__location="../r5dragon.tex:14423";
 EpredL_by_1012(n) = {
   my(v = vector_delete_3s(digits(n,5)));
   forstep(i=#v,1, -1,  /* low to high */
     if(v[i]==1 && i<#v && v[i+1]==0, return(1));  /* 10 */
     if(v[i]==1 && i<#v && v[i+1]==2, return(1));  /* 12 */
     if(v[i]==2 && i<#v && v[i+1]==0, return(1));  /* 20 */
     if(v[i]==2 && i<#v && v[i+1]==2, return(1));  /* 22 */
     if(v[i]==1, return(0));
     if(v[i]==0,
        my(end=0);
        for(j=i+1,#v, if(v[j]!=0, end=v[j];break()));
        if(end!=2, return(0)));
     );
   0;
 }

gp_inline__location="../r5dragon.tex:14450";
 EpredL_by_pairs(n) = {
   my(v = vector_delete_3s(digits(n,5)));
   if(#v && (v[#v]==0 || v[#v]==1),return(0));
   forstep(i=#v-1,1, -1,  /* low to high */
     if(v[i]==1 && v[i+1]==0, return(1));  /* 10 */
     if(v[i]==1 && v[i+1]==2, return(1));  /* 12 */
     if(v[i]==2 && v[i+1]==0, return(1));  /* 20 */
     if(v[i]==2 && v[i+1]==2, return(1));  /* 22 */
     if(v[i]==0 && v[i+1]==4, return(0));  /* 04 */
     if(v[i]==1 && v[i+1]==4, return(0))); /* 14 */
   0;
 }

gp_inline__location="../r5dragon.tex:14509";
 EpredR_by_non_runs(n) = {
   my(v = Vecrev(digits(n,5)), non=0);
   for(i=1,#v,  /* low to high */
     if(non==0, if(v[i]==2 || v[i]==3, return(0));
                if(v[i]==non, non=2),
        if(v[i]==3 || v[i]==4, return(1));
        if(v[i]==non, non=0)));
   0;
 }

gp_inline__location="../r5dragon.tex:14552";
 EpredR_with_23_33(n) = {
   my(v = vector_delete_1s(digits(n,5)));
   if(#v && (v[#v]==2 || v[#v]==3),return(0));
   forstep(i=#v-1,1, -1,  /* low to high */
     if(v[i]==3 && v[i+1]==0, return(1));  /* 30 */
     if(v[i]==4 && v[i+1]==0, return(1));  /* 40 */
     if(v[i]==2 && v[i+1]==2, return(0));  /* 22 */
     if(v[i]==3 && v[i+1]==2, return(0));  /* 32 */
     if(v[i]==2 && v[i+1]==3, return(0));  /* 23 */
     if(v[i]==3 && v[i+1]==3, return(0));  /* 33 */
     if(v[i]==2 && v[i+1]==4, return(0));  /* 24 */
     if(v[i]==3 && v[i+1]==4, return(0))); /* 34 */
   0;
 }

gp_inline__location="../r5dragon.tex:14576";
 EpredR_by_20_30(n) = {
   my(v = vector_delete_1s(digits(n,5)));
   forstep(i=#v,1, -1,  /* low to high */
     if(v[i]==2 || v[i]==3,  /* any 2,3 as 20,30 */
        if(! (i<#v && v[i+1]==0), return(0)));
     if(i < #v && (v[i]==3 || v[i]==4) && v[i+1]==0,
        return(1));  /* 30,40 */
     );
   0;
 }

gp_inline__location="../r5dragon.tex:14602";
 EpredR_by_Tperm(n) = {
   my(v = digits(n,5));
   v = vector_delete_1s(v);
   v = Tperm_digits(v);
   forstep(i=#v,1, -1,  /* low to high */
     if(v[i]==2 || v[i]==3,  /* 2,3 has 0 below */
        return(i<#v && v[i+1]==0)));
   0;
 }

gp_inline__location="../r5dragon.tex:14614";
 DeleteOnes(n) = fromdigits(vector_delete_1s(digits(n,5)),5);

gp_inline__location="../r5dragon.tex:14627";
 Tperm_digits(v) = {
   for(i=1,#v-1,
       if(v[i]==4 && v[i+1]==0, v[i]=2,
          v[i]==2 && v[i+1]==0, v[i]=4));
   v;
 }
 Tperm(n) = fromdigits(Tperm_digits(digits(n,5)),5);

gp_inline__location="../r5dragon.tex:14637";
 Tperm_samples = {[0,1,2,3,4,5,6,7,8,9,20,11]};

gp_inline__location="../r5dragon.tex:14668";
 AR_by_EpredR(k) = sum(n=0,5^k-1,EpredR(n));

gp_inline__location="../r5dragon.tex:14671";
 AL_by_EpredL(k) = sum(n=0,5^k-1,EpredL(n));

gp_inline__location="../r5dragon.tex:14674";
 A_by_Epred(k) = sum(n=0,5^k-1,Epred(n));

gp_inline__location="../r5dragon.tex:14732";
 EpredR_total_by_sum(k) = 2*sum(h=0,k-2, 5^h*sum(t=0,k-h-2, 3^t));

gp_inline__location="../r5dragon.tex:14737";
 EpredR_total_by_sum2(k) = \
   2*sum(h=0,k-2, 5^h*(3^(k-1-h)-1)/(3-1));

gp_inline__location="../r5dragon.tex:14748";
 EpredR_total_by_sum3(k) = \
   3*(5^(k-1) - 3^(k-1))/(5-3) - (5^(k-1) - 1)/(5-1);

gp_inline__location="../r5dragon.tex:14821";
 EpredR3(n) = ((n%25)==4 || (n%25)==24) && EpredR(n\25);
 EpredL3(n) = (n%5)==2 && EpredL(n\5);

gp_inline__location="../r5dragon.tex:14824";
 EpredR3_by_each(n) = \
   for(i=0,2, if(!EpredR(n+i), return(0))); 1;

gp_inline__location="../r5dragon.tex:14829";
 EpredL3_by_each(n) = \
   for(i=0,2, if(!EpredL(n+i), return(0))); 1;

gp_inline__location="../r5dragon.tex:14848";
 EpredR3_samples = {[379,399,504,524,1004,1024,1129,1149]};

gp_inline__location="../r5dragon.tex:14865";
 EpredL3_samples = {[37,62,137,162,187,192,197,212]};

gp_inline__location="../r5dragon.tex:15077";
 EpredSix(n) = (n%25) == 22 && EpredL(n\25);
 EpredSix_by_each(n) = \
   for(i=0,5, if(!Epred(n+i), return(0))); 1;

gp_inline__location="../r5dragon.tex:15112";
 EncSix_formula(k) = 1/100*5^k - 1/18*3^k + 1/4;
 EncSix(k) = if(k<=1,0, EncSix_formula(k));

gp_inline__location="../r5dragon.tex:15267";
 Hwp(k) = {
   my(total=0,d=1,prev_j=0);
   while(1,
         my(j=k-floor(Hwp_a_inexact*d));
         if(j<0,break());
         total+=I^(d-1)*b^j;
         d++; prev_j=j);
   if(prev_j==1, total+=I^(d-1)*b^0);
   total;
 }

gp_inline__location="../r5dragon.tex:15280";
 Hwp_samples = {[0,1,1+3*I,-6+5*I,-16-5*I,-6-38*I,71-50*I]};

gp_inline__location="../r5dragon.tex:15302";
 Hwp_a_inexact = Pi/(2*atan(2));

gp_inline__location="../r5dragon.tex:15309";
 Hwp_a_sample = {(1418776/1000000)};

gp_inline__location="../r5dragon.tex:15335";
 Hwp_exponents_samples = {[1,2,4,5,7,8,9,11,12,14,15,17,18,19]};

gp_inline__location="../r5dragon.tex:15371";
 Hep(k) = {
   my(total=0,d=2,prev_j=0);
   while(1,
         my(j=k-floor(Hwp_a_inexact*d));
         if(j<0,break());
         total+=I^(d-2)*b^j;
         d++; prev_j=j);
   if(prev_j==1, total+=I^(d-2)*b^0);
   total;
 }

gp_inline__location="../r5dragon.tex:15384";
 Hep_samples = {[0,0,1,1+3*I,-3+5*I,-14-I,-12-30*I]};

gp_inline__location="../r5dragon.tex:15623";
 b_gamma = Pi/2 - arg(b);

gp_inline__location="../r5dragon.tex:15629";
 b_gamma_degrees_sample = {(26565051/1000000)};

gp_inline__location="../r5dragon.tex:15637";
 b_gamma_radians_sample = {(0463647/1000000)};

gp_inline__location="../r5dragon.tex:15704";
 Hwp_h_inexact = Pi/2 / b_gamma;

gp_inline__location="../r5dragon.tex:15710";
 Hwp_h_sample = {(3387909/1000000)};

gp_inline__location="../r5dragon.tex:15725";
 Hwp_non_exponents_samples = {[0,3,6,10,13,16,20,23,27,30,33,37]};

gp_inline__location="../r5dragon.tex:15742";
 \\ 1/alpha + 1/beta == 1
 Beatty_other(a) = 1/(1 - 1/a);

gp_inline__location="../r5dragon.tex:15855";
 Hwpf_by_terms(max_d) = \
   sum(d=1,max_d,  I^(d-1) / b^floor(Hwp_a_inexact*d) );
 Hwpf = Hwpf_by_terms(100);

gp_inline__location="../r5dragon.tex:15861";
 Hwpf_sample = {(0385314/1000000)-(0576988/1000000)*I};

gp_inline__location="../r5dragon.tex:15886";
 Hepf_by_terms(max_d) =if(max_d<2,0, -I*(Hwpf_by_terms(max_d)-1/b));
 Hepf = Hepf_by_terms(100);

gp_inline__location="../r5dragon.tex:15891";
 Hepf_sample = {-(0176988/1000000)-(0185314/1000000)*I};

gp_inline__location="../r5dragon.tex:15923";
 Hwpf_width_by_re(k) = real(sum(d=1,k, I^d/b^floor(Hwp_a_inexact*d)));

gp_inline__location="../r5dragon.tex:15927";
 Hwpf_real_sample = {-(0576988/1000000)};

gp_inline__location="../r5dragon.tex:15939";
 Hepf_ext_by_re(k) = real(sum(d=2,k, I^d/b^floor(Hwp_a_inexact*d)));

gp_inline__location="../r5dragon.tex:15943";
 Hepf_real_sample = {-(0176988/1000000)};

gp_inline__location="../r5dragon.tex:15969";
 Hwpf_bound = 1/4*(1 + sqrt5);

gp_inline__location="../r5dragon.tex:15978";
 Hwpf_bound_sample = {(0809016/1000000)};

gp_inline__location="../r5dragon.tex:16059";
 HewA(k) = ( -2*imag(Hwp(k)/b^k) )*( 1-2*real(Hep(k)/b^k) )*5^k;

gp_inline__location="../r5dragon.tex:16062";
 HewA_samples = {[0,4,(806)/(25),(21306)/(125),(574454)/(625)]};

gp_inline__location="../r5dragon.tex:16085";
 HewAf = -2*imag(Hwpf) * (1-2*real(Hepf));

gp_inline__location="../r5dragon.tex:16088";
 Hwpf_half_width = -imag(Hwpf);
 Hepf_ext        = -real(Hepf);

gp_inline__location="../r5dragon.tex:16098";
 HewAf_sample = {(1562458/1000000)};

gp_inline__location="../r5dragon.tex:16120";
 Hwpf_symbolic(emax) = {
   sum(d=1,emax,
       my(e=floor(Hwp_a_inexact*d));
       if(e<=emax,
          my(f=I^d);
          if(f==1, 1/'rb^e, f==-1, -1/'rb^e,
             f==I, 1/'ib^e, f==-I, -1/'ib^e,  error())));
 }

gp_inline__location="../r5dragon.tex:16139";
 Hepf_bound = Hwpf_bound - 1/sqrt5;

gp_inline__location="../r5dragon.tex:16197";
 Hdigit(j) = j%Hwp_h_inexact < Hwp_h_inexact-1;

gp_inline__location="../r5dragon.tex:16261";
 vector_substrings_set(v,len) = \
   Set(vector(#v-(len-1),i, v[i..i+len-1]));

gp_inline__location="../r5dragon.tex:16267";
 Sturmian_substrings_completion(v,len) = {
   my(l=List([]));
   for(i=len,#v,
       my(s=v[i-len+1..i],
          pos=setsearch(l,s,1));
       if(pos,
          listinsert(l,s,pos);
          if(#l==len+1,  \\ print(l);
             return(i))));
   0;
 }

gp_inline__location="../r5dragon.tex:16297";
 one_over_Hwp_a_sample = {(0704832/1000000)};

gp_inline__location="../r5dragon.tex:16341";
 HwN_vector(k) = vector(k,j, j==k || Hdigit(j));
 HwN(k) = fromdigits(HwN_vector(k),5);

gp_inline__location="../r5dragon.tex:16348";
 HwN_base5_samples = {[0,1,11,111,1101,11011,110111,1101101]};

gp_inline__location="../r5dragon.tex:16366";
 HwN_samples = {[0,1,6,31,151,756,3781,18901]};

gp_inline__location="../r5dragon.tex:16382";
 HeN(k) = if(k<=1,0, HwN(k)-5^(k-1));

gp_inline__location="../r5dragon.tex:16388";
 HeN_base5_samples = {[0,0,1,11,101,1011,10111,101101]};

gp_inline__location="../r5dragon.tex:16406";
 HeN_samples = {[0,0,1,6,26,131,656,3276]};

gp_inline__location="../r5dragon.tex:16422";
 my(k=200); HwNf = HwN(k)/5^k;

gp_inline__location="../r5dragon.tex:16429";
 HwNf_base5_sample = {(0110110111/1000000000)};

gp_inline__location="../r5dragon.tex:16437";
 HwNf_sample = {(0241935/1000000)};

gp_inline__location="../r5dragon.tex:16451";
 my(k=200); HeNf = HeN(k)/5^k;

gp_inline__location="../r5dragon.tex:16458";
 HeNf_base5_sample = {(0010110111/1000000000)};

gp_inline__location="../r5dragon.tex:16466";
 HeNf_sample = {(0041935/1000000)};

gp_inline__location="../r5dragon.tex:16501";
 Sturmian_vector(a,len) = {
   vector(len,n, floor((n+1)*a)
               - floor( n   *a));
 }

gp_inline__location="../r5dragon.tex:16552";
 \\ convex hulls around R5 curve k, first side horizontal
 hull_generated_samples = {[
   /* k=0 */ [ 0+I*0,1+I*0 ],
   /* k=1 */ [ 0+I*0,1+I*0,1+I*2,0+I*2 ],
   /* k=2 */ [ 0+I*0,1+I*0,1+I*3,-3+I*4,-4+I*4,-4+I*1 ],
   /* k=3 */ [ -8+I*-7,-5+I*-7,1+I*0,1+I*3,-3+I*5,-6+I*5,-12+I*-2,
     -12+I*-5 ],
   /* k=4 */ [ -4+I*-29,-1+I*-29,7+I*-23,9+I*-19,9+I*-16,1+I*3,
     -3+I*5,-6+I*5,-14+I*-1,-16+I*-5,-16+I*-8,-8+I*-27 ],
   /* k=5 */ [ 44+I*-43,47+I*-43,55+I*-37,57+I*-33,57+I*-30,
     53+I*-8,47+I*0,43+I*2,-3+I*5,-6+I*5,-14+I*-1,-16+I*-5,
     -16+I*-8,-12+I*-30,-6+I*-38,-2+I*-40 ],
   /* k=6 */ [ 46+I*-54,49+I*-54,71+I*-50,79+I*-44,131+I*45,
     133+I*49,133+I*52,129+I*74,123+I*82,119+I*84,71+I*98,68+I*98,
     46+I*94,38+I*88,-14+I*-1,-16+I*-5,-16+I*-8,-12+I*-30,-6+I*-38,
     -2+I*-40 ],
   /* k=7 */ [ 46+I*-54,49+I*-54,71+I*-50,79+I*-44,155+I*38,
     157+I*42,171+I*90,171+I*93,167+I*115,41+I*308,35+I*316,
     31+I*318,-17+I*332,-20+I*332,-42+I*328,-50+I*322,-126+I*240,
     -128+I*236,-142+I*188,-142+I*185,-138+I*163,-12+I*-30,
     -6+I*-38,-2+I*-40 ],
   /* k=8 */ [ -466+I*-113,-463+I*-113,49+I*-54,71+I*-50,79+I*-44,
     155+I*38,157+I*42,171+I*90,171+I*93,167+I*115,79+I*349,
     73+I*357,-9+I*433,-13+I*435,-61+I*449,-64+I*449,-576+I*390,
     -598+I*386,-606+I*380,-682+I*298,-684+I*294,-698+I*246,
     -698+I*243,-694+I*221,-606+I*-13,-600+I*-21,-518+I*-97,
     -514+I*-99 ],
   /* k=9 */ [ -582+I*-1225,-579+I*-1225,-557+I*-1221,-323+I*-1133,
     -315+I*-1127,-239+I*-1045,-237+I*-1041,157+I*42,171+I*90,
     171+I*93,167+I*115,79+I*349,73+I*357,-9+I*433,-13+I*435,
     -61+I*449,-617+I*507,-620+I*507,-642+I*503,-876+I*415,
     -884+I*409,-960+I*327,-962+I*323,-1356+I*-760,-1370+I*-808,
     -1370+I*-811,-1366+I*-833,-1278+I*-1067,-1272+I*-1075,
     -1190+I*-1151,-1186+I*-1153,-1138+I*-1167 ],
   /* k=10 */ [ 854+I*-3623,857+I*-3623,879+I*-3619,1113+I*-3531,
     1121+I*-3525,1197+I*-3443,1869+I*-2389,1871+I*-2385,
     1885+I*-2337,1943+I*-1781,1943+I*-1778,1939+I*-1756,
     1851+I*-1522,1845+I*-1514,73+I*357,-9+I*433,-13+I*435,
     -61+I*449,-617+I*507,-620+I*507,-642+I*503,-876+I*415,
     -884+I*409,-960+I*327,-1632+I*-727,-1634+I*-731,-1648+I*-779,
     -1706+I*-1335,-1706+I*-1338,-1702+I*-1360,-1614+I*-1594,
     -1608+I*-1602,164+I*-3473,246+I*-3549,250+I*-3551,298+I*-3565
     ],
   /* k=11 */ [ 1572+I*-4822,1575+I*-4822,1597+I*-4818,
     7111+I*-3145,7345+I*-3057,7353+I*-3051,7429+I*-2969,
     8101+I*-1915,8103+I*-1911,8117+I*-1863,8175+I*-1307,
     8175+I*-1304,8171+I*-1282,8083+I*-1048,6647+I*1350,
     6641+I*1358,6559+I*1434,5505+I*2106,5501+I*2108,5453+I*2122,
     4897+I*2180,4894+I*2180,4872+I*2176,-642+I*503,-876+I*415,
     -884+I*409,-960+I*327,-1632+I*-727,-1634+I*-731,-1648+I*-779,
     -1706+I*-1335,-1706+I*-1338,-1702+I*-1360,-1614+I*-1594,
     -178+I*-3992,-172+I*-4000,-90+I*-4076,964+I*-4748,968+I*-4750,
     1016+I*-4764 ],
   /* k=12 */ [ 1572+I*-4822,1575+I*-4822,7807+I*-4348,
     7829+I*-4344,8063+I*-4256,10461+I*-2820,10469+I*-2814,
     10545+I*-2732,11217+I*-1678,11219+I*-1674,11233+I*-1626,
     13401+I*11075,13459+I*11631,13459+I*11634,13455+I*11656,
     13367+I*11890,11931+I*14288,11925+I*14296,11843+I*14372,
     10789+I*15044,10785+I*15046,10737+I*15060,10181+I*15118,
     10178+I*15118,3946+I*14644,3924+I*14640,3690+I*14552,
     1292+I*13116,1284+I*13110,1208+I*13028,536+I*11974,
     534+I*11970,520+I*11922,-1648+I*-779,-1706+I*-1335,
     -1706+I*-1338,-1702+I*-1360,-1614+I*-1594,-178+I*-3992,
     -172+I*-4000,-90+I*-4076,964+I*-4748,968+I*-4750,1016+I*-4764
     ],
   /* k=13 */ [ 1572+I*-4822,1575+I*-4822,7807+I*-4348,
     7829+I*-4344,8063+I*-4256,10461+I*-2820,10469+I*-2814,
     10545+I*-2732,11217+I*-1678,11219+I*-1674,16503+I*11264,
     16517+I*11312,16575+I*11868,16575+I*11871,16101+I*18103,
     16097+I*18125,16009+I*18359,14573+I*20757,14567+I*20765,
     14485+I*20841,-8749+I*37878,-9803+I*38550,-9807+I*38552,
     -9855+I*38566,-10411+I*38624,-10414+I*38624,-16646+I*38150,
     -16668+I*38146,-16902+I*38058,-19300+I*36622,-19308+I*36616,
     -19384+I*36534,-20056+I*35480,-20058+I*35476,-25342+I*22538,
     -25356+I*22490,-25414+I*21934,-25414+I*21931,-24940+I*15699,
     -24936+I*15677,-24848+I*15443,-23412+I*13045,-23406+I*13037,
     -23324+I*12961,-90+I*-4076,964+I*-4748,968+I*-4750,
     1016+I*-4764 ],
   /* k=14 */ [ -55736+I*-34253,-55733+I*-34253,-49501+I*-33779,
     -49479+I*-33775,-49245+I*-33687,8063+I*-4256,10461+I*-2820,
     10469+I*-2814,10545+I*-2732,11217+I*-1678,11219+I*-1674,
     16503+I*11264,16517+I*11312,16575+I*11868,16575+I*11871,
     16101+I*18103,16097+I*18125,16009+I*18359,14573+I*20757,
     14567+I*20765,-6025+I*44271,-6107+I*44347,-7161+I*45019,
     -7165+I*45021,-20103+I*50305,-20151+I*50319,-20707+I*50377,
     -20710+I*50377,-26942+I*49903,-26964+I*49899,-27198+I*49811,
     -84506+I*20380,-86904+I*18944,-86912+I*18938,-86988+I*18856,
     -87660+I*17802,-87662+I*17798,-92946+I*4860,-92960+I*4812,
     -93018+I*4256,-93018+I*4253,-92544+I*-1979,-92540+I*-2001,
     -92452+I*-2235,-91016+I*-4633,-91010+I*-4641,-70418+I*-28147,
     -70336+I*-28223,-69282+I*-28895,-69278+I*-28897,
     -56340+I*-34181,-56292+I*-34195 ],
   /* k=15 */ [ -87984+I*-187139,-87981+I*-187139,-81749+I*-186665,
     -81727+I*-186661,-14123+I*-168983,-13889+I*-168895,
     -11491+I*-167459,-11483+I*-167453,12023+I*-146861,
     12099+I*-146779,12771+I*-145725,12773+I*-145721,
     18057+I*-132783,18071+I*-132735,18129+I*-132179,
     18129+I*-132176,16575+I*11871,16101+I*18103,16097+I*18125,
     16009+I*18359,14573+I*20757,14567+I*20765,-6025+I*44271,
     -6107+I*44347,-7161+I*45019,-7165+I*45021,-20103+I*50305,
     -20151+I*50319,-20707+I*50377,-20710+I*50377,-26942+I*49903,
     -26964+I*49899,-94568+I*32221,-94802+I*32133,-97200+I*30697,
     -97208+I*30691,-120714+I*10099,-120790+I*10017,-121462+I*8963,
     -121464+I*8959,-126748+I*-3979,-126762+I*-4027,
     -126820+I*-4583,-126820+I*-4586,-125266+I*-148633,
     -124792+I*-154865,-124788+I*-154887,-124700+I*-155121,
     -123264+I*-157519,-123258+I*-157527,-102666+I*-181033,
     -102584+I*-181109,-101530+I*-181781,-101526+I*-181783,
     -88588+I*-187067,-88540+I*-187081 ],
   /* k=16 */ [ 185540+I*-404521,185543+I*-404521,191775+I*-404047,
     191797+I*-404043,259401+I*-386365,259635+I*-386277,
     262033+I*-384841,262041+I*-384835,285547+I*-364243,
     285623+I*-364161,286295+I*-363107,286297+I*-363103,
     291581+I*-350165,291595+I*-350117,323843+I*-197231,
     323901+I*-196675,323901+I*-196672,323427+I*-190440,
     323423+I*-190418,305745+I*-122814,305657+I*-122580,
     304221+I*-120182,304215+I*-120174,283623+I*-96668,
     283541+I*-96592,282487+I*-95920,282483+I*-95918,-7165+I*45021,
     -20103+I*50305,-20151+I*50319,-20707+I*50377,-20710+I*50377,
     -26942+I*49903,-26964+I*49899,-94568+I*32221,-94802+I*32133,
     -97200+I*30697,-97208+I*30691,-120714+I*10099,-120790+I*10017,
     -121462+I*8963,-121464+I*8959,-126748+I*-3979,-126762+I*-4027,
     -159010+I*-156913,-159068+I*-157469,-159068+I*-157472,
     -158594+I*-163704,-158590+I*-163726,-140912+I*-231330,
     -140824+I*-231564,-139388+I*-233962,-139382+I*-233970,
     -118790+I*-257476,-118708+I*-257552,-117654+I*-258224,
     -117650+I*-258226,171998+I*-399165,184936+I*-404449,
     184984+I*-404463 ],
   /* k=17 */ [ 322302+I*-513212,322305+I*-513212,328537+I*-512738,
     328559+I*-512734,396163+I*-495056,396397+I*-494968,
     398795+I*-493532,398803+I*-493526,970329+I*-55169,
     993835+I*-34577,993911+I*-34495,994583+I*-33441,
     994585+I*-33437,999869+I*-20499,999883+I*-20451,
     1032131+I*132435,1032189+I*132991,1032189+I*132994,
     1031715+I*139226,1031711+I*139248,1014033+I*206852,
     1013945+I*207086,1012509+I*209484,1012503+I*209492,
     991911+I*232998,991829+I*233074,718305+I*450456,
     717251+I*451128,717247+I*451130,704309+I*456414,
     704261+I*456428,551375+I*488676,550819+I*488734,
     550816+I*488734,544584+I*488260,544562+I*488256,
     476958+I*470578,476724+I*470490,474326+I*469054,
     474318+I*469048,-97208+I*30691,-120714+I*10099,
     -120790+I*10017,-121462+I*8963,-121464+I*8959,-126748+I*-3979,
     -126762+I*-4027,-159010+I*-156913,-159068+I*-157469,
     -159068+I*-157472,-158594+I*-163704,-158590+I*-163726,
     -140912+I*-231330,-140824+I*-231564,-139388+I*-233962,
     -139382+I*-233970,-118790+I*-257476,-118708+I*-257552,
     154816+I*-474934,155870+I*-475606,155874+I*-475608,
     168812+I*-480892,168860+I*-480906,321746+I*-513154 ],
   /* k=18 */ [ 322302+I*-513212,322305+I*-513212,328537+I*-512738,
     328559+I*-512734,396163+I*-495056,396397+I*-494968,
     1104685+I*-165302,1107083+I*-163866,1107091+I*-163860,
     1130597+I*-143268,1130673+I*-143186,1348055+I*130338,
     1348727+I*131392,1348729+I*131396,1354013+I*144334,
     1354027+I*144382,1386275+I*297268,1386333+I*297824,
     1386333+I*297827,1385859+I*304059,1385855+I*304081,
     1080667+I*1885490,1062989+I*1953094,1062901+I*1953328,
     1061465+I*1955726,1061459+I*1955734,1040867+I*1979240,
     1040785+I*1979316,767261+I*2196698,766207+I*2197370,
     766203+I*2197372,753265+I*2202656,753217+I*2202670,
     600331+I*2234918,599775+I*2234976,599772+I*2234976,
     593540+I*2234502,593518+I*2234498,525914+I*2216820,
     525680+I*2216732,-182608+I*1887066,-185006+I*1885630,
     -185014+I*1885624,-208520+I*1865032,-208596+I*1864950,
     -425978+I*1591426,-426650+I*1590372,-426652+I*1590368,
     -431936+I*1577430,-431950+I*1577382,-464198+I*1424496,
     -464256+I*1423940,-464256+I*1423937,-463782+I*1417705,
     -463778+I*1417683,-158590+I*-163726,-140912+I*-231330,
     -140824+I*-231564,-139388+I*-233962,-139382+I*-233970,
     -118790+I*-257476,-118708+I*-257552,154816+I*-474934,
     155870+I*-475606,155874+I*-475608,168812+I*-480892,
     168860+I*-480906,321746+I*-513154 ],
   /* k=19 */ [ 322302+I*-513212,322305+I*-513212,328537+I*-512738,
     328559+I*-512734,396163+I*-495056,396397+I*-494968,
     1104685+I*-165302,1107083+I*-163866,1107091+I*-163860,
     1130597+I*-143268,1130673+I*-143186,1348055+I*130338,
     1348727+I*131392,1348729+I*131396,1354013+I*144334,
     1354027+I*144382,1386275+I*297268,1386333+I*297824,
     1435289+I*2044066,1435289+I*2044069,1434815+I*2050301,
     1434811+I*2050323,1417133+I*2117927,1417045+I*2118161,
     1087379+I*2826449,1085943+I*2828847,1085937+I*2828855,
     1065345+I*2852361,1065263+I*2852437,791739+I*3069819,
     790685+I*3070491,790681+I*3070493,777743+I*3075777,
     777695+I*3075791,-2690311+I*4046824,-2843197+I*4079072,
     -2843753+I*4079130,-2843756+I*4079130,-2849988+I*4078656,
     -2850010+I*4078652,-2917614+I*4060974,-2917848+I*4060886,
     -3626136+I*3731220,-3628534+I*3729784,-3628542+I*3729778,
     -3652048+I*3709186,-3652124+I*3709104,-3869506+I*3435580,
     -3870178+I*3434526,-3870180+I*3434522,-3875464+I*3421584,
     -3875478+I*3421536,-3907726+I*3268650,-3907784+I*3268094,
     -3956740+I*1521852,-3956740+I*1521849,-3956266+I*1515617,
     -3956262+I*1515595,-3938584+I*1447991,-3938496+I*1447757,
     -3608830+I*739469,-3607394+I*737071,-3607388+I*737063,
     -3586796+I*713557,-3586714+I*713481,-3313190+I*496099,
     -3312136+I*495427,-3312132+I*495425,-3299194+I*490141,
     -3299146+I*490127,168860+I*-480906,321746+I*-513154 ],
   /* k=20 */ [ -5087770+I*-6478191,-5087767+I*-6478191,
     -5081535+I*-6477717,-5081513+I*-6477713,-5013909+I*-6460035,
     -5013675+I*-6459947,-4305387+I*-6130281,-4302989+I*-6128845,
     -4302981+I*-6128839,-4279475+I*-6108247,-4279399+I*-6108165,
     1130673+I*-143186,1348055+I*130338,1348727+I*131392,
     1348729+I*131396,1354013+I*144334,1354027+I*144382,
     1386275+I*297268,1386333+I*297824,1435289+I*2044066,
     1435289+I*2044069,1434815+I*2050301,1434811+I*2050323,
     1417133+I*2117927,1417045+I*2118161,1087379+I*2826449,
     1085943+I*2828847,1085937+I*2828855,1065345+I*2852361,
     1065263+I*2852437,791739+I*3069819,790685+I*3070491,
     -2652843+I*4914645,-2652847+I*4914647,-2665785+I*4919931,
     -2665833+I*4919945,-2818719+I*4952193,-2819275+I*4952251,
     -4565517+I*5001207,-4565520+I*5001207,-4571752+I*5000733,
     -4571774+I*5000729,-4639378+I*4983051,-4639612+I*4982963,
     -5347900+I*4653297,-5350298+I*4651861,-5350306+I*4651855,
     -5373812+I*4631263,-5373888+I*4631181,-10783960+I*-1333798,
     -11001342+I*-1607322,-11002014+I*-1608376,
     -11002016+I*-1608380,-11007300+I*-1621318,
     -11007314+I*-1621366,-11039562+I*-1774252,
     -11039620+I*-1774808,-11088576+I*-3521050,
     -11088576+I*-3521053,-11088102+I*-3527285,
     -11088098+I*-3527307,-11070420+I*-3594911,
     -11070332+I*-3595145,-10740666+I*-4303433,
     -10739230+I*-4305831,-10739224+I*-4305839,
     -10718632+I*-4329345,-10718550+I*-4329421,
     -10445026+I*-4546803,-10443972+I*-4547475,-7000444+I*-6391629,
     -7000440+I*-6391631,-6987502+I*-6396915,-6987454+I*-6396929,
     -6834568+I*-6429177,-6834012+I*-6429235 ],
   /* k=21 */ [ -2133802+I*-25784765,-2133799+I*-25784765,
     -2127567+I*-25784291,-2127545+I*-25784287,
     -2059941+I*-25766609,-2059707+I*-25766521,
     -1351419+I*-25436855,-1349021+I*-25435419,5782815+I*-20392517,
     5782823+I*-20392511,5806329+I*-20371919,5806405+I*-20371837,
     6023787+I*-20098313,6024459+I*-20097259,7868613+I*-16653731,
     7868615+I*-16653727,7873899+I*-16640789,7873913+I*-16640741,
     7906161+I*-16487855,7906219+I*-16487299,7955175+I*-14741057,
     7955175+I*-14741054,7954701+I*-14734822,7954697+I*-14734800,
     7937019+I*-14667196,7936931+I*-14666962,1417045+I*2118161,
     1087379+I*2826449,1085943+I*2828847,1085937+I*2828855,
     1065345+I*2852361,1065263+I*2852437,791739+I*3069819,
     790685+I*3070491,-2652843+I*4914645,-2652847+I*4914647,
     -2665785+I*4919931,-2665833+I*4919945,-2818719+I*4952193,
     -2819275+I*4952251,-4565517+I*5001207,-4565520+I*5001207,
     -4571752+I*5000733,-4571774+I*5000729,-4639378+I*4983051,
     -4639612+I*4982963,-5347900+I*4653297,-5350298+I*4651861,
     -12482134+I*-391041,-12482142+I*-391047,-12505648+I*-411639,
     -12505724+I*-411721,-12723106+I*-685245,-12723778+I*-686299,
     -14567932+I*-4129827,-14567934+I*-4129831,
     -14573218+I*-4142769,-14573232+I*-4142817,
     -14605480+I*-4295703,-14605538+I*-4296259,
     -14654494+I*-6042501,-14654494+I*-6042504,
     -14654020+I*-6048736,-14654016+I*-6048758,
     -14636338+I*-6116362,-14636250+I*-6116596,
     -8116364+I*-22901719,-7786698+I*-23610007,
     -7785262+I*-23612405,-7785256+I*-23612413,
     -7764664+I*-23635919,-7764582+I*-23635995,
     -7491058+I*-23853377,-7490004+I*-23854049,
     -4046476+I*-25698203,-4046472+I*-25698205,
     -4033534+I*-25703489,-4033486+I*-25703503,
     -3880600+I*-25735751,-3880044+I*-25735809 ],
   /* k=22 */ [ 39433314+I*-39183403,39433317+I*-39183403,
     39439549+I*-39182929,39439571+I*-39182925,
     39507175+I*-39165247,39507409+I*-39165159,
     40215697+I*-38835493,40218095+I*-38834057,
     47349931+I*-33791155,47349939+I*-33791149,
     47373445+I*-33770557,47373521+I*-33770475,
     47590903+I*-33496951,47591575+I*-33495897,
     49435729+I*-30052369,49435731+I*-30052365,
     49441015+I*-30039427,49441029+I*-30039379,
     49473277+I*-29886493,49473335+I*-29885937,
     49522291+I*-28139695,49522291+I*-28139692,
     49521817+I*-28133460,46567849+I*-8826886,46567845+I*-8826864,
     46550167+I*-8759260,46550079+I*-8759026,46220413+I*-8050738,
     46218977+I*-8048340,41176075+I*-916504,41176069+I*-916496,
     41155477+I*-892990,41155395+I*-892914,40881871+I*-675532,
     40880817+I*-674860,37437289+I*1169294,37437285+I*1169296,
     37424347+I*1174580,37424299+I*1174594,37271413+I*1206842,
     37270857+I*1206900,-2819275+I*4952251,-4565517+I*5001207,
     -4565520+I*5001207,-4571752+I*5000733,-4571774+I*5000729,
     -4639378+I*4983051,-4639612+I*4982963,-5347900+I*4653297,
     -5350298+I*4651861,-12482134+I*-391041,-12482142+I*-391047,
     -12505648+I*-411639,-12505724+I*-411721,-12723106+I*-685245,
     -12723778+I*-686299,-14567932+I*-4129827,-14567934+I*-4129831,
     -14573218+I*-4142769,-14573232+I*-4142817,
     -14605480+I*-4295703,-14605538+I*-4296259,
     -14654494+I*-6042501,-14654494+I*-6042504,
     -14654020+I*-6048736,-11700052+I*-25355310,
     -11700048+I*-25355332,-11682370+I*-25422936,
     -11682282+I*-25423170,-11352616+I*-26131458,
     -11351180+I*-26133856,-6308278+I*-33265692,
     -6308272+I*-33265700,-6287680+I*-33289206,
     -6287598+I*-33289282,-6014074+I*-33506664,
     -6013020+I*-33507336,-2569492+I*-35351490,
     -2569488+I*-35351492,-2556550+I*-35356776,
     -2556502+I*-35356790,-2403616+I*-35389038,
     -2403060+I*-35389096,37687072+I*-39134447 ],
   /* k=23 */ [ 40910298+I*-48836690,40910301+I*-48836690,
     40916533+I*-48836216,60223107+I*-45882248,
     60223129+I*-45882244,60290733+I*-45864566,
     60290967+I*-45864478,60999255+I*-45534812,
     61001653+I*-45533376,68133489+I*-40490474,
     68133497+I*-40490468,68157003+I*-40469876,
     68157079+I*-40469794,68374461+I*-40196270,
     68375133+I*-40195216,115955967+I*36239697,
     117800121+I*39683225,117800123+I*39683229,
     117805407+I*39696167,117805421+I*39696215,
     117837669+I*39849101,117837727+I*39849657,
     117886683+I*41595899,117886683+I*41595902,
     117886209+I*41602134,114932241+I*60908708,
     114932237+I*60908730,114914559+I*60976334,
     114914471+I*60976568,114584805+I*61684856,
     114583369+I*61687254,109540467+I*68819090,
     109540461+I*68819098,109519869+I*68842604,
     109519787+I*68842680,109246263+I*69060062,
     109245209+I*69060734,105801681+I*70904888,
     105801677+I*70904890,105788739+I*70910174,64221623+I*84308812,
     64221575+I*84308826,64068689+I*84341074,64068133+I*84341132,
     62321891+I*84390088,62321888+I*84390088,62315656+I*84389614,
     43009082+I*81435646,43009060+I*81435642,42941456+I*81417964,
     42941222+I*81417876,42232934+I*81088210,42230536+I*81086774,
     35098700+I*76043872,35098692+I*76043866,35075186+I*76023274,
     35075110+I*76023192,34857728+I*75749668,34857056+I*75748614,
     -12723778+I*-686299,-14567932+I*-4129827,-14567934+I*-4129831,
     -14573218+I*-4142769,-14573232+I*-4142817,
     -14605480+I*-4295703,-14605538+I*-4296259,
     -14654494+I*-6042501,-14654494+I*-6042504,
     -14654020+I*-6048736,-11700052+I*-25355310,
     -11700048+I*-25355332,-11682370+I*-25422936,
     -11682282+I*-25423170,-11352616+I*-26131458,
     -11351180+I*-26133856,-6308278+I*-33265692,
     -6308272+I*-33265700,-6287680+I*-33289206,
     -6287598+I*-33289282,-6014074+I*-33506664,
     -6013020+I*-33507336,-2569492+I*-35351490,
     -2569488+I*-35351492,-2556550+I*-35356776,
     39010566+I*-48755414,39010614+I*-48755428,
     39163500+I*-48787676,39164056+I*-48787734 ],
   /* k=24 */ [ 40910298+I*-48836690,40910301+I*-48836690,
     40916533+I*-48836216,60223107+I*-45882248,
     60223129+I*-45882244,60290733+I*-45864566,
     60290967+I*-45864478,60999255+I*-45534812,
     61001653+I*-45533376,68133489+I*-40490474,
     68133497+I*-40490468,68157003+I*-40469876,
     136521395+I*29265718,136521471+I*29265800,
     136738853+I*29539324,136739525+I*29540378,
     138583679+I*32983906,138583681+I*32983910,
     138588965+I*32996848,151987603+I*74563964,
     151987617+I*74564012,152019865+I*74716898,
     152019923+I*74717454,152068879+I*76463696,
     152068879+I*76463699,152068405+I*76469931,
     149114437+I*95776505,149114433+I*95776527,
     149096755+I*95844131,149096667+I*95844365,
     148767001+I*96552653,148765565+I*96555051,
     43476573+I*268151632,38433671+I*275283468,
     38433665+I*275283476,38413073+I*275306982,
     38412991+I*275307058,38139467+I*275524440,
     38138413+I*275525112,34694885+I*277369266,
     34694881+I*277369268,34681943+I*277374552,
     -6885173+I*290773190,-6885221+I*290773204,
     -7038107+I*290805452,-7038663+I*290805510,
     -8784905+I*290854466,-8784908+I*290854466,
     -8791140+I*290853992,-28097714+I*287900024,
     -28097736+I*287900020,-28165340+I*287882342,
     -28165574+I*287882254,-28873862+I*287552588,
     -28876260+I*287551152,-36008096+I*282508250,
     -36008104+I*282508244,-36031610+I*282487652,
     -104396002+I*212752058,-104396078+I*212751976,
     -104613460+I*212478452,-104614132+I*212477398,
     -106458286+I*209033870,-106458288+I*209033866,
     -106463572+I*209020928,-119862210+I*167453812,
     -119862224+I*167453764,-119894472+I*167300878,
     -119894530+I*167300322,-119943486+I*165554080,
     -119943486+I*165554077,-119943012+I*165547845,
     -116989044+I*146241271,-116989040+I*146241249,
     -116971362+I*146173645,-116971274+I*146173411,
     -116641608+I*145465123,-116640172+I*145462725,
     -11351180+I*-26133856,-6308278+I*-33265692,
     -6308272+I*-33265700,-6287680+I*-33289206,
     -6287598+I*-33289282,-6014074+I*-33506664,
     -6013020+I*-33507336,-2569492+I*-35351490,
     -2569488+I*-35351492,-2556550+I*-35356776,
     39010566+I*-48755414,39010614+I*-48755428,
     39163500+I*-48787676,39164056+I*-48787734 ],
   /* k=25 */ [ -407571856+I*-87818093,-407571853+I*-87818093,
     -407565621+I*-87817619,40916533+I*-48836216,
     60223107+I*-45882248,60223129+I*-45882244,
     60290733+I*-45864566,60290967+I*-45864478,
     60999255+I*-45534812,61001653+I*-45533376,
     68133489+I*-40490474,68133497+I*-40490468,
     68157003+I*-40469876,136521395+I*29265718,
     136521471+I*29265800,136738853+I*29539324,
     136739525+I*29540378,138583679+I*32983906,
     138583681+I*32983910,138588965+I*32996848,
     151987603+I*74563964,151987617+I*74564012,
     152019865+I*74716898,152019923+I*74717454,
     152068879+I*76463696,152068879+I*76463699,
     152068405+I*76469931,149114437+I*95776505,
     149114433+I*95776527,149096755+I*95844131,
     77989959+I*302308509,77989871+I*302308743,
     77660205+I*303017031,77658769+I*303019429,
     72615867+I*310151265,72615861+I*310151273,
     72595269+I*310174779,2859675+I*378539171,2859593+I*378539247,
     2586069+I*378756629,2585015+I*378757301,-858513+I*380601455,
     -858517+I*380601457,-871455+I*380606741,-42438571+I*394005379,
     -42438619+I*394005393,-42591505+I*394037641,
     -42592061+I*394037699,-44338303+I*394086655,
     -44338306+I*394086655,-44344538+I*394086181,
     -492826692+I*355104778,-512133266+I*352150810,
     -512133288+I*352150806,-512200892+I*352133128,
     -512201126+I*352133040,-512909414+I*351803374,
     -512911812+I*351801938,-520043648+I*346759036,
     -520043656+I*346759030,-520067162+I*346738438,
     -588431554+I*277002844,-588431630+I*277002762,
     -588649012+I*276729238,-588649684+I*276728184,
     -590493838+I*273284656,-590493840+I*273284652,
     -590499124+I*273271714,-603897762+I*231704598,
     -603897776+I*231704550,-603930024+I*231551664,
     -603930082+I*231551108,-603979038+I*229804866,
     -603979038+I*229804863,-603978564+I*229798631,
     -601024596+I*210492057,-601024592+I*210492035,
     -601006914+I*210424431,-529900118+I*3960053,
     -529900030+I*3959819,-529570364+I*3251531,
     -529568928+I*3249133,-524526026+I*-3882703,
     -524526020+I*-3882711,-524505428+I*-3906217,
     -454769834+I*-72270609,-454769752+I*-72270685,
     -454496228+I*-72488067,-454495174+I*-72488739,
     -451051646+I*-74332893,-451051642+I*-74332895,
     -451038704+I*-74338179,-409471588+I*-87736817,
     -409471540+I*-87736831,-409318654+I*-87769079,
     -409318098+I*-87769137 ],
   /* k=26 */ [ -536073428+I*-1055889197,-536073425+I*-1055889197,
     -536067193+I*-1055888723,-516760619+I*-1052934755,
     -516760597+I*-1052934751,-516692993+I*-1052917073,
     -310228615+I*-981810277,-310228381+I*-981810189,
     -309520093+I*-981480523,-309517695+I*-981479087,
     -302385859+I*-976436185,-302385851+I*-976436179,
     -302362345+I*-976415587,-233997953+I*-906679993,
     -233997877+I*-906679911,-233780495+I*-906406387,
     -233779823+I*-906405333,-231935669+I*-902961805,
     -231935667+I*-902961801,-231930383+I*-902948863,
     138588965+I*32996848,151987603+I*74563964,
     151987617+I*74564012,152019865+I*74716898,
     152019923+I*74717454,152068879+I*76463696,
     152068879+I*76463699,152068405+I*76469931,
     149114437+I*95776505,149114433+I*95776527,
     149096755+I*95844131,77989959+I*302308509,
     77989871+I*302308743,77660205+I*303017031,
     77658769+I*303019429,72615867+I*310151265,
     72615861+I*310151273,72595269+I*310174779,2859675+I*378539171,
     2859593+I*378539247,2586069+I*378756629,2585015+I*378757301,
     -858513+I*380601455,-858517+I*380601457,-871455+I*380606741,
     -42438571+I*394005379,-42438619+I*394005393,
     -42591505+I*394037641,-526627057+I*458288427,
     -526627613+I*458288485,-528373855+I*458337441,
     -528373858+I*458337441,-528380090+I*458336967,
     -547686664+I*455382999,-547686686+I*455382995,
     -547754290+I*455365317,-754218668+I*384258521,
     -754218902+I*384258433,-754927190+I*383928767,
     -754929588+I*383927331,-762061424+I*378884429,
     -762061432+I*378884423,-762084938+I*378863831,
     -830449330+I*309128237,-830449406+I*309128155,
     -830666788+I*308854631,-830667460+I*308853577,
     -832511614+I*305410049,-832511616+I*305410045,
     -832516900+I*305397107,-1203036248+I*-630548604,
     -1216434886+I*-672115720,-1216434900+I*-672115768,
     -1216467148+I*-672268654,-1216467206+I*-672269210,
     -1216516162+I*-674015452,-1216516162+I*-674015455,
     -1216515688+I*-674021687,-1213561720+I*-693328261,
     -1213561716+I*-693328283,-1213544038+I*-693395887,
     -1142437242+I*-899860265,-1142437154+I*-899860499,
     -1142107488+I*-900568787,-1142106052+I*-900571185,
     -1137063150+I*-907703021,-1137063144+I*-907703029,
     -1137042552+I*-907726535,-1067306958+I*-976090927,
     -1067306876+I*-976091003,-1067033352+I*-976308385,
     -1067032298+I*-976309057,-1063588770+I*-978153211,
     -1063588766+I*-978153213,-1063575828+I*-978158497,
     -1022008712+I*-991557135,-1022008664+I*-991557149,
     -1021855778+I*-991589397,-537820226+I*-1055840183,
     -537819670+I*-1055840241 ],
   /* k=27 */ [ 659030084+I*-3184783763,659030087+I*-3184783763,
     659036319+I*-3184783289,678342893+I*-3181829321,
     678342915+I*-3181829317,678410519+I*-3181811639,
     884874897+I*-3110704843,884875131+I*-3110704755,
     885583419+I*-3110375089,885585817+I*-3110373653,
     892717653+I*-3105330751,892717661+I*-3105330745,
     892741167+I*-3105310153,961105559+I*-3035574559,
     961105635+I*-3035574477,961323017+I*-3035300953,
     1573860141+I*-2131480635,1573860813+I*-2131479581,
     1575704967+I*-2128036053,1575704969+I*-2128036049,
     1575710253+I*-2128023111,1589108891+I*-2086455995,
     1589108905+I*-2086455947,1589141153+I*-2086303061,
     1653391939+I*-1602267509,1653391997+I*-1602266953,
     1653440953+I*-1600520711,1653440953+I*-1600520708,
     1653440479+I*-1600514476,1650486511+I*-1581207902,
     1650486507+I*-1581207880,1650468829+I*-1581140276,
     1579362033+I*-1374675898,1579361945+I*-1374675664,
     1579032279+I*-1373967376,1579030843+I*-1373964978,
     1573987941+I*-1366833142,1573987935+I*-1366833134,
     1573967343+I*-1366809628,72595269+I*310174779,
     2859675+I*378539171,2859593+I*378539247,2586069+I*378756629,
     2585015+I*378757301,-858513+I*380601455,-858517+I*380601457,
     -871455+I*380606741,-42438571+I*394005379,
     -42438619+I*394005393,-42591505+I*394037641,
     -526627057+I*458288427,-526627613+I*458288485,
     -528373855+I*458337441,-528373858+I*458337441,
     -528380090+I*458336967,-547686664+I*455382999,
     -547686686+I*455382995,-547754290+I*455365317,
     -754218668+I*384258521,-754218902+I*384258433,
     -754927190+I*383928767,-754929588+I*383927331,
     -762061424+I*378884429,-762061432+I*378884423,
     -762084938+I*378863831,-830449330+I*309128237,
     -830449406+I*309128155,-830666788+I*308854631,
     -1443203912+I*-594965687,-1443204584+I*-594966741,
     -1445048738+I*-598410269,-1445048740+I*-598410273,
     -1445054024+I*-598423211,-1458452662+I*-639990327,
     -1458452676+I*-639990375,-1458484924+I*-640143261,
     -1522735710+I*-1124178813,-1522735768+I*-1124179369,
     -1522784724+I*-1125925611,-1522784724+I*-1125925614,
     -1522784250+I*-1125931846,-1519830282+I*-1145238420,
     -1519830278+I*-1145238442,-1519812600+I*-1145306046,
     -1448705804+I*-1351770424,-1448705716+I*-1351770658,
     -1448376050+I*-1352478946,-1448374614+I*-1352481344,
     -1443331712+I*-1359613180,-1443331706+I*-1359613188,
     -1443311114+I*-1359636694,58060960+I*-3036621101,
     127796554+I*-3104985493,127796636+I*-3104985569,
     128070160+I*-3105202951,128071214+I*-3105203623,
     131514742+I*-3107047777,131514746+I*-3107047779,
     131527684+I*-3107053063,173094800+I*-3120451701,
     173094848+I*-3120451715,173247734+I*-3120483963,
     657283286+I*-3184734749,657283842+I*-3184734807 ],
   /* k=28 */ [ 1256581840+I*-4249231046,1256581843+I*-4249231046,
     1256588075+I*-4249230572,1275894649+I*-4246276604,
     1275894671+I*-4246276600,1275962275+I*-4246258922,
     6131303163+I*-2920499181,6337767541+I*-2849392385,
     6337767775+I*-2849392297,6338476063+I*-2849062631,
     6338478461+I*-2849061195,6345610297+I*-2844018293,
     6345610305+I*-2844018287,6345633811+I*-2843997695,
     6413998203+I*-2774262101,6413998279+I*-2774262019,
     6414215661+I*-2773988495,7026752785+I*-1870168177,
     7026753457+I*-1870167123,7028597611+I*-1866723595,
     7028597613+I*-1866723591,7028602897+I*-1866710653,
     7042001535+I*-1825143537,7042001549+I*-1825143489,
     7042033797+I*-1824990603,7106284583+I*-1340955051,
     7106284641+I*-1340954495,7106333597+I*-1339208253,
     7106333597+I*-1339208250,7106333123+I*-1339202018,
     7103379155+I*-1319895444,7103379151+I*-1319895422,
     7103361473+I*-1319827818,7032254677+I*-1113363440,
     7032254589+I*-1113363206,7031924923+I*-1112654918,
     5836821411+I*1016239648,5836819975+I*1016242046,
     5831777073+I*1023373882,5831777067+I*1023373890,
     5831756475+I*1023397396,5762020881+I*1091761788,
     5762020799+I*1091761864,5761747275+I*1091979246,
     4857926957+I*1704516370,4857925903+I*1704517042,
     4854482375+I*1706361196,4854482371+I*1706361198,
     4854469433+I*1706366482,4812902317+I*1719765120,
     4812902269+I*1719765134,4812749383+I*1719797382,
     4328713831+I*1784048168,4328713275+I*1784048226,
     4326967033+I*1784097182,4326967030+I*1784097182,
     4326960798+I*1784096708,4307654224+I*1781142740,
     4307654202+I*1781142736,4307586598+I*1781125058,
     -547754290+I*455365317,-754218668+I*384258521,
     -754218902+I*384258433,-754927190+I*383928767,
     -754929588+I*383927331,-762061424+I*378884429,
     -762061432+I*378884423,-762084938+I*378863831,
     -830449330+I*309128237,-830449406+I*309128155,
     -830666788+I*308854631,-1443203912+I*-594965687,
     -1443204584+I*-594966741,-1445048738+I*-598410269,
     -1445048740+I*-598410273,-1445054024+I*-598423211,
     -1458452662+I*-639990327,-1458452676+I*-639990375,
     -1458484924+I*-640143261,-1522735710+I*-1124178813,
     -1522735768+I*-1124179369,-1522784724+I*-1125925611,
     -1522784724+I*-1125925614,-1522784250+I*-1125931846,
     -1519830282+I*-1145238420,-1519830278+I*-1145238442,
     -1519812600+I*-1145306046,-1448705804+I*-1351770424,
     -1448705716+I*-1351770658,-1448376050+I*-1352478946,
     -253272538+I*-3481373512,-253271102+I*-3481375910,
     -248228200+I*-3488507746,-248228194+I*-3488507754,
     -248207602+I*-3488531260,-178472008+I*-3556895652,
     -178471926+I*-3556895728,-178198402+I*-3557113110,
     725621916+I*-4169650234,725622970+I*-4169650906,
     729066498+I*-4171495060,729066502+I*-4171495062,
     729079440+I*-4171500346,770646556+I*-4184898984,
     770646604+I*-4184898998,770799490+I*-4184931246,
     1254835042+I*-4249182032,1254835598+I*-4249182090 ],
   /* k=29 */ [ 1256581840+I*-4249231046,1256581843+I*-4249231046,
     6709474487+I*-3987918588,6709480719+I*-3987918114,
     6728787293+I*-3984964146,6728787315+I*-3984964142,
     6728854919+I*-3984946464,6935319297+I*-3913839668,
     6935319531+I*-3913839580,6936027819+I*-3913509914,
     9064922385+I*-2718406402,9064924783+I*-2718404966,
     9072056619+I*-2713362064,9072056627+I*-2713362058,
     9072080133+I*-2713341466,9140444525+I*-2643605872,
     9140444601+I*-2643605790,9140661983+I*-2643332266,
     9753199107+I*-1739511948,9753199779+I*-1739510894,
     9755043933+I*-1736067366,9755043935+I*-1736067362,
     9755049219+I*-1736054424,9768447857+I*-1694487308,
     9768447871+I*-1694487260,9768480119+I*-1694334374,
     11972301525+I*9342107143,12036552311+I*9826142695,
     12036552369+I*9826143251,12036601325+I*9827889493,
     12036601325+I*9827889496,12036600851+I*9827895728,
     12033646883+I*9847202302,12033646879+I*9847202324,
     12033629201+I*9847269928,11962522405+I*10053734306,
     11962522317+I*10053734540,11962192651+I*10054442828,
     10767089139+I*12183337394,10767087703+I*12183339792,
     10762044801+I*12190471628,10762044795+I*12190471636,
     10762024203+I*12190495142,10692288609+I*12258859534,
     10692288527+I*12258859610,10692015003+I*12259076992,
     9788194685+I*12871614116,9788193631+I*12871614788,
     9784750103+I*12873458942,9784750099+I*12873458944,
     9784737161+I*12873464228,9743170045+I*12886862866,
     9743169997+I*12886862880,9743017111+I*12886895128,
     9258981559+I*12951145914,9258981003+I*12951145972,
     9257234761+I*12951194928,9257234758+I*12951194928,
     3804342114+I*12689882470,3804335882+I*12689881996,
     3785029308+I*12686928028,3785029286+I*12686928024,
     3784961682+I*12686910346,3578497304+I*12615803550,
     3578497070+I*12615803462,3577788782+I*12615473796,
     1448894216+I*11420370284,1448891818+I*11420368848,
     1441759982+I*11415325946,1441759974+I*11415325940,
     1441736468+I*11415305348,1373372076+I*11345569754,
     1373372000+I*11345569672,1373154618+I*11345296148,
     760617494+I*10441475830,760616822+I*10441474776,
     758772668+I*10438031248,758772666+I*10438031244,
     758767382+I*10438018306,745368744+I*10396451190,
     745368730+I*10396451142,745336482+I*10396298256,
     -1458484924+I*-640143261,-1522735710+I*-1124178813,
     -1522735768+I*-1124179369,-1522784724+I*-1125925611,
     -1522784724+I*-1125925614,-1522784250+I*-1125931846,
     -1519830282+I*-1145238420,-1519830278+I*-1145238442,
     -1519812600+I*-1145306046,-1448705804+I*-1351770424,
     -1448705716+I*-1351770658,-1448376050+I*-1352478946,
     -253272538+I*-3481373512,-253271102+I*-3481375910,
     -248228200+I*-3488507746,-248228194+I*-3488507754,
     -248207602+I*-3488531260,-178472008+I*-3556895652,
     -178471926+I*-3556895728,-178198402+I*-3557113110,
     725621916+I*-4169650234,725622970+I*-4169650906,
     729066498+I*-4171495060,729066502+I*-4171495062,
     729079440+I*-4171500346,770646556+I*-4184898984,
     770646604+I*-4184898998,770799490+I*-4184931246,
     1254835042+I*-4249182032,1254835598+I*-4249182090 ],
   /* k=30 */ [ 1256581840+I*-4249231046,1256581843+I*-4249231046,
     6709474487+I*-3987918588,6709480719+I*-3987918114,
     6728787293+I*-3984964146,6728787315+I*-3984964142,
     6728854919+I*-3984946464,6935319297+I*-3913839668,
     6935319531+I*-3913839580,6936027819+I*-3913509914,
     9064922385+I*-2718406402,9064924783+I*-2718404966,
     9072056619+I*-2713362064,9072056627+I*-2713362058,
     9072080133+I*-2713341466,9140444525+I*-2643605872,
     9140444601+I*-2643605790,9140661983+I*-2643332266,
     9753199107+I*-1739511948,9753199779+I*-1739510894,
     9755043933+I*-1736067366,9755043935+I*-1736067362,
     14685311663+I*9431030384,14685316947+I*9431043322,
     14698715585+I*9472610438,14698715599+I*9472610486,
     14698747847+I*9472763372,14762998633+I*9956798924,
     14762998691+I*9956799480,14763047647+I*9958545722,
     14763047647+I*9958545725,14501735189+I*15411438369,
     14501734715+I*15411444601,14498780747+I*15430751175,
     14498780743+I*15430751197,14498763065+I*15430818801,
     14427656269+I*15637283179,14427656181+I*15637283413,
     14427326515+I*15637991701,13232223003+I*17766886267,
     13232221567+I*17766888665,13227178665+I*17774020501,
     13227178659+I*17774020509,13227158067+I*17774044015,
     13157422473+I*17842408407,13157422391+I*17842408483,
     13157148867+I*17842625865,-6711912761+I*33286710194,
     -7615733079+I*33899247318,-7615734133+I*33899247990,
     -7619177661+I*33901092144,-7619177665+I*33901092146,
     -7619190603+I*33901097430,-7660757719+I*33914496068,
     -7660757767+I*33914496082,-7660910653+I*33914528330,
     -8144946205+I*33978779116,-8144946761+I*33978779174,
     -8146693003+I*33978828130,-8146693006+I*33978828130,
     -13599585650+I*33717515672,-13599591882+I*33717515198,
     -13618898456+I*33714561230,-13618898478+I*33714561226,
     -13618966082+I*33714543548,-13825430460+I*33643436752,
     -13825430694+I*33643436664,-13826138982+I*33643106998,
     -15955033548+I*32448003486,-15955035946+I*32448002050,
     -15962167782+I*32442959148,-15962167790+I*32442959142,
     -15962191296+I*32442938550,-16030555688+I*32373202956,
     -16030555764+I*32373202874,-16030773146+I*32372929350,
     -16643310270+I*31469109032,-16643310942+I*31469107978,
     -16645155096+I*31465664450,-16645155098+I*31465664446,
     -21575422826+I*20298566700,-21575428110+I*20298553762,
     -21588826748+I*20256986646,-21588826762+I*20256986598,
     -21588859010+I*20256833712,-21653109796+I*19772798160,
     -21653109854+I*19772797604,-21653158810+I*19771051362,
     -21653158810+I*19771051359,-21391846352+I*14318158715,
     -21391845878+I*14318152483,-21388891910+I*14298845909,
     -21388891906+I*14298845887,-21388874228+I*14298778283,
     -21317767432+I*14092313905,-21317767344+I*14092313671,
     -21317437678+I*14091605383,-20122334166+I*11962710817,
     -20122332730+I*11962708419,-20117289828+I*11955576583,
     -20117289822+I*11955576575,-20117269230+I*11955553069,
     -20047533636+I*11887188677,-20047533554+I*11887188601,
     -20047260030+I*11886971219,-178198402+I*-3557113110,
     725621916+I*-4169650234,725622970+I*-4169650906,
     729066498+I*-4171495060,729066502+I*-4171495062,
     729079440+I*-4171500346,770646556+I*-4184898984,
     770646604+I*-4184898998,770799490+I*-4184931246,
     1254835042+I*-4249182032,1254835598+I*-4249182090 ],
   /* k=31 */ [ -49500648446+I*-28543269973,
     -49500648443+I*-28543269973,-44047755799+I*-28281957515,
     -44047749567+I*-28281957041,-44028442993+I*-28279003073,
     -44028442971+I*-28279003069,-44028375367+I*-28278985391,
     -43821910989+I*-28207878595,-43821910755+I*-28207878507,
     -43821202467+I*-28207548841,6936027819+I*-3913509914,
     9064922385+I*-2718406402,9064924783+I*-2718404966,
     9072056619+I*-2713362064,9072056627+I*-2713362058,
     9072080133+I*-2713341466,9140444525+I*-2643605872,
     9140444601+I*-2643605790,9140661983+I*-2643332266,
     9753199107+I*-1739511948,9753199779+I*-1739510894,
     9755043933+I*-1736067366,9755043935+I*-1736067362,
     14685311663+I*9431030384,14685316947+I*9431043322,
     14698715585+I*9472610438,14698715599+I*9472610486,
     14698747847+I*9472763372,14762998633+I*9956798924,
     14762998691+I*9956799480,14763047647+I*9958545722,
     14763047647+I*9958545725,14501735189+I*15411438369,
     14501734715+I*15411444601,14498780747+I*15430751175,
     14498780743+I*15430751197,14498763065+I*15430818801,
     14427656269+I*15637283179,14427656181+I*15637283413,
     14427326515+I*15637991701,13232223003+I*17766886267,
     13232221567+I*17766888665,13227178665+I*17774020501,
     13227178659+I*17774020509,-4176749105+I*38801653711,
     -4176769697+I*38801677217,-4246505291+I*38870041609,
     -4246505373+I*38870041685,-4246778897+I*38870259067,
     -5150599215+I*39482796191,-5150600269+I*39482796863,
     -5154043797+I*39484641017,-5154043801+I*39484641019,
     -16321141547+I*44414908747,-16321154485+I*44414914031,
     -16362721601+I*44428312669,-16362721649+I*44428312683,
     -16362874535+I*44428344931,-16846910087+I*44492595717,
     -16846910643+I*44492595775,-16848656885+I*44492644731,
     -16848656888+I*44492644731,-22301549532+I*44231332273,
     -22301555764+I*44231331799,-22320862338+I*44228377831,
     -22320862360+I*44228377827,-22320929964+I*44228360149,
     -22527394342+I*44157253353,-22527394576+I*44157253265,
     -22528102864+I*44156923599,-73285333150+I*19862884672,
     -75414227716+I*18667781160,-75414230114+I*18667779724,
     -75421361950+I*18662736822,-75421361958+I*18662736816,
     -75421385464+I*18662716224,-75489749856+I*18592980630,
     -75489749932+I*18592980548,-75489967314+I*18592707024,
     -76102504438+I*17688886706,-76102505110+I*17688885652,
     -76104349264+I*17685442124,-76104349266+I*17685442120,
     -81034616994+I*6518344374,-81034622278+I*6518331436,
     -81048020916+I*6476764320,-81048020930+I*6476764272,
     -81048053178+I*6476611386,-81112303964+I*5992575834,
     -81112304022+I*5992575278,-81112352978+I*5990829036,
     -81112352978+I*5990829033,-80851040520+I*537936389,
     -80851040046+I*537930157,-80848086078+I*518623583,
     -80848086074+I*518623561,-80848068396+I*518555957,
     -80776961600+I*312091579,-80776961512+I*312091345,
     -80776631846+I*311383057,-79581528334+I*-1817511509,
     -79581526898+I*-1817513907,-79576483996+I*-1824645743,
     -79576483990+I*-1824645751,-62172556226+I*-22852278953,
     -62172535634+I*-22852302459,-62102800040+I*-22920666851,
     -62102799958+I*-22920666927,-62102526434+I*-22920884309,
     -61198706116+I*-23533421433,-61198705062+I*-23533422105,
     -61195261534+I*-23535266259,-61195261530+I*-23535266261,
     -50028163784+I*-28465533989,-50028150846+I*-28465539273,
     -49986583730+I*-28478937911,-49986583682+I*-28478937925,
     -49986430796+I*-28478970173,-49502395244+I*-28543220959,
     -49502394688+I*-28543221017 ],
   /* k=32 */ [ -81399397962+I*-161241880635,
     -81399397959+I*-161241880635,-75946505315+I*-160980568177,
     -75946499083+I*-160980567703,-75927192509+I*-160977613735,
     -75927192487+I*-160977613731,-16467998319+I*-147197391405,
     -16467930715+I*-147197373727,-16261466337+I*-147126266931,
     -16261466103+I*-147126266843,-16260757815+I*-147125937177,
     -14131863249+I*-145930833665,-14131860851+I*-145930832229,
     -14124729015+I*-145925789327,-14124729007+I*-145925789321,
     6902904195+I*-128521861557,6902927701+I*-128521840965,
     6971292093+I*-128452105371,6971292169+I*-128452105289,
     6971509551+I*-128451831765,7584046675+I*-127548011447,
     7584047347+I*-127548010393,7585891501+I*-127544566865,
     7585891503+I*-127544566861,12516159231+I*-116377469115,
     12516164515+I*-116377456177,12529563153+I*-116335889061,
     12529563167+I*-116335889013,12529595415+I*-116335736127,
     12593846201+I*-115851700575,12593846259+I*-115851700019,
     12593895215+I*-115849953777,14763047647+I*9958545722,
     14763047647+I*9958545725,14501735189+I*15411438369,
     14501734715+I*15411444601,14498780747+I*15430751175,
     14498780743+I*15430751197,14498763065+I*15430818801,
     14427656269+I*15637283179,14427656181+I*15637283413,
     14427326515+I*15637991701,13232223003+I*17766886267,
     13232221567+I*17766888665,13227178665+I*17774020501,
     13227178659+I*17774020509,-4176749105+I*38801653711,
     -4176769697+I*38801677217,-4246505291+I*38870041609,
     -4246505373+I*38870041685,-4246778897+I*38870259067,
     -5150599215+I*39482796191,-5150600269+I*39482796863,
     -5154043797+I*39484641017,-5154043801+I*39484641019,
     -16321141547+I*44414908747,-16321154485+I*44414914031,
     -16362721601+I*44428312669,-16362721649+I*44428312683,
     -16362874535+I*44428344931,-16846910087+I*44492595717,
     -16846910643+I*44492595775,-16848656885+I*44492644731,
     -16848656888+I*44492644731,-22301549532+I*44231332273,
     -22301555764+I*44231331799,-22320862338+I*44228377831,
     -22320862360+I*44228377827,-81780056528+I*30448155501,
     -81780124132+I*30448137823,-81986588510+I*30377031027,
     -81986588744+I*30377030939,-81987297032+I*30376701273,
     -84116191598+I*29181597761,-84116193996+I*29181596325,
     -84123325832+I*29176553423,-84123325840+I*29176553417,
     -105150959042+I*11772625653,-105150982548+I*11772605061,
     -105219346940+I*11702869467,-105219347016+I*11702869385,
     -105219564398+I*11702595861,-105832101522+I*10798775543,
     -105832102194+I*10798774489,-105833946348+I*10795330961,
     -105833946350+I*10795330957,-110764214078+I*-371766789,
     -110764219362+I*-371779727,-110777618000+I*-413346843,
     -110777618014+I*-413346891,-110777650262+I*-413499777,
     -110841901048+I*-897535329,-110841901106+I*-897535885,
     -110841950062+I*-899282127,-113011102494+I*-126707781626,
     -113011102494+I*-126707781629,-112749790036+I*-132160674273,
     -112749789562+I*-132160680505,-112746835594+I*-132179987079,
     -112746835590+I*-132179987101,-112746817912+I*-132180054705,
     -112675711116+I*-132386519083,-112675711028+I*-132386519317,
     -112675381362+I*-132387227605,-111480277850+I*-134516122171,
     -111480276414+I*-134516124569,-111475233512+I*-134523256405,
     -111475233506+I*-134523256413,-94071305742+I*-155550889615,
     -94071285150+I*-155550913121,-94001549556+I*-155619277513,
     -94001549474+I*-155619277589,-94001275950+I*-155619494971,
     -93097455632+I*-156232032095,-93097454578+I*-156232032767,
     -93094011050+I*-156233876921,-93094011046+I*-156233876923,
     -81926913300+I*-161164144651,-81926900362+I*-161164149935,
     -81885333246+I*-161177548573,-81885333198+I*-161177548587,
     -81885180312+I*-161177580835,-81401144760+I*-161241831621,
     -81401144204+I*-161241831679 ],
   /* k=33 */ [ 152099073846+I*-357737990329,
     152099073849+I*-357737990329,157551966493+I*-357476677871,
     157551972725+I*-357476677397,157571279299+I*-357473723429,
     157571279321+I*-357473723425,217030473489+I*-343693501099,
     217030541093+I*-343693483421,217237005471+I*-343622376625,
     217237005705+I*-343622376537,217237713993+I*-343622046871,
     219366608559+I*-342426943359,219366610957+I*-342426941923,
     219373742793+I*-342421899021,219373742801+I*-342421899015,
     240401376003+I*-325017971251,240401399509+I*-325017950659,
     240469763901+I*-324948215065,240469763977+I*-324948214983,
     240469981359+I*-324947941459,241082518483+I*-324044121141,
     241082519155+I*-324044120087,241084363309+I*-324040676559,
     241084363311+I*-324040676555,246014631039+I*-312873578809,
     246014636323+I*-312873565871,246028034961+I*-312831998755,
     246028034975+I*-312831998707,277926784491+I*-180133388045,
     277926816739+I*-180133235159,277991067525+I*-179649199607,
     277991067583+I*-179649199051,277991116539+I*-179647452809,
     277991116539+I*-179647452806,277729804081+I*-174194560162,
     277729803607+I*-174194553930,277726849639+I*-174175247356,
     277726849635+I*-174175247334,263946627309+I*-114716053166,
     263946609631+I*-114715985562,263875502835+I*-114509521184,
     263875502747+I*-114509520950,263875173081+I*-114508812662,
     262680069569+I*-112379918096,262680068133+I*-112379915698,
     262675025231+I*-112372783862,262675025225+I*-112372783854,
     245271097461+I*-91345150652,245271076869+I*-91345127146,
     245201341275+I*-91276762754,245201341193+I*-91276762678,
     245201067669+I*-91276545296,244297247351+I*-90664008172,
     244297246297+I*-90664007500,244293802769+I*-90662163346,
     -5154043797+I*39484641017,-5154043801+I*39484641019,
     -16321141547+I*44414908747,-16321154485+I*44414914031,
     -16362721601+I*44428312669,-16362721649+I*44428312683,
     -16362874535+I*44428344931,-16846910087+I*44492595717,
     -16846910643+I*44492595775,-16848656885+I*44492644731,
     -16848656888+I*44492644731,-22301549532+I*44231332273,
     -22301555764+I*44231331799,-22320862338+I*44228377831,
     -22320862360+I*44228377827,-81780056528+I*30448155501,
     -81780124132+I*30448137823,-81986588510+I*30377031027,
     -81986588744+I*30377030939,-81987297032+I*30376701273,
     -84116191598+I*29181597761,-84116193996+I*29181596325,
     -84123325832+I*29176553423,-84123325840+I*29176553417,
     -105150959042+I*11772625653,-105150982548+I*11772605061,
     -105219346940+I*11702869467,-105219347016+I*11702869385,
     -105219564398+I*11702595861,-105832101522+I*10798775543,
     -105832102194+I*10798774489,-105833946348+I*10795330961,
     -105833946350+I*10795330957,-110764214078+I*-371766789,
     -110764219362+I*-371779727,-110777618000+I*-413346843,
     -110777618014+I*-413346891,-142676367530+I*-133111957553,
     -142676399778+I*-133112110439,-142740650564+I*-133596145991,
     -142740650622+I*-133596146547,-142740699578+I*-133597892789,
     -142740699578+I*-133597892792,-142479387120+I*-139050785436,
     -142479386646+I*-139050791668,-142476432678+I*-139070098242,
     -142476432674+I*-139070098264,-128696210348+I*-198529292432,
     -128696192670+I*-198529360036,-128625085874+I*-198735824414,
     -128625085786+I*-198735824648,-128624756120+I*-198736532936,
     -127429652608+I*-200865427502,-127429651172+I*-200865429900,
     -127424608270+I*-200872561736,-127424608264+I*-200872561744,
     -110020680500+I*-221900194946,-110020659908+I*-221900218452,
     -109950924314+I*-221968582844,-109950924232+I*-221968582920,
     -109950650708+I*-221968800302,-109046830390+I*-222581337426,
     -109046829336+I*-222581338098,-109043385808+I*-222583182252,
     140404460758+I*-352729986615,140404460762+I*-352729986617,
     151571558508+I*-357660254345,151571571446+I*-357660259629,
     151613138562+I*-357673658267,151613138610+I*-357673658281,
     151613291496+I*-357673690529,152097327048+I*-357737941315,
     152097327604+I*-357737941373 ],
   /* k=34 */ [ 268848309750+I*-455986045176,
     268848309753+I*-455986045176,274301202397+I*-455724732718,
     274301208629+I*-455724732244,274320515203+I*-455721778276,
     274320515225+I*-455721778272,333779709393+I*-441941555946,
     333779776997+I*-441941538268,333986241375+I*-441870431472,
     333986241609+I*-441870431384,333986949897+I*-441870101718,
     336115844463+I*-440674998206,336115846861+I*-440674996770,
     336122978697+I*-440669953868,845864433989+I*-71921065099,
     845864433997+I*-71921065093,866892067199+I*-54517137329,
     866892090705+I*-54517116737,866960455097+I*-54447381143,
     866960455173+I*-54447381061,866960672555+I*-54447107537,
     867573209679+I*-53543287219,867573210351+I*-53543286165,
     867575054505+I*-53539842637,867575054507+I*-53539842633,
     872505322235+I*-42372744887,872505327519+I*-42372731949,
     872518726157+I*-42331164833,872518726171+I*-42331164785,
     904417475687+I*90367445877,904417507935+I*90367598763,
     904481758721+I*90851634315,904481758779+I*90851634871,
     904481807735+I*90853381113,904481807735+I*90853381116,
     904220495277+I*96306273760,904220494803+I*96306279992,
     904217540835+I*96325586566,904217540831+I*96325586588,
     890437318505+I*155784780756,890437300827+I*155784848360,
     890366194031+I*155991312738,890366193943+I*155991312972,
     890365864277+I*155992021260,889170760765+I*158120915826,
     889170759329+I*158120918224,889165716427+I*158128050060,
     889165716421+I*158128050068,871761788657+I*179155683270,
     871761768065+I*179155706776,871692032471+I*179224071168,
     871692032389+I*179224071244,638193560581+I*375720180938,
     638193287057+I*375720398320,637289466739+I*376332935444,
     637289465685+I*376332936116,637286022157+I*376334780270,
     637286022153+I*376334780272,626118924407+I*381265048000,
     626118911469+I*381265053284,626077344353+I*381278451922,
     626077344305+I*381278451936,493378733643+I*413177201452,
     493378580757+I*413177233700,492894545205+I*413241484486,
     492894544649+I*413241484544,492892798407+I*413241533500,
     492892798404+I*413241533500,487439905760+I*412980221042,
     487439899528+I*412980220568,487420592954+I*412977266600,
     487420592932+I*412977266596,427961398764+I*399197044270,
     427961331160+I*399197026592,427754866782+I*399125919796,
     427754866548+I*399125919708,427754158260+I*399125590042,
     425625263694+I*397930486530,425625261296+I*397930485094,
     425618129460+I*397925442192,-84123325832+I*29176553423,
     -84123325840+I*29176553417,-105150959042+I*11772625653,
     -105150982548+I*11772605061,-105219346940+I*11702869467,
     -105219347016+I*11702869385,-105219564398+I*11702595861,
     -105832101522+I*10798775543,-105832102194+I*10798774489,
     -105833946348+I*10795330961,-105833946350+I*10795330957,
     -110764214078+I*-371766789,-110764219362+I*-371779727,
     -110777618000+I*-413346843,-110777618014+I*-413346891,
     -142676367530+I*-133111957553,-142676399778+I*-133112110439,
     -142740650564+I*-133596145991,-142740650622+I*-133596146547,
     -142740699578+I*-133597892789,-142740699578+I*-133597892792,
     -142479387120+I*-139050785436,-142479386646+I*-139050791668,
     -142476432678+I*-139070098242,-142476432674+I*-139070098264,
     -128696210348+I*-198529292432,-128696192670+I*-198529360036,
     -128625085874+I*-198735824414,-128625085786+I*-198735824648,
     -128624756120+I*-198736532936,-127429652608+I*-200865427502,
     -127429651172+I*-200865429900,-127424608270+I*-200872561736,
     -127424608264+I*-200872561744,-110020680500+I*-221900194946,
     -110020659908+I*-221900218452,-109950924314+I*-221968582844,
     -109950924232+I*-221968582920,123547547576+I*-418464692614,
     123547821100+I*-418464909996,124451641418+I*-419077447120,
     124451642472+I*-419077447792,124455086000+I*-419079291946,
     124455086004+I*-419079291948,135622183750+I*-424009559676,
     135622196688+I*-424009564960,135663763804+I*-424022963598,
     135663763852+I*-424022963612,268362374514+I*-455921713128,
     268362527400+I*-455921745376,268846562952+I*-455985996162,
     268846563508+I*-455985996220 ],
   /* k=35 */ [ 268848309750+I*-455986045176,
     268848309753+I*-455986045176,274301202397+I*-455724732718,
     274301208629+I*-455724732244,274320515203+I*-455721778276,
     274320515225+I*-455721778272,333779709393+I*-441941555946,
     333779776997+I*-441941538268,333986241375+I*-441870431472,
     333986241609+I*-441870431384,960476932805+I*-171369597462,
     960477641093+I*-171369267796,962606535659+I*-170174164284,
     962606538057+I*-170174162848,962613669893+I*-170169119946,
     962613669901+I*-170169119940,983641303103+I*-152765192176,
     983641326609+I*-152765171584,983709691001+I*-152695435990,
     983709691077+I*-152695435908,1180205800771+I*80803035900,
     1180206018153+I*80803309424,1180818555277+I*81707129742,
     1180818555949+I*81707130796,1180820400103+I*81710574324,
     1180820400105+I*81710574328,1185750667833+I*92877672074,
     1185750673117+I*92877685012,1185764071755+I*92919252128,
     1185764071769+I*92919252176,1217662821285+I*225617862838,
     1217662853533+I*225618015724,1217727104319+I*226102051276,
     1217727104377+I*226102051832,1217727153333+I*226103798074,
     1217727153333+I*226103798077,1217465840875+I*231556690721,
     1217465840401+I*231556696953,1217462886433+I*231576003527,
     989706564187+I*1619807802880,989706564183+I*1619807802902,
     975926341857+I*1679266997070,975926324179+I*1679267064674,
     975855217383+I*1679473529052,975855217295+I*1679473529286,
     975854887629+I*1679474237574,974659784117+I*1681603132140,
     974659782681+I*1681603134538,974654739779+I*1681610266374,
     974654739773+I*1681610266382,957250812009+I*1702637899584,
     957250791417+I*1702637923090,957181055823+I*1702706287482,
     957181055741+I*1702706287558,723682583933+I*1899202397252,
     723682310409+I*1899202614634,722778490091+I*1899815151758,
     722778489037+I*1899815152430,722775045509+I*1899816996584,
     722775045505+I*1899816996586,711607947759+I*1904747264314,
     711607934821+I*1904747269598,711566367705+I*1904760668236,
     711566367657+I*1904760668250,578867756995+I*1936659417766,
     578867604109+I*1936659450014,578383568557+I*1936723700800,
     578383568001+I*1936723700858,578381821759+I*1936723749814,
     578381821756+I*1936723749814,572928929112+I*1936462437356,
     572928922880+I*1936462436882,572909616306+I*1936459482914,
     572909616284+I*1936459482910,513450422116+I*1922679260584,
     513450354512+I*1922679242906,513243890134+I*1922608136110,
     513243889900+I*1922608136022,-113246801296+I*1652107302100,
     -113247509584+I*1652106972434,-115376404150+I*1650911868922,
     -115376406548+I*1650911867486,-115383538384+I*1650906824584,
     -115383538392+I*1650906824578,-136411171594+I*1633502896814,
     -136411195100+I*1633502876222,-136479559492+I*1633433140628,
     -136479559568+I*1633433140546,-332975669262+I*1399934668738,
     -332975886644+I*1399934395214,-333588423768+I*1399030574896,
     -333588424440+I*1399030573842,-333590268594+I*1399027130314,
     -333590268596+I*1399027130310,-338520536324+I*1387860032564,
     -338520541608+I*1387860019626,-338533940246+I*1387818452510,
     -338533940260+I*1387818452462,-370432689776+I*1255119841800,
     -370432722024+I*1255119688914,-370496972810+I*1254635653362,
     -370496972868+I*1254635652806,-370497021824+I*1254633906564,
     -370497021824+I*1254633906561,-370235709366+I*1249181013917,
     -370235708892+I*1249181007685,-370232754924+I*1249161701111,
     -142476432678+I*-139070098242,-142476432674+I*-139070098264,
     -128696210348+I*-198529292432,-128696192670+I*-198529360036,
     -128625085874+I*-198735824414,-128625085786+I*-198735824648,
     -128624756120+I*-198736532936,-127429652608+I*-200865427502,
     -127429651172+I*-200865429900,-127424608270+I*-200872561736,
     -127424608264+I*-200872561744,-110020680500+I*-221900194946,
     -110020659908+I*-221900218452,-109950924314+I*-221968582844,
     -109950924232+I*-221968582920,123547547576+I*-418464692614,
     123547821100+I*-418464909996,124451641418+I*-419077447120,
     124451642472+I*-419077447792,124455086000+I*-419079291946,
     124455086004+I*-419079291948,135622183750+I*-424009559676,
     135622196688+I*-424009564960,135663763804+I*-424022963598,
     135663763852+I*-424022963612,268362374514+I*-455921713128,
     268362527400+I*-455921745376,268846562952+I*-455985996162,
     268846563508+I*-455985996220 ],
   /* k=36 */ [ 268848309750+I*-455986045176,
     268848309753+I*-455986045176,274301202397+I*-455724732718,
     274301208629+I*-455724732244,274320515203+I*-455721778276,
     274320515225+I*-455721778272,333779709393+I*-441941555946,
     333779776997+I*-441941538268,333986241375+I*-441870431472,
     333986241609+I*-441870431384,960476932805+I*-171369597462,
     960477641093+I*-171369267796,962606535659+I*-170174164284,
     962606538057+I*-170174162848,962613669893+I*-170169119946,
     962613669901+I*-170169119940,983641303103+I*-152765192176,
     983641326609+I*-152765171584,983709691001+I*-152695435990,
     983709691077+I*-152695435908,1180205800771+I*80803035900,
     1180206018153+I*80803309424,1180818555277+I*81707129742,
     1180818555949+I*81707130796,1180820400103+I*81710574324,
     1180820400105+I*81710574328,1185750667833+I*92877672074,
     1185750673117+I*92877685012,1185764071755+I*92919252128,
     1185764071769+I*92919252176,1217662821285+I*225617862838,
     1217662853533+I*225618015724,1217727104319+I*226102051276,
     1217727104377+I*226102051832,1303216127729+I*1749584268146,
     1303216176685+I*1749586014388,1303216176685+I*1749586014391,
     1302954864227+I*1755038907035,1302954863753+I*1755038913267,
     1302951909785+I*1755058219841,1302951909781+I*1755058219863,
     1289171687455+I*1814517414031,1289171669777+I*1814517481635,
     1289100562981+I*1814723946013,1289100562893+I*1814723946247,
     1018599728971+I*2441214637443,1018599399305+I*2441215345731,
     1017404295793+I*2443344240297,1017404294357+I*2443344242695,
     1017399251455+I*2443351374531,1017399251449+I*2443351374539,
     999995323685+I*2464379007741,999995303093+I*2464379031247,
     999925567499+I*2464447395639,999925567417+I*2464447395715,
     766427095609+I*2660943505409,766426822085+I*2660943722791,
     765523001767+I*2661556259915,765523000713+I*2661556260587,
     765519557185+I*2661558104741,765519557181+I*2661558104743,
     754352459435+I*2666488372471,754352446497+I*2666488377755,
     754310879381+I*2666501776393,-2249909041571+I*3599220931254,
     -2249909041619+I*3599220931268,-2382607652281+I*3631119680784,
     -2382607805167+I*3631119713032,-2383091840719+I*3631183963818,
     -2383091841275+I*3631183963876,-2383093587517+I*3631184012832,
     -2383093587520+I*3631184012832,-2388546480164+I*3630922700374,
     -2388546486396+I*3630922699900,-2388565792970+I*3630919745932,
     -2388565792992+I*3630919745928,-2448024987160+I*3617139523602,
     -2448025054764+I*3617139505924,-2448231519142+I*3617068399128,
     -2448231519376+I*3617068399040,-3074722210572+I*3346567565118,
     -3074722918860+I*3346567235452,-3076851813426+I*3345372131940,
     -3076851815824+I*3345372130504,-3076858947660+I*3345367087602,
     -3076858947668+I*3345367087596,-3097886580870+I*3327963159832,
     -3097886604376+I*3327963139240,-3097954968768+I*3327893403646,
     -3097954968844+I*3327893403564,-3294451078538+I*3094394931756,
     -3294451295920+I*3094394658232,-3295063833044+I*3093490837914,
     -3295063833716+I*3093490836860,-3295065677870+I*3093487393332,
     -3295065677872+I*3093487393328,-3299995945600+I*3082320295582,
     -3299995950884+I*3082320282644,-3300009349522+I*3082278715528,
     -3300009349536+I*3082278715480,-3331908099052+I*2949580104818,
     -3331908131300+I*2949579951932,-3331972382086+I*2949095916380,
     -3331972382144+I*2949095915824,-3417461405496+I*1425613699510,
     -3417461454452+I*1425611953268,-3417461454452+I*1425611953265,
     -3417200141994+I*1420159060621,-3417200141520+I*1420159054389,
     -3417197187552+I*1420139747815,-3417197187548+I*1420139747793,
     -3403416965222+I*1360680553625,-3403416947544+I*1360680486021,
     -3403345840748+I*1360474021643,-3403345840660+I*1360474021409,
     -3132845006738+I*733983330213,-3132844677072+I*733982621925,
     -3131649573560+I*731853727359,-3131649572124+I*731853724961,
     -3131644529222+I*731846593125,-3131644529216+I*731846593117,
     -3114240601452+I*710818959915,-3114240580860+I*710818936409,
     -3114170845266+I*710750572017,-3114170845184+I*710750571941,
     -2880672373376+I*514254462247,-2880672099852+I*514254244865,
     -2879768279534+I*513641707741,-2879768278480+I*513641707069,
     -2879764834952+I*513639862915,-2879764834948+I*513639862913,
     -2868597737202+I*508709595185,-2868597724264+I*508709589901,
     -2868556157148+I*508696191263,135663763804+I*-424022963598,
     135663763852+I*-424022963612,268362374514+I*-455921713128,
     268362527400+I*-455921745376,268846562952+I*-455985996162,
     268846563508+I*-455985996220 ],
   /* k=37 */ [ -4600809920924+I*-5531706732219,
     -4600809920921+I*-5531706732219,
     -4595357028277+I*-5531445419761,
     -4595357022045+I*-5531445419287,
     -4595337715471+I*-5531442465319,
     -4595337715449+I*-5531442465315,
     -4535878521281+I*-5517662242989,
     -4535878453677+I*-5517662225311,
     -4535671989299+I*-5517591118515,
     -4535671989065+I*-5517591118427,
     -3909181297869+I*-5247090284505,
     -3909180589581+I*-5247089954839,
     -3907051695015+I*-5245894851327,
     -3907051692617+I*-5245894849891,
     -3907044560781+I*-5245889806989,
     -3907044560773+I*-5245889806983,
     -3886016927571+I*-5228485879219,
     -3886016904065+I*-5228485858627,
     -3885948539673+I*-5228416123033,983709691001+I*-152695435990,
     983709691077+I*-152695435908,1180205800771+I*80803035900,
     1180206018153+I*80803309424,1180818555277+I*81707129742,
     1180818555949+I*81707130796,1180820400103+I*81710574324,
     1180820400105+I*81710574328,1185750667833+I*92877672074,
     1185750673117+I*92877685012,1185764071755+I*92919252128,
     1185764071769+I*92919252176,1217662821285+I*225617862838,
     1217662853533+I*225618015724,1217727104319+I*226102051276,
     1217727104377+I*226102051832,1303216127729+I*1749584268146,
     1303216176685+I*1749586014388,1303216176685+I*1749586014391,
     1302954864227+I*1755038907035,1302954863753+I*1755038913267,
     1302951909785+I*1755058219841,1302951909781+I*1755058219863,
     1289171687455+I*1814517414031,1289171669777+I*1814517481635,
     1289100562981+I*1814723946013,1289100562893+I*1814723946247,
     1018599728971+I*2441214637443,1018599399305+I*2441215345731,
     1017404295793+I*2443344240297,1017404294357+I*2443344242695,
     1017399251455+I*2443351374531,1017399251449+I*2443351374539,
     999995323685+I*2464379007741,999995303093+I*2464379031247,
     999925567499+I*2464447395639,999925567417+I*2464447395715,
     766427095609+I*2660943505409,766426822085+I*2660943722791,
     765523001767+I*2661556259915,765523000713+I*2661556260587,
     -2195952408563+I*4356016523605,-2195955852091+I*4356018367759,
     -2195955852095+I*4356018367761,-2207122949841+I*4360948635489,
     -2207122962779+I*4360948640773,-2207164529895+I*4360962039411,
     -2207164529943+I*4360962039425,-2339863140605+I*4392860788941,
     -2339863293491+I*4392860821189,-2340347329043+I*4392925071975,
     -2340347329599+I*4392925072033,-3863829545913+I*4478414095385,
     -3863831292155+I*4478414144341,-3863831292158+I*4478414144341,
     -3869284184802+I*4478152831883,-3869284191034+I*4478152831409,
     -3869303497608+I*4478149877441,-3869303497630+I*4478149877437,
     -3928762691798+I*4464369655111,-3928762759402+I*4464369637433,
     -3928969223780+I*4464298530637,-3928969224014+I*4464298530549,
     -4555459915210+I*4193797696627,-4555460623498+I*4193797366961,
     -4557589518064+I*4192602263449,-4557589520462+I*4192602262013,
     -4557596652298+I*4192597219111,-4557596652306+I*4192597219105,
     -4578624285508+I*4175193291341,-4578624309014+I*4175193270749,
     -4578692673406+I*4175123535155,-9448350904080+I*-900597151888,
     -9448350904156+I*-900597151970,
     -9644847013850+I*-1134095623778,
     -9644847231232+I*-1134095897302,
     -9645459768356+I*-1134999717620,
     -9645459769028+I*-1134999718674,
     -9645461613182+I*-1135003162202,
     -9645461613184+I*-1135003162206,
     -9650391880912+I*-1146170259952,
     -9650391886196+I*-1146170272890,
     -9650405284834+I*-1146211840006,
     -9650405284848+I*-1146211840054,
     -9682304034364+I*-1278910450716,
     -9682304066612+I*-1278910603602,
     -9682368317398+I*-1279394639154,
     -9682368317456+I*-1279394639710,
     -9767857340808+I*-2802876856024,
     -9767857389764+I*-2802878602266,
     -9767857389764+I*-2802878602269,
     -9767596077306+I*-2808331494913,
     -9767596076832+I*-2808331501145,
     -9767593122864+I*-2808350807719,
     -9767593122860+I*-2808350807741,
     -9753812900534+I*-2867810001909,
     -9753812882856+I*-2867810069513,
     -9753741776060+I*-2868016533891,
     -9753741775972+I*-2868016534125,
     -9483240942050+I*-3494507225321,
     -9483240612384+I*-3494507933609,
     -9482045508872+I*-3496636828175,
     -9482045507436+I*-3496636830573,
     -9482040464534+I*-3496643962409,
     -9482040464528+I*-3496643962417,
     -9464636536764+I*-3517671595619,
     -9464636516172+I*-3517671619125,
     -9464566780578+I*-3517739983517,
     -9464566780496+I*-3517739983593,
     -9231068308688+I*-3714236093287,
     -9231068035164+I*-3714236310669,
     -9230164214846+I*-3714848847793,
     -9230164213792+I*-3714848848465,
     -6268688804516+I*-5409309111483,
     -6268685360988+I*-5409310955637,
     -6268685360984+I*-5409310955639,
     -6257518263238+I*-5414241223367,
     -6257518250300+I*-5414241228651,
     -6257476683184+I*-5414254627289,
     -6257476683136+I*-5414254627303,
     -6124778072474+I*-5446153376819,
     -6124777919588+I*-5446153409067,
     -6124293884036+I*-5446217659853,
     -6124293883480+I*-5446217659911,
     -4600811667166+I*-5531706683263 ],
   /* k=38 */ [ -2494224745168+I*-22460989158377,
     -2494224745165+I*-22460989158377,
     -2488771852521+I*-22460727845919,
     -2488771846289+I*-22460727845445,
     -2488752539715+I*-22460724891477,
     -2488752539693+I*-22460724891473,
     -2429293345525+I*-22446944669147,
     -2429293277921+I*-22446944651469,
     -2429086813543+I*-22446873544673,
     -2429086813309+I*-22446873544585,
     -1802596122113+I*-22176372710663,
     -1802595413825+I*-22176372380997,
     -1800466519259+I*-22175177277485,
     -1800466516861+I*-22175177276049,
     4549929418451+I*-17946686720515,
     4549936550287+I*-17946681677613,
     4549936550295+I*-17946681677607,
     4570964183497+I*-17929277749843,
     4570964207003+I*-17929277729251,
     4571032571395+I*-17929207993657,
     4571032571471+I*-17929207993575,
     4767528681165+I*-17695709521767,
     4767528898547+I*-17695709248243,
     4768141435671+I*-17694805427925,
     4768141436343+I*-17694805426871,
     6462601699361+I*-14733330017595,
     6462603543515+I*-14733326574067,
     6462603543517+I*-14733326574063,
     6467533811245+I*-14722159476317,
     6467533816529+I*-14722159463379,
     6467547215167+I*-14722117896263,
     6467547215181+I*-14722117896215,
     6499445964697+I*-14589419285553,
     6499445996945+I*-14589419132667,
     6499510247731+I*-14588935097115,
     6499510247789+I*-14588935096559,
     6584999271141+I*-13065452880245,
     6584999320097+I*-13065451134003,
     6584999320097+I*-13065451134000,
     6584738007639+I*-13059998241356,
     6584738007165+I*-13059998235124,
     6584735053197+I*-13059978928550,
     6584735053193+I*-13059978928528,
     6570954830867+I*-13000519734360,
     6570954813189+I*-13000519666756,
     6570883706393+I*-13000313202378,1289100562981+I*1814723946013,
     1289100562893+I*1814723946247,1018599728971+I*2441214637443,
     1018599399305+I*2441215345731,1017404295793+I*2443344240297,
     1017404294357+I*2443344242695,1017399251455+I*2443351374531,
     1017399251449+I*2443351374539,999995323685+I*2464379007741,
     999995303093+I*2464379031247,999925567499+I*2464447395639,
     999925567417+I*2464447395715,766427095609+I*2660943505409,
     766426822085+I*2660943722791,765523001767+I*2661556259915,
     765523000713+I*2661556260587,-2195952408563+I*4356016523605,
     -2195955852091+I*4356018367759,-2195955852095+I*4356018367761,
     -2207122949841+I*4360948635489,-2207122962779+I*4360948640773,
     -2207164529895+I*4360962039411,-2207164529943+I*4360962039425,
     -2339863140605+I*4392860788941,-2339863293491+I*4392860821189,
     -2340347329043+I*4392925071975,-2340347329599+I*4392925072033,
     -3863829545913+I*4478414095385,-3863831292155+I*4478414144341,
     -3863831292158+I*4478414144341,-3869284184802+I*4478152831883,
     -3869284191034+I*4478152831409,-3869303497608+I*4478149877441,
     -3869303497630+I*4478149877437,-3928762691798+I*4464369655111,
     -3928762759402+I*4464369637433,-3928969223780+I*4464298530637,
     -3928969224014+I*4464298530549,-4555459915210+I*4193797696627,
     -4555460623498+I*4193797366961,-4557589518064+I*4192602263449,
     -4557589520462+I*4192602262013,-10907985455774+I*-35888293521,
     -10907992587610+I*-35893336423,-10907992587618+I*-35893336429,
     -10929020220820+I*-53297264193,-10929020244326+I*-53297284785,
     -10929088608718+I*-53367020379,-10929088608794+I*-53367020461,
     -11125584718488+I*-286865492269,
     -11125584935870+I*-286865765793,
     -11126197472994+I*-287769586111,
     -11126197473666+I*-287769587165,
     -12820657736684+I*-3249244996441,
     -12820659580838+I*-3249248439969,
     -12820659580840+I*-3249248439973,
     -12825589848568+I*-3260415537719,
     -12825589853852+I*-3260415550657,
     -12825603252490+I*-3260457117773,
     -12825603252504+I*-3260457117821,
     -12857502002020+I*-3393155728483,
     -12857502034268+I*-3393155881369,
     -12857566285054+I*-3393639916921,
     -12857566285112+I*-3393639917477,
     -12943055308464+I*-4917122133791,
     -12943055357420+I*-4917123880033,
     -12943055357420+I*-4917123880036,
     -12942794044962+I*-4922576772680,
     -12942794044488+I*-4922576778912,
     -12942791090520+I*-4922596085486,
     -12942791090516+I*-4922596085508,
     -12929010868190+I*-4982055279676,
     -12929010850512+I*-4982055347280,
     -12928939743716+I*-4982261811658,
     -7647156600304+I*-19797298960049,
     -7647156600216+I*-19797298960283,
     -7376655766294+I*-20423789651479,
     -7376655436628+I*-20423790359767,
     -7375460333116+I*-20425919254333,
     -7375460331680+I*-20425919256731,
     -7375455288778+I*-20425926388567,
     -7375455288772+I*-20425926388575,
     -7358051361008+I*-20446954021777,
     -7358051340416+I*-20446954045283,
     -7357981604822+I*-20447022409675,
     -7357981604740+I*-20447022409751,
     -7124483132932+I*-20643518519445,
     -7124482859408+I*-20643518736827,
     -7123579039090+I*-20644131273951,
     -7123579038036+I*-20644131274623,
     -4162103628760+I*-22338591537641,
     -4162100185232+I*-22338593381795,
     -4162100185228+I*-22338593381797,
     -4150933087482+I*-22343523649525,
     -4150933074544+I*-22343523654809,
     -4150891507428+I*-22343537053447,
     -4150891507380+I*-22343537053461,
     -4018192896718+I*-22375435802977,
     -4018192743832+I*-22375435835225,
     -4017708708280+I*-22375500086011,
     -4017708707724+I*-22375500086069,
     -2494226491410+I*-22460989109421 ],
   /* k=39 */ [ 33470925282904+I*-35177101233023,
     33470925282907+I*-35177101233023,
     33476378175551+I*-35176839920565,
     33476378181783+I*-35176839920091,
     33476397488357+I*-35176836966123,
     33476397488379+I*-35176836966119,
     33535856682547+I*-35163056743793,
     33535856750151+I*-35163056726115,
     33536063214529+I*-35162985619319,
     33536063214763+I*-35162985619231,
     34162553905959+I*-34892484785309,
     34162554614247+I*-34892484455643,
     34164683508813+I*-34891289352131,
     34164683511211+I*-34891289350695,
     40515079446523+I*-30662798795161,
     40515086578359+I*-30662793752259,
     40515086578367+I*-30662793752253,
     40536114211569+I*-30645389824489,
     40536114235075+I*-30645389803897,
     40536182599467+I*-30645320068303,
     40536182599543+I*-30645320068221,
     40732678709237+I*-30411821596413,
     40732678926619+I*-30411821322889,
     40733291463743+I*-30410917502571,
     40733291464415+I*-30410917501517,
     42427751727433+I*-27449442092241,
     42427753571587+I*-27449438648713,
     42427753571589+I*-27449438648709,
     42432683839317+I*-27438271550963,
     42432683844601+I*-27438271538025,
     42432697243239+I*-27438229970909,
     42432697243253+I*-27438229970861,
     42464595992769+I*-27305531360199,
     42464596025017+I*-27305531207313,
     42464660275803+I*-27305047171761,
     42464660275861+I*-27305047171205,
     42550149299213+I*-25781564954891,
     42550149348169+I*-25781563208649,
     42550149348169+I*-25781563208646,
     42549888035711+I*-25776110316002,
     42549888035237+I*-25776110309770,
     40443302859481+I*-8846827883612,
     40443299905513+I*-8846808577038,
     40443299905509+I*-8846808577016,
     40429519683183+I*-8787349382848,
     40429519665505+I*-8787349315244,
     40429448558709+I*-8787142850866,
     40429448558621+I*-8787142850632,
     40158947724699+I*-8160652159436,
     40158947395033+I*-8160651451148,
     40157752291521+I*-8158522556582,
     40157752290085+I*-8158522554184,
     35929261734551+I*-1808126618872,
     35929256691649+I*-1808119487036,
     35929256691643+I*-1808119487028,
     35911852763879+I*-1787091853826,
     35911852743287+I*-1787091830320,
     35911783007693+I*-1787023465928,
     35911783007611+I*-1787023465852,
     35678284535803+I*-1590527356158,
     35678284262279+I*-1590527138776,
     35677380441961+I*-1589914601652,
     35677380440907+I*-1589914600980,32715905031631+I*104545662038,
     32715901588103+I*104547506192,32715901588099+I*104547506194,
     32704734490353+I*109477773922,32704734477415+I*109477779206,
     32704692910299+I*109491177844,32704692910251+I*109491177858,
     32571994299589+I*141389927374,32571994146703+I*141389959622,
     32571510111151+I*141454210408,-2340347329043+I*4392925071975,
     -2340347329599+I*4392925072033,-3863829545913+I*4478414095385,
     -3863831292155+I*4478414144341,-3863831292158+I*4478414144341,
     -3869284184802+I*4478152831883,-3869284191034+I*4478152831409,
     -3869303497608+I*4478149877441,-3869303497630+I*4478149877437,
     -3928762691798+I*4464369655111,-3928762759402+I*4464369637433,
     -3928969223780+I*4464298530637,-3928969224014+I*4464298530549,
     -4555459915210+I*4193797696627,-4555460623498+I*4193797366961,
     -4557589518064+I*4192602263449,-4557589520462+I*4192602262013,
     -10907985455774+I*-35888293521,-10907992587610+I*-35893336423,
     -10907992587618+I*-35893336429,-10929020220820+I*-53297264193,
     -10929020244326+I*-53297284785,-10929088608718+I*-53367020379,
     -10929088608794+I*-53367020461,
     -11125584718488+I*-286865492269,
     -11125584935870+I*-286865765793,
     -11126197472994+I*-287769586111,
     -11126197473666+I*-287769587165,
     -12820657736684+I*-3249244996441,
     -12820659580838+I*-3249248439969,
     -12820659580840+I*-3249248439973,
     -12825589848568+I*-3260415537719,
     -12825589853852+I*-3260415550657,
     -12825603252490+I*-3260457117773,
     -12825603252504+I*-3260457117821,
     -12857502002020+I*-3393155728483,
     -12857502034268+I*-3393155881369,
     -12857566285054+I*-3393639916921,
     -12857566285112+I*-3393639917477,
     -12943055308464+I*-4917122133791,
     -12943055357420+I*-4917123880033,
     -12943055357420+I*-4917123880036,
     -12942794044962+I*-4922576772680,
     -12942794044488+I*-4922576778912,
     -10836208868732+I*-21851859205070,
     -10836205914764+I*-21851878511644,
     -10836205914760+I*-21851878511666,
     -10822425692434+I*-21911337705834,
     -10822425674756+I*-21911337773438,
     -10822354567960+I*-21911544237816,
     -10822354567872+I*-21911544238050,
     -10551853733950+I*-22538034929246,
     -10551853404284+I*-22538035637534,
     -10550658300772+I*-22540164532100,
     -10550658299336+I*-22540164534498,
     -6322167743802+I*-28890560469810,
     -6322162700900+I*-28890567601646,
     -6322162700894+I*-28890567601654,
     -6304758773130+I*-28911595234856,
     -6304758752538+I*-28911595258362,
     -6304689016944+I*-28911663622754,
     -6304689016862+I*-28911663622830,
     -6071190545054+I*-29108159732524,
     -6071190271530+I*-29108159949906,
     -6070286451212+I*-29108772487030,
     -6070286450158+I*-29108772487702,
     -3108811040882+I*-30803232750720,
     -3108807597354+I*-30803234594874,
     -3108807597350+I*-30803234594876,
     -3097640499604+I*-30808164862604,
     -3097640486666+I*-30808164867888,
     -3097598919550+I*-30808178266526,
     -3097598919502+I*-30808178266540,
     -2964900308840+I*-30840077016056,
     -2964900155954+I*-30840077048304,
     -2964416120402+I*-30840141299090,
     31947441319792+I*-35091612160657,
     31947441320348+I*-35091612160715,
     33470923536662+I*-35177101184067 ],
   /* k=40 */ [ 34524217870782+I*-43641742446102,
     34524217870785+I*-43641742446102,
     34529670763429+I*-43641481133644,
     34529670769661+I*-43641481133170,
     51458953195819+I*-41534895957414,
     51458972502393+I*-41534893003446,
     51458972502415+I*-41534893003442,
     51518431696583+I*-41521112781116,
     51518431764187+I*-41521112763438,
     51518638228565+I*-41521041656642,
     51518638228799+I*-41521041656554,
     52145128919995+I*-41250540822632,
     52145129628283+I*-41250540492966,
     52147258522849+I*-41249345389454,
     52147258525247+I*-41249345388018,
     58497654460559+I*-37020854832484,
     58497661592395+I*-37020849789582,
     58497661592403+I*-37020849789576,
     58518689225605+I*-37003445861812,
     58518689249111+I*-37003445841220,
     58518757613503+I*-37003376105626,
     58518757613579+I*-37003376105544,
     58715253723273+I*-36769877633736,
     58715253940655+I*-36769877360212,
     58715866477779+I*-36768973539894,
     102130665641107+I*28803270478927,
     102130665641779+I*28803270479981,
     103825125904797+I*31764745889257,
     103825127748951+I*31764749332785,
     103825127748953+I*31764749332789,
     103830058016681+I*31775916430535,
     103830058021965+I*31775916443473,
     103830071420603+I*31775958010589,
     103830071420617+I*31775958010637,
     103861970170133+I*31908656621299,
     103861970202381+I*31908656774185,
     103862034453167+I*31909140809737,
     103862034453225+I*31909140810293,
     103947523476577+I*33432623026607,
     103947523525533+I*33432624772849,
     103947523525533+I*33432624772852,
     103947262213075+I*33438077665496,
     103947262212601+I*33438077671728,
     101840677036845+I*50367360097886,
     101840674082877+I*50367379404460,
     101840674082873+I*50367379404482,
     101826893860547+I*50426838598650,
     101826893842869+I*50426838666254,
     101826822736073+I*50427045130632,
     101826822735985+I*50427045130866,
     101556321902063+I*51053535822062,
     101556321572397+I*51053536530350,
     101555126468885+I*51055665424916,
     101555126467449+I*51055665427314,
     97326635911915+I*57406061362626,
     97326630869013+I*57406068494462,
     97326630869007+I*57406068494470,
     97309226941243+I*57427096127672,
     97309226920651+I*57427096151178,
     97309157185057+I*57427164515570,
     97309157184975+I*57427164515646,
     97075658713167+I*57623660625340,
     97075658439643+I*57623660842722,
     97074754619325+I*57624273379846,
     97074754618271+I*57624273380518,
     94113279208995+I*59318733643536,
     94113275765467+I*59318735487690,
     94113275765463+I*59318735487692,
     94102108667717+I*59323665755420,
     94102108654779+I*59323665760704,
     58136958626707+I*72039777835350,
     58136917059591+I*72039791233988,
     58136917059543+I*72039791234002,
     58004218448881+I*72071689983518,
     58004218295995+I*72071690015766,
     58003734260443+I*72071754266552,
     58003734259887+I*72071754266610,
     56480252043573+I*72157243289962,
     56480250297331+I*72157243338918,
     56480250297328+I*72157243338918,
     56474797404684+I*72156982026460,
     56474797398452+I*72156982025986,
     39545514972294+I*70050396850230,
     39545495665720+I*70050393896262,
     39545495665698+I*70050393896258,
     39486036471530+I*70036613673932,
     39486036403926+I*70036613656254,
     39485829939548+I*70036542549458,
     39485829939314+I*70036542549370,
     38859339248118+I*69766041715448,
     38859338539830+I*69766041385782,
     38857209645264+I*69764846282270,
     38857209642866+I*69764846280834,
     32506813707554+I*65536355725300,
     32506806575718+I*65536350682398,
     32506806575710+I*65536350682392,
     32485778942508+I*65518946754628,
     32485778919002+I*65518946734036,
     32485710554610+I*65518876998442,
     32485710554534+I*65518876998360,
     32289214444840+I*65285378526552,
     32289214227458+I*65285378253028,
     32288601690334+I*65284474432710,
     -11126197472994+I*-287769586111,
     -11126197473666+I*-287769587165,
     -12820657736684+I*-3249244996441,
     -12820659580838+I*-3249248439969,
     -12820659580840+I*-3249248439973,
     -12825589848568+I*-3260415537719,
     -12825589853852+I*-3260415550657,
     -12825603252490+I*-3260457117773,
     -12825603252504+I*-3260457117821,
     -12857502002020+I*-3393155728483,
     -12857502034268+I*-3393155881369,
     -12857566285054+I*-3393639916921,
     -12857566285112+I*-3393639917477,
     -12943055308464+I*-4917122133791,
     -12943055357420+I*-4917123880033,
     -12943055357420+I*-4917123880036,
     -12942794044962+I*-4922576772680,
     -12942794044488+I*-4922576778912,
     -10836208868732+I*-21851859205070,
     -10836205914764+I*-21851878511644,
     -10836205914760+I*-21851878511666,
     -10822425692434+I*-21911337705834,
     -10822425674756+I*-21911337773438,
     -10822354567960+I*-21911544237816,
     -10822354567872+I*-21911544238050,
     -10551853733950+I*-22538034929246,
     -10551853404284+I*-22538035637534,
     -10550658300772+I*-22540164532100,
     -10550658299336+I*-22540164534498,
     -6322167743802+I*-28890560469810,
     -6322162700900+I*-28890567601646,
     -6322162700894+I*-28890567601654,
     -6304758773130+I*-28911595234856,
     -6304758752538+I*-28911595258362,
     -6304689016944+I*-28911663622754,
     -6304689016862+I*-28911663622830,
     -6071190545054+I*-29108159732524,
     -6071190271530+I*-29108159949906,
     -6070286451212+I*-29108772487030,
     -6070286450158+I*-29108772487702,
     -3108811040882+I*-30803232750720,
     -3108807597354+I*-30803234594874,
     -3108807597350+I*-30803234594876,
     -3097640499604+I*-30808164862604,
     -3097640486666+I*-30808164867888,
     32867509541406+I*-43524276942534,
     32867551108522+I*-43524290341172,
     32867551108570+I*-43524290341186,
     33000249719232+I*-43556189090702,
     33000249872118+I*-43556189122950,
     33000733907670+I*-43556253373736,
     33000733908226+I*-43556253373794,
     34524216124540+I*-43641742397146 ],
   /* k=41 */ [ 34524217870782+I*-43641742446102,
     34524217870785+I*-43641742446102,
     34529670763429+I*-43641481133644,
     34529670769661+I*-43641481133170,
     51458953195819+I*-41534895957414,
     51458972502393+I*-41534893003446,
     51458972502415+I*-41534893003442,
     51518431696583+I*-41521112781116,
     51518431764187+I*-41521112763438,
     51518638228565+I*-41521041656642,
     51518638228799+I*-41521041656554,
     52145128919995+I*-41250540822632,
     52145129628283+I*-41250540492966,
     52147258522849+I*-41249345389454,
     52147258525247+I*-41249345388018,
     58497654460559+I*-37020854832484,
     58497661592395+I*-37020849789582,
     58497661592403+I*-37020849789576,
     58518689225605+I*-37003445861812,
     58518689249111+I*-37003445841220,
     119916063426475+I*22210742140278,
     119916131790867+I*22210811875872,
     119916131790943+I*22210811875954,
     120112627900637+I*22444310347762,
     120112628118019+I*22444310621286,
     120113240655143+I*22445214441604,
     120113240655815+I*22445214442658,
     121807700918833+I*25406689851934,
     121807702762987+I*25406693295462,
     121807702762989+I*25406693295466,
     121812633030717+I*25417860393212,
     121812633036001+I*25417860406150,
     134528745110647+I*61383010434222,
     134528758509285+I*61383052001338,
     134528758509299+I*61383052001386,
     134560657258815+I*61515750612048,
     134560657291063+I*61515750764934,
     134560721541849+I*61516234800486,
     134560721541907+I*61516234801042,
     134646210565259+I*63039717017356,
     134646210614215+I*63039718763598,
     134646210614215+I*63039718763601,
     134645949301757+I*63045171656245,
     134645949301283+I*63045171662477,
     132539364125527+I*79974454088635,
     132539361171559+I*79974473395209,
     132539361171555+I*79974473395231,
     132525580949229+I*80033932589399,
     132525580931551+I*80033932657003,
     132525509824755+I*80034139121381,
     132525509824667+I*80034139121615,
     132255008990745+I*80660629812811,
     132255008661079+I*80660630521099,
     132253813557567+I*80662759415665,
     44524124683253+I*233064601761142,
     44524124681817+I*233064601763540,
     40295634126283+I*239414997698852,
     40295629083381+I*239415004830688,
     40295629083375+I*239415004830696,
     40278225155611+I*239436032463898,
     40278225135019+I*239436032487404,
     40278155399425+I*239436100851796,
     40278155399343+I*239436100851872,
     40044656927535+I*239632596961566,
     40044656654011+I*239632597178948,
     40043752833693+I*239633209716072,
     40043752832639+I*239633209716744,
     37082277423363+I*241327669979762,
     37082273979835+I*241327671823916,
     37082273979831+I*241327671823918,
     37071106882085+I*241332602091646,
     37071106869147+I*241332602096930,
     1105956841075+I*254048714171576,
     1105915273959+I*254048727570214,
     1105915273911+I*254048727570228,
     973216663249+I*254080626319744,973216510363+I*254080626351992,
     972732474811+I*254080690602778,972732474255+I*254080690602836,
     -550749742059+I*254166179626188,
     -550751488301+I*254166179675144,
     -550751488304+I*254166179675144,
     -556204380948+I*254165918362686,
     -556204387180+I*254165918362212,
     -17485486813338+I*252059333186456,
     -17485506119912+I*252059330232488,
     -17485506119934+I*252059330232484,
     -17544965314102+I*252045550010158,
     -17544965381706+I*252045549992480,
     -17545171846084+I*252045478885684,
     -17545171846318+I*252045478885596,
     -18171662537514+I*251774978051674,
     -18171663245802+I*251774977722008,
     -18173792140368+I*251773782618496,
     -18173792142766+I*251773782617060,
     -24524188078078+I*247545292061526,
     -24524195209914+I*247545287018624,
     -24524195209922+I*247545287018618,
     -24545222843124+I*247527883090854,
     -24545222866630+I*247527883070262,
     -85942597043994+I*188313695088764,
     -85942665408386+I*188313625353170,
     -85942665408462+I*188313625353088,
     -86139161518156+I*188080126881280,
     -86139161735538+I*188080126607756,
     -86139774272662+I*188079222787438,
     -86139774273334+I*188079222786384,
     -87834234536352+I*185117747377108,
     -87834236380506+I*185117743933580,
     -87834236380508+I*185117743933576,
     -87839166648236+I*185106576835830,
     -87839166653520+I*185106576822892,
     -100555278728166+I*149141426794820,
     -100555292126804+I*149141385227704,
     -100555292126818+I*149141385227656,
     -100587190876334+I*149008686616994,
     -100587190908582+I*149008686464108,
     -100587255159368+I*149008202428556,
     -100587255159426+I*149008202428000,
     -100672744182778+I*147484720211686,
     -100672744231734+I*147484718465444,
     -100672744231734+I*147484718465441,
     -100672482919276+I*147479265572797,
     -100672482918802+I*147479265566565,
     -98565897743046+I*130549983140407,
     -98565894789078+I*130549963833833,
     -98565894789074+I*130549963833811,
     -98552114566748+I*130490504639643,
     -98552114549070+I*130490504572039,
     -98552043442274+I*130490298107661,
     -98552043442186+I*130490298107427,
     -98281542608264+I*129863807416231,
     -98281542278598+I*129863806707943,
     -98280347175086+I*129861677813377,
     -10550658300772+I*-22540164532100,
     -10550658299336+I*-22540164534498,
     -6322167743802+I*-28890560469810,
     -6322162700900+I*-28890567601646,
     -6322162700894+I*-28890567601654,
     -6304758773130+I*-28911595234856,
     -6304758752538+I*-28911595258362,
     -6304689016944+I*-28911663622754,
     -6304689016862+I*-28911663622830,
     -6071190545054+I*-29108159732524,
     -6071190271530+I*-29108159949906,
     -6070286451212+I*-29108772487030,
     -6070286450158+I*-29108772487702,
     -3108811040882+I*-30803232750720,
     -3108807597354+I*-30803234594874,
     -3108807597350+I*-30803234594876,
     -3097640499604+I*-30808164862604,
     -3097640486666+I*-30808164867888,
     32867509541406+I*-43524276942534,
     32867551108522+I*-43524290341172,
     32867551108570+I*-43524290341186,
     33000249719232+I*-43556189090702,
     33000249872118+I*-43556189122950,
     33000733907670+I*-43556253373736,
     33000733908226+I*-43556253373794,
     34524216124540+I*-43641742397146 ],
   /* k=42 */ [ -358009155694486+I*-66699277849253,
     -358009155694483+I*-66699277849253,
     -358003702801839+I*-66699016536795,
     34529670763429+I*-43641481133644,
     34529670769661+I*-43641481133170,
     51458953195819+I*-41534895957414,
     51458972502393+I*-41534893003446,
     51458972502415+I*-41534893003442,
     51518431696583+I*-41521112781116,
     51518431764187+I*-41521112763438,
     51518638228565+I*-41521041656642,
     51518638228799+I*-41521041656554,
     52145128919995+I*-41250540822632,
     52145129628283+I*-41250540492966,
     52147258522849+I*-41249345389454,
     52147258525247+I*-41249345388018,
     58497654460559+I*-37020854832484,
     58497661592395+I*-37020849789582,
     58497661592403+I*-37020849789576,
     58518689225605+I*-37003445861812,
     58518689249111+I*-37003445841220,
     119916063426475+I*22210742140278,
     119916131790867+I*22210811875872,
     119916131790943+I*22210811875954,
     120112627900637+I*22444310347762,
     120112628118019+I*22444310621286,
     120113240655143+I*22445214441604,
     120113240655815+I*22445214442658,
     121807700918833+I*25406689851934,
     121807702762987+I*25406693295462,
     121807702762989+I*25406693295466,
     121812633030717+I*25417860393212,
     121812633036001+I*25417860406150,
     134528745110647+I*61383010434222,
     134528758509285+I*61383052001338,
     134528758509299+I*61383052001386,
     134560657258815+I*61515750612048,
     134560657291063+I*61515750764934,
     134560721541849+I*61516234800486,
     134560721541907+I*61516234801042,
     134646210565259+I*63039717017356,
     134646210614215+I*63039718763598,
     134646210614215+I*63039718763601,
     134645949301757+I*63045171656245,
     134645949301283+I*63045171662477,
     132539364125527+I*79974454088635,
     132539361171559+I*79974473395209,
     132539361171555+I*79974473395231,
     132525580949229+I*80033932589399,
     132525580931551+I*80033932657003,
     75494579145919+I*262042868993229,
     75494508039123+I*262043075457607,
     75494508039035+I*262043075457841,
     75224007205113+I*262669566149037,
     75224006875447+I*262669566857325,
     75222811771935+I*262671695751891,
     75222811770499+I*262671695754289,
     70994321214965+I*269022091689601,
     70994316172063+I*269022098821437,
     70994316172057+I*269022098821445,
     70976912244293+I*269043126454647,
     70976912223701+I*269043126478153,
     11762724242203+I*330440500655517,
     11762654506609+I*330440569019909,
     11762654506527+I*330440569019985,
     11529156034719+I*330637065129679,
     11529155761195+I*330637065347061,
     11528251940877+I*330637677884185,
     11528251939823+I*330637677884857,
     8566776530547+I*332332138147875,
     8566773087019+I*332332139992029,
     8566773087015+I*332332139992031,
     8555605989269+I*332337070259759,
     8555605976331+I*332337070265043,
     -27409544051741+I*345053182339689,
     -27409585618857+I*345053195738327,
     -27409585618905+I*345053195738341,
     -27542284229567+I*345085094487857,
     -27542284382453+I*345085094520105,
     -27542768418005+I*345085158770891,
     -27542768418561+I*345085158770949,
     -29066250634875+I*345170647794301,
     -29066252381117+I*345170647843257,
     -29066252381120+I*345170647843257,
     -29071705273764+I*345170386530799,
     -421605078839032+I*322112851127648,
     -421605078845264+I*322112851127174,
     -438534361271422+I*320006265951418,
     -438534380577996+I*320006262997450,
     -438534380578018+I*320006262997446,
     -438593839772186+I*319992482775120,
     -438593839839790+I*319992482757442,
     -438594046304168+I*319992411650646,
     -438594046304402+I*319992411650558,
     -439220536995598+I*319721910816636,
     -439220537703886+I*319721910486970,
     -439222666598452+I*319720715383458,
     -439222666600850+I*319720715382022,
     -445573062536162+I*315492224826488,
     -445573069667998+I*315492219783586,
     -445573069668006+I*315492219783580,
     -445594097301208+I*315474815855816,
     -445594097324714+I*315474815835224,
     -506991471502078+I*256260627853726,
     -506991539866470+I*256260558118132,
     -506991539866546+I*256260558118050,
     -507188035976240+I*256027059646242,
     -507188036193622+I*256027059372718,
     -507188648730746+I*256026155552400,
     -507188648731418+I*256026155551346,
     -508883108994436+I*253064680142070,
     -508883110838590+I*253064676698542,
     -508883110838592+I*253064676698538,
     -508888041106320+I*253053509600792,
     -508888041111604+I*253053509587854,
     -521604153186250+I*217088359559782,
     -521604166584888+I*217088317992666,
     -521604166584902+I*217088317992618,
     -521636065334418+I*216955619381956,
     -521636065366666+I*216955619229070,
     -521636129617452+I*216955135193518,
     -521636129617510+I*216955135192962,
     -521721618640862+I*215431652976648,
     -521721618689818+I*215431651230406,
     -521721618689818+I*215431651230403,
     -521721357377360+I*215426198337759,
     -521721357376886+I*215426198331527,
     -519614772201130+I*198496915905369,
     -519614769247162+I*198496896598795,
     -519614769247158+I*198496896598773,
     -519600989024832+I*198437437404605,
     -519600989007154+I*198437437337001,
     -462569987221522+I*16428501000775,
     -462569916114726+I*16428294536397,
     -462569916114638+I*16428294536163,
     -462299415280716+I*15801803844967,
     -462299414951050+I*15801803136679,
     -462298219847538+I*15799674242113,
     -462298219846102+I*15799674239715,
     -458069729290568+I*9449278304403,
     -458069724247666+I*9449271172567,
     -458069724247660+I*9449271172559,
     -458052320319896+I*9428243539357,
     -458052320299304+I*9428243515851,
     -398838132317806+I*-51969130661513,
     -398838062582212+I*-51969199025905,
     -398838062582130+I*-51969199025981,
     -398604564110322+I*-52165695135675,
     -398604563836798+I*-52165695353057,
     -398603660016480+I*-52166307890181,
     -398603660015426+I*-52166307890853,
     -395642184606150+I*-53860768153871,
     -395642181162622+I*-53860769998025,
     -395642181162618+I*-53860769998027,
     -395631014064872+I*-53865700265755,
     -395631014051934+I*-53865700271039,
     -359665864023862+I*-66581812345685,
     -359665822456746+I*-66581825744323,
     -359665822456698+I*-66581825744337,
     -359533123846036+I*-66613724493853,
     -359533123693150+I*-66613724526101,
     -359532639657598+I*-66613788776887,
     -359532639657042+I*-66613788776945,
     -358009157440728+I*-66699277800297 ],
   /* k=43 */ [ -493903021224410+I*-908797026765421,
     -493903021224407+I*-908797026765421,
     -493897568331763+I*-908796765452963,
     -493897568325531+I*-908796765452489,
     -476968285899373+I*-906690180276733,
     -476968266592799+I*-906690177322765,
     -476968266592777+I*-906690177322761,
     -476908807398609+I*-906676397100435,
     -476908807331005+I*-906676397082757,
     -294899870994779+I*-849645395297125,
     -294899664530401+I*-849645324190329,
     -294899664530167+I*-849645324190241,
     -294273173838971+I*-849374823356319,
     -294273173130683+I*-849374823026653,
     -294271044236117+I*-849373627923141,
     -294271044233719+I*-849373627921705,
     -287920648298407+I*-845145137366171,
     -287920641166571+I*-845145132323269,
     -287920641166563+I*-845145132323263,
     -287899613533361+I*-845127728395499,
     -287899613509855+I*-845127728374907,
     -226502239332491+I*-785913540393409,
     -226502170968099+I*-785913470657815,
     -226502170968023+I*-785913470657733,
     -226305674858329+I*-785679972185925,
     -226305674640947+I*-785679971912401,
     -226305062103823+I*-785679068092083,
     -226305062103151+I*-785679068091029,
     -224610601840133+I*-782717592681753,
     -224610599995979+I*-782717589238225,
     -224610599995977+I*-782717589238221,
     -224605669728249+I*-782706422140475,
     121812633030717+I*25417860393212,
     121812633036001+I*25417860406150,
     134528745110647+I*61383010434222,
     134528758509285+I*61383052001338,
     134528758509299+I*61383052001386,
     134560657258815+I*61515750612048,
     134560657291063+I*61515750764934,
     134560721541849+I*61516234800486,
     134560721541907+I*61516234801042,
     134646210565259+I*63039717017356,
     134646210614215+I*63039718763598,
     134646210614215+I*63039718763601,
     134645949301757+I*63045171656245,
     134645949301283+I*63045171662477,
     132539364125527+I*79974454088635,
     132539361171559+I*79974473395209,
     132539361171555+I*79974473395231,
     132525580949229+I*80033932589399,
     132525580931551+I*80033932657003,
     75494579145919+I*262042868993229,
     75494508039123+I*262043075457607,
     75494508039035+I*262043075457841,
     75224007205113+I*262669566149037,
     75224006875447+I*262669566857325,
     75222811771935+I*262671695751891,
     75222811770499+I*262671695754289,
     70994321214965+I*269022091689601,
     70994316172063+I*269022098821437,
     70994316172057+I*269022098821445,
     70976912244293+I*269043126454647,
     70976912223701+I*269043126478153,
     11762724242203+I*330440500655517,
     11762654506609+I*330440569019909,
     11762654506527+I*330440569019985,
     11529156034719+I*330637065129679,
     11529155761195+I*330637065347061,
     11528251940877+I*330637677884185,
     11528251939823+I*330637677884857,
     8566776530547+I*332332138147875,
     8566773087019+I*332332139992029,
     8566773087015+I*332332139992031,
     8555605989269+I*332337070259759,
     8555605976331+I*332337070265043,
     -27409544051741+I*345053182339689,
     -27409585618857+I*345053195738327,
     -27409585618905+I*345053195738341,
     -27542284229567+I*345085094487857,
     -27542284382453+I*345085094520105,
     -448591158840537+I*413032027285067,
     -448591642876089+I*413032091535853,
     -448591642876645+I*413032091535911,
     -450115125092959+I*413117580559263,
     -450115126839201+I*413117580608219,
     -450115126839204+I*413117580608219,
     -450120579731848+I*413117319295761,
     -450120579738080+I*413117319295287,
     -467049862164238+I*411010734119531,
     -467049881470812+I*411010731165563,
     -467049881470834+I*411010731165559,
     -467109340665002+I*410996950943233,
     -467109340732606+I*410996950925555,
     -649118277068832+I*353965949139923,
     -649118483533210+I*353965878033127,
     -649118483533444+I*353965878033039,
     -649744974224640+I*353695377199117,
     -649744974932928+I*353695376869451,
     -649747103827494+I*353694181765939,
     -649747103829892+I*353694181764503,
     -656097499765204+I*349465691208969,
     -656097506897040+I*349465686166067,
     -656097506897048+I*349465686166061,
     -656118534530250+I*349448282238297,
     -656118534553756+I*349448282217705,
     -717515908731120+I*290234094236207,
     -717515977095512+I*290234024500613,
     -717515977095588+I*290234024500531,
     -717712473205282+I*290000526028723,
     -717712473422664+I*290000525755199,
     -717713085959788+I*289999621934881,
     -717713085960460+I*289999621933827,
     -719407546223478+I*287038146524551,
     -719407548067632+I*287038143081023,
     -719407548067634+I*287038143081019,
     -719412478335362+I*287026975983273,
     -1065830781094328+I*-521097306550414,
     -1065830781099612+I*-521097306563352,
     -1078546893174258+I*-557062456591424,
     -1078546906572896+I*-557062498158540,
     -1078546906572910+I*-557062498158588,
     -1078578805322426+I*-557195196769250,
     -1078578805354674+I*-557195196922136,
     -1078578869605460+I*-557195680957688,
     -1078578869605518+I*-557195680958244,
     -1078664358628870+I*-558719163174558,
     -1078664358677826+I*-558719164920800,
     -1078664358677826+I*-558719164920803,
     -1078664097365368+I*-558724617813447,
     -1078664097364894+I*-558724617819679,
     -1076557512189138+I*-575653900245837,
     -1076557509235170+I*-575653919552411,
     -1076557509235166+I*-575653919552433,
     -1076543729012840+I*-575713378746601,
     -1076543728995162+I*-575713378814205,
     -1019512727209530+I*-757722315150431,
     -1019512656102734+I*-757722521614809,
     -1019512656102646+I*-757722521615043,
     -1019242155268724+I*-758349012306239,
     -1019242154939058+I*-758349013014527,
     -1019240959835546+I*-758351141909093,
     -1019240959834110+I*-758351141911491,
     -1015012469278576+I*-764701537846803,
     -1015012464235674+I*-764701544978639,
     -1015012464235668+I*-764701544978647,
     -1014995060307904+I*-764722572611849,
     -1014995060287312+I*-764722572635355,
     -955780872305814+I*-826119946812719,
     -955780802570220+I*-826120015177111,
     -955780802570138+I*-826120015177187,
     -955547304098330+I*-826316511286881,
     -955547303824806+I*-826316511504263,
     -955546400004488+I*-826317124041387,
     -955546400003434+I*-826317124042059,
     -952584924594158+I*-828011584305077,
     -952584921150630+I*-828011586149231,
     -952584921150626+I*-828011586149233,
     -952573754052880+I*-828016516416961,
     -952573754039942+I*-828016516422245,
     -916608604011870+I*-840732628496891,
     -916608562444754+I*-840732641895529,
     -916608562444706+I*-840732641895543,
     -916475863834044+I*-840764540645059,
     -916475863681158+I*-840764540677307,
     -495426989223074+I*-908711473442269,
     -495426505187522+I*-908711537693055,
     -495426505186966+I*-908711537693113,
     -493903022970652+I*-908797026716465 ],
   /* k=44 */ [ 497455871089994+I*-2796833322892643,
     497455871089997+I*-2796833322892643,
     497461323982641+I*-2796833061580185,
     497461323988873+I*-2796833061579711,
     514390606415031+I*-2794726476403955,
     514390625721605+I*-2794726473449987,
     514390625721627+I*-2794726473449983,
     514450084915795+I*-2794712693227657,
     514450084983399+I*-2794712693209979,
     696459021319625+I*-2737681691424347,
     696459227784003+I*-2737681620317551,
     696459227784237+I*-2737681620317463,
     697085718475433+I*-2737411119483541,
     697085719183721+I*-2737411119153875,
     697087848078287+I*-2737409924050363,
     697087848080685+I*-2737409924048927,
     703438244015997+I*-2733181433493393,
     703438251147833+I*-2733181428450491,
     703438251147841+I*-2733181428450485,
     703459278781043+I*-2733164024522721,
     703459278804549+I*-2733164024502129,
     764856652981913+I*-2673949836520631,
     764856721346305+I*-2673949766785037,
     764856721346381+I*-2673949766784955,
     765053217456075+I*-2673716268313147,
     765053217673457+I*-2673716268039623,
     1321995957661465+I*-1899565451888417,
     1321996570198589+I*-1899564548068099,
     1321996570199261+I*-1899564548067045,
     1323691030462279+I*-1896603072657769,
     1323691032306433+I*-1896603069214241,
     1323691032306435+I*-1896603069214237,
     1323695962574163+I*-1896591902116491,
     1323695962579447+I*-1896591902103553,
     1336412074654093+I*-1860626752075481,
     1336412088052731+I*-1860626710508365,
     1336412088052745+I*-1860626710508317,
     1336443986802261+I*-1860494011897655,
     1336443986834509+I*-1860494011744769,
     1404390919599471+I*-1439445137286685,
     1404390983850257+I*-1439444653251133,
     1404390983850315+I*-1439444653250577,
     1404476472873667+I*-1437921171034263,
     1404476472922623+I*-1437921169288021,
     1404476472922623+I*-1437921169288018,
     1404476211610165+I*-1437915716395374,
     1404476211609691+I*-1437915716389142,
     1402369626433935+I*-1420986433962984,
     1402369623479967+I*-1420986414656410,
     1402369623479963+I*-1420986414656388,
     1402355843257637+I*-1420926955462220,
     1402355843239959+I*-1420926955394616,
     1345324841454327+I*-1238918019058390,
     1345324770347531+I*-1238917812594012,
     1345324770347443+I*-1238917812593778,
     1345054269513521+I*-1238291321902582,
     1345054269183855+I*-1238291321194294,
     1345053074080343+I*-1238289192299728,
     1345053074078907+I*-1238289192297330,
     1340824583523373+I*-1231938796362018,
     1340824578480471+I*-1231938789230182,
     1340824578480465+I*-1231938789230174,
     1340807174552701+I*-1231917761596972,
     70976912244293+I*269043126454647,
     70976912223701+I*269043126478153,
     11762724242203+I*330440500655517,
     11762654506609+I*330440569019909,
     11762654506527+I*330440569019985,
     11529156034719+I*330637065129679,
     11529155761195+I*330637065347061,
     11528251940877+I*330637677884185,
     11528251939823+I*330637677884857,
     8566776530547+I*332332138147875,
     8566773087019+I*332332139992029,
     8566773087015+I*332332139992031,
     8555605989269+I*332337070259759,
     8555605976331+I*332337070265043,
     -27409544051741+I*345053182339689,
     -27409585618857+I*345053195738327,
     -27409585618905+I*345053195738341,
     -27542284229567+I*345085094487857,
     -27542284382453+I*345085094520105,
     -448591158840537+I*413032027285067,
     -448591642876089+I*413032091535853,
     -448591642876645+I*413032091535911,
     -450115125092959+I*413117580559263,
     -450115126839201+I*413117580608219,
     -450115126839204+I*413117580608219,
     -450120579731848+I*413117319295761,
     -450120579738080+I*413117319295287,
     -467049862164238+I*411010734119531,
     -467049881470812+I*411010731165563,
     -467049881470834+I*411010731165559,
     -467109340665002+I*410996950943233,
     -467109340732606+I*410996950925555,
     -649118277068832+I*353965949139923,
     -649118483533210+I*353965878033127,
     -649118483533444+I*353965878033039,
     -649744974224640+I*353695377199117,
     -649744974932928+I*353695376869451,
     -649747103827494+I*353694181765939,
     -649747103829892+I*353694181764503,
     -656097499765204+I*349465691208969,
     -656097506897040+I*349465686166067,
     -656097506897048+I*349465686166061,
     -656118534530250+I*349448282238297,
     -656118534553756+I*349448282217705,
     -717515908731120+I*290234094236207,
     -717515977095512+I*290234024500613,
     -717515977095588+I*290234024500531,
     -717712473205282+I*290000526028723,
     -717712473422664+I*290000525755199,
     -1274655213410672+I*-484150290396007,
     -1274655825947796+I*-484151194216325,
     -1274655825948468+I*-484151194217379,
     -1276350286211486+I*-487112669626655,
     -1276350288055640+I*-487112673070183,
     -1276350288055642+I*-487112673070187,
     -1276355218323370+I*-487123840167933,
     -1276355218328654+I*-487123840180871,
     -1289071330403300+I*-523088990208943,
     -1289071343801938+I*-523089031776059,
     -1289071343801952+I*-523089031776107,
     -1289103242551468+I*-523221730386769,
     -1289103242583716+I*-523221730539655,
     -1357050175348678+I*-944270604997739,
     -1357050239599464+I*-944271089033291,
     -1357050239599522+I*-944271089033847,
     -1357135728622874+I*-945794571250161,
     -1357135728671830+I*-945794572996403,
     -1357135728671830+I*-945794572996406,
     -1357135467359372+I*-945800025889050,
     -1357135467358898+I*-945800025895282,
     -1355028882183142+I*-962729308321440,
     -1355028879229174+I*-962729327628014,
     -1355028879229170+I*-962729327628036,
     -1355015099006844+I*-962788786822204,
     -1355015098989166+I*-962788786889808,
     -1297984097203534+I*-1144797723226034,
     -1297984026096738+I*-1144797929690412,
     -1297984026096650+I*-1144797929690646,
     -1297713525262728+I*-1145424420381842,
     -1297713524933062+I*-1145424421090130,
     -1297712329829550+I*-1145426549984696,
     -1297712329828114+I*-1145426549987094,
     -1293483839272580+I*-1151776945922406,
     -1293483834229678+I*-1151776953054242,
     -1293483834229672+I*-1151776953054250,
     -1293466430301908+I*-1151797980687452,
     -23636167993500+I*-2652758868739071,
     -23636167972908+I*-2652758868762577,
     35578020008590+I*-2714156242939941,
     35578089744184+I*-2714156311304333,
     35578089744266+I*-2714156311304409,
     35811588216074+I*-2714352807414103,
     35811588489598+I*-2714352807631485,
     35812492309916+I*-2714353420168609,
     35812492310970+I*-2714353420169281,
     38773967720246+I*-2716047880432299,
     38773971163774+I*-2716047882276453,
     38773971163778+I*-2716047882276455,
     38785138261524+I*-2716052812544183,
     38785138274462+I*-2716052812549467,
     74750288302534+I*-2728768924624113,
     74750329869650+I*-2728768938022751,
     74750329869698+I*-2728768938022765,
     74883028480360+I*-2728800836772281,
     74883028633246+I*-2728800836804529,
     495931903091330+I*-2796747769569491,
     495932387126882+I*-2796747833820277,
     495932387127438+I*-2796747833820335,
     497455869343752+I*-2796833322843687 ],
   /* k=45 */ [ 993135317247196+I*-3740851470956254,
     993135317247199+I*-3740851470956254,
     993140770139843+I*-3740851209643796,
     993140770146075+I*-3740851209643322,
     1010070052572233+I*-3738744624467566,
     1010070071878807+I*-3738744621513598,
     1010070071878829+I*-3738744621513594,
     1010129531072997+I*-3738730841291268,
     5281881569484643+I*-2700031204726071,
     5281881569552247+I*-2700031204708393,
     5463890505888473+I*-2643000202922761,
     5463890712352851+I*-2643000131815965,
     5463890712353085+I*-2643000131815877,
     5464517203044281+I*-2642729630981955,
     5464517203752569+I*-2642729630652289,
     5464519332647135+I*-2642728435548777,
     5464519332649533+I*-2642728435547341,
     5470869728584845+I*-2638499944991807,
     5470869735716681+I*-2638499939948905,
     5470869735716689+I*-2638499939948899,
     5470890763349891+I*-2638482536021135,
     5470890763373397+I*-2638482536000543,
     5532288137550761+I*-2579268348019045,
     5532288205915153+I*-2579268278283451,
     5532288205915229+I*-2579268278283369,
     5532484702024923+I*-2579034779811561,
     5532484702242305+I*-2579034779538037,
     6089427442230313+I*-1804883963386831,
     6089428054767437+I*-1804883059566513,
     6089428054768109+I*-1804883059565459,
     6091122515031127+I*-1801921584156183,
     6091122516875281+I*-1801921580712655,
     6091122516875283+I*-1801921580712651,
     6091127447143011+I*-1801910413614905,
     6091127447148295+I*-1801910413601967,
     6103843559222941+I*-1765945263573895,
     6103843572621579+I*-1765945222006779,
     6103843572621593+I*-1765945222006731,
     6103875471371109+I*-1765812523396069,
     6103875471403357+I*-1765812523243183,
     6171822404168319+I*-1344763648785099,
     6171822468419105+I*-1344763164749547,
     6171822468419163+I*-1344763164748991,
     6171907957442515+I*-1343239682532677,
     6171907957491471+I*-1343239680786435,
     6171907957491471+I*-1343239680786432,
     6171907696179013+I*-1343234227893788,
     6171907696178539+I*-1343234227887556,
     6169801111002783+I*-1326304945461398,
     6169801108048815+I*-1326304926154824,
     6169801108048811+I*-1326304926154802,
     6169787327826485+I*-1326245466960634,
     6169787327808807+I*-1326245466893030,
     6112756326023175+I*-1144236530556804,
     6112756254916379+I*-1144236324092426,
     6112756254916291+I*-1144236324092192,
     6112485754082369+I*-1143609833400996,
     6112485753752703+I*-1143609832692708,
     5121126861438299+I*744426463434514,
     5121125666334787+I*744428592329080,
     5121125666333351+I*744428592331478,
     5116897175777817+I*750778988266790,
     5116897170734915+I*750778995398626,
     5116897170734909+I*750778995398634,
     5116879766807145+I*750800023031836,
     5116879766786553+I*750800023055342,
     5057665578805055+I*812197397232706,
     5057665509069461+I*812197465597098,
     5057665509069379+I*812197465597174,
     5057432010597571+I*812393961706868,
     5057432010324047+I*812393961924250,
     4283281194172841+I*1369336701912258,
     4283280290352523+I*1369337314449382,
     4283280290351469+I*1369337314450054,
     4280318814942193+I*1371031774713072,
     4280318811498665+I*1371031776557226,
     4280318811498661+I*1371031776557228,
     4280307644400915+I*1371036706824956,
     4280307644387977+I*1371036706830240,
     4244342494359905+I*1383752818904886,
     4244342452792789+I*1383752832303524,
     4244342452792741+I*1383752832303538,
     4244209754182079+I*1383784731053054,
     4244209754029193+I*1383784731085302,
     3823160879571109+I*1451731663850264,
     3823160395535557+I*1451731728101050,
     3823160395535001+I*1451731728101108,
     3821636913318687+I*1451817217124460,
     3821636911572445+I*1451817217173416,
     3821636911572442+I*1451817217173416,
     3821631458679798+I*1451816955860958,
     3821631458673566+I*1451816955860484,
     3804702176247408+I*1449710370684728,
     3804702156940834+I*1449710367730760,
     3804702156940812+I*1449710367730756,
     3804642697746644+I*1449696587508430,
     -467109340665002+I*410996950943233,
     -467109340732606+I*410996950925555,
     -649118277068832+I*353965949139923,
     -649118483533210+I*353965878033127,
     -649118483533444+I*353965878033039,
     -649744974224640+I*353695377199117,
     -649744974932928+I*353695376869451,
     -649747103827494+I*353694181765939,
     -649747103829892+I*353694181764503,
     -656097499765204+I*349465691208969,
     -656097506897040+I*349465686166067,
     -656097506897048+I*349465686166061,
     -656118534530250+I*349448282238297,
     -656118534553756+I*349448282217705,
     -717515908731120+I*290234094236207,
     -717515977095512+I*290234024500613,
     -717515977095588+I*290234024500531,
     -717712473205282+I*290000526028723,
     -717712473422664+I*290000525755199,
     -1274655213410672+I*-484150290396007,
     -1274655825947796+I*-484151194216325,
     -1274655825948468+I*-484151194217379,
     -1276350286211486+I*-487112669626655,
     -1276350288055640+I*-487112673070183,
     -1276350288055642+I*-487112673070187,
     -1276355218323370+I*-487123840167933,
     -1276355218328654+I*-487123840180871,
     -1289071330403300+I*-523088990208943,
     -1289071343801938+I*-523089031776059,
     -1289071343801952+I*-523089031776107,
     -1289103242551468+I*-523221730386769,
     -1289103242583716+I*-523221730539655,
     -1357050175348678+I*-944270604997739,
     -1357050239599464+I*-944271089033291,
     -1357050239599522+I*-944271089033847,
     -1357135728622874+I*-945794571250161,
     -1357135728671830+I*-945794572996403,
     -1357135728671830+I*-945794572996406,
     -1357135467359372+I*-945800025889050,
     -1357135467358898+I*-945800025895282,
     -1355028882183142+I*-962729308321440,
     -1355028879229174+I*-962729327628014,
     -1355028879229170+I*-962729327628036,
     -1355015099006844+I*-962788786822204,
     -1355015098989166+I*-962788786889808,
     -1297984097203534+I*-1144797723226034,
     -1297984026096738+I*-1144797929690412,
     -1297984026096650+I*-1144797929690646,
     -1297713525262728+I*-1145424420381842,
     -1297713524933062+I*-1145424421090130,
     -306354632618658+I*-3033460717217352,
     -306353437515146+I*-3033462846111918,
     -306353437513710+I*-3033462846114316,
     -302124946958176+I*-3039813242049628,
     -302124941915274+I*-3039813249181464,
     -302124941915268+I*-3039813249181472,
     -302107537987504+I*-3039834276814674,
     -302107537966912+I*-3039834276838180,
     -242893349985414+I*-3101231651015544,
     -242893280249820+I*-3101231719379936,
     -242893280249738+I*-3101231719380012,
     -242659781777930+I*-3101428215489706,
     -242659781504406+I*-3101428215707088,
     531491034646800+I*-3658370955695096,
     531491938467118+I*-3658371568232220,
     531491938468172+I*-3658371568232892,
     534453413877448+I*-3660066028495910,
     534453417320976+I*-3660066030340064,
     534453417320980+I*-3660066030340066,
     534464584418726+I*-3660070960607794,
     534464584431664+I*-3660070960613078,
     570429734459736+I*-3672787072687724,
     570429776026852+I*-3672787086086362,
     570429776026900+I*-3672787086086376,
     570562474637562+I*-3672818984835892,
     570562474790448+I*-3672818984868140,
     991611349248532+I*-3740765917633102,
     991611833284084+I*-3740765981883888,
     991611833284640+I*-3740765981883946,
     993135315500954+I*-3740851470907298 ],
   /* k=46 */ [ 993135317247196+I*-3740851470956254,
     993135317247199+I*-3740851470956254,
     5760566801816047+I*-3646169982454668,
     5760572254708691+I*-3646169721142210,
     5760572254714923+I*-3646169721141736,
     5777501537141081+I*-3644063135965980,
     5777501556447655+I*-3644063133012012,
     5777501556447677+I*-3644063133012008,
     5777561015641845+I*-3644049352789682,
     5777561015709449+I*-3644049352772004,
     5959569952045675+I*-3587018350986372,
     5959570158510053+I*-3587018279879576,
     5959570158510287+I*-3587018279879488,
     5960196649201483+I*-3586747779045566,
     5960196649909771+I*-3586747778715900,
     7848232946036993+I*-2595388886401496,
     7848235074931559+I*-2595387691297984,
     7848235074933957+I*-2595387691296548,
     7854585470869269+I*-2591159200741014,
     7854585478001105+I*-2591159195698112,
     7854585478001113+I*-2591159195698106,
     7854606505634315+I*-2591141791770342,
     7854606505657821+I*-2591141791749750,
     7916003879835185+I*-2531927603768252,
     7916003948199577+I*-2531927534032658,
     7916003948199653+I*-2531927534032576,
     7916200444309347+I*-2531694035560768,
     7916200444526729+I*-2531694035287244,
     8473143184514737+I*-1757543219136038,
     8473143797051861+I*-1757542315315720,
     8473143797052533+I*-1757542315314666,
     8474838257315551+I*-1754580839905390,
     8474838259159705+I*-1754580836461862,
     8474838259159707+I*-1754580836461858,
     8474843189427435+I*-1754569669364112,
     8474843189432719+I*-1754569669351174,
     8487559301507365+I*-1718604519323102,
     8487559314906003+I*-1718604477755986,
     8487559314906017+I*-1718604477755938,
     8487591213655533+I*-1718471779145276,
     10681943978936785+I*7863731934243213,
     10681943978969033+I*7863731934396099,
     10749890911733995+I*8284780808854183,
     10749890975984781+I*8284781292889735,
     10749890975984839+I*8284781292890291,
     10749976465008191+I*8286304775106605,
     10749976465057147+I*8286304776852847,
     10749976465057147+I*8286304776852850,
     10749976203744689+I*8286310229745494,
     10749976203744215+I*8286310229751726,
     10747869618568459+I*8303239512177884,
     10747869615614491+I*8303239531484458,
     10747869615614487+I*8303239531484480,
     10747855835392161+I*8303298990678648,
     10747855835374483+I*8303298990746252,
     10690824833588851+I*8485307927082478,
     10690824762482055+I*8485308133546856,
     10690824762481967+I*8485308133547090,
     10690554261648045+I*8485934624238286,
     10690554261318379+I*8485934624946574,
     9699195369003975+I*10373970921073796,
     9699194173900463+I*10373973049968362,
     9699194173899027+I*10373973049970760,
     9694965683343493+I*10380323445906072,
     9694965678300591+I*10380323453037908,
     9694965678300585+I*10380323453037916,
     9694948274372821+I*10380344480671118,
     9694948274352229+I*10380344480694624,
     9635734086370731+I*10441741854871988,
     9635734016635137+I*10441741923236380,
     9635734016635055+I*10441741923236456,
     9635500518163247+I*10441938419346150,
     9635500517889723+I*10441938419563532,
     8861349701738517+I*10998881159551540,
     8861348797918199+I*10998881772088664,
     8861348797917145+I*10998881772089336,
     8858387322507869+I*11000576232352354,
     8858387319064341+I*11000576234196508,
     8858387319064337+I*11000576234196510,
     8858376151966591+I*11000581164464238,
     8858376151953653+I*11000581164469522,
     8822411001925581+I*11013297276544168,
     8822410960358465+I*11013297289942806,
     8822410960358417+I*11013297289942820,
     8822278261747755+I*11013329188692336,
     8822278261594869+I*11013329188724584,
     8401229387136785+I*11081276121489546,
     8401228903101233+I*11081276185740332,
     8401228903100677+I*11081276185740390,
     8399705420884363+I*11081361674763742,
     8399705419138121+I*11081361674812698,
     8399705419138118+I*11081361674812698,
     3632273934569270+I*10986680186311112,
     3632268481676626+I*10986679924998654,
     3632268481670394+I*10986679924998180,
     3615339199244236+I*10984573339822424,
     3615339179937662+I*10984573336868456,
     3615339179937640+I*10984573336868452,
     3615279720743472+I*10984559556646126,
     3615279720675868+I*10984559556628448,
     3433270784339642+I*10927528554842816,
     3433270577875264+I*10927528483736020,
     3433270577875030+I*10927528483735932,
     3432644087183834+I*10927257982902010,
     3432644086475546+I*10927257982572344,
     1544607790348324+I*9935899090257940,
     1544605661453758+I*9935897895154428,
     1544605661451360+I*9935897895152992,
     1538255265516048+I*9931669404597458,
     1538255258384212+I*9931669399554556,
     1538255258384204+I*9931669399554550,
     1538234230751002+I*9931651995626786,
     1538234230727496+I*9931651995606194,
     1476836856550132+I*9872437807624696,
     1476836788185740+I*9872437737889102,
     1476836788185664+I*9872437737889020,
     1476640292075970+I*9872204239417212,
     1476640291858588+I*9872204239143688,
     919697551870580+I*9098053422992482,
     919696939333456+I*9098052519172164,
     919696939332784+I*9098052519171110,
     918002479069766+I*9095091043761834,
     918002477225612+I*9095091040318306,
     918002477225610+I*9095091040318302,
     917997546957882+I*9095079873220556,
     917997546952598+I*9095079873207618,
     905281434877952+I*9059114723179546,
     905281421479314+I*9059114681612430,
     905281421479300+I*9059114681612382,
     905249522729784+I*9058981983001720,
     -1289103242551468+I*-523221730386769,
     -1289103242583716+I*-523221730539655,
     -1357050175348678+I*-944270604997739,
     -1357050239599464+I*-944271089033291,
     -1357050239599522+I*-944271089033847,
     -1357135728622874+I*-945794571250161,
     -1357135728671830+I*-945794572996403,
     -1357135728671830+I*-945794572996406,
     -1357135467359372+I*-945800025889050,
     -1357135467358898+I*-945800025895282,
     -1355028882183142+I*-962729308321440,
     -1355028879229174+I*-962729327628014,
     -1355028879229170+I*-962729327628036,
     -1355015099006844+I*-962788786822204,
     -1355015098989166+I*-962788786889808,
     -1297984097203534+I*-1144797723226034,
     -1297984026096738+I*-1144797929690412,
     -1297984026096650+I*-1144797929690646,
     -1297713525262728+I*-1145424420381842,
     -1297713524933062+I*-1145424421090130,
     -306354632618658+I*-3033460717217352,
     -306353437515146+I*-3033462846111918,
     -306353437513710+I*-3033462846114316,
     -302124946958176+I*-3039813242049628,
     -302124941915274+I*-3039813249181464,
     -302124941915268+I*-3039813249181472,
     -302107537987504+I*-3039834276814674,
     -302107537966912+I*-3039834276838180,
     -242893349985414+I*-3101231651015544,
     -242893280249820+I*-3101231719379936,
     -242893280249738+I*-3101231719380012,
     -242659781777930+I*-3101428215489706,
     -242659781504406+I*-3101428215707088,
     531491034646800+I*-3658370955695096,
     531491938467118+I*-3658371568232220,
     531491938468172+I*-3658371568232892,
     534453413877448+I*-3660066028495910,
     534453417320976+I*-3660066030340064,
     534453417320980+I*-3660066030340066,
     534464584418726+I*-3660070960607794,
     534464584431664+I*-3660070960613078,
     570429734459736+I*-3672787072687724,
     570429776026852+I*-3672787086086362,
     570429776026900+I*-3672787086086376,
     570562474637562+I*-3672818984835892,
     570562474790448+I*-3672818984868140,
     991611349248532+I*-3740765917633102,
     991611833284084+I*-3740765981883888,
     991611833284640+I*-3740765981883946,
     993135315500954+I*-3740851470907298 ],
   /* k=47 */ [ 993135317247196+I*-3740851470956254,
     993135317247199+I*-3740851470956254,
     5760566801816047+I*-3646169982454668,
     5760572254708691+I*-3646169721142210,
     5760572254714923+I*-3646169721141736,
     5777501537141081+I*-3644063135965980,
     5777501556447655+I*-3644063133012012,
     5777501556447677+I*-3644063133012008,
     5777561015641845+I*-3644049352789682,
     5777561015709449+I*-3644049352772004,
     5959569952045675+I*-3587018350986372,
     5959570158510053+I*-3587018279879576,
     5959570158510287+I*-3587018279879488,
     5960196649201483+I*-3586747779045566,
     5960196649909771+I*-3586747778715900,
     7848232946036993+I*-2595388886401496,
     7848235074931559+I*-2595387691297984,
     7848235074933957+I*-2595387691296548,
     7854585470869269+I*-2591159200741014,
     7854585478001105+I*-2591159195698112,
     7854585478001113+I*-2591159195698106,
     7854606505634315+I*-2591141791770342,
     7854606505657821+I*-2591141791749750,
     7916003879835185+I*-2531927603768252,
     7916003948199577+I*-2531927534032658,
     7916003948199653+I*-2531927534032576,
     7916200444309347+I*-2531694035560768,
     7916200444526729+I*-2531694035287244,
     8473143184514737+I*-1757543219136038,
     8473143797051861+I*-1757542315315720,
     8473143797052533+I*-1757542315314666,
     8474838257315551+I*-1754580839905390,
     8474838259159705+I*-1754580836461862,
     8474838259159707+I*-1754580836461858,
     13052906766725383+I*7874963621177424,
     13052911696993111+I*7874974788275170,
     13052911696998395+I*7874974788288108,
     13065627809073041+I*7910939938316180,
     13065627822471679+I*7910939979883296,
     13065627822471693+I*7910939979883344,
     13065659721221209+I*7911072678494006,
     13065659721253457+I*7911072678646892,
     13133606654018419+I*8332121553104976,
     13133606718269205+I*8332122037140528,
     13133606718269263+I*8332122037141084,
     13133692207292615+I*8333645519357398,
     13133692207341571+I*8333645521103640,
     13133692207341571+I*8333645521103643,
     13039010718839985+I*13101077005672491,
     13039010457527527+I*13101082458565135,
     13039010457527053+I*13101082458571367,
     13036903872351297+I*13118011740997525,
     13036903869397329+I*13118011760304099,
     13036903869397325+I*13118011760304121,
     13036890089174999+I*13118071219498289,
     13036890089157321+I*13118071219565893,
     12979859087371689+I*13300080155902119,
     12979859016264893+I*13300080362366497,
     12979859016264805+I*13300080362366731,
     12979588515430883+I*13300706853057927,
     12979588515101217+I*13300706853766215,
     11988229622786813+I*15188743149893437,
     11988228427683301+I*15188745278788003,
     11988228427681865+I*15188745278790401,
     11983999937126331+I*15195095674725713,
     11983999932083429+I*15195095681857549,
     11983999932083423+I*15195095681857557,
     11983982528155659+I*15195116709490759,
     11983982528135067+I*15195116709514265,
     11924768340153569+I*15256514083691629,
     11924768270417975+I*15256514152056021,
     11924768270417893+I*15256514152056097,
     11924534771946085+I*15256710648165791,
     -5045519889549641+I*29227619892116784,
     -5045519889823165+I*29227619892334166,
     -5819670705974371+I*29784562632322174,
     -5819671609794689+I*29784563244859298,
     -5819671609795743+I*29784563244859970,
     -5822633085205019+I*29786257705122988,
     -5822633088648547+I*29786257706967142,
     -5822633088648551+I*29786257706967144,
     -5822644255746297+I*29786262637234872,
     -5822644255759235+I*29786262637240156,
     -5858609405787307+I*29798978749314802,
     -5858609447354423+I*29798978762713440,
     -5858609447354471+I*29798978762713454,
     -5858742145965133+I*29799010661462970,
     -5858742146118019+I*29799010661495218,
     -6279791020576103+I*29866957594260180,
     -6279791504611655+I*29866957658510966,
     -6279791504612211+I*29866957658511024,
     -6281314986828525+I*29867043147534376,
     -6281314988574767+I*29867043147583332,
     -6281314988574770+I*29867043147583332,
     -11048746473143618+I*29772361659081746,
     -11048751926036262+I*29772361397769288,
     -11048751926042494+I*29772361397768814,
     -11065681208468652+I*29770254812593058,
     -11065681227775226+I*29770254809639090,
     -11065681227775248+I*29770254809639086,
     -11065740686969416+I*29770241029416760,
     -11065740687037020+I*29770241029399082,
     -11247749623373246+I*29713210027613450,
     -11247749829837624+I*29713209956506654,
     -11247749829837858+I*29713209956506566,
     -11248376320529054+I*29712939455672644,
     -11248376321237342+I*29712939455342978,
     -13136412617364564+I*28721580563028574,
     -13136414746259130+I*28721579367925062,
     -13136414746261528+I*28721579367923626,
     -13142765142196840+I*28717350877368092,
     -13142765149328676+I*28717350872325190,
     -13142765149328684+I*28717350872325184,
     -13142786176961886+I*28717333468397420,
     -13142786176985392+I*28717333468376828,
     -13204183551162756+I*28658119280395330,
     -13204183619527148+I*28658119210659736,
     -13204183619527224+I*28658119210659654,
     -13204380115636918+I*28657885712187846,
     -13204380115854300+I*28657885711914322,
     -13761322855842308+I*27883734895763116,
     -13761323468379432+I*27883733991942798,
     -13761323468380104+I*27883733991941744,
     -13763017928643122+I*27880772516532468,
     -13763017930487276+I*27880772513088940,
     -13763017930487278+I*27880772513088936,
     -18341086438052954+I*18251228055449654,
     -18341091368320682+I*18251216888351908,
     -18341091368325966+I*18251216888338970,
     -18353807480400612+I*18215251738310898,
     -18353807493799250+I*18215251696743782,
     -18353807493799264+I*18215251696743734,
     -18353839392548780+I*18215118998133072,
     -18353839392581028+I*18215118997980186,
     -18421786325345990+I*17794070123522102,
     -18421786389596776+I*17794069639486550,
     -18421786389596834+I*17794069639485994,
     -18421871878620186+I*17792546157269680,
     -18421871878669142+I*17792546155523438,
     -18421871878669142+I*17792546155523435,
     -18327190390167556+I*13025114670954587,
     -18327190128855098+I*13025109218061943,
     -18327190128854624+I*13025109218055711,
     -18325083543678868+I*13008179935629553,
     -18325083540724900+I*13008179916322979,
     -18325083540724896+I*13008179916322957,
     -18325069760502570+I*13008120457128789,
     -18325069760484892+I*13008120457061185,
     -18268038758699260+I*12826111520724959,
     -18268038687592464+I*12826111314260581,
     -18268038687592376+I*12826111314260347,
     -18267768186758454+I*12825484823569151,
     -18267768186428788+I*12825484822860863,
     -17276409294114384+I*10937448526733641,
     -17276408099010872+I*10937446397839075,
     -17276408099009436+I*10937446397836677,
     -17272179608453902+I*10931096001901365,
     -17272179603411000+I*10931095994769529,
     -17272179603410994+I*10931095994769521,
     -17272162199483230+I*10931074967136319,
     -17272162199462638+I*10931074967112813,
     -17212948011481140+I*10869677592935449,
     -17212947941745546+I*10869677524571057,
     -17212947941745464+I*10869677524570981,
     -17212714443273656+I*10869481028461287,
     -242659781777930+I*-3101428215489706,
     -242659781504406+I*-3101428215707088,
     531491034646800+I*-3658370955695096,
     531491938467118+I*-3658371568232220,
     531491938468172+I*-3658371568232892,
     534453413877448+I*-3660066028495910,
     534453417320976+I*-3660066030340064,
     534453417320980+I*-3660066030340066,
     534464584418726+I*-3660070960607794,
     534464584431664+I*-3660070960613078,
     570429734459736+I*-3672787072687724,
     570429776026852+I*-3672787086086362,
     570429776026900+I*-3672787086086376,
     570562474637562+I*-3672818984835892,
     570562474790448+I*-3672818984868140,
     991611349248532+I*-3740765917633102,
     991611833284084+I*-3740765981883888,
     991611833284640+I*-3740765981883946,
     993135315500954+I*-3740851470907298 ],
   /* k=48 */ [ -43918737832150516+I*-23710051549996713,
     -43918737832150513+I*-23710051549996713,
     -39151306347581665+I*-23615370061495127,
     -39151300894689021+I*-23615369800182669,
     -39151300894682789+I*-23615369800182195,
     -39134371612256631+I*-23613263215006439,
     -39134371592950057+I*-23613263212052471,
     -39134371592950035+I*-23613263212052467,
     -39134312133755867+I*-23613249431830141,
     -39134312133688263+I*-23613249431812463,
     -38952303197352037+I*-23556218430026831,
     -38952302990887659+I*-23556218358920035,
     -38952302990887425+I*-23556218358919947,
     -38951676500196229+I*-23555947858086025,
     5960196649201483+I*-3586747779045566,
     5960196649909771+I*-3586747778715900,
     7848232946036993+I*-2595388886401496,
     7848235074931559+I*-2595387691297984,
     7848235074933957+I*-2595387691296548,
     7854585470869269+I*-2591159200741014,
     7854585478001105+I*-2591159195698112,
     7854585478001113+I*-2591159195698106,
     7854606505634315+I*-2591141791770342,
     7854606505657821+I*-2591141791749750,
     7916003879835185+I*-2531927603768252,
     7916003948199577+I*-2531927534032658,
     7916003948199653+I*-2531927534032576,
     7916200444309347+I*-2531694035560768,
     7916200444526729+I*-2531694035287244,
     8473143184514737+I*-1757543219136038,
     8473143797051861+I*-1757542315315720,
     8473143797052533+I*-1757542315314666,
     8474838257315551+I*-1754580839905390,
     8474838259159705+I*-1754580836461862,
     8474838259159707+I*-1754580836461858,
     13052906766725383+I*7874963621177424,
     13052911696993111+I*7874974788275170,
     13052911696998395+I*7874974788288108,
     13065627809073041+I*7910939938316180,
     13065627822471679+I*7910939979883296,
     13065627822471693+I*7910939979883344,
     13065659721221209+I*7911072678494006,
     13065659721253457+I*7911072678646892,
     13133606654018419+I*8332121553104976,
     13133606718269205+I*8332122037140528,
     13133606718269263+I*8332122037141084,
     13133692207292615+I*8333645519357398,
     13133692207341571+I*8333645521103640,
     13133692207341571+I*8333645521103643,
     13039010718839985+I*13101077005672491,
     13039010457527527+I*13101082458565135,
     13039010457527053+I*13101082458571367,
     13036903872351297+I*13118011740997525,
     13036903869397329+I*13118011760304099,
     13036903869397325+I*13118011760304121,
     13036890089174999+I*13118071219498289,
     13036890089157321+I*13118071219565893,
     12979859087371689+I*13300080155902119,
     12979859016264893+I*13300080362366497,
     12979859016264805+I*13300080362366731,
     12979588515430883+I*13300706853057927,
     12979588515101217+I*13300706853766215,
     11988229622786813+I*15188743149893437,
     11988228427683301+I*15188745278788003,
     11988228427681865+I*15188745278790401,
     11983999937126331+I*15195095674725713,
     11983999932083429+I*15195095681857549,
     11983999932083423+I*15195095681857557,
     -2697020475629465+I*33980777154628191,
     -2697037879557229+I*33980798182261393,
     -2697037879577821+I*33980798182284899,
     -2756252067559319+I*34042195556462263,
     -2756252137294913+I*34042195624826655,
     -2756252137294995+I*34042195624826731,
     -2756485635766803+I*34042392120936425,
     -2756485636040327+I*34042392121153807,
     -3530636452191533+I*34599334861141815,
     -3530637356011851+I*34599335473678939,
     -3530637356012905+I*34599335473679611,
     -3533598831422181+I*34601029933942629,
     -3533598834865709+I*34601029935786783,
     -3533598834865713+I*34601029935786785,
     -13163143292504995+I*39179098443352461,
     -13163154459602741+I*39179103373620189,
     -13163154459615679+I*39179103373625473,
     -13199119609643751+I*39191819485700119,
     -13199119651210867+I*39191819499098757,
     -13199119651210915+I*39191819499098771,
     -13199252349821577+I*39191851397848287,
     -13199252349974463+I*39191851397880535,
     -13620301224432547+I*39259798330645497,
     -13620301708468099+I*39259798394896283,
     -13620301708468655+I*39259798394896341,
     -13621825190684969+I*39259883883919693,
     -13621825192431211+I*39259883883968649,
     -13621825192431214+I*39259883883968649,
     -18389256677000062+I*39165202395467063,
     -18389262129892706+I*39165202134154605,
     -18389262129898938+I*39165202134154131,
     -18406191412325096+I*39163095548978375,
     -18406191431631670+I*39163095546024407,
     -18406191431631692+I*39163095546024403,
     -18406250890825860+I*39163081765802077,
     -18406250890893464+I*39163081765784399,
     -18588259827229690+I*39106050763998767,
     -18588260033694068+I*39106050692891971,
     -18588260033694302+I*39106050692891883,
     -18588886524385498+I*39105780192057961,
     -63500759673783210+I*19136580113017502,
     -63500759674491498+I*19136580112687836,
     -65388795970618720+I*18145221220373432,
     -65388798099513286+I*18145220025269920,
     -65388798099515684+I*18145220025268484,
     -65395148495450996+I*18140991534712950,
     -65395148502582832+I*18140991529670048,
     -65395148502582840+I*18140991529670042,
     -65395169530216042+I*18140974125742278,
     -65395169530239548+I*18140974125721686,
     -65456566904416912+I*18081759937740188,
     -65456566972781304+I*18081759868004594,
     -65456566972781380+I*18081759868004512,
     -65456763468891074+I*18081526369532704,
     -65456763469108456+I*18081526369259180,
     -66013706209096464+I*17307375553107974,
     -66013706821633588+I*17307374649287656,
     -66013706821634260+I*17307374649286602,
     -66015401281897278+I*17304413173877326,
     -66015401283741432+I*17304413170433798,
     -66015401283741434+I*17304413170433794,
     -70593469791307110+I*7674868712794512,
     -70593474721574838+I*7674857545696766,
     -70593474721580122+I*7674857545683828,
     -70606190833654768+I*7638892395655756,
     -70606190847053406+I*7638892354088640,
     -70606190847053420+I*7638892354088592,
     -70606222745802936+I*7638759655477930,
     -70606222745835184+I*7638759655325044,
     -70674169678600146+I*7217710780866960,
     -70674169742850932+I*7217710296831408,
     -70674169742850990+I*7217710296830852,
     -70674255231874342+I*7216186814614538,
     -70674255231923298+I*7216186812868296,
     -70674255231923298+I*7216186812868293,
     -70579573743421712+I*2448755328299445,
     -70579573482109254+I*2448749875406801,
     -70579573482108780+I*2448749875400569,
     -70577466896933024+I*2431820592974411,
     -70577466893979056+I*2431820573667837,
     -70577466893979052+I*2431820573667815,
     -70577453113756726+I*2431761114473647,
     -70577453113739048+I*2431761114406043,
     -70520422111953416+I*2249752178069817,
     -70520422040846620+I*2249751971605439,
     -70520422040846532+I*2249751971605205,
     -70520151540012610+I*2249125480914009,
     -70520151539682944+I*2249125480205721,
     -69528792647368540+I*361089184078499,
     -69528791452265028+I*361087055183933,
     -69528791452263592+I*361087055181535,
     -69524562961708058+I*354736659246223,
     -69524562956665156+I*354736652114387,
     -69524562956665150+I*354736652114379,
     -54843542548952262+I*-18430944820656255,
     -54843525145024498+I*-18430965848289457,
     -54843525145003906+I*-18430965848312963,
     -54784310957022408+I*-18492363222490327,
     -54784310887286814+I*-18492363290854719,
     -54784310887286732+I*-18492363290854795,
     -54784077388814924+I*-18492559786964489,
     -54784077388541400+I*-18492559787181871,
     -54009926572390194+I*-19049502527169879,
     -54009925668569876+I*-19049503139707003,
     -54009925668568822+I*-19049503139707675,
     -54006964193159546+I*-19051197599970693,
     -54006964189716018+I*-19051197601814847,
     -54006964189716014+I*-19051197601814849,
     -44377419732076732+I*-23629266109380525,
     -44377408564978986+I*-23629271039648253,
     -44377408564966048+I*-23629271039653537,
     -44341443414937976+I*-23641987151728183,
     -44341443373370860+I*-23641987165126821,
     -44341443373370812+I*-23641987165126835,
     -44341310674760150+I*-23642019063876351,
     -44341310674607264+I*-23642019063908599,
     -43920261800149180+I*-23709965996673561,
     -43920261316113628+I*-23709966060924347,
     -43920261316113072+I*-23709966060924405,
     -43918737833896758+I*-23710051549947757 ],
   /* k=49 */ [ -75018402500094388+I*-138791177599160167,
     -75018402500094385+I*-138791177599160167,
     -70250971015525537+I*-138696496110658581,
     -70250965562632893+I*-138696495849346123,
     -70250965562626661+I*-138696495849345649,
     -70234036280200503+I*-138694389264169893,
     -70234036260893929+I*-138694389261215925,
     -70234036260893907+I*-138694389261215921,
     -17981652907639751+I*-128118029918560779,
     -17981593448445583+I*-128118016138338453,
     -17981593448377979+I*-128118016138320775,
     -17799584512041753+I*-128060985136535143,
     -17799584305577375+I*-128060985065428347,
     -17799584305577141+I*-128060985065428259,
     -17798957814885945+I*-128060714564594337,
     -17798957814177657+I*-128060714564264671,
     -15910921518050435+I*-127069355671950267,
     -15910919389155869+I*-127069354476846755,
     -15910919389153471+I*-127069354476845319,
     -15904568993218159+I*-127065125986289785,
     -15904568986086323+I*-127065125981246883,
     -15904568986086315+I*-127065125981246877,
     2881112486684319+I*-112384105573533989,
     2881133514317521+I*-112384088169606225,
     2881133514341027+I*-112384088169585633,
     2942530888518391+I*-112324873981604135,
     2942530956882783+I*-112324873911868541,
     2942530956882859+I*-112324873911868459,
     2942727452992553+I*-112324640413396651,
     2942727453209935+I*-112324640413123127,
     3499670193197943+I*-111550489596971921,
     3499670805735067+I*-111550488693151603,
     3499670805735739+I*-111550488693150549,
     3501365265998757+I*-111547527217741273,
     3501365267842911+I*-111547527214297745,
     3501365267842913+I*-111547527214297741,
     8079433775408589+I*-101917982756658459,
     8079438705676317+I*-101917971589560713,
     8079438705681601+I*-101917971589547775,
     8092154817756247+I*-101882006439519703,
     8092154831154885+I*-101882006397952587,
     8092154831154899+I*-101882006397952539,
     8092186729904415+I*-101881873699341877,
     8092186729936663+I*-101881873699188991,
     8160133662701625+I*-101460824824730907,
     8160133726952411+I*-101460824340695355,
     8160133726952469+I*-101460824340694799,
     8160219215975821+I*-101459300858478485,
     13133692207292615+I*8333645519357398,
     13133692207341571+I*8333645521103640,
     13133692207341571+I*8333645521103643,
     13039010718839985+I*13101077005672491,
     13039010457527527+I*13101082458565135,
     13039010457527053+I*13101082458571367,
     13036903872351297+I*13118011740997525,
     13036903869397329+I*13118011760304099,
     13036903869397325+I*13118011760304121,
     13036890089174999+I*13118071219498289,
     13036890089157321+I*13118071219565893,
     12979859087371689+I*13300080155902119,
     12979859016264893+I*13300080362366497,
     12979859016264805+I*13300080362366731,
     12979588515430883+I*13300706853057927,
     12979588515101217+I*13300706853766215,
     11988229622786813+I*15188743149893437,
     11988228427683301+I*15188745278788003,
     11988228427681865+I*15188745278790401,
     11983999937126331+I*15195095674725713,
     11983999932083429+I*15195095681857549,
     11983999932083423+I*15195095681857557,
     -2697020475629465+I*33980777154628191,
     -2697037879557229+I*33980798182261393,
     -2697037879577821+I*33980798182284899,
     -2756252067559319+I*34042195556462263,
     -2756252137294913+I*34042195624826655,
     -2756252137294995+I*34042195624826731,
     -2756485635766803+I*34042392120936425,
     -2756485636040327+I*34042392121153807,
     -3530636452191533+I*34599334861141815,
     -3530637356011851+I*34599335473678939,
     -3530637356012905+I*34599335473679611,
     -3533598831422181+I*34601029933942629,
     -3533598834865709+I*34601029935786783,
     -3533598834865713+I*34601029935786785,
     -13163143292504995+I*39179098443352461,
     -13163154459602741+I*39179103373620189,
     -13163154459615679+I*39179103373625473,
     -13199119609643751+I*39191819485700119,
     -13199119651210867+I*39191819499098757,
     -13199119651210915+I*39191819499098771,
     -13199252349821577+I*39191851397848287,
     -13199252349974463+I*39191851397880535,
     -13620301224432547+I*39259798330645497,
     -13620301708468099+I*39259798394896283,
     -13620301708468655+I*39259798394896341,
     -13621825190684969+I*39259883883919693,
     -13621825192431211+I*39259883883968649,
     -13621825192431214+I*39259883883968649,
     -18389256677000062+I*39165202395467063,
     -18389262129892706+I*39165202134154605,
     -18389262129898938+I*39165202134154131,
     -18406191412325096+I*39163095548978375,
     -18406191431631670+I*39163095546024407,
     -18406191431631692+I*39163095546024403,
     -70658574784885848+I*28586736203369261,
     -70658634244080016+I*28586722423146935,
     -70658634244147620+I*28586722423129257,
     -70840643180483846+I*28529691421343625,
     -70840643386948224+I*28529691350236829,
     -70840643386948458+I*28529691350236741,
     -70841269877639654+I*28529420849402819,
     -70841269878347942+I*28529420849073153,
     -72729306174475164+I*27538061956758749,
     -72729308303369730+I*27538060761655237,
     -72729308303372128+I*27538060761653801,
     -72735658699307440+I*27533832271098267,
     -72735658706439276+I*27533832266055365,
     -72735658706439284+I*27533832266055359,
     -91521340179209918+I*12852811858342471,
     -91521361206843120+I*12852794454414707,
     -91521361206866626+I*12852794454394115,
     -91582758581043990+I*12793580266412617,
     -91582758649408382+I*12793580196677023,
     -91582758649408458+I*12793580196676941,
     -91582955145518152+I*12793346698205133,
     -91582955145735534+I*12793346697931609,
     -92139897885723542+I*12019195881780403,
     -92139898498260666+I*12019194977960085,
     -92139898498261338+I*12019194977959031,
     -92141592958524356+I*12016233502549755,
     -92141592960368510+I*12016233499106227,
     -92141592960368512+I*12016233499106223,
     -96719661467934188+I*2386689041466941,
     -96719666398201916+I*2386677874369195,
     -96719666398207200+I*2386677874356257,
     -96732382510281846+I*2350712724328185,
     -96732382523680484+I*2350712682761069,
     -96732382523680498+I*2350712682761021,
     -96732414422430014+I*2350579984150359,
     -96732414422462262+I*2350579983997473,
     -96800361355227224+I*1929531109539389,
     -96800361419478010+I*1929530625503837,
     -96800361419478068+I*1929530625503281,
     -96800446908501420+I*1928007143286967,
     -101773919899818214+I*-107864939234548916,
     -101773919899867170+I*-107864939236295158,
     -101773919899867170+I*-107864939236295161,
     -101679238411365584+I*-112632370720864009,
     -101679238150053126+I*-112632376173756653,
     -101679238150052652+I*-112632376173762885,
     -101677131564876896+I*-112649305456189043,
     -101677131561922928+I*-112649305475495617,
     -101677131561922924+I*-112649305475495639,
     -101677117781700598+I*-112649364934689807,
     -101677117781682920+I*-112649364934757411,
     -101620086779897288+I*-112831373871093637,
     -101620086708790492+I*-112831374077558015,
     -101620086708790404+I*-112831374077558249,
     -101619816207956482+I*-112832000568249445,
     -101619816207626816+I*-112832000568957733,
     -100628457315312412+I*-114720036865084955,
     -100628456120208900+I*-114720038993979521,
     -100628456120207464+I*-114720038993981919,
     -100624227629651930+I*-114726389389917231,
     -100624227624609028+I*-114726389397049067,
     -100624227624609022+I*-114726389397049075,
     -85943207216896134+I*-133512070869819709,
     -85943189812968370+I*-133512091897452911,
     -85943189812947778+I*-133512091897476417,
     -85883975624966280+I*-133573489271653781,
     -85883975555230686+I*-133573489340018173,
     -85883975555230604+I*-133573489340018249,
     -85883742056758796+I*-133573685836127943,
     -85883742056485272+I*-133573685836345325,
     -85109591240334066+I*-134130628576333333,
     -85109590336513748+I*-134130629188870457,
     -85109590336512694+I*-134130629188871129,
     -85106628861103418+I*-134132323649134147,
     -85106628857659890+I*-134132323650978301,
     -85106628857659886+I*-134132323650978303,
     -75477084400020604+I*-138710392158543979,
     -75477073232922858+I*-138710397088811707,
     -75477073232909920+I*-138710397088816991,
     -75441108082881848+I*-138723113200891637,
     -75441108041314732+I*-138723113214290275,
     -75441108041314684+I*-138723113214290289,
     -75440975342704022+I*-138723145113039805,
     -75440975342551136+I*-138723145113072053,
     -75019926468093052+I*-138791092045837015,
     -75019925984057500+I*-138791092110087801,
     -75019925984056944+I*-138791092110087859,
     -75018402501840630+I*-138791177599111211 ],
   /* k=50 */ [ 124044184930288648+I*-316071632984211365,
     124044184930288651+I*-316071632984211365,
     128811616414857499+I*-315976951495709779,
     128811621867750143+I*-315976951234397321,
     128811621867756375+I*-315976951234396847,
     128828551150182533+I*-315974844649221091,
     128828551169489107+I*-315974844646267123,
     128828551169489129+I*-315974844646267119,
     181080934522743285+I*-305398485303611977,
     181080993981937453+I*-305398471523389651,
     181080993982005057+I*-305398471523371973,
     181263002918341283+I*-305341440521586341,
     181263003124805661+I*-305341440450479545,
     181263003124805895+I*-305341440450479457,
     181263629615497091+I*-305341169949645535,
     181263629616205379+I*-305341169949315869,
     183151665912332601+I*-304349811057001465,
     183151668041227167+I*-304349809861897953,
     183151668041229565+I*-304349809861896517,
     183158018437164877+I*-304345581371340983,
     183158018444296713+I*-304345581366298081,
     183158018444296721+I*-304345581366298075,
     201943699917067355+I*-289664560958585187,
     201943720944700557+I*-289664543554657423,
     201943720944724063+I*-289664543554636831,
     202005118318901427+I*-289605329366655333,
     202005118387265819+I*-289605329296919739,
     202005118387265895+I*-289605329296919657,
     202005314883375589+I*-289605095798447849,
     202005314883592971+I*-289605095798174325,
     202562257623580979+I*-288830944982023119,
     202562258236118103+I*-288830944078202801,
     202562258236118775+I*-288830944078201747,
     202563952696381793+I*-288827982602792471,
     202563952698225947+I*-288827982599348943,
     202563952698225949+I*-288827982599348939,
     207142021205791625+I*-279198438141709657,
     207142026136059353+I*-279198426974611911,
     207142026136064637+I*-279198426974598973,
     207154742248139283+I*-279162461824570901,
     207154742261537921+I*-279162461783003785,
     207154742261537935+I*-279162461783003737,
     238254406929481807+I*-164081335733840283,
     238254438828231323+I*-164081203035229621,
     238254438828263571+I*-164081203035076735,
     238322385761028533+I*-163660154160618651,
     238322385825279319+I*-163660153676583099,
     238322385825279377+I*-163660153676582543,
     238322471314302729+I*-163658630194366229,
     238322471314351685+I*-163658630192619987,
     238322471314351685+I*-163658630192619984,
     238227789825850099+I*-158891198708051136,
     238227789564537641+I*-158891193255158492,
     238227789564537167+I*-158891193255152260,
     238225682979361411+I*-158874263972726102,
     238225682976407443+I*-158874263953419528,
     238225682976407439+I*-158874263953419506,
     227649323633752297+I*-106621880600165350,
     227649309853529971+I*-106621821140971182,
     227649309853512293+I*-106621821140903578,
     227592278851726661+I*-106439812204567352,
     227592278780619865+I*-106439811998102974,
     227592278780619777+I*-106439811998102740,
     227592008279785855+I*-106439185507411544,
     227592008279456189+I*-106439185506703256,
     226600649387141785+I*-104551149210576034,
     226600648192038273+I*-104551147081681468,
     226600648192036837+I*-104551147081679070,
     226596419701481303+I*-104544796685743758,
     226596419696438401+I*-104544796678611922,
     226596419696438395+I*-104544796678611914,
     211915399288725507+I*-85759115205841280,
     211915381884797743+I*-85759094178208078,
     211915381884777151+I*-85759094178184572,
     211856167696795653+I*-85697696804007208,
     211856167627060059+I*-85697696735642816,
     211856167627059977+I*-85697696735642740,
     211855934128588169+I*-85697500239533046,
     211855934128314645+I*-85697500239315664,
     211081783312163439+I*-85140557499327656,
     211081782408343121+I*-85140556886790532,
     211081782408342067+I*-85140556886789860,
     211078820932932791+I*-85138862426526842,
     -3533598831422181+I*34601029933942629,
     -3533598834865709+I*34601029935786783,
     -3533598834865713+I*34601029935786785,
     -13163143292504995+I*39179098443352461,
     -13163154459602741+I*39179103373620189,
     -13163154459615679+I*39179103373625473,
     -13199119609643751+I*39191819485700119,
     -13199119651210867+I*39191819499098757,
     -13199119651210915+I*39191819499098771,
     -13199252349821577+I*39191851397848287,
     -13199252349974463+I*39191851397880535,
     -13620301224432547+I*39259798330645497,
     -13620301708468099+I*39259798394896283,
     -13620301708468655+I*39259798394896341,
     -13621825190684969+I*39259883883919693,
     -13621825192431211+I*39259883883968649,
     -13621825192431214+I*39259883883968649,
     -18389256677000062+I*39165202395467063,
     -18389262129892706+I*39165202134154605,
     -18389262129898938+I*39165202134154131,
     -18406191412325096+I*39163095548978375,
     -18406191431631670+I*39163095546024407,
     -18406191431631692+I*39163095546024403,
     -70658574784885848+I*28586736203369261,
     -70658634244080016+I*28586722423146935,
     -70658634244147620+I*28586722423129257,
     -70840643180483846+I*28529691421343625,
     -70840643386948224+I*28529691350236829,
     -70840643386948458+I*28529691350236741,
     -70841269877639654+I*28529420849402819,
     -70841269878347942+I*28529420849073153,
     -72729306174475164+I*27538061956758749,
     -72729308303369730+I*27538060761655237,
     -72729308303372128+I*27538060761653801,
     -72735658699307440+I*27533832271098267,
     -72735658706439276+I*27533832266055365,
     -72735658706439284+I*27533832266055359,
     -91521340179209918+I*12852811858342471,
     -91521361206843120+I*12852794454414707,
     -91521361206866626+I*12852794454394115,
     -91582758581043990+I*12793580266412617,
     -91582758649408382+I*12793580196677023,
     -91582758649408458+I*12793580196676941,
     -91582955145518152+I*12793346698205133,
     -91582955145735534+I*12793346697931609,
     -92139897885723542+I*12019195881780403,
     -92139898498260666+I*12019194977960085,
     -92139898498261338+I*12019194977959031,
     -92141592958524356+I*12016233502549755,
     -92141592960368510+I*12016233499106227,
     -92141592960368512+I*12016233499106223,
     -96719661467934188+I*2386689041466941,
     -96719666398201916+I*2386677874369195,
     -96719666398207200+I*2386677874356257,
     -96732382510281846+I*2350712724328185,
     -96732382523680484+I*2350712682761069,
     -96732382523680498+I*2350712682761021,
     -127832047191624370+I*-112730413366402433,
     -127832079090373886+I*-112730546065013095,
     -127832079090406134+I*-112730546065165981,
     -127900026023171096+I*-113151594939624065,
     -127900026087421882+I*-113151595423659617,
     -127900026087421940+I*-113151595423660173,
     -127900111576445292+I*-113153118905876487,
     -127900111576494248+I*-113153118907622729,
     -127900111576494248+I*-113153118907622732,
     -127805430087992662+I*-117920550392191580,
     -127805429826680204+I*-117920555845084224,
     -127805429826679730+I*-117920555845090456,
     -127803323241503974+I*-117937485127516614,
     -127803323238550006+I*-117937485146823188,
     -127803323238550002+I*-117937485146823210,
     -117226963895894860+I*-170189868500077366,
     -117226950115672534+I*-170189927959271534,
     -117226950115654856+I*-170189927959339138,
     -117169919113869224+I*-170371936895675364,
     -117169919042762428+I*-170371937102139742,
     -117169919042762340+I*-170371937102139976,
     -117169648541928418+I*-170372563592831172,
     -117169648541598752+I*-170372563593539460,
     -116178289649284348+I*-172260599889666682,
     -116178288454180836+I*-172260602018561248,
     -116178288454179400+I*-172260602018563646,
     -116174059963623866+I*-172266952414498958,
     -116174059958580964+I*-172266952421630794,
     -116174059958580958+I*-172266952421630802,
     -101493039550868070+I*-191052633894401436,
     -101493022146940306+I*-191052654922034638,
     -101493022146919714+I*-191052654922058144,
     -101433807958938216+I*-191114052296235508,
     -101433807889202622+I*-191114052364599900,
     -101433807889202540+I*-191114052364599976,
     -101433574390730732+I*-191114248860709670,
     -101433574390457208+I*-191114248860927052,
     -100659423574306002+I*-191671191600915060,
     -100659422670485684+I*-191671192213452184,
     -100659422670484630+I*-191671192213452856,
     -100656461195075354+I*-191672886673715874,
     113955958569279618+I*-311412779034185345,
     113955958572723146+I*-311412779036029499,
     113955958572723150+I*-311412779036029501,
     123585503030362432+I*-315990847543595177,
     123585514197460178+I*-315990852473862905,
     123585514197473116+I*-315990852473868189,
     123621479347501188+I*-316003568585942835,
     123621479389068304+I*-316003568599341473,
     123621479389068352+I*-316003568599341487,
     123621612087679014+I*-316003600498091003,
     123621612087831900+I*-316003600498123251,
     124042660962289984+I*-316071547430888213,
     124042661446325536+I*-316071547495138999,
     124042661446326092+I*-316071547495139057,
     124044184928542406+I*-316071632984162409 ],
   /* k=51 */ [ 223575478645480166+I*-404711860676736964,
     223575478645480169+I*-404711860676736964,
     228342910130049017+I*-404617179188235378,
     228342915582941661+I*-404617178926922920,
     228342915582947893+I*-404617178926922446,
     228359844865374051+I*-404615072341746690,
     228359844884680625+I*-404615072338792722,
     228359844884680647+I*-404615072338792718,
     280612228237934803+I*-394038712996137576,
     280612287697128971+I*-394038699215915250,
     280612287697196575+I*-394038699215897572,
     280794296633532801+I*-393981668214111940,
     280794296839997179+I*-393981668143005144,
     280794296839997413+I*-393981668143005056,
     280794923330688609+I*-393981397642171134,
     280794923331396897+I*-393981397641841468,
     282682959627524119+I*-392990038749527064,
     282682961756418685+I*-392990037554423552,
     282682961756421083+I*-392990037554422116,
     282689312152356395+I*-392985809063866582,
     736781516637650309+I*-83500861895626109,
     736781516644782145+I*-83500861890583207,
     736781516644782153+I*-83500861890583201,
     755567198117552787+I*-68819841482870313,
     755567219145185989+I*-68819824078942549,
     755567219145209495+I*-68819824078921957,
     755628616519386859+I*-68760609890940459,
     755628616587751251+I*-68760609821204865,
     755628616587751327+I*-68760609821204783,
     755628813083861021+I*-68760376322732975,
     755628813084078403+I*-68760376322459451,
     756185755824066411+I*-67986225506308245,
     756185756436603535+I*-67986224602487927,
     756185756436604207+I*-67986224602486873,
     756187450896867225+I*-67983263127077597,
     756187450898711379+I*-67983263123634069,
     756187450898711381+I*-67983263123634065,
     760765519406277057+I*-58353718665994783,
     760765524336544785+I*-58353707498897037,
     760765524336550069+I*-58353707498884099,
     760778240448624715+I*-58317742348856027,
     760778240462023353+I*-58317742307288911,
     760778240462023367+I*-58317742307288863,
     791877905129967239+I*56763383741874591,
     791877937028716755+I*56763516440485253,
     791877937028749003+I*56763516440638139,
     791945883961513965+I*57184565315096223,
     791945884025764751+I*57184565799131775,
     791945884025764809+I*57184565799132331,
     791945969514788161+I*57186089281348645,
     791945969514837117+I*57186089283094887,
     791945969514837117+I*57186089283094890,
     791851288026335531+I*61953520767663738,
     791851287765023073+I*61953526220556382,
     791851287765022599+I*61953526220562614,
     791849181179846843+I*61970455502988772,
     791849181176892875+I*61970455522295346,
     791849181176892871+I*61970455522295368,
     781272821834237729+I*114222838875549524,
     781272808054015403+I*114222898334743692,
     781272808053997725+I*114222898334811296,
     781215777052212093+I*114404907271147522,
     781215776981105297+I*114404907477611900,
     781215776981105209+I*114404907477612134,
     781215506480271287+I*114405533968303330,
     781215506479941621+I*114405533969011618,
     780224147587627217+I*116293570265138840,
     780224146392523705+I*116293572394033406,
     780224146392522269+I*116293572394035804,
     780219917901966735+I*116299922789971116,
     780219917896923833+I*116299922797102952,
     780219917896923827+I*116299922797102960,
     765538897489210939+I*135085604269873594,
     765538880085283175+I*135085625297506796,
     765538880085262583+I*135085625297530302,
     765479665897281085+I*135147022671707666,
     765479665827545491+I*135147022740072058,
     765479665827545409+I*135147022740072134,
     566417078397162373+I*312427478125123332,
     566416844898690565+I*312427674621233026,
     566416844898417041+I*312427674621450408,
     565642694082265835+I*312984617361438416,
     565642693178445517+I*312984617973975540,
     565642693178444463+I*312984617973976212,
     565639731703035187+I*312986312434239230,
     565639731699591659+I*312986312436083384,
     565639731699591655+I*312986312436083386,
     556010187241952373+I*317564380943649062,
     556010176074854627+I*317564385873916790,
     556010176074841689+I*317564385873922074,
     555974210924813617+I*317577101985996720,
     555974210883246501+I*317577101999395358,
     555974210883246453+I*317577101999395372,
     440893084834082999+I*348676766667339244,
     440892952135472337+I*348676798566088760,
     440892952135319451+I*348676798566121008,
     440471903260861367+I*348744745498885970,
     440471902776825815+I*348744745563136756,
     440471902776825259+I*348744745563136814,
     440470379294608945+I*348744831052160166,
     440470379292862703+I*348744831052209122,
     440470379292862700+I*348744831052209122,
     435702947808293852+I*348650149563707536,
     435702942355401208+I*348650149302395078,
     435702942355394976+I*348650149302394604,
     435686013072968818+I*348648042717218848,
     435686013053662244+I*348648042714264880,
     435686013053662222+I*348648042714264876,
     383433629700408066+I*338071683371609734,
     383433570241213898+I*338071669591387408,
     383433570241146294+I*338071669591369730,
     383251561304810068+I*338014638589584098,
     383251561098345690+I*338014638518477302,
     383251561098345456+I*338014638518477214,
     383250934607654260+I*338014368017643292,
     383250934606945972+I*338014368017313626,
     381362898310818750+I*337023009124999222,
     381362896181924184+I*337023007929895710,
     381362896181921786+I*337023007929894274,
     381356545785986474+I*337018779439338740,
     -72735658699307440+I*27533832271098267,
     -72735658706439276+I*27533832266055365,
     -72735658706439284+I*27533832266055359,
     -91521340179209918+I*12852811858342471,
     -91521361206843120+I*12852794454414707,
     -91521361206866626+I*12852794454394115,
     -91582758581043990+I*12793580266412617,
     -91582758649408382+I*12793580196677023,
     -91582758649408458+I*12793580196676941,
     -91582955145518152+I*12793346698205133,
     -91582955145735534+I*12793346697931609,
     -92139897885723542+I*12019195881780403,
     -92139898498260666+I*12019194977960085,
     -92139898498261338+I*12019194977959031,
     -92141592958524356+I*12016233502549755,
     -92141592960368510+I*12016233499106227,
     -92141592960368512+I*12016233499106223,
     -96719661467934188+I*2386689041466941,
     -96719666398201916+I*2386677874369195,
     -96719666398207200+I*2386677874356257,
     -96732382510281846+I*2350712724328185,
     -96732382523680484+I*2350712682761069,
     -96732382523680498+I*2350712682761021,
     -127832047191624370+I*-112730413366402433,
     -127832079090373886+I*-112730546065013095,
     -127832079090406134+I*-112730546065165981,
     -127900026023171096+I*-113151594939624065,
     -127900026087421882+I*-113151595423659617,
     -127900026087421940+I*-113151595423660173,
     -127900111576445292+I*-113153118905876487,
     -127900111576494248+I*-113153118907622729,
     -127900111576494248+I*-113153118907622732,
     -127805430087992662+I*-117920550392191580,
     -127805429826680204+I*-117920555845084224,
     -127805429826679730+I*-117920555845090456,
     -127803323241503974+I*-117937485127516614,
     -127803323238550006+I*-117937485146823188,
     -127803323238550002+I*-117937485146823210,
     -117226963895894860+I*-170189868500077366,
     -117226950115672534+I*-170189927959271534,
     -117226950115654856+I*-170189927959339138,
     -117169919113869224+I*-170371936895675364,
     -117169919042762428+I*-170371937102139742,
     -117169919042762340+I*-170371937102139976,
     -117169648541928418+I*-170372563592831172,
     -117169648541598752+I*-170372563593539460,
     -116178289649284348+I*-172260599889666682,
     -116178288454180836+I*-172260602018561248,
     -116178288454179400+I*-172260602018563646,
     -116174059963623866+I*-172266952414498958,
     -116174059958580964+I*-172266952421630794,
     -116174059958580958+I*-172266952421630802,
     -101493039550868070+I*-191052633894401436,
     -101493022146940306+I*-191052654922034638,
     -101493022146919714+I*-191052654922058144,
     -101433807958938216+I*-191114052296235508,
     -101433807889202622+I*-191114052364599900,
     -101433807889202540+I*-191114052364599976,
     97628779541180496+I*-368394507749651174,
     97629013039652304+I*-368394704245760868,
     97629013039925828+I*-368394704245978250,
     98403163856077034+I*-368951646985966258,
     98403164759897352+I*-368951647598503382,
     98403164759898406+I*-368951647598504054,
     98406126235307682+I*-368953342058767072,
     98406126238751210+I*-368953342060611226,
     98406126238751214+I*-368953342060611228,
     108035670696390496+I*-373531410568176904,
     108035681863488242+I*-373531415498444632,
     108035681863501180+I*-373531415498449916,
     108071647013529252+I*-373544131610524562,
     108071647055096368+I*-373544131623923200,
     108071647055096416+I*-373544131623923214,
     223152773104259870+I*-404643796291867086,
     223152905802870532+I*-404643828190616602,
     223152905803023418+I*-404643828190648850,
     223573954677481502+I*-404711775123413812,
     223573955161517054+I*-404711775187664598,
     223573955161517610+I*-404711775187664656,
     223575478643733924+I*-404711860676688008 ],
   /* k=52 */ [ 223575478645480166+I*-404711860676736964,
     223575478645480169+I*-404711860676736964,
     228342910130049017+I*-404617179188235378,
     228342915582941661+I*-404617178926922920,
     228342915582947893+I*-404617178926922446,
     228359844865374051+I*-404615072341746690,
     228359844884680625+I*-404615072338792722,
     228359844884680647+I*-404615072338792718,
     280612228237934803+I*-394038712996137576,
     280612287697128971+I*-394038699215915250,
     280612287697196575+I*-394038699215897572,
     280794296633532801+I*-393981668214111940,
     280794296839997179+I*-393981668143005144,
     280794296839997413+I*-393981668143005056,
     834417795040482845+I*-173136948667290182,
     834418421531174041+I*-173136678166456260,
     834418421531882329+I*-173136678166126594,
     836306457828009551+I*-172145319273812190,
     836306459956904117+I*-172145318078708678,
     836306459956906515+I*-172145318078707242,
     836312810352841827+I*-172141089588151708,
     836312810359973663+I*-172141089583108806,
     836312810359973671+I*-172141089583108800,
     855098491832744305+I*-157460069175395912,
     855098512860377507+I*-157460051771468148,
     855098512860401013+I*-157460051771447556,
     855159910234578377+I*-157400837583466058,
     855159910302942769+I*-157400837513730464,
     855159910302942845+I*-157400837513730382,
     1032440365687994043+I*41661749916652654,
     1032440562184103737+I*41661983415124462,
     1032440562184321119+I*41661983415397986,
     1032997504924309127+I*42436134231549192,
     1032997505536846251+I*42436135135369510,
     1032997505536846923+I*42436135135370564,
     1032999199997109941+I*42439096610779840,
     1032999199998954095+I*42439096614223368,
     1032999199998954097+I*42439096614223372,
     1037577268506519773+I*52068641071862654,
     1037577273436787501+I*52068652238960400,
     1037577273436792785+I*52068652238973338,
     1037589989548867431+I*52104617389001410,
     1037589989562266069+I*52104617430568526,
     1037589989562266083+I*52104617430568574,
     1068689654230209955+I*167185743479732028,
     1068689686128959471+I*167185876178342690,
     1068689686128991719+I*167185876178495576,
     1068757633061756681+I*167606925052953660,
     1068757633126007467+I*167606925536989212,
     1068757633126007525+I*167606925536989768,
     1068757718615030877+I*167608449019206082,
     1068757718615079833+I*167608449020952324,
     1068757718615079833+I*167608449020952327,
     1068663037126578247+I*172375880505521175,
     1068663036865265789+I*172375885958413819,
     1068663036865265315+I*172375885958420051,
     1068660930280089559+I*172392815240846209,
     903783240428902527+I*1390062171379674510,
     903783240425948559+I*1390062171398981084,
     903783240425948555+I*1390062171398981106,
     893206881083293413+I*1442314554752235262,
     893206867303071087+I*1442314614211429430,
     893206867303053409+I*1442314614211497034,
     893149836301267777+I*1442496623147833260,
     893149836230160981+I*1442496623354297638,
     893149836230160893+I*1442496623354297872,
     893149565729326971+I*1442497249844989068,
     893149565728997305+I*1442497249845697356,
     892158206836682901+I*1444385286141824578,
     892158205641579389+I*1444385288270719144,
     892158205641577953+I*1444385288270721542,
     892153977151022419+I*1444391638666656854,
     892153977145979517+I*1444391638673788690,
     892153977145979511+I*1444391638673788698,
     877472956738266623+I*1463177320146559332,
     877472939334338859+I*1463177341174192534,
     877472939334318267+I*1463177341174216040,
     877413725146336769+I*1463238738548393404,
     877413725076601175+I*1463238738616757796,
     877413725076601093+I*1463238738616757872,
     678351137646218057+I*1640519194001809070,
     678350904147746249+I*1640519390497918764,
     678350904147472725+I*1640519390498136146,
     677576753331321519+I*1641076333238124154,
     677576752427501201+I*1641076333850661278,
     677576752427500147+I*1641076333850661950,
     677573790952090871+I*1641078028310924968,
     677573790948647343+I*1641078028312769122,
     677573790948647339+I*1641078028312769124,
     667944246491008057+I*1645656096820334800,
     667944235323910311+I*1645656101750602528,
     667944235323897373+I*1645656101750607812,
     667908270173869301+I*1645668817862682458,
     667908270132302185+I*1645668817876081096,
     667908270132302137+I*1645668817876081110,
     552827144083138683+I*1676768482544024982,
     552827011384528021+I*1676768514442774498,
     552827011384375135+I*1676768514442806746,
     552405962509917051+I*1676836461375571708,
     552405962025881499+I*1676836461439822494,
     552405962025880943+I*1676836461439822552,
     552404438543664629+I*1676836546928845904,
     552404438541918387+I*1676836546928894860,
     552404438541918384+I*1676836546928894860,
     547637007057349536+I*1676741865440393274,
     547637001604456892+I*1676741865179080816,
     547637001604450660+I*1676741865179080342,
     547620072322024502+I*1676739758593904586,
     547620072302717928+I*1676739758590950618,
     547620072302717906+I*1676739758590950614,
     495367688949463750+I*1666163399248295472,
     495367629490269582+I*1666163385468073146,
     495367629490201978+I*1666163385468055468,
     495185620553865752+I*1666106354466269836,
     495185620347401374+I*1666106354395163040,
     495185620347401140+I*1666106354395162952,
     -58437877853084292+I*1445261634919448078,
     -58438504343775488+I*1445261364418614156,
     -58438504344483776+I*1445261364418284490,
     -60326540640610998+I*1444270005525970086,
     -60326542769505564+I*1444270004330866574,
     -60326542769507962+I*1444270004330865138,
     -60332893165443274+I*1444265775840309604,
     -60332893172575110+I*1444265775835266702,
     -60332893172575118+I*1444265775835266696,
     -79118574645345752+I*1429584755427553808,
     -79118595672978954+I*1429584738023626044,
     -79118595673002460+I*1429584738023605452,
     -79179993047179824+I*1429525523835623954,
     -79179993115544216+I*1429525523765888360,
     -79179993115544292+I*1429525523765888278,
     -256460448500595490+I*1230462936335505242,
     -256460644996705184+I*1230462702837033434,
     -256460644996922566+I*1230462702836759910,
     -257017587736910574+I*1229688552020608704,
     -257017588349447698+I*1229688551116788386,
     -257017588349448370+I*1229688551116787332,
     -257019282809711388+I*1229685589641378056,
     -257019282811555542+I*1229685589637934528,
     -257019282811555544+I*1229685589637934524,
     -261597351319121220+I*1220056045180295242,
     -261597356249388948+I*1220056034013197496,
     -261597356249394232+I*1220056034013184558,
     -261610072361468878+I*1220020068863156486,
     -261610072374867516+I*1220020068821589370,
     -261610072374867530+I*1220020068821589322,
     -292709737042811402+I*1104938942772425868,
     -292709768941560918+I*1104938810073815206,
     -292709768941593166+I*1104938810073662320,
     -292777715874358128+I*1104517761199204236,
     -292777715938608914+I*1104517760715168684,
     -292777715938608972+I*1104517760715168128,
     -292777801427632324+I*1104516237232951814,
     -292777801427681280+I*1104516237231205572,
     -292777801427681280+I*1104516237231205569,
     -292683119939179694+I*1099748805746636721,
     -292683119677867236+I*1099748800293744077,
     -292683119677866762+I*1099748800293737845,
     -292681013092691006+I*1099731871011311687,
     -127803323241503974+I*-117937485127516614,
     -127803323238550006+I*-117937485146823188,
     -127803323238550002+I*-117937485146823210,
     -117226963895894860+I*-170189868500077366,
     -117226950115672534+I*-170189927959271534,
     -117226950115654856+I*-170189927959339138,
     -117169919113869224+I*-170371936895675364,
     -117169919042762428+I*-170371937102139742,
     -117169919042762340+I*-170371937102139976,
     -117169648541928418+I*-170372563592831172,
     -117169648541598752+I*-170372563593539460,
     -116178289649284348+I*-172260599889666682,
     -116178288454180836+I*-172260602018561248,
     -116178288454179400+I*-172260602018563646,
     -116174059963623866+I*-172266952414498958,
     -116174059958580964+I*-172266952421630794,
     -116174059958580958+I*-172266952421630802,
     -101493039550868070+I*-191052633894401436,
     -101493022146940306+I*-191052654922034638,
     -101493022146919714+I*-191052654922058144,
     -101433807958938216+I*-191114052296235508,
     -101433807889202622+I*-191114052364599900,
     -101433807889202540+I*-191114052364599976,
     97628779541180496+I*-368394507749651174,
     97629013039652304+I*-368394704245760868,
     97629013039925828+I*-368394704245978250,
     98403163856077034+I*-368951646985966258,
     98403164759897352+I*-368951647598503382,
     98403164759898406+I*-368951647598504054,
     98406126235307682+I*-368953342058767072,
     98406126238751210+I*-368953342060611226,
     98406126238751214+I*-368953342060611228,
     108035670696390496+I*-373531410568176904,
     108035681863488242+I*-373531415498444632,
     108035681863501180+I*-373531415498449916,
     108071647013529252+I*-373544131610524562,
     108071647055096368+I*-373544131623923200,
     108071647055096416+I*-373544131623923214,
     223152773104259870+I*-404643796291867086,
     223152905802870532+I*-404643828190616602,
     223152905803023418+I*-404643828190648850,
     223573954677481502+I*-404711775123413812,
     223573955161517054+I*-404711775187664598,
     223573955161517610+I*-404711775187664656,
     223575478643733924+I*-404711860676688008 ],
   /* k=53 */ [ 223575478645480166+I*-404711860676736964,
     223575478645480169+I*-404711860676736964,
     228342910130049017+I*-404617179188235378,
     228342915582941661+I*-404617178926922920,
     228342915582947893+I*-404617178926922446,
     228359844865374051+I*-404615072341746690,
     228359844884680625+I*-404615072338792722,
     228359844884680647+I*-404615072338792718,
     280612228237934803+I*-394038712996137576,
     280612287697128971+I*-394038699215915250,
     280612287697196575+I*-394038699215897572,
     280794296633532801+I*-393981668214111940,
     280794296839997179+I*-393981668143005144,
     280794296839997413+I*-393981668143005056,
     834417795040482845+I*-173136948667290182,
     834418421531174041+I*-173136678166456260,
     834418421531882329+I*-173136678166126594,
     836306457828009551+I*-172145319273812190,
     836306459956904117+I*-172145318078708678,
     836306459956906515+I*-172145318078707242,
     836312810352841827+I*-172141089588151708,
     836312810359973663+I*-172141089583108806,
     836312810359973671+I*-172141089583108800,
     855098491832744305+I*-157460069175395912,
     855098512860377507+I*-157460051771468148,
     855098512860401013+I*-157460051771447556,
     855159910234578377+I*-157400837583466058,
     855159910302942769+I*-157400837513730464,
     855159910302942845+I*-157400837513730382,
     1032440365687994043+I*41661749916652654,
     1032440562184103737+I*41661983415124462,
     1032440562184321119+I*41661983415397986,
     1032997504924309127+I*42436134231549192,
     1032997505536846251+I*42436135135369510,
     1032997505536846923+I*42436135135370564,
     1032999199997109941+I*42439096610779840,
     1032999199998954095+I*42439096614223368,
     1032999199998954097+I*42439096614223372,
     1037577268506519773+I*52068641071862654,
     1037577273436787501+I*52068652238960400,
     1037577273436792785+I*52068652238973338,
     1037589989548867431+I*52104617389001410,
     1037589989562266069+I*52104617430568526,
     1037589989562266083+I*52104617430568574,
     1068689654230209955+I*167185743479732028,
     1068689686128959471+I*167185876178342690,
     1068689686128991719+I*167185876178495576,
     1068757633061756681+I*167606925052953660,
     1068757633126007467+I*167606925536989212,
     1068757633126007525+I*167606925536989768,
     1180691692375063209+I*1495698641413675506,
     1180691777864086561+I*1495700164895891820,
     1180691777864135517+I*1495700164897638062,
     1180691777864135517+I*1495700164897638065,
     1180597096375633931+I*1500467596382206913,
     1180597096114321473+I*1500467601835099557,
     1180597096114320999+I*1500467601835105789,
     1180594989529145243+I*1500484531117531947,
     1180594989526191275+I*1500484531136838521,
     1180594989526191271+I*1500484531136838543,
     1170018630183536129+I*1552736914490092699,
     1170018616403313803+I*1552736973949286867,
     1170018616403296125+I*1552736973949354471,
     1169961585401510493+I*1552918982885690697,
     1169961585330403697+I*1552918983092155075,
     1169961585330403609+I*1552918983092155309,
     949116865854688735+I*2106542481292640741,
     949116595353854813+I*2106543107783331937,
     949116595353525147+I*2106543107784040225,
     948125236461210743+I*2108431144080167447,
     948125235266107231+I*2108431146209062013,
     948125235266105795+I*2108431146209064411,
     948121006775550261+I*2108437496604999723,
     948121006770507359+I*2108437496612131559,
     948121006770507353+I*2108437496612131567,
     933439986362794465+I*2127223178084902201,
     933439968958866701+I*2127223199112535403,
     933439968958846109+I*2127223199112558909,
     933380754770864611+I*2127284596486736273,
     933380754701129017+I*2127284596555100665,
     933380754701128935+I*2127284596555100741,
     734318167270745899+I*2304565051940151939,
     734317933772274091+I*2304565248436261633,
     734317933772000567+I*2304565248436479015,
     733543782955849361+I*2305122191176467023,
     733543782052029043+I*2305122191789004147,
     733543782052027989+I*2305122191789004819,
     733540820576618713+I*2305123886249267837,
     733540820573175185+I*2305123886251111991,
     733540820573175181+I*2305123886251111993,
     723911276115535899+I*2309701954758677669,
     723911264948438153+I*2309701959688945397,
     723911264948425215+I*2309701959688950681,
     723875299798397143+I*2309714675801025327,
     -1876341102330446491+I*3197628652237479564,
     -1876341102372013607+I*3197628652250878202,
     -1876341102372013655+I*3197628652250878216,
     -1991422228421177109+I*3228728316918822088,
     -1991422361119787771+I*3228728348817571604,
     -1991422361119940657+I*3228728348817603852,
     -1991843409994398741+I*3228796295750368814,
     -1991843410478434293+I*3228796295814619600,
     -1991843410478434849+I*3228796295814619658,
     -1991844933960651163+I*3228796381303643010,
     -1991844933962397405+I*3228796381303691966,
     -1991844933962397408+I*3228796381303691966,
     -1996612365446966256+I*3228701699815190380,
     -1996612370899858900+I*3228701699553877922,
     -1996612370899865132+I*3228701699553877448,
     -1996629300182291290+I*3228699592968701692,
     -1996629300201597864+I*3228699592965747724,
     -1996629300201597886+I*3228699592965747720,
     -2048881683554852042+I*3218123233623092578,
     -2048881743014046210+I*3218123219842870252,
     -2048881743014113814+I*3218123219842852574,
     -2049063751950450040+I*3218066188841066942,
     -2049063752156914418+I*3218066188769960146,
     -2049063752156914652+I*3218066188769960058,
     -2602687250357400084+I*2997221469294245184,
     -2602687876848091280+I*2997221198793411262,
     -2602687876848799568+I*2997221198793081596,
     -2604575913144926790+I*2996229839900767192,
     -2604575915273821356+I*2996229838705663680,
     -2604575915273823754+I*2996229838705662244,
     -2604582265669759066+I*2996225610215106710,
     -2604582265676890902+I*2996225610210063808,
     -2604582265676890910+I*2996225610210063802,
     -2623367947149661544+I*2981544589802350914,
     -2623367968177294746+I*2981544572398423150,
     -2623367968177318252+I*2981544572398402558,
     -2623429365551495616+I*2981485358210421060,
     -2623429365619860008+I*2981485358140685466,
     -2623429365619860084+I*2981485358140685384,
     -2800709821004911282+I*2782422770710302348,
     -2800710017501020976+I*2782422537211830540,
     -2800710017501238358+I*2782422537211557016,
     -2801266960241226366+I*2781648386395405810,
     -2801266960853763490+I*2781648385491585492,
     -2801266960853764162+I*2781648385491584438,
     -2801268655314027180+I*2781645424016175162,
     -2801268655315871334+I*2781645424012731634,
     -2801268655315871336+I*2781645424012731630,
     -2805846723823437012+I*2772015879555092348,
     -2805846728753704740+I*2772015868387994602,
     -2805846728753710024+I*2772015868387981664,
     -2805859444865784670+I*2771979903237953592,
     -2805859444879183308+I*2771979903196386476,
     -2805859444879183322+I*2771979903196386428,
     -2836959109547127194+I*2656898777147222974,
     -2836959141445876710+I*2656898644448612312,
     -2836959141445908958+I*2656898644448459426,
     -2837027088378673920+I*2656477595574001342,
     -2837027088442924706+I*2656477595089965790,
     -2837027088442924764+I*2656477595089965234,
     -2948961147691980448+I*1328385879213279496,
     -2948961233181003800+I*1328384355731063182,
     -2948961233181052756+I*1328384355729316940,
     -2948961233181052756+I*1328384355729316937,
     -2948866551692551170+I*1323616924244748089,
     -2948866551431238712+I*1323616918791855445,
     -2948866551431238238+I*1323616918791849213,
     -2948864444846062482+I*1323599989509423055,
     -2948864444843108514+I*1323599989490116481,
     -2948864444843108510+I*1323599989490116459,
     -2938288085500453368+I*1271347606136862303,
     -2938288071720231042+I*1271347546677668135,
     -2938288071720213364+I*1271347546677600531,
     -2938231040718427732+I*1271165537741264305,
     -2938231040647320936+I*1271165537534799927,
     -2938231040647320848+I*1271165537534799693,
     -2717386321171605974+I*717542039334314261,
     -2717386050670772052+I*717541412843623065,
     -2717386050670442386+I*717541412842914777,
     -2716394691778127982+I*715653376546787555,
     -2716394690583024470+I*715653374417892989,
     -2716394690583023034+I*715653374417890591,
     -2716390462092467500+I*715647024021955279,
     -2716390462087424598+I*715647024014823443,
     -2716390462087424592+I*715647024014823435,
     -2701709441679711704+I*696861342542052801,
     -2701709424275783940+I*696861321514419599,
     -2701709424275763348+I*696861321514396093,
     -2701650210087781850+I*696799924140218729,
     -2701650210018046256+I*696799924071854337,
     -2701650210018046174+I*696799924071854261,
     -2502587622587663138+I*519519468686803063,
     -2502587389089191330+I*519519272190693369,
     -2502587389088917806+I*519519272190475987,
     -2501813238272766600+I*518962329450487979,
     -2501813237368946282+I*518962328837950855,
     -2501813237368945228+I*518962328837950183,
     -2501810275893535952+I*518960634377687165,
     -2501810275890092424+I*518960634375843011,
     -2501810275890092420+I*518960634375843009,
     -2492180731432453138+I*514382565868277333,
     -2492180720265355392+I*514382560938009605,
     -2492180720265342454+I*514382560938004321,
     -2492144755115314382+I*514369844825929675,
     108071647013529252+I*-373544131610524562,
     108071647055096368+I*-373544131623923200,
     108071647055096416+I*-373544131623923214,
     223152773104259870+I*-404643796291867086,
     223152905802870532+I*-404643828190616602,
     223152905803023418+I*-404643828190648850,
     223573954677481502+I*-404711775123413812,
     223573955161517054+I*-404711775187664598,
     223573955161517610+I*-404711775187664656,
     223575478643733924+I*-404711860676688008 ],
   /* k=54 */ [ -4152468876356271942+I*-4717230688497969995,
     -4152468876356271939+I*-4717230688497969995,
     -4147701444871703091+I*-4717136007009468409,
     -4147701439418810447+I*-4717136006748155951,
     -4147701439418804215+I*-4717136006748155477,
     -4147684510136378057+I*-4717133900162979721,
     -4147684510117071483+I*-4717133900160025753,
     -4147684510117071461+I*-4717133900160025749,
     -4095432126763817305+I*-4706557540817370607,
     -4095432067304623137+I*-4706557527037148281,
     -4095432067304555533+I*-4706557527037130603,
     -4095250058368219307+I*-4706500496035344971,
     -4095250058161754929+I*-4706500495964238175,
     -4095250058161754695+I*-4706500495964238087,
     -3541626559961269263+I*-4485655776488523213,
     -3541625933470578067+I*-4485655505987689291,
     -3541625933469869779+I*-4485655505987359625,
     -3539737897173742557+I*-4484664147095045221,
     -3539737895044847991+I*-4484664145899941709,
     -3539737895044845593+I*-4484664145899940273,
     -3539731544648910281+I*-4484659917409384739,
     -3539731544641778445+I*-4484659917404341837,
     -3539731544641778437+I*-4484659917404341831,
     -3520945863169007803+I*-4469978896996628943,
     -3520945842141374601+I*-4469978879592701179,
     -3520945842141351095+I*-4469978879592680587,
     -3520884444767173731+I*-4469919665404699089,
     855159910234578377+I*-157400837583466058,
     855159910302942769+I*-157400837513730464,
     855159910302942845+I*-157400837513730382,
     1032440365687994043+I*41661749916652654,
     1032440562184103737+I*41661983415124462,
     1032440562184321119+I*41661983415397986,
     1032997504924309127+I*42436134231549192,
     1032997505536846251+I*42436135135369510,
     1032997505536846923+I*42436135135370564,
     1032999199997109941+I*42439096610779840,
     1032999199998954095+I*42439096614223368,
     1032999199998954097+I*42439096614223372,
     1037577268506519773+I*52068641071862654,
     1037577273436787501+I*52068652238960400,
     1037577273436792785+I*52068652238973338,
     1037589989548867431+I*52104617389001410,
     1037589989562266069+I*52104617430568526,
     1037589989562266083+I*52104617430568574,
     1068689654230209955+I*167185743479732028,
     1068689686128959471+I*167185876178342690,
     1068689686128991719+I*167185876178495576,
     1068757633061756681+I*167606925052953660,
     1068757633126007467+I*167606925536989212,
     1068757633126007525+I*167606925536989768,
     1180691692375063209+I*1495698641413675506,
     1180691777864086561+I*1495700164895891820,
     1180691777864135517+I*1495700164897638062,
     1180691777864135517+I*1495700164897638065,
     1180597096375633931+I*1500467596382206913,
     1180597096114321473+I*1500467601835099557,
     1180597096114320999+I*1500467601835105789,
     1180594989529145243+I*1500484531117531947,
     1180594989526191275+I*1500484531136838521,
     1180594989526191271+I*1500484531136838543,
     1170018630183536129+I*1552736914490092699,
     1170018616403313803+I*1552736973949286867,
     1170018616403296125+I*1552736973949354471,
     1169961585401510493+I*1552918982885690697,
     1169961585330403697+I*1552918983092155075,
     1169961585330403609+I*1552918983092155309,
     949116865854688735+I*2106542481292640741,
     949116595353854813+I*2106543107783331937,
     949116595353525147+I*2106543107784040225,
     948125236461210743+I*2108431144080167447,
     948125235266107231+I*2108431146209062013,
     948125235266105795+I*2108431146209064411,
     948121006775550261+I*2108437496604999723,
     948121006770507359+I*2108437496612131559,
     948121006770507353+I*2108437496612131567,
     933439986362794465+I*2127223178084902201,
     933439968958866701+I*2127223199112535403,
     933439968958846109+I*2127223199112558909,
     933380754770864611+I*2127284596486736273,
     933380754701129017+I*2127284596555100665,
     933380754701128935+I*2127284596555100741,
     734318167270745899+I*2304565051940151939,
     734317933772274091+I*2304565248436261633,
     734317933772000567+I*2304565248436479015,
     733543782955849361+I*2305122191176467023,
     733543782052029043+I*2305122191789004147,
     733543782052027989+I*2305122191789004819,
     -1810705590452287803+I*3857082026163801925,
     -1810708551927697079+I*3857083720624064943,
     -1810708551931140607+I*3857083720625909097,
     -1810708551931140611+I*3857083720625909099,
     -1820338096388779893+I*3861661789133474775,
     -1820338107555877639+I*3861661794063742503,
     -1820338107555890577+I*3861661794063747787,
     -1820374072705918649+I*3861674510175822433,
     -1820374072747485765+I*3861674510189221071,
     -1820374072747485813+I*3861674510189221085,
     -1935455198796649267+I*3892774174857164957,
     -1935455331495259929+I*3892774206755914473,
     -1935455331495412815+I*3892774206755946721,
     -1935876380369870899+I*3892842153688711683,
     -1935876380853906451+I*3892842153752962469,
     -1935876380853907007+I*3892842153752962527,
     -3263968096730592745+I*4004776213002018211,
     -3263969620212809059+I*4004776298491041563,
     -3263969620214555301+I*4004776298491090519,
     -3263969620214555304+I*4004776298491090519,
     -3268737051699124152+I*4004681617002588933,
     -3268737057152016796+I*4004681616741276475,
     -3268737057152023028+I*4004681616741276001,
     -3268753986434449186+I*4004679510156100245,
     -3268753986453755760+I*4004679510153146277,
     -3268753986453755782+I*4004679510153146273,
     -3321006369807009938+I*3994103150810491131,
     -3321006429266204106+I*3994103137030268805,
     -3321006429266271710+I*3994103137030251127,
     -3321188438202607936+I*3994046106028465495,
     -3321188438409072314+I*3994046105957358699,
     -3321188438409072548+I*3994046105957358611,
     -3874811936609557980+I*3773201386481643737,
     -3874812563100249176+I*3773201115980809815,
     -3874812563100957464+I*3773201115980480149,
     -3876700599397084686+I*3772209757088165745,
     -3876700601525979252+I*3772209755893062233,
     -3876700601525981650+I*3772209755893060797,
     -3876706951921916962+I*3772205527402505263,
     -3876706951929048798+I*3772205527397462361,
     -3876706951929048806+I*3772205527397462355,
     -3895492633401819440+I*3757524506989749467,
     -3895492654429452642+I*3757524489585821703,
     -3895492654429476148+I*3757524489585801111,
     -3895554051803653512+I*3757465275397819613,
     -8271598406805405620+I*-555053552423413418,
     -8271598406873770012+I*-555053552493149012,
     -8271598406873770088+I*-555053552493149094,
     -8448878862258821286+I*-754116139923532130,
     -8448879058754930980+I*-754116373422003938,
     -8448879058755148362+I*-754116373422277462,
     -8449436001495136370+I*-754890524238428668,
     -8449436002107673494+I*-754890525142248986,
     -8449436002107674166+I*-754890525142250040,
     -8449437696567937184+I*-754893486617659316,
     -8449437696569781338+I*-754893486621102844,
     -8449437696569781340+I*-754893486621102848,
     -8454015765077347016+I*-764523031078742130,
     -8454015770007614744+I*-764523042245839876,
     -8454015770007620028+I*-764523042245852814,
     -8454028486119694674+I*-764559007395880886,
     -8454028486133093312+I*-764559007437448002,
     -8454028486133093326+I*-764559007437448050,
     -8485128150801037198+I*-879640133486611504,
     -8485128182699786714+I*-879640266185222166,
     -8485128182699818962+I*-879640266185375052,
     -8485196129632583924+I*-880061315059833136,
     -8485196129696834710+I*-880061315543868688,
     -8485196129696834768+I*-880061315543869244,
     -8597130188945890452+I*-2208153031420554982,
     -8597130274434913804+I*-2208154554902771296,
     -8597130274434962760+I*-2208154554904517538,
     -8597130274434962760+I*-2208154554904517541,
     -8597035592946461174+I*-2212921986389086389,
     -8597035592685148716+I*-2212921991841979033,
     -8597035592685148242+I*-2212921991841985265,
     -8597033486099972486+I*-2212938921124411423,
     -8597033486097018518+I*-2212938921143717997,
     -8597033486097018514+I*-2212938921143718019,
     -8586457126754363372+I*-2265191304496972175,
     -8586457112974141046+I*-2265191363956166343,
     -8586457112974123368+I*-2265191363956233947,
     -8586400081972337736+I*-2265373372892570173,
     -8586400081901230940+I*-2265373373099034551,
     -8586400081901230852+I*-2265373373099034785,
     -8365555362425515978+I*-2818996871299520217,
     -8365555091924682056+I*-2818997497790211413,
     -8365555091924352390+I*-2818997497790919701,
     -8364563733032037986+I*-2820885534087046923,
     -8364563731836934474+I*-2820885536215941489,
     -8364563731836933038+I*-2820885536215943887,
     -8364559503346377504+I*-2820891886611879199,
     -8364559503341334602+I*-2820891886619011035,
     -8364559503341334596+I*-2820891886619011043,
     -8349878482933621708+I*-2839677568091781677,
     -8349878465529693944+I*-2839677589119414879,
     -8349878465529673352+I*-2839677589119438385,
     -8349819251341691854+I*-2839738986493615749,
     -8349819251271956260+I*-2839738986561980141,
     -8349819251271956178+I*-2839738986561980217,
     -8150756663841573142+I*-3017019441947031415,
     -8150756430343101334+I*-3017019638443141109,
     -8150756430342827810+I*-3017019638443358491,
     -8149982279526676604+I*-3017576581183346499,
     -8149982278622856286+I*-3017576581795883623,
     -8149982278622855232+I*-3017576581795884295,
     -5605732906118539440+I*-4569536416170681401,
     -5605729944643130164+I*-4569538110630944419,
     -5605729944639686636+I*-4569538110632788573,
     -5605729944639686632+I*-4569538110632788575,
     -5596100400182047350+I*-4574116179140354251,
     -5596100389014949604+I*-4574116184070621979,
     -5596100389014936666+I*-4574116184070627263,
     -5596064423864908594+I*-4574128900182701909,
     -5596064423823341478+I*-4574128900196100547,
     -5596064423823341430+I*-4574128900196100561,
     -5480983297774177976+I*-4605228564864044433,
     -5480983165075567314+I*-4605228596762793949,
     -5480983165075414428+I*-4605228596762826197,
     -5480562116200956344+I*-4605296543695591159,
     -5480562115716920792+I*-4605296543759841945,
     -5480562115716920236+I*-4605296543759842003,
     -4152470399840234498+I*-4717230603008897687,
     -4152468876358018184+I*-4717230688497921039 ],
   /* k=55 */ [ -2727560096342512990+I*-19550107681639624481,
     -2727560096342512987+I*-19550107681639624481,
     -2722792664857944139+I*-19550013000151122895,
     -2722792659405051495+I*-19550012999889810437,
     -2722792659405045263+I*-19550012999889809963,
     -2722775730122619105+I*-19550010893304634207,
     -2722775730103312531+I*-19550010893301680239,
     -2722775730103312509+I*-19550010893301680235,
     -2670523346750058353+I*-19539434533959025093,
     -2670523287290864185+I*-19539434520178802767,
     -2670523287290796581+I*-19539434520178785089,
     -2670341278354460355+I*-19539377489176999457,
     -2670341278147995977+I*-19539377489105892661,
     -2670341278147995743+I*-19539377489105892573,
     -2116717779947510311+I*-19318532769630177699,
     -2116717153456819115+I*-19318532499129343777,
     -2116717153456110827+I*-19318532499129014111,
     -2114829117159983605+I*-19317541140236699707,
     -2114829115031089039+I*-19317541139041596195,
     -2114829115031086641+I*-19317541139041594759,
     3533339926222823363+I*-15781002228407760281,
     3533346276618758675+I*-15780997999917204747,
     3533346276625890511+I*-15780997999912161845,
     3533346276625890519+I*-15780997999912161839,
     3552131958098661153+I*-15766316979504448951,
     3552131979126294355+I*-15766316962100521187,
     3552131979126317861+I*-15766316962100500595,
     3552193376500495225+I*-15766257747912519097,
     3552193376568859617+I*-15766257747842783503,
     3552193376568859693+I*-15766257747842783421,
     3729473831953910891+I*-15567195160412400385,
     3729474028450020585+I*-15567194926913928577,
     3729474028450237967+I*-15567194926913655053,
     3730030971190225975+I*-15566420776097503847,
     3730030971802763099+I*-15566420775193683529,
     3730030971802763771+I*-15566420775193682475,
     5281990806177560877+I*-13022171402689366683,
     5281992500637823895+I*-13022168441213957407,
     5281992500639668049+I*-13022168441210513879,
     5281992500639668051+I*-13022168441210513875,
     5286570569147233727+I*-13012538896752874593,
     5286570574077501455+I*-13012538885585776847,
     5286570574077506739+I*-13012538885585763909,
     5286583290189581385+I*-13012502920435735837,
     5286583290202980023+I*-13012502920394168721,
     5286583290202980037+I*-13012502920394168673,
     5317682954870923909+I*-12897421794345005219,
     5317682986769673425+I*-12897421661646394557,
     5317682986769705673+I*-12897421661646241671,
     5317750933702470635+I*-12897000612771783587,
     5317750933766721421+I*-12897000612287748035,
     5317750933766721479+I*-12897000612287747479,
     5429684993015777163+I*-11568908896411061741,
     5429685078504800515+I*-11568907372928845427,
     5429685078504849471+I*-11568907372927099185,
     5429685078504849471+I*-11568907372927099182,
     5429590397016347885+I*-11564139941442530334,
     5429590396755035427+I*-11564139935989637690,
     5429590396755034953+I*-11564139935989631458,
     5429588290169859197+I*-11564123006707205300,
     5429588290166905229+I*-11564123006687898726,
     5429588290166905225+I*-11564123006687898704,
     5419011930824250083+I*-11511870623334644548,
     5419011917044027757+I*-11511870563875450380,
     5419011917044010079+I*-11511870563875382776,
     5418954886042224447+I*-11511688554939046550,
     1169961585401510493+I*1552918982885690697,
     1169961585330403697+I*1552918983092155075,
     1169961585330403609+I*1552918983092155309,
     949116865854688735+I*2106542481292640741,
     949116595353854813+I*2106543107783331937,
     949116595353525147+I*2106543107784040225,
     948125236461210743+I*2108431144080167447,
     948125235266107231+I*2108431146209062013,
     948125235266105795+I*2108431146209064411,
     948121006775550261+I*2108437496604999723,
     948121006770507359+I*2108437496612131559,
     948121006770507353+I*2108437496612131567,
     933439986362794465+I*2127223178084902201,
     933439968958866701+I*2127223199112535403,
     933439968958846109+I*2127223199112558909,
     933380754770864611+I*2127284596486736273,
     933380754701129017+I*2127284596555100665,
     933380754701128935+I*2127284596555100741,
     734318167270745899+I*2304565051940151939,
     734317933772274091+I*2304565248436261633,
     734317933772000567+I*2304565248436479015,
     733543782955849361+I*2305122191176467023,
     733543782052029043+I*2305122191789004147,
     733543782052027989+I*2305122191789004819,
     -1810705590452287803+I*3857082026163801925,
     -1810708551927697079+I*3857083720624064943,
     -1810708551931140607+I*3857083720625909097,
     -1810708551931140611+I*3857083720625909099,
     -1820338096388779893+I*3861661789133474775,
     -1820338107555877639+I*3861661794063742503,
     -1820338107555890577+I*3861661794063747787,
     -1820374072705918649+I*3861674510175822433,
     -1820374072747485765+I*3861674510189221071,
     -1820374072747485813+I*3861674510189221085,
     -1935455198796649267+I*3892774174857164957,
     -1935455331495259929+I*3892774206755914473,
     -1935455331495412815+I*3892774206755946721,
     -1935876380369870899+I*3892842153688711683,
     -1935876380853906451+I*3892842153752962469,
     -1935876380853907007+I*3892842153752962527,
     -3263968096730592745+I*4004776213002018211,
     -3263969620212809059+I*4004776298491041563,
     -3263969620214555301+I*4004776298491090519,
     -3263969620214555304+I*4004776298491090519,
     -3268737051699124152+I*4004681617002588933,
     -3268737057152016796+I*4004681616741276475,
     -3268737057152023028+I*4004681616741276001,
     -3268753986434449186+I*4004679510156100245,
     -3268753986453755760+I*4004679510153146277,
     -3268753986453755782+I*4004679510153146273,
     -3321006369807009938+I*3994103150810491131,
     -3321006429266204106+I*3994103137030268805,
     -3321006429266271710+I*3994103137030251127,
     -3321188438202607936+I*3994046106028465495,
     -3321188438409072314+I*3994046105957358699,
     -3321188438409072548+I*3994046105957358611,
     -3874811936609557980+I*3773201386481643737,
     -3874812563100249176+I*3773201115980809815,
     -3874812563100957464+I*3773201115980480149,
     -3876700599397084686+I*3772209757088165745,
     -3876700601525979252+I*3772209755893062233,
     -3876700601525981650+I*3772209755893060797,
     -9524869642779891654+I*235670845259226319,
     -9524875993175826966+I*235666616768670785,
     -9524875993182958802+I*235666616763627883,
     -9524875993182958810+I*235666616763627877,
     -9543661674655729444+I*220985596355914989,
     -9543661695683362646+I*220985578951987225,
     -9543661695683386152+I*220985578951966633,
     -9543723093057563516+I*220926364763985135,
     -9543723093125927908+I*220926364694249541,
     -9543723093125927984+I*220926364694249459,
     -9721003548510979182+I*21863777263866423,
     -9721003745007088876+I*21863543765394615,
     -9721003745007306258+I*21863543765121091,
     -9721560687747294266+I*21089392948969885,
     -9721560688359831390+I*21089392045149567,
     -9721560688359832062+I*21089392045148513,
     -11273520522734629168+I*-2523159980459167279,
     -11273522217194892186+I*-2523162941934576555,
     -11273522217196736340+I*-2523162941938020083,
     -11273522217196736342+I*-2523162941938020087,
     -11278100285704302018+I*-2532792486395659369,
     -11278100290634569746+I*-2532792497562757115,
     -11278100290634575030+I*-2532792497562770053,
     -11278113006746649676+I*-2532828462712798125,
     -11278113006760048314+I*-2532828462754365241,
     -11278113006760048328+I*-2532828462754365289,
     -11309212671427992200+I*-2647909588803528743,
     -11309212703326741716+I*-2647909721502139405,
     -11309212703326773964+I*-2647909721502292291,
     -11309280650259538926+I*-2648330770376750375,
     -11309280650323789712+I*-2648330770860785927,
     -11309280650323789770+I*-2648330770860786483,
     -11421214709572845454+I*-3976422486737472221,
     -11421214795061868806+I*-3976424010219688535,
     -11421214795061917762+I*-3976424010221434777,
     -11421214795061917762+I*-3976424010221434780,
     -11421120113573416176+I*-3981191441706003628,
     -11421120113312103718+I*-3981191447158896272,
     -11421120113312103244+I*-3981191447158902504,
     -11421118006726927488+I*-3981208376441328662,
     -11421118006723973520+I*-3981208376460635236,
     -11421118006723973516+I*-3981208376460635258,
     -11410541647381318374+I*-4033460759813889414,
     -11410541633601096048+I*-4033460819273083582,
     -11410541633601078370+I*-4033460819273151186,
     -11410484602599292738+I*-4033642828209487412,
     -7161491301958578784+I*-17098250366034224659,
     -7161491301887471988+I*-17098250366240689037,
     -7161491301887471900+I*-17098250366240689271,
     -6940646582411757026+I*-17651873864441174703,
     -6940646311910923104+I*-17651874490931865899,
     -6940646311910593438+I*-17651874490932574187,
     -6939654953018279034+I*-17653762527228701409,
     -6939654951823175522+I*-17653762529357595975,
     -6939654951823174086+I*-17653762529357598373,
     -6939650723332618552+I*-17653768879753533685,
     -6939650723327575650+I*-17653768879760665521,
     -6939650723327575644+I*-17653768879760665529,
     -6924969702919862756+I*-17672554561233436163,
     -6924969685515934992+I*-17672554582261069365,
     -6924969685515914400+I*-17672554582261092871,
     -6924910471327932902+I*-17672615979635270235,
     -6924910471258197308+I*-17672615979703634627,
     -6924910471258197226+I*-17672615979703634703,
     -6725847883827814190+I*-17849896435088685901,
     -6725847650329342382+I*-17849896631584795595,
     -6725847650329068858+I*-17849896631585012977,
     -6725073499512917652+I*-17850453574325000985,
     -6725073498609097334+I*-17850453574937538109,
     -6725073498609096280+I*-17850453574937538781,
     -4180824126104780488+I*-19402413409312335887,
     -4180821164629371212+I*-19402415103772598905,
     -4180821164625927684+I*-19402415103774443059,
     -4180821164625927680+I*-19402415103774443061,
     -4171191620168288398+I*-19406993172282008737,
     -4171191609001190652+I*-19406993177212276465,
     -4171191609001177714+I*-19406993177212281749,
     -4171155643851149642+I*-19407005893324356395,
     -4171155643809582526+I*-19407005893337755033,
     -4171155643809582478+I*-19407005893337755047,
     -4056074517760419024+I*-19438105558005698919,
     -4056074385061808362+I*-19438105589904448435,
     -4056074385061655476+I*-19438105589904480683,
     -4055653336187197392+I*-19438173536837245645,
     -4055653335703161840+I*-19438173536901496431,
     -4055653335703161284+I*-19438173536901496489,
     -2727561619826475546+I*-19550107596150552173,
     -2727560096344259232+I*-19550107681639575525 ],
   /* k=56 */ [ 28363102669954554934+I*-31533167114753761063,
     28363102669954554937+I*-31533167114753761063,
     28367870101439123785+I*-31533072433265259477,
     28367870106892016429+I*-31533072433003947019,
     28367870106892022661+I*-31533072433003946545,
     28367887036174448819+I*-31533070326418770789,
     28367887036193755393+I*-31533070326415816821,
     28367887036193755415+I*-31533070326415816817,
     28420139419547009571+I*-31522493967073161675,
     28420139479006203739+I*-31522493953292939349,
     28420139479006271343+I*-31522493953292921671,
     28420321487942607569+I*-31522436922291136039,
     28420321488149071947+I*-31522436922220029243,
     28420321488149072181+I*-31522436922220029155,
     28973944986349557613+I*-31301592202744314281,
     28973945612840248809+I*-31301591932243480359,
     28973945612840957097+I*-31301591932243150693,
     28975833649137084319+I*-31300600573350836289,
     28975833651265978885+I*-31300600572155732777,
     28975833651265981283+I*-31300600572155731341,
     34624002692519891287+I*-27764061661521896863,
     34624009042915826599+I*-27764057433031341329,
     34624009042922958435+I*-27764057433026298427,
     34624009042922958443+I*-27764057433026298421,
     34642794724395729077+I*-27749376412618585533,
     34642794745423362279+I*-27749376395214657769,
     34642794745423385785+I*-27749376395214637177,
     34642856142797563149+I*-27749317181026655679,
     34642856142865927541+I*-27749317180956920085,
     34642856142865927617+I*-27749317180956920003,
     34820136598250978815+I*-27550254593526536967,
     34820136794747088509+I*-27550254360028065159,
     34820136794747305891+I*-27550254360027791635,
     34820693737487293899+I*-27549480209211640429,
     34820693738099831023+I*-27549480208307820111,
     34820693738099831695+I*-27549480208307819057,
     36372653572474628801+I*-25005230835803503265,
     36372655266934891819+I*-25005227874328093989,
     36372655266936735973+I*-25005227874324650461,
     36372655266936735975+I*-25005227874324650457,
     36377233335444301651+I*-24995598329867011175,
     36377233340374569379+I*-24995598318699913429,
     36377233340374574663+I*-24995598318699900491,
     36377246056486649309+I*-24995562353549872419,
     36377246056500047947+I*-24995562353508305303,
     36377246056500047961+I*-24995562353508305255,
     36408345721167991833+I*-24880481227459141801,
     36408345753066741349+I*-24880481094760531139,
     36408345753066773597+I*-24880481094760378253,
     36408413699999538559+I*-24880060045885920169,
     36408413700063789345+I*-24880060045401884617,
     36408413700063789403+I*-24880060045401884061,
     36520347759312845087+I*-23551968329525198323,
     36520347844801868439+I*-23551966806042982009,
     36520347844801917395+I*-23551966806041235767,
     36520347844801917395+I*-23551966806041235764,
     36520253163313415809+I*-23547199374556666916,
     36520253163052103351+I*-23547199369103774272,
     36520253163052102877+I*-23547199369103768040,
     35095344383038343925+I*-8714322375962113554,
     35095342276453168169+I*-8714305446679687396,
     35095342276450214201+I*-8714305446660380822,
     35095342276450214197+I*-8714305446660380800,
     35084765917107559055+I*-8662053063307126644,
     35084765903327336729+I*-8662053003847932476,
     35084765903327319051+I*-8662053003847864872,
     35084708872325533419+I*-8661870994911528646,
     35084708872254426623+I*-8661870994705064268,
     35084708872254426535+I*-8661870994705064034,
     34863864152778711661+I*-8108247496504578602,
     34863863882277877739+I*-8108246870013887406,
     34863863882277548073+I*-8108246870013179118,
     34862872523385233669+I*-8106358833717051896,
     34862872522190130157+I*-8106358831588157330,
     34862872522190128721+I*-8106358831588154932,
     31326333611556294243+I*-2458189790334244928,
     31326329383065738709+I*-2458183439938309616,
     31326329383060695807+I*-2458183439931177780,
     31326329383060695801+I*-2458183439931177772,
     31311648362652982913+I*-2439397758458407138,
     31311648345249055149+I*-2439397737430773936,
     31311648345249034557+I*-2439397737430750430,
     31311589131061053059+I*-2439336340056573066,
     31311589130991317465+I*-2439336339988208674,
     31311589130991317383+I*-2439336339988208598,
     31112526543560934347+I*-2262055884603157400,
     31112526310062462539+I*-2262055688107047706,
     31112526310062189015+I*-2262055688106830324,
     31111752159246037809+I*-2261498745366842316,
     31111752158342217491+I*-2261498744754305192,
     31111752158342216437+I*-2261498744754304520,
     28567502785837900645+I*-709538910379507414,
     28567499824362491369+I*-709537215919244396,
     28567499824359047841+I*-709537215917400242,
     28567499824359047837+I*-709537215917400240,
     28557870279901408555+I*-704959147409834564,
     28557870268734310809+I*-704959142479566836,
     28557870268734297871+I*-704959142479561552,
     28557834303584269799+I*-704946426367486906,
     28557834303542702683+I*-704946426354088268,
     28557834303542702635+I*-704946426354088254,
     28442753177493539181+I*-673846761686144382,
     28442753044794928519+I*-673846729787394866,
     28442753044794775633+I*-673846729787362618,
     28442331995920317549+I*-673778782854597656,
     -1935876380369870899+I*3892842153688711683,
     -1935876380853906451+I*3892842153752962469,
     -1935876380853907007+I*3892842153752962527,
     -3263968096730592745+I*4004776213002018211,
     -3263969620212809059+I*4004776298491041563,
     -3263969620214555301+I*4004776298491090519,
     -3263969620214555304+I*4004776298491090519,
     -3268737051699124152+I*4004681617002588933,
     -3268737057152016796+I*4004681616741276475,
     -3268737057152023028+I*4004681616741276001,
     -3268753986434449186+I*4004679510156100245,
     -3268753986453755760+I*4004679510153146277,
     -3268753986453755782+I*4004679510153146273,
     -3321006369807009938+I*3994103150810491131,
     -3321006429266204106+I*3994103137030268805,
     -3321006429266271710+I*3994103137030251127,
     -3321188438202607936+I*3994046106028465495,
     -3321188438409072314+I*3994046105957358699,
     -3321188438409072548+I*3994046105957358611,
     -3874811936609557980+I*3773201386481643737,
     -3874812563100249176+I*3773201115980809815,
     -3874812563100957464+I*3773201115980480149,
     -3876700599397084686+I*3772209757088165745,
     -3876700601525979252+I*3772209755893062233,
     -3876700601525981650+I*3772209755893060797,
     -9524869642779891654+I*235670845259226319,
     -9524875993175826966+I*235666616768670785,
     -9524875993182958802+I*235666616763627883,
     -9524875993182958810+I*235666616763627877,
     -9543661674655729444+I*220985596355914989,
     -9543661695683362646+I*220985578951987225,
     -9543661695683386152+I*220985578951966633,
     -9543723093057563516+I*220926364763985135,
     -9543723093125927908+I*220926364694249541,
     -9543723093125927984+I*220926364694249459,
     -9721003548510979182+I*21863777263866423,
     -9721003745007088876+I*21863543765394615,
     -9721003745007306258+I*21863543765121091,
     -9721560687747294266+I*21089392948969885,
     -9721560688359831390+I*21089392045149567,
     -9721560688359832062+I*21089392045148513,
     -11273520522734629168+I*-2523159980459167279,
     -11273522217194892186+I*-2523162941934576555,
     -11273522217196736340+I*-2523162941938020083,
     -11273522217196736342+I*-2523162941938020087,
     -11278100285704302018+I*-2532792486395659369,
     -11278100290634569746+I*-2532792497562757115,
     -11278100290634575030+I*-2532792497562770053,
     -11278113006746649676+I*-2532828462712798125,
     -11278113006760048314+I*-2532828462754365241,
     -11278113006760048328+I*-2532828462754365289,
     -11309212671427992200+I*-2647909588803528743,
     -11309212703326741716+I*-2647909721502139405,
     -11309212703326773964+I*-2647909721502292291,
     -11309280650259538926+I*-2648330770376750375,
     -11309280650323789712+I*-2648330770860785927,
     -11309280650323789770+I*-2648330770860786483,
     -11421214709572845454+I*-3976422486737472221,
     -11421214795061868806+I*-3976424010219688535,
     -11421214795061917762+I*-3976424010221434777,
     -11421214795061917762+I*-3976424010221434780,
     -11421120113573416176+I*-3981191441706003628,
     -11421120113312103718+I*-3981191447158896272,
     -11421120113312103244+I*-3981191447158902504,
     -9996211333298344292+I*-18814068440300556990,
     -9996209226713168536+I*-18814085369582983148,
     -9996209226710214568+I*-18814085369602289722,
     -9996209226710214564+I*-18814085369602289744,
     -9985632867367559422+I*-18866337752955543900,
     -9985632853587337096+I*-18866337812414738068,
     -9985632853587319418+I*-18866337812414805672,
     -9985575822585533786+I*-18866519821351141898,
     -9985575822514426990+I*-18866519821557606276,
     -9985575822514426902+I*-18866519821557606510,
     -9764731103038712028+I*-19420143319758091942,
     -9764730832537878106+I*-19420143946248783138,
     -9764730832537548440+I*-19420143946249491426,
     -9763739473645234036+I*-19422031982545618648,
     -9763739472450130524+I*-19422031984674513214,
     -9763739472450129088+I*-19422031984674515612,
     -6227200561816294610+I*-25070201025928425616,
     -6227196333325739076+I*-25070207376324360928,
     -6227196333320696174+I*-25070207376331492764,
     -6227196333320696168+I*-25070207376331492772,
     -6212515312912983280+I*-25088993057804263406,
     -6212515295509055516+I*-25088993078831896608,
     -6212515295509034924+I*-25088993078831920114,
     -6212456081321053426+I*-25089054476206097478,
     -6212456081251317832+I*-25089054476274461870,
     -6212456081251317750+I*-25089054476274461946,
     -6013393493820934714+I*-25266334931659513144,
     -6013393260322462906+I*-25266335128155622838,
     -6013393260322189382+I*-25266335128155840220,
     -6012619109506038176+I*-25266892070895828228,
     -6012619108602217858+I*-25266892071508365352,
     -6012619108602216804+I*-25266892071508366024,
     -3468369736097901012+I*-26818851905883163130,
     -3468366774622491736+I*-26818853600343426148,
     -3468366774619048208+I*-26818853600345270302,
     -3468366774619048204+I*-26818853600345270304,
     -3458737230161408922+I*-26823431668852835980,
     -3458737218994311176+I*-26823431673783103708,
     -3458737218994298238+I*-26823431673783108992,
     -3458701253844270166+I*-26823444389895183638,
     -3458701253802703050+I*-26823444389908582276,
     -3458701253802703002+I*-26823444389908582290,
     -3343620127753539548+I*-26854544054576526162,
     -3343619995054928886+I*-26854544086475275678,
     -3343619995054776000+I*-26854544086475307926,
     -3343198946180317916+I*-26854612033408072888,
     27035009430109870532+I*-31421232969951382227,
     27035009430593906084+I*-31421232970015633013,
     27035009430593906640+I*-31421232970015633071,
     28363101146470592378+I*-31533167029264688755,
     28363102669952808692+I*-31533167114753712107 ],
   /* k=57 */ [ 29075557059961434410+I*-38949605611324588306,
     29075557059961434413+I*-38949605611324588306,
     29080324491446003261+I*-38949510929836086720,
     29080324496898895905+I*-38949510929574774262,
     29080324496898902137+I*-38949510929574773788,
     43913201490040556623+I*-37524602149561014836,
     43913218419322982781+I*-37524600042975839080,
     43913218419342289355+I*-37524600042972885112,
     43913218419342289377+I*-37524600042972885108,
     43965470802695543533+I*-37514023683630229966,
     43965470862154737701+I*-37514023669850007640,
     43965470862154805305+I*-37514023669849989962,
     43965652871091141531+I*-37513966638848204330,
     43965652871297605909+I*-37513966638777097534,
     43965652871297606143+I*-37513966638777097446,
     44519276369498091575+I*-37293121919301382572,
     44519276995988782771+I*-37293121648800548650,
     44519276995989491059+I*-37293121648800218984,
     44521165032285618281+I*-37292130289907904580,
     44521165034414512847+I*-37292130288712801068,
     44521165034414515245+I*-37292130288712799632,
     50169334075668425249+I*-33755591378078965154,
     50169340426064360561+I*-33755587149588409620,
     50169340426071492397+I*-33755587149583366718,
     50169340426071492405+I*-33755587149583366712,
     50188126107544263039+I*-33740906129175653824,
     50188126128571896241+I*-33740906111771726060,
     50188126128571919747+I*-33740906111771705468,
     50188187525946097111+I*-33740846897583723970,
     50188187526014461503+I*-33740846897513988376,
     50188187526014461579+I*-33740846897513988294,
     50365467981399512777+I*-33541784310083605258,
     50365468177895622471+I*-33541784076585133450,
     50365468177895839853+I*-33541784076584859926,
     50366025120635827861+I*-33541009925768708720,
     89877475370012634987+I*22648785890268358837,
     89877475370625172111+I*22648785891172179155,
     89877475370625172783+I*22648785891172180209,
     91429435204999969889+I*25193035263676496001,
     91429436899460232907+I*25193038225151905277,
     91429436899462077061+I*25193038225155348805,
     91429436899462077063+I*25193038225155348809,
     91434014967969642739+I*25202667769612988091,
     91434014972899910467+I*25202667780780085837,
     91434014972899915751+I*25202667780780098775,
     91434027689011990397+I*25202703745930126847,
     91434027689025389035+I*25202703745971693963,
     91434027689025389049+I*25202703745971694011,
     91465127353693332921+I*25317784872020857465,
     91465127385592082437+I*25317785004719468127,
     91465127385592114685+I*25317785004719621013,
     91465195332524879647+I*25318206053594079097,
     91465195332589130433+I*25318206054078114649,
     91465195332589130491+I*25318206054078115205,
     91577129391838186175+I*26646297769954800943,
     91577129477327209527+I*26646299293437017257,
     91577129477327258483+I*26646299293438763499,
     91577129477327258483+I*26646299293438763502,
     91577034795838756897+I*26651066724923332350,
     91577034795577444439+I*26651066730376224994,
     91577034795577443965+I*26651066730376231226,
     90152126015563685013+I*41483943723517885712,
     90152123908978509257+I*41483960652800311870,
     90152123908975555289+I*41483960652819618444,
     90152123908975555285+I*41483960652819618466,
     90141547549632900143+I*41536213036172872622,
     90141547535852677817+I*41536213095632066790,
     90141547535852660139+I*41536213095632134394,
     90141490504850874507+I*41536395104568470620,
     90141490504779767711+I*41536395104774934998,
     90141490504779767623+I*41536395104774935232,
     89920645785304052749+I*42090018602975420664,
     89920645514803218827+I*42090019229466111860,
     89920645514802889161+I*42090019229466820148,
     89919654155910574757+I*42091907265762947370,
     89919654154715471245+I*42091907267891841936,
     89919654154715469809+I*42091907267891844334,
     86383115244081635331+I*47740076309145754338,
     86383111015591079797+I*47740082659541689650,
     86383111015586036895+I*47740082659548821486,
     86383111015586036889+I*47740082659548821494,
     86368429995178324001+I*47758868341021592128,
     86368429977774396237+I*47758868362049225330,
     86368429977774375645+I*47758868362049248836,
     86368370763586394147+I*47758929759423426200,
     86368370763516658553+I*47758929759491790592,
     86368370763516658471+I*47758929759491790668,
     86169308176086275435+I*47936210214876841866,
     86169307942587803627+I*47936210411372951560,
     86169307942587530103+I*47936210411373168942,
     86168533791771378897+I*47936767354113156950,
     86168533790867558579+I*47936767354725694074,
     86168533790867557525+I*47936767354725694746,
     83624284418363241733+I*49488727189100491852,
     83624281456887832457+I*49488728883560754870,
     83624281456884388929+I*49488728883562599024,
     83624281456884388925+I*49488728883562599026,
     83614651912426749643+I*49493306952070164702,
     83614651901259651897+I*49493306957000432430,
     83614651901259638959+I*49493306957000437714,
     52523989134962571035+I*61476366390114574296,
     52523953169812542963+I*61476379106226648942,
     52523953169770975847+I*61476379106240047580,
     52523953169770975799+I*61476379106240047594,
     52408872043721812345+I*61507478770907991466,
     52408871911023201683+I*61507478802806740982,
     52408871911023048797+I*61507478802806773230,
     52408450862148590713+I*61507546749739538192,
     52408450861664555161+I*61507546749803788978,
     52408450861664554605+I*61507546749803789036,
     51080359145787868867+I*61619480809052844720,
     51080357622305652553+I*61619480894541868072,
     51080357622303906311+I*61619480894541917028,
     51080357622303906308+I*61619480894541917028,
     51075590190819337460+I*61619386213053415442,
     51075590185366444816+I*61619386212792102984,
     51075590185366438584+I*61619386212792102510,
     36242713192224784098+I*60194477432778343558,
     36242696262942357940+I*60194475326193167802,
     36242696262923051366+I*60194475326190213834,
     36242696262923051344+I*60194475326190213830,
     36190443879569797188+I*60183898966847558688,
     36190443820110603020+I*60183898953067336362,
     36190443820110535416+I*60183898953067318684,
     36190261811174199190+I*60183841922065533052,
     36190261810967734812+I*60183841921994426256,
     36190261810967734578+I*60183841921994426168,
     35636638312767249146+I*59962997202518711294,
     35636637686276557950+I*59962996932017877372,
     35636637686275849662+I*59962996932017547706,
     35634749649979722440+I*59962005573125233302,
     35634749647850827874+I*59962005571930129790,
     35634749647850825476+I*59962005571930128354,
     29986580606596915472+I*56425466661296293876,
     29986574256200980160+I*56425462432805738342,
     29986574256193848324+I*56425462432800695440,
     29986574256193848316+I*56425462432800695434,
     29967788574721077682+I*56410781412392982546,
     29967788553693444480+I*56410781394989054782,
     29967788553693420974+I*56410781394989034190,
     29967727156319243610+I*56410722180801052692,
     29967727156250879218+I*56410722180731317098,
     29967727156250879142+I*56410722180731317016,
     29790446700865827944+I*56211659593300933980,
     29790446504369718250+I*56211659359802462172,
     29790446504369500868+I*56211659359802188648,
     29789889561629512860+I*56210885208986037442,
     -9721560687747294266+I*21089392948969885,
     -9721560688359831390+I*21089392045149567,
     -9721560688359832062+I*21089392045148513,
     -11273520522734629168+I*-2523159980459167279,
     -11273522217194892186+I*-2523162941934576555,
     -11273522217196736340+I*-2523162941938020083,
     -11273522217196736342+I*-2523162941938020087,
     -11278100285704302018+I*-2532792486395659369,
     -11278100290634569746+I*-2532792497562757115,
     -11278100290634575030+I*-2532792497562770053,
     -11278113006746649676+I*-2532828462712798125,
     -11278113006760048314+I*-2532828462754365241,
     -11278113006760048328+I*-2532828462754365289,
     -11309212671427992200+I*-2647909588803528743,
     -11309212703326741716+I*-2647909721502139405,
     -11309212703326773964+I*-2647909721502292291,
     -11309280650259538926+I*-2648330770376750375,
     -11309280650323789712+I*-2648330770860785927,
     -11309280650323789770+I*-2648330770860786483,
     -11421214709572845454+I*-3976422486737472221,
     -11421214795061868806+I*-3976424010219688535,
     -11421214795061917762+I*-3976424010221434777,
     -11421214795061917762+I*-3976424010221434780,
     -11421120113573416176+I*-3981191441706003628,
     -11421120113312103718+I*-3981191447158896272,
     -11421120113312103244+I*-3981191447158902504,
     -9996211333298344292+I*-18814068440300556990,
     -9996209226713168536+I*-18814085369582983148,
     -9996209226710214568+I*-18814085369602289722,
     -9996209226710214564+I*-18814085369602289744,
     -9985632867367559422+I*-18866337752955543900,
     -9985632853587337096+I*-18866337812414738068,
     -9985632853587319418+I*-18866337812414805672,
     -9985575822585533786+I*-18866519821351141898,
     -9985575822514426990+I*-18866519821557606276,
     -9985575822514426902+I*-18866519821557606510,
     -9764731103038712028+I*-19420143319758091942,
     -9764730832537878106+I*-19420143946248783138,
     -9764730832537548440+I*-19420143946249491426,
     -9763739473645234036+I*-19422031982545618648,
     -9763739472450130524+I*-19422031984674513214,
     -9763739472450129088+I*-19422031984674515612,
     -6227200561816294610+I*-25070201025928425616,
     -6227196333325739076+I*-25070207376324360928,
     -6227196333320696174+I*-25070207376331492764,
     -6227196333320696168+I*-25070207376331492772,
     -6212515312912983280+I*-25088993057804263406,
     -6212515295509055516+I*-25088993078831896608,
     -6212515295509034924+I*-25088993078831920114,
     -6212456081321053426+I*-25089054476206097478,
     -6212456081251317832+I*-25089054476274461870,
     -6212456081251317750+I*-25089054476274461946,
     -6013393493820934714+I*-25266334931659513144,
     -6013393260322462906+I*-25266335128155622838,
     -6013393260322189382+I*-25266335128155840220,
     -6012619109506038176+I*-25266892070895828228,
     -6012619108602217858+I*-25266892071508365352,
     -6012619108602216804+I*-25266892071508366024,
     -3468369736097901012+I*-26818851905883163130,
     -3468366774622491736+I*-26818853600343426148,
     -3468366774619048208+I*-26818853600345270302,
     -3468366774619048204+I*-26818853600345270304,
     -3458737230161408922+I*-26823431668852835980,
     -3458737218994311176+I*-26823431673783103708,
     -3458737218994298238+I*-26823431673783108992,
     27631925547302769686+I*-38806491106897245574,
     27631961512452797758+I*-38806503823009320220,
     27631961512494364874+I*-38806503823022718858,
     27631961512494364922+I*-38806503823022718872,
     27747042638543528376+I*-38837603487690662744,
     27747042771242139038+I*-38837603519589412260,
     27747042771242291924+I*-38837603519589444508,
     27747463820116750008+I*-38837671466522209470,
     27747463820600785560+I*-38837671466586460256,
     27747463820600786116+I*-38837671466586460314,
     29075555536477471854+I*-38949605525835515998,
     29075557059959688168+I*-38949605611324539350 ],
   /* k=58 */ [ 29075557059961434410+I*-38949605611324588306,
     29075557059961434413+I*-38949605611324588306,
     29080324491446003261+I*-38949510929836086720,
     29080324496898895905+I*-38949510929574774262,
     29080324496898902137+I*-38949510929574773788,
     43913201490040556623+I*-37524602149561014836,
     43913218419322982781+I*-37524600042975839080,
     43913218419342289355+I*-37524600042972885112,
     43913218419342289377+I*-37524600042972885108,
     43965470802695543533+I*-37514023683630229966,
     43965470862154737701+I*-37514023669850007640,
     43965470862154805305+I*-37514023669849989962,
     43965652871091141531+I*-37513966638848204330,
     43965652871297605909+I*-37513966638777097534,
     43965652871297606143+I*-37513966638777097446,
     44519276369498091575+I*-37293121919301382572,
     44519276995988782771+I*-37293121648800548650,
     44519276995989491059+I*-37293121648800218984,
     44521165032285618281+I*-37292130289907904580,
     44521165034414512847+I*-37292130288712801068,
     44521165034414515245+I*-37292130288712799632,
     50169334075668425249+I*-33755591378078965154,
     50169340426064360561+I*-33755587149588409620,
     50169340426071492397+I*-33755587149583366718,
     50169340426071492405+I*-33755587149583366712,
     50188126107544263039+I*-33740906129175653824,
     50188126128571896241+I*-33740906111771726060,
     50188126128571919747+I*-33740906111771705468,
     105244907761097260835+I*16457359987708293798,
     105244969158471438199+I*16457419201896275296,
     105244969158539802591+I*16457419201966010890,
     105244969158539802667+I*16457419201966010972,
     105422249613924853865+I*16656481789396394008,
     105422249810420963559+I*16656482022894865816,
     105422249810421180941+I*16656482022895139340,
     105422806753161168949+I*16657256173711290546,
     105422806753773706073+I*16657256174615110864,
     105422806753773706745+I*16657256174615111918,
     106974766588148503851+I*19201505547119427710,
     106974768282608766869+I*19201508508594836986,
     106974768282610611023+I*19201508508598280514,
     106974768282610611025+I*19201508508598280518,
     106979346351118176701+I*19211138053055919800,
     106979346356048444429+I*19211138064223017546,
     106979346356048449713+I*19211138064223030484,
     118962405789162586295+I*50301800830520098408,
     118962418505274660941+I*50301836795670126480,
     118962418505288059579+I*50301836795711693596,
     118962418505288059593+I*50301836795711693644,
     118993518169956003465+I*50416917921760857098,
     118993518201854752981+I*50416918054459467760,
     118993518201854785229+I*50416918054459620646,
     118993586148787550191+I*50417339103334078730,
     118993586148851800977+I*50417339103818114282,
     118993586148851801035+I*50417339103818114838,
     119105520208100856719+I*51745430819694800576,
     119105520293589880071+I*51745432343177016890,
     119105520293589929027+I*51745432343178763132,
     119105520293589929027+I*51745432343178763135,
     119105425612101427441+I*51750199774663331983,
     119105425611840114983+I*51750199780116224627,
     119105425611840114509+I*51750199780116230859,
     117680516831826355557+I*66583076773257885345,
     117680514725241179801+I*66583093702540311503,
     117680514725238225833+I*66583093702559618077,
     117680514725238225829+I*66583093702559618099,
     117669938365895570687+I*66635346085912872255,
     117669938352115348361+I*66635346145372066423,
     117669938352115330683+I*66635346145372134027,
     117669881321113545051+I*66635528154308470253,
     117669881321042438255+I*66635528154514934631,
     117669881321042438167+I*66635528154514934865,
     117449036601566723293+I*67189151652715420297,
     117449036331065889371+I*67189152279206111493,
     117449036331065559705+I*67189152279206819781,
     117448044972173245301+I*67191040315502947003,
     44579903589475917313+I*202403736630293628812,
     44579903588280813801+I*202403736632422523378,
     44579903588280812365+I*202403736632422525776,
     41043364677646977887+I*208051905673676435780,
     41043360449156422353+I*208051912024072371092,
     41043360449151379451+I*208051912024079502928,
     41043360449151379445+I*208051912024079502936,
     41028679428743666557+I*208070697705552273570,
     41028679411339738793+I*208070697726579906772,
     41028679411339718201+I*208070697726579930278,
     41028620197151736703+I*208070759123954107642,
     41028620197082001109+I*208070759124022472034,
     41028620197082001027+I*208070759124022472110,
     40829557609651617991+I*208248039579407523308,
     40829557376153146183+I*208248039775903633002,
     40829557376152872659+I*208248039775903850384,
     40828783225336721453+I*208248596718643838392,
     40828783224432901135+I*208248596719256375516,
     40828783224432900081+I*208248596719256376188,
     38284533851928584289+I*209800556553631173294,
     38284530890453175013+I*209800558248091436312,
     38284530890449731485+I*209800558248093280466,
     38284530890449731481+I*209800558248093280468,
     38274901345992092199+I*209805136316600846144,
     38274901334824994453+I*209805136321531113872,
     38274901334824981515+I*209805136321531119156,
     7184238568527913591+I*221788195754645255738,
     7184202603377885519+I*221788208470757330384,
     7184202603336318403+I*221788208470770729022,
     7184202603336318355+I*221788208470770729036,
     7069121477287154901+I*221819308135438672908,
     7069121344588544239+I*221819308167337422424,
     7069121344588391353+I*221819308167337454672,
     7068700295713933269+I*221819376114270219634,
     7068700295229897717+I*221819376114334470420,
     7068700295229897161+I*221819376114334470478,
     5740608579353211423+I*221931310173583526162,
     5740607055870995109+I*221931310259072549514,
     5740607055869248867+I*221931310259072598470,
     5740607055869248864+I*221931310259072598470,
     5735839624384680016+I*221931215577584096884,
     5735839618931787372+I*221931215577322784426,
     5735839618931781140+I*221931215577322783952,
     -9097037374209873346+I*220506306797309025000,
     -9097054303492299504+I*220506304690723849244,
     -9097054303511606078+I*220506304690720895276,
     -9097054303511606100+I*220506304690720895272,
     -9149306686864860256+I*220495728331378240130,
     -9149306746324054424+I*220495728317598017804,
     -9149306746324122028+I*220495728317598000126,
     -9149488755260458254+I*220495671286596214494,
     -9149488755466922632+I*220495671286525107698,
     -9149488755466922866+I*220495671286525107610,
     -9703112253667408298+I*220274826567049392736,
     -9703112880158099494+I*220274826296548558814,
     -9703112880158807782+I*220274826296548229148,
     -9705000916454935004+I*220273834937655914744,
     -9705000918583829570+I*220273834936460811232,
     -9705000918583831968+I*220273834936460809796,
     -15353169959837741972+I*216737296025826975318,
     -15353176310233677284+I*216737291797336419784,
     -15353176310240809120+I*216737291797331376882,
     -15353176310240809128+I*216737291797331376876,
     -15371961991713579762+I*216722610776923663988,
     -15371962012741212964+I*216722610759519736224,
     -15371962012741236470+I*216722610759519715632,
     -70428743645266577558+I*166524344660039716366,
     -70428805042640754922+I*166524285445851734868,
     -70428805042709119314+I*166524285445781999274,
     -70428805042709119390+I*166524285445781999192,
     -70606085498094170588+I*166325222858351616156,
     -70606085694590280282+I*166325222624853144348,
     -70606085694590497664+I*166325222624852870824,
     -70606642637330485672+I*166324448474036719618,
     -70606642637943022796+I*166324448473132899300,
     -70606642637943023468+I*166324448473132898246,
     -72158602472317820574+I*163780199100628582454,
     -72158604166778083592+I*163780196139153173178,
     -72158604166779927746+I*163780196139149729650,
     -72158604166779927748+I*163780196139149729646,
     -72163182235287493424+I*163770566594692090364,
     -72163182240217761152+I*163770566583524992618,
     -72163182240217766436+I*163770566583524979680,
     -84146241673331903018+I*132679903817227911756,
     -84146254389443977664+I*132679867852077883684,
     -84146254389457376302+I*132679867852036316568,
     -84146254389457376316+I*132679867852036316520,
     -84177354054125320188+I*132564786725987153066,
     -84177354086024069704+I*132564786593288542404,
     -84177354086024101952+I*132564786593288389518,
     -84177422032956866914+I*132564365544413931434,
     -84177422033021117700+I*132564365543929895882,
     -84177422033021117758+I*132564365543929895326,
     -84289356092270173442+I*131236273828053209588,
     -84289356177759196794+I*131236272304570993274,
     -84289356177759245750+I*131236272304569247032,
     -84289356177759245750+I*131236272304569247029,
     -84289261496270744164+I*131231504873084678181,
     -84289261496009431706+I*131231504867631785537,
     -84289261496009431232+I*131231504867631779305,
     -82864352715995672280+I*116398627874490124819,
     -82864350609410496524+I*116398610945207698661,
     -82864350609407542556+I*116398610945188392087,
     -82864350609407542552+I*116398610945188392065,
     -82853774250064887410+I*116346358561835137909,
     -82853774236284665084+I*116346358502375943741,
     -82853774236284647406+I*116346358502375876137,
     -82853717205282861774+I*116346176493439539911,
     -82853717205211754978+I*116346176493233075533,
     -82853717205211754890+I*116346176493233075299,
     -82632872485736040016+I*115792552995032589867,
     -82632872215235206094+I*115792552368541898671,
     -82632872215234876428+I*115792552368541190383,
     -82631880856342562024+I*115790664332245063161,
     -9763739473645234036+I*-19422031982545618648,
     -9763739472450130524+I*-19422031984674513214,
     -9763739472450129088+I*-19422031984674515612,
     -6227200561816294610+I*-25070201025928425616,
     -6227196333325739076+I*-25070207376324360928,
     -6227196333320696174+I*-25070207376331492764,
     -6227196333320696168+I*-25070207376331492772,
     -6212515312912983280+I*-25088993057804263406,
     -6212515295509055516+I*-25088993078831896608,
     -6212515295509034924+I*-25088993078831920114,
     -6212456081321053426+I*-25089054476206097478,
     -6212456081251317832+I*-25089054476274461870,
     -6212456081251317750+I*-25089054476274461946,
     -6013393493820934714+I*-25266334931659513144,
     -6013393260322462906+I*-25266335128155622838,
     -6013393260322189382+I*-25266335128155840220,
     -6012619109506038176+I*-25266892070895828228,
     -6012619108602217858+I*-25266892071508365352,
     -6012619108602216804+I*-25266892071508366024,
     -3468369736097901012+I*-26818851905883163130,
     -3468366774622491736+I*-26818853600343426148,
     -3468366774619048208+I*-26818853600345270302,
     -3468366774619048204+I*-26818853600345270304,
     -3458737230161408922+I*-26823431668852835980,
     -3458737218994311176+I*-26823431673783103708,
     -3458737218994298238+I*-26823431673783108992,
     27631925547302769686+I*-38806491106897245574,
     27631961512452797758+I*-38806503823009320220,
     27631961512494364874+I*-38806503823022718858,
     27631961512494364922+I*-38806503823022718872,
     27747042638543528376+I*-38837603487690662744,
     27747042771242139038+I*-38837603519589412260,
     27747042771242291924+I*-38837603519589444508,
     27747463820116750008+I*-38837671466522209470,
     27747463820600785560+I*-38837671466586460256,
     27747463820600786116+I*-38837671466586460314,
     29075555536477471854+I*-38949605525835515998,
     29075557059959688168+I*-38949605611324539350 ],
   /* k=59 */ [ -314217976952317257196+I*-49473192061928562473,
     -314217976952317257193+I*-49473192061928562473,
     -314213209520832688345+I*-49473097380440060887,
     29080324491446003261+I*-38949510929836086720,
     29080324496898895905+I*-38949510929574774262,
     29080324496898902137+I*-38949510929574773788,
     43913201490040556623+I*-37524602149561014836,
     43913218419322982781+I*-37524600042975839080,
     43913218419342289355+I*-37524600042972885112,
     43913218419342289377+I*-37524600042972885108,
     43965470802695543533+I*-37514023683630229966,
     43965470862154737701+I*-37514023669850007640,
     43965470862154805305+I*-37514023669849989962,
     43965652871091141531+I*-37513966638848204330,
     43965652871297605909+I*-37513966638777097534,
     43965652871297606143+I*-37513966638777097446,
     44519276369498091575+I*-37293121919301382572,
     44519276995988782771+I*-37293121648800548650,
     44519276995989491059+I*-37293121648800218984,
     44521165032285618281+I*-37292130289907904580,
     44521165034414512847+I*-37292130288712801068,
     44521165034414515245+I*-37292130288712799632,
     50169334075668425249+I*-33755591378078965154,
     50169340426064360561+I*-33755587149588409620,
     50169340426071492397+I*-33755587149583366718,
     50169340426071492405+I*-33755587149583366712,
     50188126107544263039+I*-33740906129175653824,
     50188126128571896241+I*-33740906111771726060,
     50188126128571919747+I*-33740906111771705468,
     105244907761097260835+I*16457359987708293798,
     105244969158471438199+I*16457419201896275296,
     105244969158539802591+I*16457419201966010890,
     105244969158539802667+I*16457419201966010972,
     105422249613924853865+I*16656481789396394008,
     105422249810420963559+I*16656482022894865816,
     105422249810421180941+I*16656482022895139340,
     105422806753161168949+I*16657256173711290546,
     105422806753773706073+I*16657256174615110864,
     105422806753773706745+I*16657256174615111918,
     106974766588148503851+I*19201505547119427710,
     106974768282608766869+I*19201508508594836986,
     106974768282610611023+I*19201508508598280514,
     106974768282610611025+I*19201508508598280518,
     106979346351118176701+I*19211138053055919800,
     106979346356048444429+I*19211138064223017546,
     106979346356048449713+I*19211138064223030484,
     118962405789162586295+I*50301800830520098408,
     118962418505274660941+I*50301836795670126480,
     118962418505288059579+I*50301836795711693596,
     118962418505288059593+I*50301836795711693644,
     118993518169956003465+I*50416917921760857098,
     118993518201854752981+I*50416918054459467760,
     118993518201854785229+I*50416918054459620646,
     118993586148787550191+I*50417339103334078730,
     118993586148851800977+I*50417339103818114282,
     118993586148851801035+I*50417339103818114838,
     119105520208100856719+I*51745430819694800576,
     119105520293589880071+I*51745432343177016890,
     119105520293589929027+I*51745432343178763132,
     119105520293589929027+I*51745432343178763135,
     119105425612101427441+I*51750199774663331983,
     119105425611840114983+I*51750199780116224627,
     119105425611840114509+I*51750199780116230859,
     117680516831826355557+I*66583076773257885345,
     117680514725241179801+I*66583093702540311503,
     117680514725238225833+I*66583093702559618077,
     117680514725238225829+I*66583093702559618099,
     117669938365895570687+I*66635346085912872255,
     117669938352115348361+I*66635346145372066423,
     117669938352115330683+I*66635346145372134027,
     72330187785680673239+I*226947175509902815469,
     72330130754678887607+I*226947357518839151695,
     72330130754607780811+I*226947357519045616073,
     72330130754607780723+I*226947357519045616307,
     72109286035132065849+I*227500981017246101739,
     72109285764631231927+I*227500981643736792935,
     72109285764630902261+I*227500981643737501223,
     72108294405738587857+I*227502869680033628445,
     72108294404543484345+I*227502869682162523011,
     72108294404543482909+I*227502869682162525409,
     68571755493909648431+I*233151038723416435413,
     68571751265419092897+I*233151045073812370725,
     68571751265414049995+I*233151045073819502561,
     68571751265414049989+I*233151045073819502569,
     68557070245006337101+I*233169830755292273203,
     68557070227602409337+I*233169830776319906405,
     68557070227602388745+I*233169830776319929911,
     18358804128122389479+I*288226612408845270999,
     18358744913934407981+I*288226673806219448363,
     18358744913864672387+I*288226673806287812755,
     18358744913864672305+I*288226673806287812831,
     18159682326434289269+I*288403954261672864029,
     18159682092935817461+I*288403954458168973723,
     18159682092935543937+I*288403954458169191105,
     18158907942119392731+I*288404511400909179113,
     18158907941215572413+I*288404511401521716237,
     18158907941215571359+I*288404511401521716909,
     15614658568711255567+I*289956471235896514015,
     15614655607235846291+I*289956472930356777033,
     15614655607232402763+I*289956472930358621187,
     15614655607232402759+I*289956472930358621189,
     15605026062774763477+I*289961050998866186865,
     15605026051607665731+I*289961051003796454593,
     15605026051607652793+I*289961051003796459877,
     -15485636714689415131+I*301944110436910596459,
     -15485672679839443203+I*301944123153022671105,
     -15485672679881010319+I*301944123153036069743,
     -15485672679881010367+I*301944123153036069757,
     -15600753805930173821+I*301975222817704013629,
     -15600753938628784483+I*301975222849602763145,
     -15600753938628937369+I*301975222849602795393,
     -15601174987503395453+I*301975290796535560355,
     -15601174987987431005+I*301975290796599811141,
     -15601174987987431561+I*301975290796599811199,
     -16929266703864117299+I*302087224855848866883,
     -16929268227346333613+I*302087224941337890235,
     -16929268227348079855+I*302087224941337939191,
     -16929268227348079858+I*302087224941337939191,
     -16934035658832648706+I*302087130259849437605,
     -360227569671111340312+I*291563543809245463438,
     -360227569676564232956+I*291563543808984150980,
     -360227569676564239188+I*291563543808984150506,
     -375060446669705893674+I*290138635028970391554,
     -375060463598988319832+I*290138632922385215798,
     -375060463599007626406+I*290138632922382261830,
     -375060463599007626428+I*290138632922382261826,
     -375112715982360880584+I*290128056563039606684,
     -375112716041820074752+I*290128056549259384358,
     -375112716041820142356+I*290128056549259366680,
     -375112898050756478582+I*290127999518257581048,
     -375112898050962942960+I*290127999518186474252,
     -375112898050962943194+I*290127999518186474164,
     -375666521549163428626+I*289907154798710759290,
     -375666522175654119822+I*289907154528209925368,
     -375666522175654828110+I*289907154528209595702,
     -375668410211950955332+I*289906163169317281298,
     -375668410214079849898+I*289906163168122177786,
     -375668410214079852296+I*289906163168122176350,
     -381316579255333762300+I*286369624257488341872,
     -381316585605729697612+I*286369620028997786338,
     -381316585605736829448+I*286369620028992743436,
     -381316585605736829456+I*286369620028992743430,
     -381335371287209600090+I*286354939008585030542,
     -381335371308237233292+I*286354938991181102778,
     -381335371308237256798+I*286354938991181082186,
     -436392152940762597886+I*236156672891701082920,
     -436392214338136775250+I*236156613677513101422,
     -436392214338205139642+I*236156613677443365828,
     -436392214338205139718+I*236156613677443365746,
     -436569494793590190916+I*235957551090012982710,
     -436569494990086300610+I*235957550856514510902,
     -436569494990086517992+I*235957550856514237378,
     -436570051932826506000+I*235956776705698086172,
     -436570051933439043124+I*235956776704794265854,
     -436570051933439043796+I*235956776704794264800,
     -438122011767813840902+I*233412527332289949008,
     -438122013462274103920+I*233412524370814539732,
     -438122013462275948074+I*233412524370811096204,
     -438122013462275948076+I*233412524370811096200,
     -438126591530783513752+I*233402894826353456918,
     -438126591535713781480+I*233402894815186359172,
     -438126591535713786764+I*233402894815186346234,
     -450109650968827923346+I*202312232048889278310,
     -450109663684939997992+I*202312196083739250238,
     -450109663684953396630+I*202312196083697683122,
     -450109663684953396644+I*202312196083697683074,
     -450140763349621340516+I*202197114957648519620,
     -450140763381520090032+I*202197114824949908958,
     -450140763381520122280+I*202197114824949756072,
     -450140831328452887242+I*202196693776075297988,
     -450140831328517138028+I*202196693775591262436,
     -450140831328517138086+I*202196693775591261880,
     -450252765387766193770+I*200868602059714576142,
     -450252765473255217122+I*200868600536232359828,
     -450252765473255266078+I*200868600536230613586,
     -450252765473255266078+I*200868600536230613583,
     -450252670791766764492+I*200863833104746044735,
     -450252670791505452034+I*200863833099293152091,
     -450252670791505451560+I*200863833099293145859,
     -448827762011491692608+I*186030956106151491373,
     -448827759904906516852+I*186030939176869065215,
     -448827759904903562884+I*186030939176849758641,
     -448827759904903562880+I*186030939176849758619,
     -448817183545560907738+I*185978686793496504463,
     -448817183531780685412+I*185978686734037310295,
     -448817183531780667734+I*185978686734037242691,
     -403477432965346010290+I*25666857369506561249,
     -403477375934344224658+I*25666675360570225023,
     -403477375934273117862+I*25666675360363760645,
     -403477375934273117774+I*25666675360363760411,
     -403256531214797402900+I*25113051862163274979,
     -403256530944296568978+I*25113051235672583783,
     -403256530944296239312+I*25113051235671875495,
     -403255539585403924908+I*25111163199375748273,
     -403255539584208821396+I*25111163197246853707,
     -403255539584208819960+I*25111163197246851309,
     -399719000673574985482+I*19462994155992941305,
     -399718996445084429948+I*19462987805597005993,
     -399718996445079387046+I*19462987805589874157,
     -399718996445079387040+I*19462987805589874149,
     -399704315424671674152+I*19444202124117103515,
     -399704315407267746388+I*19444202103089470313,
     -399704315407267725796+I*19444202103089446807,
     -349506049307787726530+I*-35612579529435894281,
     -349505990093599745032+I*-35612640926810071645,
     -349505990093530009438+I*-35612640926878436037,
     -349505990093530009356+I*-35612640926878436113,
     -349306927506099626320+I*-35789921382263487311,
     -349306927272601154512+I*-35789921578759597005,
     -349306927272600880988+I*-35789921578759814387,
     -349306153121784729782+I*-35790478521499802395,
     -349306153120880909464+I*-35790478522112339519,
     -349306153120880908410+I*-35790478522112340191,
     -346761903748376592618+I*-37342438356487137297,
     -346761900786901183342+I*-37342440050947400315,
     -346761900786897739814+I*-37342440050949244469,
     -346761900786897739810+I*-37342440050949244471,
     -346752271242440100528+I*-37347018119456810147,
     -346752271231273002782+I*-37347018124387077875,
     -346752271231272989844+I*-37347018124387083159,
     -315661608464975921920+I*-49330077557501219741,
     -315661572499825893848+I*-49330090273613294387,
     -315661572499784326732+I*-49330090273626693025,
     -315661572499784326684+I*-49330090273626693039,
     -315546491373735163230+I*-49361189938294636911,
     -315546491241036552568+I*-49361189970193386427,
     -315546491241036399682+I*-49361189970193418675,
     -315546070192161941598+I*-49361257917126183637,
     -315546070191677906046+I*-49361257917190434423,
     -315546070191677905490+I*-49361257917190434481,
     -314217978475801219752+I*-49473191976439490165,
     -314217976952319003438+I*-49473192061928513517 ],
   /* k=60 */ [ -453482633415639990304+I*-781400010652920603129,
     -453482633415639990301+I*-781400010652920603129,
     -453477865984155421453+I*-781399915971432101543,
     -453477865978702528809+I*-781399915971170789085,
     -453477865978702522577+I*-781399915971170788611,
     -438644988985560868091+I*-779975007191157029659,
     -438644972056278441933+I*-779975005084571853903,
     -438644972056259135359+I*-779975005084568899935,
     -438644972056259135337+I*-779975005084568899931,
     -438592719672905881181+I*-779964428725226244789,
     -438592719613446687013+I*-779964428711446022463,
     -438592719613446619409+I*-779964428711446004785,
     -278280890248915937967+I*-734624678145011347341,
     -278280708239979601741+I*-734624621114009561709,
     -278280708239773137363+I*-734624621113938454913,
     -278280708239773137129+I*-734624621113938454825,
     -277727084741572651697+I*-734403776394462739951,
     -277727084115081960501+I*-734403776123961906029,
     -277727084115081252213+I*-734403776123961576363,
     -277725196078785124991+I*-734402784765069261959,
     -277725196076656230425+I*-734402784763874158447,
     -277725196076656228027+I*-734402784763874157011,
     -272077027035402318023+I*-730866245853240322533,
     -272077020685006382711+I*-730866241624749766999,
     -272077020684999250875+I*-730866241624744724097,
     -272077020684999250867+I*-730866241624744724091,
     -272058235003526480233+I*-730851560604337011203,
     -272058234982498847031+I*-730851560586933083439,
     -272058234982498823525+I*-730851560586933062847,
     -217001453349973482437+I*-680653294487453063581,
     -217001391952599305073+I*-680653235273265082083,
     -217001391952530940681+I*-680653235273195346489,
     -217001391952530940605+I*-680653235273195346407,
     -216824111497145889407+I*-680454172685764963371,
     -216824111300649779713+I*-680454172452266491563,
     -216824111300649562331+I*-680454172452266218039,
     -216823554357909574323+I*-680453398301450066833,
     -216823554357297037199+I*-680453398300546246515,
     -216823554357297036527+I*-680453398300546245461,
     -215271594522922239421+I*-677909148928041929669,
     -215271592828461976403+I*-677909145966566520393,
     -215271592828460132249+I*-677909145966563076865,
     -215271592828460132247+I*-677909145966563076861,
     -215267014759952566571+I*-677899516422105437579,
     106979346351118176701+I*19211138053055919800,
     106979346356048444429+I*19211138064223017546,
     106979346356048449713+I*19211138064223030484,
     118962405789162586295+I*50301800830520098408,
     118962418505274660941+I*50301836795670126480,
     118962418505288059579+I*50301836795711693596,
     118962418505288059593+I*50301836795711693644,
     118993518169956003465+I*50416917921760857098,
     118993518201854752981+I*50416918054459467760,
     118993518201854785229+I*50416918054459620646,
     118993586148787550191+I*50417339103334078730,
     118993586148851800977+I*50417339103818114282,
     118993586148851801035+I*50417339103818114838,
     119105520208100856719+I*51745430819694800576,
     119105520293589880071+I*51745432343177016890,
     119105520293589929027+I*51745432343178763132,
     119105520293589929027+I*51745432343178763135,
     119105425612101427441+I*51750199774663331983,
     119105425611840114983+I*51750199780116224627,
     119105425611840114509+I*51750199780116230859,
     117680516831826355557+I*66583076773257885345,
     117680514725241179801+I*66583093702540311503,
     117680514725238225833+I*66583093702559618077,
     117680514725238225829+I*66583093702559618099,
     117669938365895570687+I*66635346085912872255,
     117669938352115348361+I*66635346145372066423,
     117669938352115330683+I*66635346145372134027,
     72330187785680673239+I*226947175509902815469,
     72330130754678887607+I*226947357518839151695,
     72330130754607780811+I*226947357519045616073,
     72330130754607780723+I*226947357519045616307,
     72109286035132065849+I*227500981017246101739,
     72109285764631231927+I*227500981643736792935,
     72109285764630902261+I*227500981643737501223,
     72108294405738587857+I*227502869680033628445,
     72108294404543484345+I*227502869682162523011,
     72108294404543482909+I*227502869682162525409,
     68571755493909648431+I*233151038723416435413,
     68571751265419092897+I*233151045073812370725,
     68571751265414049995+I*233151045073819502561,
     68571751265414049989+I*233151045073819502569,
     68557070245006337101+I*233169830755292273203,
     68557070227602409337+I*233169830776319906405,
     68557070227602388745+I*233169830776319929911,
     18358804128122389479+I*288226612408845270999,
     18358744913934407981+I*288226673806219448363,
     18358744913864672387+I*288226673806287812755,
     18358744913864672305+I*288226673806287812831,
     18159682326434289269+I*288403954261672864029,
     18159682092935817461+I*288403954458168973723,
     18159682092935543937+I*288403954458169191105,
     18158907942119392731+I*288404511400909179113,
     18158907941215572413+I*288404511401521716237,
     18158907941215571359+I*288404511401521716909,
     15614658568711255567+I*289956471235896514015,
     15614655607235846291+I*289956472930356777033,
     15614655607232402763+I*289956472930358621187,
     15614655607232402759+I*289956472930358621189,
     15605026062774763477+I*289961050998866186865,
     15605026051607665731+I*289961051003796454593,
     15605026051607652793+I*289961051003796459877,
     -15485636714689415131+I*301944110436910596459,
     -15485672679839443203+I*301944123153022671105,
     -15485672679881010319+I*301944123153036069743,
     -15485672679881010367+I*301944123153036069757,
     -15600753805930173821+I*301975222817704013629,
     -15600753938628784483+I*301975222849602763145,
     -15600753938628937369+I*301975222849602795393,
     -381564163234124957697+I*371607551081264161947,
     -381564584282999415781+I*371607619028196926909,
     -381564584283483451333+I*371607619028261177695,
     -381564584283483451889+I*371607619028261177753,
     -382892675999360137627+I*371719553087510233437,
     -382892677522842353941+I*371719553172999256789,
     -382892677522844100183+I*371719553172999305745,
     -382892677522844100186+I*371719553172999305745,
     -382897444954328669034+I*371719458491510804159,
     -382897444959781561678+I*371719458491249491701,
     -382897444959781567910+I*371719458491249491227,
     -397730321952923222396+I*370294549711235732275,
     -397730338882205648554+I*370294547604650556519,
     -397730338882224955128+I*370294547604647602551,
     -397730338882224955150+I*370294547604647602547,
     -397782591265578209306+I*370283971245304947405,
     -397782591325037403474+I*370283971231524725079,
     -397782591325037471078+I*370283971231524707401,
     -558094420689568152520+I*324944220665090049957,
     -558094602698504488746+I*324944163634088264325,
     -558094602698710953124+I*324944163634017157529,
     -558094602698710953358+I*324944163634017157441,
     -558648226196911438790+I*324723318914541442567,
     -558648226823402129986+I*324723318644040608645,
     -558648226823402838274+I*324723318644040278979,
     -558650114859698965496+I*324722327285147964575,
     -558650114861827860062+I*324722327283952861063,
     -558650114861827862460+I*324722327283952859627,
     -564298283903081772464+I*321185788373319025149,
     -564298290253477707776+I*321185784144828469615,
     -564298290253484839612+I*321185784144823426713,
     -564298290253484839620+I*321185784144823426707,
     -564317075934957610254+I*321171103124415713819,
     -564317075955985243456+I*321171103107011786055,
     -564317075955985266962+I*321171103107011765463,
     -619373857588510608050+I*270972837007531766197,
     -619373918985884785414+I*270972777793343784699,
     -619373918985953149806+I*270972777793274049105,
     -619373918985953149882+I*270972777793274049023,
     -619551199441338201080+I*270773715205843665987,
     -619551199637834310774+I*270773714972345194179,
     -619551199637834528156+I*270773714972344920655,
     -619551756580574516164+I*270772940821528769449,
     -619551756581187053288+I*270772940820624949131,
     -619551756581187053960+I*270772940820624948077,
     -621103716415561851066+I*268228691448120632285,
     -621103718110022114084+I*268228688486645223009,
     -621103718110023958238+I*268228688486641779481,
     -621103718110023958240+I*268228688486641779477,
     -621108296178531523916+I*268219058942184140195,
     -943354657289602267188+I*-428891595532977217184,
     -943354657294532534916+I*-428891595544144314930,
     -943354657294532540200+I*-428891595544144327868,
     -955337716727646676782+I*-459982258310441395792,
     -955337729443758751428+I*-459982294275591423864,
     -955337729443772150066+I*-459982294275632990980,
     -955337729443772150080+I*-459982294275632991028,
     -955368829108440093952+I*-460097375401682154482,
     -955368829140338843468+I*-460097375534380765144,
     -955368829140338875716+I*-460097375534380918030,
     -955368897087271640678+I*-460097796583255376114,
     -955368897087335891464+I*-460097796583739411666,
     -955368897087335891522+I*-460097796583739412222,
     -955480831146584947206+I*-461425888299616097960,
     -955480831232073970558+I*-461425889823098314274,
     -955480831232074019514+I*-461425889823100060516,
     -955480831232074019514+I*-461425889823100060519,
     -955480736550585517928+I*-461430657254584629367,
     -955480736550324205470+I*-461430657260037522011,
     -955480736550324204996+I*-461430657260037528243,
     -954055827770310446044+I*-476263534253179182729,
     -954055825663725270288+I*-476263551182461608887,
     -954055825663722316320+I*-476263551182480915461,
     -954055825663722316316+I*-476263551182480915483,
     -954045249304379661174+I*-476315803565834169639,
     -954045249290599438848+I*-476315803625293363807,
     -954045249290599421170+I*-476315803625293431411,
     -908705498724164763726+I*-636627632989824112853,
     -908705441693162978094+I*-636627814998760449079,
     -908705441693091871298+I*-636627814998966913457,
     -908705441693091871210+I*-636627814998966913691,
     -908484596973616156336+I*-637181438497167399123,
     -908484596703115322414+I*-637181439123658090319,
     -908484596703114992748+I*-637181439123658798607,
     -908483605344222678344+I*-637183327159954925829,
     -908483605343027574832+I*-637183327162083820395,
     -908483605343027573396+I*-637183327162083822793,
     -904947066432393738918+I*-642831496203337732797,
     -904947062203903183384+I*-642831502553733668109,
     -904947062203898140482+I*-642831502553740799945,
     -904947062203898140476+I*-642831502553740799953,
     -904932381183490427588+I*-642850288235213570587,
     -904932381166086499824+I*-642850288256241203789,
     -904932381166086479232+I*-642850288256241227295,
     -854734115066606479966+I*-697907069888766568383,
     -854734055852418498468+I*-697907131286140745747,
     -854734055852348762874+I*-697907131286209110139,
     -854734055852348762792+I*-697907131286209110215,
     -854534993264918379756+I*-698084411741594161413,
     -854534993031419907948+I*-698084411938090271107,
     -854534993031419634424+I*-698084411938090488489,
     -854534218880603483218+I*-698084968880830476497,
     -854534218879699662900+I*-698084968881443013621,
     -854534218879699661846+I*-698084968881443014293,
     -851989969507195346054+I*-699636928715817811399,
     -851989966545719936778+I*-699636930410278074417,
     -851989966545716493250+I*-699636930410279918571,
     -851989966545716493246+I*-699636930410279918573,
     -851980337001258853964+I*-699641508478787484249,
     -851980336990091756218+I*-699641508483717751977,
     -851980336990091743280+I*-699641508483717757261,
     -820889674223794675356+I*-711624567916831893843,
     -820889638258644647284+I*-711624580632943968489,
     -820889638258603080168+I*-711624580632957367127,
     -820889638258603080120+I*-711624580632957367141,
     -820774557132553916666+I*-711655680297625311013,
     -820774556999855306004+I*-711655680329524060529,
     -820774556999855153118+I*-711655680329524092777,
     -454811147704359132790+I*-781288008561185459331,
     -454810726655484674706+I*-781288076508118224293,
     -454810726655000639154+I*-781288076508182475079,
     -454810726655000638598+I*-781288076508182475137,
     -453482634939123952860+I*-781400010567431530821,
     -453482633415641736546+I*-781400010652920554173 ]
 ]};

gp_inline__location="../r5dragon.tex:22125";
 \\ convex hulls around R5 quad k, first side horizontal
 Qhull_generated_samples = {[
   /* k=0 */ [ 0+I*0,1+I*0,1+I*1,0+I*1 ],
   /* k=1 */ [ -2+I*0,1+I*0,1+I*3,-2+I*3 ],
   /* k=2 */ [ -4+I*-4,-1+I*-4,1+I*0,1+I*3,-3+I*5,-6+I*5,-8+I*1,
     -8+I*-2 ],
   /* k=3 */ [ -6+I*-18,-3+I*-18,5+I*-12,7+I*-8,7+I*-5,1+I*3,
     -3+I*5,-6+I*5,-14+I*-1,-16+I*-5,-16+I*-8,-10+I*-16 ],
   /* k=4 */ [ -2+I*-40,1+I*-40,23+I*-36,31+I*-30,33+I*-26,
     33+I*-23,29+I*-1,23+I*7,19+I*9,16+I*9,-6+I*5,-14+I*-1,
     -16+I*-5,-16+I*-8,-12+I*-30,-6+I*-38 ],
   /* k=5 */ [ 46+I*-54,49+I*-54,71+I*-50,79+I*-44,81+I*-40,95+I*8,
     95+I*11,91+I*33,85+I*41,81+I*43,33+I*57,30+I*57,8+I*53,0+I*47,
     -2+I*43,-16+I*-5,-16+I*-8,-12+I*-30,-6+I*-38,-2+I*-40 ],
   /* k=6 */ [ 46+I*-54,49+I*-54,71+I*-50,79+I*-44,155+I*38,
     157+I*42,171+I*90,171+I*93,167+I*115,161+I*123,79+I*199,
     75+I*201,27+I*215,24+I*215,2+I*211,-6+I*205,-82+I*123,
     -84+I*119,-98+I*71,-98+I*68,-94+I*46,-88+I*38,-6+I*-38,
     -2+I*-40 ],
   /* k=7 */ [ -188+I*-142,-185+I*-142,-163+I*-138,71+I*-50,
     79+I*-44,155+I*38,157+I*42,171+I*90,171+I*93,167+I*115,
     79+I*349,73+I*357,-9+I*433,-13+I*435,-61+I*449,-64+I*449,
     -86+I*445,-320+I*357,-328+I*351,-404+I*269,-406+I*265,
     -420+I*217,-420+I*214,-416+I*192,-328+I*-42,-322+I*-50,
     -240+I*-126,-236+I*-128 ],
   /* k=8 */ [ -246+I*-698,-243+I*-698,-221+I*-694,13+I*-606,
     21+I*-600,97+I*-518,99+I*-514,113+I*-466,171+I*90,171+I*93,
     167+I*115,79+I*349,73+I*357,-9+I*433,-13+I*435,-61+I*449,
     -617+I*507,-620+I*507,-642+I*503,-876+I*415,-884+I*409,
     -960+I*327,-962+I*323,-976+I*275,-1034+I*-281,-1034+I*-284,
     -1030+I*-306,-942+I*-540,-936+I*-548,-854+I*-624,-850+I*-626,
     -802+I*-640 ],
   /* k=9 */ [ 136+I*-2424,139+I*-2424,161+I*-2420,395+I*-2332,
     403+I*-2326,479+I*-2244,1151+I*-1190,1153+I*-1186,
     1167+I*-1138,1225+I*-582,1225+I*-579,1221+I*-557,1133+I*-323,
     1127+I*-315,1045+I*-239,-9+I*433,-13+I*435,-61+I*449,
     -617+I*507,-620+I*507,-642+I*503,-876+I*415,-884+I*409,
     -960+I*327,-1632+I*-727,-1634+I*-731,-1648+I*-779,
     -1706+I*-1335,-1706+I*-1338,-1702+I*-1360,-1614+I*-1594,
     -1608+I*-1602,-1526+I*-1678,-472+I*-2350,-468+I*-2352,
     -420+I*-2366 ],
   /* k=10 */ [ 1572+I*-4822,1575+I*-4822,1597+I*-4818,
     1831+I*-4730,4229+I*-3294,4237+I*-3288,4313+I*-3206,
     4985+I*-2152,4987+I*-2148,5001+I*-2100,5059+I*-1544,
     5059+I*-1541,5055+I*-1519,4967+I*-1285,3531+I*1113,
     3525+I*1121,3443+I*1197,2389+I*1869,2385+I*1871,2337+I*1885,
     1781+I*1943,1778+I*1943,1756+I*1939,1522+I*1851,-876+I*415,
     -884+I*409,-960+I*327,-1632+I*-727,-1634+I*-731,-1648+I*-779,
     -1706+I*-1335,-1706+I*-1338,-1702+I*-1360,-1614+I*-1594,
     -178+I*-3992,-172+I*-4000,-90+I*-4076,964+I*-4748,968+I*-4750,
     1016+I*-4764 ],
   /* k=11 */ [ 1572+I*-4822,1575+I*-4822,7807+I*-4348,
     7829+I*-4344,8063+I*-4256,10461+I*-2820,10469+I*-2814,
     10545+I*-2732,11217+I*-1678,11219+I*-1674,11233+I*-1626,
     11291+I*-1070,11291+I*-1067,10817+I*5165,10813+I*5187,
     10725+I*5421,9289+I*7819,9283+I*7827,9201+I*7903,8147+I*8575,
     8143+I*8577,8095+I*8591,7539+I*8649,7536+I*8649,1304+I*8175,
     1282+I*8171,1048+I*8083,-1350+I*6647,-1358+I*6641,
     -1434+I*6559,-2106+I*5505,-2108+I*5501,-2122+I*5453,
     -2180+I*4897,-2180+I*4894,-1706+I*-1338,-1702+I*-1360,
     -1614+I*-1594,-178+I*-3992,-172+I*-4000,-90+I*-4076,
     964+I*-4748,968+I*-4750,1016+I*-4764 ],
   /* k=12 */ [ 1572+I*-4822,1575+I*-4822,7807+I*-4348,
     7829+I*-4344,8063+I*-4256,10461+I*-2820,10469+I*-2814,
     10545+I*-2732,11217+I*-1678,11219+I*-1674,16503+I*11264,
     16517+I*11312,16575+I*11868,16575+I*11871,16101+I*18103,
     16097+I*18125,16009+I*18359,14573+I*20757,14567+I*20765,
     14485+I*20841,13431+I*21513,13427+I*21515,489+I*26799,
     441+I*26813,-115+I*26871,-118+I*26871,-6350+I*26397,
     -6372+I*26393,-6606+I*26305,-9004+I*24869,-9012+I*24863,
     -9088+I*24781,-9760+I*23727,-9762+I*23723,-15046+I*10785,
     -15060+I*10737,-15118+I*10181,-15118+I*10178,-14644+I*3946,
     -14640+I*3924,-14552+I*3690,-13116+I*1292,-13110+I*1284,
     -13028+I*1208,-11974+I*536,-11970+I*534,968+I*-4750,
     1016+I*-4764 ],
   /* k=13 */ [ -21934+I*-25414,-21931+I*-25414,-15699+I*-24940,
     -15677+I*-24936,-15443+I*-24848,-13045+I*-23412,
     -13037+I*-23406,10469+I*-2814,10545+I*-2732,11217+I*-1678,
     11219+I*-1674,16503+I*11264,16517+I*11312,16575+I*11868,
     16575+I*11871,16101+I*18103,16097+I*18125,16009+I*18359,
     14573+I*20757,14567+I*20765,-6025+I*44271,-6107+I*44347,
     -7161+I*45019,-7165+I*45021,-20103+I*50305,-20151+I*50319,
     -20707+I*50377,-20710+I*50377,-26942+I*49903,-26964+I*49899,
     -27198+I*49811,-29596+I*48375,-29604+I*48369,-53110+I*27777,
     -53186+I*27695,-53858+I*26641,-53860+I*26637,-59144+I*13699,
     -59158+I*13651,-59216+I*13095,-59216+I*13092,-58742+I*6860,
     -58738+I*6838,-58650+I*6604,-57214+I*4206,-57208+I*4198,
     -36616+I*-19308,-36534+I*-19384,-35480+I*-20056,
     -35476+I*-20058,-22538+I*-25342,-22490+I*-25356 ],
   /* k=14 */ [ -71860+I*-110696,-71857+I*-110696,-65625+I*-110222,
     -65603+I*-110218,2001+I*-92540,2235+I*-92452,4633+I*-91016,
     4641+I*-91010,28147+I*-70418,28223+I*-70336,28895+I*-69282,
     28897+I*-69278,34181+I*-56340,34195+I*-56292,34253+I*-55736,
     34253+I*-55733,33779+I*-49501,33775+I*-49479,16097+I*18125,
     16009+I*18359,14573+I*20757,14567+I*20765,-6025+I*44271,
     -6107+I*44347,-7161+I*45019,-7165+I*45021,-20103+I*50305,
     -20151+I*50319,-20707+I*50377,-20710+I*50377,-26942+I*49903,
     -26964+I*49899,-94568+I*32221,-94802+I*32133,-97200+I*30697,
     -97208+I*30691,-120714+I*10099,-120790+I*10017,-121462+I*8963,
     -121464+I*8959,-126748+I*-3979,-126762+I*-4027,
     -126820+I*-4583,-126820+I*-4586,-126346+I*-10818,
     -126342+I*-10840,-108664+I*-78444,-108576+I*-78678,
     -107140+I*-81076,-107134+I*-81084,-86542+I*-104590,
     -86460+I*-104666,-85406+I*-105338,-85402+I*-105340,
     -72464+I*-110624,-72416+I*-110638 ],
   /* k=15 */ [ 48778+I*-295830,48781+I*-295830,55013+I*-295356,
     55035+I*-295352,122639+I*-277674,122873+I*-277586,
     125271+I*-276150,125279+I*-276144,148785+I*-255552,
     148861+I*-255470,149533+I*-254416,149535+I*-254412,
     154819+I*-241474,154833+I*-241426,187081+I*-88540,
     187139+I*-87984,187139+I*-87981,186665+I*-81749,
     186661+I*-81727,168983+I*-14123,168895+I*-13889,
     167459+I*-11491,167453+I*-11483,146861+I*12023,146779+I*12099,
     145725+I*12771,145721+I*12773,132783+I*18057,132735+I*18071,
     -20151+I*50319,-20707+I*50377,-20710+I*50377,-26942+I*49903,
     -26964+I*49899,-94568+I*32221,-94802+I*32133,-97200+I*30697,
     -97208+I*30691,-120714+I*10099,-120790+I*10017,-121462+I*8963,
     -121464+I*8959,-126748+I*-3979,-126762+I*-4027,
     -159010+I*-156913,-159068+I*-157469,-159068+I*-157472,
     -158594+I*-163704,-158590+I*-163726,-140912+I*-231330,
     -140824+I*-231564,-139388+I*-233962,-139382+I*-233970,
     -118790+I*-257476,-118708+I*-257552,-117654+I*-258224,
     -117650+I*-258226,-104712+I*-263510,-104664+I*-263524,
     48222+I*-295772 ],
   /* k=16 */ [ 322302+I*-513212,322305+I*-513212,328537+I*-512738,
     328559+I*-512734,396163+I*-495056,396397+I*-494968,
     398795+I*-493532,398803+I*-493526,422309+I*-472934,
     422385+I*-472852,639767+I*-199328,640439+I*-198274,
     640441+I*-198270,645725+I*-185332,645739+I*-185284,
     677987+I*-32398,678045+I*-31842,678045+I*-31839,
     677571+I*-25607,677567+I*-25585,659889+I*42019,659801+I*42253,
     658365+I*44651,658359+I*44659,637767+I*68165,637685+I*68241,
     364161+I*285623,363107+I*286295,363103+I*286297,
     350165+I*291581,350117+I*291595,197231+I*323843,
     196675+I*323901,196672+I*323901,190440+I*323427,
     190418+I*323423,122814+I*305745,122580+I*305657,
     120182+I*304221,120174+I*304215,96668+I*283623,96592+I*283541,
     -120790+I*10017,-121462+I*8963,-121464+I*8959,-126748+I*-3979,
     -126762+I*-4027,-159010+I*-156913,-159068+I*-157469,
     -159068+I*-157472,-158594+I*-163704,-158590+I*-163726,
     -140912+I*-231330,-140824+I*-231564,-139388+I*-233962,
     -139382+I*-233970,-118790+I*-257476,-118708+I*-257552,
     154816+I*-474934,155870+I*-475606,155874+I*-475608,
     168812+I*-480892,168860+I*-480906,321746+I*-513154 ],
   /* k=17 */ [ 322302+I*-513212,322305+I*-513212,328537+I*-512738,
     328559+I*-512734,396163+I*-495056,396397+I*-494968,
     1104685+I*-165302,1107083+I*-163866,1107091+I*-163860,
     1130597+I*-143268,1130673+I*-143186,1348055+I*130338,
     1348727+I*131392,1348729+I*131396,1354013+I*144334,
     1354027+I*144382,1386275+I*297268,1386333+I*297824,
     1386333+I*297827,1385859+I*304059,1385855+I*304081,
     1368177+I*371685,1368089+I*371919,1038423+I*1080207,
     1036987+I*1082605,1036981+I*1082613,1016389+I*1106119,
     1016307+I*1106195,742783+I*1323577,741729+I*1324249,
     741725+I*1324251,728787+I*1329535,728739+I*1329549,
     575853+I*1361797,575297+I*1361855,575294+I*1361855,
     569062+I*1361381,569040+I*1361377,501436+I*1343699,
     501202+I*1343611,-207086+I*1013945,-209484+I*1012509,
     -209492+I*1012503,-232998+I*991911,-233074+I*991829,
     -450456+I*718305,-451128+I*717251,-451130+I*717247,
     -456414+I*704309,-456428+I*704261,-488676+I*551375,
     -488734+I*550819,-488734+I*550816,-488260+I*544584,
     -488256+I*544562,-470578+I*476958,-470490+I*476724,
     -140824+I*-231564,-139388+I*-233962,-139382+I*-233970,
     -118790+I*-257476,-118708+I*-257552,154816+I*-474934,
     155870+I*-475606,155874+I*-475608,168812+I*-480892,
     168860+I*-480906,321746+I*-513154 ],
   /* k=18 */ [ 322302+I*-513212,322305+I*-513212,328537+I*-512738,
     328559+I*-512734,396163+I*-495056,396397+I*-494968,
     1104685+I*-165302,1107083+I*-163866,1107091+I*-163860,
     1130597+I*-143268,1130673+I*-143186,1348055+I*130338,
     1348727+I*131392,1348729+I*131396,1354013+I*144334,
     1354027+I*144382,1386275+I*297268,1386333+I*297824,
     1435289+I*2044066,1435289+I*2044069,1434815+I*2050301,
     1434811+I*2050323,1417133+I*2117927,1417045+I*2118161,
     1087379+I*2826449,1085943+I*2828847,1085937+I*2828855,
     1065345+I*2852361,1065263+I*2852437,791739+I*3069819,
     790685+I*3070491,790681+I*3070493,777743+I*3075777,
     777695+I*3075791,624809+I*3108039,624253+I*3108097,
     -1121989+I*3157053,-1121992+I*3157053,-1128224+I*3156579,
     -1128246+I*3156575,-1195850+I*3138897,-1196084+I*3138809,
     -1904372+I*2809143,-1906770+I*2807707,-1906778+I*2807701,
     -1930284+I*2787109,-1930360+I*2787027,-2147742+I*2513503,
     -2148414+I*2512449,-2148416+I*2512445,-2153700+I*2499507,
     -2153714+I*2499459,-2185962+I*2346573,-2186020+I*2346017,
     -2234976+I*599775,-2234976+I*599772,-2234502+I*593540,
     -2234498+I*593518,-2216820+I*525914,-2216732+I*525680,
     -1887066+I*-182608,-1885630+I*-185006,-1885624+I*-185014,
     -1865032+I*-208520,-1864950+I*-208596,-1591426+I*-425978,
     -1590372+I*-426650,-1590368+I*-426652,-1577430+I*-431936,
     -1577382+I*-431950,-1424496+I*-464198,-1423940+I*-464256 ],
   /* k=19 */ [ -1521852+I*-3956740,-1521849+I*-3956740,
     -1515617+I*-3956266,-1515595+I*-3956262,-1447991+I*-3938584,
     -1447757+I*-3938496,-739469+I*-3608830,-737071+I*-3607394,
     -737063+I*-3607388,-713557+I*-3586796,-713481+I*-3586714,
     -496099+I*-3313190,-495427+I*-3312136,1348727+I*131392,
     1348729+I*131396,1354013+I*144334,1354027+I*144382,
     1386275+I*297268,1386333+I*297824,1435289+I*2044066,
     1435289+I*2044069,1434815+I*2050301,1434811+I*2050323,
     1417133+I*2117927,1417045+I*2118161,1087379+I*2826449,
     1085943+I*2828847,1085937+I*2828855,1065345+I*2852361,
     1065263+I*2852437,791739+I*3069819,790685+I*3070491,
     -2652843+I*4914645,-2652847+I*4914647,-2665785+I*4919931,
     -2665833+I*4919945,-2818719+I*4952193,-2819275+I*4952251,
     -4565517+I*5001207,-4565520+I*5001207,-4571752+I*5000733,
     -4571774+I*5000729,-4639378+I*4983051,-4639612+I*4982963,
     -5347900+I*4653297,-5350298+I*4651861,-5350306+I*4651855,
     -5373812+I*4631263,-5373888+I*4631181,-5591270+I*4357657,
     -5591942+I*4356603,-7436096+I*913075,-7436098+I*913071,
     -7441382+I*900133,-7441396+I*900085,-7473644+I*747199,
     -7473702+I*746643,-7522658+I*-999599,-7522658+I*-999602,
     -7522184+I*-1005834,-7522180+I*-1005856,-7504502+I*-1073460,
     -7504414+I*-1073694,-7174748+I*-1781982,-7173312+I*-1784380,
     -7173306+I*-1784388,-7152714+I*-1807894,-7152632+I*-1807970,
     -6879108+I*-2025352,-6878054+I*-2026024,-3434526+I*-3870178,
     -3434522+I*-3870180,-3421584+I*-3875464,-3421536+I*-3875478,
     -3268650+I*-3907726,-3268094+I*-3907784 ],
   /* k=20 */ [ -3610786+I*-16131478,-3610783+I*-16131478,
     -3604551+I*-16131004,-3604529+I*-16131000,
     -3536925+I*-16113322,-3536691+I*-16113234,
     -2828403+I*-15783568,-2826005+I*-15782132,4305831+I*-10739230,
     4305839+I*-10739224,4329345+I*-10718632,4329421+I*-10718550,
     4546803+I*-10445026,4547475+I*-10443972,6391629+I*-7000444,
     6391631+I*-7000440,6396915+I*-6987502,6396929+I*-6987454,
     6429177+I*-6834568,6429235+I*-6834012,6478191+I*-5087770,
     6478191+I*-5087767,6477717+I*-5081535,6477713+I*-5081513,
     6460035+I*-5013909,6459947+I*-5013675,6130281+I*-4305387,
     6128845+I*-4302989,1085943+I*2828847,1085937+I*2828855,
     1065345+I*2852361,1065263+I*2852437,791739+I*3069819,
     790685+I*3070491,-2652843+I*4914645,-2652847+I*4914647,
     -2665785+I*4919931,-2665833+I*4919945,-2818719+I*4952193,
     -2819275+I*4952251,-4565517+I*5001207,-4565520+I*5001207,
     -4571752+I*5000733,-4571774+I*5000729,-4639378+I*4983051,
     -4639612+I*4982963,-5347900+I*4653297,-5350298+I*4651861,
     -12482134+I*-391041,-12482142+I*-391047,-12505648+I*-411639,
     -12505724+I*-411721,-12723106+I*-685245,-12723778+I*-686299,
     -14567932+I*-4129827,-14567934+I*-4129831,
     -14573218+I*-4142769,-14573232+I*-4142817,
     -14605480+I*-4295703,-14605538+I*-4296259,
     -14654494+I*-6042501,-14654494+I*-6042504,
     -14654020+I*-6048736,-14654016+I*-6048758,
     -14636338+I*-6116362,-14636250+I*-6116596,
     -14306584+I*-6824884,-14305148+I*-6827282,
     -9262246+I*-13959118,-9262240+I*-13959126,
     -9241648+I*-13982632,-9241566+I*-13982708,
     -8968042+I*-14200090,-8966988+I*-14200762,
     -5523460+I*-16044916,-5523456+I*-16044918,
     -5510518+I*-16050202,-5510470+I*-16050216,
     -5357584+I*-16082464,-5357028+I*-16082522 ],
   /* k=21 */ [ -656818+I*-35438052,-656815+I*-35438052,
     -650583+I*-35437578,18655991+I*-32483610,18656013+I*-32483606,
     18723617+I*-32465928,18723851+I*-32465840,
     19432139+I*-32136174,19434537+I*-32134738,
     26566373+I*-27091836,26566381+I*-27091830,
     26589887+I*-27071238,26589963+I*-27071156,
     26807345+I*-26797632,26808017+I*-26796578,
     28652171+I*-23353050,28652173+I*-23353046,
     28657457+I*-23340108,28657471+I*-23340060,
     28689719+I*-23187174,28689777+I*-23186618,
     28738733+I*-21440376,28738733+I*-21440373,
     28738259+I*-21434141,25784291+I*-2127567,25784287+I*-2127545,
     25766609+I*-2059941,25766521+I*-2059707,25436855+I*-1351419,
     25435419+I*-1349021,20392517+I*5782815,20392511+I*5782823,
     20371919+I*5806329,20371837+I*5806405,20098313+I*6023787,
     20097259+I*6024459,16653731+I*7868613,16653727+I*7868615,
     16640789+I*7873899,16640741+I*7873913,16487855+I*7906161,
     16487299+I*7906219,14741057+I*7955175,14741054+I*7955175,
     14734822+I*7954701,-4571752+I*5000733,-4571774+I*5000729,
     -4639378+I*4983051,-4639612+I*4982963,-5347900+I*4653297,
     -5350298+I*4651861,-12482134+I*-391041,-12482142+I*-391047,
     -12505648+I*-411639,-12505724+I*-411721,-12723106+I*-685245,
     -12723778+I*-686299,-14567932+I*-4129827,-14567934+I*-4129831,
     -14573218+I*-4142769,-14573232+I*-4142817,
     -14605480+I*-4295703,-14605538+I*-4296259,
     -14654494+I*-6042501,-14654494+I*-6042504,
     -14654020+I*-6048736,-11700052+I*-25355310,
     -11700048+I*-25355332,-11682370+I*-25422936,
     -11682282+I*-25423170,-11352616+I*-26131458,
     -11351180+I*-26133856,-6308278+I*-33265692,
     -6308272+I*-33265700,-6287680+I*-33289206,
     -6287598+I*-33289282,-6014074+I*-33506664,
     -6013020+I*-33507336,-2569492+I*-35351490,
     -2569488+I*-35351492,-2556550+I*-35356776,
     -2556502+I*-35356790,-2403616+I*-35389038,-2403060+I*-35389096
     ],
   /* k=22 */ [ 40910298+I*-48836690,40910301+I*-48836690,
     40916533+I*-48836216,60223107+I*-45882248,
     60223129+I*-45882244,60290733+I*-45864566,
     60290967+I*-45864478,60999255+I*-45534812,
     61001653+I*-45533376,68133489+I*-40490474,
     68133497+I*-40490468,68157003+I*-40469876,
     68157079+I*-40469794,68374461+I*-40196270,
     68375133+I*-40195216,70219287+I*-36751688,
     70219289+I*-36751684,70224573+I*-36738746,83623211+I*4828370,
     83623225+I*4828418,83655473+I*4981304,83655531+I*4981860,
     83704487+I*6728102,83704487+I*6728105,83704013+I*6734337,
     80750045+I*26040911,80750041+I*26040933,80732363+I*26108537,
     80732275+I*26108771,80402609+I*26817059,80401173+I*26819457,
     75358271+I*33951293,75358265+I*33951301,75337673+I*33974807,
     75337591+I*33974883,75064067+I*34192265,75063013+I*34192937,
     71619485+I*36037091,71619481+I*36037093,71606543+I*36042377,
     30039427+I*49441015,30039379+I*49441029,29886493+I*49473277,
     29885937+I*49473335,28139695+I*49522291,28139692+I*49522291,
     28133460+I*49521817,8826886+I*46567849,8826864+I*46567845,
     8759260+I*46550167,8759026+I*46550079,8050738+I*46220413,
     8048340+I*46218977,916504+I*41176075,916496+I*41176069,
     892990+I*41155477,892914+I*41155395,675532+I*40881871,
     674860+I*40880817,-1169294+I*37437289,-1169296+I*37437285,
     -1174580+I*37424347,-14573218+I*-4142769,-14573232+I*-4142817,
     -14605480+I*-4295703,-14605538+I*-4296259,
     -14654494+I*-6042501,-14654494+I*-6042504,
     -14654020+I*-6048736,-11700052+I*-25355310,
     -11700048+I*-25355332,-11682370+I*-25422936,
     -11682282+I*-25423170,-11352616+I*-26131458,
     -11351180+I*-26133856,-6308278+I*-33265692,
     -6308272+I*-33265700,-6287680+I*-33289206,
     -6287598+I*-33289282,-6014074+I*-33506664,
     -6013020+I*-33507336,-2569492+I*-35351490,
     -2569488+I*-35351492,-2556550+I*-35356776,
     39010566+I*-48755414,39010614+I*-48755428,
     39163500+I*-48787676,39164056+I*-48787734 ],
   /* k=23 */ [ 40910298+I*-48836690,40910301+I*-48836690,
     40916533+I*-48836216,60223107+I*-45882248,
     60223129+I*-45882244,60290733+I*-45864566,
     60290967+I*-45864478,60999255+I*-45534812,
     61001653+I*-45533376,68133489+I*-40490474,
     68133497+I*-40490468,68157003+I*-40469876,
     136521395+I*29265718,136521471+I*29265800,
     136738853+I*29539324,136739525+I*29540378,
     138583679+I*32983906,138583681+I*32983910,
     138588965+I*32996848,151987603+I*74563964,
     151987617+I*74564012,152019865+I*74716898,
     152019923+I*74717454,152068879+I*76463696,
     152068879+I*76463699,152068405+I*76469931,
     149114437+I*95776505,149114433+I*95776527,
     149096755+I*95844131,149096667+I*95844365,
     148767001+I*96552653,148765565+I*96555051,
     143722663+I*103686887,143722657+I*103686895,
     143702065+I*103710401,73966471+I*172074793,
     73966389+I*172074869,73692865+I*172292251,
     73691811+I*172292923,70248283+I*174137077,
     70248279+I*174137079,70235341+I*174142363,
     28668225+I*187541001,28668177+I*187541015,
     28515291+I*187573263,28514735+I*187573321,
     26768493+I*187622277,26768490+I*187622277,
     26762258+I*187621803,7455684+I*184667835,7455662+I*184667831,
     7388058+I*184650153,7387824+I*184650065,6679536+I*184320399,
     6677138+I*184318963,-454698+I*179276061,-454706+I*179276055,
     -478212+I*179255463,-68842604+I*109519869,
     -68842680+I*109519787,-69060062+I*109246263,
     -69060734+I*109245209,-70904888+I*105801681,
     -70904890+I*105801677,-70910174+I*105788739,
     -84308812+I*64221623,-84308826+I*64221575,
     -84341074+I*64068689,-84341132+I*64068133,
     -84390088+I*62321891,-84390088+I*62321888,
     -84389614+I*62315656,-81435646+I*43009082,
     -81435642+I*43009060,-81417964+I*42941456,
     -81417876+I*42941222,-81088210+I*42232934,
     -81086774+I*42230536,-76043872+I*35098700,
     -76043866+I*35098692,-76023274+I*35075186,
     -6287680+I*-33289206,-6287598+I*-33289282,
     -6014074+I*-33506664,-6013020+I*-33507336,
     -2569492+I*-35351490,-2569488+I*-35351492,
     -2556550+I*-35356776,39010566+I*-48755414,
     39010614+I*-48755428,39163500+I*-48787676,39164056+I*-48787734
     ],
   /* k=24 */ [ -165554080+I*-119943486,-165554077+I*-119943486,
     -165547845+I*-119943012,-146241271+I*-116989044,
     -146241249+I*-116989040,-146173645+I*-116971362,
     60290733+I*-45864566,60290967+I*-45864478,
     60999255+I*-45534812,61001653+I*-45533376,
     68133489+I*-40490474,68133497+I*-40490468,
     68157003+I*-40469876,136521395+I*29265718,
     136521471+I*29265800,136738853+I*29539324,
     136739525+I*29540378,138583679+I*32983906,
     138583681+I*32983910,138588965+I*32996848,
     151987603+I*74563964,151987617+I*74564012,
     152019865+I*74716898,152019923+I*74717454,
     152068879+I*76463696,152068879+I*76463699,
     152068405+I*76469931,149114437+I*95776505,
     149114433+I*95776527,149096755+I*95844131,
     77989959+I*302308509,77989871+I*302308743,
     77660205+I*303017031,77658769+I*303019429,
     72615867+I*310151265,72615861+I*310151273,
     72595269+I*310174779,2859675+I*378539171,2859593+I*378539247,
     2586069+I*378756629,2585015+I*378757301,-858513+I*380601455,
     -858517+I*380601457,-871455+I*380606741,-42438571+I*394005379,
     -42438619+I*394005393,-42591505+I*394037641,
     -42592061+I*394037699,-44338303+I*394086655,
     -44338306+I*394086655,-44344538+I*394086181,
     -63651112+I*391132213,-63651134+I*391132209,
     -63718738+I*391114531,-270183116+I*320007735,
     -270183350+I*320007647,-270891638+I*319677981,
     -270894036+I*319676545,-278025872+I*314633643,
     -278025880+I*314633637,-278049386+I*314613045,
     -346413778+I*244877451,-346413854+I*244877369,
     -346631236+I*244603845,-346631908+I*244602791,
     -348476062+I*241159263,-348476064+I*241159259,
     -348481348+I*241146321,-361879986+I*199579205,
     -361880000+I*199579157,-361912248+I*199426271,
     -361912306+I*199425715,-361961262+I*197679473,
     -361961262+I*197679470,-361960788+I*197673238,
     -359006820+I*178366664,-359006816+I*178366642,
     -358989138+I*178299038,-287882342+I*-28165340,
     -287882254+I*-28165574,-287552588+I*-28873862,
     -287551152+I*-28876260,-282508250+I*-36008096,
     -282508244+I*-36008104,-282487652+I*-36031610,
     -212752058+I*-104396002,-212751976+I*-104396078,
     -212478452+I*-104613460,-212477398+I*-104614132,
     -209033870+I*-106458286,-209033866+I*-106458288,
     -209020928+I*-106463572,-167453812+I*-119862210,
     -167453764+I*-119862224,-167300878+I*-119894472,
     -167300322+I*-119894530 ],
   /* k=25 */ [ -229804866+I*-603979038,-229804863+I*-603979038,
     -229798631+I*-603978564,-210492057+I*-601024596,
     -210492035+I*-601024592,-210424431+I*-601006914,
     -3960053+I*-529900118,-3959819+I*-529900030,
     -3251531+I*-529570364,-3249133+I*-529568928,
     3882703+I*-524526026,3882711+I*-524526020,
     3906217+I*-524505428,72270609+I*-454769834,
     72270685+I*-454769752,72488067+I*-454496228,
     72488739+I*-454495174,74332893+I*-451051646,
     74332895+I*-451051642,74338179+I*-451038704,
     87736817+I*-409471588,87736831+I*-409471540,
     87769079+I*-409318654,152019865+I*74716898,
     152019923+I*74717454,152068879+I*76463696,
     152068879+I*76463699,152068405+I*76469931,
     149114437+I*95776505,149114433+I*95776527,
     149096755+I*95844131,77989959+I*302308509,
     77989871+I*302308743,77660205+I*303017031,
     77658769+I*303019429,72615867+I*310151265,
     72615861+I*310151273,72595269+I*310174779,2859675+I*378539171,
     2859593+I*378539247,2586069+I*378756629,2585015+I*378757301,
     -858513+I*380601455,-858517+I*380601457,-871455+I*380606741,
     -42438571+I*394005379,-42438619+I*394005393,
     -42591505+I*394037641,-526627057+I*458288427,
     -526627613+I*458288485,-528373855+I*458337441,
     -528373858+I*458337441,-528380090+I*458336967,
     -547686664+I*455382999,-547686686+I*455382995,
     -547754290+I*455365317,-754218668+I*384258521,
     -754218902+I*384258433,-754927190+I*383928767,
     -754929588+I*383927331,-762061424+I*378884429,
     -762061432+I*378884423,-762084938+I*378863831,
     -830449330+I*309128237,-830449406+I*309128155,
     -830666788+I*308854631,-830667460+I*308853577,
     -832511614+I*305410049,-832511616+I*305410045,
     -832516900+I*305397107,-845915538+I*263829991,
     -845915552+I*263829943,-845947800+I*263677057,
     -910198586+I*-220358495,-910198644+I*-220359051,
     -910247600+I*-222105293,-910247600+I*-222105296,
     -910247126+I*-222111528,-907293158+I*-241418102,
     -907293154+I*-241418124,-907275476+I*-241485728,
     -836168680+I*-447950106,-836168592+I*-447950340,
     -835838926+I*-448658628,-835837490+I*-448661026,
     -830794588+I*-455792862,-830794582+I*-455792870,
     -830773990+I*-455816376,-761038396+I*-524180768,
     -761038314+I*-524180844,-760764790+I*-524398226,
     -760763736+I*-524398898,-757320208+I*-526243052,
     -757320204+I*-526243054,-757307266+I*-526248338,
     -715740150+I*-539646976,-715740102+I*-539646990,
     -715587216+I*-539679238,-231551664+I*-603930024,
     -231551108+I*-603930082 ],
   /* k=26 */ [ 61478328+I*-2120336480,61478331+I*-2120336480,
     61484563+I*-2120336006,80791137+I*-2117382038,
     80791159+I*-2117382034,80858763+I*-2117364356,
     287323141+I*-2046257560,287323375+I*-2046257472,
     288031663+I*-2045927806,288034061+I*-2045926370,
     295165897+I*-2040883468,295165905+I*-2040883462,
     295189411+I*-2040862870,363553803+I*-1971127276,
     363553879+I*-1971127194,363771261+I*-1970853670,
     976308385+I*-1067033352,976309057+I*-1067032298,
     978153211+I*-1063588770,978153213+I*-1063588766,
     978158497+I*-1063575828,991557135+I*-1022008712,
     991557149+I*-1022008664,991589397+I*-1021855778,
     1055840183+I*-537820226,1055840241+I*-537819670,
     1055889197+I*-536073428,1055889197+I*-536073425,
     1055888723+I*-536067193,1052934755+I*-516760619,
     1052934751+I*-516760597,1052917073+I*-516692993,
     981810277+I*-310228615,981810189+I*-310228381,
     981480523+I*-309520093,981479087+I*-309517695,
     976436185+I*-302385859,976436179+I*-302385851,
     976415587+I*-302362345,906679993+I*-233997953,
     906679911+I*-233997877,906406387+I*-233780495,
     2586069+I*378756629,2585015+I*378757301,-858513+I*380601455,
     -858517+I*380601457,-871455+I*380606741,-42438571+I*394005379,
     -42438619+I*394005393,-42591505+I*394037641,
     -526627057+I*458288427,-526627613+I*458288485,
     -528373855+I*458337441,-528373858+I*458337441,
     -528380090+I*458336967,-547686664+I*455382999,
     -547686686+I*455382995,-547754290+I*455365317,
     -754218668+I*384258521,-754218902+I*384258433,
     -754927190+I*383928767,-754929588+I*383927331,
     -762061424+I*378884429,-762061432+I*378884423,
     -762084938+I*378863831,-830449330+I*309128237,
     -830449406+I*309128155,-830666788+I*308854631,
     -1443203912+I*-594965687,-1443204584+I*-594966741,
     -1445048738+I*-598410269,-1445048740+I*-598410273,
     -1445054024+I*-598423211,-1458452662+I*-639990327,
     -1458452676+I*-639990375,-1458484924+I*-640143261,
     -1522735710+I*-1124178813,-1522735768+I*-1124179369,
     -1522784724+I*-1125925611,-1522784724+I*-1125925614,
     -1522784250+I*-1125931846,-1519830282+I*-1145238420,
     -1519830278+I*-1145238442,-1519812600+I*-1145306046,
     -1448705804+I*-1351770424,-1448705716+I*-1351770658,
     -1448376050+I*-1352478946,-1448374614+I*-1352481344,
     -1443331712+I*-1359613180,-1443331706+I*-1359613188,
     -1443311114+I*-1359636694,-1373575520+I*-1428001086,
     -1373575438+I*-1428001162,-1373301914+I*-1428218544,
     -469481596+I*-2040755668,-469480542+I*-2040756340,
     -466037014+I*-2042600494,-466037010+I*-2042600496,
     -466024072+I*-2042605780,-424456956+I*-2056004418,
     -424456908+I*-2056004432,-424304022+I*-2056036680,
     59731530+I*-2120287466,59732086+I*-2120287524 ],
   /* k=27 */ [ 1256581840+I*-4249231046,1256581843+I*-4249231046,
     1256588075+I*-4249230572,1275894649+I*-4246276604,
     1275894671+I*-4246276600,1275962275+I*-4246258922,
     1482426653+I*-4175152126,1482426887+I*-4175152038,
     1483135175+I*-4174822372,3612029741+I*-2979718860,
     3612032139+I*-2979717424,3619163975+I*-2974674522,
     3619163983+I*-2974674516,3619187489+I*-2974653924,
     3687551881+I*-2904918330,3687551957+I*-2904918248,
     3687769339+I*-2904644724,4300306463+I*-2000824406,
     4300307135+I*-2000823352,4302151289+I*-1997379824,
     4302151291+I*-1997379820,4302156575+I*-1997366882,
     4315555213+I*-1955799766,4315555227+I*-1955799718,
     4315587475+I*-1955646832,4379838261+I*-1471611280,
     4379838319+I*-1471610724,4379887275+I*-1469864482,
     4379887275+I*-1469864479,4379886801+I*-1469858247,
     4376932833+I*-1450551673,4376932829+I*-1450551651,
     4376915151+I*-1450484047,4305808355+I*-1244019669,
     4305808267+I*-1244019435,4305478601+I*-1243311147,
     3110375089+I*885583419,3110373653+I*885585817,
     3105330751+I*892717653,3105330745+I*892717661,
     3105310153+I*892741167,3035574559+I*961105559,
     3035574477+I*961105635,3035300953+I*961323017,
     2131480635+I*1573860141,2131479581+I*1573860813,
     2128036053+I*1575704967,2128036049+I*1575704969,
     2128023111+I*1575710253,2086455995+I*1589108891,
     2086455947+I*1589108905,2086303061+I*1589141153,
     1602267509+I*1653391939,1602266953+I*1653391997,
     1600520711+I*1653440953,1600520708+I*1653440953,
     1600514476+I*1653440479,1581207902+I*1650486511,
     1581207880+I*1650486507,1581140276+I*1650468829,
     1374675898+I*1579362033,1374675664+I*1579361945,
     1373967376+I*1579032279,-754927190+I*383928767,
     -754929588+I*383927331,-762061424+I*378884429,
     -762061432+I*378884423,-762084938+I*378863831,
     -830449330+I*309128237,-830449406+I*309128155,
     -830666788+I*308854631,-1443203912+I*-594965687,
     -1443204584+I*-594966741,-1445048738+I*-598410269,
     -1445048740+I*-598410273,-1445054024+I*-598423211,
     -1458452662+I*-639990327,-1458452676+I*-639990375,
     -1458484924+I*-640143261,-1522735710+I*-1124178813,
     -1522735768+I*-1124179369,-1522784724+I*-1125925611,
     -1522784724+I*-1125925614,-1522784250+I*-1125931846,
     -1519830282+I*-1145238420,-1519830278+I*-1145238442,
     -1519812600+I*-1145306046,-1448705804+I*-1351770424,
     -1448705716+I*-1351770658,-1448376050+I*-1352478946,
     -253272538+I*-3481373512,-253271102+I*-3481375910,
     -248228200+I*-3488507746,-248228194+I*-3488507754,
     -248207602+I*-3488531260,-178472008+I*-3556895652,
     -178471926+I*-3556895728,-178198402+I*-3557113110,
     725621916+I*-4169650234,725622970+I*-4169650906,
     729066498+I*-4171495060,729066502+I*-4171495062,
     729079440+I*-4171500346,770646556+I*-4184898984,
     770646604+I*-4184898998,770799490+I*-4184931246,
     1254835042+I*-4249182032,1254835598+I*-4249182090 ],
   /* k=28 */ [ 1256581840+I*-4249231046,1256581843+I*-4249231046,
     6709474487+I*-3987918588,6709480719+I*-3987918114,
     6728787293+I*-3984964146,6728787315+I*-3984964142,
     6728854919+I*-3984946464,6935319297+I*-3913839668,
     6935319531+I*-3913839580,6936027819+I*-3913509914,
     9064922385+I*-2718406402,9064924783+I*-2718404966,
     9072056619+I*-2713362064,9072056627+I*-2713362058,
     9072080133+I*-2713341466,9140444525+I*-2643605872,
     9140444601+I*-2643605790,9140661983+I*-2643332266,
     9753199107+I*-1739511948,9753199779+I*-1739510894,
     9755043933+I*-1736067366,9755043935+I*-1736067362,
     9755049219+I*-1736054424,9768447857+I*-1694487308,
     9768447871+I*-1694487260,9768480119+I*-1694334374,
     9832730905+I*-1210298822,9832730963+I*-1210298266,
     9832779919+I*-1208552024,9832779919+I*-1208552021,
     9571467461+I*4244340623,9571466987+I*4244346855,
     9568513019+I*4263653429,9568513015+I*4263653451,
     9568495337+I*4263721055,9497388541+I*4470185433,
     9497388453+I*4470185667,9497058787+I*4470893955,
     8301955275+I*6599788521,8301953839+I*6599790919,
     8296910937+I*6606922755,8296910931+I*6606922763,
     8296890339+I*6606946269,8227154745+I*6675310661,
     8227154663+I*6675310737,8226881139+I*6675528119,
     7323060821+I*7288065243,7323059767+I*7288065915,
     7319616239+I*7289910069,7319616235+I*7289910071,
     7319603297+I*7289915355,7278036181+I*7303313993,
     7278036133+I*7303314007,7277883247+I*7303346255,
     6793847695+I*7367597041,6793847139+I*7367597099,
     6792100897+I*7367646055,6792100894+I*7367646055,
     1339208250+I*7106333597,1339202018+I*7106333123,
     1319895444+I*7103379155,1319895422+I*7103379151,
     1319827818+I*7103361473,1113363440+I*7032254677,
     1113363206+I*7032254589,1112654918+I*7031924923,
     -1016239648+I*5836821411,-1016242046+I*5836819975,
     -1023373882+I*5831777073,-1023373890+I*5831777067,
     -1023397396+I*5831756475,-1091761788+I*5762020881,
     -1091761864+I*5762020799,-1091979246+I*5761747275,
     -1704516370+I*4857926957,-1704517042+I*4857925903,
     -1706361196+I*4854482375,-1706361198+I*4854482371,
     -1706366482+I*4854469433,-1719765120+I*4812902317,
     -1719765134+I*4812902269,-1719797382+I*4812749383,
     -1784048168+I*4328713831,-1784048226+I*4328713275,
     -1784097182+I*4326967033,-1784097182+I*4326967030,
     -1522784724+I*-1125925614,-1522784250+I*-1125931846,
     -1519830282+I*-1145238420,-1519830278+I*-1145238442,
     -1519812600+I*-1145306046,-1448705804+I*-1351770424,
     -1448705716+I*-1351770658,-1448376050+I*-1352478946,
     -253272538+I*-3481373512,-253271102+I*-3481375910,
     -248228200+I*-3488507746,-248228194+I*-3488507754,
     -248207602+I*-3488531260,-178472008+I*-3556895652,
     -178471926+I*-3556895728,-178198402+I*-3557113110,
     725621916+I*-4169650234,725622970+I*-4169650906,
     729066498+I*-4171495060,729066502+I*-4171495062,
     729079440+I*-4171500346,770646556+I*-4184898984,
     770646604+I*-4184898998,770799490+I*-4184931246,
     1254835042+I*-4249182032,1254835598+I*-4249182090 ],
   /* k=29 */ [ 1256581840+I*-4249231046,1256581843+I*-4249231046,
     6709474487+I*-3987918588,6709480719+I*-3987918114,
     6728787293+I*-3984964146,6728787315+I*-3984964142,
     6728854919+I*-3984946464,6935319297+I*-3913839668,
     6935319531+I*-3913839580,6936027819+I*-3913509914,
     9064922385+I*-2718406402,9064924783+I*-2718404966,
     9072056619+I*-2713362064,9072056627+I*-2713362058,
     9072080133+I*-2713341466,9140444525+I*-2643605872,
     9140444601+I*-2643605790,9140661983+I*-2643332266,
     9753199107+I*-1739511948,9753199779+I*-1739510894,
     9755043933+I*-1736067366,9755043935+I*-1736067362,
     14685311663+I*9431030384,14685316947+I*9431043322,
     14698715585+I*9472610438,14698715599+I*9472610486,
     14698747847+I*9472763372,14762998633+I*9956798924,
     14762998691+I*9956799480,14763047647+I*9958545722,
     14763047647+I*9958545725,14501735189+I*15411438369,
     14501734715+I*15411444601,14498780747+I*15430751175,
     14498780743+I*15430751197,14498763065+I*15430818801,
     14427656269+I*15637283179,14427656181+I*15637283413,
     14427326515+I*15637991701,13232223003+I*17766886267,
     13232221567+I*17766888665,13227178665+I*17774020501,
     13227178659+I*17774020509,13227158067+I*17774044015,
     13157422473+I*17842408407,13157422391+I*17842408483,
     13157148867+I*17842625865,12253328549+I*18455162989,
     12253327495+I*18455163661,12249883967+I*18457007815,
     12249883963+I*18457007817,1082786217+I*23387275545,
     1082773279+I*23387280829,1041206163+I*23400679467,
     1041206115+I*23400679481,1041053229+I*23400711729,
     557017677+I*23464962515,557017121+I*23464962573,
     555270879+I*23465011529,555270876+I*23465011529,
     -4897621768+I*23203699071,-4897628000+I*23203698597,
     -4916934574+I*23200744629,-4916934596+I*23200744625,
     -4917002200+I*23200726947,-5123466578+I*23129620151,
     -5123466812+I*23129620063,-5124175100+I*23129290397,
     -7253069666+I*21934186885,-7253072064+I*21934185449,
     -7260203900+I*21929142547,-7260203908+I*21929142541,
     -7260227414+I*21929121949,-7328591806+I*21859386355,
     -7328591882+I*21859386273,-7328809264+I*21859112749,
     -7941346388+I*20955292431,-7941347060+I*20955291377,
     -7943191214+I*20951847849,-7943191216+I*20951847845,
     -12873458944+I*9784750099,-12873464228+I*9784737161,
     -12886862866+I*9743170045,-12886862880+I*9743169997,
     -12886895128+I*9743017111,-12951145914+I*9258981559,
     -12951145972+I*9258981003,-12951194928+I*9257234761,
     -12951194928+I*9257234758,-12689882470+I*3804342114,
     -12689881996+I*3804335882,-12686928028+I*3785029308,
     -12686928024+I*3785029286,-12686910346+I*3784961682,
     -12615803550+I*3578497304,-12615803462+I*3578497070,
     -12615473796+I*3577788782,-11420370284+I*1448894216,
     -11420368848+I*1448891818,-11415325946+I*1441759982,
     -11415325940+I*1441759974,-11415305348+I*1441736468,
     -11345569754+I*1373372076,-11345569672+I*1373372000,
     -11345296148+I*1373154618,-10441475830+I*760617494,
     -10441474776+I*760616822,-10438031248+I*758772668,
     -10438031244+I*758772666,729066502+I*-4171495062,
     729079440+I*-4171500346,770646556+I*-4184898984,
     770646604+I*-4184898998,770799490+I*-4184931246,
     1254835042+I*-4249182032,1254835598+I*-4249182090 ],
   /* k=30 */ [ -19771051362+I*-21653158810,
     -19771051359+I*-21653158810,-14318158715+I*-21391846352,
     -14318152483+I*-21391845878,-14298845909+I*-21388891910,
     -14298845887+I*-21388891906,-14298778283+I*-21388874228,
     -14092313905+I*-21317767432,-14092313671+I*-21317767344,
     -14091605383+I*-21317437678,-11962710817+I*-20122334166,
     -11962708419+I*-20122332730,-11955576583+I*-20117289828,
     -11955576575+I*-20117289822,9072056627+I*-2713362058,
     9072080133+I*-2713341466,9140444525+I*-2643605872,
     9140444601+I*-2643605790,9140661983+I*-2643332266,
     9753199107+I*-1739511948,9753199779+I*-1739510894,
     9755043933+I*-1736067366,9755043935+I*-1736067362,
     14685311663+I*9431030384,14685316947+I*9431043322,
     14698715585+I*9472610438,14698715599+I*9472610486,
     14698747847+I*9472763372,14762998633+I*9956798924,
     14762998691+I*9956799480,14763047647+I*9958545722,
     14763047647+I*9958545725,14501735189+I*15411438369,
     14501734715+I*15411444601,14498780747+I*15430751175,
     14498780743+I*15430751197,14498763065+I*15430818801,
     14427656269+I*15637283179,14427656181+I*15637283413,
     14427326515+I*15637991701,13232223003+I*17766886267,
     13232221567+I*17766888665,13227178665+I*17774020501,
     13227178659+I*17774020509,-4176749105+I*38801653711,
     -4176769697+I*38801677217,-4246505291+I*38870041609,
     -4246505373+I*38870041685,-4246778897+I*38870259067,
     -5150599215+I*39482796191,-5150600269+I*39482796863,
     -5154043797+I*39484641017,-5154043801+I*39484641019,
     -16321141547+I*44414908747,-16321154485+I*44414914031,
     -16362721601+I*44428312669,-16362721649+I*44428312683,
     -16362874535+I*44428344931,-16846910087+I*44492595717,
     -16846910643+I*44492595775,-16848656885+I*44492644731,
     -16848656888+I*44492644731,-22301549532+I*44231332273,
     -22301555764+I*44231331799,-22320862338+I*44228377831,
     -22320862360+I*44228377827,-22320929964+I*44228360149,
     -22527394342+I*44157253353,-22527394576+I*44157253265,
     -22528102864+I*44156923599,-24656997430+I*42961820087,
     -24656999828+I*42961818651,-24664131664+I*42956775749,
     -24664131672+I*42956775743,-45691764874+I*25552847979,
     -45691788380+I*25552827387,-45760152772+I*25483091793,
     -45760152848+I*25483091711,-45760370230+I*25482818187,
     -46372907354+I*24578997869,-46372908026+I*24578996815,
     -46374752180+I*24575553287,-46374752182+I*24575553283,
     -51305019910+I*13408455537,-51305025194+I*13408442599,
     -51318423832+I*13366875483,-51318423846+I*13366875435,
     -51318456094+I*13366722549,-51382706880+I*12882686997,
     -51382706938+I*12882686441,-51382755894+I*12880940199,
     -51382755894+I*12880940196,-51121443436+I*7428047552,
     -51121442962+I*7428041320,-51118488994+I*7408734746,
     -51118488990+I*7408734724,-51118471312+I*7408667120,
     -51047364516+I*7202202742,-51047364428+I*7202202508,
     -51047034762+I*7201494220,-49851931250+I*5072599654,
     -49851929814+I*5072597256,-49846886912+I*5065465420,
     -49846886906+I*5065465412,-32442959142+I*-15962167790,
     -32442938550+I*-15962191296,-32373202956+I*-16030555688,
     -32373202874+I*-16030555764,-32372929350+I*-16030773146,
     -31469109032+I*-16643310270,-31469107978+I*-16643310942,
     -31465664450+I*-16645155096,-31465664446+I*-16645155098,
     -20298566700+I*-21575422826,-20298553762+I*-21575428110,
     -20256986646+I*-21588826748,-20256986598+I*-21588826762,
     -20256833712+I*-21588859010,-19772798160+I*-21653109796,
     -19772797604+I*-21653109854 ],
   /* k=31 */ [ -65450023204+I*-94892575304,
     -65450023201+I*-94892575304,-59997130557+I*-94631262846,
     -59997124325+I*-94631262372,-59977817751+I*-94628308404,
     -59977817729+I*-94628308400,-518623561+I*-80848086074,
     -518555957+I*-80848068396,-312091579+I*-80776961600,
     -312091345+I*-80776961512,-311383057+I*-80776631846,
     1817511509+I*-79581528334,1817513907+I*-79581526898,
     1824645743+I*-79576483996,1824645751+I*-79576483990,
     22852278953+I*-62172556226,22852302459+I*-62172535634,
     22920666851+I*-62102800040,22920666927+I*-62102799958,
     22920884309+I*-62102526434,23533421433+I*-61198706116,
     23533422105+I*-61198705062,23535266259+I*-61195261534,
     23535266261+I*-61195261530,28465533989+I*-50028163784,
     28465539273+I*-50028150846,28478937911+I*-49986583730,
     28478937925+I*-49986583682,28478970173+I*-49986430796,
     28543220959+I*-49502395244,28543221017+I*-49502394688,
     28543269973+I*-49500648446,28543269973+I*-49500648443,
     28281957515+I*-44047755799,28281957041+I*-44047749567,
     28279003073+I*-44028442993,28279003069+I*-44028442971,
     14498780743+I*15430751197,14498763065+I*15430818801,
     14427656269+I*15637283179,14427656181+I*15637283413,
     14427326515+I*15637991701,13232223003+I*17766886267,
     13232221567+I*17766888665,13227178665+I*17774020501,
     13227178659+I*17774020509,-4176749105+I*38801653711,
     -4176769697+I*38801677217,-4246505291+I*38870041609,
     -4246505373+I*38870041685,-4246778897+I*38870259067,
     -5150599215+I*39482796191,-5150600269+I*39482796863,
     -5154043797+I*39484641017,-5154043801+I*39484641019,
     -16321141547+I*44414908747,-16321154485+I*44414914031,
     -16362721601+I*44428312669,-16362721649+I*44428312683,
     -16362874535+I*44428344931,-16846910087+I*44492595717,
     -16846910643+I*44492595775,-16848656885+I*44492644731,
     -16848656888+I*44492644731,-22301549532+I*44231332273,
     -22301555764+I*44231331799,-22320862338+I*44228377831,
     -22320862360+I*44228377827,-81780056528+I*30448155501,
     -81780124132+I*30448137823,-81986588510+I*30377031027,
     -81986588744+I*30377030939,-81987297032+I*30376701273,
     -84116191598+I*29181597761,-84116193996+I*29181596325,
     -84123325832+I*29176553423,-84123325840+I*29176553417,
     -105150959042+I*11772625653,-105150982548+I*11772605061,
     -105219346940+I*11702869467,-105219347016+I*11702869385,
     -105219564398+I*11702595861,-105832101522+I*10798775543,
     -105832102194+I*10798774489,-105833946348+I*10795330961,
     -105833946350+I*10795330957,-110764214078+I*-371766789,
     -110764219362+I*-371779727,-110777618000+I*-413346843,
     -110777618014+I*-413346891,-110777650262+I*-413499777,
     -110841901048+I*-897535329,-110841901106+I*-897535885,
     -110841950062+I*-899282127,-110841950062+I*-899282130,
     -110580637604+I*-6352174774,-110580637130+I*-6352181006,
     -110577683162+I*-6371487580,-110577683158+I*-6371487602,
     -96797460832+I*-65830681770,-96797443154+I*-65830749374,
     -96726336358+I*-66037213752,-96726336270+I*-66037213986,
     -96726006604+I*-66037922274,-95530903092+I*-68166816840,
     -95530901656+I*-68166819238,-95525858754+I*-68173951074,
     -95525858748+I*-68173951082,-78121930984+I*-89201584284,
     -78121910392+I*-89201607790,-78052174798+I*-89269972182,
     -78052174716+I*-89269972258,-78051901192+I*-89270189640,
     -77148080874+I*-89882726764,-77148079820+I*-89882727436,
     -77144636292+I*-89884571590,-77144636288+I*-89884571592,
     -65977538542+I*-94814839320,-65977525604+I*-94814844604,
     -65935958488+I*-94828243242,-65935958440+I*-94828243256,
     -65935805554+I*-94828275504,-65451770002+I*-94892526290,
     -65451769446+I*-94892526348 ],
   /* k=32 */ [ 35349837942+I*-259489935482,
     35349837945+I*-259489935482,40802730589+I*-259228623024,
     40802736821+I*-259228622550,40822043395+I*-259225668582,
     40822043417+I*-259225668578,100281237585+I*-245445446252,
     100281305189+I*-245445428574,100487769567+I*-245374321778,
     100487769801+I*-245374321690,100488478089+I*-245373992024,
     102617372655+I*-244178888512,102617375053+I*-244178887076,
     102624506889+I*-244173844174,102624506897+I*-244173844168,
     123652140099+I*-226769916404,123652163605+I*-226769895812,
     123720527997+I*-226700160218,123720528073+I*-226700160136,
     123720745455+I*-226699886612,124333282579+I*-225796066294,
     124333283251+I*-225796065240,124335127405+I*-225792621712,
     124335127407+I*-225792621708,129265395135+I*-214625523962,
     129265400419+I*-214625511024,129278799057+I*-214583943908,
     129278799071+I*-214583943860,161177548587+I*-81885333198,
     161177580835+I*-81885180312,161241831621+I*-81401144760,
     161241831679+I*-81401144204,161241880635+I*-81399397962,
     161241880635+I*-81399397959,160980568177+I*-75946505315,
     160980567703+I*-75946499083,160977613735+I*-75927192509,
     160977613731+I*-75927192487,147197391405+I*-16467998319,
     147197373727+I*-16467930715,147126266931+I*-16261466337,
     147126266843+I*-16261466103,147125937177+I*-16260757815,
     145930833665+I*-14131863249,145930832229+I*-14131860851,
     145925789327+I*-14124729015,145925789321+I*-14124729007,
     128521861557+I*6902904195,128521840965+I*6902927701,
     128452105371+I*6971292093,128452105289+I*6971292169,
     128451831765+I*6971509551,127548011447+I*7584046675,
     127548010393+I*7584047347,127544566865+I*7585891501,
     127544566861+I*7585891503,116377469115+I*12516159231,
     116377456177+I*12516164515,116335889061+I*12529563153,
     116335889013+I*12529563167,-16362721649+I*44428312683,
     -16362874535+I*44428344931,-16846910087+I*44492595717,
     -16846910643+I*44492595775,-16848656885+I*44492644731,
     -16848656888+I*44492644731,-22301549532+I*44231332273,
     -22301555764+I*44231331799,-22320862338+I*44228377831,
     -22320862360+I*44228377827,-81780056528+I*30448155501,
     -81780124132+I*30448137823,-81986588510+I*30377031027,
     -81986588744+I*30377030939,-81987297032+I*30376701273,
     -84116191598+I*29181597761,-84116193996+I*29181596325,
     -84123325832+I*29176553423,-84123325840+I*29176553417,
     -105150959042+I*11772625653,-105150982548+I*11772605061,
     -105219346940+I*11702869467,-105219347016+I*11702869385,
     -105219564398+I*11702595861,-105832101522+I*10798775543,
     -105832102194+I*10798774489,-105833946348+I*10795330961,
     -105833946350+I*10795330957,-110764214078+I*-371766789,
     -110764219362+I*-371779727,-110777618000+I*-413346843,
     -110777618014+I*-413346891,-142676367530+I*-133111957553,
     -142676399778+I*-133112110439,-142740650564+I*-133596145991,
     -142740650622+I*-133596146547,-142740699578+I*-133597892789,
     -142740699578+I*-133597892792,-142479387120+I*-139050785436,
     -142479386646+I*-139050791668,-142476432678+I*-139070098242,
     -142476432674+I*-139070098264,-128696210348+I*-198529292432,
     -128696192670+I*-198529360036,-128625085874+I*-198735824414,
     -128625085786+I*-198735824648,-128624756120+I*-198736532936,
     -127429652608+I*-200865427502,-127429651172+I*-200865429900,
     -127424608270+I*-200872561736,-127424608264+I*-200872561744,
     -110020680500+I*-221900194946,-110020659908+I*-221900218452,
     -109950924314+I*-221968582844,-109950924232+I*-221968582920,
     -109950650708+I*-221968800302,-109046830390+I*-222581337426,
     -109046829336+I*-222581338098,-109043385808+I*-222583182252,
     -109043385804+I*-222583182254,-97876288058+I*-227513449982,
     -97876275120+I*-227513455266,-97834708004+I*-227526853904,
     -97834707956+I*-227526853918,34863902706+I*-259425603434,
     34864055592+I*-259425635682,35348091144+I*-259489886468,
     35348091700+I*-259489886526 ],
   /* k=33 */ [ 268848309750+I*-455986045176,
     268848309753+I*-455986045176,274301202397+I*-455724732718,
     274301208629+I*-455724732244,274320515203+I*-455721778276,
     274320515225+I*-455721778272,333779709393+I*-441941555946,
     333779776997+I*-441941538268,333986241375+I*-441870431472,
     333986241609+I*-441870431384,333986949897+I*-441870101718,
     336115844463+I*-440674998206,336115846861+I*-440674996770,
     336122978697+I*-440669953868,336122978705+I*-440669953862,
     357150611907+I*-423266026098,357150635413+I*-423266005506,
     357218999805+I*-423196269912,357218999881+I*-423196269830,
     553715109575+I*-189697798022,553715326957+I*-189697524498,
     554327864081+I*-188793704180,554327864753+I*-188793703126,
     554329708907+I*-188790259598,554329708909+I*-188790259594,
     559259976637+I*-177623161848,559259981921+I*-177623148910,
     559273380559+I*-177581581794,559273380573+I*-177581581746,
     591172130089+I*-44882971084,591172162337+I*-44882818198,
     591236413123+I*-44398782646,591236413181+I*-44398782090,
     591236462137+I*-44397035848,591236462137+I*-44397035845,
     590975149679+I*-38944143201,590975149205+I*-38944136969,
     590972195237+I*-38924830395,590972195233+I*-38924830373,
     577191972907+I*20534363795,577191955229+I*20534431399,
     577120848433+I*20740895777,577120848345+I*20740896011,
     577120518679+I*20741604299,575925415167+I*22870498865,
     575925413731+I*22870501263,575920370829+I*22877633099,
     575920370823+I*22877633107,558516443059+I*43905266309,
     558516422467+I*43905289815,558446686873+I*43973654207,
     558446686791+I*43973654283,324948214983+I*240469763977,
     324947941459+I*240469981359,324044121141+I*241082518483,
     324044120087+I*241082519155,324040676559+I*241084363309,
     324040676555+I*241084363311,312873578809+I*246014631039,
     312873565871+I*246014636323,312831998755+I*246028034961,
     312831998707+I*246028034975,180133388045+I*277926784491,
     180133235159+I*277926816739,179649199607+I*277991067525,
     179649199051+I*277991067583,179647452809+I*277991116539,
     179647452806+I*277991116539,174194560162+I*277729804081,
     174194553930+I*277729803607,174175247356+I*277726849639,
     174175247334+I*277726849635,114716053166+I*263946627309,
     114715985562+I*263946609631,114509521184+I*263875502835,
     114509520950+I*263875502747,114508812662+I*263875173081,
     112379918096+I*262680069569,112379915698+I*262680068133,
     112372783862+I*262675025231,112372783854+I*262675025225,
     91345150652+I*245271097461,91345127146+I*245271076869,
     91276762754+I*245201341275,91276762678+I*245201341193,
     -105219347016+I*11702869385,-105219564398+I*11702595861,
     -105832101522+I*10798775543,-105832102194+I*10798774489,
     -105833946348+I*10795330961,-105833946350+I*10795330957,
     -110764214078+I*-371766789,-110764219362+I*-371779727,
     -110777618000+I*-413346843,-110777618014+I*-413346891,
     -142676367530+I*-133111957553,-142676399778+I*-133112110439,
     -142740650564+I*-133596145991,-142740650622+I*-133596146547,
     -142740699578+I*-133597892789,-142740699578+I*-133597892792,
     -142479387120+I*-139050785436,-142479386646+I*-139050791668,
     -142476432678+I*-139070098242,-142476432674+I*-139070098264,
     -128696210348+I*-198529292432,-128696192670+I*-198529360036,
     -128625085874+I*-198735824414,-128625085786+I*-198735824648,
     -128624756120+I*-198736532936,-127429652608+I*-200865427502,
     -127429651172+I*-200865429900,-127424608270+I*-200872561736,
     -127424608264+I*-200872561744,-110020680500+I*-221900194946,
     -110020659908+I*-221900218452,-109950924314+I*-221968582844,
     -109950924232+I*-221968582920,123547547576+I*-418464692614,
     123547821100+I*-418464909996,124451641418+I*-419077447120,
     124451642472+I*-419077447792,124455086000+I*-419079291946,
     124455086004+I*-419079291948,135622183750+I*-424009559676,
     135622196688+I*-424009564960,135663763804+I*-424022963598,
     135663763852+I*-424022963612,268362374514+I*-455921713128,
     268362527400+I*-455921745376,268846562952+I*-455985996162,
     268846563508+I*-455985996220 ],
   /* k=34 */ [ 268848309750+I*-455986045176,
     268848309753+I*-455986045176,274301202397+I*-455724732718,
     274301208629+I*-455724732244,274320515203+I*-455721778276,
     274320515225+I*-455721778272,333779709393+I*-441941555946,
     333779776997+I*-441941538268,333986241375+I*-441870431472,
     333986241609+I*-441870431384,960476932805+I*-171369597462,
     960477641093+I*-171369267796,962606535659+I*-170174164284,
     962606538057+I*-170174162848,962613669893+I*-170169119946,
     962613669901+I*-170169119940,983641303103+I*-152765192176,
     983641326609+I*-152765171584,983709691001+I*-152695435990,
     983709691077+I*-152695435908,1180205800771+I*80803035900,
     1180206018153+I*80803309424,1180818555277+I*81707129742,
     1180818555949+I*81707130796,1180820400103+I*81710574324,
     1180820400105+I*81710574328,1185750667833+I*92877672074,
     1185750673117+I*92877685012,1185764071755+I*92919252128,
     1185764071769+I*92919252176,1217662821285+I*225617862838,
     1217662853533+I*225618015724,1217727104319+I*226102051276,
     1217727104377+I*226102051832,1217727153333+I*226103798074,
     1217727153333+I*226103798077,1217465840875+I*231556690721,
     1217465840401+I*231556696953,1217462886433+I*231576003527,
     1217462886429+I*231576003549,1203682664103+I*291035197717,
     1203682646425+I*291035265321,1203611539629+I*291241729699,
     1203611539541+I*291241729933,933110705619+I*917732421129,
     933110375953+I*917733129417,931915272441+I*919862023983,
     931915271005+I*919862026381,931910228103+I*919869158217,
     931910228097+I*919869158225,914506300333+I*940896791427,
     914506279741+I*940896814933,914436544147+I*940965179325,
     914436544065+I*940965179401,680938072257+I*1137461289095,
     680937798733+I*1137461506477,680033978415+I*1138074043601,
     680033977361+I*1138074044273,680030533833+I*1138075888427,
     680030533829+I*1138075888429,668863436083+I*1143006156157,
     668863423145+I*1143006161441,668821856029+I*1143019560079,
     668821855981+I*1143019560093,536123245319+I*1174918309609,
     536123092433+I*1174918341857,535639056881+I*1174982592643,
     535639056325+I*1174982592701,535637310083+I*1174982641657,
     535637310080+I*1174982641657,530184417436+I*1174721329199,
     530184411204+I*1174721328725,530165104630+I*1174718374757,
     530165104608+I*1174718374753,470705910440+I*1160938152427,
     470705842836+I*1160938134749,470499378458+I*1160867027953,
     470499378224+I*1160867027865,-155991312972+I*890366193943,
     -155992021260+I*890365864277,-158120915826+I*889170760765,
     -158120918224+I*889170759329,-158128050060+I*889165716427,
     -158128050068+I*889165716421,-179155683270+I*871761788657,
     -179155706776+I*871761768065,-179224071168+I*871692032471,
     -179224071244+I*871692032389,-375720180938+I*638193560581,
     -375720398320+I*638193287057,-376332935444+I*637289466739,
     -376332936116+I*637289465685,-376334780270+I*637286022157,
     -376334780272+I*637286022153,-381265048000+I*626118924407,
     -381265053284+I*626118911469,-381278451922+I*626077344353,
     -381278451936+I*626077344305,-413177201452+I*493378733643,
     -413177233700+I*493378580757,-413241484486+I*492894545205,
     -413241484544+I*492894544649,-413241533500+I*492892798407,
     -413241533500+I*492892798404,-412980221042+I*487439905760,
     -412980220568+I*487439899528,-412977266600+I*487420592954,
     -412977266596+I*487420592932,-399197044270+I*427961398764,
     -399197026592+I*427961331160,-399125919796+I*427754866782,
     -399125919708+I*427754866548,-128625085786+I*-198735824648,
     -128624756120+I*-198736532936,-127429652608+I*-200865427502,
     -127429651172+I*-200865429900,-127424608270+I*-200872561736,
     -127424608264+I*-200872561744,-110020680500+I*-221900194946,
     -110020659908+I*-221900218452,-109950924314+I*-221968582844,
     -109950924232+I*-221968582920,123547547576+I*-418464692614,
     123547821100+I*-418464909996,124451641418+I*-419077447120,
     124451642472+I*-419077447792,124455086000+I*-419079291946,
     124455086004+I*-419079291948,135622183750+I*-424009559676,
     135622196688+I*-424009564960,135663763804+I*-424022963598,
     135663763852+I*-424022963612,268362374514+I*-455921713128,
     268362527400+I*-455921745376,268846562952+I*-455985996162,
     268846563508+I*-455985996220 ],
   /* k=35 */ [ 268848309750+I*-455986045176,
     268848309753+I*-455986045176,274301202397+I*-455724732718,
     274301208629+I*-455724732244,274320515203+I*-455721778276,
     274320515225+I*-455721778272,333779709393+I*-441941555946,
     333779776997+I*-441941538268,333986241375+I*-441870431472,
     333986241609+I*-441870431384,960476932805+I*-171369597462,
     960477641093+I*-171369267796,962606535659+I*-170174164284,
     962606538057+I*-170174162848,962613669893+I*-170169119946,
     962613669901+I*-170169119940,983641303103+I*-152765192176,
     983641326609+I*-152765171584,983709691001+I*-152695435990,
     983709691077+I*-152695435908,1180205800771+I*80803035900,
     1180206018153+I*80803309424,1180818555277+I*81707129742,
     1180818555949+I*81707130796,1180820400103+I*81710574324,
     1180820400105+I*81710574328,1185750667833+I*92877672074,
     1185750673117+I*92877685012,1185764071755+I*92919252128,
     1185764071769+I*92919252176,1217662821285+I*225617862838,
     1217662853533+I*225618015724,1217727104319+I*226102051276,
     1217727104377+I*226102051832,1303216127729+I*1749584268146,
     1303216176685+I*1749586014388,1303216176685+I*1749586014391,
     1302954864227+I*1755038907035,1302954863753+I*1755038913267,
     1302951909785+I*1755058219841,1302951909781+I*1755058219863,
     1289171687455+I*1814517414031,1289171669777+I*1814517481635,
     1289100562981+I*1814723946013,1289100562893+I*1814723946247,
     1018599728971+I*2441214637443,1018599399305+I*2441215345731,
     1017404295793+I*2443344240297,1017404294357+I*2443344242695,
     1017399251455+I*2443351374531,1017399251449+I*2443351374539,
     999995323685+I*2464379007741,999995303093+I*2464379031247,
     999925567499+I*2464447395639,999925567417+I*2464447395715,
     766427095609+I*2660943505409,766426822085+I*2660943722791,
     765523001767+I*2661556259915,765523000713+I*2661556260587,
     765519557185+I*2661558104741,765519557181+I*2661558104743,
     754352459435+I*2666488372471,754352446497+I*2666488377755,
     754310879381+I*2666501776393,754310879333+I*2666501776407,
     621612268671+I*2698400525923,621612115785+I*2698400558171,
     621128080233+I*2698464808957,621128079677+I*2698464809015,
     -902354136637+I*2783953832367,-902355882879+I*2783953881323,
     -902355882882+I*2783953881323,-907808775526+I*2783692568865,
     -907808781758+I*2783692568391,-907828088332+I*2783689614423,
     -907828088354+I*2783689614419,-967287282522+I*2769909392093,
     -967287350126+I*2769909374415,-967493814504+I*2769838267619,
     -967493814738+I*2769838267531,-1593984505934+I*2499337433609,
     -1593985214222+I*2499337103943,-1596114108788+I*2498142000431,
     -1596114111186+I*2498141998995,-1596121243022+I*2498136956093,
     -1596121243030+I*2498136956087,-1617148876232+I*2480733028323,
     -1617148899738+I*2480733007731,-1617217264130+I*2480663272137,
     -1617217264206+I*2480663272055,-1813713373900+I*2247164800247,
     -1813713591282+I*2247164526723,-1814326128406+I*2246260706405,
     -1814326129078+I*2246260705351,-1814327973232+I*2246257261823,
     -1814327973234+I*2246257261819,-1819258240962+I*2235090164073,
     -1819258246246+I*2235090151135,-1819271644884+I*2235048584019,
     -1819271644898+I*2235048583971,-1851170394414+I*2102349973309,
     -1851170426662+I*2102349820423,-1851234677448+I*2101865784871,
     -1851234677506+I*2101865784315,-1936723700858+I*578383568001,
     -1936723749814+I*578381821759,-1936723749814+I*578381821756,
     -1936462437356+I*572928929112,-1936462436882+I*572928922880,
     -1936459482914+I*572909616306,-1936459482910+I*572909616284,
     -1922679260584+I*513450422116,-1922679242906+I*513450354512,
     -1922608136110+I*513243890134,-1922608136022+I*513243889900,
     -1652107302100+I*-113246801296,-1652106972434+I*-113247509584,
     -1650911868922+I*-115376404150,-1650911867486+I*-115376406548,
     -1650906824584+I*-115383538384,-1650906824578+I*-115383538392,
     -1633502896814+I*-136411171594,-1633502876222+I*-136411195100,
     -1633433140628+I*-136479559492,-1633433140546+I*-136479559568,
     -1399934668738+I*-332975669262,-1399934395214+I*-332975886644,
     -1399030574896+I*-333588423768,-1399030573842+I*-333588424440,
     -1399027130314+I*-333590268594,-1399027130310+I*-333590268596,
     -1387860032564+I*-338520536324,-1387860019626+I*-338520541608,
     -1387818452510+I*-338533940246,-1387818452462+I*-338533940260,
     -1255119841800+I*-370432689776,-1255119688914+I*-370432722024,
     -1254635653362+I*-370496972810,-1254635652806+I*-370496972868,
     268846563508+I*-455985996220 ],
   /* k=36 */ [ -1425611953268+I*-3417461454452,
     -1425611953265+I*-3417461454452,
     -1420159060621+I*-3417200141994,
     -1420159054389+I*-3417200141520,
     -1420139747815+I*-3417197187552,
     -1420139747793+I*-3417197187548,
     -1360680553625+I*-3403416965222,
     -1360680486021+I*-3403416947544,
     -1360474021643+I*-3403345840748,
     -1360474021409+I*-3403345840660,
     -733983330213+I*-3132845006738,-733982621925+I*-3132844677072,
     -731853727359+I*-3131649573560,-731853724961+I*-3131649572124,
     -731846593125+I*-3131644529222,-731846593117+I*-3131644529216,
     -710818959915+I*-3114240601452,-710818936409+I*-3114240580860,
     -710750572017+I*-3114170845266,-710750571941+I*-3114170845184,
     -514254462247+I*-2880672373376,-514254244865+I*-2880672099852,
     -513641707741+I*-2879768279534,-513641707069+I*-2879768278480,
     1180818555949+I*81707130796,1180820400103+I*81710574324,
     1180820400105+I*81710574328,1185750667833+I*92877672074,
     1185750673117+I*92877685012,1185764071755+I*92919252128,
     1185764071769+I*92919252176,1217662821285+I*225617862838,
     1217662853533+I*225618015724,1217727104319+I*226102051276,
     1217727104377+I*226102051832,1303216127729+I*1749584268146,
     1303216176685+I*1749586014388,1303216176685+I*1749586014391,
     1302954864227+I*1755038907035,1302954863753+I*1755038913267,
     1302951909785+I*1755058219841,1302951909781+I*1755058219863,
     1289171687455+I*1814517414031,1289171669777+I*1814517481635,
     1289100562981+I*1814723946013,1289100562893+I*1814723946247,
     1018599728971+I*2441214637443,1018599399305+I*2441215345731,
     1017404295793+I*2443344240297,1017404294357+I*2443344242695,
     1017399251455+I*2443351374531,1017399251449+I*2443351374539,
     999995323685+I*2464379007741,999995303093+I*2464379031247,
     999925567499+I*2464447395639,999925567417+I*2464447395715,
     766427095609+I*2660943505409,766426822085+I*2660943722791,
     765523001767+I*2661556259915,765523000713+I*2661556260587,
     -2195952408563+I*4356016523605,-2195955852091+I*4356018367759,
     -2195955852095+I*4356018367761,-2207122949841+I*4360948635489,
     -2207122962779+I*4360948640773,-2207164529895+I*4360962039411,
     -2207164529943+I*4360962039425,-2339863140605+I*4392860788941,
     -2339863293491+I*4392860821189,-2340347329043+I*4392925071975,
     -2340347329599+I*4392925072033,-3863829545913+I*4478414095385,
     -3863831292155+I*4478414144341,-3863831292158+I*4478414144341,
     -3869284184802+I*4478152831883,-3869284191034+I*4478152831409,
     -3869303497608+I*4478149877441,-3869303497630+I*4478149877437,
     -3928762691798+I*4464369655111,-3928762759402+I*4464369637433,
     -3928969223780+I*4464298530637,-3928969224014+I*4464298530549,
     -4555459915210+I*4193797696627,-4555460623498+I*4193797366961,
     -4557589518064+I*4192602263449,-4557589520462+I*4192602262013,
     -4557596652298+I*4192597219111,-4557596652306+I*4192597219105,
     -4578624285508+I*4175193291341,-4578624309014+I*4175193270749,
     -4578692673406+I*4175123535155,-4578692673482+I*4175123535073,
     -4775188783176+I*3941625063265,-4775189000558+I*3941624789741,
     -4775801537682+I*3940720969423,-4775801538354+I*3940720968369,
     -6470261801372+I*979245559093,-6470263645526+I*979242115565,
     -6470263645528+I*979242115561,-6475193913256+I*968075017815,
     -6475193918540+I*968075004877,-6475207317178+I*968033437761,
     -6475207317192+I*968033437713,-6507106066708+I*835334827051,
     -6507106098956+I*835334674165,-6507170349742+I*834850638613,
     -6507170349800+I*834850638057,-6592659373152+I*-688631578257,
     -6592659422108+I*-688633324499,-6592659422108+I*-688633324502,
     -6592398109650+I*-694086217146,-6592398109176+I*-694086223378,
     -6592395155208+I*-694105529952,-6592395155204+I*-694105529974,
     -6578614932878+I*-753564724142,-6578614915200+I*-753564791746,
     -6578543808404+I*-753771256124,-6578543808316+I*-753771256358,
     -6308042974394+I*-1380261947554,
     -6308042644728+I*-1380262655842,
     -6306847541216+I*-1382391550408,
     -6306847539780+I*-1382391552806,
     -6306842496878+I*-1382398684642,
     -6306842496872+I*-1382398684650,
     -6289438569108+I*-1403426317852,
     -6289438548516+I*-1403426341358,
     -6289368812922+I*-1403494705750,
     -6289368812840+I*-1403494705826,
     -6055870341032+I*-1599990815520,
     -6055870067508+I*-1599991032902,
     -6054966247190+I*-1600603570026,
     -6054966246136+I*-1600603570698,
     -3093490836860+I*-3295063833716,
     -3093487393332+I*-3295065677870,
     -3093487393328+I*-3295065677872,
     -3082320295582+I*-3299995945600,
     -3082320282644+I*-3299995950884,
     -3082278715528+I*-3300009349522,
     -3082278715480+I*-3300009349536,
     -2949580104818+I*-3331908099052,
     -2949579951932+I*-3331908131300,
     -2949095916380+I*-3331972382086,
     -2949095915824+I*-3331972382144,
     -1425613699510+I*-3417461405496 ],
   /* k=37 */ [ -3547517333046+I*-13996347945298,
     -3547517333043+I*-13996347945298,
     -3542064440399+I*-13996086632840,
     -3542064434167+I*-13996086632366,
     -3542045127593+I*-13996083678398,
     -3542045127571+I*-13996083678394,
     -3482585933403+I*-13982303456068,
     -3482585865799+I*-13982303438390,
     -3482379401421+I*-13982232331594,
     -3482379401187+I*-13982232331506,
     -2855888709991+I*-13711731497584,
     -2855888001703+I*-13711731167918,
     -2853759107137+I*-13710536064406,
     -2853759104739+I*-13710536062970,
     3496636830573+I*-9482045507436,3496643962409+I*-9482040464534,
     3496643962417+I*-9482040464528,3517671595619+I*-9464636536764,
     3517671619125+I*-9464636516172,3517739983517+I*-9464566780578,
     3517739983593+I*-9464566780496,3714236093287+I*-9231068308688,
     3714236310669+I*-9231068035164,3714848847793+I*-9230164214846,
     3714848848465+I*-9230164213792,5409309111483+I*-6268688804516,
     5409310955637+I*-6268685360988,5409310955639+I*-6268685360984,
     5414241223367+I*-6257518263238,5414241228651+I*-6257518250300,
     5414254627289+I*-6257476683184,5414254627303+I*-6257476683136,
     5446153376819+I*-6124778072474,5446153409067+I*-6124777919588,
     5446217659853+I*-6124293884036,5446217659911+I*-6124293883480,
     5531706683263+I*-4600811667166,5531706732219+I*-4600809920924,
     5531706732219+I*-4600809920921,5531445419761+I*-4595357028277,
     5531445419287+I*-4595357022045,5531442465319+I*-4595337715471,
     5531442465315+I*-4595337715449,5517662242989+I*-4535878521281,
     5517662225311+I*-4535878453677,5517591118515+I*-4535671989299,
     5517591118427+I*-4535671989065,5247090284505+I*-3909181297869,
     5247089954839+I*-3909180589581,5245894851327+I*-3907051695015,
     5245894849891+I*-3907051692617,1017404294357+I*2443344242695,
     1017399251455+I*2443351374531,1017399251449+I*2443351374539,
     999995323685+I*2464379007741,999995303093+I*2464379031247,
     999925567499+I*2464447395639,999925567417+I*2464447395715,
     766427095609+I*2660943505409,766426822085+I*2660943722791,
     765523001767+I*2661556259915,765523000713+I*2661556260587,
     -2195952408563+I*4356016523605,-2195955852091+I*4356018367759,
     -2195955852095+I*4356018367761,-2207122949841+I*4360948635489,
     -2207122962779+I*4360948640773,-2207164529895+I*4360962039411,
     -2207164529943+I*4360962039425,-2339863140605+I*4392860788941,
     -2339863293491+I*4392860821189,-2340347329043+I*4392925071975,
     -2340347329599+I*4392925072033,-3863829545913+I*4478414095385,
     -3863831292155+I*4478414144341,-3863831292158+I*4478414144341,
     -3869284184802+I*4478152831883,-3869284191034+I*4478152831409,
     -3869303497608+I*4478149877441,-3869303497630+I*4478149877437,
     -3928762691798+I*4464369655111,-3928762759402+I*4464369637433,
     -3928969223780+I*4464298530637,-3928969224014+I*4464298530549,
     -4555459915210+I*4193797696627,-4555460623498+I*4193797366961,
     -4557589518064+I*4192602263449,-4557589520462+I*4192602262013,
     -10907985455774+I*-35888293521,-10907992587610+I*-35893336423,
     -10907992587618+I*-35893336429,-10929020220820+I*-53297264193,
     -10929020244326+I*-53297284785,-10929088608718+I*-53367020379,
     -10929088608794+I*-53367020461,
     -11125584718488+I*-286865492269,
     -11125584935870+I*-286865765793,
     -11126197472994+I*-287769586111,
     -11126197473666+I*-287769587165,
     -12820657736684+I*-3249244996441,
     -12820659580838+I*-3249248439969,
     -12820659580840+I*-3249248439973,
     -12825589848568+I*-3260415537719,
     -12825589853852+I*-3260415550657,
     -12825603252490+I*-3260457117773,
     -12825603252504+I*-3260457117821,
     -12857502002020+I*-3393155728483,
     -12857502034268+I*-3393155881369,
     -12857566285054+I*-3393639916921,
     -12857566285112+I*-3393639917477,
     -12943055308464+I*-4917122133791,
     -12943055357420+I*-4917123880033,
     -12943055357420+I*-4917123880036,
     -12942794044962+I*-4922576772680,
     -12942794044488+I*-4922576778912,
     -12942791090520+I*-4922596085486,
     -12942791090516+I*-4922596085508,
     -12929010868190+I*-4982055279676,
     -12929010850512+I*-4982055347280,
     -12928939743716+I*-4982261811658,
     -12928939743628+I*-4982261811892,
     -12658438909706+I*-5608752503088,
     -12658438580040+I*-5608753211376,
     -12657243476528+I*-5610882105942,
     -12657243475092+I*-5610882108340,
     -8428752919558+I*-11961278043652,
     -8428747876656+I*-11961285175488,
     -8428747876650+I*-11961285175496,
     -8411343948886+I*-11982312808698,
     -8411343928294+I*-11982312832204,
     -8411274192700+I*-11982381196596,
     -8411274192618+I*-11982381196672,
     -8177775720810+I*-12178877306366,
     -8177775447286+I*-12178877523748,
     -8176871626968+I*-12179490060872,
     -8176871625914+I*-12179490061544,
     -5215396216638+I*-13873950324562,
     -5215392773110+I*-13873952168716,
     -5215392773106+I*-13873952168718,
     -5204225675360+I*-13878882436446,
     -5204225662422+I*-13878882441730,
     -5204184095306+I*-13878895840368,
     -5204184095258+I*-13878895840382,
     -5071485484596+I*-13910794589898,
     -5071485331710+I*-13910794622146,
     -5071001296158+I*-13910858872932,
     -5071001295602+I*-13910858872990,
     -3547519079288+I*-13996347896342 ],
   /* k=38 */ [ -1440932157290+I*-30925630371456,
     -1440932157287+I*-30925630371456,
     -1435479264643+I*-30925369058998,
     -1435479258411+I*-30925369058524,
     15493803167747+I*-28818783882768,
     15493822474321+I*-28818780928800,
     15493822474343+I*-28818780928796,
     15553281668511+I*-28805000706470,
     15553281736115+I*-28805000688792,
     15553488200493+I*-28804929581996,
     15553488200727+I*-28804929581908,
     16179978891923+I*-28534428747986,
     16179979600211+I*-28534428418320,
     16182108494777+I*-28533233314808,
     16182108497175+I*-28533233313372,
     22532504432487+I*-24304742757838,
     22532511564323+I*-24304737714936,
     22532511564331+I*-24304737714930,
     22553539197533+I*-24287333787166,
     22553539221039+I*-24287333766574,
     22553607585431+I*-24287264030980,
     22553607585507+I*-24287264030898,
     22750103695201+I*-24053765559090,
     22750103912583+I*-24053765285566,
     22750716449707+I*-24052861465248,
     22750716450379+I*-24052861464194,
     24445176713397+I*-21091386054918,
     24445178557551+I*-21091382611390,
     24445178557553+I*-21091382611386,
     24450108825281+I*-21080215513640,
     24450108830565+I*-21080215500702,
     24450122229203+I*-21080173933586,
     24450122229217+I*-21080173933538,
     24482020978733+I*-20947475322876,
     24482021010981+I*-20947475169990,
     24482085261767+I*-20946991134438,
     24482085261825+I*-20946991133882,
     24567574285177+I*-19423508917568,
     24567574334133+I*-19423507171326,
     24567574334133+I*-19423507171323,
     24567313021675+I*-19418054278679,
     24567313021201+I*-19418054272447,
     22460727845445+I*-2488771846289,
     22460724891477+I*-2488752539715,
     22460724891473+I*-2488752539693,
     22446944669147+I*-2429293345525,
     22446944651469+I*-2429293277921,
     22446873544673+I*-2429086813543,
     22446873544585+I*-2429086813309,
     22176372710663+I*-1802596122113,
     22176372380997+I*-1802595413825,
     22175177277485+I*-1800466519259,
     22175177276049+I*-1800466516861,
     17946686720515+I*4549929418451,17946681677613+I*4549936550287,
     17946681677607+I*4549936550295,17929277749843+I*4570964183497,
     17929277729251+I*4570964207003,17929207993657+I*4571032571395,
     17929207993575+I*4571032571471,17695709521767+I*4767528681165,
     17695709248243+I*4767528898547,17694805427925+I*4768141435671,
     17694805426871+I*4768141436343,14733330017595+I*6462601699361,
     14733326574067+I*6462603543515,14733326574063+I*6462603543517,
     14722159476317+I*6467533811245,14722159463379+I*6467533816529,
     14722117896263+I*6467547215167,14722117896215+I*6467547215181,
     14589419285553+I*6499445964697,14589419132667+I*6499445996945,
     14588935097115+I*6499510247731,14588935096559+I*6499510247789,
     13065452880245+I*6584999271141,13065451134003+I*6584999320097,
     13065451134000+I*6584999320097,13059998241356+I*6584738007639,
     13059998235124+I*6584738007165,-3869284191034+I*4478152831409,
     -3869303497608+I*4478149877441,-3869303497630+I*4478149877437,
     -3928762691798+I*4464369655111,-3928762759402+I*4464369637433,
     -3928969223780+I*4464298530637,-3928969224014+I*4464298530549,
     -4555459915210+I*4193797696627,-4555460623498+I*4193797366961,
     -4557589518064+I*4192602263449,-4557589520462+I*4192602262013,
     -10907985455774+I*-35888293521,-10907992587610+I*-35893336423,
     -10907992587618+I*-35893336429,-10929020220820+I*-53297264193,
     -10929020244326+I*-53297284785,-10929088608718+I*-53367020379,
     -10929088608794+I*-53367020461,
     -11125584718488+I*-286865492269,
     -11125584935870+I*-286865765793,
     -11126197472994+I*-287769586111,
     -11126197473666+I*-287769587165,
     -12820657736684+I*-3249244996441,
     -12820659580838+I*-3249248439969,
     -12820659580840+I*-3249248439973,
     -12825589848568+I*-3260415537719,
     -12825589853852+I*-3260415550657,
     -12825603252490+I*-3260457117773,
     -12825603252504+I*-3260457117821,
     -12857502002020+I*-3393155728483,
     -12857502034268+I*-3393155881369,
     -12857566285054+I*-3393639916921,
     -12857566285112+I*-3393639917477,
     -12943055308464+I*-4917122133791,
     -12943055357420+I*-4917123880033,
     -12943055357420+I*-4917123880036,
     -12942794044962+I*-4922576772680,
     -12942794044488+I*-4922576778912,
     -10836208868732+I*-21851859205070,
     -10836205914764+I*-21851878511644,
     -10836205914760+I*-21851878511666,
     -10822425692434+I*-21911337705834,
     -10822425674756+I*-21911337773438,
     -10822354567960+I*-21911544237816,
     -10822354567872+I*-21911544238050,
     -10551853733950+I*-22538034929246,
     -10551853404284+I*-22538035637534,
     -10550658300772+I*-22540164532100,
     -10550658299336+I*-22540164534498,
     -6322167743802+I*-28890560469810,
     -6322162700900+I*-28890567601646,
     -6322162700894+I*-28890567601654,
     -6304758773130+I*-28911595234856,
     -6304758752538+I*-28911595258362,
     -6304689016944+I*-28911663622754,
     -6304689016862+I*-28911663622830,
     -6071190545054+I*-29108159732524,
     -6071190271530+I*-29108159949906,
     -6070286451212+I*-29108772487030,
     -6070286450158+I*-29108772487702,
     -3108811040882+I*-30803232750720,
     -3108807597354+I*-30803234594874,
     -3108807597350+I*-30803234594876,
     -3097640499604+I*-30808164862604,
     -3097640486666+I*-30808164867888,
     -3097598919550+I*-30808178266526,
     -3097598919502+I*-30808178266540,
     -2964900308840+I*-30840077016056,
     -2964900155954+I*-30840077048304,
     -2964416120402+I*-30840141299090,
     -2964416119846+I*-30840141299148,
     -1440933903532+I*-30925630322500 ],
   /* k=39 */ [ 34524217870782+I*-43641742446102,
     34524217870785+I*-43641742446102,
     34529670763429+I*-43641481133644,
     34529670769661+I*-43641481133170,
     51458953195819+I*-41534895957414,
     51458972502393+I*-41534893003446,
     51458972502415+I*-41534893003442,
     51518431696583+I*-41521112781116,
     51518431764187+I*-41521112763438,
     51518638228565+I*-41521041656642,
     51518638228799+I*-41521041656554,
     52145128919995+I*-41250540822632,
     52145129628283+I*-41250540492966,
     52147258522849+I*-41249345389454,
     52147258525247+I*-41249345388018,
     58497654460559+I*-37020854832484,
     58497661592395+I*-37020849789582,
     58497661592403+I*-37020849789576,
     58518689225605+I*-37003445861812,
     58518689249111+I*-37003445841220,
     58518757613503+I*-37003376105626,
     58518757613579+I*-37003376105544,
     58715253723273+I*-36769877633736,
     58715253940655+I*-36769877360212,
     58715866477779+I*-36768973539894,
     58715866478451+I*-36768973538840,
     60410326741469+I*-33807498129564,
     60410328585623+I*-33807494686036,
     60410328585625+I*-33807494686032,
     60415258853353+I*-33796327588286,
     60415258858637+I*-33796327575348,
     73131370933283+I*2168822452724,73131384331921+I*2168864019840,
     73131384331935+I*2168864019888,73163283081451+I*2301562630550,
     73163283113699+I*2301562783436,73163347364485+I*2302046818988,
     73163347364543+I*2302046819544,73248836387895+I*3825529035858,
     73248836436851+I*3825530782100,73248836436851+I*3825530782103,
     73248575124393+I*3830983674747,73248575123919+I*3830983680979,
     71141989948163+I*20760266107137,
     71141986994195+I*20760285413711,
     71141986994191+I*20760285413733,
     71128206771865+I*20819744607901,
     71128206754187+I*20819744675505,
     71128135647391+I*20819951139883,
     71128135647303+I*20819951140117,
     70857634813381+I*21446441831313,
     70857634483715+I*21446442539601,
     70856439380203+I*21448571434167,
     70856439378767+I*21448571436565,
     66627948823233+I*27798967371877,
     66627943780331+I*27798974503713,
     66627943780325+I*27798974503721,
     66610539852561+I*27820002136923,
     66610539831969+I*27820002160429,
     66610470096375+I*27820070524821,
     66610470096293+I*27820070524897,
     66376971624485+I*28016566634591,
     66376971350961+I*28016566851973,
     66376067530643+I*28017179389097,
     66376067529589+I*28017179389769,
     63414592120313+I*29711639652787,
     63414588676785+I*29711641496941,
     63414588676781+I*29711641496943,
     63403421579035+I*29716571764671,
     63403421566097+I*29716571769955,
     27438271538025+I*42432683844601,
     27438229970909+I*42432697243239,
     27438229970861+I*42432697243253,
     27305531360199+I*42464595992769,
     27305531207313+I*42464596025017,
     27305047171761+I*42464660275803,
     27305047171205+I*42464660275861,
     25781564954891+I*42550149299213,
     25781563208649+I*42550149348169,
     25781563208646+I*42550149348169,
     25776110316002+I*42549888035711,
     25776110309770+I*42549888035237,
     8846827883612+I*40443302859481,8846808577038+I*40443299905513,
     8846808577016+I*40443299905509,8787349382848+I*40429519683183,
     8787349315244+I*40429519665505,8787142850866+I*40429448558709,
     8787142850632+I*40429448558621,8160652159436+I*40158947724699,
     8160651451148+I*40158947395033,8158522556582+I*40157752291521,
     8158522554184+I*40157752290085,1808126618872+I*35929261734551,
     1808119487036+I*35929256691649,1808119487028+I*35929256691643,
     1787091853826+I*35911852763879,1787091830320+I*35911852743287,
     1787023465928+I*35911783007693,1787023465852+I*35911783007611,
     1590527356158+I*35678284535803,1590527138776+I*35678284262279,
     1589914601652+I*35677380441961,1589914600980+I*35677380440907,
     -104545662038+I*32715905031631,-104547506192+I*32715901588103,
     -104547506194+I*32715901588099,-109477773922+I*32704734490353,
     -109477779206+I*32704734477415,
     -12825589853852+I*-3260415550657,
     -12825603252490+I*-3260457117773,
     -12825603252504+I*-3260457117821,
     -12857502002020+I*-3393155728483,
     -12857502034268+I*-3393155881369,
     -12857566285054+I*-3393639916921,
     -12857566285112+I*-3393639917477,
     -12943055308464+I*-4917122133791,
     -12943055357420+I*-4917123880033,
     -12943055357420+I*-4917123880036,
     -12942794044962+I*-4922576772680,
     -12942794044488+I*-4922576778912,
     -10836208868732+I*-21851859205070,
     -10836205914764+I*-21851878511644,
     -10836205914760+I*-21851878511666,
     -10822425692434+I*-21911337705834,
     -10822425674756+I*-21911337773438,
     -10822354567960+I*-21911544237816,
     -10822354567872+I*-21911544238050,
     -10551853733950+I*-22538034929246,
     -10551853404284+I*-22538035637534,
     -10550658300772+I*-22540164532100,
     -10550658299336+I*-22540164534498,
     -6322167743802+I*-28890560469810,
     -6322162700900+I*-28890567601646,
     -6322162700894+I*-28890567601654,
     -6304758773130+I*-28911595234856,
     -6304758752538+I*-28911595258362,
     -6304689016944+I*-28911663622754,
     -6304689016862+I*-28911663622830,
     -6071190545054+I*-29108159732524,
     -6071190271530+I*-29108159949906,
     -6070286451212+I*-29108772487030,
     -6070286450158+I*-29108772487702,
     -3108811040882+I*-30803232750720,
     -3108807597354+I*-30803234594874,
     -3108807597350+I*-30803234594876,
     -3097640499604+I*-30808164862604,
     -3097640486666+I*-30808164867888,
     32867509541406+I*-43524276942534,
     32867551108522+I*-43524290341172,
     32867551108570+I*-43524290341186,
     33000249719232+I*-43556189090702,
     33000249872118+I*-43556189122950,
     33000733907670+I*-43556253373736,
     33000733908226+I*-43556253373794,
     34524216124540+I*-43641742397146 ],
   /* k=40 */ [ 34524217870782+I*-43641742446102,
     34524217870785+I*-43641742446102,
     34529670763429+I*-43641481133644,
     34529670769661+I*-43641481133170,
     51458953195819+I*-41534895957414,
     51458972502393+I*-41534893003446,
     51458972502415+I*-41534893003442,
     51518431696583+I*-41521112781116,
     51518431764187+I*-41521112763438,
     51518638228565+I*-41521041656642,
     51518638228799+I*-41521041656554,
     52145128919995+I*-41250540822632,
     52145129628283+I*-41250540492966,
     52147258522849+I*-41249345389454,
     52147258525247+I*-41249345388018,
     58497654460559+I*-37020854832484,
     58497661592395+I*-37020849789582,
     58497661592403+I*-37020849789576,
     58518689225605+I*-37003445861812,
     58518689249111+I*-37003445841220,
     119916063426475+I*22210742140278,
     119916131790867+I*22210811875872,
     119916131790943+I*22210811875954,
     120112627900637+I*22444310347762,
     120112628118019+I*22444310621286,
     120113240655143+I*22445214441604,
     120113240655815+I*22445214442658,
     121807700918833+I*25406689851934,
     121807702762987+I*25406693295462,
     121807702762989+I*25406693295466,
     121812633030717+I*25417860393212,
     121812633036001+I*25417860406150,
     134528745110647+I*61383010434222,
     134528758509285+I*61383052001338,
     134528758509299+I*61383052001386,
     134560657258815+I*61515750612048,
     134560657291063+I*61515750764934,
     134560721541849+I*61516234800486,
     134560721541907+I*61516234801042,
     134646210565259+I*63039717017356,
     134646210614215+I*63039718763598,
     134646210614215+I*63039718763601,
     134645949301757+I*63045171656245,
     134645949301283+I*63045171662477,
     132539364125527+I*79974454088635,
     132539361171559+I*79974473395209,
     132539361171555+I*79974473395231,
     132525580949229+I*80033932589399,
     132525580931551+I*80033932657003,
     132525509824755+I*80034139121381,
     132525509824667+I*80034139121615,
     132255008990745+I*80660629812811,
     132255008661079+I*80660630521099,
     132253813557567+I*80662759415665,
     132253813556131+I*80662759418063,
     128025323000597+I*87013155353375,
     128025317957695+I*87013162485211,
     128025317957689+I*87013162485219,
     128007914029925+I*87034190118421,
     128007914009333+I*87034190141927,
     68793726027835+I*148431564319291,
     68793656292241+I*148431632683683,
     68793656292159+I*148431632683759,
     68560157820351+I*148628128793453,
     68560157546827+I*148628129010835,
     68559253726509+I*148628741547959,
     68559253725455+I*148628741548631,
     65597778316179+I*150323201811649,
     65597774872651+I*150323203655803,
     65597774872647+I*150323203655805,
     65586607774901+I*150328133923533,
     65586607761963+I*150328133928817,
     29621457733891+I*163044246003463,
     29621416166775+I*163044259402101,
     29621416166727+I*163044259402115,
     29488717556065+I*163076158151631,
     29488717403179+I*163076158183879,
     29488233367627+I*163076222434665,
     29488233367071+I*163076222434723,
     27964751150757+I*163161711458075,
     27964749404515+I*163161711507031,
     27964749404512+I*163161711507031,
     27959296511868+I*163161450194573,
     27959296505636+I*163161450194099,
     11030014079478+I*161054865018343,
     11029994772904+I*161054862064375,
     11029994772882+I*161054862064371,
     10970535578714+I*161041081842045,
     10970535511110+I*161041081824367,
     10970329046732+I*161041010717571,
     10970329046498+I*161041010717483,
     10343838355302+I*160770509883561,
     10343837647014+I*160770509553895,
     10341708752448+I*160769314450383,
     10341708750050+I*160769314448947,
     3991312814738+I*156540823893413,
     3991305682902+I*156540818850511,
     3991305682894+I*156540818850505,
     3970278049692+I*156523414922741,
     3970278026186+I*156523414902149,
     -57427096151178+I*97309226920651,
     -57427164515570+I*97309157185057,
     -57427164515646+I*97309157184975,
     -57623660625340+I*97075658713167,
     -57623660842722+I*97075658439643,
     -57624273379846+I*97074754619325,
     -57624273380518+I*97074754618271,
     -59318733643536+I*94113279208995,
     -59318735487690+I*94113275765467,
     -59318735487692+I*94113275765463,
     -59323665755420+I*94102108667717,
     -59323665760704+I*94102108654779,
     -72039777835350+I*58136958626707,
     -72039791233988+I*58136917059591,
     -72039791234002+I*58136917059543,
     -72071689983518+I*58004218448881,
     -72071690015766+I*58004218295995,
     -72071754266552+I*58003734260443,
     -72071754266610+I*58003734259887,
     -72157243289962+I*56480252043573,
     -72157243338918+I*56480250297331,
     -72157243338918+I*56480250297328,
     -72156982026460+I*56474797404684,
     -72156982025986+I*56474797398452,
     -70050396850230+I*39545514972294,
     -70050393896262+I*39545495665720,
     -70050393896258+I*39545495665698,
     -70036613673932+I*39486036471530,
     -70036613656254+I*39486036403926,
     -70036542549458+I*39485829939548,
     -70036542549370+I*39485829939314,
     -69766041715448+I*38859339248118,
     -69766041385782+I*38859338539830,
     -69764846282270+I*38857209645264,
     -69764846280834+I*38857209642866,
     -65536355725300+I*32506813707554,
     -65536350682398+I*32506806575718,
     -65536350682392+I*32506806575710,
     -65518946754628+I*32485778942508,
     -65518946734036+I*32485778919002,
     -6304758752538+I*-28911595258362,
     -6304689016944+I*-28911663622754,
     -6304689016862+I*-28911663622830,
     -6071190545054+I*-29108159732524,
     -6071190271530+I*-29108159949906,
     -6070286451212+I*-29108772487030,
     -6070286450158+I*-29108772487702,
     -3108811040882+I*-30803232750720,
     -3108807597354+I*-30803234594874,
     -3108807597350+I*-30803234594876,
     -3097640499604+I*-30808164862604,
     -3097640486666+I*-30808164867888,
     32867509541406+I*-43524276942534,
     32867551108522+I*-43524290341172,
     32867551108570+I*-43524290341186,
     33000249719232+I*-43556189090702,
     33000249872118+I*-43556189122950,
     33000733907670+I*-43556253373736,
     33000733908226+I*-43556253373794,
     34524216124540+I*-43641742397146 ],
   /* k=41 */ [ -147484718465444+I*-100672744231734,
     -147484718465441+I*-100672744231734,
     -147479265572797+I*-100672482919276,
     -147479265566565+I*-100672482918802,
     -130549983140407+I*-98565897743046,
     -130549963833833+I*-98565894789078,
     -130549963833811+I*-98565894789074,
     -130490504639643+I*-98552114566748,
     -130490504572039+I*-98552114549070,
     51518431764187+I*-41521112763438,
     51518638228565+I*-41521041656642,
     51518638228799+I*-41521041656554,
     52145128919995+I*-41250540822632,
     52145129628283+I*-41250540492966,
     52147258522849+I*-41249345389454,
     52147258525247+I*-41249345388018,
     58497654460559+I*-37020854832484,
     58497661592395+I*-37020849789582,
     58497661592403+I*-37020849789576,
     58518689225605+I*-37003445861812,
     58518689249111+I*-37003445841220,
     119916063426475+I*22210742140278,
     119916131790867+I*22210811875872,
     119916131790943+I*22210811875954,
     120112627900637+I*22444310347762,
     120112628118019+I*22444310621286,
     120113240655143+I*22445214441604,
     120113240655815+I*22445214442658,
     121807700918833+I*25406689851934,
     121807702762987+I*25406693295462,
     121807702762989+I*25406693295466,
     121812633030717+I*25417860393212,
     121812633036001+I*25417860406150,
     134528745110647+I*61383010434222,
     134528758509285+I*61383052001338,
     134528758509299+I*61383052001386,
     134560657258815+I*61515750612048,
     134560657291063+I*61515750764934,
     134560721541849+I*61516234800486,
     134560721541907+I*61516234801042,
     134646210565259+I*63039717017356,
     134646210614215+I*63039718763598,
     134646210614215+I*63039718763601,
     134645949301757+I*63045171656245,
     134645949301283+I*63045171662477,
     132539364125527+I*79974454088635,
     132539361171559+I*79974473395209,
     132539361171555+I*79974473395231,
     132525580949229+I*80033932589399,
     132525580931551+I*80033932657003,
     75494579145919+I*262042868993229,
     75494508039123+I*262043075457607,
     75494508039035+I*262043075457841,
     75224007205113+I*262669566149037,
     75224006875447+I*262669566857325,
     75222811771935+I*262671695751891,
     75222811770499+I*262671695754289,
     70994321214965+I*269022091689601,
     70994316172063+I*269022098821437,
     70994316172057+I*269022098821445,
     70976912244293+I*269043126454647,
     70976912223701+I*269043126478153,
     11762724242203+I*330440500655517,
     11762654506609+I*330440569019909,
     11762654506527+I*330440569019985,
     11529156034719+I*330637065129679,
     11529155761195+I*330637065347061,
     11528251940877+I*330637677884185,
     11528251939823+I*330637677884857,
     8566776530547+I*332332138147875,
     8566773087019+I*332332139992029,
     8566773087015+I*332332139992031,
     8555605989269+I*332337070259759,
     8555605976331+I*332337070265043,
     -27409544051741+I*345053182339689,
     -27409585618857+I*345053195738327,
     -27409585618905+I*345053195738341,
     -27542284229567+I*345085094487857,
     -27542284382453+I*345085094520105,
     -27542768418005+I*345085158770891,
     -27542768418561+I*345085158770949,
     -29066250634875+I*345170647794301,
     -29066252381117+I*345170647843257,
     -29066252381120+I*345170647843257,
     -29071705273764+I*345170386530799,
     -29071705279996+I*345170386530325,
     -46000987706154+I*343063801354569,
     -46001007012728+I*343063798400601,
     -46001007012750+I*343063798400597,
     -46060466206918+I*343050018178271,
     -46060466274522+I*343050018160593,
     -228069402610748+I*286019016374961,
     -228069609075126+I*286018945268165,
     -228069609075360+I*286018945268077,
     -228696099766556+I*285748444434155,
     -228696100474844+I*285748444104489,
     -228698229369410+I*285747249000977,
     -228698229371808+I*285747248999541,
     -235048625307120+I*281518758444007,
     -235048632438956+I*281518753401105,
     -235048632438964+I*281518753401099,
     -235069660072166+I*281501349473335,
     -235069660095672+I*281501349452743,
     -296467034273036+I*222287161471245,
     -296467102637428+I*222287091735651,
     -296467102637504+I*222287091735569,
     -296663598747198+I*222053593263761,
     -296663598964580+I*222053592990237,
     -296664211501704+I*222052689169919,
     -296664211502376+I*222052689168865,
     -298358671765394+I*219091213759589,
     -298358673609548+I*219091210316061,
     -298358673609550+I*219091210316057,
     -298363603877278+I*219080043218311,
     -298363603882562+I*219080043205373,
     -311079715957208+I*183114893177301,
     -311079729355846+I*183114851610185,
     -311079729355860+I*183114851610137,
     -311111628105376+I*182982152999475,
     -311111628137624+I*182982152846589,
     -311111692388410+I*182981668811037,
     -311111692388468+I*182981668810481,
     -311197181411820+I*181458186594167,
     -311197181460776+I*181458184847925,
     -311197181460776+I*181458184847922,
     -311196920148318+I*181452731955278,
     -311196920147844+I*181452731949046,
     -309090334972088+I*164523449522888,
     -309090332018120+I*164523430216314,
     -309090332018116+I*164523430216292,
     -309076551795790+I*164463971022124,
     -309076551778112+I*164463970954520,
     -252045549992480+I*-17544965381706,
     -252045478885684+I*-17545171846084,
     -252045478885596+I*-17545171846318,
     -251774978051674+I*-18171662537514,
     -251774977722008+I*-18171663245802,
     -251773782618496+I*-18173792140368,
     -251773782617060+I*-18173792142766,
     -247545292061526+I*-24524188078078,
     -247545287018624+I*-24524195209914,
     -247545287018618+I*-24524195209922,
     -247527883090854+I*-24545222843124,
     -247527883070262+I*-24545222866630,
     -188313695088764+I*-85942597043994,
     -188313625353170+I*-85942665408386,
     -188313625353088+I*-85942665408462,
     -188080126881280+I*-86139161518156,
     -188080126607756+I*-86139161735538,
     -188079222787438+I*-86139774272662,
     -188079222786384+I*-86139774273334,
     -185117747377108+I*-87834234536352,
     -185117743933580+I*-87834236380506,
     -185117743933576+I*-87834236380508,
     -185106576835830+I*-87839166648236,
     -185106576822892+I*-87839166653520,
     -149141426794820+I*-100555278728166,
     -149141385227704+I*-100555292126804,
     -149141385227656+I*-100555292126818,
     -149008686616994+I*-100587190876334,
     -149008686464108+I*-100587190908582,
     -149008202428556+I*-100587255159368,
     -149008202428000+I*-100587255159426,
     -147484720211686+I*-100672744182778 ],
   /* k=42 */ [ -215431651230406+I*-521721618689818,
     -215431651230403+I*-521721618689818,
     -215426198337759+I*-521721357377360,
     -215426198331527+I*-521721357376886,
     -198496915905369+I*-519614772201130,
     -198496896598795+I*-519614769247162,
     -198496896598773+I*-519614769247158,
     -198437437404605+I*-519600989024832,
     -198437437337001+I*-519600989007154,
     -16428501000775+I*-462569987221522,
     -16428294536397+I*-462569916114726,
     -16428294536163+I*-462569916114638,
     -15801803844967+I*-462299415280716,
     -15801803136679+I*-462299414951050,
     -15799674242113+I*-462298219847538,
     -15799674239715+I*-462298219846102,
     -9449278304403+I*-458069729290568,
     -9449271172567+I*-458069724247666,
     -9449271172559+I*-458069724247660,
     -9428243539357+I*-458052320319896,
     -9428243515851+I*-458052320299304,
     51969130661513+I*-398838132317806,
     51969199025905+I*-398838062582212,
     51969199025981+I*-398838062582130,
     52165695135675+I*-398604564110322,
     52165695353057+I*-398604563836798,
     52166307890181+I*-398603660016480,
     52166307890853+I*-398603660015426,
     53860768153871+I*-395642184606150,
     53860769998025+I*-395642181162622,
     53860769998027+I*-395642181162618,
     53865700265755+I*-395631014064872,
     53865700271039+I*-395631014051934,
     66581812345685+I*-359665864023862,
     66581825744323+I*-359665822456746,
     66581825744337+I*-359665822456698,
     66613724493853+I*-359533123846036,
     66613724526101+I*-359533123693150,
     134560657291063+I*61515750764934,
     134560721541849+I*61516234800486,
     134560721541907+I*61516234801042,
     134646210565259+I*63039717017356,
     134646210614215+I*63039718763598,
     134646210614215+I*63039718763601,
     134645949301757+I*63045171656245,
     134645949301283+I*63045171662477,
     132539364125527+I*79974454088635,
     132539361171559+I*79974473395209,
     132539361171555+I*79974473395231,
     132525580949229+I*80033932589399,
     132525580931551+I*80033932657003,
     75494579145919+I*262042868993229,
     75494508039123+I*262043075457607,
     75494508039035+I*262043075457841,
     75224007205113+I*262669566149037,
     75224006875447+I*262669566857325,
     75222811771935+I*262671695751891,
     75222811770499+I*262671695754289,
     70994321214965+I*269022091689601,
     70994316172063+I*269022098821437,
     70994316172057+I*269022098821445,
     70976912244293+I*269043126454647,
     70976912223701+I*269043126478153,
     11762724242203+I*330440500655517,
     11762654506609+I*330440569019909,
     11762654506527+I*330440569019985,
     11529156034719+I*330637065129679,
     11529155761195+I*330637065347061,
     11528251940877+I*330637677884185,
     11528251939823+I*330637677884857,
     8566776530547+I*332332138147875,
     8566773087019+I*332332139992029,
     8566773087015+I*332332139992031,
     8555605989269+I*332337070259759,
     8555605976331+I*332337070265043,
     -27409544051741+I*345053182339689,
     -27409585618857+I*345053195738327,
     -27409585618905+I*345053195738341,
     -27542284229567+I*345085094487857,
     -27542284382453+I*345085094520105,
     -448591158840537+I*413032027285067,
     -448591642876089+I*413032091535853,
     -448591642876645+I*413032091535911,
     -450115125092959+I*413117580559263,
     -450115126839201+I*413117580608219,
     -450115126839204+I*413117580608219,
     -450120579731848+I*413117319295761,
     -450120579738080+I*413117319295287,
     -467049862164238+I*411010734119531,
     -467049881470812+I*411010731165563,
     -467049881470834+I*411010731165559,
     -467109340665002+I*410996950943233,
     -467109340732606+I*410996950925555,
     -649118277068832+I*353965949139923,
     -649118483533210+I*353965878033127,
     -649118483533444+I*353965878033039,
     -649744974224640+I*353695377199117,
     -649744974932928+I*353695376869451,
     -649747103827494+I*353694181765939,
     -649747103829892+I*353694181764503,
     -656097499765204+I*349465691208969,
     -656097506897040+I*349465686166067,
     -656097506897048+I*349465686166061,
     -656118534530250+I*349448282238297,
     -656118534553756+I*349448282217705,
     -717515908731120+I*290234094236207,
     -717515977095512+I*290234024500613,
     -717515977095588+I*290234024500531,
     -717712473205282+I*290000526028723,
     -717712473422664+I*290000525755199,
     -717713085959788+I*289999621934881,
     -717713085960460+I*289999621933827,
     -719407546223478+I*287038146524551,
     -719407548067632+I*287038143081023,
     -719407548067634+I*287038143081019,
     -719412478335362+I*287026975983273,
     -719412478340646+I*287026975970335,
     -732128590415292+I*251061825942263,
     -732128603813930+I*251061784375147,
     -732128603813944+I*251061784375099,
     -732160502563460+I*250929085764437,
     -732160502595708+I*250929085611551,
     -800107435360670+I*-170119788846533,
     -800107499611456+I*-170120272882085,
     -800107499611514+I*-170120272882641,
     -800192988634866+I*-171643755098955,
     -800192988683822+I*-171643756845197,
     -800192988683822+I*-171643756845200,
     -800192727371364+I*-171649209737844,
     -800192727370890+I*-171649209744076,
     -798086142195134+I*-188578492170234,
     -798086139241166+I*-188578511476808,
     -798086139241162+I*-188578511476830,
     -798072359018836+I*-188637970670998,
     -798072359001158+I*-188637970738602,
     -741041357215526+I*-370646907074828,
     -741041286108730+I*-370647113539206,
     -741041286108642+I*-370647113539440,
     -740770785274720+I*-371273604230636,
     -740770784945054+I*-371273604938924,
     -740769589841542+I*-371275733833490,
     -740769589840106+I*-371275733835888,
     -736541099284572+I*-377626129771200,
     -736541094241670+I*-377626136903036,
     -736541094241664+I*-377626136903044,
     -736523690313900+I*-377647164536246,
     -736523690293308+I*-377647164559752,
     -677309502311810+I*-439044538737116,
     -677309432576216+I*-439044607101508,
     -677309432576134+I*-439044607101584,
     -677075934104326+I*-439241103211278,
     -677075933830802+I*-439241103428660,
     -677075030010484+I*-439241715965784,
     -677075030009430+I*-439241715966456,
     -674113554600154+I*-440936176229474,
     -674113551156626+I*-440936178073628,
     -674113551156622+I*-440936178073630,
     -674102384058876+I*-440941108341358,
     -674102384045938+I*-440941108346642,
     -638137234017866+I*-453657220421288,
     -638137192450750+I*-453657233819926,
     -638137192450702+I*-453657233819940,
     -638004493840040+I*-453689132569456,
     -638004493687154+I*-453689132601704,
     -216955619229070+I*-521636065366666,
     -216955135193518+I*-521636129617452,
     -216955135192962+I*-521636129617510,
     -215431652976648+I*-521721618640862 ],
   /* k=43 */ [ 1776424932792+I*-1852815174829032,
     1776424932795+I*-1852815174829032,
     1781877825439+I*-1852814913516574,
     1781877831671+I*-1852814913516100,
     18711160257829+I*-1850708328340344,
     18711179564403+I*-1850708325386376,
     18711179564425+I*-1850708325386372,
     18770638758593+I*-1850694545164046,
     18770638826197+I*-1850694545146368,
     200779575162423+I*-1793663543360736,
     200779781626801+I*-1793663472253940,
     200779781627035+I*-1793663472253852,
     201406272318231+I*-1793392971419930,
     201406273026519+I*-1793392971090264,
     201408401921085+I*-1793391775986752,
     201408401923483+I*-1793391775985316,
     207758797858795+I*-1789163285429782,
     207758804990631+I*-1789163280386880,
     207758804990639+I*-1789163280386874,
     207779832623841+I*-1789145876459110,
     207779832647347+I*-1789145876438518,
     269177206824711+I*-1729931688457020,
     269177275189103+I*-1729931618721426,
     269177275189179+I*-1729931618721344,
     269373771298873+I*-1729698120249536,
     269373771516255+I*-1729698119976012,
     826316511504263+I*-955547303824806,
     826317124041387+I*-955546400004488,
     826317124042059+I*-955546400003434,
     828011584305077+I*-952584924594158,
     828011586149231+I*-952584921150630,
     828011586149233+I*-952584921150626,
     828016516416961+I*-952573754052880,
     828016516422245+I*-952573754039942,
     840732628496891+I*-916608604011870,
     840732641895529+I*-916608562444754,
     840732641895543+I*-916608562444706,
     840764540645059+I*-916475863834044,
     840764540677307+I*-916475863681158,
     908711473442269+I*-495426989223074,
     908711537693055+I*-495426505187522,
     908711537693113+I*-495426505186966,
     908797026716465+I*-493903022970652,
     908797026765421+I*-493903021224410,
     908797026765421+I*-493903021224407,
     908796765452963+I*-493897568331763,
     908796765452489+I*-493897568325531,
     906690180276733+I*-476968285899373,
     906690177322765+I*-476968266592799,
     906690177322761+I*-476968266592777,
     906676397100435+I*-476908807398609,
     906676397082757+I*-476908807331005,
     849645395297125+I*-294899870994779,
     849645324190329+I*-294899664530401,
     849645324190241+I*-294899664530167,
     849374823356319+I*-294273173838971,
     849374823026653+I*-294273173130683,
     849373627923141+I*-294271044236117,
     849373627921705+I*-294271044233719,
     845145137366171+I*-287920648298407,
     845145132323269+I*-287920641166571,
     845145132323263+I*-287920641166563,
     845127728395499+I*-287899613533361,
     845127728374907+I*-287899613509855,
     785913540393409+I*-226502239332491,
     785913470657815+I*-226502170968099,
     785913470657733+I*-226502170968023,
     785679972185925+I*-226305674858329,
     785679971912401+I*-226305674640947,
     11529155761195+I*330637065347061,
     11528251940877+I*330637677884185,
     11528251939823+I*330637677884857,
     8566776530547+I*332332138147875,
     8566773087019+I*332332139992029,
     8566773087015+I*332332139992031,
     8555605989269+I*332337070259759,
     8555605976331+I*332337070265043,
     -27409544051741+I*345053182339689,
     -27409585618857+I*345053195738327,
     -27409585618905+I*345053195738341,
     -27542284229567+I*345085094487857,
     -27542284382453+I*345085094520105,
     -448591158840537+I*413032027285067,
     -448591642876089+I*413032091535853,
     -448591642876645+I*413032091535911,
     -450115125092959+I*413117580559263,
     -450115126839201+I*413117580608219,
     -450115126839204+I*413117580608219,
     -450120579731848+I*413117319295761,
     -450120579738080+I*413117319295287,
     -467049862164238+I*411010734119531,
     -467049881470812+I*411010731165563,
     -467049881470834+I*411010731165559,
     -467109340665002+I*410996950943233,
     -467109340732606+I*410996950925555,
     -649118277068832+I*353965949139923,
     -649118483533210+I*353965878033127,
     -649118483533444+I*353965878033039,
     -649744974224640+I*353695377199117,
     -649744974932928+I*353695376869451,
     -649747103827494+I*353694181765939,
     -649747103829892+I*353694181764503,
     -656097499765204+I*349465691208969,
     -656097506897040+I*349465686166067,
     -656097506897048+I*349465686166061,
     -656118534530250+I*349448282238297,
     -656118534553756+I*349448282217705,
     -717515908731120+I*290234094236207,
     -717515977095512+I*290234024500613,
     -717515977095588+I*290234024500531,
     -717712473205282+I*290000526028723,
     -717712473422664+I*290000525755199,
     -1274655213410672+I*-484150290396007,
     -1274655825947796+I*-484151194216325,
     -1274655825948468+I*-484151194217379,
     -1276350286211486+I*-487112669626655,
     -1276350288055640+I*-487112673070183,
     -1276350288055642+I*-487112673070187,
     -1276355218323370+I*-487123840167933,
     -1276355218328654+I*-487123840180871,
     -1289071330403300+I*-523088990208943,
     -1289071343801938+I*-523089031776059,
     -1289071343801952+I*-523089031776107,
     -1289103242551468+I*-523221730386769,
     -1289103242583716+I*-523221730539655,
     -1357050175348678+I*-944270604997739,
     -1357050239599464+I*-944271089033291,
     -1357050239599522+I*-944271089033847,
     -1357135728622874+I*-945794571250161,
     -1357135728671830+I*-945794572996403,
     -1357135728671830+I*-945794572996406,
     -1357135467359372+I*-945800025889050,
     -1357135467358898+I*-945800025895282,
     -1355028882183142+I*-962729308321440,
     -1355028879229174+I*-962729327628014,
     -1355028879229170+I*-962729327628036,
     -1355015099006844+I*-962788786822204,
     -1355015098989166+I*-962788786889808,
     -1297984097203534+I*-1144797723226034,
     -1297984026096738+I*-1144797929690412,
     -1297984026096650+I*-1144797929690646,
     -1297713525262728+I*-1145424420381842,
     -1297713524933062+I*-1145424421090130,
     -1297712329829550+I*-1145426549984696,
     -1297712329828114+I*-1145426549987094,
     -1293483839272580+I*-1151776945922406,
     -1293483834229678+I*-1151776953054242,
     -1293483834229672+I*-1151776953054250,
     -1293466430301908+I*-1151797980687452,
     -1293466430281316+I*-1151797980710958,
     -1234252242299818+I*-1213195354888322,
     -1234252172564224+I*-1213195423252714,
     -1234252172564142+I*-1213195423252790,
     -1234018674092334+I*-1213391919362484,
     -1234018673818810+I*-1213391919579866,
     -459867857667604+I*-1770334659567874,
     -459866953847286+I*-1770335272104998,
     -459866953846232+I*-1770335272105670,
     -456905478436956+I*-1772029732368688,
     -456905474993428+I*-1772029734212842,
     -456905474993424+I*-1772029734212844,
     -456894307895678+I*-1772034664480572,
     -456894307882740+I*-1772034664485856,
     -420929157854668+I*-1784750776560502,
     -420929116287552+I*-1784750789959140,
     -420929116287504+I*-1784750789959154,
     -420796417676842+I*-1784782688708670,
     -420796417523956+I*-1784782688740918,
     252456934128+I*-1852729621505880,
     252940969680+I*-1852729685756666,
     252940970236+I*-1852729685756724,
     1776423186550+I*-1852815174780076 ],
   /* k=44 */ [ 993135317247196+I*-3740851470956254,
     993135317247199+I*-3740851470956254,
     993140770139843+I*-3740851209643796,
     993140770146075+I*-3740851209643322,
     1010070052572233+I*-3738744624467566,
     1010070071878807+I*-3738744621513598,
     1010070071878829+I*-3738744621513594,
     1010129531072997+I*-3738730841291268,
     1010129531140601+I*-3738730841273590,
     1192138467476827+I*-3681699839487958,
     1192138673941205+I*-3681699768381162,
     1192138673941439+I*-3681699768381074,
     1192765164632635+I*-3681429267547152,
     1192765165340923+I*-3681429267217486,
     3080801461468145+I*-2690070374903082,
     3080803590362711+I*-2690069179799570,
     3080803590365109+I*-2690069179798134,
     3087153986300421+I*-2685840689242600,
     3087153993432257+I*-2685840684199698,
     3087153993432265+I*-2685840684199692,
     3087175021065467+I*-2685823280271928,
     3087175021088973+I*-2685823280251336,
     3148572395266337+I*-2626609092269838,
     3148572463630729+I*-2626609022534244,
     3148572463630805+I*-2626609022534162,
     3148768959740499+I*-2626375524062354,
     3148768959957881+I*-2626375523788830,
     3705711699945889+I*-1852224707637624,
     3705712312483013+I*-1852223803817306,
     3705712312483685+I*-1852223803816252,
     3707406772746703+I*-1849262328406976,
     3707406774590857+I*-1849262324963448,
     3707406774590859+I*-1849262324963444,
     3707411704858587+I*-1849251157865698,
     3707411704863871+I*-1849251157852760,
     3720127816938517+I*-1813286007824688,
     3720127830337155+I*-1813285966257572,
     3720127830337169+I*-1813285966257524,
     3720159729086685+I*-1813153267646862,
     3720159729118933+I*-1813153267493976,
     3788106661883895+I*-1392104393035892,
     3788106726134681+I*-1392103909000340,
     3788106726134739+I*-1392103908999784,
     3788192215158091+I*-1390580426783470,
     3788192215207047+I*-1390580425037228,
     3788192215207047+I*-1390580425037225,
     3788191953894589+I*-1390574972144581,
     3788191953894115+I*-1390574972138349,
     3786085368718359+I*-1373645689712191,
     3786085365764391+I*-1373645670405617,
     3786085365764387+I*-1373645670405595,
     3786071585542061+I*-1373586211211427,
     3786071585524383+I*-1373586211143823,
     3729040583738751+I*-1191577274807597,
     3729040512631955+I*-1191577068343219,
     3729040512631867+I*-1191577068342985,
     3728770011797945+I*-1190950577651789,
     3728770011468279+I*-1190950576943501,
     2737411119153875+I*697085719183721,
     2737409924050363+I*697087848078287,
     2737409924048927+I*697087848080685,
     2733181433493393+I*703438244015997,
     2733181428450491+I*703438251147833,
     2733181428450485+I*703438251147841,
     2733164024522721+I*703459278781043,
     2733164024502129+I*703459278804549,
     2673949836520631+I*764856652981913,
     2673949766785037+I*764856721346305,
     2673949766784955+I*764856721346381,
     2673716268313147+I*765053217456075,
     2673716268039623+I*765053217673457,
     1899565451888417+I*1321995957661465,
     1899564548068099+I*1321996570198589,
     1899564548067045+I*1321996570199261,
     1896603072657769+I*1323691030462279,
     1896603069214241+I*1323691032306433,
     1896603069214237+I*1323691032306435,
     1896591902116491+I*1323695962574163,
     1896591902103553+I*1323695962579447,
     1860626752075481+I*1336412074654093,
     1860626710508365+I*1336412088052731,
     1860626710508317+I*1336412088052745,
     1860494011897655+I*1336443986802261,
     1860494011744769+I*1336443986834509,
     1439445137286685+I*1404390919599471,
     1439444653251133+I*1404390983850257,
     1439444653250577+I*1404390983850315,
     1437921171034263+I*1404476472873667,
     1437921169288021+I*1404476472922623,
     1437921169288018+I*1404476472922623,
     1437915716395374+I*1404476211610165,
     1437915716389142+I*1404476211609691,
     1420986433962984+I*1402369626433935,
     1420986414656410+I*1402369623479967,
     1420986414656388+I*1402369623479963,
     1420926955462220+I*1402355843257637,
     1420926955394616+I*1402355843239959,
     1238918019058390+I*1345324841454327,
     1238917812594012+I*1345324770347531,
     1238917812593778+I*1345324770347443,
     1238291321902582+I*1345054269513521,
     1238291321194294+I*1345054269183855,
     -649744974932928+I*353695376869451,
     -649747103827494+I*353694181765939,
     -649747103829892+I*353694181764503,
     -656097499765204+I*349465691208969,
     -656097506897040+I*349465686166067,
     -656097506897048+I*349465686166061,
     -656118534530250+I*349448282238297,
     -656118534553756+I*349448282217705,
     -717515908731120+I*290234094236207,
     -717515977095512+I*290234024500613,
     -717515977095588+I*290234024500531,
     -717712473205282+I*290000526028723,
     -717712473422664+I*290000525755199,
     -1274655213410672+I*-484150290396007,
     -1274655825947796+I*-484151194216325,
     -1274655825948468+I*-484151194217379,
     -1276350286211486+I*-487112669626655,
     -1276350288055640+I*-487112673070183,
     -1276350288055642+I*-487112673070187,
     -1276355218323370+I*-487123840167933,
     -1276355218328654+I*-487123840180871,
     -1289071330403300+I*-523088990208943,
     -1289071343801938+I*-523089031776059,
     -1289071343801952+I*-523089031776107,
     -1289103242551468+I*-523221730386769,
     -1289103242583716+I*-523221730539655,
     -1357050175348678+I*-944270604997739,
     -1357050239599464+I*-944271089033291,
     -1357050239599522+I*-944271089033847,
     -1357135728622874+I*-945794571250161,
     -1357135728671830+I*-945794572996403,
     -1357135728671830+I*-945794572996406,
     -1357135467359372+I*-945800025889050,
     -1357135467358898+I*-945800025895282,
     -1355028882183142+I*-962729308321440,
     -1355028879229174+I*-962729327628014,
     -1355028879229170+I*-962729327628036,
     -1355015099006844+I*-962788786822204,
     -1355015098989166+I*-962788786889808,
     -1297984097203534+I*-1144797723226034,
     -1297984026096738+I*-1144797929690412,
     -1297984026096650+I*-1144797929690646,
     -1297713525262728+I*-1145424420381842,
     -1297713524933062+I*-1145424421090130,
     -306354632618658+I*-3033460717217352,
     -306353437515146+I*-3033462846111918,
     -306353437513710+I*-3033462846114316,
     -302124946958176+I*-3039813242049628,
     -302124941915274+I*-3039813249181464,
     -302124941915268+I*-3039813249181472,
     -302107537987504+I*-3039834276814674,
     -302107537966912+I*-3039834276838180,
     -242893349985414+I*-3101231651015544,
     -242893280249820+I*-3101231719379936,
     -242893280249738+I*-3101231719380012,
     -242659781777930+I*-3101428215489706,
     -242659781504406+I*-3101428215707088,
     531491034646800+I*-3658370955695096,
     531491938467118+I*-3658371568232220,
     531491938468172+I*-3658371568232892,
     534453413877448+I*-3660066028495910,
     534453417320976+I*-3660066030340064,
     534453417320980+I*-3660066030340066,
     534464584418726+I*-3660070960607794,
     534464584431664+I*-3660070960613078,
     570429734459736+I*-3672787072687724,
     570429776026852+I*-3672787086086362,
     570429776026900+I*-3672787086086376,
     570562474637562+I*-3672818984835892,
     570562474790448+I*-3672818984868140,
     991611349248532+I*-3740765917633102,
     991611833284084+I*-3740765981883888,
     991611833284640+I*-3740765981883946,
     993135315500954+I*-3740851470907298 ],
   /* k=45 */ [ 993135317247196+I*-3740851470956254,
     993135317247199+I*-3740851470956254,
     5760566801816047+I*-3646169982454668,
     5760572254708691+I*-3646169721142210,
     5760572254714923+I*-3646169721141736,
     5777501537141081+I*-3644063135965980,
     5777501556447655+I*-3644063133012012,
     5777501556447677+I*-3644063133012008,
     5777561015641845+I*-3644049352789682,
     5777561015709449+I*-3644049352772004,
     5959569952045675+I*-3587018350986372,
     5959570158510053+I*-3587018279879576,
     5959570158510287+I*-3587018279879488,
     5960196649201483+I*-3586747779045566,
     5960196649909771+I*-3586747778715900,
     7848232946036993+I*-2595388886401496,
     7848235074931559+I*-2595387691297984,
     7848235074933957+I*-2595387691296548,
     7854585470869269+I*-2591159200741014,
     7854585478001105+I*-2591159195698112,
     7854585478001113+I*-2591159195698106,
     7854606505634315+I*-2591141791770342,
     7854606505657821+I*-2591141791749750,
     7916003879835185+I*-2531927603768252,
     7916003948199577+I*-2531927534032658,
     7916003948199653+I*-2531927534032576,
     7916200444309347+I*-2531694035560768,
     7916200444526729+I*-2531694035287244,
     8473143184514737+I*-1757543219136038,
     8473143797051861+I*-1757542315315720,
     8473143797052533+I*-1757542315314666,
     8474838257315551+I*-1754580839905390,
     8474838259159705+I*-1754580836461862,
     8474838259159707+I*-1754580836461858,
     8474843189427435+I*-1754569669364112,
     8474843189432719+I*-1754569669351174,
     8487559301507365+I*-1718604519323102,
     8487559314906003+I*-1718604477755986,
     8487559314906017+I*-1718604477755938,
     8487591213655533+I*-1718471779145276,
     8487591213687781+I*-1718471778992390,
     8555538146452743+I*-1297422904534306,
     8555538210703529+I*-1297422420498754,
     8555538210703587+I*-1297422420498198,
     8555623699726939+I*-1295898938281884,
     8555623699775895+I*-1295898936535642,
     8555623699775895+I*-1295898936535639,
     8460942211274309+I*3471532548033209,
     8460941949961851+I*3471538000925853,
     8460941949961377+I*3471538000932085,
     8458835364785621+I*3488467283358243,
     8458835361831653+I*3488467302664817,
     8458835361831649+I*3488467302664839,
     8458821581609323+I*3488526761859007,
     8458821581591645+I*3488526761926611,
     8401790579806013+I*3670535698262837,
     8401790508699217+I*3670535904727215,
     8401790508699129+I*3670535904727449,
     8401520007865207+I*3671162395418645,
     8401520007535541+I*3671162396126933,
     7410161115221137+I*5559198692254155,
     7410159920117625+I*5559200821148721,
     7410159920116189+I*5559200821151119,
     7405931429560655+I*5565551217086431,
     7405931424517753+I*5565551224218267,
     7405931424517747+I*5565551224218275,
     7405914020589983+I*5565572251851477,
     7405914020569391+I*5565572251874983,
     7346699832587893+I*5626969626052347,
     7346699762852299+I*5626969694416739,
     7346699762852217+I*5626969694416815,
     7346466264380409+I*5627166190526509,
     7346466264106885+I*5627166190743891,
     6572315447955679+I*6184108930731899,
     6572314544135361+I*6184109543269023,
     6572314544134307+I*6184109543269695,
     6569353068725031+I*6185804003532713,
     6569353065281503+I*6185804005376867,
     6569353065281499+I*6185804005376869,
     6569341898183753+I*6185808935644597,
     6569341898170815+I*6185808935649881,
     6533376748142743+I*6198525047724527,
     6533376706575627+I*6198525061123165,
     6533376706575579+I*6198525061123179,
     6533244007964917+I*6198556959872695,
     6533244007812031+I*6198556959904943,
     6112195133353947+I*6266503892669905,
     6112194649318395+I*6266503956920691,
     6112194649317839+I*6266503956920749,
     6110671167101525+I*6266589445944101,
     6110671165355283+I*6266589445993057,
     6110671165355280+I*6266589445993057,
     1343239680786432+I*6171907957491471,
     1343234227893788+I*6171907696179013,
     1343234227887556+I*6171907696178539,
     1326304945461398+I*6169801111002783,
     1326304926154824+I*6169801108048815,
     1326304926154802+I*6169801108048811,
     1326245466960634+I*6169787327826485,
     1326245466893030+I*6169787327808807,
     1144236530556804+I*6112756326023175,
     1144236324092426+I*6112756254916379,
     1144236324092192+I*6112756254916291,
     1143609833400996+I*6112485754082369,
     1143609832692708+I*6112485753752703,
     -744426463434514+I*5121126861438299,
     -744428592329080+I*5121125666334787,
     -744428592331478+I*5121125666333351,
     -750778988266790+I*5116897175777817,
     -750778995398626+I*5116897170734915,
     -750778995398634+I*5116897170734909,
     -750800023031836+I*5116879766807145,
     -750800023055342+I*5116879766786553,
     -812197397232706+I*5057665578805055,
     -812197465597098+I*5057665509069461,
     -812197465597174+I*5057665509069379,
     -812393961706868+I*5057432010597571,
     -812393961924250+I*5057432010324047,
     -1369336701912258+I*4283281194172841,
     -1369337314449382+I*4283280290352523,
     -1369337314450054+I*4283280290351469,
     -1371031774713072+I*4280318814942193,
     -1371031776557226+I*4280318811498665,
     -1371031776557228+I*4280318811498661,
     -1371036706824956+I*4280307644400915,
     -1371036706830240+I*4280307644387977,
     -1383752818904886+I*4244342494359905,
     -1383752832303524+I*4244342452792789,
     -1383752832303538+I*4244342452792741,
     -1383784731053054+I*4244209754182079,
     -1383784731085302+I*4244209754029193,
     -1451731663850264+I*3823160879571109,
     -1451731728101050+I*3823160395535557,
     -1451731728101108+I*3823160395535001,
     -1451817217124460+I*3821636913318687,
     -1451817217173416+I*3821636911572445,
     -1451817217173416+I*3821636911572442,
     -1357135728671830+I*-945794572996406,
     -1357135467359372+I*-945800025889050,
     -1357135467358898+I*-945800025895282,
     -1355028882183142+I*-962729308321440,
     -1355028879229174+I*-962729327628014,
     -1355028879229170+I*-962729327628036,
     -1355015099006844+I*-962788786822204,
     -1355015098989166+I*-962788786889808,
     -1297984097203534+I*-1144797723226034,
     -1297984026096738+I*-1144797929690412,
     -1297984026096650+I*-1144797929690646,
     -1297713525262728+I*-1145424420381842,
     -1297713524933062+I*-1145424421090130,
     -306354632618658+I*-3033460717217352,
     -306353437515146+I*-3033462846111918,
     -306353437513710+I*-3033462846114316,
     -302124946958176+I*-3039813242049628,
     -302124941915274+I*-3039813249181464,
     -302124941915268+I*-3039813249181472,
     -302107537987504+I*-3039834276814674,
     -302107537966912+I*-3039834276838180,
     -242893349985414+I*-3101231651015544,
     -242893280249820+I*-3101231719379936,
     -242893280249738+I*-3101231719380012,
     -242659781777930+I*-3101428215489706,
     -242659781504406+I*-3101428215707088,
     531491034646800+I*-3658370955695096,
     531491938467118+I*-3658371568232220,
     531491938468172+I*-3658371568232892,
     534453413877448+I*-3660066028495910,
     534453417320976+I*-3660066030340064,
     534453417320980+I*-3660066030340066,
     534464584418726+I*-3660070960607794,
     534464584431664+I*-3660070960613078,
     570429734459736+I*-3672787072687724,
     570429776026852+I*-3672787086086362,
     570429776026900+I*-3672787086086376,
     570562474637562+I*-3672818984835892,
     570562474790448+I*-3672818984868140,
     991611349248532+I*-3740765917633102,
     991611833284084+I*-3740765981883888,
     991611833284640+I*-3740765981883946,
     993135315500954+I*-3740851470907298 ],
   /* k=46 */ [ 993135317247196+I*-3740851470956254,
     993135317247199+I*-3740851470956254,
     5760566801816047+I*-3646169982454668,
     5760572254708691+I*-3646169721142210,
     5760572254714923+I*-3646169721141736,
     5777501537141081+I*-3644063135965980,
     5777501556447655+I*-3644063133012012,
     5777501556447677+I*-3644063133012008,
     5777561015641845+I*-3644049352789682,
     5777561015709449+I*-3644049352772004,
     5959569952045675+I*-3587018350986372,
     5959570158510053+I*-3587018279879576,
     5959570158510287+I*-3587018279879488,
     5960196649201483+I*-3586747779045566,
     5960196649909771+I*-3586747778715900,
     7848232946036993+I*-2595388886401496,
     7848235074931559+I*-2595387691297984,
     7848235074933957+I*-2595387691296548,
     7854585470869269+I*-2591159200741014,
     7854585478001105+I*-2591159195698112,
     7854585478001113+I*-2591159195698106,
     7854606505634315+I*-2591141791770342,
     7854606505657821+I*-2591141791749750,
     7916003879835185+I*-2531927603768252,
     7916003948199577+I*-2531927534032658,
     7916003948199653+I*-2531927534032576,
     7916200444309347+I*-2531694035560768,
     7916200444526729+I*-2531694035287244,
     8473143184514737+I*-1757543219136038,
     8473143797051861+I*-1757542315315720,
     8473143797052533+I*-1757542315314666,
     8474838257315551+I*-1754580839905390,
     8474838259159705+I*-1754580836461862,
     8474838259159707+I*-1754580836461858,
     13052906766725383+I*7874963621177424,
     13052911696993111+I*7874974788275170,
     13052911696998395+I*7874974788288108,
     13065627809073041+I*7910939938316180,
     13065627822471679+I*7910939979883296,
     13065627822471693+I*7910939979883344,
     13065659721221209+I*7911072678494006,
     13065659721253457+I*7911072678646892,
     13133606654018419+I*8332121553104976,
     13133606718269205+I*8332122037140528,
     13133606718269263+I*8332122037141084,
     13133692207292615+I*8333645519357398,
     13133692207341571+I*8333645521103640,
     13133692207341571+I*8333645521103643,
     13039010718839985+I*13101077005672491,
     13039010457527527+I*13101082458565135,
     13039010457527053+I*13101082458571367,
     13036903872351297+I*13118011740997525,
     13036903869397329+I*13118011760304099,
     13036903869397325+I*13118011760304121,
     13036890089174999+I*13118071219498289,
     13036890089157321+I*13118071219565893,
     12979859087371689+I*13300080155902119,
     12979859016264893+I*13300080362366497,
     12979859016264805+I*13300080362366731,
     12979588515430883+I*13300706853057927,
     12979588515101217+I*13300706853766215,
     11988229622786813+I*15188743149893437,
     11988228427683301+I*15188745278788003,
     11988228427681865+I*15188745278790401,
     11983999937126331+I*15195095674725713,
     11983999932083429+I*15195095681857549,
     11983999932083423+I*15195095681857557,
     11983982528155659+I*15195116709490759,
     11983982528135067+I*15195116709514265,
     11924768340153569+I*15256514083691629,
     11924768270417975+I*15256514152056021,
     11924768270417893+I*15256514152056097,
     11924534771946085+I*15256710648165791,
     11924534771672561+I*15256710648383173,
     11150383955521355+I*15813653388371181,
     11150383051701037+I*15813654000908305,
     11150383051699983+I*15813654000908977,
     11147421576290707+I*15815348461171995,
     11147421572847179+I*15815348463016149,
     11147421572847175+I*15815348463016151,
     1517877115207893+I*20393416970581827,
     1517865948110147+I*20393421900849555,
     1517865948097209+I*20393421900854839,
     1481900798069137+I*20406138012929485,
     1481900756502021+I*20406138026328123,
     1481900756501973+I*20406138026328137,
     1481768057891311+I*20406169925077653,
     1481768057738425+I*20406169925109901,
     1060719183280341+I*20474116857874863,
     1060718699244789+I*20474116922125649,
     1060718699244233+I*20474116922125707,
     1059195217027919+I*20474202411149059,
     1059195215281677+I*20474202411198015,
     1059195215281674+I*20474202411198015,
     -3708236269287174+I*20379520922696429,
     -3708241722179818+I*20379520661383971,
     -3708241722186050+I*20379520661383497,
     -3725171004612208+I*20377414076207741,
     -3725171023918782+I*20377414073253773,
     -3725171023918804+I*20377414073253769,
     -3725230483112972+I*20377400293031443,
     -3725230483180576+I*20377400293013765,
     -3907239419516802+I*20320369291228133,
     -3907239625981180+I*20320369220121337,
     -3907239625981414+I*20320369220121249,
     -3907866116672610+I*20320098719287327,
     -3907866117380898+I*20320098718957661,
     -5795902413508120+I*19328739826643257,
     -5795904542402686+I*19328738631539745,
     -5795904542405084+I*19328738631538309,
     -5802254938340396+I*19324510140982775,
     -5802254945472232+I*19324510135939873,
     -5802254945472240+I*19324510135939867,
     -5802275973105442+I*19324492732012103,
     -5802275973128948+I*19324492731991511,
     -5863673347306312+I*19265278544010013,
     -5863673415670704+I*19265278474274419,
     -5863673415670780+I*19265278474274337,
     -5863869911780474+I*19265044975802529,
     -5863869911997856+I*19265044975529005,
     -6420812651985864+I*18490894159377799,
     -6420813264522988+I*18490893255557481,
     -6420813264523660+I*18490893255556427,
     -6422507724786678+I*18487931780147151,
     -6422507726630832+I*18487931776703623,
     -6422507726630834+I*18487931776703619,
     -11000576234196510+I*8858387319064337,
     -11000581164464238+I*8858376151966591,
     -11000581164469522+I*8858376151953653,
     -11013297276544168+I*8822411001925581,
     -11013297289942806+I*8822410960358465,
     -11013297289942820+I*8822410960358417,
     -11013329188692336+I*8822278261747755,
     -11013329188724584+I*8822278261594869,
     -11081276121489546+I*8401229387136785,
     -11081276185740332+I*8401228903101233,
     -11081276185740390+I*8401228903100677,
     -11081361674763742+I*8399705420884363,
     -11081361674812698+I*8399705419138121,
     -11081361674812698+I*8399705419138118,
     -10986680186311112+I*3632273934569270,
     -10986679924998654+I*3632268481676626,
     -10986679924998180+I*3632268481670394,
     -10984573339822424+I*3615339199244236,
     -10984573336868456+I*3615339179937662,
     -10984573336868452+I*3615339179937640,
     -10984559556646126+I*3615279720743472,
     -10984559556628448+I*3615279720675868,
     -10927528554842816+I*3433270784339642,
     -10927528483736020+I*3433270577875264,
     -10927528483735932+I*3433270577875030,
     -10927257982902010+I*3432644087183834,
     -10927257982572344+I*3432644086475546,
     -9935899090257940+I*1544607790348324,
     -9935897895154428+I*1544605661453758,
     -9935897895152992+I*1544605661451360,
     -9931669404597458+I*1538255265516048,
     -9931669399554556+I*1538255258384212,
     -9931669399554550+I*1538255258384204,
     -9931651995626786+I*1538234230751002,
     -9931651995606194+I*1538234230727496,
     -9872437807624696+I*1476836856550132,
     -9872437737889102+I*1476836788185740,
     -9872437737889020+I*1476836788185664,
     -9872204239417212+I*1476640292075970,
     -9872204239143688+I*1476640291858588,
     -9098053422992482+I*919697551870580,
     -9098052519172164+I*919696939333456,
     -9098052519171110+I*919696939332784,
     -9095091043761834+I*918002479069766,
     -9095091040318306+I*918002477225612,
     -9095091040318302+I*918002477225610,
     534453417320980+I*-3660066030340066,
     534464584418726+I*-3660070960607794,
     534464584431664+I*-3660070960613078,
     570429734459736+I*-3672787072687724,
     570429776026852+I*-3672787086086362,
     570429776026900+I*-3672787086086376,
     570562474637562+I*-3672818984835892,
     570562474790448+I*-3672818984868140,
     991611349248532+I*-3740765917633102,
     991611833284084+I*-3740765981883888,
     991611833284640+I*-3740765981883946,
     993135315500954+I*-3740851470907298 ],
   /* k=47 */ [ -17792546155523438+I*-18421871878669142,
     -17792546155523435+I*-18421871878669142,
     -13025114670954587+I*-18327190390167556,
     -13025109218061943+I*-18327190128855098,
     -13025109218055711+I*-18327190128854624,
     -13008179935629553+I*-18325083543678868,
     -13008179916322979+I*-18325083540724900,
     -13008179916322957+I*-18325083540724896,
     -13008120457128789+I*-18325069760502570,
     -13008120457061185+I*-18325069760484892,
     -12826111520724959+I*-18268038758699260,
     -12826111314260581+I*-18268038687592464,
     -12826111314260347+I*-18268038687592376,
     -12825484823569151+I*-18267768186758454,
     -12825484822860863+I*-18267768186428788,
     -10937448526733641+I*-17276409294114384,
     -10937446397839075+I*-17276408099010872,
     -10937446397836677+I*-17276408099009436,
     -10931096001901365+I*-17272179608453902,
     -10931095994769529+I*-17272179603411000,
     -10931095994769521+I*-17272179603410994,
     7854585478001113+I*-2591159195698106,
     7854606505634315+I*-2591141791770342,
     7854606505657821+I*-2591141791749750,
     7916003879835185+I*-2531927603768252,
     7916003948199577+I*-2531927534032658,
     7916003948199653+I*-2531927534032576,
     7916200444309347+I*-2531694035560768,
     7916200444526729+I*-2531694035287244,
     8473143184514737+I*-1757543219136038,
     8473143797051861+I*-1757542315315720,
     8473143797052533+I*-1757542315314666,
     8474838257315551+I*-1754580839905390,
     8474838259159705+I*-1754580836461862,
     8474838259159707+I*-1754580836461858,
     13052906766725383+I*7874963621177424,
     13052911696993111+I*7874974788275170,
     13052911696998395+I*7874974788288108,
     13065627809073041+I*7910939938316180,
     13065627822471679+I*7910939979883296,
     13065627822471693+I*7910939979883344,
     13065659721221209+I*7911072678494006,
     13065659721253457+I*7911072678646892,
     13133606654018419+I*8332121553104976,
     13133606718269205+I*8332122037140528,
     13133606718269263+I*8332122037141084,
     13133692207292615+I*8333645519357398,
     13133692207341571+I*8333645521103640,
     13133692207341571+I*8333645521103643,
     13039010718839985+I*13101077005672491,
     13039010457527527+I*13101082458565135,
     13039010457527053+I*13101082458571367,
     13036903872351297+I*13118011740997525,
     13036903869397329+I*13118011760304099,
     13036903869397325+I*13118011760304121,
     13036890089174999+I*13118071219498289,
     13036890089157321+I*13118071219565893,
     12979859087371689+I*13300080155902119,
     12979859016264893+I*13300080362366497,
     12979859016264805+I*13300080362366731,
     12979588515430883+I*13300706853057927,
     12979588515101217+I*13300706853766215,
     11988229622786813+I*15188743149893437,
     11988228427683301+I*15188745278788003,
     11988228427681865+I*15188745278790401,
     11983999937126331+I*15195095674725713,
     11983999932083429+I*15195095681857549,
     11983999932083423+I*15195095681857557,
     -2697020475629465+I*33980777154628191,
     -2697037879557229+I*33980798182261393,
     -2697037879577821+I*33980798182284899,
     -2756252067559319+I*34042195556462263,
     -2756252137294913+I*34042195624826655,
     -2756252137294995+I*34042195624826731,
     -2756485635766803+I*34042392120936425,
     -2756485636040327+I*34042392121153807,
     -3530636452191533+I*34599334861141815,
     -3530637356011851+I*34599335473678939,
     -3530637356012905+I*34599335473679611,
     -3533598831422181+I*34601029933942629,
     -3533598834865709+I*34601029935786783,
     -3533598834865713+I*34601029935786785,
     -13163143292504995+I*39179098443352461,
     -13163154459602741+I*39179103373620189,
     -13163154459615679+I*39179103373625473,
     -13199119609643751+I*39191819485700119,
     -13199119651210867+I*39191819499098757,
     -13199119651210915+I*39191819499098771,
     -13199252349821577+I*39191851397848287,
     -13199252349974463+I*39191851397880535,
     -13620301224432547+I*39259798330645497,
     -13620301708468099+I*39259798394896283,
     -13620301708468655+I*39259798394896341,
     -13621825190684969+I*39259883883919693,
     -13621825192431211+I*39259883883968649,
     -13621825192431214+I*39259883883968649,
     -18389256677000062+I*39165202395467063,
     -18389262129892706+I*39165202134154605,
     -18389262129898938+I*39165202134154131,
     -18406191412325096+I*39163095548978375,
     -18406191431631670+I*39163095546024407,
     -18406191431631692+I*39163095546024403,
     -18406250890825860+I*39163081765802077,
     -18406250890893464+I*39163081765784399,
     -18588259827229690+I*39106050763998767,
     -18588260033694068+I*39106050692891971,
     -18588260033694302+I*39106050692891883,
     -18588886524385498+I*39105780192057961,
     -18588886525093786+I*39105780191728295,
     -20476922821221008+I*38114421299413891,
     -20476924950115574+I*38114420104310379,
     -20476924950117972+I*38114420104308943,
     -20483275346053284+I*38110191613753409,
     -20483275353185120+I*38110191608710507,
     -20483275353185128+I*38110191608710501,
     -39268956825955762+I*23429171200997613,
     -39268977853588964+I*23429153797069849,
     -39268977853612470+I*23429153797049257,
     -39330375227789834+I*23369939609067759,
     -39330375296154226+I*23369939539332165,
     -39330375296154302+I*23369939539332083,
     -39330571792263996+I*23369706040860275,
     -39330571792481378+I*23369706040586751,
     -39887514532469386+I*22595555224435545,
     -39887515145006510+I*22595554320615227,
     -39887515145007182+I*22595554320614173,
     -39889209605270200+I*22592592845204897,
     -39889209607114354+I*22592592841761369,
     -39889209607114356+I*22592592841761365,
     -44467278114680032+I*12963048384122083,
     -44467283044947760+I*12963037217024337,
     -44467283044953044+I*12963037217011399,
     -44479999157027690+I*12927072066983327,
     -44479999170426328+I*12927072025416211,
     -44479999170426342+I*12927072025416163,
     -44480031069175858+I*12926939326805501,
     -44480031069208106+I*12926939326652615,
     -44547978001973068+I*12505890452194531,
     -44547978066223854+I*12505889968158979,
     -44547978066223912+I*12505889968158423,
     -44548063555247264+I*12504366485942109,
     -44548063555296220+I*12504366484195867,
     -44548063555296220+I*12504366484195864,
     -44453382066794634+I*7736934999627016,
     -44453381805482176+I*7736929546734372,
     -44453381805481702+I*7736929546728140,
     -44451275220305946+I*7720000264301982,
     -44451275217351978+I*7720000244995408,
     -44451275217351974+I*7720000244995386,
     -44451261437129648+I*7719940785801218,
     -44451261437111970+I*7719940785733614,
     -44394230435326338+I*7537931849397388,
     -44394230364219542+I*7537931642933010,
     -44394230364219454+I*7537931642932776,
     -44393959863385532+I*7537305152241580,
     -44393959863055866+I*7537305151533292,
     -43402600970741462+I*5649268855406070,
     -43402599775637950+I*5649266726511504,
     -43402599775636514+I*5649266726509106,
     -43398371285080980+I*5642916330573794,
     -43398371280038078+I*5642916323441958,
     -43398371280038072+I*5642916323441950,
     -28717350872325184+I*-13142765149328684,
     -28717333468397420+I*-13142786176961886,
     -28717333468376828+I*-13142786176985392,
     -28658119280395330+I*-13204183551162756,
     -28658119210659736+I*-13204183619527148,
     -28658119210659654+I*-13204183619527224,
     -28657885712187846+I*-13204380115636918,
     -28657885711914322+I*-13204380115854300,
     -27883734895763116+I*-13761322855842308,
     -27883733991942798+I*-13761323468379432,
     -27883733991941744+I*-13761323468380104,
     -27880772516532468+I*-13763017928643122,
     -27880772513088940+I*-13763017930487276,
     -27880772513088936+I*-13763017930487278,
     -18251228055449654+I*-18341086438052954,
     -18251216888351908+I*-18341091368320682,
     -18251216888338970+I*-18341091368325966,
     -18215251738310898+I*-18353807480400612,
     -18215251696743782+I*-18353807493799250,
     -18215251696743734+I*-18353807493799264,
     -18215118998133072+I*-18353839392548780,
     -18215118997980186+I*-18353839392581028,
     -17794070123522102+I*-18421786325345990,
     -17794069639486550+I*-18421786389596776,
     -17794069639485994+I*-18421786389596834,
     -17792546157269680+I*-18421871878620186 ],
   /* k=48 */ [ -59468570166122452+I*-81250614574578440,
     -59468570166122449+I*-81250614574578440,
     -54701138681553601+I*-81155933086076854,
     -54701133228660957+I*-81155932824764396,
     -54701133228654725+I*-81155932824763922,
     -54684203946228567+I*-81153826239588166,
     -54684203926921993+I*-81153826236634198,
     -54684203926921971+I*-81153826236634194,
     -2431820573667815+I*-70577466893979052,
     -2431761114473647+I*-70577453113756726,
     -2431761114406043+I*-70577453113739048,
     -2249752178069817+I*-70520422111953416,
     -2249751971605439+I*-70520422040846620,
     -2249751971605205+I*-70520422040846532,
     -2249125480914009+I*-70520151540012610,
     -2249125480205721+I*-70520151539682944,
     -361089184078499+I*-69528792647368540,
     -361087055183933+I*-69528791452265028,
     -361087055181535+I*-69528791452263592,
     -354736659246223+I*-69524562961708058,
     -354736652114387+I*-69524562956665156,
     -354736652114379+I*-69524562956665150,
     18430944820656255+I*-54843542548952262,
     18430965848289457+I*-54843525145024498,
     18430965848312963+I*-54843525145003906,
     18492363222490327+I*-54784310957022408,
     18492363290854719+I*-54784310887286814,
     18492363290854795+I*-54784310887286732,
     18492559786964489+I*-54784077388814924,
     18492559787181871+I*-54784077388541400,
     19049502527169879+I*-54009926572390194,
     19049503139707003+I*-54009925668569876,
     19049503139707675+I*-54009925668568822,
     19051197599970693+I*-54006964193159546,
     19051197601814847+I*-54006964189716018,
     19051197601814849+I*-54006964189716014,
     23629266109380525+I*-44377419732076732,
     23629271039648253+I*-44377408564978986,
     23629271039653537+I*-44377408564966048,
     23641987151728183+I*-44341443414937976,
     23641987165126821+I*-44341443373370860,
     23641987165126835+I*-44341443373370812,
     23642019063876351+I*-44341310674760150,
     23642019063908599+I*-44341310674607264,
     23709965996673561+I*-43920261800149180,
     23709966060924347+I*-43920261316113628,
     23709966060924405+I*-43920261316113072,
     23710051549947757+I*-43918737833896758,
     23710051549996713+I*-43918737832150516,
     23710051549996713+I*-43918737832150513,
     23615370061495127+I*-39151306347581665,
     23615369800182669+I*-39151300894689021,
     23615369800182195+I*-39151300894682789,
     23613263215006439+I*-39134371612256631,
     23613263212052471+I*-39134371592950057,
     23613263212052467+I*-39134371592950035,
     13036903869397325+I*13118011760304121,
     13036890089174999+I*13118071219498289,
     13036890089157321+I*13118071219565893,
     12979859087371689+I*13300080155902119,
     12979859016264893+I*13300080362366497,
     12979859016264805+I*13300080362366731,
     12979588515430883+I*13300706853057927,
     12979588515101217+I*13300706853766215,
     11988229622786813+I*15188743149893437,
     11988228427683301+I*15188745278788003,
     11988228427681865+I*15188745278790401,
     11983999937126331+I*15195095674725713,
     11983999932083429+I*15195095681857549,
     11983999932083423+I*15195095681857557,
     -2697020475629465+I*33980777154628191,
     -2697037879557229+I*33980798182261393,
     -2697037879577821+I*33980798182284899,
     -2756252067559319+I*34042195556462263,
     -2756252137294913+I*34042195624826655,
     -2756252137294995+I*34042195624826731,
     -2756485635766803+I*34042392120936425,
     -2756485636040327+I*34042392121153807,
     -3530636452191533+I*34599334861141815,
     -3530637356011851+I*34599335473678939,
     -3530637356012905+I*34599335473679611,
     -3533598831422181+I*34601029933942629,
     -3533598834865709+I*34601029935786783,
     -3533598834865713+I*34601029935786785,
     -13163143292504995+I*39179098443352461,
     -13163154459602741+I*39179103373620189,
     -13163154459615679+I*39179103373625473,
     -13199119609643751+I*39191819485700119,
     -13199119651210867+I*39191819499098757,
     -13199119651210915+I*39191819499098771,
     -13199252349821577+I*39191851397848287,
     -13199252349974463+I*39191851397880535,
     -13620301224432547+I*39259798330645497,
     -13620301708468099+I*39259798394896283,
     -13620301708468655+I*39259798394896341,
     -13621825190684969+I*39259883883919693,
     -13621825192431211+I*39259883883968649,
     -13621825192431214+I*39259883883968649,
     -18389256677000062+I*39165202395467063,
     -18389262129892706+I*39165202134154605,
     -18389262129898938+I*39165202134154131,
     -18406191412325096+I*39163095548978375,
     -18406191431631670+I*39163095546024407,
     -18406191431631692+I*39163095546024403,
     -70658574784885848+I*28586736203369261,
     -70658634244080016+I*28586722423146935,
     -70658634244147620+I*28586722423129257,
     -70840643180483846+I*28529691421343625,
     -70840643386948224+I*28529691350236829,
     -70840643386948458+I*28529691350236741,
     -70841269877639654+I*28529420849402819,
     -70841269878347942+I*28529420849073153,
     -72729306174475164+I*27538061956758749,
     -72729308303369730+I*27538060761655237,
     -72729308303372128+I*27538060761653801,
     -72735658699307440+I*27533832271098267,
     -72735658706439276+I*27533832266055365,
     -72735658706439284+I*27533832266055359,
     -91521340179209918+I*12852811858342471,
     -91521361206843120+I*12852794454414707,
     -91521361206866626+I*12852794454394115,
     -91582758581043990+I*12793580266412617,
     -91582758649408382+I*12793580196677023,
     -91582758649408458+I*12793580196676941,
     -91582955145518152+I*12793346698205133,
     -91582955145735534+I*12793346697931609,
     -92139897885723542+I*12019195881780403,
     -92139898498260666+I*12019194977960085,
     -92139898498261338+I*12019194977959031,
     -92141592958524356+I*12016233502549755,
     -92141592960368510+I*12016233499106227,
     -92141592960368512+I*12016233499106223,
     -96719661467934188+I*2386689041466941,
     -96719666398201916+I*2386677874369195,
     -96719666398207200+I*2386677874356257,
     -96732382510281846+I*2350712724328185,
     -96732382523680484+I*2350712682761069,
     -96732382523680498+I*2350712682761021,
     -96732414422430014+I*2350579984150359,
     -96732414422462262+I*2350579983997473,
     -96800361355227224+I*1929531109539389,
     -96800361419478010+I*1929530625503837,
     -96800361419478068+I*1929530625503281,
     -96800446908501420+I*1928007143286967,
     -96800446908550376+I*1928007141540725,
     -96800446908550376+I*1928007141540722,
     -96705765420048790+I*-2839424343028126,
     -96705765158736332+I*-2839429795920770,
     -96705765158735858+I*-2839429795927002,
     -96703658573560102+I*-2856359078353160,
     -96703658570606134+I*-2856359097659734,
     -96703658570606130+I*-2856359097659756,
     -86127299227950988+I*-55108742450913912,
     -86127285447728662+I*-55108801910108080,
     -86127285447710984+I*-55108801910175684,
     -86070254445925352+I*-55290810846511910,
     -86070254374818556+I*-55290811052976288,
     -86070254374818468+I*-55290811052976522,
     -86069983873984546+I*-55291437543667718,
     -86069983873654880+I*-55291437544376006,
     -85078624981340476+I*-57179473840503228,
     -85078623786236964+I*-57179475969397794,
     -85078623786235528+I*-57179475969400192,
     -85074395295679994+I*-57185826365335504,
     -85074395290637092+I*-57185826372467340,
     -85074395290637086+I*-57185826372467348,
     -70393374882924198+I*-75971507845237982,
     -70393357478996434+I*-75971528872871184,
     -70393357478975842+I*-75971528872894690,
     -70334143290994344+I*-76032926247072054,
     -70334143221258750+I*-76032926315436446,
     -70334143221258668+I*-76032926315436522,
     -70333909722786860+I*-76033122811546216,
     -70333909722513336+I*-76033122811763598,
     -69559758906362130+I*-76590065551751606,
     -69559758002541812+I*-76590066164288730,
     -69559758002540758+I*-76590066164289402,
     -69556796527131482+I*-76591760624552420,
     -69556796523687954+I*-76591760626396574,
     -69556796523687950+I*-76591760626396576,
     -59927252066048668+I*-81169829133962252,
     -59927240898950922+I*-81169834064229980,
     -59927240898937984+I*-81169834064235264,
     -59891275748909912+I*-81182550176309910,
     -59891275707342796+I*-81182550189708548,
     -59891275707342748+I*-81182550189708562,
     -59891143008732086+I*-81182582088458078,
     -59891143008579200+I*-81182582088490326,
     -59470094134121116+I*-81250529021255288,
     -59470093650085564+I*-81250529085506074,
     -59470093650085008+I*-81250529085506132,
     -59468570167868694+I*-81250614574529484 ],
   /* k=49 */ [ 24512891215097130+I*-227431405291685766,
     24512891215097133+I*-227431405291685766,
     29280322699665981+I*-227336723803184180,
     29280328152558625+I*-227336723541871722,
     29280328152564857+I*-227336723541871248,
     29297257434991015+I*-227334616956695492,
     29297257454297589+I*-227334616953741524,
     29297257454297611+I*-227334616953741520,
     81549640807551767+I*-216758257611086378,
     81549700266745935+I*-216758243830864052,
     81549700266813539+I*-216758243830846374,
     81731709203149765+I*-216701212829060742,
     81731709409614143+I*-216701212757953946,
     81731709409614377+I*-216701212757953858,
     81732335900305573+I*-216700942257119936,
     81732335901013861+I*-216700942256790270,
     83620372197141083+I*-215709583364475866,
     83620374326035649+I*-215709582169372354,
     83620374326038047+I*-215709582169370918,
     83626724721973359+I*-215705353678815384,
     83626724729105195+I*-215705353673772482,
     83626724729105203+I*-215705353673772476,
     102412406201875837+I*-201024333266059588,
     102412427229509039+I*-201024315862131824,
     102412427229532545+I*-201024315862111232,
     102473824603709909+I*-200965101674129734,
     102473824672074301+I*-200965101604394140,
     102473824672074377+I*-200965101604394058,
     102474021168184071+I*-200964868105922250,
     102474021168401453+I*-200964868105648726,
     103030963908389461+I*-200190717289497520,
     103030964520926585+I*-200190716385677202,
     103030964520927257+I*-200190716385676148,
     103032658981190275+I*-200187754910266872,
     103032658983034429+I*-200187754906823344,
     103032658983034431+I*-200187754906823340,
     107610727490600107+I*-190558210449184058,
     107610732420867835+I*-190558199282086312,
     107610732420873119+I*-190558199282073374,
     107623448532947765+I*-190522234132045302,
     107623448546346403+I*-190522234090478186,
     107623448546346417+I*-190522234090478138,
     138723113214290289+I*-75441108041314684,
     138723145113039805+I*-75440975342704022,
     138723145113072053+I*-75440975342551136,
     138791092045837015+I*-75019926468093052,
     138791092110087801+I*-75019925984057500,
     138791092110087859+I*-75019925984056944,
     138791177599111211+I*-75018402501840630,
     138791177599160167+I*-75018402500094388,
     138791177599160167+I*-75018402500094385,
     138696496110658581+I*-70250971015525537,
     138696495849346123+I*-70250965562632893,
     138696495849345649+I*-70250965562626661,
     138694389264169893+I*-70234036280200503,
     138694389261215925+I*-70234036260893929,
     138694389261215921+I*-70234036260893907,
     128118029918560779+I*-17981652907639751,
     128118016138338453+I*-17981593448445583,
     128118016138320775+I*-17981593448377979,
     128060985136535143+I*-17799584512041753,
     128060985065428347+I*-17799584305577375,
     128060985065428259+I*-17799584305577141,
     128060714564594337+I*-17798957814885945,
     128060714564264671+I*-17798957814177657,
     127069355671950267+I*-15910921518050435,
     127069354476846755+I*-15910919389155869,
     127069354476845319+I*-15910919389153471,
     127065125986289785+I*-15904568993218159,
     127065125981246883+I*-15904568986086323,
     127065125981246877+I*-15904568986086315,
     112384105573533989+I*2881112486684319,
     112384088169606225+I*2881133514317521,
     112384088169585633+I*2881133514341027,
     112324873981604135+I*2942530888518391,
     112324873911868541+I*2942530956882783,
     112324873911868459+I*2942530956882859,
     112324640413396651+I*2942727452992553,
     112324640413123127+I*2942727453209935,
     111550489596971921+I*3499670193197943,
     111550488693151603+I*3499670805735067,
     111550488693150549+I*3499670805735739,
     111547527217741273+I*3501365265998757,
     111547527214297745+I*3501365267842911,
     111547527214297741+I*3501365267842913,
     101917982756658459+I*8079433775408589,
     101917971589560713+I*8079438705676317,
     101917971589547775+I*8079438705681601,
     101882006439519703+I*8092154817756247,
     101882006397952587+I*8092154831154885,
     101882006397952539+I*8092154831154899,
     -13199119651210915+I*39191819499098771,
     -13199252349821577+I*39191851397848287,
     -13199252349974463+I*39191851397880535,
     -13620301224432547+I*39259798330645497,
     -13620301708468099+I*39259798394896283,
     -13620301708468655+I*39259798394896341,
     -13621825190684969+I*39259883883919693,
     -13621825192431211+I*39259883883968649,
     -13621825192431214+I*39259883883968649,
     -18389256677000062+I*39165202395467063,
     -18389262129892706+I*39165202134154605,
     -18389262129898938+I*39165202134154131,
     -18406191412325096+I*39163095548978375,
     -18406191431631670+I*39163095546024407,
     -18406191431631692+I*39163095546024403,
     -70658574784885848+I*28586736203369261,
     -70658634244080016+I*28586722423146935,
     -70658634244147620+I*28586722423129257,
     -70840643180483846+I*28529691421343625,
     -70840643386948224+I*28529691350236829,
     -70840643386948458+I*28529691350236741,
     -70841269877639654+I*28529420849402819,
     -70841269878347942+I*28529420849073153,
     -72729306174475164+I*27538061956758749,
     -72729308303369730+I*27538060761655237,
     -72729308303372128+I*27538060761653801,
     -72735658699307440+I*27533832271098267,
     -72735658706439276+I*27533832266055365,
     -72735658706439284+I*27533832266055359,
     -91521340179209918+I*12852811858342471,
     -91521361206843120+I*12852794454414707,
     -91521361206866626+I*12852794454394115,
     -91582758581043990+I*12793580266412617,
     -91582758649408382+I*12793580196677023,
     -91582758649408458+I*12793580196676941,
     -91582955145518152+I*12793346698205133,
     -91582955145735534+I*12793346697931609,
     -92139897885723542+I*12019195881780403,
     -92139898498260666+I*12019194977960085,
     -92139898498261338+I*12019194977959031,
     -92141592958524356+I*12016233502549755,
     -92141592960368510+I*12016233499106227,
     -92141592960368512+I*12016233499106223,
     -96719661467934188+I*2386689041466941,
     -96719666398201916+I*2386677874369195,
     -96719666398207200+I*2386677874356257,
     -96732382510281846+I*2350712724328185,
     -96732382523680484+I*2350712682761069,
     -96732382523680498+I*2350712682761021,
     -127832047191624370+I*-112730413366402433,
     -127832079090373886+I*-112730546065013095,
     -127832079090406134+I*-112730546065165981,
     -127900026023171096+I*-113151594939624065,
     -127900026087421882+I*-113151595423659617,
     -127900026087421940+I*-113151595423660173,
     -127900111576445292+I*-113153118905876487,
     -127900111576494248+I*-113153118907622729,
     -127900111576494248+I*-113153118907622732,
     -127805430087992662+I*-117920550392191580,
     -127805429826680204+I*-117920555845084224,
     -127805429826679730+I*-117920555845090456,
     -127803323241503974+I*-117937485127516614,
     -127803323238550006+I*-117937485146823188,
     -127803323238550002+I*-117937485146823210,
     -117226963895894860+I*-170189868500077366,
     -117226950115672534+I*-170189927959271534,
     -117226950115654856+I*-170189927959339138,
     -117169919113869224+I*-170371936895675364,
     -117169919042762428+I*-170371937102139742,
     -117169919042762340+I*-170371937102139976,
     -117169648541928418+I*-170372563592831172,
     -117169648541598752+I*-170372563593539460,
     -116178289649284348+I*-172260599889666682,
     -116178288454180836+I*-172260602018561248,
     -116178288454179400+I*-172260602018563646,
     -116174059963623866+I*-172266952414498958,
     -116174059958580964+I*-172266952421630794,
     -116174059958580958+I*-172266952421630802,
     -101493039550868070+I*-191052633894401436,
     -101493022146940306+I*-191052654922034638,
     -101493022146919714+I*-191052654922058144,
     -101433807958938216+I*-191114052296235508,
     -101433807889202622+I*-191114052364599900,
     -101433807889202540+I*-191114052364599976,
     -101433574390730732+I*-191114248860709670,
     -101433574390457208+I*-191114248860927052,
     -100659423574306002+I*-191671191600915060,
     -100659422670485684+I*-191671192213452184,
     -100659422670484630+I*-191671192213452856,
     -100656461195075354+I*-191672886673715874,
     -100656461191631826+I*-191672886675560028,
     -100656461191631822+I*-191672886675560030,
     -91026916733992540+I*-196250955183125706,
     -91026905566894794+I*-196250960113393434,
     -91026905566881856+I*-196250960113398718,
     -90990940416853784+I*-196263676225473364,
     -90990940375286668+I*-196263676238872002,
     -90990940375286620+I*-196263676238872016,
     24090185673876834+I*-227363340906815888,
     24090318372487496+I*-227363372805565404,
     24090318372640382+I*-227363372805597652,
     24511367247098466+I*-227431319738362614,
     24511367731134018+I*-227431319802613400,
     24511367731134574+I*-227431319802613458,
     24512891213350888+I*-227431405291636810 ],
   /* k=50 */ [ 223575478645480166+I*-404711860676736964,
     223575478645480169+I*-404711860676736964,
     228342910130049017+I*-404617179188235378,
     228342915582941661+I*-404617178926922920,
     228342915582947893+I*-404617178926922446,
     228359844865374051+I*-404615072341746690,
     228359844884680625+I*-404615072338792722,
     228359844884680647+I*-404615072338792718,
     280612228237934803+I*-394038712996137576,
     280612287697128971+I*-394038699215915250,
     280612287697196575+I*-394038699215897572,
     280794296633532801+I*-393981668214111940,
     280794296839997179+I*-393981668143005144,
     280794296839997413+I*-393981668143005056,
     280794923330688609+I*-393981397642171134,
     280794923331396897+I*-393981397641841468,
     282682959627524119+I*-392990038749527064,
     282682961756418685+I*-392990037554423552,
     282682961756421083+I*-392990037554422116,
     282689312152356395+I*-392985809063866582,
     282689312159488231+I*-392985809058823680,
     282689312159488239+I*-392985809058823674,
     301474993632258873+I*-378304788651110786,
     301475014659892075+I*-378304771247183022,
     301475014659915581+I*-378304771247162430,
     301536412034092945+I*-378245557059180932,
     301536412102457337+I*-378245556989445338,
     301536412102457413+I*-378245556989445256,
     478816867487508611+I*-179182969559062220,
     478817063983618305+I*-179182736060590412,
     478817063983835687+I*-179182736060316888,
     479374006723823695+I*-178408585244165682,
     479374007336360819+I*-178408584340345364,
     479374007336361491+I*-178408584340344310,
     479375701796624509+I*-178405622864935034,
     479375701798468663+I*-178405622861491506,
     479375701798468665+I*-178405622861491502,
     483953770306034341+I*-168776078403852220,
     483953775236302069+I*-168776067236754474,
     483953775236307353+I*-168776067236741536,
     483966491348381999+I*-168740102086713464,
     483966491361780637+I*-168740102045146348,
     483966491361780651+I*-168740102045146300,
     515066156029724523+I*-53658975995982846,
     515066187928474039+I*-53658843297372184,
     515066187928506287+I*-53658843297219298,
     515134134861271249+I*-53237794422761214,
     515134134925522035+I*-53237793938725662,
     515134134925522093+I*-53237793938725106,
     515134220414545445+I*-53236270456508792,
     515134220414594401+I*-53236270454762550,
     515134220414594401+I*-53236270454762547,
     515039538926092815+I*-48468838970193699,
     515039538664780357+I*-48468833517301055,
     515039538664779883+I*-48468833517294823,
     515037432079604127+I*-48451904234868665,
     515037432076650159+I*-48451904215562091,
     515037432076650155+I*-48451904215562069,
     504461072733995013+I*3800479137692087,
     504461058953772687+I*3800538596886255,
     504461058953755009+I*3800538596953859,
     504404027951969377+I*3982547533290085,
     504404027880862581+I*3982547739754463,
     504404027880862493+I*3982547739754697,
     504403757380028571+I*3983174230445893,
     504403757379698905+I*3983174231154181,
     503412398487384501+I*5871210527281403,
     503412397292280989+I*5871212656175969,
     503412397292279553+I*5871212656178367,
     503408168801724019+I*5877563052113679,
     503408168796681117+I*5877563059245515,
     503408168796681111+I*5877563059245523,
     488727148388968223+I*24663244532016157,
     488727130985040459+I*24663265559649359,
     488727130985019867+I*24663265559672865,
     488667916797038369+I*24724662933850229,
     488667916727302775+I*24724663002214621,
     488667916727302693+I*24724663002214697,
     289605329296919657+I*202005118387265895,
     289605095798447849+I*202005314883375589,
     289605095798174325+I*202005314883592971,
     288830944982023119+I*202562257623580979,
     288830944078202801+I*202562258236118103,
     288830944078201747+I*202562258236118775,
     288827982602792471+I*202563952696381793,
     288827982599348943+I*202563952698225947,
     288827982599348939+I*202563952698225949,
     279198438141709657+I*207142021205791625,
     279198426974611911+I*207142026136059353,
     279198426974598973+I*207142026136064637,
     279162461824570901+I*207154742248139283,
     279162461783003785+I*207154742261537921,
     279162461783003737+I*207154742261537935,
     164081335733840283+I*238254406929481807,
     164081203035229621+I*238254438828231323,
     164081203035076735+I*238254438828263571,
     163660154160618651+I*238322385761028533,
     163660153676583099+I*238322385825279319,
     163660153676582543+I*238322385825279377,
     163658630194366229+I*238322471314302729,
     163658630192619987+I*238322471314351685,
     163658630192619984+I*238322471314351685,
     158891198708051136+I*238227789825850099,
     158891193255158492+I*238227789564537641,
     158891193255152260+I*238227789564537167,
     158874263972726102+I*238225682979361411,
     158874263953419528+I*238225682976407443,
     158874263953419506+I*238225682976407439,
     106621880600165350+I*227649323633752297,
     106621821140971182+I*227649309853529971,
     106621821140903578+I*227649309853512293,
     106439812204567352+I*227592278851726661,
     106439811998102974+I*227592278780619865,
     106439811998102740+I*227592278780619777,
     106439185507411544+I*227592008279785855,
     106439185506703256+I*227592008279456189,
     104551149210576034+I*226600649387141785,
     104551147081681468+I*226600648192038273,
     104551147081679070+I*226600648192036837,
     104544796685743758+I*226596419701481303,
     104544796678611922+I*226596419696438401,
     104544796678611914+I*226596419696438395,
     85759115205841280+I*211915399288725507,
     85759094178208078+I*211915381884797743,
     85759094178184572+I*211915381884777151,
     85697696804007208+I*211856167696795653,
     85697696735642816+I*211856167627060059,
     85697696735642740+I*211856167627059977,
     -91582758649408458+I*12793580196676941,
     -91582955145518152+I*12793346698205133,
     -91582955145735534+I*12793346697931609,
     -92139897885723542+I*12019195881780403,
     -92139898498260666+I*12019194977960085,
     -92139898498261338+I*12019194977959031,
     -92141592958524356+I*12016233502549755,
     -92141592960368510+I*12016233499106227,
     -92141592960368512+I*12016233499106223,
     -96719661467934188+I*2386689041466941,
     -96719666398201916+I*2386677874369195,
     -96719666398207200+I*2386677874356257,
     -96732382510281846+I*2350712724328185,
     -96732382523680484+I*2350712682761069,
     -96732382523680498+I*2350712682761021,
     -127832047191624370+I*-112730413366402433,
     -127832079090373886+I*-112730546065013095,
     -127832079090406134+I*-112730546065165981,
     -127900026023171096+I*-113151594939624065,
     -127900026087421882+I*-113151595423659617,
     -127900026087421940+I*-113151595423660173,
     -127900111576445292+I*-113153118905876487,
     -127900111576494248+I*-113153118907622729,
     -127900111576494248+I*-113153118907622732,
     -127805430087992662+I*-117920550392191580,
     -127805429826680204+I*-117920555845084224,
     -127805429826679730+I*-117920555845090456,
     -127803323241503974+I*-117937485127516614,
     -127803323238550006+I*-117937485146823188,
     -127803323238550002+I*-117937485146823210,
     -117226963895894860+I*-170189868500077366,
     -117226950115672534+I*-170189927959271534,
     -117226950115654856+I*-170189927959339138,
     -117169919113869224+I*-170371936895675364,
     -117169919042762428+I*-170371937102139742,
     -117169919042762340+I*-170371937102139976,
     -117169648541928418+I*-170372563592831172,
     -117169648541598752+I*-170372563593539460,
     -116178289649284348+I*-172260599889666682,
     -116178288454180836+I*-172260602018561248,
     -116178288454179400+I*-172260602018563646,
     -116174059963623866+I*-172266952414498958,
     -116174059958580964+I*-172266952421630794,
     -116174059958580958+I*-172266952421630802,
     -101493039550868070+I*-191052633894401436,
     -101493022146940306+I*-191052654922034638,
     -101493022146919714+I*-191052654922058144,
     -101433807958938216+I*-191114052296235508,
     -101433807889202622+I*-191114052364599900,
     -101433807889202540+I*-191114052364599976,
     97628779541180496+I*-368394507749651174,
     97629013039652304+I*-368394704245760868,
     97629013039925828+I*-368394704245978250,
     98403163856077034+I*-368951646985966258,
     98403164759897352+I*-368951647598503382,
     98403164759898406+I*-368951647598504054,
     98406126235307682+I*-368953342058767072,
     98406126238751210+I*-368953342060611226,
     98406126238751214+I*-368953342060611228,
     108035670696390496+I*-373531410568176904,
     108035681863488242+I*-373531415498444632,
     108035681863501180+I*-373531415498449916,
     108071647013529252+I*-373544131610524562,
     108071647055096368+I*-373544131623923200,
     108071647055096416+I*-373544131623923214,
     223152773104259870+I*-404643796291867086,
     223152905802870532+I*-404643828190616602,
     223152905803023418+I*-404643828190648850,
     223573954677481502+I*-404711775123413812,
     223573955161517054+I*-404711775187664598,
     223573955161517610+I*-404711775187664656,
     223575478643733924+I*-404711860676688008 ],
   /* k=51 */ [ 223575478645480166+I*-404711860676736964,
     223575478645480169+I*-404711860676736964,
     228342910130049017+I*-404617179188235378,
     228342915582941661+I*-404617178926922920,
     228342915582947893+I*-404617178926922446,
     228359844865374051+I*-404615072341746690,
     228359844884680625+I*-404615072338792722,
     228359844884680647+I*-404615072338792718,
     280612228237934803+I*-394038712996137576,
     280612287697128971+I*-394038699215915250,
     280612287697196575+I*-394038699215897572,
     280794296633532801+I*-393981668214111940,
     280794296839997179+I*-393981668143005144,
     280794296839997413+I*-393981668143005056,
     834417795040482845+I*-173136948667290182,
     834418421531174041+I*-173136678166456260,
     834418421531882329+I*-173136678166126594,
     836306457828009551+I*-172145319273812190,
     836306459956904117+I*-172145318078708678,
     836306459956906515+I*-172145318078707242,
     836312810352841827+I*-172141089588151708,
     836312810359973663+I*-172141089583108806,
     836312810359973671+I*-172141089583108800,
     855098491832744305+I*-157460069175395912,
     855098512860377507+I*-157460051771468148,
     855098512860401013+I*-157460051771447556,
     855159910234578377+I*-157400837583466058,
     855159910302942769+I*-157400837513730464,
     855159910302942845+I*-157400837513730382,
     1032440365687994043+I*41661749916652654,
     1032440562184103737+I*41661983415124462,
     1032440562184321119+I*41661983415397986,
     1032997504924309127+I*42436134231549192,
     1032997505536846251+I*42436135135369510,
     1032997505536846923+I*42436135135370564,
     1032999199997109941+I*42439096610779840,
     1032999199998954095+I*42439096614223368,
     1032999199998954097+I*42439096614223372,
     1037577268506519773+I*52068641071862654,
     1037577273436787501+I*52068652238960400,
     1037577273436792785+I*52068652238973338,
     1037589989548867431+I*52104617389001410,
     1037589989562266069+I*52104617430568526,
     1037589989562266083+I*52104617430568574,
     1068689654230209955+I*167185743479732028,
     1068689686128959471+I*167185876178342690,
     1068689686128991719+I*167185876178495576,
     1068757633061756681+I*167606925052953660,
     1068757633126007467+I*167606925536989212,
     1068757633126007525+I*167606925536989768,
     1068757718615030877+I*167608449019206082,
     1068757718615079833+I*167608449020952324,
     1068757718615079833+I*167608449020952327,
     1068663037126578247+I*172375880505521175,
     1068663036865265789+I*172375885958413819,
     1068663036865265315+I*172375885958420051,
     1068660930280089559+I*172392815240846209,
     1068660930277135591+I*172392815260152783,
     1068660930277135587+I*172392815260152805,
     1058084570934480445+I*224645198613406961,
     1058084557154258119+I*224645258072601129,
     1058084557154240441+I*224645258072668733,
     1058027526152454809+I*224827267009004959,
     1058027526081348013+I*224827267215469337,
     1058027526081347925+I*224827267215469571,
     837182806605633051+I*778450765415955003,
     837182536104799129+I*778451391906646199,
     837182536104469463+I*778451391907354487,
     836191177212155059+I*780339428203481709,
     836191176017051547+I*780339430332376275,
     836191176017050111+I*780339430332378673,
     836186947526494577+I*780345780728313985,
     836186947521451675+I*780345780735445821,
     836186947521451669+I*780345780735445829,
     821505927113738781+I*799131462208216463,
     821505909709811017+I*799131483235849665,
     821505909709790425+I*799131483235873171,
     821446695521808927+I*799192880610050535,
     821446695452073333+I*799192880678414927,
     821446695452073251+I*799192880678415003,
     622384108021690215+I*976473336063466201,
     622383874523218407+I*976473532559575895,
     622383874522944883+I*976473532559793277,
     621609723706793677+I*977030475299781285,
     621609722802973359+I*977030475912318409,
     621609722802972305+I*977030475912319081,
     621606761327563029+I*977032170372582099,
     621606761324119501+I*977032170374426253,
     621606761324119497+I*977032170374426255,
     611977216866480215+I*981610238881991931,
     611977205699382469+I*981610243812259659,
     611977205699369531+I*981610243812264943,
     611941240549341459+I*981622959924339589,
     611941240507774343+I*981622959937738227,
     611941240507774295+I*981622959937738241,
     496860114458610841+I*1012722624605682113,
     496859981760000179+I*1012722656504431629,
     496859981759847293+I*1012722656504463877,
     496438932885389209+I*1012790603437228839,
     496438932401353657+I*1012790603501479625,
     496438932401353101+I*1012790603501479683,
     496437408919136787+I*1012790688990503035,
     496437408917390545+I*1012790688990551991,
     496437408917390542+I*1012790688990551991,
     491669977432821694+I*1012696007502050405,
     491669971979929050+I*1012696007240737947,
     491669971979922818+I*1012696007240737473,
     491653042697496660+I*1012693900655561717,
     491653042678190086+I*1012693900652607749,
     491653042678190064+I*1012693900652607745,
     439400659324935908+I*1002117541309952603,
     439400599865741740+I*1002117527529730277,
     439400599865674136+I*1002117527529712599,
     439218590929337910+I*1002060496527926967,
     439218590722873532+I*1002060496456820171,
     439218590722873298+I*1002060496456820083,
     -114404907477612134+I*781215776981105209,
     -114405533968303330+I*781215506480271287,
     -114405533969011618+I*781215506479941621,
     -116293570265138840+I*780224147587627217,
     -116293572394033406+I*780224146392523705,
     -116293572394035804+I*780224146392522269,
     -116299922789971116+I*780219917901966735,
     -116299922797102952+I*780219917896923833,
     -116299922797102960+I*780219917896923827,
     -135085604269873594+I*765538897489210939,
     -135085625297506796+I*765538880085283175,
     -135085625297530302+I*765538880085262583,
     -135147022671707666+I*765479665897281085,
     -135147022740072058+I*765479665827545491,
     -135147022740072134+I*765479665827545409,
     -312427478125123332+I*566417078397162373,
     -312427674621233026+I*566416844898690565,
     -312427674621450408+I*566416844898417041,
     -312984617361438416+I*565642694082265835,
     -312984617973975540+I*565642693178445517,
     -312984617973976212+I*565642693178444463,
     -312986312434239230+I*565639731703035187,
     -312986312436083384+I*565639731699591659,
     -312986312436083386+I*565639731699591655,
     -317564380943649062+I*556010187241952373,
     -317564385873916790+I*556010176074854627,
     -317564385873922074+I*556010176074841689,
     -317577101985996720+I*555974210924813617,
     -317577101999395358+I*555974210883246501,
     -317577101999395372+I*555974210883246453,
     -348676766667339244+I*440893084834082999,
     -348676798566088760+I*440892952135472337,
     -348676798566121008+I*440892952135319451,
     -348744745498885970+I*440471903260861367,
     -348744745563136756+I*440471902776825815,
     -348744745563136814+I*440471902776825259,
     -348744831052160166+I*440470379294608945,
     -348744831052209122+I*440470379292862703,
     -348744831052209122+I*440470379292862700,
     -348650149563707536+I*435702947808293852,
     -348650149302395078+I*435702942355401208,
     -348650149302394604+I*435702942355394976,
     -348648042717218848+I*435686013072968818,
     -348648042714264880+I*435686013053662244,
     -348648042714264876+I*435686013053662222,
     -338071683371609734+I*383433629700408066,
     -338071669591387408+I*383433570241213898,
     -338071669591369730+I*383433570241146294,
     -338014638589584098+I*383251561304810068,
     -338014638518477302+I*383251561098345690,
     -338014638518477214+I*383251561098345456,
     -117169919042762340+I*-170371937102139976,
     -117169648541928418+I*-170372563592831172,
     -117169648541598752+I*-170372563593539460,
     -116178289649284348+I*-172260599889666682,
     -116178288454180836+I*-172260602018561248,
     -116178288454179400+I*-172260602018563646,
     -116174059963623866+I*-172266952414498958,
     -116174059958580964+I*-172266952421630794,
     -116174059958580958+I*-172266952421630802,
     -101493039550868070+I*-191052633894401436,
     -101493022146940306+I*-191052654922034638,
     -101493022146919714+I*-191052654922058144,
     -101433807958938216+I*-191114052296235508,
     -101433807889202622+I*-191114052364599900,
     -101433807889202540+I*-191114052364599976,
     97628779541180496+I*-368394507749651174,
     97629013039652304+I*-368394704245760868,
     97629013039925828+I*-368394704245978250,
     98403163856077034+I*-368951646985966258,
     98403164759897352+I*-368951647598503382,
     98403164759898406+I*-368951647598504054,
     98406126235307682+I*-368953342058767072,
     98406126238751210+I*-368953342060611226,
     98406126238751214+I*-368953342060611228,
     108035670696390496+I*-373531410568176904,
     108035681863488242+I*-373531415498444632,
     108035681863501180+I*-373531415498449916,
     108071647013529252+I*-373544131610524562,
     108071647055096368+I*-373544131623923200,
     108071647055096416+I*-373544131623923214,
     223152773104259870+I*-404643796291867086,
     223152905802870532+I*-404643828190616602,
     223152905803023418+I*-404643828190648850,
     223573954677481502+I*-404711775123413812,
     223573955161517054+I*-404711775187664598,
     223573955161517610+I*-404711775187664656,
     223575478643733924+I*-404711860676688008 ],
   /* k=52 */ [ 223575478645480166+I*-404711860676736964,
     223575478645480169+I*-404711860676736964,
     228342910130049017+I*-404617179188235378,
     228342915582941661+I*-404617178926922920,
     228342915582947893+I*-404617178926922446,
     228359844865374051+I*-404615072341746690,
     228359844884680625+I*-404615072338792722,
     228359844884680647+I*-404615072338792718,
     280612228237934803+I*-394038712996137576,
     280612287697128971+I*-394038699215915250,
     280612287697196575+I*-394038699215897572,
     280794296633532801+I*-393981668214111940,
     280794296839997179+I*-393981668143005144,
     280794296839997413+I*-393981668143005056,
     834417795040482845+I*-173136948667290182,
     834418421531174041+I*-173136678166456260,
     834418421531882329+I*-173136678166126594,
     836306457828009551+I*-172145319273812190,
     836306459956904117+I*-172145318078708678,
     836306459956906515+I*-172145318078707242,
     836312810352841827+I*-172141089588151708,
     836312810359973663+I*-172141089583108806,
     836312810359973671+I*-172141089583108800,
     855098491832744305+I*-157460069175395912,
     855098512860377507+I*-157460051771468148,
     855098512860401013+I*-157460051771447556,
     855159910234578377+I*-157400837583466058,
     855159910302942769+I*-157400837513730464,
     855159910302942845+I*-157400837513730382,
     1032440365687994043+I*41661749916652654,
     1032440562184103737+I*41661983415124462,
     1032440562184321119+I*41661983415397986,
     1032997504924309127+I*42436134231549192,
     1032997505536846251+I*42436135135369510,
     1032997505536846923+I*42436135135370564,
     1032999199997109941+I*42439096610779840,
     1032999199998954095+I*42439096614223368,
     1032999199998954097+I*42439096614223372,
     1037577268506519773+I*52068641071862654,
     1037577273436787501+I*52068652238960400,
     1037577273436792785+I*52068652238973338,
     1037589989548867431+I*52104617389001410,
     1037589989562266069+I*52104617430568526,
     1037589989562266083+I*52104617430568574,
     1068689654230209955+I*167185743479732028,
     1068689686128959471+I*167185876178342690,
     1068689686128991719+I*167185876178495576,
     1068757633061756681+I*167606925052953660,
     1068757633126007467+I*167606925536989212,
     1068757633126007525+I*167606925536989768,
     1180691692375063209+I*1495698641413675506,
     1180691777864086561+I*1495700164895891820,
     1180691777864135517+I*1495700164897638062,
     1180691777864135517+I*1495700164897638065,
     1180597096375633931+I*1500467596382206913,
     1180597096114321473+I*1500467601835099557,
     1180597096114320999+I*1500467601835105789,
     1180594989529145243+I*1500484531117531947,
     1180594989526191275+I*1500484531136838521,
     1180594989526191271+I*1500484531136838543,
     1170018630183536129+I*1552736914490092699,
     1170018616403313803+I*1552736973949286867,
     1170018616403296125+I*1552736973949354471,
     1169961585401510493+I*1552918982885690697,
     1169961585330403697+I*1552918983092155075,
     1169961585330403609+I*1552918983092155309,
     949116865854688735+I*2106542481292640741,
     949116595353854813+I*2106543107783331937,
     949116595353525147+I*2106543107784040225,
     948125236461210743+I*2108431144080167447,
     948125235266107231+I*2108431146209062013,
     948125235266105795+I*2108431146209064411,
     948121006775550261+I*2108437496604999723,
     948121006770507359+I*2108437496612131559,
     948121006770507353+I*2108437496612131567,
     933439986362794465+I*2127223178084902201,
     933439968958866701+I*2127223199112535403,
     933439968958846109+I*2127223199112558909,
     933380754770864611+I*2127284596486736273,
     933380754701129017+I*2127284596555100665,
     933380754701128935+I*2127284596555100741,
     734318167270745899+I*2304565051940151939,
     734317933772274091+I*2304565248436261633,
     734317933772000567+I*2304565248436479015,
     733543782955849361+I*2305122191176467023,
     733543782052029043+I*2305122191789004147,
     733543782052027989+I*2305122191789004819,
     733540820576618713+I*2305123886249267837,
     733540820573175185+I*2305123886251111991,
     733540820573175181+I*2305123886251111993,
     723911276115535899+I*2309701954758677669,
     723911264948438153+I*2309701959688945397,
     723911264948425215+I*2309701959688950681,
     723875299798397143+I*2309714675801025327,
     723875299756830027+I*2309714675814423965,
     723875299756829979+I*2309714675814423979,
     608794173707666525+I*2340814340482367851,
     608794041009055863+I*2340814372381117367,
     608794041008902977+I*2340814372381149615,
     608372992134444893+I*2340882319313914577,
     608372991650409341+I*2340882319378165363,
     608372991650408785+I*2340882319378165421,
     -719718724226276953+I*2452816378627221105,
     -719720247708493267+I*2452816464116244457,
     -719720247710239509+I*2452816464116293413,
     -719720247710239512+I*2452816464116293413,
     -724487679194808360+I*2452721782627791827,
     -724487684647701004+I*2452721782366479369,
     -724487684647707236+I*2452721782366478895,
     -724504613930133394+I*2452719675781303139,
     -724504613949439968+I*2452719675778349171,
     -724504613949439990+I*2452719675778349167,
     -776756997302694146+I*2442143316435694025,
     -776757056761888314+I*2442143302655471699,
     -776757056761955918+I*2442143302655454021,
     -776939065698292144+I*2442086271653668389,
     -776939065904756522+I*2442086271582561593,
     -776939065904756756+I*2442086271582561505,
     -1330562564105242188+I*2221241552106846631,
     -1330563190595933384+I*2221241281606012709,
     -1330563190596641672+I*2221241281605683043,
     -1332451226892768894+I*2220249922713368639,
     -1332451229021663460+I*2220249921518265127,
     -1332451229021665858+I*2220249921518263691,
     -1332457579417601170+I*2220245693027708157,
     -1332457579424733006+I*2220245693022665255,
     -1332457579424733014+I*2220245693022665249,
     -1351243260897503648+I*2205564672614952361,
     -1351243281925136850+I*2205564655211024597,
     -1351243281925160356+I*2205564655211004005,
     -1351304679299337720+I*2205505441023022507,
     -1351304679367702112+I*2205505440953286913,
     -1351304679367702188+I*2205505440953286831,
     -1528585134752753386+I*2006442853522903795,
     -1528585331248863080+I*2006442620024431987,
     -1528585331249080462+I*2006442620024158463,
     -1529142273989068470+I*2005668469208007257,
     -1529142274601605594+I*2005668468304186939,
     -1529142274601606266+I*2005668468304185885,
     -1529143969061869284+I*2005665506828776609,
     -1529143969063713438+I*2005665506825333081,
     -1529143969063713440+I*2005665506825333077,
     -1533722037571279116+I*1996035962367693795,
     -1533722042501546844+I*1996035951200596049,
     -1533722042501552128+I*1996035951200583111,
     -1533734758613626774+I*1995999986050555039,
     -1533734758627025412+I*1995999986008987923,
     -1533734758627025426+I*1995999986008987875,
     -1564834423294969298+I*1880918859959824421,
     -1564834455193718814+I*1880918727261213759,
     -1564834455193751062+I*1880918727261060873,
     -1564902402126516024+I*1880497678386602789,
     -1564902402190766810+I*1880497677902567237,
     -1564902402190766868+I*1880497677902566681,
     -1676836461439822552+I*552405962025880943,
     -1676836546928845904+I*552404438543664629,
     -1676836546928894860+I*552404438541918387,
     -1676836546928894860+I*552404438541918384,
     -1676741865440393274+I*547637007057349536,
     -1676741865179080816+I*547637001604456892,
     -1676741865179080342+I*547637001604450660,
     -1676739758593904586+I*547620072322024502,
     -1676739758590950618+I*547620072302717928,
     -1676739758590950614+I*547620072302717906,
     -1666163399248295472+I*495367688949463750,
     -1666163385468073146+I*495367629490269582,
     -1666163385468055468+I*495367629490201978,
     -1666106354466269836+I*495185620553865752,
     -1666106354395163040+I*495185620347401374,
     -1666106354395162952+I*495185620347401140,
     -1445261634919448078+I*-58437877853084292,
     -1445261364418614156+I*-58438504343775488,
     -1445261364418284490+I*-58438504344483776,
     -1444270005525970086+I*-60326540640610998,
     -1444270004330866574+I*-60326542769505564,
     -1444270004330865138+I*-60326542769507962,
     -1444265775840309604+I*-60332893165443274,
     -1444265775835266702+I*-60332893172575110,
     -1444265775835266696+I*-60332893172575118,
     -1429584755427553808+I*-79118574645345752,
     -1429584738023626044+I*-79118595672978954,
     -1429584738023605452+I*-79118595673002460,
     -1429525523835623954+I*-79179993047179824,
     -1429525523765888360+I*-79179993115544216,
     -1429525523765888278+I*-79179993115544292,
     -1230462936335505242+I*-256460448500595490,
     -1230462702837033434+I*-256460644996705184,
     -1230462702836759910+I*-256460644996922566,
     -1229688552020608704+I*-257017587736910574,
     -1229688551116788386+I*-257017588349447698,
     -1229688551116787332+I*-257017588349448370,
     -1229685589641378056+I*-257019282809711388,
     -1229685589637934528+I*-257019282811555542,
     -1229685589637934524+I*-257019282811555544,
     -1220056045180295242+I*-261597351319121220,
     -1220056034013197496+I*-261597356249388948,
     -1220056034013184558+I*-261597356249394232,
     -1220020068863156486+I*-261610072361468878,
     -1220020068821589370+I*-261610072374867516,
     -1220020068821589322+I*-261610072374867530,
     -1104938942772425868+I*-292709737042811402,
     -1104938810073815206+I*-292709768941560918,
     -1104938810073662320+I*-292709768941593166,
     -1104517761199204236+I*-292777715874358128,
     -1104517760715168684+I*-292777715938608914,
     -1104517760715168128+I*-292777715938608972,
     223573955161517610+I*-404711775187664656,
     223575478643733924+I*-404711860676688008 ],
   /* k=53 */ [ -1328384355729316940+I*-2948961233181052756,
     -1328384355729316937+I*-2948961233181052756,
     -1323616924244748089+I*-2948866551692551170,
     -1323616918791855445+I*-2948866551431238712,
     -1323616918791849213+I*-2948866551431238238,
     -1323599989509423055+I*-2948864444846062482,
     -1323599989490116481+I*-2948864444843108514,
     -1323599989490116459+I*-2948864444843108510,
     -1271347606136862303+I*-2938288085500453368,
     -1271347546677668135+I*-2938288071720231042,
     -1271347546677600531+I*-2938288071720213364,
     -1271165537741264305+I*-2938231040718427732,
     -1271165537534799927+I*-2938231040647320936,
     -1271165537534799693+I*-2938231040647320848,
     -717542039334314261+I*-2717386321171605974,
     -717541412843623065+I*-2717386050670772052,
     -717541412842914777+I*-2717386050670442386,
     -715653376546787555+I*-2716394691778127982,
     -715653374417892989+I*-2716394690583024470,
     -715653374417890591+I*-2716394690583023034,
     -715647024021955279+I*-2716390462092467500,
     -715647024014823443+I*-2716390462087424598,
     -715647024014823435+I*-2716390462087424592,
     -696861342542052801+I*-2701709441679711704,
     -696861321514419599+I*-2701709424275783940,
     -696861321514396093+I*-2701709424275763348,
     -696799924140218729+I*-2701650210087781850,
     -696799924071854337+I*-2701650210018046256,
     -696799924071854261+I*-2701650210018046174,
     -519519468686803063+I*-2502587622587663138,
     -519519272190693369+I*-2502587389089191330,
     -519519272190475987+I*-2502587389088917806,
     -518962329450487979+I*-2501813238272766600,
     -518962328837950855+I*-2501813237368946282,
     -518962328837950183+I*-2501813237368945228,
     1032997505536846923+I*42436135135370564,
     1032999199997109941+I*42439096610779840,
     1032999199998954095+I*42439096614223368,
     1032999199998954097+I*42439096614223372,
     1037577268506519773+I*52068641071862654,
     1037577273436787501+I*52068652238960400,
     1037577273436792785+I*52068652238973338,
     1037589989548867431+I*52104617389001410,
     1037589989562266069+I*52104617430568526,
     1037589989562266083+I*52104617430568574,
     1068689654230209955+I*167185743479732028,
     1068689686128959471+I*167185876178342690,
     1068689686128991719+I*167185876178495576,
     1068757633061756681+I*167606925052953660,
     1068757633126007467+I*167606925536989212,
     1068757633126007525+I*167606925536989768,
     1180691692375063209+I*1495698641413675506,
     1180691777864086561+I*1495700164895891820,
     1180691777864135517+I*1495700164897638062,
     1180691777864135517+I*1495700164897638065,
     1180597096375633931+I*1500467596382206913,
     1180597096114321473+I*1500467601835099557,
     1180597096114320999+I*1500467601835105789,
     1180594989529145243+I*1500484531117531947,
     1180594989526191275+I*1500484531136838521,
     1180594989526191271+I*1500484531136838543,
     1170018630183536129+I*1552736914490092699,
     1170018616403313803+I*1552736973949286867,
     1170018616403296125+I*1552736973949354471,
     1169961585401510493+I*1552918982885690697,
     1169961585330403697+I*1552918983092155075,
     1169961585330403609+I*1552918983092155309,
     949116865854688735+I*2106542481292640741,
     949116595353854813+I*2106543107783331937,
     949116595353525147+I*2106543107784040225,
     948125236461210743+I*2108431144080167447,
     948125235266107231+I*2108431146209062013,
     948125235266105795+I*2108431146209064411,
     948121006775550261+I*2108437496604999723,
     948121006770507359+I*2108437496612131559,
     948121006770507353+I*2108437496612131567,
     933439986362794465+I*2127223178084902201,
     933439968958866701+I*2127223199112535403,
     933439968958846109+I*2127223199112558909,
     933380754770864611+I*2127284596486736273,
     933380754701129017+I*2127284596555100665,
     933380754701128935+I*2127284596555100741,
     734318167270745899+I*2304565051940151939,
     734317933772274091+I*2304565248436261633,
     734317933772000567+I*2304565248436479015,
     733543782955849361+I*2305122191176467023,
     733543782052029043+I*2305122191789004147,
     733543782052027989+I*2305122191789004819,
     -1810705590452287803+I*3857082026163801925,
     -1810708551927697079+I*3857083720624064943,
     -1810708551931140607+I*3857083720625909097,
     -1810708551931140611+I*3857083720625909099,
     -1820338096388779893+I*3861661789133474775,
     -1820338107555877639+I*3861661794063742503,
     -1820338107555890577+I*3861661794063747787,
     -1820374072705918649+I*3861674510175822433,
     -1820374072747485765+I*3861674510189221071,
     -1820374072747485813+I*3861674510189221085,
     -1935455198796649267+I*3892774174857164957,
     -1935455331495259929+I*3892774206755914473,
     -1935455331495412815+I*3892774206755946721,
     -1935876380369870899+I*3892842153688711683,
     -1935876380853906451+I*3892842153752962469,
     -1935876380853907007+I*3892842153752962527,
     -3263968096730592745+I*4004776213002018211,
     -3263969620212809059+I*4004776298491041563,
     -3263969620214555301+I*4004776298491090519,
     -3263969620214555304+I*4004776298491090519,
     -3268737051699124152+I*4004681617002588933,
     -3268737057152016796+I*4004681616741276475,
     -3268737057152023028+I*4004681616741276001,
     -3268753986434449186+I*4004679510156100245,
     -3268753986453755760+I*4004679510153146277,
     -3268753986453755782+I*4004679510153146273,
     -3321006369807009938+I*3994103150810491131,
     -3321006429266204106+I*3994103137030268805,
     -3321006429266271710+I*3994103137030251127,
     -3321188438202607936+I*3994046106028465495,
     -3321188438409072314+I*3994046105957358699,
     -3321188438409072548+I*3994046105957358611,
     -3874811936609557980+I*3773201386481643737,
     -3874812563100249176+I*3773201115980809815,
     -3874812563100957464+I*3773201115980480149,
     -3876700599397084686+I*3772209757088165745,
     -3876700601525979252+I*3772209755893062233,
     -3876700601525981650+I*3772209755893060797,
     -3876706951921916962+I*3772205527402505263,
     -3876706951929048798+I*3772205527397462361,
     -3876706951929048806+I*3772205527397462355,
     -3895492633401819440+I*3757524506989749467,
     -3895492654429452642+I*3757524489585821703,
     -3895492654429476148+I*3757524489585801111,
     -3895554051803653512+I*3757465275397819613,
     -3895554051872017904+I*3757465275328084019,
     -3895554051872017980+I*3757465275328083937,
     -4072834507257069178+I*3558402687897700901,
     -4072834703753178872+I*3558402454399229093,
     -4072834703753396254+I*3558402454398955569,
     -4073391646493384262+I*3557628303582804363,
     -4073391647105921386+I*3557628302678984045,
     -4073391647105922058+I*3557628302678982991,
     -5625351481480719164+I*1013378930174667199,
     -5625353175940982182+I*1013375968699257923,
     -5625353175942826336+I*1013375968695814395,
     -5625353175942826338+I*1013375968695814391,
     -5629931244450392014+I*1003746424238175109,
     -5629931249380659742+I*1003746413071077363,
     -5629931249380665026+I*1003746413071064425,
     -5629943965492739672+I*1003710447921036353,
     -5629943965506138310+I*1003710447879469237,
     -5629943965506138324+I*1003710447879469189,
     -5661043630174082196+I*888629321830305735,
     -5661043662072831712+I*888629189131695073,
     -5661043662072863960+I*888629189131542187,
     -5661111609005628922+I*888208140257084103,
     -5661111609069879708+I*888208139773048551,
     -5661111609069879766+I*888208139773047995,
     -5773045668318935450+I*-439883576103637743,
     -5773045753807958802+I*-439885099585854057,
     -5773045753808007758+I*-439885099587600299,
     -5773045753808007758+I*-439885099587600302,
     -5772951072319506172+I*-444652531072169150,
     -5772951072058193714+I*-444652536525061794,
     -5772951072058193240+I*-444652536525068026,
     -5772948965473017484+I*-444669465807494184,
     -5772948965470063516+I*-444669465826800758,
     -5772948965470063512+I*-444669465826800780,
     -5762372606127408370+I*-496921849180054936,
     -5762372592347186044+I*-496921908639249104,
     -5762372592347168366+I*-496921908639316708,
     -5762315561345382734+I*-497103917575652934,
     -5762315561274275938+I*-497103917782117312,
     -5762315561274275850+I*-497103917782117546,
     -5541470841798560976+I*-1050727415982602978,
     -5541470571297727054+I*-1050728042473294174,
     -5541470571297397388+I*-1050728042474002462,
     -5540479212405082984+I*-1052616078770129684,
     -5540479211209979472+I*-1052616080899024250,
     -5540479211209978036+I*-1052616080899026648,
     -5540474982719422502+I*-1052622431294961960,
     -5540474982714379600+I*-1052622431302093796,
     -5540474982714379594+I*-1052622431302093804,
     -5525793962306666706+I*-1071408112774864438,
     -5525793944902738942+I*-1071408133802497640,
     -5525793944902718350+I*-1071408133802521146,
     -5525734730714736852+I*-1071469531176698510,
     -5525734730645001258+I*-1071469531245062902,
     -5525734730645001176+I*-1071469531245062978,
     -5326672143214618140+I*-1248749986630114176,
     -5326671909716146332+I*-1248750183126223870,
     -5326671909715872808+I*-1248750183126441252,
     -5325897758899721602+I*-1249307125866429260,
     -5325897757995901284+I*-1249307126478966384,
     -5325897757995900230+I*-1249307126478967056,
     -2781648385491584438+I*-2801266960853764162,
     -2781645424016175162+I*-2801268655314027180,
     -2781645424012731634+I*-2801268655315871334,
     -2781645424012731630+I*-2801268655315871336,
     -2772015879555092348+I*-2805846723823437012,
     -2772015868387994602+I*-2805846728753704740,
     -2772015868387981664+I*-2805846728753710024,
     -2771979903237953592+I*-2805859444865784670,
     -2771979903196386476+I*-2805859444879183308,
     -2771979903196386428+I*-2805859444879183322,
     -2656898777147222974+I*-2836959109547127194,
     -2656898644448612312+I*-2836959141445876710,
     -2656898644448459426+I*-2836959141445908958,
     -2656477595574001342+I*-2837027088378673920,
     -2656477595089965790+I*-2837027088442924706,
     -2656477595089965234+I*-2837027088442924764,
     -1328385879213279496+I*-2948961147691980448,
     -1328384355731063182+I*-2948961233181003800 ],
   /* k=54 */ [ -3440014486349392466+I*-12133669185068797238,
     -3440014486349392463+I*-12133669185068797238,
     -3435247054864823615+I*-12133574503580295652,
     -3435247049411930971+I*-12133574503318983194,
     -3435247049411924739+I*-12133574503318982720,
     -3435230120129498581+I*-12133572396733806964,
     -3435230120110192007+I*-12133572396730852996,
     -3435230120110191985+I*-12133572396730852992,
     -3382977736756937829+I*-12122996037388197850,
     -3382977677297743661+I*-12122996023607975524,
     -3382977677297676057+I*-12122996023607957846,
     -3382795668361339831+I*-12122938992606172214,
     -3382795668154875453+I*-12122938992535065418,
     -3382795668154875219+I*-12122938992535065330,
     -2829172169954389787+I*-11902094273059350456,
     -2829171543463698591+I*-11902094002558516534,
     -2829171543462990303+I*-11902094002558186868,
     -2827283507166863081+I*-11901102643665872464,
     -2827283505037968515+I*-11901102642470768952,
     -2827283505037966117+I*-11901102642470767516,
     2820885536215943887+I*-8364563731836933038,
     2820891886611879199+I*-8364559503346377504,
     2820891886619011035+I*-8364559503341334602,
     2820891886619011043+I*-8364559503341334596,
     2839677568091781677+I*-8349878482933621708,
     2839677589119414879+I*-8349878465529693944,
     2839677589119438385+I*-8349878465529673352,
     2839738986493615749+I*-8349819251341691854,
     2839738986561980141+I*-8349819251271956260,
     2839738986561980217+I*-8349819251271956178,
     3017019441947031415+I*-8150756663841573142,
     3017019638443141109+I*-8150756430343101334,
     3017019638443358491+I*-8150756430342827810,
     3017576581183346499+I*-8149982279526676604,
     3017576581795883623+I*-8149982278622856286,
     3017576581795884295+I*-8149982278622855232,
     4569536416170681401+I*-5605732906118539440,
     4569538110630944419+I*-5605729944643130164,
     4569538110632788573+I*-5605729944639686636,
     4569538110632788575+I*-5605729944639686632,
     4574116179140354251+I*-5596100400182047350,
     4574116184070621979+I*-5596100389014949604,
     4574116184070627263+I*-5596100389014936666,
     4574128900182701909+I*-5596064423864908594,
     4574128900196100547+I*-5596064423823341478,
     4574128900196100561+I*-5596064423823341430,
     4605228564864044433+I*-5480983297774177976,
     4605228596762793949+I*-5480983165075567314,
     4605228596762826197+I*-5480983165075414428,
     4605296543695591159+I*-5480562116200956344,
     4605296543759841945+I*-5480562115716920792,
     4605296543759842003+I*-5480562115716920236,
     4717230603008897687+I*-4152470399840234498,
     4717230688497921039+I*-4152468876358018184,
     4717230688497969995+I*-4152468876356271942,
     4717230688497969995+I*-4152468876356271939,
     4717136007009468409+I*-4147701444871703091,
     4717136006748155951+I*-4147701439418810447,
     4717136006748155477+I*-4147701439418804215,
     4717133900162979721+I*-4147684510136378057,
     4717133900160025753+I*-4147684510117071483,
     4717133900160025749+I*-4147684510117071461,
     4706557540817370607+I*-4095432126763817305,
     4706557527037148281+I*-4095432067304623137,
     4706557527037130603+I*-4095432067304555533,
     4706500496035344971+I*-4095250058368219307,
     4706500495964238175+I*-4095250058161754929,
     4706500495964238087+I*-4095250058161754695,
     4485655776488523213+I*-3541626559961269263,
     4485655505987689291+I*-3541625933470578067,
     4485655505987359625+I*-3541625933469869779,
     4484664147095045221+I*-3539737897173742557,
     4484664145899941709+I*-3539737895044847991,
     4484664145899940273+I*-3539737895044845593,
     948125235266105795+I*2108431146209064411,
     948121006775550261+I*2108437496604999723,
     948121006770507359+I*2108437496612131559,
     948121006770507353+I*2108437496612131567,
     933439986362794465+I*2127223178084902201,
     933439968958866701+I*2127223199112535403,
     933439968958846109+I*2127223199112558909,
     933380754770864611+I*2127284596486736273,
     933380754701129017+I*2127284596555100665,
     933380754701128935+I*2127284596555100741,
     734318167270745899+I*2304565051940151939,
     734317933772274091+I*2304565248436261633,
     734317933772000567+I*2304565248436479015,
     733543782955849361+I*2305122191176467023,
     733543782052029043+I*2305122191789004147,
     733543782052027989+I*2305122191789004819,
     -1810705590452287803+I*3857082026163801925,
     -1810708551927697079+I*3857083720624064943,
     -1810708551931140607+I*3857083720625909097,
     -1810708551931140611+I*3857083720625909099,
     -1820338096388779893+I*3861661789133474775,
     -1820338107555877639+I*3861661794063742503,
     -1820338107555890577+I*3861661794063747787,
     -1820374072705918649+I*3861674510175822433,
     -1820374072747485765+I*3861674510189221071,
     -1820374072747485813+I*3861674510189221085,
     -1935455198796649267+I*3892774174857164957,
     -1935455331495259929+I*3892774206755914473,
     -1935455331495412815+I*3892774206755946721,
     -1935876380369870899+I*3892842153688711683,
     -1935876380853906451+I*3892842153752962469,
     -1935876380853907007+I*3892842153752962527,
     -3263968096730592745+I*4004776213002018211,
     -3263969620212809059+I*4004776298491041563,
     -3263969620214555301+I*4004776298491090519,
     -3263969620214555304+I*4004776298491090519,
     -3268737051699124152+I*4004681617002588933,
     -3268737057152016796+I*4004681616741276475,
     -3268737057152023028+I*4004681616741276001,
     -3268753986434449186+I*4004679510156100245,
     -3268753986453755760+I*4004679510153146277,
     -3268753986453755782+I*4004679510153146273,
     -3321006369807009938+I*3994103150810491131,
     -3321006429266204106+I*3994103137030268805,
     -3321006429266271710+I*3994103137030251127,
     -3321188438202607936+I*3994046106028465495,
     -3321188438409072314+I*3994046105957358699,
     -3321188438409072548+I*3994046105957358611,
     -3874811936609557980+I*3773201386481643737,
     -3874812563100249176+I*3773201115980809815,
     -3874812563100957464+I*3773201115980480149,
     -3876700599397084686+I*3772209757088165745,
     -3876700601525979252+I*3772209755893062233,
     -3876700601525981650+I*3772209755893060797,
     -9524869642779891654+I*235670845259226319,
     -9524875993175826966+I*235666616768670785,
     -9524875993182958802+I*235666616763627883,
     -9524875993182958810+I*235666616763627877,
     -9543661674655729444+I*220985596355914989,
     -9543661695683362646+I*220985578951987225,
     -9543661695683386152+I*220985578951966633,
     -9543723093057563516+I*220926364763985135,
     -9543723093125927908+I*220926364694249541,
     -9543723093125927984+I*220926364694249459,
     -9721003548510979182+I*21863777263866423,
     -9721003745007088876+I*21863543765394615,
     -9721003745007306258+I*21863543765121091,
     -9721560687747294266+I*21089392948969885,
     -9721560688359831390+I*21089392045149567,
     -9721560688359832062+I*21089392045148513,
     -11273520522734629168+I*-2523159980459167279,
     -11273522217194892186+I*-2523162941934576555,
     -11273522217196736340+I*-2523162941938020083,
     -11273522217196736342+I*-2523162941938020087,
     -11278100285704302018+I*-2532792486395659369,
     -11278100290634569746+I*-2532792497562757115,
     -11278100290634575030+I*-2532792497562770053,
     -11278113006746649676+I*-2532828462712798125,
     -11278113006760048314+I*-2532828462754365241,
     -11278113006760048328+I*-2532828462754365289,
     -11309212671427992200+I*-2647909588803528743,
     -11309212703326741716+I*-2647909721502139405,
     -11309212703326773964+I*-2647909721502292291,
     -11309280650259538926+I*-2648330770376750375,
     -11309280650323789712+I*-2648330770860785927,
     -11309280650323789770+I*-2648330770860786483,
     -11421214709572845454+I*-3976422486737472221,
     -11421214795061868806+I*-3976424010219688535,
     -11421214795061917762+I*-3976424010221434777,
     -11421214795061917762+I*-3976424010221434780,
     -11421120113573416176+I*-3981191441706003628,
     -11421120113312103718+I*-3981191447158896272,
     -11421120113312103244+I*-3981191447158902504,
     -11421118006726927488+I*-3981208376441328662,
     -11421118006723973520+I*-3981208376460635236,
     -11421118006723973516+I*-3981208376460635258,
     -11410541647381318374+I*-4033460759813889414,
     -11410541633601096048+I*-4033460819273083582,
     -11410541633601078370+I*-4033460819273151186,
     -11410484602599292738+I*-4033642828209487412,
     -11410484602528185942+I*-4033642828415951790,
     -11410484602528185854+I*-4033642828415952024,
     -11189639883052470980+I*-4587266326616437456,
     -11189639612551637058+I*-4587266953107128652,
     -11189639612551307392+I*-4587266953107836940,
     -11188648253658992988+I*-4589154989403964162,
     -11188648252463889476+I*-4589154991532858728,
     -11188648252463888040+I*-4589154991532861126,
     -7652109341830053562+I*-10237324032786771130,
     -7652105113339498028+I*-10237330383182706442,
     -7652105113334455126+I*-10237330383189838278,
     -7652105113334455120+I*-10237330383189838286,
     -7637424092926742232+I*-10256116064662608920,
     -7637424075522814468+I*-10256116085690242122,
     -7637424075522793876+I*-10256116085690265628,
     -7637364861334812378+I*-10256177483064442992,
     -7637364861265076784+I*-10256177483132807384,
     -7637364861265076702+I*-10256177483132807460,
     -7438302273834693666+I*-10433457938517858658,
     -7438302040336221858+I*-10433458135013968352,
     -7438302040335948334+I*-10433458135014185734,
     -7437527889519797128+I*-10434015077754173742,
     -7437527888615976810+I*-10434015078366710866,
     -7437527888615975756+I*-10434015078366711538,
     -4893278516111659964+I*-11985974912741508644,
     -4893275554636250688+I*-11985976607201771662,
     -4893275554632807160+I*-11985976607203615816,
     -4893275554632807156+I*-11985976607203615818,
     -4883646010175167874+I*-11990554675711181494,
     -4883645999008070128+I*-11990554680641449222,
     -4883645999008057190+I*-11990554680641454506,
     -4883610033858029118+I*-11990567396753529152,
     -4883610033816462002+I*-11990567396766927790,
     -4883610033816461954+I*-11990567396766927804,
     -4768528907767298500+I*-12021667061434871676,
     -4768528775068687838+I*-12021667093333621192,
     -4768528775068534952+I*-12021667093333653440,
     -4768107726194076868+I*-12021735040266418402,
     -4768107725710041316+I*-12021735040330669188,
     -4768107725710040760+I*-12021735040330669246,
     -3440016009833355022+I*-12133669099579724930,
     -3440014486351138708+I*-12133669185068748282 ],
   /* k=55 */ [ -2015105706335633514+I*-26966546178210451724,
     -2015105706335633511+I*-26966546178210451724,
     -2010338274851064663+I*-26966451496721950138,
     -2010338269398172019+I*-26966451496460637680,
     -2010338269398165787+I*-26966451496460637206,
     12822538723743488699+I*-25541542716446878254,
     12822555653025914857+I*-25541540609861702498,
     12822555653045221431+I*-25541540609858748530,
     12822555653045221453+I*-25541540609858748526,
     12874808036398475609+I*-25530964250516093384,
     12874808095857669777+I*-25530964236735871058,
     12874808095857737381+I*-25530964236735853380,
     12874990104794073607+I*-25530907205734067748,
     12874990105000537985+I*-25530907205662960952,
     12874990105000538219+I*-25530907205662960864,
     13428613603201023651+I*-25310062486187245990,
     13428614229691714847+I*-25310062215686412068,
     13428614229692423135+I*-25310062215686082402,
     13430502265988550357+I*-25309070856793767998,
     13430502268117444923+I*-25309070855598664486,
     13430502268117447321+I*-25309070855598663050,
     19078671309371357325+I*-21772531944964828572,
     19078677659767292637+I*-21772527716474273038,
     19078677659774424473+I*-21772527716469230136,
     19078677659774424481+I*-21772527716469230130,
     19097463341247195115+I*-21757846696061517242,
     19097463362274828317+I*-21757846678657589478,
     19097463362274851823+I*-21757846678657568886,
     19097524759649029187+I*-21757787464469587388,
     19097524759717393579+I*-21757787464399851794,
     19097524759717393655+I*-21757787464399851712,
     19274805215102444853+I*-21558724876969468676,
     19274805411598554547+I*-21558724643470996868,
     19274805411598771929+I*-21558724643470723344,
     19275362354338759937+I*-21557950492654572138,
     19275362354951297061+I*-21557950491750751820,
     19275362354951297733+I*-21557950491750750766,
     20827322189326094839+I*-19013701119246434974,
     20827323883786357857+I*-19013698157771025698,
     20827323883788202011+I*-19013698157767582170,
     20827323883788202013+I*-19013698157767582166,
     20831901952295767689+I*-19004068613309942884,
     20831901957226035417+I*-19004068602142845138,
     20831901957226040701+I*-19004068602142832200,
     20831914673338115347+I*-19004032636992804128,
     20831914673351513985+I*-19004032636951237012,
     20831914673351513999+I*-19004032636951236964,
     20863014338019457871+I*-18888951510902073510,
     20863014369918207387+I*-18888951378203462848,
     20863014369918239635+I*-18888951378203309962,
     20863082316851004597+I*-18888530329328851878,
     20863082316915255383+I*-18888530328844816326,
     20863082316915255441+I*-18888530328844815770,
     20975016376164311125+I*-17560438612968130032,
     20975016461653334477+I*-17560437089485913718,
     20975016461653383433+I*-17560437089484167476,
     20975016461653383433+I*-17560437089484167473,
     20974921780164881847+I*-17555669657999598625,
     20974921779903569389+I*-17555669652546705981,
     20974921779903568915+I*-17555669652546699749,
     19550012999889809963+I*-2722792659405045263,
     19550010893304634207+I*-2722775730122619105,
     19550010893301680239+I*-2722775730103312531,
     19550010893301680235+I*-2722775730103312509,
     19539434533959025093+I*-2670523346750058353,
     19539434520178802767+I*-2670523287290864185,
     19539434520178785089+I*-2670523287290796581,
     19539377489176999457+I*-2670341278354460355,
     19539377489105892661+I*-2670341278147995977,
     19539377489105892573+I*-2670341278147995743,
     19318532769630177699+I*-2116717779947510311,
     19318532499129343777+I*-2116717153456819115,
     19318532499129014111+I*-2116717153456110827,
     19317541140236699707+I*-2114829117159983605,
     19317541139041596195+I*-2114829115031089039,
     19317541139041594759+I*-2114829115031086641,
     15781002228407760281+I*3533339926222823363,
     15780997999917204747+I*3533346276618758675,
     15780997999912161845+I*3533346276625890511,
     15780997999912161839+I*3533346276625890519,
     15766316979504448951+I*3552131958098661153,
     15766316962100521187+I*3552131979126294355,
     15766316962100500595+I*3552131979126317861,
     15766257747912519097+I*3552193376500495225,
     15766257747842783503+I*3552193376568859617,
     15766257747842783421+I*3552193376568859693,
     15567195160412400385+I*3729473831953910891,
     15567194926913928577+I*3729474028450020585,
     15567194926913655053+I*3729474028450237967,
     15566420776097503847+I*3730030971190225975,
     15566420775193683529+I*3730030971802763099,
     15566420775193682475+I*3730030971802763771,
     13022171402689366683+I*5281990806177560877,
     13022168441213957407+I*5281992500637823895,
     13022168441210513879+I*5281992500639668049,
     13022168441210513875+I*5281992500639668051,
     13012538896752874593+I*5286570569147233727,
     13012538885585776847+I*5286570574077501455,
     13012538885585763909+I*5286570574077506739,
     13012502920435735837+I*5286583290189581385,
     13012502920394168721+I*5286583290202980023,
     13012502920394168673+I*5286583290202980037,
     12897421794345005219+I*5317682954870923909,
     12897421661646394557+I*5317682986769673425,
     12897421661646241671+I*5317682986769705673,
     12897000612771783587+I*5317750933702470635,
     12897000612287748035+I*5317750933766721421,
     12897000612287747479+I*5317750933766721479,
     11568908896411061741+I*5429684993015777163,
     11568907372928845427+I*5429685078504800515,
     11568907372927099185+I*5429685078504849471,
     11568907372927099182+I*5429685078504849471,
     11564139941442530334+I*5429590397016347885,
     11564139935989637690+I*5429590396755035427,
     11564139935989631458+I*5429590396755034953,
     -3268737057152023028+I*4004681616741276001,
     -3268753986434449186+I*4004679510156100245,
     -3268753986453755760+I*4004679510153146277,
     -3268753986453755782+I*4004679510153146273,
     -3321006369807009938+I*3994103150810491131,
     -3321006429266204106+I*3994103137030268805,
     -3321006429266271710+I*3994103137030251127,
     -3321188438202607936+I*3994046106028465495,
     -3321188438409072314+I*3994046105957358699,
     -3321188438409072548+I*3994046105957358611,
     -3874811936609557980+I*3773201386481643737,
     -3874812563100249176+I*3773201115980809815,
     -3874812563100957464+I*3773201115980480149,
     -3876700599397084686+I*3772209757088165745,
     -3876700601525979252+I*3772209755893062233,
     -3876700601525981650+I*3772209755893060797,
     -9524869642779891654+I*235670845259226319,
     -9524875993175826966+I*235666616768670785,
     -9524875993182958802+I*235666616763627883,
     -9524875993182958810+I*235666616763627877,
     -9543661674655729444+I*220985596355914989,
     -9543661695683362646+I*220985578951987225,
     -9543661695683386152+I*220985578951966633,
     -9543723093057563516+I*220926364763985135,
     -9543723093125927908+I*220926364694249541,
     -9543723093125927984+I*220926364694249459,
     -9721003548510979182+I*21863777263866423,
     -9721003745007088876+I*21863543765394615,
     -9721003745007306258+I*21863543765121091,
     -9721560687747294266+I*21089392948969885,
     -9721560688359831390+I*21089392045149567,
     -9721560688359832062+I*21089392045148513,
     -11273520522734629168+I*-2523159980459167279,
     -11273522217194892186+I*-2523162941934576555,
     -11273522217196736340+I*-2523162941938020083,
     -11273522217196736342+I*-2523162941938020087,
     -11278100285704302018+I*-2532792486395659369,
     -11278100290634569746+I*-2532792497562757115,
     -11278100290634575030+I*-2532792497562770053,
     -11278113006746649676+I*-2532828462712798125,
     -11278113006760048314+I*-2532828462754365241,
     -11278113006760048328+I*-2532828462754365289,
     -11309212671427992200+I*-2647909588803528743,
     -11309212703326741716+I*-2647909721502139405,
     -11309212703326773964+I*-2647909721502292291,
     -11309280650259538926+I*-2648330770376750375,
     -11309280650323789712+I*-2648330770860785927,
     -11309280650323789770+I*-2648330770860786483,
     -11421214709572845454+I*-3976422486737472221,
     -11421214795061868806+I*-3976424010219688535,
     -11421214795061917762+I*-3976424010221434777,
     -11421214795061917762+I*-3976424010221434780,
     -11421120113573416176+I*-3981191441706003628,
     -11421120113312103718+I*-3981191447158896272,
     -11421120113312103244+I*-3981191447158902504,
     -9996211333298344292+I*-18814068440300556990,
     -9996209226713168536+I*-18814085369582983148,
     -9996209226710214568+I*-18814085369602289722,
     -9996209226710214564+I*-18814085369602289744,
     -9985632867367559422+I*-18866337752955543900,
     -9985632853587337096+I*-18866337812414738068,
     -9985632853587319418+I*-18866337812414805672,
     -9985575822585533786+I*-18866519821351141898,
     -9985575822514426990+I*-18866519821557606276,
     -9985575822514426902+I*-18866519821557606510,
     -9764731103038712028+I*-19420143319758091942,
     -9764730832537878106+I*-19420143946248783138,
     -9764730832537548440+I*-19420143946249491426,
     -9763739473645234036+I*-19422031982545618648,
     -9763739472450130524+I*-19422031984674513214,
     -9763739472450129088+I*-19422031984674515612,
     -6227200561816294610+I*-25070201025928425616,
     -6227196333325739076+I*-25070207376324360928,
     -6227196333320696174+I*-25070207376331492764,
     -6227196333320696168+I*-25070207376331492772,
     -6212515312912983280+I*-25088993057804263406,
     -6212515295509055516+I*-25088993078831896608,
     -6212515295509034924+I*-25088993078831920114,
     -6212456081321053426+I*-25089054476206097478,
     -6212456081251317832+I*-25089054476274461870,
     -6212456081251317750+I*-25089054476274461946,
     -6013393493820934714+I*-25266334931659513144,
     -6013393260322462906+I*-25266335128155622838,
     -6013393260322189382+I*-25266335128155840220,
     -6012619109506038176+I*-25266892070895828228,
     -6012619108602217858+I*-25266892071508365352,
     -6012619108602216804+I*-25266892071508366024,
     -3468369736097901012+I*-26818851905883163130,
     -3468366774622491736+I*-26818853600343426148,
     -3468366774619048208+I*-26818853600345270302,
     -3468366774619048204+I*-26818853600345270304,
     -3458737230161408922+I*-26823431668852835980,
     -3458737218994311176+I*-26823431673783103708,
     -3458737218994298238+I*-26823431673783108992,
     -3458701253844270166+I*-26823444389895183638,
     -3458701253802703050+I*-26823444389908582276,
     -3458701253802703002+I*-26823444389908582290,
     -3343620127753539548+I*-26854544054576526162,
     -3343619995054928886+I*-26854544086475275678,
     -3343619995054776000+I*-26854544086475307926,
     -3343198946180317916+I*-26854612033408072888,
     -3343198945696282364+I*-26854612033472323674,
     -3343198945696281808+I*-26854612033472323732,
     -2015107229819596070+I*-26966546092721379416,
     -2015105706337379756+I*-26966546178210402768 ],
   /* k=56 */ [ 29075557059961434410+I*-38949605611324588306,
     29075557059961434413+I*-38949605611324588306,
     29080324491446003261+I*-38949510929836086720,
     29080324496898895905+I*-38949510929574774262,
     29080324496898902137+I*-38949510929574773788,
     43913201490040556623+I*-37524602149561014836,
     43913218419322982781+I*-37524600042975839080,
     43913218419342289355+I*-37524600042972885112,
     43913218419342289377+I*-37524600042972885108,
     43965470802695543533+I*-37514023683630229966,
     43965470862154737701+I*-37514023669850007640,
     43965470862154805305+I*-37514023669849989962,
     43965652871091141531+I*-37513966638848204330,
     43965652871297605909+I*-37513966638777097534,
     43965652871297606143+I*-37513966638777097446,
     44519276369498091575+I*-37293121919301382572,
     44519276995988782771+I*-37293121648800548650,
     44519276995989491059+I*-37293121648800218984,
     44521165032285618281+I*-37292130289907904580,
     44521165034414512847+I*-37292130288712801068,
     44521165034414515245+I*-37292130288712799632,
     50169334075668425249+I*-33755591378078965154,
     50169340426064360561+I*-33755587149588409620,
     50169340426071492397+I*-33755587149583366718,
     50169340426071492405+I*-33755587149583366712,
     50188126107544263039+I*-33740906129175653824,
     50188126128571896241+I*-33740906111771726060,
     50188126128571919747+I*-33740906111771705468,
     50188187525946097111+I*-33740846897583723970,
     50188187526014461503+I*-33740846897513988376,
     50188187526014461579+I*-33740846897513988294,
     50365467981399512777+I*-33541784310083605258,
     50365468177895622471+I*-33541784076585133450,
     50365468177895839853+I*-33541784076584859926,
     50366025120635827861+I*-33541009925768708720,
     50366025121248364985+I*-33541009924864888402,
     50366025121248365657+I*-33541009924864887348,
     51917984955623162763+I*-30996760552360571556,
     51917986650083425781+I*-30996757590885162280,
     51917986650085269935+I*-30996757590881718752,
     51917986650085269937+I*-30996757590881718748,
     51922564718592835613+I*-30987128046424079466,
     51922564723523103341+I*-30987128035256981720,
     51922564723523108625+I*-30987128035256968782,
     63905624156637245207+I*103534731040099142,
     63905636872749319853+I*103570696190127214,
     63905636872762718491+I*103570696231694330,
     63905636872762718505+I*103570696231694378,
     63936736537430662377+I*218651822280857832,
     63936736569329411893+I*218651954979468494,
     63936736569329444141+I*218651954979621380,
     63936804516262209103+I*219073003854079464,
     63936804516326459889+I*219073004338115016,
     63936804516326459947+I*219073004338115572,
     64048738575575515631+I*1547164720214801310,
     64048738661064538983+I*1547166243697017624,
     64048738661064587939+I*1547166243698763866,
     64048738661064587939+I*1547166243698763869,
     64048643979576086353+I*1551933675183332717,
     64048643979314773895+I*1551933680636225361,
     64048643979314773421+I*1551933680636231593,
     62623735199301014469+I*16384810673777886079,
     62623733092715838713+I*16384827603060312237,
     62623733092712884745+I*16384827603079618811,
     62623733092712884741+I*16384827603079618833,
     62613156733370229599+I*16437079986432872989,
     62613156719590007273+I*16437080045892067157,
     62613156719589989595+I*16437080045892134761,
     62613099688588203963+I*16437262054828470987,
     62613099688517097167+I*16437262055034935365,
     62613099688517097079+I*16437262055034935599,
     62392254969041382205+I*16990885553235421031,
     62392254698540548283+I*16990886179726112227,
     62392254698540218617+I*16990886179726820515,
     62391263339647904213+I*16992774216022947737,
     62391263338452800701+I*16992774218151842303,
     62391263338452799265+I*16992774218151844701,
     58854724427818964787+I*22640943259405754705,
     58854720199328409253+I*22640949609801690017,
     58854720199323366351+I*22640949609808821853,
     58854720199323366345+I*22640949609808821861,
     58840039178915653457+I*22659735291281592495,
     58840039161511725693+I*22659735312309225697,
     58840039161511705101+I*22659735312309249203,
     58839979947323723603+I*22659796709683426567,
     58839979947253988009+I*22659796709751790959,
     58839979947253987927+I*22659796709751791035,
     58640917359823604891+I*22837077165136842233,
     58640917126325133083+I*22837077361632951927,
     58640917126324859559+I*22837077361633169309,
     58640142975508708353+I*22837634304373157317,
     58640142974604888035+I*22837634304985694441,
     58640142974604886981+I*22837634304985695113,
     56095893602100571189+I*24389594139360492219,
     56095890640625161913+I*24389595833820755237,
     56095890640621718385+I*24389595833822599391,
     56095890640621718381+I*24389595833822599393,
     56086261096164079099+I*24394173902330165069,
     56086261084996981353+I*24394173907260432797,
     56086261084996968415+I*24394173907260438081,
     24995598318699900491+I*36377233340374574663,
     24995562353549872419+I*36377246056486649309,
     24995562353508305303+I*36377246056500047947,
     24995562353508305255+I*36377246056500047961,
     24880481227459141801+I*36408345721167991833,
     24880481094760531139+I*36408345753066741349,
     24880481094760378253+I*36408345753066773597,
     24880060045885920169+I*36408413699999538559,
     24880060045401884617+I*36408413700063789345,
     24880060045401884061+I*36408413700063789403,
     23551968329525198323+I*36520347759312845087,
     23551966806042982009+I*36520347844801868439,
     23551966806041235767+I*36520347844801917395,
     23551966806041235764+I*36520347844801917395,
     23547199374556666916+I*36520253163313415809,
     23547199369103774272+I*36520253163052103351,
     23547199369103768040+I*36520253163052102877,
     8714322375962113554+I*35095344383038343925,
     8714305446679687396+I*35095342276453168169,
     8714305446660380822+I*35095342276450214201,
     8714305446660380800+I*35095342276450214197,
     8662053063307126644+I*35084765917107559055,
     8662053003847932476+I*35084765903327336729,
     8662053003847864872+I*35084765903327319051,
     8661870994911528646+I*35084708872325533419,
     8661870994705064268+I*35084708872254426623,
     8661870994705064034+I*35084708872254426535,
     8108247496504578602+I*34863864152778711661,
     8108246870013887406+I*34863863882277877739,
     8108246870013179118+I*34863863882277548073,
     8106358833717051896+I*34862872523385233669,
     8106358831588157330+I*34862872522190130157,
     8106358831588154932+I*34862872522190128721,
     2458189790334244928+I*31326333611556294243,
     2458183439938309616+I*31326329383065738709,
     2458183439931177780+I*31326329383060695807,
     2458183439931177772+I*31326329383060695801,
     2439397758458407138+I*31311648362652982913,
     2439397737430773936+I*31311648345249055149,
     2439397737430750430+I*31311648345249034557,
     2439336340056573066+I*31311589131061053059,
     2439336339988208674+I*31311589130991317465,
     2439336339988208598+I*31311589130991317383,
     2262055884603157400+I*31112526543560934347,
     2262055688107047706+I*31112526310062462539,
     2262055688106830324+I*31112526310062189015,
     2261498745366842316+I*31111752159246037809,
     2261498744754305192+I*31111752158342217491,
     2261498744754304520+I*31111752158342216437,
     709538910379507414+I*28567502785837900645,
     709537215919244396+I*28567499824362491369,
     709537215917400242+I*28567499824359047841,
     709537215917400240+I*28567499824359047837,
     704959147409834564+I*28557870279901408555,
     704959142479566836+I*28557870268734310809,
     704959142479561552+I*28557870268734297871,
     -11278100290634575030+I*-2532792497562770053,
     -11278113006746649676+I*-2532828462712798125,
     -11278113006760048314+I*-2532828462754365241,
     -11278113006760048328+I*-2532828462754365289,
     -11309212671427992200+I*-2647909588803528743,
     -11309212703326741716+I*-2647909721502139405,
     -11309212703326773964+I*-2647909721502292291,
     -11309280650259538926+I*-2648330770376750375,
     -11309280650323789712+I*-2648330770860785927,
     -11309280650323789770+I*-2648330770860786483,
     -11421214709572845454+I*-3976422486737472221,
     -11421214795061868806+I*-3976424010219688535,
     -11421214795061917762+I*-3976424010221434777,
     -11421214795061917762+I*-3976424010221434780,
     -11421120113573416176+I*-3981191441706003628,
     -11421120113312103718+I*-3981191447158896272,
     -11421120113312103244+I*-3981191447158902504,
     -9996211333298344292+I*-18814068440300556990,
     -9996209226713168536+I*-18814085369582983148,
     -9996209226710214568+I*-18814085369602289722,
     -9996209226710214564+I*-18814085369602289744,
     -9985632867367559422+I*-18866337752955543900,
     -9985632853587337096+I*-18866337812414738068,
     -9985632853587319418+I*-18866337812414805672,
     -9985575822585533786+I*-18866519821351141898,
     -9985575822514426990+I*-18866519821557606276,
     -9985575822514426902+I*-18866519821557606510,
     -9764731103038712028+I*-19420143319758091942,
     -9764730832537878106+I*-19420143946248783138,
     -9764730832537548440+I*-19420143946249491426,
     -9763739473645234036+I*-19422031982545618648,
     -9763739472450130524+I*-19422031984674513214,
     -9763739472450129088+I*-19422031984674515612,
     -6227200561816294610+I*-25070201025928425616,
     -6227196333325739076+I*-25070207376324360928,
     -6227196333320696174+I*-25070207376331492764,
     -6227196333320696168+I*-25070207376331492772,
     -6212515312912983280+I*-25088993057804263406,
     -6212515295509055516+I*-25088993078831896608,
     -6212515295509034924+I*-25088993078831920114,
     -6212456081321053426+I*-25089054476206097478,
     -6212456081251317832+I*-25089054476274461870,
     -6212456081251317750+I*-25089054476274461946,
     -6013393493820934714+I*-25266334931659513144,
     -6013393260322462906+I*-25266335128155622838,
     -6013393260322189382+I*-25266335128155840220,
     -6012619109506038176+I*-25266892070895828228,
     -6012619108602217858+I*-25266892071508365352,
     -6012619108602216804+I*-25266892071508366024,
     -3468369736097901012+I*-26818851905883163130,
     -3468366774622491736+I*-26818853600343426148,
     -3468366774619048208+I*-26818853600345270302,
     -3468366774619048204+I*-26818853600345270304,
     -3458737230161408922+I*-26823431668852835980,
     -3458737218994311176+I*-26823431673783103708,
     -3458737218994298238+I*-26823431673783108992,
     27631925547302769686+I*-38806491106897245574,
     27631961512452797758+I*-38806503823009320220,
     27631961512494364874+I*-38806503823022718858,
     27631961512494364922+I*-38806503823022718872,
     27747042638543528376+I*-38837603487690662744,
     27747042771242139038+I*-38837603519589412260,
     27747042771242291924+I*-38837603519589444508,
     27747463820116750008+I*-38837671466522209470,
     27747463820600785560+I*-38837671466586460256,
     27747463820600786116+I*-38837671466586460314,
     29075555536477471854+I*-38949605525835515998,
     29075557059959688168+I*-38949605611324539350 ],
   /* k=57 */ [ 29075557059961434410+I*-38949605611324588306,
     29075557059961434413+I*-38949605611324588306,
     29080324491446003261+I*-38949510929836086720,
     29080324496898895905+I*-38949510929574774262,
     29080324496898902137+I*-38949510929574773788,
     43913201490040556623+I*-37524602149561014836,
     43913218419322982781+I*-37524600042975839080,
     43913218419342289355+I*-37524600042972885112,
     43913218419342289377+I*-37524600042972885108,
     43965470802695543533+I*-37514023683630229966,
     43965470862154737701+I*-37514023669850007640,
     43965470862154805305+I*-37514023669849989962,
     43965652871091141531+I*-37513966638848204330,
     43965652871297605909+I*-37513966638777097534,
     43965652871297606143+I*-37513966638777097446,
     44519276369498091575+I*-37293121919301382572,
     44519276995988782771+I*-37293121648800548650,
     44519276995989491059+I*-37293121648800218984,
     44521165032285618281+I*-37292130289907904580,
     44521165034414512847+I*-37292130288712801068,
     44521165034414515245+I*-37292130288712799632,
     50169334075668425249+I*-33755591378078965154,
     50169340426064360561+I*-33755587149588409620,
     50169340426071492397+I*-33755587149583366718,
     50169340426071492405+I*-33755587149583366712,
     50188126107544263039+I*-33740906129175653824,
     50188126128571896241+I*-33740906111771726060,
     50188126128571919747+I*-33740906111771705468,
     105244907761097260835+I*16457359987708293798,
     105244969158471438199+I*16457419201896275296,
     105244969158539802591+I*16457419201966010890,
     105244969158539802667+I*16457419201966010972,
     105422249613924853865+I*16656481789396394008,
     105422249810420963559+I*16656482022894865816,
     105422249810421180941+I*16656482022895139340,
     105422806753161168949+I*16657256173711290546,
     105422806753773706073+I*16657256174615110864,
     105422806753773706745+I*16657256174615111918,
     106974766588148503851+I*19201505547119427710,
     106974768282608766869+I*19201508508594836986,
     106974768282610611023+I*19201508508598280514,
     106974768282610611025+I*19201508508598280518,
     106979346351118176701+I*19211138053055919800,
     106979346356048444429+I*19211138064223017546,
     106979346356048449713+I*19211138064223030484,
     118962405789162586295+I*50301800830520098408,
     118962418505274660941+I*50301836795670126480,
     118962418505288059579+I*50301836795711693596,
     118962418505288059593+I*50301836795711693644,
     118993518169956003465+I*50416917921760857098,
     118993518201854752981+I*50416918054459467760,
     118993518201854785229+I*50416918054459620646,
     118993586148787550191+I*50417339103334078730,
     118993586148851800977+I*50417339103818114282,
     118993586148851801035+I*50417339103818114838,
     119105520208100856719+I*51745430819694800576,
     119105520293589880071+I*51745432343177016890,
     119105520293589929027+I*51745432343178763132,
     119105520293589929027+I*51745432343178763135,
     119105425612101427441+I*51750199774663331983,
     119105425611840114983+I*51750199780116224627,
     119105425611840114509+I*51750199780116230859,
     117680516831826355557+I*66583076773257885345,
     117680514725241179801+I*66583093702540311503,
     117680514725238225833+I*66583093702559618077,
     117680514725238225829+I*66583093702559618099,
     117669938365895570687+I*66635346085912872255,
     117669938352115348361+I*66635346145372066423,
     117669938352115330683+I*66635346145372134027,
     117669881321113545051+I*66635528154308470253,
     117669881321042438255+I*66635528154514934631,
     117669881321042438167+I*66635528154514934865,
     117449036601566723293+I*67189151652715420297,
     117449036331065889371+I*67189152279206111493,
     117449036331065559705+I*67189152279206819781,
     117448044972173245301+I*67191040315502947003,
     117448044970978141789+I*67191040317631841569,
     117448044970978140353+I*67191040317631843967,
     113911506060344305875+I*72839209358885753971,
     113911501831853750341+I*72839215709281689283,
     113911501831848707439+I*72839215709288821119,
     113911501831848707433+I*72839215709288821127,
     113896820811440994545+I*72858001390761591761,
     113896820794037066781+I*72858001411789224963,
     113896820794037046189+I*72858001411789248469,
     63698554694557046923+I*127914783044314589557,
     63698495480369065425+I*127914844441688766921,
     63698495480299329831+I*127914844441757131313,
     63698495480299329749+I*127914844441757131389,
     63499432892868946713+I*128092124897142182587,
     63499432659370474905+I*128092125093638292281,
     63499432659370201381+I*128092125093638509663,
     63498658508554050175+I*128092682036378497671,
     63498658507650229857+I*128092682036991034795,
     63498658507650228803+I*128092682036991035467,
     60954409135145913011+I*129644641871365832573,
     60954406173670503735+I*129644643565826095591,
     60954406173667060207+I*129644643565827939745,
     60954406173667060203+I*129644643565827939747,
     60944776629209420921+I*129649221634335505423,
     60944776618042323175+I*129649221639265773151,
     60944776618042310237+I*129649221639265778435,
     29854113851745242313+I*141632281072379915017,
     29854077886595214241+I*141632293788491989663,
     29854077886553647125+I*141632293788505388301,
     29854077886553647077+I*141632293788505388315,
     29738996760504483623+I*141663393453173332187,
     29738996627805872961+I*141663393485072081703,
     29738996627805720075+I*141663393485072113951,
     29738575578931261991+I*141663461432004878913,
     29738575578447226439+I*141663461432069129699,
     29738575578447225883+I*141663461432069129757,
     28410483862570540145+I*141775395491318185441,
     28410482339088323831+I*141775395576807208793,
     28410482339086577589+I*141775395576807257749,
     28410482339086577586+I*141775395576807257749,
     28405714907602008738+I*141775300895318756163,
     28405714902149116094+I*141775300895057443705,
     28405714902149109862+I*141775300895057443231,
     13572837909007455376+I*140350392115043684279,
     13572820979725029218+I*140350390008458508523,
     13572820979705722644+I*140350390008455554555,
     13572820979705722622+I*140350390008455554551,
     13520568596352468466+I*140339813649112899409,
     13520568536893274298+I*140339813635332677083,
     13520568536893206694+I*140339813635332659405,
     13520386527956870468+I*140339756604330873773,
     13520386527750406090+I*140339756604259766977,
     13520386527750405856+I*140339756604259766889,
     12966763029549920424+I*140118911884784052015,
     12966762403059229228+I*140118911614283218093,
     12966762403058520940+I*140118911614282888427,
     12964874366762393718+I*140117920255390574023,
     12964874364633499152+I*140117920254195470511,
     12964874364633496754+I*140117920254195469075,
     7316705323379586750+I*136581381343561634597,
     7316698972983651438+I*136581377115071079063,
     7316698972976519602+I*136581377115066036161,
     7316698972976519594+I*136581377115066036155,
     7297913291503748960+I*136566696094658323267,
     7297913270476115758+I*136566696077254395503,
     7297913270476092252+I*136566696077254374911,
     -47758868362049248836+I*86368429977774375645,
     -47758929759423426200+I*86368370763586394147,
     -47758929759491790592+I*86368370763516658553,
     -47758929759491790668+I*86368370763516658471,
     -47936210214876841866+I*86169308176086275435,
     -47936210411372951560+I*86169307942587803627,
     -47936210411373168942+I*86169307942587530103,
     -47936767354113156950+I*86168533791771378897,
     -47936767354725694074+I*86168533790867558579,
     -47936767354725694746+I*86168533790867557525,
     -49488727189100491852+I*83624284418363241733,
     -49488728883560754870+I*83624281456887832457,
     -49488728883562599024+I*83624281456884388929,
     -49488728883562599026+I*83624281456884388925,
     -49493306952070164702+I*83614651912426749643,
     -49493306957000432430+I*83614651901259651897,
     -49493306957000437714+I*83614651901259638959,
     -61476366390114574296+I*52523989134962571035,
     -61476379106226648942+I*52523953169812542963,
     -61476379106240047580+I*52523953169770975847,
     -61476379106240047594+I*52523953169770975799,
     -61507478770907991466+I*52408872043721812345,
     -61507478802806740982+I*52408871911023201683,
     -61507478802806773230+I*52408871911023048797,
     -61507546749739538192+I*52408450862148590713,
     -61507546749803788978+I*52408450861664555161,
     -61507546749803789036+I*52408450861664554605,
     -61619480809052844720+I*51080359145787868867,
     -61619480894541868072+I*51080357622305652553,
     -61619480894541917028+I*51080357622303906311,
     -61619480894541917028+I*51080357622303906308,
     -61619386213053415442+I*51075590190819337460,
     -61619386212792102984+I*51075590185366444816,
     -61619386212792102510+I*51075590185366438584,
     -60194477432778343558+I*36242713192224784098,
     -60194475326193167802+I*36242696262942357940,
     -60194475326190213834+I*36242696262923051366,
     -60194475326190213830+I*36242696262923051344,
     -60183898966847558688+I*36190443879569797188,
     -60183898953067336362+I*36190443820110603020,
     -60183898953067318684+I*36190443820110535416,
     -60183841922065533052+I*36190261811174199190,
     -60183841921994426256+I*36190261810967734812,
     -60183841921994426168+I*36190261810967734578,
     -59962997202518711294+I*35636638312767249146,
     -59962996932017877372+I*35636637686276557950,
     -59962996932017547706+I*35636637686275849662,
     -59962005573125233302+I*35634749649979722440,
     -59962005571930129790+I*35634749647850827874,
     -59962005571930128354+I*35634749647850825476,
     -56425466661296293876+I*29986580606596915472,
     -56425462432805738342+I*29986574256200980160,
     -56425462432800695440+I*29986574256193848324,
     -56425462432800695434+I*29986574256193848316,
     -56410781412392982546+I*29967788574721077682,
     -56410781394989054782+I*29967788553693444480,
     -56410781394989034190+I*29967788553693420974,
     -6212515295509034924+I*-25088993078831920114,
     -6212456081321053426+I*-25089054476206097478,
     -6212456081251317832+I*-25089054476274461870,
     -6212456081251317750+I*-25089054476274461946,
     -6013393493820934714+I*-25266334931659513144,
     -6013393260322462906+I*-25266335128155622838,
     -6013393260322189382+I*-25266335128155840220,
     -6012619109506038176+I*-25266892070895828228,
     -6012619108602217858+I*-25266892071508365352,
     -6012619108602216804+I*-25266892071508366024,
     -3468369736097901012+I*-26818851905883163130,
     -3468366774622491736+I*-26818853600343426148,
     -3468366774619048208+I*-26818853600345270302,
     -3468366774619048204+I*-26818853600345270304,
     -3458737230161408922+I*-26823431668852835980,
     -3458737218994311176+I*-26823431673783103708,
     -3458737218994298238+I*-26823431673783108992,
     27631925547302769686+I*-38806491106897245574,
     27631961512452797758+I*-38806503823009320220,
     27631961512494364874+I*-38806503823022718858,
     27631961512494364922+I*-38806503823022718872,
     27747042638543528376+I*-38837603487690662744,
     27747042771242139038+I*-38837603519589412260,
     27747042771242291924+I*-38837603519589444508,
     27747463820116750008+I*-38837671466522209470,
     27747463820600785560+I*-38837671466586460256,
     27747463820600786116+I*-38837671466586460314,
     29075555536477471854+I*-38949605525835515998,
     29075557059959688168+I*-38949605611324539350 ],
   /* k=58 */ [ -131236272304569247032+I*-84289356177759245750,
     -131236272304569247029+I*-84289356177759245750,
     -131231504873084678181+I*-84289261496270744164,
     -131231504867631785537+I*-84289261496009431706,
     -131231504867631779305+I*-84289261496009431232,
     -116398627874490124819+I*-82864352715995672280,
     -116398610945207698661+I*-82864350609410496524,
     -116398610945188392087+I*-82864350609407542556,
     -116398610945188392065+I*-82864350609407542552,
     -116346358561835137909+I*-82853774250064887410,
     -116346358502375943741+I*-82853774236284665084,
     -116346358502375876137+I*-82853774236284647406,
     43965470862154805305+I*-37514023669849989962,
     43965652871091141531+I*-37513966638848204330,
     43965652871297605909+I*-37513966638777097534,
     43965652871297606143+I*-37513966638777097446,
     44519276369498091575+I*-37293121919301382572,
     44519276995988782771+I*-37293121648800548650,
     44519276995989491059+I*-37293121648800218984,
     44521165032285618281+I*-37292130289907904580,
     44521165034414512847+I*-37292130288712801068,
     44521165034414515245+I*-37292130288712799632,
     50169334075668425249+I*-33755591378078965154,
     50169340426064360561+I*-33755587149588409620,
     50169340426071492397+I*-33755587149583366718,
     50169340426071492405+I*-33755587149583366712,
     50188126107544263039+I*-33740906129175653824,
     50188126128571896241+I*-33740906111771726060,
     50188126128571919747+I*-33740906111771705468,
     105244907761097260835+I*16457359987708293798,
     105244969158471438199+I*16457419201896275296,
     105244969158539802591+I*16457419201966010890,
     105244969158539802667+I*16457419201966010972,
     105422249613924853865+I*16656481789396394008,
     105422249810420963559+I*16656482022894865816,
     105422249810421180941+I*16656482022895139340,
     105422806753161168949+I*16657256173711290546,
     105422806753773706073+I*16657256174615110864,
     105422806753773706745+I*16657256174615111918,
     106974766588148503851+I*19201505547119427710,
     106974768282608766869+I*19201508508594836986,
     106974768282610611023+I*19201508508598280514,
     106974768282610611025+I*19201508508598280518,
     106979346351118176701+I*19211138053055919800,
     106979346356048444429+I*19211138064223017546,
     106979346356048449713+I*19211138064223030484,
     118962405789162586295+I*50301800830520098408,
     118962418505274660941+I*50301836795670126480,
     118962418505288059579+I*50301836795711693596,
     118962418505288059593+I*50301836795711693644,
     118993518169956003465+I*50416917921760857098,
     118993518201854752981+I*50416918054459467760,
     118993518201854785229+I*50416918054459620646,
     118993586148787550191+I*50417339103334078730,
     118993586148851800977+I*50417339103818114282,
     118993586148851801035+I*50417339103818114838,
     119105520208100856719+I*51745430819694800576,
     119105520293589880071+I*51745432343177016890,
     119105520293589929027+I*51745432343178763132,
     119105520293589929027+I*51745432343178763135,
     119105425612101427441+I*51750199774663331983,
     119105425611840114983+I*51750199780116224627,
     119105425611840114509+I*51750199780116230859,
     117680516831826355557+I*66583076773257885345,
     117680514725241179801+I*66583093702540311503,
     117680514725238225833+I*66583093702559618077,
     117680514725238225829+I*66583093702559618099,
     117669938365895570687+I*66635346085912872255,
     117669938352115348361+I*66635346145372066423,
     117669938352115330683+I*66635346145372134027,
     72330187785680673239+I*226947175509902815469,
     72330130754678887607+I*226947357518839151695,
     72330130754607780811+I*226947357519045616073,
     72330130754607780723+I*226947357519045616307,
     72109286035132065849+I*227500981017246101739,
     72109285764631231927+I*227500981643736792935,
     72109285764630902261+I*227500981643737501223,
     72108294405738587857+I*227502869680033628445,
     72108294404543484345+I*227502869682162523011,
     72108294404543482909+I*227502869682162525409,
     68571755493909648431+I*233151038723416435413,
     68571751265419092897+I*233151045073812370725,
     68571751265414049995+I*233151045073819502561,
     68571751265414049989+I*233151045073819502569,
     68557070245006337101+I*233169830755292273203,
     68557070227602409337+I*233169830776319906405,
     68557070227602388745+I*233169830776319929911,
     18358804128122389479+I*288226612408845270999,
     18358744913934407981+I*288226673806219448363,
     18358744913864672387+I*288226673806287812755,
     18358744913864672305+I*288226673806287812831,
     18159682326434289269+I*288403954261672864029,
     18159682092935817461+I*288403954458168973723,
     18159682092935543937+I*288403954458169191105,
     18158907942119392731+I*288404511400909179113,
     18158907941215572413+I*288404511401521716237,
     18158907941215571359+I*288404511401521716909,
     15614658568711255567+I*289956471235896514015,
     15614655607235846291+I*289956472930356777033,
     15614655607232402763+I*289956472930358621187,
     15614655607232402759+I*289956472930358621189,
     15605026062774763477+I*289961050998866186865,
     15605026051607665731+I*289961051003796454593,
     15605026051607652793+I*289961051003796459877,
     -15485636714689415131+I*301944110436910596459,
     -15485672679839443203+I*301944123153022671105,
     -15485672679881010319+I*301944123153036069743,
     -15485672679881010367+I*301944123153036069757,
     -15600753805930173821+I*301975222817704013629,
     -15600753938628784483+I*301975222849602763145,
     -15600753938628937369+I*301975222849602795393,
     -15601174987503395453+I*301975290796535560355,
     -15601174987987431005+I*301975290796599811141,
     -15601174987987431561+I*301975290796599811199,
     -16929266703864117299+I*302087224855848866883,
     -16929268227346333613+I*302087224941337890235,
     -16929268227348079855+I*302087224941337939191,
     -16929268227348079858+I*302087224941337939191,
     -16934035658832648706+I*302087130259849437605,
     -16934035664285541350+I*302087130259588125147,
     -16934035664285547582+I*302087130259588124673,
     -31766912657427202068+I*300662221479574365721,
     -31766929586709628226+I*300662219372989189965,
     -31766929586728934800+I*300662219372986235997,
     -31766929586728934822+I*300662219372986235993,
     -31819181970082188978+I*300651643013643580851,
     -31819182029541383146+I*300651642999863358525,
     -31819182029541450750+I*300651642999863340847,
     -192131011394072132192+I*255311892433428683403,
     -192131193403008468418+I*255311835402426897771,
     -192131193403214932796+I*255311835402355790975,
     -192131193403214933030+I*255311835402355790887,
     -192684816901415418462+I*255090990682880076013,
     -192684817527906109658+I*255090990412379242091,
     -192684817527906817946+I*255090990412378912425,
     -192686705564202945168+I*255089999053486598021,
     -192686705566331839734+I*255089999052291494509,
     -192686705566331842132+I*255089999052291493073,
     -198334874607585752136+I*251553460141657658595,
     -198334880957981687448+I*251553455913167103061,
     -198334880957988819284+I*251553455913162060159,
     -198334880957988819292+I*251553455913162060153,
     -198353666639461589926+I*251538774892754347265,
     -198353666660489223128+I*251538774875350419501,
     -198353666660489246634+I*251538774875350398909,
     -253410448293014587722+I*201340508775870399643,
     -253410509690388765086+I*201340449561682418145,
     -253410509690457129478+I*201340449561612682551,
     -253410509690457129554+I*201340449561612682469,
     -253587790145842180752+I*201141386974182299433,
     -253587790342338290446+I*201141386740683827625,
     -253587790342338507828+I*201141386740683554101,
     -253588347285078495836+I*201140612589867402895,
     -253588347285691032960+I*201140612588963582577,
     -253588347285691033632+I*201140612588963581523,
     -255140307120065830738+I*198596363216459265731,
     -255140308814526093756+I*198596360254983856455,
     -255140308814527937910+I*198596360254980412927,
     -255140308814527937912+I*198596360254980412923,
     -255144886883035503588+I*198586730710522773641,
     -255144886887965771316+I*198586730699355675895,
     -255144886887965776600+I*198586730699355662957,
     -267127946321079913182+I*167496067933058595033,
     -267127959037191987828+I*167496031967908566961,
     -267127959037205386466+I*167496031967866999845,
     -267127959037205386480+I*167496031967866999797,
     -267159058701873330352+I*167380950841817836343,
     -267159058733772079868+I*167380950709119225681,
     -267159058733772112116+I*167380950709119072795,
     -267159126680704877078+I*167380529660244614711,
     -267159126680769127864+I*167380529659760579159,
     -267159126680769127922+I*167380529659760578603,
     -267271060740018183606+I*166052437943883892865,
     -267271060825507206958+I*166052436420401676551,
     -267271060825507255914+I*166052436420399930309,
     -267271060825507255914+I*166052436420399930306,
     -267270966144018754328+I*166047668988915361458,
     -267270966143757441870+I*166047668983462468814,
     -267270966143757441396+I*166047668983462462582,
     -265846057363743682444+I*151214791990320808096,
     -265846055257158506688+I*151214775061038381938,
     -265846055257155552720+I*151214775061019075364,
     -265846055257155552716+I*151214775061019075342,
     -265835478897812897574+I*151162522677665821186,
     -265835478884032675248+I*151162522618206627018,
     -265835478884032657570+I*151162522618206559414,
     -220495728317598000126+I*-9149306746324122028,
     -220495671286596214494+I*-9149488755260458254,
     -220495671286525107698+I*-9149488755466922632,
     -220495671286525107610+I*-9149488755466922866,
     -220274826567049392736+I*-9703112253667408298,
     -220274826296548558814+I*-9703112880158099494,
     -220274826296548229148+I*-9703112880158807782,
     -220273834937655914744+I*-9705000916454935004,
     -220273834936460811232+I*-9705000918583829570,
     -220273834936460809796+I*-9705000918583831968,
     -216737296025826975318+I*-15353169959837741972,
     -216737291797336419784+I*-15353176310233677284,
     -216737291797331376882+I*-15353176310240809120,
     -216737291797331376876+I*-15353176310240809128,
     -216722610776923663988+I*-15371961991713579762,
     -216722610759519736224+I*-15371962012741212964,
     -216722610759519715632+I*-15371962012741236470,
     -166524344660039716366+I*-70428743645266577558,
     -166524285445851734868+I*-70428805042640754922,
     -166524285445781999274+I*-70428805042709119314,
     -166524285445781999192+I*-70428805042709119390,
     -166325222858351616156+I*-70606085498094170588,
     -166325222624853144348+I*-70606085694590280282,
     -166325222624852870824+I*-70606085694590497664,
     -166324448474036719618+I*-70606642637330485672,
     -166324448473132899300+I*-70606642637943022796,
     -166324448473132898246+I*-70606642637943023468,
     -163780199100628582454+I*-72158602472317820574,
     -163780196139153173178+I*-72158604166778083592,
     -163780196139149729650+I*-72158604166779927746,
     -163780196139149729646+I*-72158604166779927748,
     -163770566594692090364+I*-72163182235287493424,
     -163770566583524992618+I*-72163182240217761152,
     -163770566583524979680+I*-72163182240217766436,
     -132679903817227911756+I*-84146241673331903018,
     -132679867852077883684+I*-84146254389443977664,
     -132679867852036316568+I*-84146254389457376302,
     -132679867852036316520+I*-84146254389457376316,
     -132564786725987153066+I*-84177354054125320188,
     -132564786593288542404+I*-84177354086024069704,
     -132564786593288389518+I*-84177354086024101952,
     -132564365544413931434+I*-84177422032956866914,
     -132564365543929895882+I*-84177422033021117700,
     -132564365543929895326+I*-84177422033021117758,
     -131236273828053209588+I*-84289356092270173442,
     -131236272304570993274+I*-84289356177759196794 ],
   /* k=59 */ [ -200868600536230613586+I*-450252765473255266078,
     -200868600536230613583+I*-450252765473255266078,
     -200863833104746044735+I*-450252670791766764492,
     -200863833099293152091+I*-450252670791505452034,
     -200863833099293145859+I*-450252670791505451560,
     -186030956106151491373+I*-448827762011491692608,
     -186030939176869065215+I*-448827759904906516852,
     -186030939176849758641+I*-448827759904903562884,
     -186030939176849758619+I*-448827759904903562880,
     -185978686793496504463+I*-448817183545560907738,
     -185978686734037310295+I*-448817183531780685412,
     -185978686734037242691+I*-448817183531780667734,
     -25666857369506561249+I*-403477432965346010290,
     -25666675360570225023+I*-403477375934344224658,
     -25666675360363760645+I*-403477375934273117862,
     -25666675360363760411+I*-403477375934273117774,
     -25113051862163274979+I*-403256531214797402900,
     -25113051235672583783+I*-403256530944296568978,
     -25113051235671875495+I*-403256530944296239312,
     -25111163199375748273+I*-403255539585403924908,
     -25111163197246853707+I*-403255539584208821396,
     -25111163197246851309+I*-403255539584208819960,
     -19462994155992941305+I*-399719000673574985482,
     -19462987805597005993+I*-399718996445084429948,
     -19462987805589874157+I*-399718996445079387046,
     -19462987805589874149+I*-399718996445079387040,
     -19444202124117103515+I*-399704315424671674152,
     -19444202103089470313+I*-399704315407267746388,
     -19444202103089446807+I*-399704315407267725796,
     35612579529435894281+I*-349506049307787726530,
     35612640926810071645+I*-349505990093599745032,
     35612640926878436037+I*-349505990093530009438,
     35612640926878436113+I*-349505990093530009356,
     35789921382263487311+I*-349306927506099626320,
     35789921578759597005+I*-349306927272601154512,
     35789921578759814387+I*-349306927272600880988,
     35790478521499802395+I*-349306153121784729782,
     35790478522112339519+I*-349306153120880909464,
     35790478522112340191+I*-349306153120880908410,
     37342438356487137297+I*-346761903748376592618,
     37342440050947400315+I*-346761900786901183342,
     37342440050949244469+I*-346761900786897739814,
     37342440050949244471+I*-346761900786897739810,
     37347018119456810147+I*-346752271242440100528,
     37347018124387077875+I*-346752271231273002782,
     37347018124387083159+I*-346752271231272989844,
     49330077557501219741+I*-315661608464975921920,
     49330090273613294387+I*-315661572499825893848,
     49330090273626693025+I*-315661572499784326732,
     49330090273626693039+I*-315661572499784326684,
     49361189938294636911+I*-315546491373735163230,
     49361189970193386427+I*-315546491241036552568,
     49361189970193418675+I*-315546491241036399682,
     118993518201854785229+I*50416918054459620646,
     118993586148787550191+I*50417339103334078730,
     118993586148851800977+I*50417339103818114282,
     118993586148851801035+I*50417339103818114838,
     119105520208100856719+I*51745430819694800576,
     119105520293589880071+I*51745432343177016890,
     119105520293589929027+I*51745432343178763132,
     119105520293589929027+I*51745432343178763135,
     119105425612101427441+I*51750199774663331983,
     119105425611840114983+I*51750199780116224627,
     119105425611840114509+I*51750199780116230859,
     117680516831826355557+I*66583076773257885345,
     117680514725241179801+I*66583093702540311503,
     117680514725238225833+I*66583093702559618077,
     117680514725238225829+I*66583093702559618099,
     117669938365895570687+I*66635346085912872255,
     117669938352115348361+I*66635346145372066423,
     117669938352115330683+I*66635346145372134027,
     72330187785680673239+I*226947175509902815469,
     72330130754678887607+I*226947357518839151695,
     72330130754607780811+I*226947357519045616073,
     72330130754607780723+I*226947357519045616307,
     72109286035132065849+I*227500981017246101739,
     72109285764631231927+I*227500981643736792935,
     72109285764630902261+I*227500981643737501223,
     72108294405738587857+I*227502869680033628445,
     72108294404543484345+I*227502869682162523011,
     72108294404543482909+I*227502869682162525409,
     68571755493909648431+I*233151038723416435413,
     68571751265419092897+I*233151045073812370725,
     68571751265414049995+I*233151045073819502561,
     68571751265414049989+I*233151045073819502569,
     68557070245006337101+I*233169830755292273203,
     68557070227602409337+I*233169830776319906405,
     68557070227602388745+I*233169830776319929911,
     18358804128122389479+I*288226612408845270999,
     18358744913934407981+I*288226673806219448363,
     18358744913864672387+I*288226673806287812755,
     18358744913864672305+I*288226673806287812831,
     18159682326434289269+I*288403954261672864029,
     18159682092935817461+I*288403954458168973723,
     18159682092935543937+I*288403954458169191105,
     18158907942119392731+I*288404511400909179113,
     18158907941215572413+I*288404511401521716237,
     18158907941215571359+I*288404511401521716909,
     15614658568711255567+I*289956471235896514015,
     15614655607235846291+I*289956472930356777033,
     15614655607232402763+I*289956472930358621187,
     15614655607232402759+I*289956472930358621189,
     15605026062774763477+I*289961050998866186865,
     15605026051607665731+I*289961051003796454593,
     15605026051607652793+I*289961051003796459877,
     -15485636714689415131+I*301944110436910596459,
     -15485672679839443203+I*301944123153022671105,
     -15485672679881010319+I*301944123153036069743,
     -15485672679881010367+I*301944123153036069757,
     -15600753805930173821+I*301975222817704013629,
     -15600753938628784483+I*301975222849602763145,
     -15600753938628937369+I*301975222849602795393,
     -381564163234124957697+I*371607551081264161947,
     -381564584282999415781+I*371607619028196926909,
     -381564584283483451333+I*371607619028261177695,
     -381564584283483451889+I*371607619028261177753,
     -382892675999360137627+I*371719553087510233437,
     -382892677522842353941+I*371719553172999256789,
     -382892677522844100183+I*371719553172999305745,
     -382892677522844100186+I*371719553172999305745,
     -382897444954328669034+I*371719458491510804159,
     -382897444959781561678+I*371719458491249491701,
     -382897444959781567910+I*371719458491249491227,
     -397730321952923222396+I*370294549711235732275,
     -397730338882205648554+I*370294547604650556519,
     -397730338882224955128+I*370294547604647602551,
     -397730338882224955150+I*370294547604647602547,
     -397782591265578209306+I*370283971245304947405,
     -397782591325037403474+I*370283971231524725079,
     -397782591325037471078+I*370283971231524707401,
     -558094420689568152520+I*324944220665090049957,
     -558094602698504488746+I*324944163634088264325,
     -558094602698710953124+I*324944163634017157529,
     -558094602698710953358+I*324944163634017157441,
     -558648226196911438790+I*324723318914541442567,
     -558648226823402129986+I*324723318644040608645,
     -558648226823402838274+I*324723318644040278979,
     -558650114859698965496+I*324722327285147964575,
     -558650114861827860062+I*324722327283952861063,
     -558650114861827862460+I*324722327283952859627,
     -564298283903081772464+I*321185788373319025149,
     -564298290253477707776+I*321185784144828469615,
     -564298290253484839612+I*321185784144823426713,
     -564298290253484839620+I*321185784144823426707,
     -564317075934957610254+I*321171103124415713819,
     -564317075955985243456+I*321171103107011786055,
     -564317075955985266962+I*321171103107011765463,
     -619373857588510608050+I*270972837007531766197,
     -619373918985884785414+I*270972777793343784699,
     -619373918985953149806+I*270972777793274049105,
     -619373918985953149882+I*270972777793274049023,
     -619551199441338201080+I*270773715205843665987,
     -619551199637834310774+I*270773714972345194179,
     -619551199637834528156+I*270773714972344920655,
     -619551756580574516164+I*270772940821528769449,
     -619551756581187053288+I*270772940820624949131,
     -619551756581187053960+I*270772940820624948077,
     -621103716415561851066+I*268228691448120632285,
     -621103718110022114084+I*268228688486645223009,
     -621103718110023958238+I*268228688486641779481,
     -621103718110023958240+I*268228688486641779477,
     -621108296178531523916+I*268219058942184140195,
     -621108296183461791644+I*268219058931017042449,
     -621108296183461796928+I*268219058931017029511,
     -633091355616575933510+I*237128396164719961587,
     -633091368332688008156+I*237128360199569933515,
     -633091368332701406794+I*237128360199528366399,
     -633091368332701406808+I*237128360199528366351,
     -633122467997369350680+I*237013279073479202897,
     -633122468029268100196+I*237013278940780592235,
     -633122468029268132444+I*237013278940780439349,
     -702754796260929498998+I*-128950130354715580979,
     -702754864207862263960+I*-128950551403590039063,
     -702754864207926514746+I*-128950551404074074615,
     -702754864207926514804+I*-128950551404074075171,
     -702866798267175570488+I*-130278643119950760909,
     -702866798352664593840+I*-130278644643432977223,
     -702866798352664642796+I*-130278644643434723465,
     -702866798352664642796+I*-130278644643434723468,
     -702866703671176141210+I*-130283412074919292316,
     -702866703670914828752+I*-130283412080372184960,
     -702866703670914828278+I*-130283412080372191192,
     -701441794890901069326+I*-145116289073513845678,
     -701441792784315893570+I*-145116306002796271836,
     -701441792784312939602+I*-145116306002815578410,
     -701441792784312939598+I*-145116306002815578432,
     -701431216424970284456+I*-145168558386168832588,
     -701431216411190062130+I*-145168558445628026756,
     -701431216411190044452+I*-145168558445628094360,
     -656091465844755387008+I*-305480387810158775802,
     -656091408813753601376+I*-305480569819095112028,
     -656091408813682494580+I*-305480569819301576406,
     -656091408813682494492+I*-305480569819301576640,
     -655870564094206779618+I*-306034193317502062072,
     -655870563823705945696+I*-306034193943992753268,
     -655870563823705616030+I*-306034193943993461556,
     -655869572464813301626+I*-306036081980289588778,
     -655869572463618198114+I*-306036081982418483344,
     -655869572463618196678+I*-306036081982418485742,
     -652333033552984362200+I*-311684251023672395746,
     -652333029324493806666+I*-311684257374068331058,
     -652333029324488763764+I*-311684257374075462894,
     -652333029324488763758+I*-311684257374075462902,
     -652318348304081050870+I*-311703043055548233536,
     -652318348286677123106+I*-311703043076575866738,
     -652318348286677102514+I*-311703043076575890244,
     -602120082187197103248+I*-366759824709101231332,
     -602120022973009121750+I*-366759886106475408696,
     -602120022972939386156+I*-366759886106543773088,
     -602120022972939386074+I*-366759886106543773164,
     -601920960385509003038+I*-366937166561928824362,
     -601920960152010531230+I*-366937166758424934056,
     -601920960152010257706+I*-366937166758425151438,
     -601920186001194106500+I*-366937723701165139446,
     -601920186000290286182+I*-366937723701777676570,
     -601920186000290285128+I*-366937723701777677242,
     -599375936627785969336+I*-368489683536152474348,
     -599375933666310560060+I*-368489685230612737366,
     -599375933666307116532+I*-368489685230614581520,
     -599375933666307116528+I*-368489685230614581522,
     -599366304121849477246+I*-368494263299122147198,
     -599366304110682379500+I*-368494263304052414926,
     -599366304110682366562+I*-368494263304052420210,
     -568275641344385298638+I*-380477322737166556792,
     -568275605379235270566+I*-380477335453278631438,
     -568275605379193703450+I*-380477335453292030076,
     -568275605379193703402+I*-380477335453292030090,
     -568160524253144539948+I*-380508435117959973962,
     -568160524120445929286+I*-380508435149858723478,
     -568160524120445776400+I*-380508435149858755726,
     -202197114824949756072+I*-450140763381520122280,
     -202196693776075297988+I*-450140831328452887242,
     -202196693775591262436+I*-450140831328517138028,
     -202196693775591261880+I*-450140831328517138086,
     -200868602059714576142+I*-450252765387766193770,
     -200868600536232359828+I*-450252765473255217122 ],
   /* k=60 */ [ -43802175935718692920+I*-1617775321591404693616,
     -43802175935718692917+I*-1617775321591404693616,
     -43797408504234124069+I*-1617775226909916192030,
     -43797408498781231425+I*-1617775226909654879572,
     -43797408498781225193+I*-1617775226909654879098,
     -28964531505639570707+I*-1616350318129641120146,
     -28964514576357144549+I*-1616350316023055944390,
     -28964514576337837975+I*-1616350316023052990422,
     -28964514576337837953+I*-1616350316023052990418,
     -28912262192984583797+I*-1616339739663710335276,
     -28912262133525389629+I*-1616339739649930112950,
     -28912262133525322025+I*-1616339739649930095272,
     131399567231005359417+I*-1570999989083495437828,
     131399749239941695643+I*-1570999932052493652196,
     131399749240148160021+I*-1570999932052422545400,
     131399749240148160255+I*-1570999932052422545312,
     131953372738348645687+I*-1570779087332946830438,
     131953373364839336883+I*-1570779087062445996516,
     131953373364840045171+I*-1570779087062445666850,
     131955261401136172393+I*-1570778095703553352446,
     131955261403265066959+I*-1570778095702358248934,
     131955261403265069357+I*-1570778095702358247498,
     137603430444518979361+I*-1567241556791724413020,
     137603436794914914673+I*-1567241552563233857486,
     137603436794922046509+I*-1567241552563228814584,
     137603436794922046517+I*-1567241552563228814578,
     137622222476394817151+I*-1567226871542821101690,
     137622222497422450353+I*-1567226871525417173926,
     137622222497422473859+I*-1567226871525417153334,
     192679004129947814947+I*-1517028605425937154068,
     192679065527321992311+I*-1517028546211749172570,
     192679065527390356703+I*-1517028546211679436976,
     192679065527390356779+I*-1517028546211679436894,
     192856345982775407977+I*-1516829483624249053858,
     192856346179271517671+I*-1516829483390750582050,
     192856346179271735053+I*-1516829483390750308526,
     698084411938090488489+I*-854534993031419634424,
     698084968880830476497+I*-854534218880603483218,
     698084968881443013621+I*-854534218879699662900,
     698084968881443014293+I*-854534218879699661846,
     699636928715817811399+I*-851989969507195346054,
     699636930410278074417+I*-851989966545719936778,
     699636930410279918571+I*-851989966545716493250,
     699636930410279918573+I*-851989966545716493246,
     699641508478787484249+I*-851980337001258853964,
     699641508483717751977+I*-851980336990091756218,
     699641508483717757261+I*-851980336990091743280,
     711624567916831893843+I*-820889674223794675356,
     711624580632943968489+I*-820889638258644647284,
     711624580632957367127+I*-820889638258603080168,
     711624580632957367141+I*-820889638258603080120,
     711655680297625311013+I*-820774557132553916666,
     711655680329524060529+I*-820774556999855306004,
     711655680329524092777+I*-820774556999855153118,
     781288008561185459331+I*-454811147704359132790,
     781288076508118224293+I*-454810726655484674706,
     781288076508182475079+I*-454810726655000639154,
     781288076508182475137+I*-454810726655000638598,
     781400010567431530821+I*-453482634939123952860,
     781400010652920554173+I*-453482633415641736546,
     781400010652920603129+I*-453482633415639990304,
     781400010652920603129+I*-453482633415639990301,
     781399915971432101543+I*-453477865984155421453,
     781399915971170789085+I*-453477865978702528809,
     781399915971170788611+I*-453477865978702522577,
     779975007191157029659+I*-438644988985560868091,
     779975005084571853903+I*-438644972056278441933,
     779975005084568899935+I*-438644972056259135359,
     779975005084568899931+I*-438644972056259135337,
     779964428725226244789+I*-438592719672905881181,
     779964428711446022463+I*-438592719613446687013,
     779964428711446004785+I*-438592719613446619409,
     734624678145011347341+I*-278280890248915937967,
     734624621114009561709+I*-278280708239979601741,
     734624621113938454913+I*-278280708239773137363,
     734624621113938454825+I*-278280708239773137129,
     734403776394462739951+I*-277727084741572651697,
     734403776123961906029+I*-277727084115081960501,
     734403776123961576363+I*-277727084115081252213,
     734402784765069261959+I*-277725196078785124991,
     734402784763874158447+I*-277725196076656230425,
     734402784763874157011+I*-277725196076656228027,
     730866245853240322533+I*-272077027035402318023,
     730866241624749766999+I*-272077020685006382711,
     730866241624744724097+I*-272077020684999250875,
     730866241624744724091+I*-272077020684999250867,
     730851560604337011203+I*-272058235003526480233,
     730851560586933083439+I*-272058234982498847031,
     730851560586933062847+I*-272058234982498823525,
     680653294487453063581+I*-217001453349973482437,
     680653235273265082083+I*-217001391952599305073,
     680653235273195346489+I*-217001391952530940681,
     680653235273195346407+I*-217001391952530940605,
     680454172685764963371+I*-216824111497145889407,
     680454172452266491563+I*-216824111300649779713,
     680454172452266218039+I*-216824111300649562331,
     18159682092935543937+I*288403954458169191105,
     18158907942119392731+I*288404511400909179113,
     18158907941215572413+I*288404511401521716237,
     18158907941215571359+I*288404511401521716909,
     15614658568711255567+I*289956471235896514015,
     15614655607235846291+I*289956472930356777033,
     15614655607232402763+I*289956472930358621187,
     15614655607232402759+I*289956472930358621189,
     15605026062774763477+I*289961050998866186865,
     15605026051607665731+I*289961051003796454593,
     15605026051607652793+I*289961051003796459877,
     -15485636714689415131+I*301944110436910596459,
     -15485672679839443203+I*301944123153022671105,
     -15485672679881010319+I*301944123153036069743,
     -15485672679881010367+I*301944123153036069757,
     -15600753805930173821+I*301975222817704013629,
     -15600753938628784483+I*301975222849602763145,
     -15600753938628937369+I*301975222849602795393,
     -381564163234124957697+I*371607551081264161947,
     -381564584282999415781+I*371607619028196926909,
     -381564584283483451333+I*371607619028261177695,
     -381564584283483451889+I*371607619028261177753,
     -382892675999360137627+I*371719553087510233437,
     -382892677522842353941+I*371719553172999256789,
     -382892677522844100183+I*371719553172999305745,
     -382892677522844100186+I*371719553172999305745,
     -382897444954328669034+I*371719458491510804159,
     -382897444959781561678+I*371719458491249491701,
     -382897444959781567910+I*371719458491249491227,
     -397730321952923222396+I*370294549711235732275,
     -397730338882205648554+I*370294547604650556519,
     -397730338882224955128+I*370294547604647602551,
     -397730338882224955150+I*370294547604647602547,
     -397782591265578209306+I*370283971245304947405,
     -397782591325037403474+I*370283971231524725079,
     -397782591325037471078+I*370283971231524707401,
     -558094420689568152520+I*324944220665090049957,
     -558094602698504488746+I*324944163634088264325,
     -558094602698710953124+I*324944163634017157529,
     -558094602698710953358+I*324944163634017157441,
     -558648226196911438790+I*324723318914541442567,
     -558648226823402129986+I*324723318644040608645,
     -558648226823402838274+I*324723318644040278979,
     -558650114859698965496+I*324722327285147964575,
     -558650114861827860062+I*324722327283952861063,
     -558650114861827862460+I*324722327283952859627,
     -564298283903081772464+I*321185788373319025149,
     -564298290253477707776+I*321185784144828469615,
     -564298290253484839612+I*321185784144823426713,
     -564298290253484839620+I*321185784144823426707,
     -564317075934957610254+I*321171103124415713819,
     -564317075955985243456+I*321171103107011786055,
     -564317075955985266962+I*321171103107011765463,
     -619373857588510608050+I*270972837007531766197,
     -619373918985884785414+I*270972777793343784699,
     -619373918985953149806+I*270972777793274049105,
     -619373918985953149882+I*270972777793274049023,
     -619551199441338201080+I*270773715205843665987,
     -619551199637834310774+I*270773714972345194179,
     -619551199637834528156+I*270773714972344920655,
     -1124779265396653281592+I*-391520775386985753447,
     -1124779822339393269600+I*-391521549537801904653,
     -1124779822340005806724+I*-391521549538705724971,
     -1124779822340005807396+I*-391521549538705726025,
     -1126331782174380604502+I*-394065798911210041817,
     -1126331783868840867520+I*-394065801872685451093,
     -1126331783868842711674+I*-394065801872688894621,
     -1126331783868842711676+I*-394065801872688894625,
     -1126336361937350277352+I*-394075431417146533907,
     -1126336361942280545080+I*-394075431428313631653,
     -1126336361942280550364+I*-394075431428313644591,
     -1138319421375394686946+I*-425166094194610712515,
     -1138319434091506761592+I*-425166130159760740587,
     -1138319434091520160230+I*-425166130159802307703,
     -1138319434091520160244+I*-425166130159802307751,
     -1138350533756188104116+I*-425281211285851471205,
     -1138350533788086853632+I*-425281211418550081867,
     -1138350533788086885880+I*-425281211418550234753,
     -1207982862019748252434+I*-791244620714046255081,
     -1207982929966681017396+I*-791245041762920713165,
     -1207982929966745268182+I*-791245041763404748717,
     -1207982929966745268240+I*-791245041763404749273,
     -1208094864025994323924+I*-792573133479281435011,
     -1208094864111483347276+I*-792573135002763651325,
     -1208094864111483396232+I*-792573135002765397567,
     -1208094864111483396232+I*-792573135002765397570,
     -1208094769429994894646+I*-792577902434249966418,
     -1208094769429733582188+I*-792577902439702859062,
     -1208094769429733581714+I*-792577902439702865294,
     -1206669860649719822762+I*-807410779432844519780,
     -1206669858543134647006+I*-807410796362126945938,
     -1206669858543131693038+I*-807410796362146252512,
     -1206669858543131693034+I*-807410796362146252534,
     -1206659282183789037892+I*-807463048745499506690,
     -1206659282170008815566+I*-807463048804958700858,
     -1206659282170008797888+I*-807463048804958768462,
     -1161319531603574140444+I*-967774878169489449904,
     -1161319474572572354812+I*-967775060178425786130,
     -1161319474572501248016+I*-967775060178632250508,
     -1161319474572501247928+I*-967775060178632250742,
     -1161098629853025533054+I*-968328683676832736174,
     -1161098629582524699132+I*-968328684303323427370,
     -1161098629582524369466+I*-968328684303324135658,
     -1161097638223632055062+I*-968330572339620262880,
     -1161097638222436951550+I*-968330572341749157446,
     -1161097638222436950114+I*-968330572341749159844,
     -1157561099311803115636+I*-973978741383003069848,
     -1157561095083312560102+I*-973978747733399005160,
     -1157561095083307517200+I*-973978747733406136996,
     -1157561095083307517194+I*-973978747733406137004,
     -1157546414062899804306+I*-973997533414878907638,
     -1157546414045495876542+I*-973997533435906540840,
     -1157546414045495855950+I*-973997533435906564346,
     -1107348147946015856684+I*-1029054315068431905434,
     -1107348088731827875186+I*-1029054376465806082798,
     -1107348088731758139592+I*-1029054376465874447190,
     -1107348088731758139510+I*-1029054376465874447266,
     -1107149026144327756474+I*-1029231656921259498464,
     -1107149025910829284666+I*-1029231657117755608158,
     -1107149025910829011142+I*-1029231657117755825540,
     -444854535551498337040+I*-1534459722876574578976,
     -444853761400682185834+I*-1534460279819314566984,
     -444853761399778365516+I*-1534460279819927104108,
     -444853761399778364462+I*-1534460279819927104780,
     -442309512027274048670+I*-1536012239654301901886,
     -442309509065798639394+I*-1536012241348762164904,
     -442309509065795195866+I*-1536012241348764009058,
     -442309509065795195862+I*-1536012241348764009060,
     -442299879521337556580+I*-1536016819417271574736,
     -442299879510170458834+I*-1536016819422201842464,
     -442299879510170445896+I*-1536016819422201847748,
     -411209216743873377972+I*-1547999878855315984330,
     -411209180778723349900+I*-1547999891571428058976,
     -411209180778681782784+I*-1547999891571441457614,
     -411209180778681782736+I*-1547999891571441457628,
     -411094099652632619282+I*-1548030991236109401500,
     -411094099519934008620+I*-1548030991268008151016,
     -411094099519933855734+I*-1548030991268008183264,
     -45130690224437835406+I*-1617663319499669549818,
     -45130269175563377322+I*-1617663387446602314780,
     -45130269175079341770+I*-1617663387446666565566,
     -45130269175079341214+I*-1617663387446666565624,
     -43802177459202655476+I*-1617775321505915621308,
     -43802175935720439162+I*-1617775321591404644660 ]
 ]};

gp_inline__location="../r5dragon.tex:27870";
 hull_points_by_generated(k) = hull_generated_samples[k+1];
 Hsides_by_generated(k) = \
   geom_sides_of_polygon(hull_points_by_generated(k));
 HA_by_generated(k) = \
   geom_area_of_polygon(hull_points_by_generated(k));

gp_inline__location="../r5dragon.tex:27876";
 Qhull_points_by_generated(k) = Qhull_generated_samples[k+1];
 QHsides_by_generated(k) = \
   geom_sides_of_polygon(Qhull_points_by_generated(k));
 QHA_by_generated(k) = \
   geom_area_of_polygon(Qhull_points_by_generated(k));

gp_inline__location="../r5dragon.tex:27882";
 z_grid_points(w,h) = \
   concat(vector(2*w-1,x,x-=w; vector(2*h-1,y,y-=h; x+I*y)));

gp_inline__location="../r5dragon.tex:27886";
 \\ return z rotated by suitable I^d to put it in the first
 \\ quadrant, so angle 0 <= arg(ret) < 90
 z_rotate_firstquad(z) = {
   if(imag(z)<0||(imag(z)==0&&real(z)<0),z=-z);
   if(real(z)<0||(real(z)==0&&imag(z)),z*=-I);
   z;
 }

gp_inline__location="../r5dragon.tex:28068";
 Hsides_unsorted_formula(k) = {
   my(l=List([]));
   for(d=0,3,
       listput(l, 3*I^d));
   for(d=0,3,
       for(j=1,k-3,
           listput(l, 2*b^j*I^d)));
   forstep(s=1,-1,-2,   \\ +/- 1
           listput(l, s *  2*I *b^(k-2));
           listput(l, s * (4-I)*b^(k-2)));
   Vec(l);
 }
 Hsides_unsorted(k) = {
   k>=0 || error("HA is for k>=0");
   if(k==0, return([1,-1]));
   if(k==1, return([1,2*I,-1,-2*I]));
   if(k==2, return([1, 3*I, -4 + I, -1, -3*I, 4 - I]));
   Hsides_unsorted_formula(k);
 }

gp_inline__location="../r5dragon.tex:28109";
 HnumSides_formula(k) = 4*k-4;
 HnumSides(k) = if(k==0,2, k==1,4, k==2,6, HnumSides_formula(k));

gp_inline__location="../r5dragon.tex:28115";
 HnumSides_samples = {[2,4,6,8,12,16,20]};

gp_inline__location="../r5dragon.tex:28601";
 Hsides_unsorted_formula_by_combined(k) = {
   my(l=List([]));
   for(j=0,k-2,
       for(d=0,3,
           my(coeff=if(j==0, 3,
                       j==k-2 && d%2==0, 4-I,
                       2));
           listput(l, coeff * I^d * b^j)));
   Vec(l);
 }

gp_inline__location="../r5dragon.tex:28631";
 HB_initial = {[2,6]};

gp_inline__location="../r5dragon.tex:28648";
 HB_inexact_formula(k) = {
   ( 6 + 2*sqrt5 + 2*sqrt(17) ) * sqrt5^(k-2)
   + 2 - 2*sqrt5;
 }

gp_inline__location="../r5dragon.tex:28657";
 HB_inexact(k) = \
   if(k<#HB_initial,HB_initial[k+1], HB_inexact_formula(k));

gp_inline__location="../r5dragon.tex:28660";
 \\ return pair v = [ f, s ]
 \\ f is a quad in sqrt5 (including a rational part)
 \\ s is a quad in either sqrt17 or sqrt85 (no rational part)
 HB_vec_formula(k) = {
   [ (6+2*sqrt5)*sqrt5^(k-2) + 2-2*sqrt5,
      if(k%2==0,
         2*sqrt17                  *simplify(sqrt5^(k-2)),
         2*geom_sqrt_as_quad(17*5) *simplify(sqrt5^(k-2-1)))
   ];
 }

gp_inline__location="../r5dragon.tex:28676";
 HB_vec(k) = \
   if(k<#HB_initial,[HB_initial[k+1],0], HB_vec_formula(k));

gp_inline__location="../r5dragon.tex:28684";
 HB_vec_by_sides(k) = {
   my(v=Hsides_unsorted(k),ret=[0,0]);
   for(i=1,#v,
       my(l=geom_abs(v[i]));
       if(type(l)=="t_INT",ret[1]+=l,
          l.disc==sqrt5.disc,ret[1]+=l,
          ret[2]+=l));
   ret;
 }

gp_inline__location="../r5dragon.tex:28723";
 HB_by_sides_inexact(k) = vecsum(abs(Hsides(k)*1.0));

gp_inline__location="../r5dragon.tex:28764";
 Hsidesf_unsorted(k) = {
   my(l=List([]));
   for(d=0,3, for(j=3,k-1, listput(l, 2*I^d / b^j)));
   forstep(s=-1,1,2,   \\ +/- 1
           listput(l, s *  2*I /b^2);
           listput(l, s * (4-I)/b^2));
   Vec(l);
 }

gp_inline__location="../r5dragon.tex:28787";
 HBf_inexact = (6 + 2*sqrt(5) + 2*sqrt(17))/5;

gp_inline__location="../r5dragon.tex:28798";
 HBf_sample = {(3743669/1000000)};

gp_inline__location="../r5dragon.tex:28818";
 Manhattan(z) = abs(real(z)) + abs(imag(z));
 HBmanhattan(k) = vecsum(apply(Manhattan,Hsides_unsorted(k)));

gp_inline__location="../r5dragon.tex:28837";
 HBf_poly(x) = 625*x^4 - 3000*x^3 + 1000*x^2 + 6240*x - 2736;

gp_inline__location="../r5dragon.tex:28842";
 A349010_samps = {[
   3,7,4,3,6,6,9,4,4,1,2,4,6,9,8,0,0,9,8,4,9,2,2,3,3,4,0,9,8,8,2,1,4,1,3,
   0,4,2,3,5,1,2,7,0,3,3,9,9,4,0,5,8,4,6,3,4,6,7,8,1,2,3,2,7,4,0,2,1,9,0,
   1,0,8,7,9,0,1,7,0,5,9,7,2,0,0,9,1,1,2,2,3,6,7,5,7,8,6,6,2,8,6,6,1,6,2
  ]};

gp_inline__location="../r5dragon.tex:28878";
 \\ my line of code
 { my(c=352+32*quadgen(5*17*4));
 my(a_vector(len) = my(s=10^(len-2)); digits(sqrtint(floor(c*s^2)) + floor(12*s));
   ); HBf_digits = a_vector; }

gp_inline__location="../r5dragon.tex:28911";
 HA_initial = {[0,2]};

gp_inline__location="../r5dragon.tex:28944";
 HA_formula(k) = {
   17*5^(k-2)  - 1
   + sum(j=1,k-2,  (3*5^(k-2-j) - 1)*HAgrow(  2  *b^j)
                 +  2*5^(k-2-j)     *HAgrow((4-I)*b^j));
 }
 HA(k) = {
   k>=0 || error("HA is for k>=0");
   if(k<=1,HA_initial[k+1], HA_formula(k));
 }

gp_inline__location="../r5dragon.tex:28954";
 HA_formula2(k) = {
   17*5^(k-2)  - 1
   + sum(j=3,k-2,  (3*5^(k-2-j) - 1)*HAgrow(  2  *b^j)
                 +  2*5^(k-2-j)     *HAgrow((4-I)*b^j));
 }

gp_inline__location="../r5dragon.tex:28964";
 HA_samples = {[0,2,16,106,578,2954,15064]};

gp_inline__location="../r5dragon.tex:28991";
 HAgrow_to_negib(z) = if(z==0,0, z * I^-z_quadrant((2+I)*z));
 HAgrow(z) = abs1or2(imag(HAgrow_to_negib(z)));

gp_inline__location="../r5dragon.tex:28994";
 HAgrow_to_negib_by_rotate(z) = \
   z_rotate_firstquad(z/(b/I))*(b/I);

gp_inline__location="../r5dragon.tex:29033";
 abs1or2(x) = abs(x) * if(x>=0,1,2);

gp_inline__location="../r5dragon.tex:29040";
 z_quadrant_by_arg(z) = if(z==0,0, floor(arg(z) / (Pi/2)));
 z_quadrant(z) = {
   if(z==0,return(0));
   for(q=0,3, if(real(z)>0&&imag(z)>=0,return(q)); z*=-I);
   error("oops");
 }

gp_inline__location="../r5dragon.tex:29413";
 HA_by_recurrence(k) = \
   5*HA(k-1) + 4 + vecsum(apply(HAgrow,Hsides_unsorted(k)));

gp_inline__location="../r5dragon.tex:29497";
 A349008 = HA;
 A349008_samps={[
  0,2,16,106,578,2954,15064,75908,380334,1904330,9528618,47654840,238295096,
   1191556256,5957936770,29789860126,148950201902,744752899780,3723767329212,
    18618843605284,93094240114350,465471240742354,2327356261817746,11636781564585616
 ]};

gp_inline__location="../r5dragon.tex:29534";
 my(k=1000); HAf_inexact = HA(k)/5.0^k;

gp_inline__location="../r5dragon.tex:29550";
 \\ terms of the sum up to j=k inclusive
 HAf_upto_k(k) = {
   17/25 + 1/25*sum(j=1,k, 6*HAgrow(      (b/5)^j)
                         + 2*HAgrow((4-I)*(b/5)^j) );
 }

gp_inline__location="../r5dragon.tex:29576";
 HAtermf(j) = {
   j>=1 || error("HAtermf() is for j>=1");
   HAtermfZ(1/b^j);
 }
 HAtermfZ(z) = {
   1/25 *( 6*HAgrowf(      z)
         + 2*HAgrowf((4+I)*z) );
 }
 HAf_upto_k_by_HAgrowf(k) = \
   17/25 + sum(j=1,k, HAtermf(j));

gp_inline__location="../r5dragon.tex:29594";
 HAf_sample = {(0976164/1000000)};

gp_inline__location="../r5dragon.tex:29635";
 HAgrowf_to_negib(z) = if(z==0,0, z * I^-z_quadrant(b*z));
 HAgrowf(z) = abs2or1(imag(HAgrowf_to_negib(z)));

gp_inline__location="../r5dragon.tex:29654";
 abs2or1(x) = abs(x) * if(x>=0,2,1);

gp_inline__location="../r5dragon.tex:29724";
 HAtermZ_bound_factor = 1/25 * 26/sqrt5;

gp_inline__location="../r5dragon.tex:29747";
 HAf_bound_factor = 1/25 * 26/sqrt5 * (5 + sqrt5)/4;

gp_inline__location="../r5dragon.tex:29753";
HAf_bound_factor_sample = {(0841377/1000000)};

gp_inline__location="../r5dragon.tex:29794";
 HAtermf_bound(j) = (26/25)/sqrt5^(j+1);

gp_inline__location="../r5dragon.tex:29830";
 { my(b = 1+2*I,
      HAgrowf = (z)->if(sign(imag(z))==sign(real(z)), z*=I);
                     min(abs(2*real(z)),abs(imag(z))));
 \\ stopping at k, sum of omitted terms is at most sqrt(5)^k
 HAf_a_sum(k) = 17/25 + 1/25*sum(j=1,k,
            6*HAgrowf(1/b^j) + 2*HAgrowf((4+I)/b^j)); }

gp_inline__location="../r5dragon.tex:29844";
 \\ return a vector of the first len many decimal digits
 HAf_vector(len) = {
   my(debug=0);
   my(f=10^len, s=17/25*f, e=f);
   for(j=1,oo,
     if(debug,print("at s="floor(s)" e="e*1.0));
     s += f/25*( 6*HAgrowf(1/b^j) + 2*HAgrowf((4+I)/b^j));
     if(j%2==0 && frac(s)+(e/=5) < 1,
       if(debug,print("final s="floor(s)"  "s));
       if(debug,print(" frac ",frac(s)*1.0));
       if(debug,print("    e=",e*1.0));
       s == f*HAf_upto_k(j) || error("want ",f*HAf_upto_k(j));
       e == f/sqrt5^j || error("j=",j," want error ",f/sqrt5^j);
       return(digits(floor(s))));
     );
  }

gp_inline__location="../r5dragon.tex:29866";
 { my(b = 1+2*I,
      HAgrowf = (z)->if(sign(imag(z))!=sign(real(z)), z*=I);
                     min(abs(real(z)),2*abs(imag(z))));
 HAf_compact(len) = my(e=10^len, f=e/25, s=17*f);
   for(j=1,oo, s += f*(6*HAgrowf(1/b^j) + 2*HAgrowf((4+I)/b^j));
     if(j%2==0 && frac(s)+(e/=5) < 1, return(digits(floor(s))))); }

gp_inline__location="../r5dragon.tex:30009";
 HA_by_powers(k) = {
   if(k==0,return(0));
   if(k==1,return(2));
   if(k==2,return(16));
   5^(k-2) - 1
   + sum(j=1,k-3, (5^(k-2)-1)/4 * HAgrow(2*b^j))
   + 2 * sum(j=1,k-3, HAgrow((4-I)*b^j))
   + 2 * HAgrow(2*b^(k-2));

   my(lmax=1);
   sum(l=0,lmax,
     5^l * 4
     + 5^l * vecsum(apply(HAgrow,Hsides_unsorted(k-l))))
   + 5^(lmax+1)*HA_by_generated(k-(lmax+1));

   my(lmax=(k-3));
   sum(l=0,(k-3),
     5^l * 4
     + 5^l * vecsum(apply(HAgrow,Hsides_unsorted(k-l))))
   + 5^((k-3)+1)*HA_by_generated(k-((k-3)+1));

   \\ k-(k-3) == 3  smallest
   5^(k-2) - 1
   + sum(l=0,k-3,
     + 5^l * vecsum(apply(HAgrow,Hsides_unsorted(k-l))))
   + 5^(k-2)*HA_by_generated(2);  \\ = 16

   \\ k-(k-3) == 3  smallest
   5^(k-2) - 1
   + sum(l=0,k-3,
     + 5^l * 4*sum(j=1,k-l-3, HAgrow(2*b^j))
     + 5^l * 2*HAgrow(2*b^(k-l-2))
     + 5^l * 2*HAgrow((4-I)*b^(k-l-2))
     )
   + 5^(k-2)*HA_by_generated(2);  \\ = 16

   5^(k-2) - 1
   + 16*5^(k-2)
   + sum(l=0,k-3,
       5^l * 4*sum(j=1,k-l-3, HAgrow(2*b^j)))
   + sum(j=1,k-2,  5^(k-2-j) * 2*HAgrow(2*b^j))
   + sum(j=1,k-2,  5^(k-2-j) * 2*HAgrow((4-I)*b^j))
   ;
   \\ j=k-l-3 max k-3
   \\ j=1     min
   \\ l=0  5^0 *  j=1 .. k-3       so j=k-3 sum pows 1  (* 4)
   \\ l=1  5^1 *  j=1 .. k-2

   17*5^(k-2)
   - 1
   + sum(j=1,k-3,  (5^(k-j-2) - 1)*HAgrow(2*b^j))
   + sum(j=1,k-2,  5^(k-2-j) * 2*HAgrow(2*b^j))
   + sum(j=1,k-2,  5^(k-2-j) * 2*HAgrow((4-I)*b^j))
   ;

   17*5^(k-2)
   - 1
   + sum(j=1,k-3,  (3*5^(k-2-j) - 1)*HAgrow(2*b^j))
   + 2*HAgrow(2*b^(k-2))
   + sum(j=1,k-2,   2*5^(k-2-j)     *HAgrow((4-I)*b^j))
   ;

   17*5^(k-2)  - 1
   + sum(j=1,k-2,  (3*5^(k-2-j) - 1)*HAgrow(2*b^j))
   + sum(j=1,k-2,   2*5^(k-2-j)     *HAgrow((4-I)*b^j))
   ;
 }

gp_inline__location="../r5dragon.tex:30097";
 HAgrow_by_shear(z) = {
   z = shear_re_im(z);
   min(abs(real(z)), abs(imag(z)));
 }

gp_inline__location="../r5dragon.tex:30117";
 shear_re_im(z) = \
   z + if(sign(real(z)) == sign(imag(z)), real(z), I*imag(z));

gp_inline__location="../r5dragon.tex:30215";
 GRQ_generated_samples = {[
   1/4*1/1*(2 + I*-2),
   1/4*1/3*(10 + I*6),
   1/4*1/9*(-14 + I*78),
   1/4*1/27*(-518 + I*150),
   1/4*1/81*(-2462 + I*-2658),
   1/4*1/243*(8554 + I*-22746),
   1/4*1/729*(162130 + I*-16914),
   1/4*1/2187*(587866 + I*922038),
   1/4*1/6561*(-3768638 + I*6293310),
   1/4*1/19683*(-49065782 + I*-3731898),
   1/4*1/59049*(-124805966 + I*-305590386),
   1/4*1/177147*(1459124410 + I*-1665606954)
 ]};

gp_inline__location="../r5dragon.tex:30240";
 GRQ(k) = (2/5 - 1/5*I)*b^k + (1/10 - 3/10*I)*(1/3)^k;

gp_inline__location="../r5dragon.tex:30243";
 GRQ0 = 1/2 - 1/2*I;
 GRQ1 = 1/3 * (1/2-1/2*I + 3/2+1/2*I + 1/2+3/2*I);
 GRQ2 = 1/3*(GRQ1  + b^1 + I*GRQ1  + b^2 - I*GRQ1);

gp_inline__location="../r5dragon.tex:30373";
 GRQ_recurrence(k) = 1/3*( GRQ(k-1) + b^(k-1) + I*GRQ(k-1) + b^k - I*GRQ(k-1) );

gp_inline__location="../r5dragon.tex:30383";
 GRQ_recurrence2(k) = (2+2*I)/3*b^(k-1) + 1/3*GRQ(k-1);

gp_inline__location="../r5dragon.tex:30392";
 GRQ_sum(k) = GRQ0*(1/3)^k + 1/3*(2+2*I) * sum(j=0,k-1, (1/3)^j * b^(k-1-j));

gp_inline__location="../r5dragon.tex:30404";
 GRQ_sum2(k) = GRQ0*(1/3)^k + 1/3*(2+2*I) *((1/3)^k - b^k)/(1/3 - b);

gp_inline__location="../r5dragon.tex:30409";
 GRQ_sum3(k) = (1/10 - 3/10*I)*(1/3)^k + (2/5 - 1/5*I)*b^k;

gp_inline__location="../r5dragon.tex:30552";
 GJN_samples_by_path = { 1/2 * [
    (1) + (5)*I,   /* k=1 */
    (-32) + (24)*I,   /* k=2 */
    (-247) + (-131)*I,   /* k=3 */
    (64) + (-1904)*I,   /* k=4 */
    (11697) + (-5323)*I,   /* k=5 */
    (67104) + (54376)*I,   /* k=6 */
    (-125191) + (566061)*I,  /* k=7 */
    (-3772800) + (946848)*I,   /* k=8 */
    (-16999967) + (-19798171)*I   /* k=9 */
 ]};

gp_inline__location="../r5dragon.tex:30641";
 GJ(k) = (9/10 + 3/10*I)*b^k + (2/5-1/5*I)*(b^k-1)/(3^k-1);

gp_inline__location="../r5dragon.tex:30652";
 GJ_samples = [ (1+5*I)/2, (-32+24*I)/8, (-247-131*I)/26 ];

gp_inline__location="../r5dragon.tex:30666";
 J_times_GJ_samples = [ (1+5*I)/2, -16+12*I, (-247-131*I)/2 ];

gp_inline__location="../r5dragon.tex:30741";
 GJ_unsimplified(k) = {
   ( RQ(k)*( (1+I)*b^k - I*GRQ(k))
     - (1/2+1/2*I)*b^k
     - J(k)*b^k
   ) / ((1+I)*J(k));
 }

gp_inline__location="../r5dragon.tex:30989";
 GJf = 9/10 + 3/10*I;

gp_inline__location="../r5dragon.tex:31012";
 GJQ(k) = 1/4*(3^k - 2 + (-1)^k);

gp_inline__location="../r5dragon.tex:31015";
 GJQ_samples = {[0,2,6,20,60,182,546,1640,4920]};

gp_inline__location="../r5dragon.tex:31031";
 GJQ_ternary_digits(k) = vector(k-1,i, if(i%2==1,2,0));
 GJQ_by_ternary(k) = fromdigits(GJQ_ternary_digits(k), 3);

gp_inline__location="../r5dragon.tex:31048";
 GJN(k) = 1/12*(11*5^k - 2*(-1)^k - 9);

gp_inline__location="../r5dragon.tex:31051";
 GJN_samples = {[4,22,114,572,2864,14322,71614]};

gp_inline__location="../r5dragon.tex:31067";
 GJN_base5_digits(k) = vector(k,i, if(i%2==1,4,2));
 GJN_by_base5(k) = fromdigits(GJN_base5_digits(k), 5);

gp_inline__location="../r5dragon.tex:31214";
 pos_GJN_by_sum(k) = my(total=0); for(j=1,k, total += b^(k-j) * vector_modulo([2*I, 1+I, -2*I, -(1+I)], j-1)); total;

gp_inline__location="../r5dragon.tex:31222";
 pos_GJN_vector = {[-(9)/(10)-(3)/(10)*I,-(3)/(10)-(1)/(10)*I,(9)/(10)+(3)/(10)*I,(3)/(10)+(1)/(10)*I]};

gp_inline__location="../r5dragon.tex:31230";
 pos_GJN_by_power(k) = (9/10+3/10*I)*b^k + vector_modulo(pos_GJN_vector,k);

gp_inline__location="../r5dragon.tex:31250";
 g_pos_GJN(x) = x* (2*I + (1+I)*x - 2*I*x^2 - (1+I)*x^3)/(1-x^4) * 1/(1-b*x);

gp_inline__location="../r5dragon.tex:31259";
 g_pos_GJN_period4_part(x) = - 1/10*(9+3*I + (3+I)*x) / (1+x^2);

gp_inline__location="../r5dragon.tex:31283";
 GJmid(k) = r5dragon_point(GJN(k)) + (1/2+1/2*I)*I^dir(GJN(k));

gp_inline__location="../r5dragon.tex:31289";
 GJmid_vector = {[-(2)/(5)+(1)/(5)*I,(1)/(5)+(2)/(5)*I,(2)/(5)-(1)/(5)*I,-(1)/(5)-(2)/(5)*I]};
 GJmid_by_power(k) = (9/10+3/10*I)*b^k + vector_modulo(GJmid_vector,k);

gp_inline__location="../r5dragon.tex:31341";
 max_abs_real_or_imag(z) = max(abs(real(z)),abs(imag(z)));

gp_inline__location="../r5dragon.tex:31357";
 GJQ_by_RQ(k) = my(total=0); forstep(j=2,k,2, total += 2*RQ(k-j)); total;

gp_inline__location="../r5dragon.tex:31363";
 GJQ_by_3s(k) = my(total=0); forstep(j=2,k,2, total += 2*3^(k-j)); total;

gp_inline__location="../r5dragon.tex:31393";
 Re_b_k_mod_10_samples = {[1,1,7,9,3,1,7,9,3]};

gp_inline__location="../r5dragon.tex:31410";
 Im_b_k_mod_10_samples = {[0,2,4,8,6,2,4,8,6]};

gp_inline__location="../r5dragon.tex:31643";
 r5dragon_midpoints(k) = {
   my(end=1,v=[1/2]);
   for(i=1,k,
      v = concat(v,
          concat(apply(z->end       + z*I, v),
          concat(apply(z->end*(1+I) - z,   v),
          concat(apply(z->end*I     + z*I, v),
                 apply(z->end*2*I   + z,   v)))));
      end *= b);
   v;
 }

gp_inline__location="../r5dragon.tex:31657";
 r5dragon_end(k) = b^k;
 r5dragon_centroid(k) = r5dragon_end(k)/2;

gp_inline__location="../r5dragon.tex:31660";
 r5dragon_midpoints_at_centroid(k) = {
   my(centroid = r5dragon_centroid(k));
   apply(z -> z-centroid, r5dragon_midpoints(k));
 }

gp_inline__location="../r5dragon.tex:31665";
 Ika_by_sum(k,a) =
 {
   my(v = r5dragon_midpoints_at_centroid(k));
   my(rotate = 1/((b/abs(b))^k * exp(a*I)));
   v = apply(z->z*rotate, v);
   sum(i=1,#v, imag(v[i])^2) / #v;
 }
 Ika_by_sum = memoize(Ika_by_sum);

gp_inline__location="../r5dragon.tex:31676";
 Iz_by_sum(k) = {
   my(v = r5dragon_midpoints_at_centroid(k));
   sum(i=1,#v, abs(v[i]^2)) / 5^k;
 }

gp_inline__location="../r5dragon.tex:31683";
 I_inertia_by_points(k) = {
   my(v = r5dragon_midpoints_at_centroid(k));
   my(m = geom_inertia_of_points(v) / 5^k); \\ total unit mass
   geom_inertia_rotate_xydir(m, b^k);
 }

gp_inline__location="../r5dragon.tex:31704";
 I_inertia(k) = {[  Ix(k), -Ixy(k),  0;
                   -Ixy(k), Iy(k),   0;
                     0,      0,     Iz(k) ]};

gp_inline__location="../r5dragon.tex:31730";
 Ix(k) =   54/820 * 5^k \
         +  3/328 * real((b/5)^(2*k)) \
         + 14/328 * imag((b/5)^(2*k)) \
         - 3/40;

gp_inline__location="../r5dragon.tex:31735";
 Ix_by_fractions(k) =   54/820 * 5^k \
                +  3/328 * real((-3/25 + 4/25*I)^k) \
                + 14/328 * imag((-3/25 + 4/25*I)^k) \
                - 3/40;

gp_inline__location="../r5dragon.tex:31744";
 Ix_samples = {[0,(13)/(50),(981)/(625),(254903)/(31250),(16048266)/(390625),(4017923233)/(19531250)]};

gp_inline__location="../r5dragon.tex:31761";
 Iy(k) =   69/820 * 5^k \
         -  3/328 * real((b/5)^(2*k)) \
         - 14/328 * imag((b/5)^(2*k)) \
         - 3/40;

gp_inline__location="../r5dragon.tex:31768";
 Iy_samples = {[0,(17)/(50),(1269)/(625),(326347)/(31250),(20514234)/(390625),(5134420517)/(19531250)]};

gp_inline__location="../r5dragon.tex:31785";
 Ixy(k) =   7/164 * 5^k \
          - 7/164 * real((-3/25 + 4/25*I)^k) \
          + 3/328 * imag((-3/25 + 4/25*I)^k);

gp_inline__location="../r5dragon.tex:31791";
 Ixy_samples = {[0,(11)/(50),(667)/(625),(166721)/(31250),(10420662)/(390625),(2605159031)/(19531250)]};

gp_inline__location="../r5dragon.tex:31809";
 Iz(k) = 3/20 * (5^k - 1);

gp_inline__location="../r5dragon.tex:31813";
 Iz_samples = {[0,(3)/(5),(18)/(5),(93)/(5),(468)/(5),(2343)/(5),(11718)/(5),(58593)/(5)]};

gp_inline__location="../r5dragon.tex:31849";
 Ika(k,a)= Ix(k)*cos(a)^2 - 2*Ixy(k)*cos(a)*sin(a) + Iy(k)*sin(a)^2;

gp_inline__location="../r5dragon.tex:31975";
 xydir21 = [1; 2; 0];
 R21 = {[ 1/sqrt5, -2/sqrt5, 0;
          2/sqrt5,  1/sqrt5, 0;
          0,        0,       1 ]};

gp_inline__location="../r5dragon.tex:31995";
 xydir12 = [2; 1; 0];
 S12 = {[ 2/sqrt5, -1/sqrt5, 0;
          1/sqrt5, 2/sqrt5,  0;
          0,       0,        1 ]};

gp_inline__location="../r5dragon.tex:32017";
 xydir31 = [1; 3; 0];
 T31 = {[1/sqrt10, -3/sqrt10, 0;
         3/sqrt10,  1/sqrt10, 0;
         0,         0,        1 ]};

gp_inline__location="../r5dragon.tex:32026";
 R90 = [0,-1,0;  1,0,0;  0,0,1];

gp_inline__location="../r5dragon.tex:32032";
 R45_shear = [ 1,-1,0; 1,1,0; 0,0,1 ]; \\ +45deg*sqrt2 in x,y parts

gp_inline__location="../r5dragon.tex:32040";
 matrix_pad_2x2_to_3x3(m) = concat([m,[0;0],[0,0,0]]);

gp_inline__location="../r5dragon.tex:32066";
 I_centroid_matrix = {[0,0,0;0,1,0;0,0,1]};

gp_inline__location="../r5dragon.tex:32096";
 I_inertia_by_parts(k) = {
   if(k==0,return([0,0,0; 0,0,0; 0,0,0]));
     3/5 * R21^-1 * I_inertia(k-1) * R21
   + 2/5 * S12    * I_inertia(k-1) * S12^-1
   + 2 * 1/5 * ( sqrt5^(k-1) )^2
       * S12 * I_centroid_matrix * S12^-1
   + 2 * 1/5 * 1/2 * ( sqrt5^(k-1) )^2
       * simplify(
         T31 * I_centroid_matrix * T31^-1
         );
 }

gp_inline__location="../r5dragon.tex:32182";
 Ix_by_recurrence(k) = \
    -   6/25  * Ix(k-1) \
    -   1/25  * Ix(k-2) \
    + 216/125 * 5^(k-2) \
    -  12/125;

gp_inline__location="../r5dragon.tex:32234";
 I_fractal = {[ 54/820, -7/164, 0;
               -7/164, 69/820, 0;
                 0,      0,    123/820 ]};

gp_inline__location="../r5dragon.tex:32238";
 I_k(k) = geom_inertia_of_Ix_Iy_Ixy(Ix(k), Iy(k), Ixy(k));

gp_inline__location="../r5dragon.tex:32253";
 \\ return 1 if matrix m has its non-diagonals nearly_equal to 0
 nearly_diagonal(m) = {
   for(i=1,matsize(m)[1],
     for(j=1,matsize(m)[2],
       if(i!=j && ! nearly_equal(m[i,j],0), return(0))));
   1;
 }

gp_inline__location="../r5dragon.tex:32278";
 I_d_squared_xydir(k) = [Ix(k)-Iy(k), -2*Ixy(k)];
 I_d_xydir(k) = geom_xydir_half(I_d_squared_xydir(k));

gp_inline__location="../r5dragon.tex:32282";
 xydir_real_x(p) = {
   [ simplify(geom_point_x(p) * conj(geom_point_x(p))),
     geom_sqrt_as_quad(simplify(conj(geom_point_x(p))^2)
                       * simplify(geom_point_y(p)^2))
   ];
 }

gp_inline__location="../r5dragon.tex:32291";
 I_arctan(k) = -2*Ixy(k) / (Ix(k) - Iy(k));

gp_inline__location="../r5dragon.tex:32295";
 Ix_sub_Iy(k) =  -3/164 * 5^k \
                + 3/164 * real((-3/25 + 4/25*I)^k) \
                + 7/82  * imag((-3/25 + 4/25*I)^k);

gp_inline__location="../r5dragon.tex:32300";
 I_arctan__1(k) = { if(k==0,return(0));
                 (- 7/82  * 5^k
                  + 7/82  * real((-3/25 + 4/25*I)^k)
                  - 3/164 * imag((-3/25 + 4/25*I)^k) ) /
                ( -3/164 * 5^k
                  + 3/164 * real((-3/25 + 4/25*I)^k)
                  + 7/82  * imag((-3/25 + 4/25*I)^k)
                ); }

gp_inline__location="../r5dragon.tex:32311";
 I_arctan__2(k) = { if(k==0,return(0));
                 14/3 +
                 imag((-3/25 + 4/25*I)^k)
                 /
                 (   9/205 * 5^k
                  -  9/205 * real((-3/25 + 4/25*I)^k)
                  - 42/205 * imag((-3/25 + 4/25*I)^k)
                 ); }

gp_inline__location="../r5dragon.tex:32328";
 I_2arctan(k) = -2 * Ixy(k) / (Ix(k) - Iy(k));
 xydir_arg(p) = arg(geom_point_x(p)*1.0 + geom_point_y(p)*1.0*I);

gp_inline__location="../r5dragon.tex:32337";
 I_arctan_by_epsilon(k) = 14/3 + I_epsilon(k);

gp_inline__location="../r5dragon.tex:32348";
 I_epsilon(k) ={ imag((-3/25 + 4/25*I)^k)
                 / (   9/205 * 5^k
                    -  9/205 * real((b/5)^(2*k))
                    - 42/205 * imag((b/5)^(2*k))
                   ); }

gp_inline__location="../r5dragon.tex:32368";
 quadrant_1to4(a) = floor(a / (Pi/2)) % 4 + 1;
 I_minimum_alpha = 1/2 * atan(14/3);

gp_inline__location="../r5dragon.tex:32373";
 I_minimum_alpha_sample = {(3895262/100000)};

gp_inline__location="../r5dragon.tex:32384";
 atan_other(x) = my(a=Pi + atan(x)); if(a<0,a+=2*Pi); a;
 I_maximum_alpha = 1/2 * atan_other(14/3);

gp_inline__location="../r5dragon.tex:32393";
 I_maximum_alpha_sample = {(12895262/100000)};

gp_inline__location="../r5dragon.tex:32465";
 sqrt205 = geom_sqrt_as_quad(205);

gp_inline__location="../r5dragon.tex:32469";
 I_fractal_d = 3 + 14*I;
 I_minimum_xydir = geom_xydir_half(I_fractal_d);
 angle_of_xydir(p) = atan(geom_point_y(p) / geom_point_x(p));

gp_inline__location="../r5dragon.tex:32487";
 \\ I_fractal with unit mass
 I_fractal_principal = {[ 3/40 - sqrt205/328, 0, 0;
                          0, 3/40 + sqrt205/328, 0;
                          0, 0, 3/20 ]};

gp_inline__location="../r5dragon.tex:32660";
 QAI(k) = 5^k;

gp_inline__location="../r5dragon.tex:32662";
 QAO(k) = 5^k - 2*3^k + 1;

gp_inline__location="../r5dragon.tex:32676";
 QA(k) = 2*5^k - 2*3^k + 1;

gp_inline__location="../r5dragon.tex:32722";
 QHsides_unsorted_formula(k) = {
   my(l=List([]));
   for(d=0,3,
       listput(l, 3*I^d));
   for(d=0,3,
       for(j=1,k-1,
           listput(l, 2*b^j*I^d)));
   Vec(l);
 }
 QHsides_unsorted(k) = \
   if(k==0, vector(4,d,d--; I^d), QHsides_unsorted_formula(k));

gp_inline__location="../r5dragon.tex:32751";
 QHnumSides_formula(k) = 4*k;
 QHnumSides(k) = if(k==0,4, QHnumSides_formula(k));

gp_inline__location="../r5dragon.tex:32757";
 QHnumSides_samples = {[4,4,8,12,16,20,24]};

gp_inline__location="../r5dragon.tex:32768";
 QHB_by_generated(k) = \
   geom_length_of_polygon(Qhull_points_by_generated(k));

gp_inline__location="../r5dragon.tex:32784";
 QHB(k) = 2*(1 + sqrt5) * sqrt5^k  + 2*(1 - sqrt5);

gp_inline__location="../r5dragon.tex:32795";
 QHB_by_rat_and_irrat(k) = \
   2*(5^ceil(k/2) + 1) + 2*(5^floor(k/2) - 1)*sqrt5;

gp_inline__location="../r5dragon.tex:32837";
 QHBf = 2 + 2*sqrt5;

gp_inline__location="../r5dragon.tex:32841";
 QHBf_sample = {(6472135/1000000)};

gp_inline__location="../r5dragon.tex:32870";
 QHA_by_recurrence(k) = {
   5*QHA(k-1)
   + 4
   + vecsum(apply(HAgrow,QHsides_unsorted(k)));
 }

gp_inline__location="../r5dragon.tex:32904";
 QHA(k) = 2*5^k - 1 + sum(j=1,k-1, (5^(k-j)-1) * HAgrow(2*b^j));

gp_inline__location="../r5dragon.tex:32906";
 QHA_samples = {[1,9,65,369,1905,9697,48985]};

gp_inline__location="../r5dragon.tex:32930";
 QHAf_to_k(k) = 2 + 2*sum(j=1,k, HAgrowf(1/b^j));
 my(k=1000); QHAf_inexact = QHA(k)/5.0^k;

gp_inline__location="../r5dragon.tex:32934";
 QHAf_sample = {(3153977/1000000)};

gp_inline__location="../r5dragon.tex:32954";
 b_beta = arg(b);
 b_alpha = Pi/2 - b_beta;
 HAgrow_to_negib_by_angle(z) = {
   z * I^-floor( (geom_arg_positive(z)+Pi/2-b_beta) / (Pi/2));
   z * I^-floor( (geom_arg_positive(z)+b_alpha) / (Pi/2));
 }

gp_inline__location="../r5dragon.tex:33472";
 \\ return a vector of elements [u,v,distance] which is path lengths between each
 \\ pair of vertices u,v in vec
 mutual_pair_and_path_lengths(vpar,vec) = {
   my(list=List([]));
   for(i=1,#vec,for(j=i+1,#vec,
       listput(list,[vec[i],vec[j],vpar_path_length(vpar,vec[i],vec[j])])));
   Vec(list);
 }
 \\ return a vector of distances between pairs of vertices u,v in vec
 mutual_path_lengths(vpar,vec) = \
   apply(elem->elem[3], mutual_pair_and_path_lengths(vpar,vec));

gp_inline__location="../r5dragon.tex:33485";
 \\ Vertices 1 and #vpar are a diameter.
 \\ Root vertex is unspecified.
 QA_vpar(k) = {
   my(debug=0);
   if(k==0,return([0]));
   my(vpar=[3,3,0,3,3]);
   for(i=2,k,
       my(vpar_1root=vpar_reroot(vpar,1));
       my(conns=[1,2,#vpar-1,#vpar]);
       for(c=1,#conns,
           vpar=vpar_join(vpar,conns[c],vpar_1root);
           conns[c]=#vpar);
       if(debug, print("new conns "conns);
                 print(vpar);
                 print(mutual_path_lengths(vpar,conns)));
       vpar=vpar_relabel_swap(vpar,1,      conns[1]);
       vpar=vpar_relabel_swap(vpar,2,      conns[2]);
       vpar=vpar_relabel_swap(vpar,#vpar-1,conns[3]);
       conns[4]==#vpar || error("oops");
       if(debug, print(vpar)));
   vpar_reroot(vpar,1);
 }

gp_inline__location="../r5dragon.tex:33516";
 \\ connection vertices of QA_vpar(k)
 QA_conns(k) = [1, 2, 5^k-1, 5^k];

gp_inline__location="../r5dragon.tex:33527";
 QA_vpar_sans_conns(k,n) = {
   my(vpar=QA_vpar(k), conns=QA_conns(k));
   for(i=1,n,
       vpar=vpar_delete_vertex(vpar,conns[#conns-(i-1)]));
   vpar;
 }

gp_inline__location="../r5dragon.tex:33594";
 gen_QADegCount0_samples = [1,0,0,0,0,0,0];
 gen_QADegCount1_samples = [0,4,12,52,252,1252,6252];
 gen_QADegCount2_samples = [0,0,8,48,248,1248,6248];
 gen_QADegCount4_samples = [0,1,5,25,125,625,3125];

gp_inline__location="../r5dragon.tex:33611";
 QADegCount0(k) = {
   k>=0 || error("QADegCount0 is for k>=0");
   if(k==0,1, 0);
 }

gp_inline__location="../r5dragon.tex:33629";
 QADegCount1_formula(k) = 2*5^(k-1) + 2;
 QADegCount1(k) = {
   k>=0 || error("QADegCount1 is for k>=0");
   if(k==0,0, QADegCount1_formula(k));
 }

gp_inline__location="../r5dragon.tex:33636";
 QADegCount1_samples = {[0,4,12,52,252]};

gp_inline__location="../r5dragon.tex:33666";
 QADegCount2_formula(k) = 2*5^(k-1) - 2;
 QADegCount2(k) = {
   k>=0 || error("QADegCount2 is for k>=0");
   if(k==0,0, QADegCount2_formula(k));
 }

gp_inline__location="../r5dragon.tex:33673";
 QADegCount2_samples = {[0,0,8,48,248]};

gp_inline__location="../r5dragon.tex:33701";
 QADegCount4_formula(k) = 5^(k-1);
 QADegCount4(k) = {
   k>=0 || error("QADegCount4 is for k>=0");
   if(k==0,0, QADegCount4_formula(k));
 }

gp_inline__location="../r5dragon.tex:33708";
 QADegCount4_samples = {[0,1,5,25,125]};

gp_inline__location="../r5dragon.tex:33722";
 QADegCount_vector = \
   [QADegCount0, QADegCount1, QADegCount2, (k)->0, QADegCount4];
 QADegCount(k,d) = \
   if(d<#QADegCount_vector, QADegCount_vector[d+1](k), 0);

gp_inline__location="../r5dragon.tex:33789";
 gen_QAdiameter_samples = [0,2,8,26,80];
 gen_QAdiameterCount_samples = [1, 6, 54, 486, 4374];
 gen_QAdiameterEnds_samples = [1,4,12,36,108];
 gen_QAdiameterVertices_samples = [1,5,25];

gp_inline__location="../r5dragon.tex:33807";
 QAdiameter(k) = 3^k - 1;

gp_inline__location="../r5dragon.tex:33834";
 QAdiameterCount_formula(k) = 6*9^(k-1);
 QAdiameterCount(k) = if(k==0,1, QAdiameterCount_formula(k));

gp_inline__location="../r5dragon.tex:33843";
 QAdiameterCount_samples = {[1,6,54,486,4374]};

gp_inline__location="../r5dragon.tex:33869";
 QAdiameterEnds_formula(k) = 4*3^(k-1);
 QAdiameterEnds(k) = if(k==0,1, QAdiameterEnds_formula(k));

gp_inline__location="../r5dragon.tex:33878";
 QAdiameterEnds_samples = {[1,4,12,36,108]};

gp_inline__location="../r5dragon.tex:33897";
 QAdiameterVertices(k) = {
   k>=0 || error("QAdiameterVertices is for k>=0");
   4*k*3^(k-1) + 1;
 }

gp_inline__location="../r5dragon.tex:33908";
 QAdiameterVertices_samples = {[1,5,25,109,433]};

gp_inline__location="../r5dragon.tex:34030";
 QAconnEccVertices(k) = (k+1/2)*3^k + 1/2;

gp_inline__location="../r5dragon.tex:34042";
 diameter_num_endpoints_by_eccentricities(vpar) = {
   my(diameter=vpar_diameter(vpar));
   sum(v=1,#vpar, vpar_eccentricity_of_vertex(vpar,v)==diameter);
 }

gp_inline__location="../r5dragon.tex:34049";
 diameter_vertices_by_pairs(vpar) = {
   my(diameter=vpar_diameter(vpar),
      want    =vector(#vpar));  \\ full Vec for vecsum
   for(u=1,#vpar,
     for(v=u,#vpar,
       if(vpar_path_length(vpar,u,v)==diameter,
          my(vec=vpar_path_vertices(vpar,u,v));
          for(i=1,#vec, want[vec[i]]=1))));
   Vec(select(v->v,want,1));
 }

gp_inline__location="../r5dragon.tex:34062";
 eccentricity_vertices_by_pairs(vpar,v) = {
   my(eccentricity=vpar_eccentricity_of_vertex(vpar,v),
      want        =vector(#vpar));
   for(to=1,#vpar,
     my(vec=vpar_path_vertices(vpar,v,to));
     if(#vec==eccentricity+1,
        for(i=1,#vec, want[vec[i]]=1)));
   Vec(select(v->v,want,1));
 }

gp_inline__location="../r5dragon.tex:34163";
 \\ r5 quad area tree number of vertices
 QAV(k) = 5^k;

gp_inline__location="../r5dragon.tex:34189";
 gen_QAW_samples = [0,16,1480,117400,8962000];
 gen_QApairs_samples = [0,10,300,7750,195000];
 gen_QAwC_samples = [0,7,140,2275,35000];
 gen_QATW_samples = [0,12,456,24084,1565424];
 gen_QADeg1pairs_samples = [0,6,66,1326,31626];
 gen_QAtwC_samples = [0,6,76,1014,14524];

gp_inline__location="../r5dragon.tex:34212";
 QAW(k) = 5^k * (2/7*15^k - 5^(k-1) - 3/35);

gp_inline__location="../r5dragon.tex:34214";
 QAW_samples = {[0,16,1480,117400,8962000]};

gp_inline__location="../r5dragon.tex:34261";
 QAwC(k) = 7/2 * 5^(k-1) * (3^k - 1);

gp_inline__location="../r5dragon.tex:34264";
 QAwC_samples = {[0,7,140,2275,35000]};

gp_inline__location="../r5dragon.tex:34304";
 QAW_by_recurrence(k) = {
   if(k==0,return(0));
   5* QAW_by_recurrence(k-1)
   + 4*( 5^(k-1)*2*QAwC(k-1) + 25^(k-1) )
   + 6*( 5^(k-1)*2*QAwC(k-1) + 25^(k-1)*(QAdiameter(k-1)+2) );
 }

gp_inline__location="../r5dragon.tex:34318";
 \\ number of paths making up QAW(k)
 QAWcount(k) = if(k==0,0, \
        5*QAWcount(k-1)   \
      + 4*( 25^(k-1) ) \
      + 6*( 25^(k-1) ));

gp_inline__location="../r5dragon.tex:34453";
 QApairs(k) = 1/2*5^k*(5^k - 1);

gp_inline__location="../r5dragon.tex:34466";
 QAW_mean(k) = QAW(k)/( QApairs(k)*QAdiameter(k) );

gp_inline__location="../r5dragon.tex:34469";
 QAW_mean_limit = {4/7};

gp_inline__location="../r5dragon.tex:34477";
 QAW_mean_sample = {(0571428/1000000)};

gp_inline__location="../r5dragon.tex:34497";
 gen_StarRepW_samples = {[[0,4,120,3276,88560],
   [0,9,516,25872],
   [0,16,1480,117400],
   [0,25,3390,389700],
   [0,36,6720,1055460],
   [0,49,12040],
   [0,64,20016],
   [0,81,31410],
   [0,100,47080],
   [0,121,67980],
   [0,144,95160],
   [0,169,129766],
   [0,196,173040],
   [0,225,226320],
   [0,256,291040],
   [0,289,368730],
   [0,324,461016],
   [0,361,569620]
   ]};
 gen_StarRepWC_samples = {[[0,3,36,351,3240],
   [0,5,80,1040],
   [0,7,140,2275],
   [0,9,216,4212],
   [0,11,308,7007],
   [0,13,416],
   [0,15,540],
   [0,17,680],
   [0,19,836],
   [0,21,1008],
   [0,23,1196],
   [0,25,1400],
   [0,27,1620],
   [0,29,1856],
   [0,31,2108],
   [0,33,2376],
   [0,35,2660],
   [0,37,2960]
   ]};

gp_inline__location="../r5dragon.tex:34560";
 StarRepW_formula(s,k) = {
   s^(k-1)
   *(   1/2 * (s-1)*(3*s-5)/(3*s-1)   * (3*s)^k
      - 1/2 * (s-3)                   * s^k
      -       (s+1)/(3*s-1)
    );
 }
 StarRepW(s,k) = {
   s>=3 || error("StarRepW() is for s>=3");
   StarRepW_formula(s,k);
 }

gp_inline__location="../r5dragon.tex:34582";
 StarRepW_s3_samples = {[0,4,120,3276,88560,2391444]};

gp_inline__location="../r5dragon.tex:34592";
 StarRepW_s4_samples = {[0,9,516,25872,1258560,60674304]};

gp_inline__location="../r5dragon.tex:34607";
 StarRepW_s6_samples = {[0,25,3390,389700,42928920,4666565520]};

gp_inline__location="../r5dragon.tex:34628";
 StarRep_wC(s,k) = {
   s>=3 || error("StarRep_wC() is for s>=3");
   (2*s - 3)/(2*s) * s^k * (3^k - 1);
 }

gp_inline__location="../r5dragon.tex:34639";
 \\ Wiener index of path of n vertices.
 Wpath(n) = 1/6*(n-1)*n*(n+1);  \\ tetrahedral numbers

gp_inline__location="../r5dragon.tex:34660";
 StarRep_mean(s,k) = {
   s>=3 || error("StarRep_mean() is for s>=3");
   StarRepW(s,k) / ( StarRep_pairs(s,k)*QAdiameter(k) );
 }
 StarRep_pairs(s,k) = {
   s>=3 || error("StarRep_pairs() is for s>=3");
   1/2*s^k*(s^k-1);
 }

gp_inline__location="../r5dragon.tex:34675";
 StarRep_mean_limit(s) = {
   s>=3 || error("StarRep_mean_limit() is for s>=3");
   (s-1)*(3*s-5) / ( s*(3*s-1) );
 }

gp_inline__location="../r5dragon.tex:34702";
 StarRep_mean_limit_samples = {[1/3,(21)/(44),4/7,(65)/(102),(24)/(35),(133)/(184),(88)/(117)]};

gp_inline__location="../r5dragon.tex:34738";
 StarRep_diameterCount_formula(s,k) = \
   binomial(s-1,2)*((s-2)^2)^(k-1);
 StarRep_diameterCount(s,k) = {
   s>=3 || error("StarRep_diameterCount() is for s>=3");
   if(k==0,1, StarRep_diameterCount_formula(s,k));
 }

gp_inline__location="../r5dragon.tex:34749";
 StarRep_diameterCount_by_recurrence(s,k) = \
   if(k==0,0, (s-2)^2 * StarRep_diameterCount(s,k-1));

gp_inline__location="../r5dragon.tex:34783";
 QATW_formula(k) = 8/175*75^k + 38/125*25^k + 69/175*5^k - 1;
 QATW(k) = if(k==0,0, QATW_formula(k));

gp_inline__location="../r5dragon.tex:34789";
 QATW_samples = {[0,12,456,24084,1565424]};

gp_inline__location="../r5dragon.tex:34826";
 QAtwC_by_recurrence(k) = {
   if(k==0,0, k==1,6,
      5*QAtwC_by_recurrence(k-1)              \\ self
      + (QADegCount1(k-1) - 4) * (QAdiameter(k-1)+1)
      + (QADegCount1(k-1) - 1) * ( 3*2*(QAdiameter(k-1)+1) )
      - 4*(3^(k-1) - 1)
   );
 }

gp_inline__location="../r5dragon.tex:34861";
 QAtwC_formula(k) = 7/25 * 15^k + 14/25 * 5^k - 1;
 QAtwC(k) = if(k==0,0, QAtwC_formula(k));

gp_inline__location="../r5dragon.tex:34866";
 QAtwC_samples = {[0,6,76,1014,14524]};

gp_inline__location="../r5dragon.tex:34884";
 \\ vector of path lengths occurring in QAtwC
 QAtwC_lengths(k) = {
   if(k==0,return([]));
   if(k==1,return([2,2,2]));
   my(cwith=QAtwC_lengths(k-1),
      d=QAdiameter(k-1),
      csans1=vector_delete_must(cwith,d),
      csans3=vector_deletes_must(cwith,[d,d,d]));
   my(v=concat
        ([csans1,
          apply(l->l+3^(k-1), csans3),
          vector_repeats(apply(l->l + 2*3^(k-1), cwith),
                         3)]));
   v;
 }

gp_inline__location="../r5dragon.tex:34955";
 QATW_by_recurrence(k) = {
   if(k==0,return(0));
   if(k==1,return(12));
   5* QATW_by_recurrence(k-1)
   - 8* QAtwC(k-1)
   + 6* QAdiameter(k-1)
   + 6*(   2*(QADegCount1(k-1) - 1) * QAtwC(k-1)
         + (QADegCount1(k-1) - 1)^2 * (QAdiameter(k-1)+2) )
   + 4*(   (QADegCount1(k-1) - 4)* QAtwC(k-1)
         + (QADegCount1(k-1) - 1)*(QAtwC(k-1) - 3*QAdiameter(k-1))
         + (QADegCount1(k-1) - 4)*(QADegCount1(k-1) - 1) )
   ;
 }

gp_inline__location="../r5dragon.tex:35061";
 QADeg1pairs(k) = binomial(QADegCount1(k),2);

gp_inline__location="../r5dragon.tex:35065";
 \\ Return a vector which is all path lengths occurring in QATW.
 QATW_lengths(k) = {
   if(k==0,return([]));
   if(k==1,return([2,2,2, 2,2,2]));
   my(d=QAdiameter(k-1),
      v=vector_repeats(QATW_lengths(k-1),5),
      cwith=QAtwC_lengths(k-1),
      csans1=vector_delete_must(cwith,d),
      csans3=vector_deletes_must(cwith,[d,d,d]));
   v=concat(v,vector_repeats([d],6));
   v=vector_deletes_must(v,vector_repeats(cwith,8));
   v=concat(v,vector_repeats(apply(l->l + d+2,
                             vector_cross_sums(cwith,cwith)),
                             6));
   v=concat(v,vector_repeats(apply(l->l + 1,
                             vector_cross_sums(csans3,cwith)),
                             4));
 }

gp_inline__location="../r5dragon.tex:35097";
 QATW_mean(k) = QATW(k)/( QADeg1pairs(k)*QAdiameter(k) );

gp_inline__location="../r5dragon.tex:35099";
 QATW_mean_limit = {4/7};

gp_inline__location="../r5dragon.tex:35116";
 QA4W_samples = [0,0,48,4440,352200];  \\ from r5.pl

gp_inline__location="../r5dragon.tex:35126";
 gen_QAwidth_samples = {[1,1,3,1,1,3,3,3,9,1,1,3,1,1,3,3,3,9,3,3,9,
   3,3,9,9,9,27,1,1,3,1,1,3,3,3,9,1,1,3,1,1,3,3,3,9,3,3,9,3,3,9,9,
   9,27,3,3,9,3,3,9,9,9,27,3,3,9,3,3,9,9,9,27,9,9,27,9,9,27,27,27,
   81]};

gp_inline__location="../r5dragon.tex:35141";
 QAwidth(d) = 3^CountTernaryTwos(d);

gp_inline__location="../r5dragon.tex:35143";
 QAwidth_list(k) = vector(QAdiameter(k)+1,d,d--; QAwidth(d));

gp_inline__location="../r5dragon.tex:35148";
 QAwidth_samples = {[1,1,3,1,1,3,3,3,9,1,1,3]};

gp_inline__location="../r5dragon.tex:35163";
 CountTernaryTwos(n) = vecsum(apply(d->d==2, digits(n,3)));

gp_inline__location="../r5dragon.tex:35166";
 CountTernaryTwos_samples = {[0,0,1,0,0,1,1,1,2,0,0,1]};

gp_inline__location="../r5dragon.tex:35186";
 gQAwidth(k,x) = prod(j=0,k-1, (1 + x^(3^j) + 3*x^(2*3^j)));

gp_inline__location="../r5dragon.tex:35197";
 gCountTernaryTwos(k,x) = {
   sum(j=0,k-1,
       x^(2*3^j) / ( (1-x) * (1 + x^(3^j) + x^(2*3^j)) )
      );
 }

gp_inline__location="../r5dragon.tex:35217";
 CountTernaryLowTwos(n) = my(ret=0); while(n%3==2, n\=3;ret++); ret;

gp_inline__location="../r5dragon.tex:35221";
 TernaryLowestNonZero(n) = {
   n>=1 || error("TernaryLowestNonZero() is for n>=1");
   (n / 3^valuation(n,3)) % 3;
 }

gp_inline__location="../r5dragon.tex:35349";
 gTwoMod3(x) = x^2/(1-x^3);

gp_inline__location="../r5dragon.tex:35359";
 gCountTernaryTwos_by_recurrence(k,x) = {
   if(k==0,return(0));
   (1+x+x^2)*gCountTernaryTwos_by_recurrence(k-1,x^3)
   + gTwoMod3(x);
 }

gp_inline__location="../r5dragon.tex:35394";
 gCountTernaryTwos_by_expansion(k,x) = {
   sum(j=0,k-1,
       x^(2*3^j) / (1-x^(3^(j+1)))
       * prod(l=0,j-1, 1 + x^(3^l) + x^(2*3^l))
      );
 }

gp_inline__location="../r5dragon.tex:35432";
 TernaryLowTwos(n) = {
   my(v=digits(n,3),count=0);
   forstep(i=#v,1,-1, if(v[i]!=2,return(#v-i)));
   #v;
 }

gp_inline__location="../r5dragon.tex:35448";
 \\ set represents a set of the five vertices of quad area tree k=1
 \\ 1=vertex present, 0=vertex absent
 \\ set[5] is the middle vertex
 \\ return true if set is dependent, meaning has adjacent 1s
 QAindsets1_is_dependent(set) = \
   set[5] && (set[1]||set[2]||set[3]||set[4]);

gp_inline__location="../r5dragon.tex:35455";
 \\ typ is a vector of four connection vertex states demanded
 \\ of a quad area tree
 \\ 1=vertex present, 0=vertex absent, -1=either
 \\ return true if set is independent and satisfies typ
 QAindsets1_is_ok(set,typ) = {
   if(QAindsets1_is_dependent(set), return(0));
   for(i=1,#typ,
       if(typ[i]>=0 && set[i]!=typ[i], return(0)));
   1;
 }

gp_inline__location="../r5dragon.tex:35466";
 \\ return the number of independent sets in area tree k=1
 \\ with the constraint typ
 QAindsets1_of_type(typ) = {
   my(total=0);
   forvec(j=[[0,1],[0,1],[0,1],[0,1],[0,1]],
          total+=QAindsets1_is_ok(j,typ));
   total;
 }

gp_inline__location="../r5dragon.tex:35482";
 QAindsets_types_seen = Map();
 QAindsets1_of_types_seen = Map();

gp_inline__location="../r5dragon.tex:35485";
 \\ each middle to outer j = 00, 01, 10
 QAindsets_of_type_mem(k,typ) = {
   my(debug=0);

   if(k==1,mapput(QAindsets1_of_types_seen,typ,1));

   my(prev_seen_k);
   mapisdefined(QAindsets_types_seen,typ,&prev_seen_k);
   mapput(QAindsets_types_seen,typ,max(k,prev_seen_k));

   k>=1 || error();
   if(k==1,return(QAindsets1_of_type(typ)));

   my(total=0);
   forvec(j=[[0,1],[0,1],[0,1],[0,1]],  \\ 0= 0,any  1 = 1,0
     if(debug,print("j="j));
     total +=                          \\ jbit=0 m any, jbit=1 m=0
       QAindsets_of_type(k-1, apply(jbit->if(jbit==0,-1,0),j))
     * QAindsets_of_type(k-1, [j[1],-1,-1,typ[1]])
     * QAindsets_of_type(k-1, [j[2],-1,-1,typ[2]])
     * QAindsets_of_type(k-1, [j[3],-1,-1,typ[3]])
     * QAindsets_of_type(k-1, [j[4],-1,-1,typ[4]]));
   return(total);

   my(total=0);
   forvec(j=[[0,2],[0,2],[0,2],[0,2]],         \\ 11 types
     if(debug,print("j="j));
     total +=
       QAindsets_of_type(k-1, [bitand(j[1],1),bitand(j[2],1),
                            bitand(j[3],1),bitand(j[4],1)])   \\ middle
     * QAindsets_of_type(k-1, [j[1]\2,-1,-1,typ[1]])
     * QAindsets_of_type(k-1, [j[2]\2,-1,-1,typ[2]])
     * QAindsets_of_type(k-1, [j[3]\2,-1,-1,typ[3]])
     * QAindsets_of_type(k-1, [j[4]\2,-1,-1,typ[4]]));
   return(total);

   my(total=0);
   forvec(j=[[0,1],[0,1],[0,1],[0,1]],  \\ 0= any,0  1 = 0,1
     if(debug,print("j="j));
     total +=
       QAindsets_of_type(k-1, j)  \\ middle
     * QAindsets_of_type(k-1, [-1+j[1],-1,-1,typ[1]])
     * QAindsets_of_type(k-1, [-1+j[2],-1,-1,typ[2]])
     * QAindsets_of_type(k-1, [-1+j[3],-1,-1,typ[3]])
     * QAindsets_of_type(k-1, [-1+j[4],-1,-1,typ[4]]));
   return(total);
   total;
 }
 QAindsets_of_type_mem = memoize(QAindsets_of_type_mem);
 QAindsets_of_type(k,typ) = \
   QAindsets_of_type_mem(k,vecsort(typ));

gp_inline__location="../r5dragon.tex:35539";
 QAindsets_by_types(k) = \
   if(k==0,2, QAindsets_of_type(k,[-1,-1,-1,-1]));

gp_inline__location="../r5dragon.tex:35567";
 QAindsets(k) = {
   k>=0 || error("QAindsets() is for k>=0 ");
   if(k==0,2, QAindsets_hn(k,0));
 }

gp_inline__location="../r5dragon.tex:35578";
 QAindsets_samples = {[2,17,540497,17578416511181776606206459281]};

gp_inline__location="../r5dragon.tex:35619";
 QAindsets_hn_initial = {[17,9,5,3,2]};

gp_inline__location="../r5dragon.tex:35624";
 QAindsets_hn(k,n) = {
   k>=1 || error();
   if(k==1, return(QAindsets_hn_initial[n+1]));
   my(total=0);
   forvec(v=[[0,1],[0,1],[0,1],[0,1]],  \\ outer 0 or 1
     total +=
     QAindsets_hn(k-1, vecsum(v))       \\ middle many 0s
     * prod(l=1,4,
       my(t = if(l<=n,1,0));    \\ extra 0 outermost
       if(v[l], QAindsets_hn(k-1, t) - QAindsets_hn(k-1, t+1),
                QAindsets_hn(k-1, t+1)))
     );
   total;
 }
 QAindsets_hn = memoize(QAindsets_hn);

gp_inline__location="../r5dragon.tex:35641";
 QAindsets_hn_by_types(k,n) = {
   my(typ=concat(vector_repeats([0],n),vector_repeats([-1],4-n)));
   QAindsets_of_type(k,typ);
 }

gp_inline__location="../r5dragon.tex:35671";
 QAindsets_h1_samples = {[9,290281,9440829864910747123331181609]};

gp_inline__location="../r5dragon.tex:35685";
 QAindsets_h2_samples = {[5,155905,5070380968700781122208928001]};

gp_inline__location="../r5dragon.tex:35697";
 QAindsets_h3_samples = {[3,83737,2723146538559765117772726809]};

gp_inline__location="../r5dragon.tex:35710";
 QAindsets_h4_samples = {[2,44977,1462518717280970760659330417]};

gp_inline__location="../r5dragon.tex:35791";
 QAindnum_formula(k) = 16*5^(k-2) + 1;
 QAindnum(k) = if(k==0,1, k==1,4, QAindnum_formula(k));

gp_inline__location="../r5dragon.tex:35799";
 QAindnum_samples = {[1,4,17,81,401,2001]};

gp_inline__location="../r5dragon.tex:35804";
 QAindnum_generated_samples =  /* r5dragon-nauty.c */  \
   [ 1,4,17,81,401,2001,10001,50001,250001,1250001,6250001];

gp_inline__location="../r5dragon.tex:35813";
 QAindnum_base5_samples = {[1,4,32,311,3101,31001]};

gp_inline__location="../r5dragon.tex:35844";
 QAindnumCount_exponent_formula(k) = 1/2*QADegCount(k-1,2);
 QAindnumCount_exponent(k) = \
   if(k==0,0, QAindnumCount_exponent_formula(k));
 QAindnumCount(k) = 2^QAindnumCount_exponent(k);

gp_inline__location="../r5dragon.tex:35852";
 QAindnumCount_samples = {[1,1,1,16,16777216]};

gp_inline__location="../r5dragon.tex:35858";
 QAindnumCount_generated_samples = \
   [ 1,1,1,16,16777216];    /* r5dragon-nauty.c */

gp_inline__location="../r5dragon.tex:35863";
 QAindnumCount_exponent_generated_samples = \
   [ 0,0,0,4,24];    /* r5dragon-nauty.c */

gp_inline__location="../r5dragon.tex:36008";
 QAindRatio(k) = QAindnum(k)/5^k;

gp_inline__location="../r5dragon.tex:36068";
 QAmatchnum_formula(k) = 9*5^(k-2) - 1;
 QAmatchnum(k) = if(k==0,0, k==1,1, QAmatchnum_formula(k));

gp_inline__location="../r5dragon.tex:36076";
 QAmatchnum_samples = {[0,1,8,44,224,1124]};
 QAmatchnum_generated_samples = /* r5dragon-nauty.c */ \
   [0,1,8,44,224,1124,5624,28124,140624,703124];

gp_inline__location="../r5dragon.tex:36098";
 QAmatchnum_base5_samples = {[0,1,13,134,1344,13444]};

gp_inline__location="../r5dragon.tex:36113";
 QAmatchnumCount_initial = {[1,4,513]};

gp_inline__location="../r5dragon.tex:36122";
 QAmatchnumCount_factor = 3^4*13^2 / (2^8*5^2);

gp_inline__location="../r5dragon.tex:36126";
 QAmatchnumCount_power = 2^12 * 3^8 * 5^2 * 7 * 13^2;

gp_inline__location="../r5dragon.tex:36133";
 QAmatchnumCount_formula(k) = \
   QAmatchnumCount_factor * QAmatchnumCount_power^(5^(k-3));
 QAmatchnumCount(k) = {
   if(k<=2, QAmatchnumCount_initial[k+1],
            QAmatchnumCount_formula(k));
 }

gp_inline__location="../r5dragon.tex:36145";
 QAmatchnumCount_samples = {[1,4,513,1699990476912]};

gp_inline__location="../r5dragon.tex:36311";
 QAmatchnumCount_2(n) = {
   2^n*3^(4-n)
   + sum(j=1,4, if(j<=n, 3*2^(n-1)*3^(4-n), 4*2^n*3^(3-n)));
 }

gp_inline__location="../r5dragon.tex:36321";
 QAmatchnumCount_2_samples = {[513,351,240,164,112]};

gp_inline__location="../r5dragon.tex:36416";
 QAmatchnumCount_n_formula(k,n) = QAmatchnumCount(k) * (80/117)^n;

gp_inline__location="../r5dragon.tex:36429";
 QAmatchnumCount_n1_samples = {[1162386650880]};

gp_inline__location="../r5dragon.tex:36442";
 QAmatchnumCount_n2_samples = {[794794291200]};

gp_inline__location="../r5dragon.tex:36452";
 QAmatchnumCount_n3_samples = {[543449088000]};

gp_inline__location="../r5dragon.tex:36463";
 QAmatchnumCount_n4_samples = {[371589120000]};

gp_inline__location="../r5dragon.tex:36520";
 QAdomnum(k) = if(k==0,1, 5^(k-1));

gp_inline__location="../r5dragon.tex:36527";
 QAdomnum_samples = {[1,1,5,25,125,625]};

gp_inline__location="../r5dragon.tex:36592";
 QAdomRatio(k) = if(k==0,1, 1/5);
 QAdomRatio_limit = 1/5;

gp_inline__location="../r5dragon.tex:36624";
 QAtotdomsets_formula(k) = 820815^(5^(k-2));
 QAtotdomsets(k) = if(k==0,0, k==1, 15, QAtotdomsets_formula(k));

gp_inline__location="../r5dragon.tex:36631";
 QAtotdomsets_samples = {[0,15,820815,372585905337900111961365759375]};

gp_inline__location="../r5dragon.tex:36836";
 QAtotdomnum(k) = if(k==0,-1, k==1,2, 9*5^(k-2));

gp_inline__location="../r5dragon.tex:36844";
 QAtotdomnum_samples = {[2,9,45,225,1125]};

gp_inline__location="../r5dragon.tex:36874";
 QAtotdomnumCount_exponent(k) = {
   k>=1 || error("QAtotdomnumCount_exponent() is for k>=1");
   if(k==1,2, 2*5^(k-2));
 }
 QAtotdomnumCount(k) = \
   if(k==0,0, 2^QAtotdomnumCount_exponent(k));

gp_inline__location="../r5dragon.tex:36885";
 QAtotdomnumCount_samples = {[0,4,4,1024,1125899906842624]};

gp_inline__location="../r5dragon.tex:36903";
 QAtotdomnumCount_exponent_samples = [2,2,10,50,250,1250];

gp_inline__location="../r5dragon.tex:37034";
 QAtotdomRatio(k) = if(k==0,-1, k==1,2/5, 9/25);
 QAtotdomRatio_limit = 9/25;

gp_inline__location="../r5dragon.tex:37334";
 JAV(k) = 1/4*(5^k - 1);

gp_inline__location="../r5dragon.tex:37337";
 \\ Vertex 1 is tree start.
 \\ Vertex #vpar is tree end (the QA tree k=0 single).
 \\ Root vertex is unspecified.
 JA_vpar(k) = {
   my(vpar=[]);
   for(i=0,k-1,
       my(q=QA_vpar(i));
       q=vpar_reroot(q,1);
       vpar=vpar_join(q,#q,vpar));
   vpar;
 }

gp_inline__location="../r5dragon.tex:37368";
 gen_JADegCount0_samples = [0,1,0,0,0,0,0,0];
 gen_JADegCount1_samples = [0,0,4,14,64,314,1564,7814];
 gen_JADegCount2_samples = [0,0,1,11,61,311,1561,7811];
 gen_JADegCount4_samples = [0,0,1,6,31,156,781,3906];

gp_inline__location="../r5dragon.tex:37383";
 JADegCount0(k) = {
   k>=0 || error("JADegCount0 is for k>=0");
   if(k==1,1, 0);
 }

gp_inline__location="../r5dragon.tex:37400";
 JADegCount1_formula(k) = 1/2*(5^(k-1) + 3);
 JADegCount1(k) = {
   k>=0 || error("JADegCount1() is for k>=0");
   if(k<=1,0, JADegCount1_formula(k));
 }

gp_inline__location="../r5dragon.tex:37413";
 JADegCount1_samples = {[0,0,4,14,64]};

gp_inline__location="../r5dragon.tex:37438";
 JADegCount2_formula(k) = 1/2*(5^(k-1) - 3);
 JADegCount2(k) = {
   k>=0 || error("JADegCount2() is for k>=0");
   if(k<=1,0, JADegCount2_formula(k));
 }

gp_inline__location="../r5dragon.tex:37450";
 JADegCount2_samples = {[0,0,1,11,61]};

gp_inline__location="../r5dragon.tex:37477";
 JADegCount4_formula(k) = 1/4*(5^(k-1) - 1);
 JADegCount4(k) = {
   k>=0 || error("JADegCount4() is for k>=0");
   if(k==0,0, JADegCount4_formula(k));
 }

gp_inline__location="../r5dragon.tex:37490";
 JADegCount4_samples = {[0,0,1,6,31]};

gp_inline__location="../r5dragon.tex:37511";
 JADegCount_vector = \
   [JADegCount0, JADegCount1, JADegCount2, (k)->0, JADegCount4];
 JADegCount(k,d) = \
   if(d<#JADegCount_vector, JADegCount_vector[d+1](k), 0);

gp_inline__location="../r5dragon.tex:37584";
 gen_JAdiameter_samples = [0,0,3,12,39,120];
 gen_JAdiameterCount_samples = [0,1,3,9,27,81];
 gen_JAdiameterEnds_samples = [0,1,4,10,28,82];
 gen_JAdiameterVertices_samples = [0,1,6,27];

gp_inline__location="../r5dragon.tex:37611";
 JAdiameter_formula(k) = 1/2*(3^k - 3);
 JAdiameter(k) = if(k==0,0, JAdiameter_formula(k));

gp_inline__location="../r5dragon.tex:37624";
 JAdiameter_samples = {[0,0,3,12,39,120]};

gp_inline__location="../r5dragon.tex:37641";
 JAdiameter_ternary_samples = {[0,0,10,110,1110,11110]};

gp_inline__location="../r5dragon.tex:37665";
 JAdiameterCount_formula(k) = 3^(k-1);
 JAdiameterCount(k) = if(k==0,0, JAdiameterCount_formula(k));

gp_inline__location="../r5dragon.tex:37690";
 JAdiameterEnds_formula(k) = 3^(k-1) + 1;
 JAdiameterEnds(k) = if(k==0,0, k==1,1, JAdiameterEnds_formula(k));

gp_inline__location="../r5dragon.tex:37708";
 JAdiameterEnds_samples = {[0,1,4,10,28]};

gp_inline__location="../r5dragon.tex:37732";
 JAdiameterVertices_formula(k) = k*3^(k-1);
 JAdiameterVertices(k) = \
   if(k==0,0, k==1,1, JAdiameterVertices_formula(k));

gp_inline__location="../r5dragon.tex:37743";
 JAdiameterVertices_samples = {[0,1,6,27,108]};

gp_inline__location="../r5dragon.tex:37832";
 JAindsets(k) = JAindsets_aa(k);

gp_inline__location="../r5dragon.tex:37838";
 JAindsets_samples = {[1,2,26,11050330]};

gp_inline__location="../r5dragon.tex:37852";
 JAindsets_aa(k) = {
   k>=0 || error();
   if(k==0,return(1));
   if(k==1,return(2));
     JAindsets_aa(k-1)^3 * JAindsets_paa(k-1)
   + JAindsets_az(k-1)^3 * JAindsets_paz(k-1);
 }

gp_inline__location="../r5dragon.tex:37864";
 JAindsets_az(k) = {
   k>=0 || error();
   if(k==0,return(1));
   if(k==1,return(1));
     JAindsets_aa(k-1)^3 * JAindsets_paz(k-1)
   + JAindsets_az(k-1)^3 * JAindsets_pzz(k-1);
 }

gp_inline__location="../r5dragon.tex:37875";
 JAindsets_za(k) = {
   k>=0 || error();
   if(k==0,return(1));
   if(k==1,return(1));
     JAindsets_za(k-1) * JAindsets_aa(k-1)^2 * JAindsets_paa(k-1)
   + JAindsets_zz(k-1) * JAindsets_az(k-1)^2 * JAindsets_paz(k-1);
 }

gp_inline__location="../r5dragon.tex:37887";
 JAindsets_zz(k) = {
   k>=0 || error();
   if(k==0,return(1));
   if(k==1,return(1));
     JAindsets_za(k-1) * JAindsets_aa(k-1)^2 * JAindsets_paz(k-1)
   + JAindsets_zz(k-1) * JAindsets_az(k-1)^2 * JAindsets_pzz(k-1);
 }

gp_inline__location="../r5dragon.tex:37898";
 JAindsets_paa(k) = JAindsets_aa(k)^2 - JAindsets_wa(k)^2;

gp_inline__location="../r5dragon.tex:37901";
 JAindsets_wa(k) = JAindsets_aa(k) - JAindsets_za(k);

gp_inline__location="../r5dragon.tex:37905";
 JAindsets_pzz(k) = JAindsets_az(k)^2 - JAindsets_wz(k)^2;

gp_inline__location="../r5dragon.tex:37909";
 JAindsets_wz(k) = JAindsets_az(k) - JAindsets_zz(k);

gp_inline__location="../r5dragon.tex:37914";
 JAindsets_paz(k) = {
     JAindsets_aa(k)*JAindsets_az(k)
   - JAindsets_wa(k)*JAindsets_wz(k);
 }

gp_inline__location="../r5dragon.tex:37931";
 JAindsets_az_samples = {[1,1,17,7186721]};

gp_inline__location="../r5dragon.tex:37941";
 JAindsets_paa_samples = {[1,3,532,95941084541604]};

gp_inline__location="../r5dragon.tex:37949";
 JAindsets_za_samples = {[1,1,14,5934794]};

gp_inline__location="../r5dragon.tex:37959";
 JAindsets_paz_samples = {[1,2,346,62396495941658]};

gp_inline__location="../r5dragon.tex:37968";
 JAindsets_zz_samples = {[1,1,9,3859769]};

gp_inline__location="../r5dragon.tex:37978";
 JAindsets_pzz_samples = {[1,1,225,40580349121537]};

gp_inline__location="../r5dragon.tex:37986";
 JAindsets_generated_samples = \
   [1,2,26,11050330]; /* r5dragon-nauty.c */

gp_inline__location="../r5dragon.tex:38103";
 \\ return vpar of n many vertices along a diameter of the quad
 \\ area tree, as per the _da() etc calculation below
 QA_diam_vpar(n) = {
   my(vpar=vpar_make_path(n));
   for(i=1,n,
       my(ja=JA_vpar(TernaryCountLow1s(i-1)));
       if(#ja,
          ja=vpar_reroot(ja,#ja);
          for(rep=1,2, vpar=vpar_join(vpar,i,ja))));
   vpar;
 }

gp_inline__location="../r5dragon.tex:38160";
 QAindsets_diam_da(n) = {
   n>=0 || error();
   if(n==0,1,
        QAindsets_diam_dz(n)
      + QAindsets_diam_dz(n-1)
       *QAindsets_diam_dz(TernaryLowOnes(n-1))^2 );
 }
 QAindsets_diam_da = memoize(QAindsets_diam_da);

gp_inline__location="../r5dragon.tex:38181";
 QAindsets_diam_dz(n) = {
   n>=0 || error();
   if(n==0,1,
        QAindsets_diam_da(n-1)
      * QAindsets_diam_da(TernaryLowOnes(n-1))^2 );
 }
 QAindsets_diam_dz = memoize(QAindsets_diam_dz);

gp_inline__location="../r5dragon.tex:38191";
 QAindsets_diam_da_samples = {[1,2,9,17,26,22489,40065,62554,290281,540497]};

gp_inline__location="../r5dragon.tex:38202";
 QAindsets_diam_dz_samples = {[1,1,8,9,17,17576,22489,40065,250216,290281]};

gp_inline__location="../r5dragon.tex:38219";
 TernaryLowOnes_vector(n) = {
   n>=0 || error();
   my(v=digits(n,3));
   for(i=0,#v-1,
       if(v[#v-i]!=1, return(if(i,v[#v-i+1..#v],[]))));
   v;
 }
 TernaryLowOnes(n) = fromdigits(TernaryLowOnes_vector(n),3);

gp_inline__location="../r5dragon.tex:38238";
 TernaryLowOnes_samples = {[0,1,0,0,4,0,0,1,0,0,1,0,0,13]};

gp_inline__location="../r5dragon.tex:38254";
 TernaryCountLowOnes(n) = {
   n>=0 || error();
   my(v=digits(n,3));
   for(i=0,#v-1, if(v[#v-i]!=1,return(i)));
   #v;
 }

gp_inline__location="../r5dragon.tex:38264";
 TernaryCountLowOnes_samples = {[0,1,0,0,2,0,0,1,0,0,1,0,0,3]};

gp_inline__location="../r5dragon.tex:38429";
 TernaryRunOnes(m) = {
   m>=0 || error();
   (3^m-1)/2;
 }

gp_inline__location="../r5dragon.tex:38494";
 QAindsets_diam_pair(n) = {
   n>=0 || error("QAindsets_diam_pair() is for n>0, got ",n);
   if(n==0,return([1,1]));
   my(a_prev,z_prev);
     [a_prev,z_prev]=QAindsets_diam_pair(n-1);
   my(a_branch,z_branch);
     [a_branch,z_branch]=QAindsets_diam_pair(TernaryLowOnes(n-1));

   my(z=    a_prev*a_branch^2);
   my(a=z + z_prev*z_branch^2);
   [a,z];
 }
 QAindsets_diam_pair = memoize(QAindsets_diam_pair);

gp_inline__location="../r5dragon.tex:38510";
 QAindsets_diam_pair_notfirst(n) = {
   n>=0 || error("QAindsets_diam_pair_notfirst() is for n>=0");
   if(n==0,return([1,1]));
   if(n==1,return([1,1]));
   my(a_prev,z_prev);
     [a_prev,z_prev]=QAindsets_diam_pair_notfirst(n-1);
   my(a_branch,z_branch);
     [a_branch,z_branch]=QAindsets_diam_pair(TernaryLowOnes(n-1));

   my(z=a_prev*a_branch^2);
   my(a=z + z_prev*z_branch^2);
   [a,z];
 }
 QAindsets_diam_pair_notfirst = memoize(QAindsets_diam_pair_notfirst);

gp_inline__location="../r5dragon.tex:38532";
 JAindsets_za_by_diam(k) = \
   QAindsets_diam_pair_notfirst(TernaryRunOnes(k))[1];
 JAindsets_zz_by_diam(k) = \
   QAindsets_diam_pair_notfirst(TernaryRunOnes(k))[2];

gp_inline__location="../r5dragon.tex:38542";
 \\ number of vertices in lobe k
 QAindsets_lobe_V(k) = (5^k-1)/4;

gp_inline__location="../r5dragon.tex:38552";
 \\ Return vector v[1..n] where v[i] is how many vertices at i.
 \\ This is the 1 vertex at i itself, plus the two branches above
 \\ and below.
 QAindsets_diam_V_list(n) = \
   vector(n,i, 1 + 2*QAindsets_lobe_V(TernaryLowTwos(i)));

gp_inline__location="../r5dragon.tex:38561";
 \\ return how many vertices in sub-tree up to diameter position n
 QAindsets_diam_V(n) = fromdigits(digits(n+1,3),5)-1;

gp_inline__location="../r5dragon.tex:38574";
 \\ in n each low ternary digit 2 becomes base5 digit 4
 QAindsets_diam_V_by_24(n) = {
   my(v=digits(n,3));
   forstep(i=#v,1,-1, if(v[i]==2,v[i]=4,break()));
   fromdigits(v,5);
 }

gp_inline__location="../r5dragon.tex:38606";
 JAindnum_formula(k) = 4*5^(k-2);
 JAindnum(k) = if(k==0,0, k==1,1, JAindnum_formula(k));

gp_inline__location="../r5dragon.tex:38612";
 JAindnum_generated_samples = \
   [0,1,4,20,100,500,2500,12500,62500,312500,1562500];

gp_inline__location="../r5dragon.tex:38622";
 JAindnumCount_initial = {[1,1,2]};

gp_inline__location="../r5dragon.tex:38640";
 JAindnumCount_pow_formula(k) = 1/2*(JADegCount2(k-1) + 3);
 JAindnumCount_pow(k) = \
   if(k<=1,0, k==2,1, JAindnumCount_pow_formula(k));
 JAindnumCount_formula(k) = 2^JAindnumCount_pow_formula(k);
 JAindnumCount(k) = \
   if(k<=2,JAindnumCount_initial[k+1], JAindnumCount_formula(k));

gp_inline__location="../r5dragon.tex:38650";
 JAindnumCount_samples = {[1,1,2,4,128,4294967296]};

gp_inline__location="../r5dragon.tex:38656";
 JAindnumCount_generated_samples = [1,1,2,4,128,4294967296];

gp_inline__location="../r5dragon.tex:38664";
 JAindnumCount_pow_samples = {[0,0,1,2,7,32,157,782]};

gp_inline__location="../r5dragon.tex:38770";
 JAindRatio_formula(k) = 16/25 * (1 + 1/(5^k - 1));
 JAindRatio(k) = if(k==1,1, JAindRatio_formula(k));

gp_inline__location="../r5dragon.tex:38813";
 JAmatchnum_formula(k) = 1/4*(9*5^(k-2) - 1);
 JAmatchnum(k) = if(k<=1,0, JAmatchnum_formula(k));

gp_inline__location="../r5dragon.tex:38821";
 JAmatchnum_samples = {[0,0,2,11,56,281,1406]};
 JAmatchnum_generated_samples = /* r5dragon-nauty.c */ \
   [0,0,2,11,56,281,1406,7031,35156,175781,878906];

gp_inline__location="../r5dragon.tex:38842";
 JAmatchnum_base5_samples = {[0,0,2,21,211,2111,21111]};

gp_inline__location="../r5dragon.tex:38887";
 JAdomnum(k) = if(k==0,0, 1/4*(5^(k-1) + 3));

gp_inline__location="../r5dragon.tex:38892";
 JAdomnum_samples = {[0,1,2,7,32,157]};

gp_inline__location="../r5dragon.tex:38939";
 JAdomRatio(k) = {
   if(k==0, error("no JAdomRatio for k=0 where JA tree empty"));
   1/5*(1 + 16/(5^k - 1));
 }

gp_inline__location="../r5dragon.tex:38968";
 gen_QTdiameter_samples = [0,2,10,30,86];
 gen_QTdiameterCount_samples = [0,2,2,2,2];
 gen_QTconn_samples = [0,1,7,25];
 gen_QTW_samples = [0,8,1340,116700,9021000];
 gen_QTW_pairs_samples = [0,6,276,7626,194376];
 gen_QTwC_samples = [0,4,126,2226,34876];
 gen_QTV_samples = [0,4,24,124,624];
 gen_QTV2_samples = [0,4,16,76,376];
 gen_QTV3_samples = [0,0,8,48,248];
 gen_QTE_samples = [0,4,28,148,748];

gp_inline__location="../r5dragon.tex:38981";
 gen_C_QTdiameter_samples = [0,2,10,30,86,250,738];
 gen_C_QTecc_samples = [0, 2, 9, 28, 83, 246, 733, 2192];
 gen_C_QTW_samples =[0,8,1340,116700,9021000,679480000,50936462500];

gp_inline__location="../r5dragon.tex:39701";
 QTV(k) = 5^k - 1;

gp_inline__location="../r5dragon.tex:39731";
 QTE_formula(k) = 6*5^(k-1) - 2;
 QTE(k) = if(k==0,0, QTE_formula(k));

gp_inline__location="../r5dragon.tex:39735";
 QTE_samples = {[0,4,28,148,748,3748]};

gp_inline__location="../r5dragon.tex:39770";
 QTV2_formula(k) = 3*5^(k-1) + 1;
 QTV2(k) = if(k==0,0, QTV2_formula(k));

gp_inline__location="../r5dragon.tex:39776";
 QTV2_samples = {[0,4,16,76,376,1876]};

gp_inline__location="../r5dragon.tex:39805";
 QTV3_formula(k) = 2*5^(k-1) - 2;
 QTV3(k) = if(k==0,0, QTV3_formula(k));

gp_inline__location="../r5dragon.tex:39809";
 QTV3_samples = {[0,0,8,48,248,1248]};

gp_inline__location="../r5dragon.tex:39845";
 QTdiameter_formula(k) = 3^k + 2*k - 3;
 QTdiameter(k) = {
   k>=0 || error("QTdiameter() is for k>=0");
   if(k==0,0, QTdiameter_formula(k));
 }

gp_inline__location="../r5dragon.tex:39852";
 QTdiameter_samples = {[0,2,10,30,86,250]};

gp_inline__location="../r5dragon.tex:39918";
 QTside_formula(k) = 3^k - 2;
 QTside(k) = {
   k>=0 || error("QTside() is for k>=0");
   if(k==0,0, QTside_formula(k));
 }

gp_inline__location="../r5dragon.tex:39927";
 QTside_samples = {[0,1,7,25,79,241]};

gp_inline__location="../r5dragon.tex:39954";
 QTopp(k) = {
   k>=0 || error("QTopp() is for k>=0");
   3^k - 1;
 }

gp_inline__location="../r5dragon.tex:40006";
 QTecc_formula(k) = 3^k + k - 2;
 QTecc(k) = {
   k>=0 || error("QTecc() is for k>=0");
   if(k==0,0, QTecc_formula(k));
 }

gp_inline__location="../r5dragon.tex:40015";
 QTecc_samples = {[0,2,9,28,83,246,733,2192]};

gp_inline__location="../r5dragon.tex:40104";
 QTW(k) = 5^k * (2/7*15^k + (1/5*k-17/20)*5^k + 79/140);

gp_inline__location="../r5dragon.tex:40108";
 QTW_samples = {[0,8,1340,116700,9021000]};

gp_inline__location="../r5dragon.tex:40150";
 QTwC_by_recurrence(k) = {
   if(k==0,return(0));
   if(k==1,return(4));
   5*QTwC(k-1)
   + QTV(k-1)*(  QTside(k-1) + 2
               + 2* ( 2*QTside(k-1) + 4 )
               + QTside(k-1) + QTopp(k-1) + 4   )
   + QTside(k-1) + 1
   + 2*( QTside(k-1) + QTside(k-1) + 3 )
   + QTside(k-1) + QTopp(k-1) + 3;
 }

gp_inline__location="../r5dragon.tex:40177";
 QTwC(k) = 1/10*5^k*(7*3^k + 2*k - 17) + 1;

gp_inline__location="../r5dragon.tex:40180";
 QTwC_samples = {[0,4,126,2226,34876]};

gp_inline__location="../r5dragon.tex:40259";
 QTW_by_recurrence(k) = {
   if(k==0,return(0));
   if(k==1,return(8));
   if(k==2,return(1340));
   5* QTW_by_recurrence(k-1)
   + 10* 2 * QTV(k-1) * QTwC(k-1)   \\ pairs between 5 subs
   + QTV(k-1)^2 * ( 4*2             \\ middle/outer
                  + 4*( QTside(k-1) + 4 )
                  + 2*( QTopp(k-1)  + 4 )
                  )
   + 4*( 5*QTwC(k-1)
       + QTV(k-1) * ( 2*1    \\ own outer and middle
                    + 2*(QTside(k-1) + 3)  \\ outer sides
                    +   (QTopp(k-1)  + 3)  \\ outer opposite
                    )
       )
   + 4*( QTside(k-1) + 2 )
   + 2*( QTopp(k-1)  + 2 )
   ;
 }

gp_inline__location="../r5dragon.tex:40287";
 \\ number of paths making up QTW(k)
 QTWcount(k) = if(k==0,0, \
        5*QTWcount(k-1)   \
      + 4*( 25^(k-1) ) \
      + 6*( 25^(k-1) ));

gp_inline__location="../r5dragon.tex:40326";
 QTW_mean(k) = QTW(k)/( binomial(QTV(k),2)*QTdiameter(k) );

gp_inline__location="../r5dragon.tex:40328";
 QTW_mean_limit = {4/7};

gp_inline__location="../r5dragon.tex:40361";
 \\ An "IndC" is [indnum,count].
 \\ Return the bigger of x,y, or if equal then sum of their counts.
 \\ If count==0 then there are no sets of that indnum, so it is
 \\ not taken as the biggest.
 IndC_max(x,y) = {
   if(x[2]==0,return(y));    \\ x count 0
   if(y[2]==0,return(x));    \\ y count 0
   if(x[1]>y[1],return(x));
   if(x[1]<y[1],return(y));
   x[2] += y[2];
   if(#y>2,x=concat(x,y[3..#y]));
   x;
 }

gp_inline__location="../r5dragon.tex:40381";
 \\ Return the union of IndC's x,y.
 \\ This is sum of their indnums and product of their counts.
 IndC_union(x,y) = [ x[1]+y[1], x[2]*y[2] ];

gp_inline__location="../r5dragon.tex:40387";
 \\ "set" is [0,1,0,1] of four 0s and 1s representing a set of
 \\ vertices in the 4-cycle.
 \\ Return true if set is dependent, meaning has adjacent vertices.
 QTindset1_is_dependent(set) = {
      (set[1]&&set[2])
   || (set[2]&&set[3])
   || (set[3]&&set[4])
   || (set[4]&&set[1]);
 }

gp_inline__location="../r5dragon.tex:40404";
 \\ "typ" is [0,1,-1,0] of four 0 or 1 to demand absent or present
 \\ or -1 for anything.
 \\ Return true if set is independent and satisfies typ.
 QTindset1_is_ok(set,typ) = {
   if(QTindset1_is_dependent(set), return(0));
   for(i=1,#typ, if(typ[i]>=0 && set[i]!=typ[i], return(0)));
   1;
 }

gp_inline__location="../r5dragon.tex:40416";
 \\ Return [indnum,count] of how many 4-cycles k=1 turn graph
 \\ satisfy the given typ.
 QTindC1_of_type(typ) = {
   my(ic=[0,0]);
   forvec(set=[[0,1],[0,1],[0,1],[0,1]],
          ic=IndC_max(ic,[vecsum(set),QTindset1_is_ok(set,typ)]));
   ic;
 }

gp_inline__location="../r5dragon.tex:40430";
 \\ Return [indnum,count] of independence number and count of
 \\ quad turns graph k, with its connection vertices restricted
 \\ by "typ".
 QTindC_of_type(k,typ) = {
   my(debug=0);
   k>=0 || error();
   if(k==0,return([0,1]));
   if(k==1,return(QTindC1_of_type(typ)));

   my(ret=[0,0]);
   forvec(j=[[0,1],[0,1],[0,1],[0,1]],  \\ 0= 0,any  1 = 1,0
     my(ic=QTindC_of_type(k-1,apply(jv->if(jv==0,-1,0),j)));
     my(ic_list=List([ic[1..2]]));
     for(i=1,#j,
         ic[1] += j[i];  \\ join vertex
         my(ic_outer=QTindC_of_type(k-1,[if(j[i]==0,-1,0),
                                         typ[i],   \\ adjacent
                                         -1,-1]));
         listput(ic_list,ic_outer[1..2]);
         ic=IndC_union(ic,ic_outer);
        );
     if(debug&&k==3,
        print("j="j"  ic "ic" = ",Vec(ic_list));
       );
     ic=concat(ic,["j",j]);
     ret=IndC_max(ret,ic);
   );
   ret;
 }
 QTindC_of_type = memoize(QTindC_of_type);
 QTindC_by_types(k) = QTindC_of_type(k,[-1,-1,-1,-1]);
 QTindnum_by_types(k) = QTindC_by_types(k)[1];
 QTindnumCount_by_types(k) = QTindC_by_types(k)[2];

gp_inline__location="../r5dragon.tex:40550";
 QTindnum_formula(k) = 1/2*QTV(k);
 QTindnum(k) = if(k==0,1, QTindnum_formula(k));

gp_inline__location="../r5dragon.tex:40556";
 QTindnum_samples = {[1,2,12,62,312,1562]};

gp_inline__location="../r5dragon.tex:40564";
 QTindnum_generated_samples = \
   [ 1,2,12 ];  /* r5dragon-nauty.c */

gp_inline__location="../r5dragon.tex:40732";
 QTindnum_type_reduction(typ) = {
   my(n=sum(i=1,#typ,typ[i]==0));
   if(n<=1, return(0));
   if(n==3, return(1));
   if(n==4, return(2));
   if(typ==[0,-1,0,-1] || typ==[-1,0,-1,0],return(0)); \\ opposite
   if(   typ==[0,0,-1,-1]
      || typ==[-1,0,0,-1]
      || typ==[-1,-1,0,0]
      || typ==[0,-1,-1,0], return(1)); \\ adjacent
   error("oops");
 }

gp_inline__location="../r5dragon.tex:40751";
 QTindnum_type_js_to_middle_typ(j) = apply(v->if(v==0,-1,0),j);

gp_inline__location="../r5dragon.tex:40754";
 QTindnum_type_reduction_by_js(n,opp) = {
   my(c=if(n==0,        [-1,-1,-1,-1],
           n==1,        [ 0,-1,-1,-1],
           n==2 && opp, [ 0, -1,0,-1],
           n==2 && !opp,[ 0, 0,-1,-1],
           n==3,        [ 0, 0, 0,-1],
           n==4,        [ 0, 0, 0, 0],
           error()));
   \\ print("c="c);
   my(m=99, l=List([]));
   forvec(j=[[0,1],[0,1],[0,1],[0,1]],  \\ 0= 0,any  1 = 1,0
     my(typ_list=List([]));
     listput(typ_list,QTindnum_type_js_to_middle_typ(j));
     for(i=1,#j,
       my(outer_typ=if(c[i]==0 && j[i]==1, [0,0,-1,-1], \\ 2 adj
                       c[i]==-1 && j[i]==0, [-1,-1,-1,-1], \\ 0
                       c[i]==0 && j[i]==0, [0,-1,-1,-1],  \\ 1
                       c[i]==-1 && j[i]==1, [0,-1,-1,-1],  \\ 1
                       error()));
       listput(typ_list,outer_typ));
     my(r_list=apply(QTindnum_type_reduction,Vec(typ_list)));
     my(r=2               \\ reduction from +2
          - vecsum(j)     \\ join vertices are unreductions
          + vecsum(r_list));
     if(r<m,
        \\ print("  new low r="r" at j="j"  r_list "r_list);
        \\ print("  typs "typ_list);
        m=r);
     listput(l,r);
     );
   \\ various combinations worse than the respective best 0,1,2
   \\ print(l);
   m;
 }

gp_inline__location="../r5dragon.tex:40805";
 QTindRatio(k) = QTindnum(k)/QTV(k);

gp_inline__location="../r5dragon.tex:40824";
 \\ v is a vector [ digit,digit, ..., digit, [ digit,digit, ... ] ]
 \\ which are digits interpreted as fractional base-5.
 \\     f = 0.aaaaa (ppppp)repeat
 \\
 \\ Return a similar vector suitably mangled for an R5 dragon curve
 \\ point when taken in base b=1+2*I by vecper_frac_fromdigits().
 \\
 vecper_base5_to_r5dragon(v) = {
   vecper_apply_state((digit,d)->
                      [ r5dragon_digit(digit) * I^lift(d),
                        d + dir(digit) ],
                      v,
                      Mod(0,4));  \\ state=direction
 }

gp_inline__location="../r5dragon.tex:40843";
 \\ f is a rational fraction 0 <= f <= 1
 \\ Return complex number z location in R5 dragon fractal.
 r5dragon_fpoint(f) = {
   my(v=vecper_frac_digits(f,5));
   v=vecper_base5_to_r5dragon(v);
   vecper_frac_fromdigits(v,b);
 }

gp_inline__location="../r5dragon.tex:40874";
 fpoint_by_limit(f,k) = r5dragon_point(floor(f * 5^k))/b^k;

gp_inline__location="../r5dragon.tex:40893";
 \\ den is a denominator (an integer).
 \\ Fractional points 0/(5^k*den) through (5^k*den)/(5^k*den) are
 \\ considered.
 \\ The return is a vector [count1, count2, ccount3] which are
 \\ counts of how many locations there are visited by 1, 2 or 3
 \\ of these fractional points, respectively.
 fractional_location_counts(den,k) = {
   my(m=Map()); den *= 5^k;
   for(num=0,den,
      my(z=r5dragon_fpoint(num/den));
      mapput(m,z, 1 + mapget_or_zero(m,z)));
   m=Mat(m);
   my(counts=vector(3));
   for(i=1,matsize(m)[1], counts[m[i,2]]++);
   counts;
 }

gp_inline__location="../r5dragon.tex:41040";
 fRpred_by_states_of_vecper(fv) = {
   my(state=Rpred_transitions(Rpred_R, vecper_fixed(fv)));
   fv=vecper_periodic(fv);
   if(fv==[],fv=[0]);
   for(rep=1,4, state=Rpred_transitions(state,fv));
   state!=Rpred_non;
 }
 fRpred_of_frac(f) = \
   fRpred_by_states_of_vecper(vecper_frac_digits(f,5));
 fLpred_of_frac(f) = fRpred_of_frac(1-f);

gp_inline__location="../r5dragon.tex:41057";
 fBpred_hullstouch_state_table = {[2,3,4,5,6;2,3,4,7,8;9,9,10,11,8;
   9,10,12,13,14;15,11,13,14,14;15,16,4,5,6;15,11,13,15,12;
   12,8,10,17,8;9,9,10,17,8;9,10,12,12,12;12,17,12,18,12;
   12,12,12,12,12;12,12,12,13,14;15,18,13,14,14;15,18,13,15,12;
   12,8,10,11,8;12,17,12,12,12;12,12,12,18,12]};
 fNonRpred_hullstouch_accepting = {[12,13,14,15,18]};
 fNonLpred_hullstouch_accepting = {[8,9,10,12,17]};
 fNonBpred_hullstouch_accepting = {[12]};
 fNonRpred_hullstouch_counts = [0,0,8,72,464,2640,14168,73752,377504,1913760,9647528];
 fNonLpred_hullstouch_counts = [0,0,8,72,464,2640,14168,73752,377504,1913760,9647528];
 fNonBpred_hullstouch_counts = [0,0,1,31,315,2167,12723,69391,364395,1874407,9529443];
 fNonRpred_hullstouch_minimized_size = 5;
 fNonLpred_hullstouch_minimized_size = 5;
 fNonBpred_hullstouch_minimized_size = 18;

gp_inline__location="../r5dragon.tex:41375";
 hullsnontouch_min_distance = 0.169204564;

gp_inline__location="../r5dragon.tex:42233";
 vecper_is_exact(fv,base=10) = {
   fv=vecper_periodic(fv);
   my(all_0=1, all_9=1);
   for(i=1,#fv,
       if(fv[i]!=0, all_0=0);
       if(fv[i]!=base-1, all_9=0));
   all_0 || all_9;
 }

gp_inline__location="../r5dragon.tex:42248";
 fRpred_of_vecper(fv) = fRpred_by_states_of_vecper(fv);
 fNonRpred_of_vecper(fv) = !fRpred_of_vecper(fv);
 vector_is_all_1s(v) = for(i=1,#v,if(v[i]!=1,return(0))); 1;
 vector_is_all_3s(v) = for(i=1,#v,if(v[i]!=3,return(0))); 1;

gp_inline__location="../r5dragon.tex:42253";
 fVisits_is_R_2_of_vecper(fv) = {
   if(vecper_is_exact(fv,5),return(0));
   my(p=vecper_periodic(fv));
      ! vector_is_all_1s(p)
   && ! vector_is_all_3s(p)
   && fNonRpred_of_vecper(fv)     \\ whole
   && fRpred_of_vecper([[], p]);  \\ sans fixed
 }

gp_inline__location="../r5dragon.tex:42264";
 reverse04_vecper(fv) = vecper_apply(d->4-d,fv);
 fLpred_of_vecper(fv) = \
   fRpred_by_states_of_vecper(reverse04_vecper(fv));
 fNonLpred_of_vecper(fv) = !fLpred_of_vecper(fv);

gp_inline__location="../r5dragon.tex:42269";
 fVisits_is_L_2_of_vecper(fv) = {
   if(vecper_is_exact(fv,5),return(0));
   my(p=vecper_periodic(fv));
      ! vector_is_all_1s(p)
   && ! vector_is_all_3s(p)
   && fNonLpred_of_vecper(fv)      \\ whole
   && fLpred_of_vecper([[], p]);   \\ sans fixed
 }

gp_inline__location="../r5dragon.tex:42278";
 \\ Check fOther formed by fvfunc.
 \\ fv is a vecper.
 \\ Only those which fvpred(fv) returns non-0 are considered.
 \\ Call otherfv = fvfunc(fv) gives another vecper.
 \\ fv and otherfv are checked to be different fractions f but at
 \\ same r5dragon_fpoint() location.
 fOther_2_check(fvpred,fvfunc) = {
   my(bad=0, count=0);
   forvec(v=vector(7,i,[0,4]),
   for(len=0,#v-1, \\ of the fixed part
       my(fv=[v[1..len],v[len+1..#v]]);
       fvpred(fv) || next();
       count++;
       my(f         = vecper_frac_fromdigits(fv,5),
          fOther_fv = fvfunc(fv),
          fOther    = vecper_frac_fromdigits(fOther_fv,5));
       if(f==fOther,
           error("oops, f==fOther, fv=",fv));
       my(z=r5dragon_fpoint(f),
          fOther_z=r5dragon_fpoint(fOther));
       if(z!=fOther_z,
          print("different location fv="fv" at "z);
          print("            fOther_fv="fOther_fv" at "fOther_z);
          if(bad++>=2, break(2)))));
   \\ print(count);
   bad==0;
 }

gp_inline__location="../r5dragon.tex:42320";
 otherright_digit_to_part_used = [1,-1,3,3,3];

gp_inline__location="../r5dragon.tex:42401";
 non1_digits = [0,2,3,4];

gp_inline__location="../r5dragon.tex:42403";
 fOther_R_output_matrix = {[ 4,2,2,2;
                             0,3,3,3;
                             2,4,4,4;
                             3,0,0,0 ]};

gp_inline__location="../r5dragon.tex:42411";
 fOther_R_reset_matrix = [ 0,0,0,0; 0,1,1,1; 1,1,1,1; 1,0,0,0 ];

gp_inline__location="../r5dragon.tex:42418";
 non1_pairs = vector(16,i,i--; non1_digits[(i\4)+1]*5 \
                             + non1_digits[(i%4)+1]);

gp_inline__location="../r5dragon.tex:42426";
 non3_digits = [0,1,2,4];

gp_inline__location="../r5dragon.tex:42428";
 fOther_R_output(h,l) = {
   if(h==1, error("fOther_R_output() of h=1"));
   if(l==1, error("fOther_R_output() of l=1"));
   fOther_R_output_matrix[h+(h==0), l+(l==0)];
 }
 fOther_R_reset(h,l) = {
   if(h==1, error("fOther_R_reset() of h=1"));
   if(l==1, error("fOther_R_reset() of l=1"));
   fOther_R_reset_matrix[h+(h==0), l+(l==0)];
 }
 fOther_R_vecper_non1s(fv,pv) = {
   if(#pv==0,error());
   my(out_fv=fv,out_pv=pv);
   for(i=1,#fv,
       my(h=fv[i], l=if(i==#fv,pv[1],fv[i+1]));
       if(fOther_R_reset(h,l),out_fv=fv);
       out_fv[i]=fOther_R_output(h,l));
   for(i=1,#pv,
       my(h=pv[i], l=if(i==#pv,pv[1],pv[i+1]));
       if(fOther_R_reset(h,l), return('none));
       out_pv[i]=fOther_R_output(h,l));
   [out_fv,out_pv];
 }

gp_inline__location="../r5dragon.tex:42455";
 fOther_R_vecper(fv) = {
   my(debug=0);
   my(pv=vecper_periodic(fv));
   fv=vecper_fixed(fv);
   my(orig=[fv,pv],
      sans=apply(vector_delete_1s,orig));
   if(debug,print("orig  ",orig);
            print("sans  ",sans));
   my(out=fOther_R_vecper_non1s(sans[1],sans[2]));
   if(debug,print("out   ",out));
   if(out=='none, return(out));
   for(i=1,2,
      my(pos=0);
      for(j=1,#orig[i],
        if(orig[i][j]!=1, orig[i][j]=out[i][pos++]));
      pos==#out[i] || error());
   fv=[orig[1],orig[2]];
   if(debug,
      print("fv    ",fv));
   fv;
 }
 fOther_R(f) = \
   vecper_frac_fromdigits(fOther_R_vecper(vecper_frac_digits(f,5)), 5);

gp_inline__location="../r5dragon.tex:42627";
 otherleft_digit_to_part_used = [1,1,1,-1,3];
 fOther_L_output_matrix = {[ 4,4,4,1;
                             0,0,0,2;
                             1,1,1,4;
                             2,2,2,0 ]};

gp_inline__location="../r5dragon.tex:42636";
 fOther_L_reset_matrix = [ 0,0,0,1; 1,1,1,1; 1,1,1,0; 0,0,0,0 ];

gp_inline__location="../r5dragon.tex:42643";
 fOther_L_output(h,l) = {
   if(h==3 || l==3, error());
   fOther_L_output_matrix[h+(h!=4), l+(l!=4)];
 }
 fOther_L_reset(h,l) = {
   if(h==3 || l==3, error());
   fOther_L_reset_matrix[h+(h!=4), l+(l!=4)];
 }
 fOther_L_vecper_non3s(fv,pv) = {
   if(#pv==0,error());
   my(out_fv=fv,out_pv=pv);
   for(i=1,#fv,
       my(h=fv[i], l=if(i==#fv,pv[1],fv[i+1]));
       if(fOther_L_reset(h,l),out_fv=fv);
       out_fv[i]=fOther_L_output(h,l));
   for(i=1,#pv,
       my(h=pv[i], l=if(i==#pv,pv[1],pv[i+1]));
       if(fOther_L_reset(h,l), return('none));
       out_pv[i]=fOther_L_output(h,l));
   [out_fv,out_pv];
 }

gp_inline__location="../r5dragon.tex:42666";
 fOther_L_vecper(fv) = {
   my(pv=vecper_periodic(fv));
   fv=vecper_fixed(fv);
   my(orig=[fv,pv],
      sans=apply(vector_delete_3s,orig),
      out=fOther_L_vecper_non3s(sans[1],sans[2]));
   for(i=1,2,
      my(pos=0);
      for(j=1,#orig[i],
        if(orig[i][j]!=3, orig[i][j]=out[i][pos++]));
      pos==#out[i] || error());
   [orig[1],orig[2]];
 }
 fOther_L(f) = \
   vecper_frac_fromdigits(fOther_L_vecper(vecper_frac_digits(f,5)), 5);

gp_inline__location="../r5dragon.tex:43446";
 QuartetRev(n) = my(l=LowestNon2(n)); if(l==0||l==3, 0, 1);

gp_inline__location="../r5dragon.tex:43451";
 QuartetRev_samples = {[0,1,0,0,1,0,1,1,0,1,0,1,0,0,1]};

gp_inline__location="../r5dragon.tex:43476";
 LowestNon2_samples = {[0,1,0,3,4,0,1,1,3,4,0,1,0,3,4]};

gp_inline__location="../r5dragon.tex:43491";
 gQuartetRev_term(x,k) = {
   x^((3*5^k-1)/2) * (1 + x^(3*5^k))
   / (1 - x^(5^(k+1)));
 }

gp_inline__location="../r5dragon.tex:43497";
 \\ sum terms to k=limit inclusive
 gQuartetRev_to_k(x,limit) = sum(k=0,limit, gQuartetRev_term(x,k));

gp_inline__location="../r5dragon.tex:43633";
 QuartetRev_of_digit = [0,1,0,0,1];
 QuartetRev_by_subcurves(n) = {
   my(v=digits(n,5),rev=0);
   for(i=1,#v,        \\ high to low
       my(a=v[i]);    \\ digit
       if(rev,a=4-a);
       rev=bitxor(rev,QuartetRev_of_digit[a+1]));
   rev;
 }

gp_inline__location="../r5dragon.tex:43662";
 QuartetRevA(k) = (5^k - 1)/2;

gp_inline__location="../r5dragon.tex:43707";
 QuartetTurn_from_path_samples = {[ -1,1,1,0,-1,0,-1,-1,1,1,-1,1,1,
   0,0,-1,1,1,0,0,0,-1,-1,1,-1,-1,1,1,0,0,0,-1,-1,1,0,0,-1,-1,1,-1,
   -1,1,1,0,1,0,-1,-1,1,1,-1,1,1,0,-1,0,-1,-1,1,1,-1,1,1,0,0,-1,1,
   1,0,0,0,-1,-1,1,0,-1,1,1,0,-1,0,-1,-1,1,1,-1,1,1,0,0,-1,1,1,0,0,
   0,-1,-1,1,0,-1,1,1,0,0,0,-1,-1,1,0,0,-1,-1,1,-1,-1,1,1,0,1,0,-1,
   -1,1,-1,-1,1,1,0,-1,0,-1,-1,1,1,-1,1,1,0,0,-1,1,1,0,0,0,-1,-1,1,
   0,-1,1,1,0,0,0,-1,-1,1,0,0,-1,-1,1,-1,-1,1,1,0,1,0,-1,-1,1,0,-1,
   1,1,0,0,0,-1,-1,1,0,0,-1,-1,1,-1,-1,1,1,0,1,0,-1,-1,1,-1,-1,1,1,
   0,-1,0,-1,-1,1,1,-1,1,1,0,0,-1,1,1,0,0,0,-1,-1,1,1,-1,1,1,0,0,0,
   -1,-1,1,0,0,-1,-1,1,-1,-1,1,1,0,1,0,-1,-1,1,1,-1,1,1,0,-1,0,-1,
   -1,1,1,-1,1,1,0,0,-1,1,1,0,0,0,-1,-1,1,-1,-1,1,1,0,0,0,-1,-1,1,
   0,0,-1,-1,1,-1,-1,1,1,0,1,0,-1,-1,1,1,-1,1,1,0,-1,0,-1,-1,1,1,
   -1,1,1,0,0,-1,1,1,0,0,0,-1,-1,1,0,-1,1,1,0,-1,0,-1,-1,1,1,-1,1,
   1,0,0,-1,1,1,0,0,0,-1,-1,1,0,-1,1,1,0,0,0,-1,-1,1,0,0,-1,-1,1,
   -1,-1,1,1,0,1,0,-1,-1,1,0,-1,1,1,0,-1,0,-1,-1,1,1,-1,1,1,0,0,-1,
   1,1,0,0,0,-1,-1,1,-1,-1,1,1,0,0,0,-1,-1,1,0,0,-1,-1,1,-1,-1,1,1,
   0,1,0,-1,-1,1,1,-1,1,1,0,-1,0,-1,-1,1,1,-1,1,1,0,0,-1,1,1,0,0,0,
   -1,-1,1,0,-1,1,1,0,-1,0,-1,-1,1,1,-1,1,1,0,0,-1,1,1,0,0,0,-1,-1,
   1,0,-1,1,1,0,0,0,-1,-1,1,0,0,-1,-1,1,-1,-1,1,1,0,1,0,-1,-1,1,0,
   -1,1,1,0,-1 ]};

gp_inline__location="../r5dragon.tex:43793";
 \\ Return [r,t,l] digits r and t, count zeros l.
 QuartetTurn_rtl(n) = {
   my(l=0);
   while(n%5==0, n\=5;l++);  \\ low 0s
   my(t=n%5); n\=5;          \\ t
   while(n%5==2, n\=5);      \\ skip 2s
   my(r=n%5);                \\ r
   [r,t,l];
 }
 QuartetTurn_matrix = {[
   -1,  0,    -1,  0;    \\ F,R,   low zeros F,R
    1, -1,     1,  0;
    1, -1,     0, -1;
    0,  1,     0,  1
 ]};

gp_inline__location="../r5dragon.tex:43814";
 QuartetTurn_table = {[ -1, +1, +1,  0;
                        -1, +1,  0,  0;
                         0, -1, -1, +1;
                         0,  0, -1, +1 ]};
 QuartetTurn(n) = {
   n>=1 || error("QuartetTurn() is for n>=1");
   my(r,t,l); [r,t,l]=QuartetTurn_rtl(n);
   QuartetTurn_table[1 + if(r==0||r==2||r==3,0,2) + if(l==0,0,1),
                     t];
 }

gp_inline__location="../r5dragon.tex:43908";
 QuartetTurn_samples = {[-1,+1,+1,0,-1,0,-1,-1,+1,+1,-1,+1,+1,0]};

gp_inline__location="../r5dragon.tex:44042";
 QuartetTurn_l_zero_vector = {[-1,+1,+1,0]};

gp_inline__location="../r5dragon.tex:44048";
 QuartetTurn_l_ge_1_vector = {[-1,+1,0,0]};

gp_inline__location="../r5dragon.tex:44057";
 QuartetTurn_by_arithmetic(n) = {
   my(r,t,l); [r,t,l]=QuartetTurn_rtl(n);
   my(rev=if(r==0||r==3,1,
             r==1||r==4,-1,
              error()),
      tdash=if(rev>0, t,
               rev<0, 5-t,
               error()));
   rev * if(l==0, QuartetTurn_l_zero_vector[tdash],
                  QuartetTurn_l_ge_1_vector[tdash]);
 }

gp_inline__location="../r5dragon.tex:44090";
 twos_below_k(k,w) = {
   k>=0 || error();
   w>=0 || error();
   w<=k || error();
   (5^w-1)/2 * 5^(k-w);
 }

gp_inline__location="../r5dragon.tex:44143";
 gQuartetTurn_term(x,k,l_limit) = {
    x^(5*(5^k-1)/2)
    * (
        x^3 * (1 + x^(3*5^(k+1)))
      - x^2 * (x^(5^(k+1)) + x^(4*5^(k+1)))
      ) / (1 - x^(5^(k+2)))
    +
    sum(l=0,l_limit,
       x^( 5^(l+1)*(5^k-1)/2 )
       * (
         (-x^(5^l) + x^(2*5^l)) * (1 + x^(3*5^(k+l+1)))
       + (-x^(3*5^l) + x^(4*5^l))*(x^(5^(k+l+1)) + x^(4*5^(k+l+1)))
       )  / (1 - x^(5^(k+l+2)))
    )
   ;
 }
 gQuartetTurn_to_k(x,limit) = \
   sum(k=0,limit, gQuartetTurn_term(x,k,limit-k));

gp_inline__location="../r5dragon.tex:44203";
 gQuartetTurn_term_simplified(x,k,l_limit) = {
     x^(3 + 5*(5^k - 1)/2)
     * (1 + x^(3*5^(k+1))   )
     * (1 - x^(5^(k+1) - 1) )
     / (1 - x^(5^(k+2))     )
   -
     sum(l=0,l_limit,
       x^( 5^l * (1 + 5*(5^k - 1)/2)    )
       * ( 1 + x^(3*5^(k+l+1))         )
       * ( 1 + x^(5^l * (2 + 5^(k+1))) )
       * ( 1 - x^(5^l)                 )
       / (1 - x^(5^(k+l+2)))
     );
 }

gp_inline__location="../r5dragon.tex:44227";
 gQuartetTurn_term_l_equal_0(x,k) = {
   x^(5*(5^k-1)/2)
   * (   (-x + x^2 + x^3      ) * (1 + x^(3*5^(k+1)))
       + (   - x^2 - x^3 + x^4) * (x^(5^(k+1)) + x^(4*5^(k+1))) )
   / (1 - x^(5^(k+2)));
 }

gp_inline__location="../r5dragon.tex:44243";
 QuartetDir_state_table  = {[ 1,2,1,1,2;    \\ forward
                              1,2,2,1,2 ]}; \\ reverse
 QuartetDir_output_table = {[0,-1,0, 1,1;    \\ forward
                             1, 1,0,-1,0 ]}; \\ reverse
 QuartetDir(n) = {
   my(v=digits(n,5),state=1,d=0);
   for(i=1,#v,
       d += QuartetDir_output_table[state,v[i]+1];
       state = QuartetDir_state_table[state,v[i]+1]);
   d;
 }

gp_inline__location="../r5dragon.tex:44255";
 {my(table=[1,6,1,1,6, 1,6,6,1,6], output=[0,-1,0,1,1, 1,1,0,-1,0]);
 QuartetDir_compact(n) = my(v=digits(n,5),state=1,ret=0);
   for(i=1,#v, ret+=output[state+v[i]]; state=table[state+v[i]]);
   ret; }

gp_inline__location="../r5dragon.tex:44280";
 QuartetDir_samples = {[0,-1,0,1,1,0,0,-1,-2,-1,0,-1,0,1,1]};

gp_inline__location="../r5dragon.tex:44378";
 gQuartetDir_term(x,k,l_limit) = \
   1/(1-x) * gQuartetTurn_term(x,k,l_limit);
 gQuartetDir_to_k(x,limit) = \
   sum(k=0,limit, gQuartetDir_term(x,k,limit-k));

gp_inline__location="../r5dragon.tex:44397";
 QuartetDir_min(k) = -k;
 QuartetDir_min_by_search(k) = {
   my(n_min, d=vecmin(vector(5^k,n,n--; QuartetDir(n)), &n_min));
   [d, n_min-1];
 }
 QuartetDir_min_count_by_search(k) = {
   my(v=vector(5^k,n,n--; QuartetDir(n)),
      d=vecmin(v));
   sum(i=1,#v, d==v[i]);
 }

gp_inline__location="../r5dragon.tex:44417";
 QuartetDir_min_at_n(k) = 1/3 * (5^k - vector_modulo([1,2],k));

gp_inline__location="../r5dragon.tex:44424";
 QuartetDir_min_at_n_digits(k) = vector(k,i, if(i%2==1, 1,3));

gp_inline__location="../r5dragon.tex:44435";
 QuartetDir_min_at_n_samples = {[0,1,8,41,208,1041]};

gp_inline__location="../r5dragon.tex:44485";
 QuartetDir_max(k) = k;
 QuartetDir_max_by_search(k) = {
   my(v=vector(5^k,n,n--; QuartetDir(n)));
   my(n_max, d=vecmax(v, &n_max));
   [d, n_max-1];
 }
 QuartetDir_max_count_by_search(k) = {
   my(v=vector(5^k,n,n--; QuartetDir(n)),
      d=vecmax(v));
   sum(i=1,#v, d==v[i]);
 }

gp_inline__location="../r5dragon.tex:44505";
 QuartetDir_max_Npred(k,n) = {
   my(v=digits_padded(n,k,5),rev=0);
   for(i=1,#v,
       if(rev,v[i]=4-v[i]);
       if(v[i]<3,return(0));
       rev=bitxor(rev,v[i]-3));
   1;
 }

gp_inline__location="../r5dragon.tex:44520";
 QuartetDir_max_at_n_k0_samples = {[0]};

gp_inline__location="../r5dragon.tex:44525";
 QuartetDir_max_list(k) = {
   my(l=List([]));
   for(n=0,5^k-1,
       if(QuartetDir_max_Npred(k,n), listput(l,n)));
   Vec(l);
 }

gp_inline__location="../r5dragon.tex:44535";
 QuartetDir_max_at_n_k1_samples = {[3,4]};

gp_inline__location="../r5dragon.tex:44544";
 QuartetDir_max_at_n_k2_samples = {[18,19,20,21]};

gp_inline__location="../r5dragon.tex:44552";
 QuartetDir_max_at_n_k3_samples = {[93,94,95,96,103,104,105,106]};

gp_inline__location="../r5dragon.tex:44578";
 QuartetDir_newhigh(k) = 3/4*(5^k-1);

gp_inline__location="../r5dragon.tex:44583";
 QuartetDir_newhigh_samples = {[0,3,18,93,468,2343]};

gp_inline__location="../r5dragon.tex:44594";
 QuartetDir_newhigh_base5_samples = {[0,3,33,333,3333,33333]};

gp_inline__location="../r5dragon.tex:44607";
 QuartetDir_newhighs_by_search(k) = {
   my(m=-1,l=List([]));
   for(n=0,5^k-1,
       if(QuartetDir(n)>m, m=QuartetDir(n); listput(l,n)));
   Vec(l);
 }

gp_inline__location="../r5dragon.tex:44623";
 \\            2 end
 \\ start      v          start vertex as root,
 \\    1-->3-->4          directions shown for
 \\            ^          expansion
 \\        6-->5
 QuartetTree_vpar(k) = {
   \\ 1=start and root, 2=end
   if(k==0,return([0,1]));
   \\       1 2 3 4 5 6
   my(vpar=[0,4,1,3,4,5]);
   for(i=2,k,
     vpar_roots(vpar)==[1] || error();

     \\ "noend" tree sans end vertex, its attachment now vertex 2
     my(end_neighbours=vpar_neighbours_of_vertex(vpar,2));
     #end_neighbours==1 || error();
     my(e=end_neighbours[1]);
     my(noend=vpar);
     noend=vpar_relabel_swap(noend,2,e);
     noend=vpar_delete_vertex(noend,e);

     my(new_vpar=noend);                      \\ 1--3
     my(v4=#new_vpar+2);  \\ vertex "4"
     new_vpar=vpar_join(new_vpar,2,vpar);     \\ 3--4

     noend=vpar_reroot(noend,2);     \\ for joining at its end
     my(v5=#new_vpar+1);
     new_vpar=vpar_join(new_vpar,v4, noend);  \\ 5--4
     new_vpar=vpar_join(new_vpar,v5, noend);  \\ 6--5

     e=#new_vpar+1;   \\ v2
     new_vpar=vpar_join(new_vpar,v4, noend); \\ 4--2

     new_vpar=vpar_relabel_swap(new_vpar,2,e);  \\ end of second
     vpar=new_vpar;
     vpar_validate(vpar);
     );
   vpar;
 }

gp_inline__location="../r5dragon.tex:44888";
 QuartetDiameter(k) = if(k==0,1, 4*3^(k-1));

gp_inline__location="../r5dragon.tex:44893";
 QuartetDiameter_samples = {[1,4,12,36,108,324,972]};

gp_inline__location="../r5dragon.tex:44906";
 QuartetEccS(k) = QuartetDiameter(k);

gp_inline__location="../r5dragon.tex:44917";
 QuartetEccE(k) = 3^k;

gp_inline__location="../r5dragon.tex:44984";
 QuartetW(k) = 5^k * (13/49*15^k + 7/20*5^k + 3/35*k + 377/980);

gp_inline__location="../r5dragon.tex:44988";
 QuartetW_samples = {[1,31,1725,117475,8531625]};

gp_inline__location="../r5dragon.tex:45005";
 QuartetWofS_samples = {[1,13,179,2599,38549]};

gp_inline__location="../r5dragon.tex:45017";
 QuartetWofE_samples = {[1,11,147,2117,31317]};

gp_inline__location="../r5dragon.tex:45054";
 W_sum_of_vertex(vpar,v) = vecsum(vpar_path_lengths_vector(vpar,v));

gp_inline__location="../r5dragon.tex:45062";
 QuartetWofS(k) = 53/70*15^k + 7/20*5^k - 3/28;

gp_inline__location="../r5dragon.tex:45068";
 QuartetWofE(k) = 43/70*15^k + 7/20*5^k + 1/28;

gp_inline__location="../r5dragon.tex:45119";
 QuartetW_SsubE_samples = {[0,2,32,482,7232]};

gp_inline__location="../r5dragon.tex:45257";
 QuartetWX(k) = 5^k*(1/4*5^k - 3/5*k + 3/4);

gp_inline__location="../r5dragon.tex:45263";
 QuartetWX_samples = {[1,7,145,3775,96625]};

gp_inline__location="../r5dragon.tex:45323";
 QuartetPairs(k) = 1/2*(5^k + 1)*5^k;

gp_inline__location="../r5dragon.tex:45337";
 QuartetW_mean(k) = \
   QuartetW(k)/( QuartetPairs(k)*QuartetDiameter(k) );

gp_inline__location="../r5dragon.tex:45341";
 QuartetW_mean_limit = {(39)/(98)};

gp_inline__location="../r5dragon.tex:45350";
 QuartetW_mean_sample = {(0397959/1000000)};

gp_inline__location="../r5dragon.tex:45369";
 WX_list(k) = {
   if(k==0,return([1]));
   my(v=WX_list(k-1), N=5^k+1);
   v = apply(x->[5*x-4, 5*x-3, N-5*x, 1, 2], v);
   concat(v);
 }

gp_inline__location="../r5dragon.tex:45383";
 W_by_WX_list(k) = {
   my(v=WX_list(k), n=5^k+1);
   return(sum(i=1,#v, my(x=v[i], y=n-x); x*y));

   my(v=WX_list(k-1), N=5^k+1, n=5^(k-1)+1);
   sum(i=1,#v,
       my(x=v[i], y=n-x);
       (5*x-4)*(5*y) + (5*x-3)*(5*y-1) + (5*x)*(5*y-4)
       + 1*(N-1) + 2*(N-2));
 }

gp_inline__location="../r5dragon.tex:45435";
 \\ two trees k as S-->*-->E
 QuartetTree2_vpar(k) = {
   my(vpar=QuartetTree_vpar(k));
   my(neighbours=vpar_neighbours_of_vertex(vpar,2));
   #neighbours==1 || error();
   my(sans_end=vpar_relabel_swap(vpar,1,neighbours[1]));
   sans_end=vpar_delete_vertex(sans_end,2);
   vpar=vpar_join(sans_end,1,vpar);
   vpar_validate(vpar);
   vpar;
 }

gp_inline__location="../r5dragon.tex:45464";
 QuartetTreeV(k) = 5^k+1;

gp_inline__location="../r5dragon.tex:45602";
 QuartetWidth_infinity_generated_samples = [1,1,1,2,2,1,2,3,4,4,2,3,4,2,1,2,3,4,4,3,4,6,6,8,8,4,8,8,2,2,4,5,5,6,5,8,8,2,3,4,2,1,2];

gp_inline__location="../r5dragon.tex:45668";
 QuartetWidthS_k(k,d) = {
   if(d<0,return(0));
   if(k==0,return(if(d==0||d==1,1, 0)));
       QuartetWidthS_k(k-1, d)
   +   QuartetWidthS_k(k-1, d-3^(k-1))   - if(d==3^(k-1),   1)
   + 2*QuartetWidthE_k(k-1, d-2*3^(k-1)) - if(d==2*3^(k-1), 2)
   +   QuartetWidthE_k(k-1, d-3*3^(k-1)) - if(d==3*3^(k-1), 1);
 }
 QuartetWidthE_k(k,d) = {
   if(d<0,return(0));
   if(k==0,return(if(d==0||d==1,1, 0)));
       QuartetWidthS_k(k-1, d)
   + 2*QuartetWidthE_k(k-1, d-3^(k-1))
   + 2*QuartetWidthE_k(k-1, d-2*3^(k-1))
   - if(d==3^(k-1) || d==2*3^(k-1), 2);
 }
 QuartetWidthS_k = memoize(QuartetWidthS_k);
 QuartetWidthE_k = memoize(QuartetWidthE_k);

gp_inline__location="../r5dragon.tex:45696";
 QuartetWidthS_0_samples = {[1,1]};

gp_inline__location="../r5dragon.tex:45704";
 QuartetWidthS_1_samples = {[1,1,1,2,1]};

gp_inline__location="../r5dragon.tex:45712";
 QuartetWidthS_2_samples = {[1,1,1,2,2,1,2,3,4,4,1,2,2]};

gp_inline__location="../r5dragon.tex:45739";
 QuartetWidthS_inf_k(d) = my(k=0); while(!(3^k>=d),k++); k;
 QuartetWidthS_inf(d) = QuartetWidthS_k(QuartetWidthS_inf_k(d),d);

gp_inline__location="../r5dragon.tex:45743";
 QuartetWidthS_inf_samples = {[1,1,1,2,2,1,2,3,4,4,2,3,4,2,1,2,3,4,4,3,4,6,6,8,8,4]};

gp_inline__location="../r5dragon.tex:45782";
 QuartetDOne(m) = if(m==0, 0, 1/2*(3^(m-1)+1));

gp_inline__location="../r5dragon.tex:45787";
 QuartetDOne_samples = {[0,1,2,5,14,41,122,365,1094,3281]};

gp_inline__location="../r5dragon.tex:45809";
 QuartetDOne_digits(m) = vector(m-1,i,if(i==m-1,2,1));
 QuartetDOne_by_digits(m) = fromdigits(QuartetDOne_digits(m),3);

gp_inline__location="../r5dragon.tex:46123";
 QuartetTree_vpar_sans_start_end(k) = {
   my(vpar=QuartetTree_vpar(k));
   vpar=vpar_delete_vertex(vpar,2);
   vpar_delete_vertex(vpar,1);
 }

gp_inline__location="../r5dragon.tex:46147";
 QuartetTotdomnum(k) = if(k==0,2, k==1,3, 12*5^(k-2) + 1);

gp_inline__location="../r5dragon.tex:46152";
 QuartetTotdomnum_samples = {[2,3,13,61,301,1501]};

gp_inline__location="../r5dragon.tex:46171";
 QuartetTotdomnumCount_power(k) = if(k<=1,0, 1/4*(5^(k-2) - 1));
 QuartetTotdomnumCount(k) = 12^QuartetTotdomnumCount_power(k);

gp_inline__location="../r5dragon.tex:46179";
 QuartetTotdomnumCount_samples = {[1,1,1,12,2985984]};

gp_inline__location="../r5dragon.tex:46296";
 QuartetDomnum(k) = if(k==0,1, 2*5^(k-1)+1);

gp_inline__location="../r5dragon.tex:46305";
 QuartetIndomnum(k) = if(k==0,1, 2*5^(k-1)+1);

gp_inline__location="../r5dragon.tex:46331";
 QuartetParent_rlow(n) = {
   my(low=n%5); n\=5;
   while(n%5==2, n\=5);
   my(r=n%5);
   [r,low];
 }
 QuartetParent(n) = {
   n>=1 || error("QuartetParent() is for n>=1");
   my(r,low); [r,low] = QuartetParent_rlow(n);
   n
   + if(r==0||r==2||r==3, [-1, -1, +1, +1, -3][low+1],
        r==1||r==4,       [-1, -1, -1, -1, -3][low+1],
        error());
 }

gp_inline__location="../r5dragon.tex:46490";
 \\ is_variable(str) returns true if str is the name of a variable,
 \\ meaning a "foo" not equal to its own symbol 'foo.
 is_variable(str) = eval(str)!=eval(Str("'",str));

gp_inline__location="../r5dragon.tex:46495";
 \\ exceptions b=1+2*I, s(m).
 char_variables_allowed = Set(["b","s"]);
 check_char_variables() = {
   my(char_variables_ok = 1);
   for(ch=Vecsmall("a")[1], Vecsmall("z")[1],
       my(str = Strchr(ch));
       if(is_variable(str) && ! setsearch(char_variables_allowed,str),
          print("Bad: global variable ",str," = ",eval(str)); char_variables_ok = 0));
   char_variables_ok;
 }

kill(gp_inline__location);
